// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Text;
using System.Data;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class AlertViewer : Form
    {
        private delegate void AlertViewerRefreshHandler();
        private static ArrayList alertList = new ArrayList();
        private static AlertViewer alertViewer = null;
        public static AmtTrapListener TrapListener = null;

        private static void LoadAlerts()
        {
            // Load the alert list if not already loaded
            if (alertList == null)
            {
                byte[] buf = (byte[])Application.UserAppDataRegistry.GetValue("AlertViewer", null);
                if (buf == null)
                {
                    alertList = new ArrayList();
                }
                else
                {
                    alertList = (ArrayList)Utils.UnserializeBinary(buf);
                }
            }
        }

        private static bool SaveAlerts()
        {
            try
            {
                Application.UserAppDataRegistry.SetValue("AlertViewer", Utils.SerializeBinary(alertList));
            }
            catch (System.IO.IOException)
            {
                // Too many alerts, remove half and try again.
                alertList.RemoveRange(0, alertList.Count / 2);
                SaveAlerts();
                return true;
            }
            return false;
        }

        public static void AddAlert(AmtAlert alert, string debug1, string debug2)
        {
            LoadAlerts();

            // Look for a duplicate alert
            bool dup = false;
            foreach (AmtAlert a in alertList)
            {
                if ((a.SequenceNumber == alert.SequenceNumber) && (a.PlatformGuid == alert.PlatformGuid))
                {
                    dup = true;
                    break;
                }
            }

            alert.Tag = new object[] { dup, debug1, debug2 };

            lock (alertList)
            {
                alertList.Add(alert);
                SaveAlerts();
            }

            if (alertViewer != null) alertViewer.Invoke(new AlertViewer.AlertViewerRefreshHandler(alertViewer.UpdateInfo));
        }

        public static void ShowAlertViewer(Form parent)
        {
            LoadAlerts();

            if (alertViewer != null)
            {
                alertViewer.Activate();
            }
            else
            {
                alertViewer = new AlertViewer();
                alertViewer.Show(parent);
            }
        }

        public AlertViewer()
        {
            InitializeComponent();
        }

        private void AlertViewer_FormClosing(object sender, FormClosingEventArgs e)
        {
            alertViewer = null;
        }

        private void UpdateInfo()
        {
            lock (alertList)
            {
                alertListView.BeginUpdate();
                alertListView.Items.Clear();
                foreach (AmtAlert a in alertList)
                {
                    bool dup = ((bool)(((object[])(a.Tag))[0]));
                    if ((showDuplicatesToolStripMenuItem.Checked == true) || dup == false)
                    {
                        string dupstr = "";
                        if (dup == true) dupstr = "*";
                        ListViewItem l = new ListViewItem(new string[] { a.ReceivedTime.ToShortTimeString(), a.TrapSource.ToString() + " / " + a.SequenceNumber.ToString() + dupstr});
                        l.Tag = a;
                        alertListView.Items.Insert(0,l);
                    }
                }
                alertListView.EndUpdate();
            }
        }

        private void AlertViewer_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            UpdateInfo();
        }

        private void alertListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            deleteSelectedAlertsToolStripMenuItem.Enabled = (alertListView.SelectedItems.Count > 0);

            if (alertListView.SelectedItems.Count != 1)
            {
                alertTextBox.Text = "";
                return;
            }

            ListViewItem l = alertListView.SelectedItems[0];
            AmtAlert alert = (AmtAlert)l.Tag;
            alertTextBox.Text = (string)((object[])(alert.Tag))[2];
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void clearEventsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(this, Properties.Resources.ClearAllAlertsQuestion, Properties.Resources.AlertViewer, MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
            {
                lock (alertList)
                {
                    alertList.Clear();
                    SaveAlerts();
                }
                alertTextBox.Text = "";
                UpdateInfo();
            }
        }

        private void showDuplicatesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void deleteSelectedAlertsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (alertListView.SelectedItems.Count == 0) return;

            if (showDuplicatesToolStripMenuItem.Checked == true)
            {
                // Delete Selected
                ArrayList selList = new ArrayList(alertListView.SelectedItems);
                foreach (ListViewItem l in selList)
                {
                    alertList.Remove(l.Tag);
                }
            }
            else
            {
                // Delete Selected & duplicates
                ArrayList selList = new ArrayList(alertListView.SelectedItems);
                ArrayList delList = new ArrayList();
                foreach (ListViewItem l in selList)
                {
                    AmtAlert a1 = (AmtAlert)l.Tag;
                    foreach (AmtAlert a2 in alertList)
                    {
                        if (a1.SequenceNumber == a2.SequenceNumber && a1.TrapSource == a2.TrapSource) delList.Add(a2);
                    }
                }
                foreach (AmtAlert a3 in delList)
                {
                    alertList.Remove(a3);
                }
            }
            UpdateInfo();
            SaveAlerts();
        }

        private void deleteDuplicatesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(this, Properties.Resources.DeleteDuplicates, Properties.Resources.DeleteAlertsTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.Yes)
            {
                // Delete Selected & duplicates
                ArrayList delList = new ArrayList();
                foreach (AmtAlert a in alertList)
                {
                    bool dup = ((bool)(((object[])(a.Tag))[0]));
                    if (dup == true) delList.Add(a);
                }
                foreach (AmtAlert a2 in delList)
                {
                    alertList.Remove(a2);
                }
                UpdateInfo();
                SaveAlerts();
            }
        }

        private void alertListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) deleteSelectedAlertsToolStripMenuItem_Click(this, null);
        }

        private void openTrapFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                FileStream f;
                try
                {
                    f = new FileStream(openFileDialog.FileName, FileMode.Open);
                }
                catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                byte[] buf = new byte[(int)f.Length];
                f.Read(buf, 0, buf.Length);
                f.Close();

                string[] ss = UTF8Encoding.UTF8.GetString(buf).Split(',');

                buf = new byte[ss.Length];
                for (int i = 0; i < ss.Length; i++)
                {
                    buf[i] = (byte)(int.Parse(ss[i]));
                }

                AmtAlert alert = AmtAlert.DecodeAlert(buf, 0, buf.Length);

                BerSEQUENCE s = null;
                try
                {
                    alert = AmtAlert.DecodeAlert(buf, 0, buf.Length);
                    s = BerSEQUENCE.DecodeBER(buf, 0, buf.Length);
                }
                catch (Exception ex)
                {
                    // Exception occured while parsing this alert. Lets dump it to "exception.txt".
                    AmtDebugForm.LogException(this, ex);
                    return;
                }

                string str = string.Format(Properties.Resources.AlertDebugDescription, alert.ReceivedTime.ToString(), alert.Community, alert.PlatformGuid.ToString(), alert.SequenceNumber.ToString(), alert.TrapSource.ToString(), alert.TrapVersion.ToString(), alert.TimeStamp.ToString(), alert.TimeStamp2.ToString(), alert.UtcOffset.ToString());
                str += string.Format(Properties.Resources.EventDebugDescription, alert.amtEvent.DeviceAddress.ToString(), alert.amtEvent.EntityTypeStr, alert.amtEvent.EventOffsetStr, alert.amtEvent.EventSensorTypeStr, alert.amtEvent.EventSeverityStr, alert.amtEvent.EventSourceStr, alert.amtEvent.EventTypesStr, alert.amtEvent.EventDataStr.ToString());
                str += string.Format(Properties.Resources.BerDebugDescription, s.GetDebugString());
                AmtTrapListener.TrapListener.DebugInject(alert, s.ToString(), str);
            }
        }

    }
}
