// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class AuditLogPolicyForm : Form
    {
        private AmtSystem system;

        public AuditLogPolicyForm(AmtSystem system)
        {
            this.system = system;
            InitializeComponent();

            if (system.CoreVersion >= (float)5.1)
            {
                this.storagePolicyButton.Visible = true;
                this.storagePolicyButton.Enabled = true;
            }
        }

        private void UpdateLogStatus()
        {
            AmtCallStatus r;
            AmtGeneralInfo.AmtAuditLogStatus status;
            r = system.Info.GetAuditLogStatus(out status);
            if (r == AmtCallStatus.SUCCESS && status != null)
            {
                if (status.Disabled) auditLogEnabledLabel.Text = Properties.Resources.Disabled; else auditLogEnabledLabel.Text = Properties.Resources.Enabled;
                maxAuditorsLabel.Text = status.MaxAllowedAuditors.ToString();
                numberOfRecordsLabel.Text = status.NumberOfRecords.ToString();
                percentageFreeLabel.Text = status.PercentageFree.ToString() + "%";
                if (status.Locked) lockStatusLabel.Text = Properties.Resources.Locked; else lockStatusLabel.Text = Properties.Resources.Unlocked;
                StringBuilder state = new StringBuilder();
                if (status.AlmostFull) state.Append(", ").Append(Properties.Resources.AlmostFull);
                if (status.Full) state.Append(", ").Append(Properties.Resources.Full);
                if (status.NoKey) state.Append(", ").Append(Properties.Resources.NoKey);
                if (state.Length > 0) statusLabel.Text = state.ToString().Substring(2); else statusLabel.Text = Properties.Resources.None;
                enableButton.Enabled = !status.NoKey;
                auditLogButton.Enabled = !status.Disabled;
                timeOfLastRecordLabel.Text = status.TimeOfLastRecord.ToString();

                AmtSecurityAdmin.AmtUserAcl[] acls;
                r = system.SecurityAdmin.GetAuditors(out acls);
                if (r == AmtCallStatus.SUCCESS)
                {
                    if (acls == null || acls.Length == 0)
                    {
                        // Administrator is auditor
                        auditorsLabel.Text = system.SecurityAdmin.GetAdminAclEntry();
                    }
                    else
                    {
                        // User is auditor
                        auditorsLabel.Text = acls[0].Username;
                    }
                }
            }
        }

        private void UpdatePolicies()
        {
            AmtCallStatus r;
            AuditPolicyType[] policies;
            r = system.Info.GetAuditPolicy(out policies);
            if (r == AmtCallStatus.SUCCESS && policies != null)
            {
                filterListView.BeginUpdate();
                foreach (ListViewItem l in filterListView.Items)
                {
                    int icon = 3;
                    string state = Properties.Resources.Disabled;
                    ushort e1 = (ushort)((int)(l.Tag));
                    foreach (AuditPolicyType policy in policies)
                    {
                        ushort e2 = (ushort)(policy.AuditedAppID * 1000 + policy.EventId);
                        if (e1 == e2)
                        {
                            if (policy.FlagSpecified == true && policy.Flag == 1)
                            {
                                icon = 1;
                                state = Properties.Resources.Critical;
                            }
                            else
                            {
                                icon = 0;
                                state = Properties.Resources.Enabled;
                            }
                        }
                    }
                    l.ImageIndex = icon;
                    l.SubItems[1].Text = state;
                }
                filterListView.EndUpdate();
            }
        }

        private void AuditLogPolicyForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;

            foreach (ListViewItem l in filterListView.Items)
            {
                l.ImageIndex = 3;
                l.SubItems.Add(Properties.Resources.Disabled);
            }

            // Set all of the tags
            // Security Admin (App ID = 16)
            filterListView.Items[00].Tag = 16000; // AMT Provisioning Started
            filterListView.Items[01].Tag = 16001; // AMT Provisioning Completed
            filterListView.Items[02].Tag = 16002; // ACL Entry Added
            filterListView.Items[03].Tag = 16003; // ACL Entry Modified
            filterListView.Items[04].Tag = 16004; // ACL Entry Removed
            filterListView.Items[05].Tag = 16005; // ACL Access with invalid credentials
            filterListView.Items[06].Tag = 16006; // ACL Entry Enabled
            filterListView.Items[07].Tag = 16007; // TLS State Changed
            filterListView.Items[08].Tag = 16008; // TLS Server Certificate Set
            filterListView.Items[09].Tag = 16009; // TLS Server Certificate Removed
            filterListView.Items[10].Tag = 16010; // TLS Trusted Root Certificate Added
            filterListView.Items[11].Tag = 16011; // TLS Trusted Root Certificate Removed
            filterListView.Items[12].Tag = 16012; // TLS Pre-Shared Key Set
            filterListView.Items[13].Tag = 16013; // Kerberos Settings Modified
            filterListView.Items[14].Tag = 16014; // Kerberos Master Key Modified
            filterListView.Items[15].Tag = 16015; // Flash Wear-Out Counters Reset
            filterListView.Items[16].Tag = 16016; // Power Package Modified
            filterListView.Items[17].Tag = 16017; // Set realm Authentication mode

            // Remote control (App ID = 17)
            filterListView.Items[18].Tag = 17000; // Performed Power-Up
            filterListView.Items[19].Tag = 17001; // Performed Power-Down
            filterListView.Items[20].Tag = 17002; // Performed Power-Cycle
            filterListView.Items[21].Tag = 17003; // Performed Reset
            filterListView.Items[22].Tag = 17004; // Set Boot Options

            // Redirection Manager (App ID = 18)
            filterListView.Items[23].Tag = 18000;
            filterListView.Items[24].Tag = 18001;
            filterListView.Items[25].Tag = 18002;
            filterListView.Items[26].Tag = 18003;
            filterListView.Items[27].Tag = 18004;
            filterListView.Items[28].Tag = 18005;
            filterListView.Items[29].Tag = 18006;
            filterListView.Items[30].Tag = 18007;
            filterListView.Items[31].Tag = 18008;
            filterListView.Items[32].Tag = 18009;
            filterListView.Items[33].Tag = 18010;
            filterListView.Items[34].Tag = 18011;
            filterListView.Items[35].Tag = 18012;

            filterListView.Items[36].Tag = 19000;
            filterListView.Items[37].Tag = 19001;

            filterListView.Items[38].Tag = 20000;
            filterListView.Items[39].Tag = 20001;
            filterListView.Items[40].Tag = 20002;
            filterListView.Items[41].Tag = 20003;
            filterListView.Items[42].Tag = 20004;
            filterListView.Items[43].Tag = 20005;

            filterListView.Items[44].Tag = 21000;

            filterListView.Items[45].Tag = 22000;
            filterListView.Items[46].Tag = 22001;
            filterListView.Items[47].Tag = 22002;
            filterListView.Items[48].Tag = 22003;
            filterListView.Items[49].Tag = 22004;
            filterListView.Items[50].Tag = 22005;

            filterListView.Items[51].Tag = 23000;
            filterListView.Items[52].Tag = 23001;
            filterListView.Items[53].Tag = 23002;
            filterListView.Items[54].Tag = 23003;

            filterListView.Items[55].Tag = 24000;
            filterListView.Items[56].Tag = 24001;
            filterListView.Items[57].Tag = 24002;
            filterListView.Items[58].Tag = 24003;

            filterListView.Items[59].Tag = 25000;
            filterListView.Items[60].Tag = 25001;
            filterListView.Items[61].Tag = 25002;
            filterListView.Items[62].Tag = 25003;
            filterListView.Items[63].Tag = 25004;
            filterListView.Items[64].Tag = 25005;
            filterListView.Items[65].Tag = 25006;

            filterListView.Items[66].Tag = 26000;
            filterListView.Items[67].Tag = 26001;
            filterListView.Items[68].Tag = 26002;

            filterListView.Items[69].Tag = 27000;
            filterListView.Items[70].Tag = 27001;
            filterListView.Items[71].Tag = 27002;

            filterListView.Items[72].Tag = 28000;
            filterListView.Items[73].Tag = 28001;
            filterListView.Items[74].Tag = 28002;
            filterListView.Items[75].Tag = 28003;
            filterListView.Items[76].Tag = 28004;

            filterListView.Items[77].Tag = 29000;
            filterListView.Items[78].Tag = 29001;
            filterListView.Items[79].Tag = 29002;
            filterListView.Items[80].Tag = 29003;
            filterListView.Items[81].Tag = 29004;

            UpdateLogStatus();
            UpdatePolicies();

            /*
            AuditSignatureType signature;
            r = system.AuditLog.ExportAuditLogSignature(0, out signature); // RSA_SHA1
            if (r == AmtCallStatus.SUCCESS && signature != null)
            {
                mainTextBox.AppendText("--- Signature ---\r\n");
                mainTextBox.AppendText(string.Format("EndLogTime: {0}\r\n", signature.EndLogTime));
                mainTextBox.AppendText(string.Format("FQDN: {0}\r\n", signature.FQDN));
                mainTextBox.AppendText(string.Format("GenerationTime: {0}\r\n", signature.GenerationTime));
                mainTextBox.AppendText(string.Format("LengthOfCertificates: {0}\r\n", signature.LengthOfCertificates.Length));
                mainTextBox.AppendText(string.Format("Signature: {0}\r\n", Utils.BytesToHex(signature.Signature)));
                mainTextBox.AppendText(string.Format("SignatureMechanism: {0}\r\n", signature.SignatureMechanism));
                mainTextBox.AppendText(string.Format("StartLogTime: {0}\r\n", signature.StartLogTime));
                mainTextBox.AppendText(string.Format("TotalRecordCount: {0}\r\n", signature.TotalRecordCount));
                mainTextBox.AppendText(string.Format("UUID: {0}\r\n", Utils.BytesToHex(signature.UUID)));
                mainTextBox.AppendText("\r\n");
            }
            */
            
        }

        private void refreshStatusButton_Click(object sender, EventArgs e)
        {
            system.Info.ClearAuditLogStatusCache();
            UpdateLogStatus();
        }

        private void setCertificateButton_Click(object sender, EventArgs e)
        {
            ManageabilityControls.NewCertificateForm form = new ManageabilityControls.NewCertificateForm();
            form.CertificateName = Properties.Resources.IntelAmtAuditLogCertName;
            form.LockKeySize(2048);
            form.SHA2Support = (system.CoreVersion >= 6.0 ? true : false);
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                AuditKeyingMaterialType keyAndCert = new AuditKeyingMaterialType();

                if (form.Certificate.SignatureAlgorithm.FriendlyName.Contains("sha256"))
                    keyAndCert.SignatureMechanism = 1; // RSA_SHA256
                else if (form.Certificate.SignatureAlgorithm.FriendlyName.Contains("sha384"))
                    keyAndCert.SignatureMechanism = 2; // RSA_SHA384
                else
                    keyAndCert.SignatureMechanism = 0; // RSA_SHA1

                keyAndCert.SigningKey = new KeyPairType();
                keyAndCert.SigningKey.Item = new RSAKeyPairType();
                keyAndCert.SigningKey.Item.DERKey = ManageabilityStack.CertificateOperations.GetPkcs1PrivateKeyFromCert(form.Certificate);
                byte[] cert = form.Certificate.RawData;
                keyAndCert.Certificates = cert;
                keyAndCert.LengthOfCertificates = new ushort[1];
                keyAndCert.LengthOfCertificates[0] = (ushort)(cert.Length);
                AmtCallStatus r = system.AuditLog.SetSigningKeyingMaterial(keyAndCert);
                if (r != AmtCallStatus.SUCCESS) Utils.ErrorMessageBox(this, "SetSigningKeyingMaterial", r);
                system.Info.ClearAuditLogStatusCache();
                UpdateLogStatus();
            }
        }

        private void enableButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus r;
            AmtGeneralInfo.AmtAuditLogStatus status;
            r = system.Info.GetAuditLogStatus(out status);
            if (r == AmtCallStatus.SUCCESS && status != null)
            {
                r = system.AuditLog.EnableAuditing(status.Disabled);
                if (r != AmtCallStatus.SUCCESS) Utils.ErrorMessageBox(this, "EnableAuditing", r);
            }
            else
            {
                Utils.ErrorMessageBox(this, "GetAuditLogStatus", r);
            }
            system.Info.ClearAuditLogStatusCache();
            UpdateLogStatus();
        }

        private void auditLogButton_Click(object sender, EventArgs e)
        {
            AuditLogLockForm form = new AuditLogLockForm(system);
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                system.Info.ClearAuditLogStatusCache();
                UpdateLogStatus();
            }
        }

        private void filterListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            setCriticalButton.Enabled = setEnabledButton.Enabled = setDisabledButton.Enabled = (filterListView.SelectedItems.Count > 0);
            disableToolStripMenuItem.Enabled = enableToolStripMenuItem.Enabled = criticalToolStripMenuItem.Enabled = (filterListView.SelectedItems.Count > 0);
        }

        private void setCriticalButton_Click(object sender, EventArgs e)
        {
            setFilters(true, 1);
        }

        private void setEnabledButton_Click(object sender, EventArgs e)
        {
            setFilters(true, 0);
        }

        private void setDisabledButton_Click(object sender, EventArgs e)
        {
            setFilters(false, 0);
        }

        private void setFilters(bool state, uint critical)
        {
            if (filterListView.SelectedItems.Count == 0) return;

            setCriticalButton.Enabled = setEnabledButton.Enabled = setDisabledButton.Enabled = false;
            disableToolStripMenuItem.Enabled = enableToolStripMenuItem.Enabled = criticalToolStripMenuItem.Enabled = false;
            AmtCallStatus r;
            foreach (ListViewItem l in filterListView.SelectedItems)
            {
                ushort e1 = (ushort)((int)(l.Tag));
                AuditPolicyType alp = new AuditPolicyType();
                alp.AuditedAppID = (ushort)(e1 / 1000);
                alp.EventId = (ushort)(e1 % 1000);
                alp.Flag = critical;
                alp.FlagSpecified = true;
                r = system.AuditLog.SetAuditPolicy(state, alp);
                if (r == AmtCallStatus.SUCCESS)
                {
                    if (state == true && critical == 1) { l.SubItems[1].Text = Properties.Resources.Critical; l.ImageIndex = 1; }
                    else if (state == true && critical == 0) { l.SubItems[1].Text = Properties.Resources.Enabled; l.ImageIndex = 0; }
                    else if (state == false) { l.SubItems[1].Text = Properties.Resources.Disabled; l.ImageIndex = 3; }
                }
                Application.DoEvents();
            }
            UpdatePolicies();
            setCriticalButton.Enabled = setEnabledButton.Enabled = setDisabledButton.Enabled = (filterListView.SelectedItems.Count > 0);
            disableToolStripMenuItem.Enabled = enableToolStripMenuItem.Enabled = criticalToolStripMenuItem.Enabled = (filterListView.SelectedItems.Count > 0);
            filterListView.Focus();
        }

        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            filterListView.Focus();
            SendKeys.Send("{HOME}");
            SendKeys.Send("+{END}");
        }

        private void storagePolicyButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus r;
            AmtAuditLog.AmtStoragePolicyType storagePolicy = AmtAuditLog.AmtStoragePolicyType.NO_ROLL_OVER;
            uint threshold = 1;

            AuditLogStoragePolicyForm storagePolicyForm = new AuditLogStoragePolicyForm();

            // Get the current storage policy and setup the dialog.
            r = system.AuditLog.GetStoragePolicy(out storagePolicy, out threshold);
            if (r != AmtCallStatus.SUCCESS)
            {
                Utils.ErrorMessageBox(this, "GetStoragePolicy", r);
                return;
            }

            storagePolicyForm.StoragePolicy = storagePolicy;
            storagePolicyForm.Threshold = threshold;
            if (storagePolicyForm.ShowDialog() == DialogResult.OK)
            {
                storagePolicy = storagePolicyForm.StoragePolicy;
                threshold = storagePolicyForm.Threshold;
                r = system.AuditLog.SetStoragePolicy(storagePolicy, threshold);
                if (r != AmtCallStatus.SUCCESS) Utils.ErrorMessageBox(this, "SetStoragePolicy", r);
            }
        }
    }
}
