// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class BatchForm : Form
    {
        private MainForm parentForm = null;
        private Thread statusThread = null;
        private Hashtable systemsTable = null;
        private string floppyImagePath = null;
        private string cdromImagePath = null;

        public BatchForm(MainForm parent)
        {
            parentForm = parent;
            InitializeComponent();
        }

        public void SetSystems(AmtSystem[] systems)
        {
            systemsTable = new Hashtable();
            lock (mainListView)
            {
                mainListView.Items.Clear();
                foreach (AmtSystem system in systems)
                {
                    int icon = 1;
                    if (system.State == AmtSystemObjState.Connected)
                    {
                        icon = 0;
                        if (system.Warnings != 0) icon = 2;
                    }
                    if (system.IsMobile) icon += 3;
                    system.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                    ListViewItem l = new ListViewItem(new string[] { system.HostNameIdentity.DefaultName + " / " + system.Username, "", "", "" }, icon);
                    l.Tag = system;
                    mainListView.Items.Add(l);
                    systemsTable.Add(system, l);
                    UpdateSystemEx(system, null);
                }
            }
            mainListView_SelectedIndexChanged(this, null);
            FetchState();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void BatchForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            foreach (AmtSystem system in systemsTable.Keys)
            {
                system.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
            }
            systemsTable.Clear();
            mainListView.Items.Clear();
            parentForm.batchForm = null;
        }

        private void FetchState()
        {
            if (statusThread != null) return;
            statusThread = new Thread(new ThreadStart(FetchStateEx));
            statusThread.Start();
        }

        private void FetchStateEx()
        {
            Hashtable systemsTableCopy = (Hashtable)systemsTable.Clone();

            foreach (AmtSystem system in systemsTableCopy.Keys)
            {
                if (system.State == AmtSystemObjState.Connected)
                {
                    AmtRemoteControl.AmtPowerStates PowerState = AmtRemoteControl.AmtPowerStates.UNKNOWN15;
                    bool g_BatteryPower = false;
                    bool g_OsLockup = false;
                    AmtCallStatus r = system.Remote.GetSystemPowerStateEx(out PowerState, out g_BatteryPower, out g_OsLockup);

                    string s = Utils.PowerStatesShort[(int)PowerState];
                    if (g_BatteryPower) s += Properties.Resources.ShortBatteryIndicator;
                    if (g_OsLockup) s += Properties.Resources.ShortLockupIndicator;
                    UpdateSystem(system, s);
                }
            }

            systemsTableCopy = null;
            statusThread = null;
        }

        private delegate void UpdateSystemHandler(AmtSystem system, string power);
        private void UpdateSystem(AmtSystem system, string power)
        {
            object[] p = new object[2];
            p[0] = system;
            p[1] = power;
            try
            {
                Invoke(new UpdateSystemHandler(UpdateSystemEx), p);
            }
            catch (InvalidOperationException) {}
        }

        private void UpdateSystemEx(AmtSystem system, string power)
        {
            ListViewItem l = (ListViewItem)systemsTable[system];
            if (system.State == AmtSystemObjState.Connected)
            {
                int icon = 0;
                if (system.Warnings != 0) icon = 2;
                if (system.IsMobile) icon += 3;
                l.ImageIndex = icon;

                if (power != null && l.SubItems[1].Text.CompareTo(power) != 0) l.SubItems[1].Text = power;

                string s2 = system.SecurityAdmin.GetCoreVersion();
                if (system.Assets.GetBios() != null) {s2 = system.SecurityAdmin.GetCoreVersion() + " / " + system.Assets.GetBios().Version;}
                if (l.SubItems[2].Text.CompareTo(s2) != 0) l.SubItems[2].Text = s2;

                string s = "";
                if (system.RedirectorWrapper.RedirectActive == true)
                {
                    AmtRedirectorWrapper.AmtIdeStats stats = system.RedirectorWrapper.GetRedirectorStats();
                    if (stats != null)
                    {
                        s = String.Format(ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R_Format, stats.data_sent.ToString());
                    }
                    else
                    {
                        s = ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R;
                    }
                }
                else
                {
                    s = system.State.ToString();
                }
                if (l.SubItems[3].Text.CompareTo(s) != 0) l.SubItems[3].Text = s;
            }
            else
            {
                int icon = 1;
                if (system.IsMobile) icon += 3;
                l.ImageIndex = icon;
                l.SubItems[1].Text = "";
                l.SubItems[2].Text = "";
                l.SubItems[3].Text = system.State.ToString();
            }
        }

        private void getPowerStatesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FetchState();
        }

        private void SystemStateChangedHandler(AmtSystem system)
        {
            this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { system });
        }

        private void SystemStateChangedHandlerEx(AmtSystem computer)
        {
            UpdateSystem(computer, null);
            FetchState();
        }

        private void connectSelectedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                AmtSystem computer = (AmtSystem)l.Tag;
                computer.Connect();
            }
        }

        private void disconnectSelectedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                AmtSystem computer = (AmtSystem)l.Tag;
                ManageabilityControls.TerminalForm.DisposeRemotingForm(computer);
                computer.Disconnect();
            }
        }

        private void mainListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Main Menu
            controlTerminalToolStripMenuItem.Enabled = (mainListView.SelectedItems.Count == 1 && IsSelectionConnected());

            // Context Menu
            connectToolStripMenuItem.Visible = (mainListView.SelectedItems.Count > 0);
            disconnectToolStripMenuItem.Visible = (mainListView.SelectedItems.Count > 0);
            controlTerminalToolStripMenuItem1.Enabled = (mainListView.SelectedItems.Count == 1 && IsSelectionConnected());
        }

        private void powerUpSelectedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            if (MessageBox.Show(this,
                ManageabilityCommanderTool.Properties.Resources.BatchForm_Query_PwrUp,
                ManageabilityCommanderTool.Properties.Resources.BatchForm_R_PwrUp,
                MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.No) return;
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                AmtSystem computer = (AmtSystem)l.Tag;
                AmtCallStatus s = computer.Remote.SendRemoteControl(0x11, 0x157, 0x0, 0x0, 0x0, 0x0);
            }
        }

        private void powerDownSelectedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            if (MessageBox.Show(this,
                ManageabilityCommanderTool.Properties.Resources.BatchForm_Query_PwrDwn,
                ManageabilityCommanderTool.Properties.Resources.BatchForm_R_PwrDwn,
                MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.No) return;
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                AmtSystem computer = (AmtSystem)l.Tag;
                AmtCallStatus s = computer.Remote.SendRemoteControl(0x12, 0x157, 0x0, 0x0, 0x0, 0x0);
            }
        }

        private void bootNormallyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            if (MessageBox.Show(this,
                ManageabilityCommanderTool.Properties.Resources.BatchForm_Query_Reboot,
                ManageabilityCommanderTool.Properties.Resources.BatchForm_R_Reboot,
                MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.No) return;
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                AmtSystem computer = (AmtSystem)l.Tag;
                AmtRemoteControl.AmtPowerStates PowerState = computer.Remote.GetSystemPowerState();
                byte command = 0x11; // Powerup
                if (PowerState == AmtRemoteControl.AmtPowerStates.S0_POWERED || PowerState == AmtRemoteControl.AmtPowerStates.S1_SLEEPING_WITH_CONTEXT) command = 0x10; // Reset
                AmtCallStatus s = computer.Remote.SendRemoteControl(command, 0x157, 0x0, 0x0, 0x0, 0x0);
            }
        }

        private void bootSelectedToRedirectBIOSToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            if (MessageBox.Show(this,
                ManageabilityCommanderTool.Properties.Resources.BatchForm_Query_Reboot,
                ManageabilityCommanderTool.Properties.Resources.BatchForm_R_Reboot,
                MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.No) return;
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                AmtSystem computer = (AmtSystem)l.Tag;
                AmtRemoteControl.AmtPowerStates PowerState = computer.Remote.GetSystemPowerState();
                byte command = 0x11; // Powerup
                if (PowerState == AmtRemoteControl.AmtPowerStates.S0_POWERED || PowerState == AmtRemoteControl.AmtPowerStates.S1_SLEEPING_WITH_CONTEXT) command = 0x10; // Reset
                AmtCallStatus s = computer.Remote.SendRemoteControl(command, 0x157, 0xc1, 0x8, 0x0, 0x1);
            }
        }

        private void bootSelectedToRedirectFloppyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            if (MessageBox.Show(this,
                ManageabilityCommanderTool.Properties.Resources.BatchForm_Query_Reboot_Floppy,
                ManageabilityCommanderTool.Properties.Resources.BatchForm_R_Reboot,
                MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.No) return;
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                AmtSystem computer = (AmtSystem)l.Tag;
                AmtRemoteControl.AmtPowerStates PowerState = computer.Remote.GetSystemPowerState();
                byte command = 0x11; // Powerup
                if (PowerState == AmtRemoteControl.AmtPowerStates.S0_POWERED || PowerState == AmtRemoteControl.AmtPowerStates.S1_SLEEPING_WITH_CONTEXT) command = 0x10; // Reset
                AmtCallStatus s = computer.Remote.SendRemoteControl(command, 0x157, 0xc1, 0x1, 0x0, 0x1);
            }
        }

        private void bootSelectedToRedirectCDROMToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count == 0) return;
            if (MessageBox.Show(this,
                ManageabilityCommanderTool.Properties.Resources.BatchForm_Query_Reboot_CDROM,
                ManageabilityCommanderTool.Properties.Resources.BatchForm_R_Reboot,
                MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.No) return;
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                AmtSystem computer = (AmtSystem)l.Tag;
                AmtRemoteControl.AmtPowerStates PowerState = computer.Remote.GetSystemPowerState();
                byte command = 0x11; // Powerup
                if (PowerState == AmtRemoteControl.AmtPowerStates.S0_POWERED || PowerState == AmtRemoteControl.AmtPowerStates.S1_SLEEPING_WITH_CONTEXT) command = 0x10; // Reset
                AmtCallStatus s = computer.Remote.SendRemoteControl(command, 0x157, 0xc1, 0x101, 0x0, 0x1);
            }
        }

        private bool IsSelectionConnected()
        {
            if (mainListView.SelectedItems.Count == 0) return false;

            bool r = true;
            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                AmtSystem computer = (AmtSystem)l.Tag;
                if (computer.State != AmtSystemObjState.Connected) r = false;
            }
            return r;
        }

        private void operationsToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            bool c = IsSelectionConnected();
            powerUpSelectedToolStripMenuItem.Enabled = c;
            powerDownSelectedToolStripMenuItem.Enabled = c;
            bootNormallyToolStripMenuItem.Enabled = c;
            bootSelectedToRedirectBIOSToolStripMenuItem.Enabled = c;
            bootSelectedToRedirectFloppyToolStripMenuItem.Enabled = c;
            bootSelectedToRedirectCDROMToolStripMenuItem.Enabled = c;

            connectSelectedToolStripMenuItem.Enabled = mainListView.SelectedItems.Count != 0;
            disconnectSelectedToolStripMenuItem.Enabled = mainListView.SelectedItems.Count != 0;
        }

        private uint old_data_sent = 0;
        private uint old_data_received = 0;
        private void refreshTimer_Tick(object sender, EventArgs e)
        {
            bool redirectorInUse = false;
            uint datain = 0;
            uint dataout = 0;

            foreach (AmtSystem system in systemsTable.Keys)
            {
                if (system.State == AmtSystemObjState.Connected)
                {
                    UpdateSystem(system, null);
                    if (system.RedirectorWrapper.RedirectActive == true)
                    {
                        redirectorInUse = true;
                        AmtRedirectorWrapper.AmtIdeStats stats = system.RedirectorWrapper.GetRedirectorStats();
                        if (stats != null)
                        {
                            datain += stats.data_received;
                            dataout += stats.data_sent;
                        }
                    }
                }
            }

            if (redirectorInUse == false)
            {
                redirectStatusLabel.Text = ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R_Dis;
                //redirectStatusLabel.Text = "IDE Redirect Disabled";
                if (harddiskPictureBox.Image != hdPictureBox1.Image) harddiskPictureBox.Image = hdPictureBox1.Image;
                return;
            }

            // Set IDE Redirect
            string s = "";
            //s += "IDE Redirect Active";
            s += String.Format(ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R_Act_Format, dataout.ToString(), datain.ToString());
            if (old_data_received != datain)
            {
                if (harddiskPictureBox.Image != hdPictureBox4.Image) harddiskPictureBox.Image = hdPictureBox4.Image;
            }
            else if (old_data_sent != dataout)
            {
                if (harddiskPictureBox.Image != hdPictureBox3.Image) harddiskPictureBox.Image = hdPictureBox3.Image;
            }
            else
            {
                if (harddiskPictureBox.Image != hdPictureBox2.Image) harddiskPictureBox.Image = hdPictureBox2.Image;
            }
            redirectStatusLabel.Text = s;

            old_data_sent = dataout;
            old_data_received = datain;
        }

        private void activateIDEROnSelectedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (IsSelectionConnected() == false) return;


            if (floppyImagePath == null || cdromImagePath == null)
            {
                MessageBox.Show(this,
                    ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R_FL_CD,
                    ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R,
                    MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
            else if (floppyImagePath.Length != 2 && new FileInfo(floppyImagePath).Exists == false)
            {
                MessageBox.Show(this,
                    ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R_FL,
                    ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R,
                    MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
            else if (cdromImagePath.Length != 2 && new FileInfo(cdromImagePath).Exists == false)
            {
                MessageBox.Show(this,
                    ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R_CD,
                    ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R,
                    MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
            }
            else
            {
                AmtRedirectorWrapper.RedirectState rs = AmtRedirectorWrapper.RedirectState.IDER_SET_ONRESET;
                if (gracefullyToolStripMenuItem.Checked == true) rs = AmtRedirectorWrapper.RedirectState.IDER_SET_GRACEFULLY;
                if (immediatelyToolStripMenuItem.Checked == true) rs = AmtRedirectorWrapper.RedirectState.IDER_SET_IMMEDIATELY;
                if (onResetToolStripMenuItem.Checked == true) rs = AmtRedirectorWrapper.RedirectState.IDER_SET_ONRESET;

                foreach (ListViewItem l in mainListView.SelectedItems)
                {
                    AmtSystem computer = (AmtSystem)l.Tag;
                    if (computer.RedirectorWrapper.RedirectActive == false)
                    {
                        computer.RedirectorWrapper.StartRedirect(floppyImagePath, cdromImagePath, rs);
                    }
                    UpdateSystemEx(computer, null);
                }
            }
        }

        private void deactivateIDEROnSelectedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (IsSelectionConnected() == false) return;

            foreach (ListViewItem l in mainListView.SelectedItems)
            {
                AmtSystem computer = (AmtSystem)l.Tag;
                computer.RedirectorWrapper.StopRedirect();
                UpdateSystemEx(computer, null);
            }
        }

        private void redirectToImageFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (floppyImagePath != null && floppyImagePath.Length > 2) imageFileDialog.FileName = floppyImagePath;
            imageFileDialog.Filter = ManageabilityCommanderTool.Properties.Resources.BatchForm_FDialog_IMG;
            if (imageFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                FileInfo fi = new FileInfo(imageFileDialog.FileName);
                if (fi.Exists == false)
                {
                    MessageBox.Show(this,
                        ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R_File,
                        ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R,
                        MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
                else
                {
                    floppyPathLabel.Text = fi.Name;
                    floppyImagePath = imageFileDialog.FileName;
                    Application.UserAppDataRegistry.SetValue("FloppyRedirect", floppyImagePath);
                    foreach (object z in changeTargetFloppyToolStripMenuItem.DropDownItems)
                    {
                        if (z.GetType() == typeof(ToolStripMenuItem))
                        {
                            ToolStripMenuItem x = (ToolStripMenuItem)z;
                            x.Checked = (sender == x);
                        }
                    }
                }
            }
        }

        private void redirectToImageFileToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (cdromImagePath != null && cdromImagePath.Length > 2) imageFileDialog.FileName = cdromImagePath;
            imageFileDialog.Filter = ManageabilityCommanderTool.Properties.Resources.BatchForm_FDialog_CD;
            if (imageFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                FileInfo fi = new FileInfo(imageFileDialog.FileName);
                if (fi.Exists == false)
                {
                    MessageBox.Show(this,
                        ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R_File,
                        ManageabilityCommanderTool.Properties.Resources.BatchForm_IDE_R,
                        MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                }
                else
                {
                    cdromPathLabel.Text = fi.Name;
                    cdromImagePath = imageFileDialog.FileName;
                    Application.UserAppDataRegistry.SetValue("CdromRedirect", cdromImagePath);
                    foreach (object z in changeTargetCDROMToolStripMenuItem.DropDownItems)
                    {
                        if (z.GetType() == typeof(ToolStripMenuItem))
                        {
                            ToolStripMenuItem x = (ToolStripMenuItem)z;
                            x.Checked = (sender == x);
                        }
                    }
                }
            }
        }

        private void BatchForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 
            switch ((int)Application.UserAppDataRegistry.GetValue("RedirectStart", 0))
            {
                case 1:
                    immediatelyToolStripMenuItem.Checked = true;
                    gracefullyToolStripMenuItem.Checked = false;
                    onResetToolStripMenuItem.Checked = false;
                    break;
                case 2:
                    immediatelyToolStripMenuItem.Checked = false;
                    gracefullyToolStripMenuItem.Checked = true;
                    onResetToolStripMenuItem.Checked = false;
                    break;
                case 3:
                    immediatelyToolStripMenuItem.Checked = false;
                    gracefullyToolStripMenuItem.Checked = false;
                    onResetToolStripMenuItem.Checked = true;
                    break;
            }

            floppyImagePath = (string)Application.UserAppDataRegistry.GetValue("FloppyRedirect", null);
            if (floppyImagePath != null)
            {
                if (floppyImagePath.Length > 2)
                {
                    floppyPathLabel.Text = new FileInfo(floppyImagePath).Name;
                }
                else
                {
                    floppyPathLabel.Text = floppyImagePath;
                }
                redirectToImageFileToolStripMenuItem.Checked = floppyImagePath.Length > 2;
            }
            cdromImagePath = (string)Application.UserAppDataRegistry.GetValue("CdromRedirect", null);
            if (cdromImagePath != null)
            {
                if (cdromImagePath.Length > 2)
                {
                    cdromPathLabel.Text = new FileInfo(cdromImagePath).Name;
                }
                else
                {
                    cdromPathLabel.Text = cdromImagePath;
                }
                redirectToImageFileToolStripMenuItem1.Checked = cdromImagePath.Length > 2;
            }

            toolStripMenuItem4.Visible = false;
            toolStripMenuItem5.Visible = false;
            if (!Utils.IsMono())
            {
                for (char d = 'A'; d <= 'Z'; d++)
                {
                    DriveInfo drive = new DriveInfo(d.ToString());
                    if (drive.DriveType == DriveType.Removable && (d == 'A' || d == 'B'))
                    {
                        ToolStripMenuItem x = new ToolStripMenuItem(String.Format(
                            ManageabilityCommanderTool.Properties.Resources.BatchForm_Redir_Str_Format, d.ToString()));
                        x.Click += new EventHandler(redirectDriveMenuItem1_Click);
                        changeTargetFloppyToolStripMenuItem.DropDownItems.Insert(0, x);
                        toolStripMenuItem4.Visible = true;
                        if (floppyImagePath == null)
                        {
                            floppyImagePath = d.ToString() + ":";
                            floppyPathLabel.Text = floppyImagePath;
                            x.Checked = true;
                        }
                        else
                        {
                            if (floppyImagePath != null && floppyImagePath.Length == 2 && floppyImagePath[0] == d) x.Checked = true;
                        }
                    }
                    if (drive.DriveType == DriveType.CDRom)
                    {
                        ToolStripMenuItem x = new ToolStripMenuItem(String.Format(
                            ManageabilityCommanderTool.Properties.Resources.BatchForm_Redir_Str_Format, d.ToString()));
                        x.Click += new EventHandler(redirectDriveMenuItem2_Click);
                        changeTargetCDROMToolStripMenuItem.DropDownItems.Insert(0, x);
                        toolStripMenuItem5.Visible = true;
                        if (cdromImagePath == null)
                        {
                            cdromImagePath = d.ToString() + ":";
                            cdromPathLabel.Text = cdromImagePath;
                            x.Checked = true;
                        }
                        else
                        {
                            if (cdromImagePath != null && cdromImagePath.Length == 2 && cdromImagePath[0] == d) x.Checked = true;
                        }
                    }
                }
            }
        }

        private void redirectDriveMenuItem1_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem menuitem = (ToolStripMenuItem)sender;
            floppyPathLabel.Text = menuitem.Text.Substring(12, 2);
            floppyImagePath = floppyPathLabel.Text;
            Application.UserAppDataRegistry.SetValue("FloppyRedirect", floppyImagePath);
            foreach (object z in changeTargetFloppyToolStripMenuItem.DropDownItems)
            {
                if (z.GetType() == typeof(ToolStripMenuItem))
                {
                    ToolStripMenuItem x = (ToolStripMenuItem)z;
                    x.Checked = (menuitem == x);
                }
            }
        }

        private void redirectDriveMenuItem2_Click(object sender, EventArgs e)
        {
            ToolStripMenuItem menuitem = (ToolStripMenuItem)sender;
            cdromPathLabel.Text = menuitem.Text.Substring(12, 2);
            cdromImagePath = cdromPathLabel.Text;
            Application.UserAppDataRegistry.SetValue("CdromRedirect", cdromImagePath);
            foreach (object z in changeTargetCDROMToolStripMenuItem.DropDownItems)
            {
                if (z.GetType() == typeof(ToolStripMenuItem))
                {
                    ToolStripMenuItem x = (ToolStripMenuItem)z;
                    x.Checked = (menuitem == x);
                }
            }
        }

        private void immediatelyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.UserAppDataRegistry.SetValue("RedirectStart", 1);
            immediatelyToolStripMenuItem.Checked = true;
            gracefullyToolStripMenuItem.Checked = false;
            onResetToolStripMenuItem.Checked = false;
        }

        private void gracefullyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.UserAppDataRegistry.SetValue("RedirectStart", 2);
            immediatelyToolStripMenuItem.Checked = false;
            gracefullyToolStripMenuItem.Checked = true;
            onResetToolStripMenuItem.Checked = false;
        }

        private void onResetToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.UserAppDataRegistry.SetValue("RedirectStart", 3);
            immediatelyToolStripMenuItem.Checked = false;
            gracefullyToolStripMenuItem.Checked = false;
            onResetToolStripMenuItem.Checked = true;
        }

        private void redirectionToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            activateIDEROnSelectedToolStripMenuItem.Enabled = IsSelectionConnected();
            deactivateIDEROnSelectedToolStripMenuItem.Enabled = IsSelectionConnected();
        }

        private void mainListView_DoubleClick(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count != 1) return;
            AmtSystem computer = (AmtSystem)mainListView.SelectedItems[0].Tag;

            if (computer.State == AmtSystemObjState.Disconnected)
            {
                computer.Connect();
            }
            else if (computer.State == AmtSystemObjState.Connected)
            {
                ManageabilityControls.TerminalForm.ShowRemotingForm(computer, this, false);
            }
        }

        private void controlTerminalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainListView.SelectedItems.Count != 1) return;
            AmtSystem computer = (AmtSystem)mainListView.SelectedItems[0].Tag;
            if (computer.State == AmtSystemObjState.Connected) ManageabilityControls.TerminalForm.ShowRemotingForm(computer, this, false);
        }

    }
}
