// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class CertificateStoreForm : Form
    {
        private AmtSystem computer;
        private X509Store CertificateStore;
        private X509Store CertificateRootStore = null;

        public CertificateStoreForm(AmtSystem computer)
        {
            this.computer = computer;
            InitializeComponent();
        }

        private void CertificateStoreForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;

            Text += " - " + computer.HostNameIdentity.DefaultName;
            UpdateCerts();

            // Open the certificate stores
            CertificateStore = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadWrite);
            CertificateRootStore = new X509Store(StoreName.Root, StoreLocation.CurrentUser);
            CertificateRootStore.Open(OpenFlags.ReadWrite);

            // Load the list of trusted root certificates
            foreach (X509Certificate2 tcert in CertificateRootStore.Certificates)
            {
                trustedRootComboBox.Items.Add(new Utils.DropDownItem(ManageabilityStack.CertificateOperations.GetCertificateString(tcert), 0, tcert));
            }
            if (trustedRootComboBox.Items.Count > 0) trustedRootComboBox.SelectedIndex = 0;
        }

        private void UpdateCerts()
        {
            certificateListView.BeginUpdate();
            certificateListView.Items.Clear();

            // Get the TLS certificate handle
            uint tlsHandle;
            bool tlsHandleSpecified;
            computer.SecurityAdmin.GetTLSCredentials(out tlsHandle, out tlsHandleSpecified);

            AmtSecurityAdmin.AmtStoredCert[] certs;
            if (computer.SecurityAdmin.CertStoreGetCertificates(out certs) == AmtCallStatus.SUCCESS)
            {
                foreach (AmtSecurityAdmin.AmtStoredCert cert in certs)
                {
                    int icon = 0;
                    if (CertificateOperations.IsValidCert(cert.Certificate) == false) icon = 1;
                    string tls = "";
                    if (tlsHandleSpecified == true && cert.Handle == tlsHandle) tls = " (TLS)";
                    ListViewItem l = new ListViewItem(new string[] { ManageabilityStack.CertificateOperations.GetCertificateLongString(cert.Certificate) + tls }, icon);
                    l.Tag = cert;
                    certificateListView.Items.Add(l);
                }
            }
            certificateListView.EndUpdate();

            trustedListView.BeginUpdate();
            trustedListView.Items.Clear();
            if (computer.SecurityAdmin.CertStoreGetTrustedCertificates(out certs) == AmtCallStatus.SUCCESS)
            {
                foreach (AmtSecurityAdmin.AmtStoredCert cert in certs)
                {
                    int icon = 0;
                    if (CertificateOperations.IsValidCert(cert.Certificate) == false) icon = 1;
                    ListViewItem l = new ListViewItem(new string[] { ManageabilityStack.CertificateOperations.GetCertificateString(cert.Certificate) }, icon);
                    l.Tag = cert;
                    trustedListView.Items.Add(l);
                }
            }
            trustedListView.EndUpdate();

            crlListView.BeginUpdate();
            crlListView.Items.Clear();
            CrlType[] crl;
            if (computer.SecurityAdmin.GetCRL(out crl) == AmtCallStatus.SUCCESS)
            {
                foreach (CrlType rl in crl)
                {
                    ListViewItem l = new ListViewItem(new string[] { rl.CrlUrl, rl.SerialNumber.Length.ToString() });
                    l.Tag = rl;
                    trustedListView.Items.Add(l);
                }
            }
            crlListView.EndUpdate();
        }

        private void certificateListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            removeButton.Enabled = certViewButton.Enabled = (certificateListView.SelectedItems.Count > 0);
        }

        private void certViewButton_Click(object sender, EventArgs e)
        {
            if (certificateListView.SelectedItems.Count == 0) return;
            AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)certificateListView.SelectedItems[0].Tag;
            X509Certificate2UI.DisplayCertificate(cert.Certificate);
        }

        private void certificateListView_DoubleClick(object sender, EventArgs e)
        {
            certViewButton_Click(this, null);
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (certificateListView.SelectedItems.Count == 0) return;
            AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)certificateListView.SelectedItems[0].Tag;
            if (MessageBox.Show(this, Properties.Resources.RemoveSelectedCertificateQ, Properties.Resources.RemoveCertificate, MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk) == DialogResult.OK)
            {
                AmtCallStatus r = computer.SecurityAdmin.CertStoreRemoveCertificate(cert);
                if (r != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show(this, string.Format(Properties.Resources.RemoveCertificateError, r.ToString()), Properties.Resources.RemoveCertificate, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    UpdateCerts();
                }
            }
        }

        private void trustedViewButton_Click(object sender, EventArgs e)
        {
            if (trustedListView.SelectedItems.Count == 0) return;
            AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)trustedListView.SelectedItems[0].Tag;
            X509Certificate2UI.DisplayCertificate(cert.Certificate);
        }

        private void trustedListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            trustedRemoveButton.Enabled = trustedViewButton.Enabled = (trustedListView.SelectedItems.Count > 0);
        }

        private void CertificateStoreForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            CertificateStore.Close();
            CertificateRootStore.Close();
        }

        private void trustedAddButton_Click(object sender, EventArgs e)
        {
            if (trustedRootComboBox.SelectedItem == null) return;
            X509Certificate2 tcert = (X509Certificate2)((Utils.DropDownItem)(trustedRootComboBox.SelectedItem)).Tag;
            if (computer.SecurityAdmin.CertStoreAddTrustedRootCertificate(tcert) == AmtCallStatus.SUCCESS) UpdateCerts();
        }

        private void trustedRemoveButton_Click(object sender, EventArgs e)
        {
            if (trustedListView.SelectedItems.Count != 1) return;
            AmtSecurityAdmin.AmtStoredCert cert = (AmtSecurityAdmin.AmtStoredCert)trustedListView.SelectedItems[0].Tag;
            if (MessageBox.Show(this, Properties.Resources.RemoveSelectedCertificateQ, Properties.Resources.RemoveCertificate, MessageBoxButtons.OKCancel, MessageBoxIcon.Asterisk) == DialogResult.OK)
            {
                AmtCallStatus r = computer.SecurityAdmin.CertStoreRemoveTrustedCertificate(cert);
                if (r != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show(this, string.Format(Properties.Resources.RemoveCertificateError, r.ToString()), Properties.Resources.RemoveCertificate, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    UpdateCerts();
                }
            }
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            ManageabilityControls.NewCertificateForm form = new ManageabilityControls.NewCertificateForm();
            form.CertificateName = computer.HostNameIdentity.DefaultName;
            form.SHA2Support = (computer.CoreVersion >= 6.0 ? true : false);
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                byte[] privatekey = ManageabilityStack.CertificateOperations.GetPkcs1PrivateKeyFromCert(form.Certificate);
                uint h;
                AmtCallStatus r = computer.SecurityAdmin.CertStoreAddCertificate(form.Certificate, privatekey, out h);
                if (r != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show(this, string.Format(Properties.Resources.AddCertificateError, r.ToString()), Properties.Resources.AddCertificate, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    UpdateCerts();
                }
            }
        }

        private void certificateListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) removeButton_Click(this, null);
        }

        private void trustedListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) trustedRemoveButton_Click(this, null);
        }

        private void importTrustedButton_Click(object sender, EventArgs e)
        {
            if (openTrustedCertFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                X509Certificate2 cert = new X509Certificate2(openTrustedCertFileDialog.FileName);
                if (cert != null)
                {
                    AmtCallStatus r = computer.SecurityAdmin.CertStoreAddTrustedRootCertificate(cert);
                    if (r != AmtCallStatus.SUCCESS)
                    {
                        Utils.ErrorMessageBox(this, "CertStoreAddTrustedRootCertificate", r);
                    }
                    else
                    {
                        UpdateCerts();
                    }
                }
            }
        }

        private void importCertButton_Click(object sender, EventArgs e)
        {
            if (openAuthCertFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                AskCertPasswordForm askpassword = new AskCertPasswordForm();
                if (askpassword.ShowDialog(this) == DialogResult.OK)
                {
                    X509Certificate2 cert = new X509Certificate2(openAuthCertFileDialog.FileName, askpassword.Password, X509KeyStorageFlags.Exportable);
                    if (cert != null)
                    {
                        uint h;
                        byte[] privatekey = ManageabilityStack.CertificateOperations.GetPkcs1PrivateKeyFromCert(cert);
                        AmtCallStatus r = computer.SecurityAdmin.CertStoreAddCertificate(cert, privatekey, out h);
                        if (r != AmtCallStatus.SUCCESS)
                        {
                            Utils.ErrorMessageBox(this, "CertStoreAddCertificate", r);
                        }
                        else
                        {
                            UpdateCerts();
                        }
                    }
                }
            }
        }

        private void certContextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            viewToolStripMenuItem.Visible = removeToolStripMenuItem.Visible = toolStripMenuItem1.Visible = (certificateListView.SelectedItems.Count > 0);
        }

        private void trustedContextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            viewToolStripMenuItem1.Visible = removeToolStripMenuItem1.Visible = toolStripMenuItem2.Visible = (trustedListView.SelectedItems.Count > 0);
        }
    }
}
