// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Text;
using System.Data;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class EditComputerNameForm : Form
    {
        private AmtSystem system;

        private bool _meNameDirty = false;
        private bool _sharedFqdnDirty = false;

        public EditComputerNameForm(AmtSystem system)
        {
            this.system = system;
            InitializeComponent();
        }

        private void EditComputerNameForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon; 

            if (system.AmtRev == 1)
            {
                meNameTextBox.Text = system.Network.GetHostName();
            }
            else
            {
                meNameTextBox.Text = system.Network.GetHostName() + "." + system.Network.GetDomainName();
            }
            connectionNameTextBox.Text = system.HostNameIdentity.DefaultName;
            
            dnsNameTextBox.Text = system.GetDnsName();
            if (dnsNameTextBox.Text.Length == 0) dnsNameTextBox.Text = Properties.Resources.DnsNotFound;

            if (system.AmtRev == 1)
            {
                int i = dnsNameTextBox.Text.IndexOf(".");
                if (i >= 0)
                    dnsNameTextBox.Text = dnsNameTextBox.Text.Substring(0,i);
                i = connectionNameTextBox.Text.IndexOf(".");
                if (i >= 0)
                    connectionNameTextBox.Text = connectionNameTextBox.Text.Substring(0,i);
            }

            X509Certificate2[] certs = certs = system.GetTlsCertificates();

            if (certs != null && certs.Length > 0)
            {
                certificateNameTextBox.Text = CertificateOperations.GetCertificateString(certs[0].Subject);
            }
            else
            {
                certificateNameTextBox.Text = Properties.Resources.NoCertificate;
                label4.Visible = false;
                certificateNameTextBox.Visible = false;
                Height -= (certificateNameTextBox.Top - dnsNameTextBox.Top);
            }


            // Assuming that only AMT6 and later supports the following options.
            if (system.CoreVersion >= 6.0)
            {
                this.sharedFqdnCheckBox.Checked = system.Info.GetSharedFQDN();
                this.sharedFqdnCheckBox.Enabled = false; // Only a changed Intel(R) ME hostname can enable this control.

                bool ddnsUpdateEnabled;
                uint ddnsPeriodicUpdateInterval;
                uint ddnsTtl;
                AmtCallStatus r = system.Info.GetDDNSSettings(out ddnsUpdateEnabled, out ddnsPeriodicUpdateInterval, out ddnsTtl);
                if (r != AmtCallStatus.SUCCESS)
                {
                    if (r != AmtCallStatus.NO_WSMAN) Utils.ErrorMessageBox(this, "GetDDNSSettings", r);
                    this.ddnsUpdateEnabledCheckBox.Checked = false;
                    this.ddnsUpdateEnabledCheckBox.Enabled = false;
                    this.ddnsPeriodicUpdateIntervalNumericUpDown.Enabled = false;
                    this.ddnsTtlNumericUpDown.Enabled = false;
                }
                else
                {
                    this.ddnsUpdateEnabledCheckBox.Checked = ddnsUpdateEnabled;
                    this.ddnsPeriodicUpdateIntervalNumericUpDown.Value = (decimal)ddnsPeriodicUpdateInterval;
                    this.ddnsTtlNumericUpDown.Value = (decimal)ddnsTtl;
                }
            }
            else
            {
                this.sharedFqdnCheckBox.Enabled = false;
                this.ddnsUpdateEnabledCheckBox.Checked = false;
                this.ddnsUpdateEnabledCheckBox.Enabled = false;
                this.ddnsPeriodicUpdateIntervalNumericUpDown.Enabled = false;
                this.ddnsTtlNumericUpDown.Enabled = false;
            }

            // Reset these values after initializing the form.
            this._meNameDirty = false;
            this._sharedFqdnDirty = false;
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus r = AmtCallStatus.SUCCESS;

            // Assuming that only AMT6 and later supports the following options.
            if (system.CoreVersion >= 6.0)
            {
                if (this._meNameDirty && this._sharedFqdnDirty)
                {
                    // NOTE: Assumes that this setting can only be changed when 
                    // the ME hostname is changed.
                    int domainIndex = meNameTextBox.Text.IndexOf(".");
                    string name, domain;
                    if (domainIndex > 0)
                    {
                        name = meNameTextBox.Text.Substring(0, domainIndex);
                        domain = meNameTextBox.Text.Substring(domainIndex + 1);
                    }
                    else
                    {
                        name = meNameTextBox.Text;
                        domain = String.Empty;
                    }
                    r = system.Info.SetSharedFQDN(this.sharedFqdnCheckBox.Checked, name, domain);
                    if ((r != AmtCallStatus.SUCCESS) && (r != AmtCallStatus.NO_WSMAN)) Utils.ErrorMessageBox(this, "SetSharedFQDN", r);
                }
                else if (this._sharedFqdnDirty && !this._meNameDirty)
                {
                    if (MessageBox.Show("Unable to change the \"Shared FQDN\" setting without changing the Intel(R) AMT hostname.",
                        this.Text, MessageBoxButtons.OKCancel, MessageBoxIcon.Warning) == DialogResult.Cancel)
                        return; // Allow the user to update the Intel(R) ME hostname.
                }

                bool ddnsUpdateEnabled = this.ddnsUpdateEnabledCheckBox.Checked;
                uint ddnsPeriodicUpdateInterval = (uint)this.ddnsPeriodicUpdateIntervalNumericUpDown.Value;
                uint ddnsTtl = (uint)this.ddnsTtlNumericUpDown.Value;
                r = system.Info.SetDDNSSettings(ddnsUpdateEnabled, ddnsPeriodicUpdateInterval, ddnsTtl);
                if ((r != AmtCallStatus.SUCCESS) && (r != AmtCallStatus.NO_WSMAN)) Utils.ErrorMessageBox(this, "SetDDNSSettings", r);
            }

            int i = meNameTextBox.Text.IndexOf(".");
            if (i > 0)
            {
                string name = meNameTextBox.Text.Substring(0, i);
                string domain = meNameTextBox.Text.Substring(i + 1);

                r = system.Network.SetHostDomainName(name, domain);
                if (r != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show(this,
                        ManageabilityCommanderTool.Properties.Resources.MsgBox_EditHost_Err + r.ToString(),
                        ManageabilityCommanderTool.Properties.Resources.MsgBox_EditHost_Err_Title,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                DialogResult = DialogResult.OK;
            }
        }

        private void meNameTextBox_TextChanged(object sender, EventArgs e)
        {
            this._meNameDirty = true;
            this.sharedFqdnCheckBox.Enabled = (system.WsMan != null ? true : false); // Assuming that only WSMAN can modify this setting.

            okButton.Enabled = (meNameTextBox.Text.IndexOf(".") > 0);
        }

        private void ddnsUpdateEnabledCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this.ddnsPeriodicUpdateIntervalNumericUpDown.Enabled = this.ddnsUpdateEnabledCheckBox.Checked;
            this.ddnsTtlNumericUpDown.Enabled = this.ddnsUpdateEnabledCheckBox.Checked;
        }

        private void sharedFqdnCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            this._sharedFqdnDirty = true;
        }
    }
}
