﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Windows.Forms;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class EditTLSSettingsForm : Form
    {
        #region Private Members

        private AmtSystem _computer = null;
        private bool _updated = false;
        private bool _useTls = false;
        private bool _useTlsPrev = false;
        private AmtSecurityAdmin.AmtStoredCert _tlsCert = null;
        private bool _useRemoteAuth = false;
        private bool _useLocalAuth = false;
        private string[] _trustedCNs = null;
        private bool _acceptNonSecureConnections = false;

        #endregion

        #region Constructors

        private EditTLSSettingsForm()
        {
            InitializeComponent();
        }

        public EditTLSSettingsForm(AmtSystem computer)
        {
            if (computer == null)
                throw new ArgumentNullException("A valid Intel(R) AMT system is required.");
            if (computer.State != AmtSystemObjState.Connected)
                throw new ArgumentException("The Intel(R) AMT system must be connected.");
            if (computer.SecurityAdmin == null)
                throw new ArgumentNullException("The Intel(R) AMT system must have access to the Security Administration Interface.");

            _computer = computer;

            InitializeComponent();
        }

        #endregion

        #region Public Properties

        public bool UseTls
        {
            get { return _useTls; }
            set { _useTls = value; }
        }

        public bool UseTlsPrev
        {
            get { return _useTlsPrev; }
            set { _useTlsPrev = value; }
        }

        public AmtSystem Computer
        {
            get { return _computer; }
            set { _computer = value; }
        }

        public bool IsUpdated
        {
            get { return _updated; }
            set { _updated = value; }
        }

        public AmtSecurityAdmin.AmtStoredCert TlsCert
        {
            get { return _tlsCert; }
            set { _tlsCert = value; }
        }

        public bool UseRemoteAuth
        {
            get { return _useRemoteAuth; }
            set { _useRemoteAuth = value; }
        }

        public bool UseLocalAuth
        {
            get { return _useLocalAuth; }
            set { _useLocalAuth = value; }
        }

        public string[] TrustedCNs
        {
            get { return _trustedCNs; }
            set { _trustedCNs = value; }
        }

        public bool AcceptNonSecureConnections
        {
            get { return _acceptNonSecureConnections; }
            set { _acceptNonSecureConnections = value; }
        }

        #endregion

        #region Private Methods

        private void EditTLSSettingsForm_Load(object sender, EventArgs e)
        {
            // Update the number of trusted root certificates.
            trustedRootCertNamesLabel.Text = String.Format("{0} Trusted Certificate(s)",
                Computer.SecurityAdmin.GetTrustedRootCertificates().Length);

            // Update the number of trusted certificate CNs.
            TrustedCNs = Computer.SecurityAdmin.GetTrustedFqdnCN();
            trustedCNsLabel.Text = String.Format("{0} Trusted Certificate CN(s)",
                TrustedCNs.Length);

            //
            // Determine the current TLS settings and establish the initial 
            // state of the dialog's controls (enabled, values, etc.).
            //
            TlsAthenticationType local = TlsAthenticationType.NoAuth;
            TlsAthenticationType remote = TlsAthenticationType.NoAuth;

            AmtCallStatus r = Computer.SecurityAdmin.GetTlsOptions(out local, out remote);

            if ((r != AmtCallStatus.SUCCESS) ||
                ((local == TlsAthenticationType.NoAuth) && (remote == TlsAthenticationType.NoAuth)))
            {
                UseTls = false;
                useTlsCheckBox.Checked = false;

                // Disable the remaining controls.
                tlsCertNameLabel.Visible = false;
                tlsCertButton.Enabled = false;
                tlsOptionsGroupBox.Enabled = false;
                useRemoteAuthCheckBox.Enabled = false;
                useLocalAuthCheckBox.Enabled = false;
                trustedRootCertNamesLabel.Visible = false;
                trustedRootCertsButton.Enabled = false;
                trustedCNsLabel.Visible = false;
                trustedCNsButton.Enabled = false;
                acceptNonTlsCheckBox.Enabled = false;

                IsUpdated = false;
                return;
            }

            UseTls = useTlsCheckBox.Checked = true;
            UseTlsPrev = UseTls;
            tlsCertButton.Enabled = false; // Can only be enabled when enabling TLS.

            // The remaining controls should be updated by the 
            // useTlsCheckBox CheckedChanged event handler.

            // Update the name of the certificate used for TLS.
            X509Certificate2[] tlsCert = Computer.SecurityAdmin.GetTLSCertificate();
            if ((tlsCert != null) && (tlsCert.Length > 0))
            {
                tlsCertNameLabel.Text = CertificateOperations.GetCertificateString(tlsCert[0].SubjectName.Name);

                uint handle;
                bool handleSpecified = false;
                r = Computer.SecurityAdmin.GetTLSCredentials(out handle, out handleSpecified);
                if ((r == AmtCallStatus.SUCCESS) && handleSpecified)
                    TlsCert = new AmtSecurityAdmin.AmtStoredCert(handle, tlsCert[0]);

            }

            if (remote == TlsAthenticationType.MutualAuth)
            {
                UseRemoteAuth = true;
                useRemoteAuthCheckBox.Checked = true;
            }
            else
            {
                UseRemoteAuth = false;
                useRemoteAuthCheckBox.Checked = false;
            }

            if (local == TlsAthenticationType.MutualAuth)
            {
                UseLocalAuth = true;
                useLocalAuthCheckBox.Checked = true;
            }
            else
            {
                UseLocalAuth = false;
                useLocalAuthCheckBox.Checked = false;
            }

            if ((Computer.CoreVersion >= 6.0) && (Computer.WsMan != null))
            {
                r = Computer.SecurityAdmin.GetAcceptNonSecureConnections(out _acceptNonSecureConnections);
                // TODO: Report error message?
                acceptNonTlsCheckBox.Checked = AcceptNonSecureConnections;
            }

            IsUpdated = false;
        }

        private void useTlsCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UseTls = useTlsCheckBox.Checked;

            // Update the state of the remaining controls as necessary.
            if (UseTls)
            {
                // Enabled the remaining controls.
                tlsCertNameLabel.Visible = true;
                if (!UseTlsPrev)
                    tlsCertButton.Enabled = true;
                tlsOptionsGroupBox.Enabled = true;
                useRemoteAuthCheckBox.Enabled = true;
                useLocalAuthCheckBox.Enabled = true;
                if (UseRemoteAuth || UseLocalAuth)
                {
                    trustedRootCertNamesLabel.Visible = true;
                    trustedRootCertsButton.Enabled = true;
                    trustedCNsLabel.Visible = true;
                    trustedCNsButton.Enabled = true;
                }

                if (Computer.CoreVersion >= 6.0)
                    acceptNonTlsCheckBox.Enabled = true;
                else
                    acceptNonTlsCheckBox.Enabled = false;
            }
            else
            {
                // Disable the remaining controls.
                tlsCertNameLabel.Visible = false;
                tlsCertButton.Enabled = false;
                tlsOptionsGroupBox.Enabled = false;
                useRemoteAuthCheckBox.Enabled = false;
                useLocalAuthCheckBox.Enabled = false;
                trustedRootCertNamesLabel.Visible = false;
                trustedRootCertsButton.Enabled = false;
                trustedCNsLabel.Visible = false;
                trustedCNsButton.Enabled = false;
                acceptNonTlsCheckBox.Enabled = false;
            }

            IsUpdated = true;
        }

        private void useRemoteAuthCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UseRemoteAuth = useRemoteAuthCheckBox.Checked;

            //
            // Update the state of the corresponding controls as necessary.
            //
            if (UseRemoteAuth || UseLocalAuth)
            {
                // Enable the Trusted Root Certificate and CN controls.
                trustedRootCertNamesLabel.Visible = true;
                trustedRootCertsButton.Enabled = true;
                trustedCNsLabel.Visible = true;
                trustedCNsButton.Enabled = true;
            }
            else
            {
                // Disable the Trusted Root Certificate and CN controls.
                trustedRootCertNamesLabel.Visible = false;
                trustedRootCertsButton.Enabled = false;
                trustedCNsLabel.Visible = false;
                trustedCNsButton.Enabled = false;
            }

            IsUpdated = true;
        }

        private void useLocalAuthCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            UseLocalAuth = useLocalAuthCheckBox.Checked;

            //
            // Update the state of the corresponding controls as necessary.
            //
            if (UseRemoteAuth || UseLocalAuth)
            {
                // Enable the Trusted Root Certificate and CN controls.
                trustedRootCertNamesLabel.Visible = true;
                trustedRootCertsButton.Enabled = true;
                trustedCNsLabel.Visible = true;
                trustedCNsButton.Enabled = true;
            }
            else
            {
                // Disable the Trusted Root Certificate and CN controls.
                trustedRootCertNamesLabel.Visible = false;
                trustedRootCertsButton.Enabled = false;
                trustedCNsLabel.Visible = false;
                trustedCNsButton.Enabled = false;
            }

            IsUpdated = true;
        }

        private void tlsCertButton_Click(object sender, EventArgs e)
        {
            EditTLSCertForm tlsCertForm = new EditTLSCertForm();
            tlsCertForm.Computer = Computer;
            AmtSecurityAdmin.AmtStoredCert[] certs = null;
            Computer.SecurityAdmin.CertStoreGetCertificates(out certs);
            tlsCertForm.TlsCerts = certs;
            if (TlsCert != null)
                tlsCertForm.TlsCert = TlsCert;

            if (tlsCertForm.ShowDialog() == DialogResult.OK)
            {
                TlsCert = tlsCertForm.TlsCert;
                tlsCertNameLabel.Text = ManageabilityStack.CertificateOperations.GetCertificateLongString(TlsCert.Certificate);
                IsUpdated = true;
            }
        }

        private void trustedRootCertsButton_Click(object sender, EventArgs e)
        {
            Debug.Assert(Computer.SecurityAdmin != null);

            // Display the certificate store form to allow the user to edit the 
            // certificates in the trusted root store.
            // TODO: May need to provide guidance on what to do in this form or 
            // edit the form to limit the controls to only those pertaining to 
            // the trusted root certificates.
            CertificateStoreForm certStoreForm = new CertificateStoreForm(Computer);
            if (certStoreForm.ShowDialog() == DialogResult.OK)
            {
                // Update the number of trusted root certificates.
                trustedRootCertNamesLabel.Text = String.Format("{0} Trusted Certificate(s)",
                    Computer.SecurityAdmin.GetTrustedRootCertificates().Length);
            }
        }

        private void trustedCNsButton_Click(object sender, EventArgs e)
        {
            EditTLSTrustedCNsForm tlsTrustedCNsForm = new EditTLSTrustedCNsForm(TrustedCNs);
            if (tlsTrustedCNsForm.ShowDialog() == DialogResult.OK)
            {
                TrustedCNs = tlsTrustedCNsForm.TrustedCNs;
                trustedCNsLabel.Text = String.Format("{0} Trusted Certificate CN(s)", TrustedCNs.Length);
                IsUpdated = true;
            }
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            if (AcceptNonSecureConnections != acceptNonTlsCheckBox.Checked)
            {
                AcceptNonSecureConnections = acceptNonTlsCheckBox.Checked;
                IsUpdated = true;
            }

            DialogResult = DialogResult.OK;
            Close();
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            IsUpdated = false;

            DialogResult = DialogResult.Cancel;
            Close();
        }

        #endregion
    }
}
