// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class EnvironmentDetectionForm : Form
    {
        private AmtSystem computer;
        private bool active = false;

        public EnvironmentDetectionForm(AmtSystem computer)
        {
            this.computer = computer;
            InitializeComponent();
        }

        private void EnvironmentDetectionForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);

            int selectedCb = -1;
            EnvironmentDetectionType ev = null;
            AmtCallStatus r = computer.SecurityAdmin.GetEnvironmentDetection(out ev);

            if (ev == null || ev.LocalDomains == null || ev.LocalDomains.Length == 0)
            {
                active = false;
                evEnabledLabel.Text = Properties.Resources.Disabled;
                policiesComboBox.Enabled = false;
                domainListView.Enabled = false;
                domainTextBox.Enabled = false;
                addButton.Enabled = false;
                removeButton.Enabled = false;
                ipv6LocalPrefixListView.Enabled = false;
                ipv6LocalPrefixTextBox.Enabled = false;
            }
            else
            {
                active = true;
                evEnabledLabel.Text = Properties.Resources.Enabled;
                policiesComboBox.Enabled = true;
                domainListView.Enabled = true;
                domainTextBox.Enabled = true;
                addButton.Enabled = (domainTextBox.Text.Length > 0);
                removeButton.Enabled = (domainListView.SelectedItems.Count == 1);

                if (ev.ExternalCircuitBreakerPolicySpecified == true)
                {
                    selectedCb = (int)ev.ExternalCircuitBreakerPolicy;
                }

                foreach (string d in ev.LocalDomains)
                {
                    domainListView.Items.Add(new ListViewItem(d));
                }
            }

            CircuitBreakerPolicyInfoType[] policies = computer.CircuitBreaker.GetCircuitBreakerPolicies();
            policiesComboBox.BeginUpdate();
            policiesComboBox.Items.Clear();
            policiesComboBox.Items.Add(new Utils.DropDownItem(Properties.Resources.NoPolicy, 0, null));
            policiesComboBox.SelectedIndex = 0;
            if (policies != null && policies.Length > 0)
            {
                foreach (CircuitBreakerPolicyInfoType policy in policies)
                {
                    Utils.DropDownItem item = new Utils.DropDownItem(policy.Policy.PolicyName, (int)policy.PolicyCreationHandle, null);
                    policiesComboBox.Items.Add(item);
                    if (policy.PolicyCreationHandle == selectedCb) policiesComboBox.SelectedItem = item;
                }
            }
            policiesComboBox.EndUpdate();
            UpdateInfo();

            #region IPv6 Settings

            // NOTE: Assumes that only Intel(R) AMT 6 and later supports these via WSMAN.
            if ((computer.CoreVersion >= 6.0) && (computer.WsMan != null))
            {
                // Disable the Add/Remove IPv6 Prefix buttons.
                this.ipv6LocalPrefixAddButton.Enabled = false;
                this.ipv6LocalPrefixRemoveButton.Enabled = false;

                // Retrieve the IPv6 Prefix list from the target Intel(R) AMT client.
                string[] ipv6Prefixes = null;
                r = computer.SecurityAdmin.GetEnvironmentDetection(out ev, out ipv6Prefixes);
                if (r != AmtCallStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "GetEnvironmentDetection", r);
                    return;
                }

                if (ipv6Prefixes != null)
                {
                    active = true;
                    evEnabledLabel.Text = Properties.Resources.Enabled;
                    policiesComboBox.Enabled = true;
                    domainListView.Enabled = true;
                    domainTextBox.Enabled = true;
                    addButton.Enabled = (domainTextBox.Text.Length > 0);
                    removeButton.Enabled = (domainListView.SelectedItems.Count == 1);

                    if (ev.ExternalCircuitBreakerPolicySpecified == true)
                        selectedCb = (int)ev.ExternalCircuitBreakerPolicy;

                    foreach (string ipv6Prefix in ipv6Prefixes)
                        this.ipv6LocalPrefixListView.Items.Add(ipv6Prefix);
                }
            }
            else
            {
                // Disable all advanced controls.
                this.ipv6LocalPrefixLabel.Enabled = this.ipv6LocalPrefixLabel.Visible = false;
                this.ipv6LocalPrefixListView.Enabled = this.ipv6LocalPrefixListView.Visible = false;
                this.ipv6LocalPrefixTextBox.Enabled = this.ipv6LocalPrefixTextBox.Visible = false;
                this.ipv6LocalPrefixAddButton.Enabled = this.ipv6LocalPrefixAddButton.Visible = false;
                this.ipv6LocalPrefixRemoveButton.Enabled = this.ipv6LocalPrefixRemoveButton.Visible = false;
            }

            #endregion
        }

        private void SystemStateChangedHandler(AmtSystem system)
        {
            if (system.State != AmtSystemObjState.Connected)
            {
                computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                DialogResult = DialogResult.Cancel;
            }
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus r = AmtCallStatus.SUCCESS;
            string[] ipv6Prefixes = null;

            if (active == false)
            {
                //
                // Disable Environment Detection by removing defined Local 
                // Domains and IPv6 Prefixes.
                //
                EnvironmentDetectionType ev = new EnvironmentDetectionType();
                ev.ExternalCircuitBreakerPolicy = 0;
                ev.ExternalCircuitBreakerPolicySpecified = false;
                ev.LocalDomains = new string[0];

                if ((computer.CoreVersion >= 6.0) && (computer.WsMan != null))
                {
                    // Assuming that only Intel(R) AMT 6.0 and later supports 
                    // defining IPv6 Prefixes via WSMAN for Environment Detection.
                    ipv6Prefixes = new string[0];
                    r = computer.SecurityAdmin.SetEnvironmentDetection(ev, ipv6Prefixes);
                }
                else
                {
                    r = computer.SecurityAdmin.SetEnvironmentDetection(ev);
                }

                if (r != AmtCallStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "SetEnvironmentDetection", r);
                    return;
                }
            }
            else
            {
                EnvironmentDetectionType ev = new EnvironmentDetectionType();
                if (policiesComboBox.SelectedIndex == 0)
                {
                    ev.ExternalCircuitBreakerPolicy = 0;
                    ev.ExternalCircuitBreakerPolicySpecified = false;
                }
                else
                {
                    ev.ExternalCircuitBreakerPolicy = (uint)(((Utils.DropDownItem)policiesComboBox.SelectedItem).Handle);
                    ev.ExternalCircuitBreakerPolicySpecified = true;
                }

                ArrayList s = new ArrayList();
                foreach (ListViewItem l in domainListView.Items) s.Add(l.Text);
                ev.LocalDomains = (string[])s.ToArray(typeof(string));

                if ((computer.CoreVersion >= 6.0) && (computer.WsMan != null))
                {
                    // Assuming that only Intel(R) AMT 6.0 and later supports 
                    // defining IPv6 Prefixes via WSMAN for Environment Detection.
                    ArrayList ipv6PrefixesArray = new ArrayList();
                    foreach (ListViewItem ipv6PrefixItem in this.ipv6LocalPrefixListView.Items)
                        ipv6PrefixesArray.Add(ipv6PrefixItem.Text);
                    ipv6Prefixes = (string[])ipv6PrefixesArray.ToArray(typeof(string));
                    r = computer.SecurityAdmin.SetEnvironmentDetection(ev, ipv6Prefixes);
                }
                else
                {
                    r = computer.SecurityAdmin.SetEnvironmentDetection(ev);
                }
                if (r != AmtCallStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "SetEnvironmentDetection", r);
                    return;
                }
            }

            // Disable everything
            okButton.Enabled = false;
            cancelButton.Enabled = false;
            domainTextBox.Enabled = false;
            addButton.Enabled = false;
            removeButton.Enabled = false;
            domainListView.Enabled = false;
            policiesComboBox.Enabled = false;
            enabledButton.Enabled = false;
            evEnabledLabel.Enabled = false;
            ipv6LocalPrefixLabel.Enabled = false;
            ipv6LocalPrefixListView.Enabled = false;
            ipv6LocalPrefixTextBox.Enabled = false;
            ipv6LocalPrefixAddButton.Enabled = false;
            ipv6LocalPrefixRemoveButton.Enabled = false;
            okButton.Text = Properties.Resources.WaitButton;
            new System.Threading.Thread(new System.Threading.ThreadStart(ConnectivityCheck)).Start();
        }

        private void ConnectivityCheck()
        {
            System.Threading.Thread.Sleep(3000);

            try
            {
                EnvironmentDetectionType t;
                computer.SecurityAdmin.GetEnvironmentDetection(out t);
            }
            catch (Exception)
            {
                // Computer lost connectivity
                computer.Disconnect();
            }

            computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
            DialogResult = DialogResult.OK;
        }

        private void domainTextBox_TextChanged(object sender, EventArgs e)
        {
            if (domainTextBox.Text.Length > 64) domainTextBox.Text = domainTextBox.Text.Substring(0, 64);
            addButton.Enabled = (domainTextBox.Text.Length > 0) && domainListView.Items.Count <= 5;
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            domainListView.Items.Add(new ListViewItem(domainTextBox.Text));
            domainTextBox.Text = "";
            UpdateInfo();
        }

        private void domainListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            removeButton.Enabled = (domainListView.SelectedItems.Count == 1);
            if (domainListView.SelectedItems.Count == 1)
            {
                domainTextBox.Text = domainListView.SelectedItems[0].Text;
            }
        }

        private void enabledButton_Click(object sender, EventArgs e)
        {
            active = !active;
            if (active)
            {
                evEnabledLabel.Text = Properties.Resources.Enabled;
                policiesComboBox.Enabled = true;
                domainListView.Enabled = true;
                domainTextBox.Enabled = true;
                addButton.Enabled = (domainTextBox.Text.Length > 0);
                removeButton.Enabled = (domainListView.SelectedItems.Count == 1);

                if ((computer.CoreVersion >= 6.0) && (computer.WsMan != null))
                {
                    // Enable all IPv6 Prefix controls as necessary.
                    this.ipv6LocalPrefixLabel.Enabled = true;
                    this.ipv6LocalPrefixListView.Enabled = true;
                    this.ipv6LocalPrefixTextBox.Enabled = true;
                    this.ipv6LocalPrefixAddButton.Enabled = (ipv6LocalPrefixTextBox.Text.Length > 0);
                    this.ipv6LocalPrefixRemoveButton.Enabled = (ipv6LocalPrefixListView.SelectedItems.Count >= 1);
                }
            }
            else
            {
                evEnabledLabel.Text = Properties.Resources.Disabled;
                policiesComboBox.Enabled = false;
                domainListView.Enabled = false;
                domainTextBox.Enabled = false;
                addButton.Enabled = false;
                removeButton.Enabled = false;

                if ((computer.CoreVersion >= 6.0) && (computer.WsMan != null))
                {
                    // Disable all IPv6 Prefix controls.
                    this.ipv6LocalPrefixLabel.Enabled = false;
                    this.ipv6LocalPrefixListView.Enabled = false;
                    this.ipv6LocalPrefixTextBox.Enabled = false;
                    this.ipv6LocalPrefixAddButton.Enabled = false;
                    this.ipv6LocalPrefixRemoveButton.Enabled = false;
                }
            }
            UpdateInfo();
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (domainListView.SelectedItems.Count == 1)
            {
                domainListView.Items.Remove(domainListView.SelectedItems[0]);
                UpdateInfo();
            }
        }

        private void UpdateInfo()
        {
            bool ok = true;
            if (active == true && domainListView.Items.Count == 0) ok = false;

            // If there aren't any EnvDet domains specified, then determine if 
            // there are IPv6 prefixes defined and enable the OK button 
            // accordingly.
            if (!ok && active && (ipv6LocalPrefixListView.Items.Count != 0)) ok = true;

            okButton.Enabled = ok;
        }

        private void policiesComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void ipv6LocalPrefixListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Update the enabled state of the Remove button if one or more 
            // items are selected.
            if (this.ipv6LocalPrefixListView.SelectedIndices.Count == 0)
                this.ipv6LocalPrefixRemoveButton.Enabled = false;
            else
            {
                this.ipv6LocalPrefixRemoveButton.Enabled = true;

                // Update the contents of the text box based on the first selected 
                // item.
                int selectedIpv6PrefixIndex = this.ipv6LocalPrefixListView.SelectedIndices[0];
                string selectedIpv6Prefix = this.ipv6LocalPrefixListView.Items[selectedIpv6PrefixIndex].Text;
                if (!String.IsNullOrEmpty(selectedIpv6Prefix))
                    this.ipv6LocalPrefixTextBox.Text = selectedIpv6Prefix;
            }
        }

        private void ipv6LocalPrefixTextBox_TextChanged(object sender, EventArgs e)
        {
            //
            // Validate the text against the IPv6 prefix regular expressions 
            // and enable the Add button as necessary.
            //
            // The format is: "XXXX:XXXX:XXXX:XXXX/Y" where Y is the prefix 
            // length, and the XXXX:XXXX:XXXX:XXXX part can include zeros 
            // compression (e.g. 0:0:0 or ::) formats.
            //
            // Complete IPv6 prefix (no compression)
            //    \A([0-9a-f]{1,4}:){7}[0-9a-f]{1,4}/\d\Z
            // IPv6 prefix with zero compression at begin
            //    \A:(:[0-9a-f]{1,4}){1,7}/\d\Z
            // IPv6 prefix with zero compression in middle
            //   \A([0-9a-f]{1,4}:){1,1}(:[0-9a-f]{1,4}){1,6}/\d\Z
            //   \A([0-9a-f]{1,4}:){1,2}(:[0-9a-f]{1,4}){1,5}/\d\Z
            //   \A([0-9a-f]{1,4}:){1,3}(:[0-9a-f]{1,4}){1,4}/\d\Z
            //   \A([0-9a-f]{1,4}:){1,4}(:[0-9a-f]{1,4}){1,3}/\d\Z
            //   \A([0-9a-f]{1,4}:){1,5}(:[0-9a-f]{1,4}){1,2}/\d\Z
            //   \A([0-9a-f]{1,4}:){1,6}(:[0-9a-f]{1,4}){1,1}/\d\Z
            // IPv6 prefix with zero compression at end
            //   \A(([0-9a-f]{1,4}:){1,7}|:):/\d\Z
            //

            string ipv6Regex = "(\\A([0-9a-f]{1,4}:){7}[0-9a-f]{1,4}/\\d+\\Z)|" +
                "(\\A:(:[0-9a-f]{1,4}){1,7}/\\d+\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,1}(:[0-9a-f]{1,4}){1,6}/\\d+\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,2}(:[0-9a-f]{1,4}){1,5}/\\d+\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,3}(:[0-9a-f]{1,4}){1,4}/\\d+\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,4}(:[0-9a-f]{1,4}){1,3}/\\d+\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,5}(:[0-9a-f]{1,4}){1,2}/\\d+\\Z)|" +
                "(\\A([0-9a-f]{1,4}:){1,6}(:[0-9a-f]{1,4}){1,1}/\\d+\\Z)|" +
                "(\\A(([0-9a-f]{1,4}:){1,7}|:):/\\d+\\Z)";
            Match ipv6RegexMatch = Regex.Match(this.ipv6LocalPrefixTextBox.Text, ipv6Regex, RegexOptions.IgnoreCase | RegexOptions.IgnorePatternWhitespace);

            if ((ipv6RegexMatch.Success) &&
                (this.ipv6LocalPrefixListView.Items.Count < 5))
                this.ipv6LocalPrefixAddButton.Enabled = true;
            else
                this.ipv6LocalPrefixAddButton.Enabled = false;
        }

        private void ipv6LocalPrefixAddButton_Click(object sender, EventArgs e)
        {
            Debug.Assert(this.ipv6LocalPrefixListView.Items.Count <= 5);
            this.ipv6LocalPrefixListView.Items.Add(this.ipv6LocalPrefixTextBox.Text);

            // Update the enabled state of the Add button if there are less 
            // than 5 items (as defined in the Intel(R) AMT 6 SDK).
            if (this.ipv6LocalPrefixListView.Items.Count == 5)
                this.ipv6LocalPrefixAddButton.Enabled = false;

            UpdateInfo();
        }

        private void ipv6LocalPrefixRemoveButton_Click(object sender, EventArgs e)
        {
            // Remove each selected IPv6 Prefix from the list.
            foreach (ListViewItem selectedItem in this.ipv6LocalPrefixListView.SelectedItems)
                this.ipv6LocalPrefixListView.Items.Remove(selectedItem);

            // Update the enabled state of the Add button if there are less 
            // than 5 items (as defined in the Intel(R) AMT 6 SDK).
            if (this.ipv6LocalPrefixListView.Items.Count < 5)
                this.ipv6LocalPrefixAddButton.Enabled = true;

            UpdateInfo();
        }
    }
}
