// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class ManagedInterfacesForm : Form
    {
        private AmtSystem computer;

        public ManagedInterfacesForm(AmtSystem computer)
        {
            this.computer = computer;
            InitializeComponent();
        }

        private void ManagedInterfacesForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);

            foreach (AmtNetworkAdminInterface i in computer.Network.interfaces)
            {
                InterfaceDescriptorGetType t = i.GetInterfaceSettings();
                interfacesComboBox.Items.Add(new Utils.DropDownItem(t.HardwareAddressDescription, (int)(i.Handle), i));
            }
            interfacesComboBox.SelectedIndex = 0;
            if (interfacesComboBox.Items.Count == 1) interfacesComboBox.Enabled = false;

            if (computer.Network.GetPingResponse() == true) label13.Text = Properties.Resources.Enabled; else label13.Text = Properties.Resources.Disabled;
            //vpnRoutingGroupBox.Visible = computer.IsMobile;

            // Assuming that only AMT6 and later has support for IPv6.
            if (computer.CoreVersion >= 6.0)
            {
                this.ipv6InterfacesComboBox.Items.Clear();

                IPv6PortSettings[] ipv6PortSettings = computer.Network.GetIPv6Settings();
                if (ipv6PortSettings != null)
                {
                    // Populate the available IPv6 Interface names and select the first.
                    foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)
                        this.ipv6InterfacesComboBox.Items.Add(ipv6PortSetting.InstanceID);
                    this.ipv6InterfacesComboBox.SelectedIndex = 0;

                    // The IPv6 addresses for the first IPv6 interface 
                    // will automatically be populated by updating the 
                    // selected index for the Interfaces ComboBox control.
                }

                if (this.ipv6InterfacesComboBox.Items.Count <= 1)
                    this.ipv6InterfacesComboBox.Enabled = false;

                this.amtAddrFamilyComboBox.SelectedItem = computer.Info.GetPreferredAddressFamily().ToString();

                if (this.ipv6InterfacesComboBox.Items.Count == 0)
                    this.ipv6EditButton.Enabled = false; // No IPv6 interfaces exist for editing.
            }
            else
            {
                // Given the assumptiong that only AMT6 and later can have IPv6 
                // settings, remove the tab page that offers this information.
                this.tabControl1.TabPages.Remove(this.ipv6TabPage);
            }
            
            UpdateEnvironmentDetection();
            UpdatePxeSetttings();
        }

        private void SystemStateChangedHandler(AmtSystem system)
        {
            if (system.State != AmtSystemObjState.Connected)
            {
                computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                DialogResult = DialogResult.Cancel;
            }
        }

        private void UpdatePxeSetttings()
        {
            if (computer.AmtRev >= 3 || (computer.AmtRev >= 2 && computer.AmtRevMin >= 6))
            {
                x8021GroupBox.Visible = true;
                Toggle8021xButton.Enabled = true;
                bool active;
                AmtCallStatus r = computer.Network.Get8021XActiveS0(out active);
                if (r == AmtCallStatus.SUCCESS)
                {
                    if (active == true)
                    {
                        activeInFullPowerLabel.Text = Properties.Resources.Enabled;
                    }
                    else
                    {
                        activeInFullPowerLabel.Text = Properties.Resources.Disabled;
                    }
                }
            }

            if ((computer.AmtRev == 2 && computer.AmtRevMin >= 6) ||
                (computer.AmtRev == 3 && computer.AmtRevMin >= 2) ||
                (computer.AmtRev >= 4))
            {
                changePxeTimeoutButton.Enabled = true;
                uint timeout;
                AmtCallStatus r = computer.Network.Get8021XPxeTimeout(out timeout);
                if (r == AmtCallStatus.SUCCESS)
                {
                    pxeTimeoutLabel.Text = string.Format(Properties.Resources.NumSeconds, timeout);
                }
            }
            else
            {
                if (label29.Enabled == true)
                {
                    label29.Enabled = false;
                    changePxeTimeoutButton.Visible = false;
                    pxeTimeoutLabel.Visible = false;
                    label29.Visible = false;
                    x8021GroupBox.Height -= (label29.Top - label27.Top);
                }
            }
        }

        private void UpdateEnvironmentDetection()
        {
            if (computer.AmtRev >= 3)
            {
                environmentDetectGroupBox.Visible = true;
                EnvironmentDetectionType ev = null;
                string[] ipv6Prefixes = null;
                AmtCallStatus r = computer.SecurityAdmin.GetEnvironmentDetection(out ev, out ipv6Prefixes);
                if (r == AmtCallStatus.SUCCESS)
                {
                    if (ev == null || ev.LocalDomains == null || ev.LocalDomains.Length == 0)
                    {
                        edEnabledLabel.Text = Properties.Resources.Disabled;
                        evDetectCbLabel.Enabled = false;
                        envDetTextBox.Enabled = false;
                        envDetTextBox.Clear();
                    }
                    else
                    {
                        edEnabledLabel.Text = Properties.Resources.Enabled;
                        envDetTextBox.Enabled = true;
                        envDetTextBox.Clear();
                        foreach (string s in ev.LocalDomains) envDetTextBox.AppendText(s + "\r\n");
                        if (ev.ExternalCircuitBreakerPolicySpecified == false)
                        {
                            evDetectCbLabel.Text = Properties.Resources.NoPolicy;
                        }
                        else
                        {
                            CircuitBreakerPolicyInfoType policy = computer.CircuitBreaker.GetCircuitBreakerPolicy(ev.ExternalCircuitBreakerPolicy);
                            evDetectCbLabel.Text = policy.Policy.PolicyName;
                            evDetectCbLabel.Enabled = true;
                        }
                    }

                    if (ipv6Prefixes != null)
                    {
                        edEnabledLabel.Text = Properties.Resources.Enabled;
                        envDetTextBox.Enabled = true;
                        foreach (string ipv6Prefix in ipv6Prefixes)
                            envDetTextBox.AppendText(ipv6Prefix + "\r\n");
                    }
                }
            }
            else
            {
                environmentDetectGroupBox.Visible = false;
            }
        }

        private void interfacesComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            AmtNetworkAdminInterface i = (AmtNetworkAdminInterface)(((Utils.DropDownItem)interfacesComboBox.SelectedItem).Tag);
            InterfaceDescriptorGetType t = i.GetInterfaceSettings();

            if (t.MACAddress != null)
            {
                label9.Text = t.MACAddress.ToUpper();
            }
            else
            {
                label9.Text = Properties.Resources.Unknown;
            }

            label2.Text = new IPAddress((uint)System.Net.IPAddress.NetworkToHostOrder((int)t.IPv4Parameters.LocalAddress)).ToString();
            label5.Text = new IPAddress((uint)System.Net.IPAddress.NetworkToHostOrder((int)t.IPv4Parameters.SubnetMask)).ToString();
            label15.Text = new IPAddress((uint)System.Net.IPAddress.NetworkToHostOrder((int)t.IPv4Parameters.DefaultGatewayAddress)).ToString();
            label17.Text = new IPAddress((uint)System.Net.IPAddress.NetworkToHostOrder((int)t.IPv4Parameters.PrimaryDnsAddress)).ToString();
            label17.Text += ", " + new IPAddress((uint)System.Net.IPAddress.NetworkToHostOrder((int)t.IPv4Parameters.SecondaryDnsAddress)).ToString();

            if (t.DhcpEnabled) label7.Text = Properties.Resources.Enabled; else label7.Text = Properties.Resources.Disabled;

            string linkString = "";
            if ((t.LinkPolicy & 0x01) == 0x01) linkString += ", " + Properties.Resources.LinkPolicyAcS0Only;
            if ((t.LinkPolicy & 0x0E) == 0x0E) linkString += ", " + Properties.Resources.LinkPolicy_AcSxOnly;
            if ((t.LinkPolicy & 0x10) == 0x10) linkString += ", " + Properties.Resources.LinkPolicyDcS0Only;
            if ((t.LinkPolicy & 0xE0) == 0xE0) linkString += ", " + Properties.Resources.LinkPolicy_DcSxOnly;

            if (linkString.Length == 0)
            {
                label11.Text = string.Format(Properties.Resources.LinkPolicyUnknown, t.LinkPolicy.ToString());
            }
            else
            {
                label11.Text = linkString.Substring(2);
            }

            label19.Text = "";
            if (t.InterfaceMode == InterfaceModeType.SEPARATE_MAC_ADDRESS) label19.Text = Properties.Resources.SeperateMacAddress;
            if (t.InterfaceMode == InterfaceModeType.SHARED_MAC_ADDRESS) label19.Text = Properties.Resources.SharedMacAddress;

            if (computer.CoreVersion >= 2.5 && i.Handle == 0)
            {
                xProfileEditButton.Enabled = true;
                XProfileType tt;
                i.Get8021XWiredProfile(out tt);
                label25.Text = xProfileEditForm.GetProfileString(tt);
            }
            else
            {
                xProfileEditButton.Enabled = false;
                label25.Text = Properties.Resources.NotSpecified;
            }

            if (computer.NetworkAccessControlAdmin == null || i.Handle != 0)
            {
                eacLabel.Text = Properties.Resources.Unsupported;
                eacButton.Enabled = false;
            }
            else
            {
                eacButton.Enabled = true;
                bool enabled;
                if (computer.NetworkAccessControlAdmin.GetEACStatus(out enabled) == AmtCallStatus.SUCCESS)
                {
                    if (enabled == true)
                    {
                        eacLabel.Text = Properties.Resources.Enabled;
                    }
                    else
                    {
                        eacLabel.Text = Properties.Resources.Disabled;
                    }
                }
                else
                {
                    eacLabel.Text = Properties.Resources.Error;
                }
            }

            editButton.Enabled = (i.Handle == 0);
        }

        private void enableVpnRoutingButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus r = computer.SecurityAdmin.EnableVpnRouting(true);
            if (r != AmtCallStatus.SUCCESS)
            {
                MessageBox.Show(this, string.Format(Properties.Resources.EnableVpnRoutingFailed, r.ToString()), Properties.Resources.VpnRoutingTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                r = computer.SecurityAdmin.CommitChanges();
                if (r != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show(this, string.Format(Properties.Resources.EnableVpnRoutingCommitFailed, r.ToString()), Properties.Resources.VpnRoutingTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    MessageBox.Show(this, Properties.Resources.EnableVpnRoutingSucceed, Properties.Resources.VpnRoutingTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
        }

        private void disableVpnRoutingButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus r = computer.SecurityAdmin.EnableVpnRouting(false);
            if (r != AmtCallStatus.SUCCESS)
            {
                MessageBox.Show(this, string.Format(Properties.Resources.DisableVpnRoutingFailed, r.ToString()), Properties.Resources.VpnRoutingTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                r = computer.SecurityAdmin.CommitChanges();
                if (r != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show(this, string.Format(Properties.Resources.DisableVpnRoutingCommitFailed, r.ToString()), Properties.Resources.VpnRoutingTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    MessageBox.Show(this, Properties.Resources.DisableVpnRoutingSucceed, Properties.Resources.VpnRoutingTitle, MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
        }

        private void setMePingButton_Click(object sender, EventArgs e)
        {
            if (computer.Network.SetPingResponse(!computer.Network.GetPingResponse()) == AmtCallStatus.SUCCESS)
            {
                if (computer.Network.GetPingResponse() == true) label13.Text = Properties.Resources.Enabled; else label13.Text = Properties.Resources.Disabled;
            }
        }

        private void evDetectEditButton_Click(object sender, EventArgs e)
        {
            if (new EnvironmentDetectionForm(computer).ShowDialog(this) == DialogResult.OK)
            {
                if (computer.State != AmtSystemObjState.Connected) closeButton_Click(this, null); else UpdateEnvironmentDetection();
            }
        }

        private void Toggle8021xButton_Click(object sender, EventArgs e)
        {
            bool active;
            if (computer.Network.Get8021XActiveS0(out active) == AmtCallStatus.SUCCESS)
            {
                computer.Network.Set8021XActiveS0(!active);
                UpdatePxeSetttings();
            }
        }

        private void editButton_Click(object sender, EventArgs e)
        {
            AmtNetworkAdminInterface i = (AmtNetworkAdminInterface)(((Utils.DropDownItem)interfacesComboBox.SelectedItem).Tag);
            ManagedInterfaceEditForm form = new ManagedInterfaceEditForm(computer, i);
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                computer.Disconnect();
                closeButton_Click(this, null);
            }
        }

        private void xProfileEditButton_Click(object sender, EventArgs e)
        {
            AmtNetworkAdminInterface i = (AmtNetworkAdminInterface)(((Utils.DropDownItem)interfacesComboBox.SelectedItem).Tag);
            xProfileEditForm form = new xProfileEditForm(computer, i);
            XProfileType p;
            if (i.Get8021XWiredProfile(out p) == AmtCallStatus.SUCCESS)
            {
                form.SetProfile(p);
                if (form.ShowDialog(this) == DialogResult.OK) interfacesComboBox_SelectedIndexChanged(this, null);
            }
        }

        private void eacButton_Click(object sender, EventArgs e)
        {
            EndpointAccessControlEditForm form = new EndpointAccessControlEditForm(computer);
            if (form.ShowDialog(this) == DialogResult.OK) interfacesComboBox_SelectedIndexChanged(this, null);
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
            DialogResult = DialogResult.OK;
        }

        private void ipv6InterfacesComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Assuming that only AMT6 and later has support for IPv6.
            System.Diagnostics.Debug.Assert(computer.CoreVersion >= 6.0);

            string selectedInterfaceID = (string)this.ipv6InterfacesComboBox.SelectedItem;

            // Update the displayed IPv6 addresses for the newly selected IPv6 interface.
            this.ipv6AddrsListBox.Items.Clear();

            IPv6PortSettings[] ipv6PortSettings = computer.Network.GetIPv6Settings();
            System.Diagnostics.Debug.Assert(ipv6PortSettings != null);

            // Find the matching IPv6 Port Settings and populate the IPv6 
            // addresses for the interface.
            foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)
            {
                if (String.Compare(ipv6PortSetting.InstanceID, selectedInterfaceID, true) == 0)
                {
                    // Update the interface ID information.
                    this.ipv6IntfIdValueLabel.Text = ipv6PortSetting.InterfaceID.ToString();
                    if (ipv6PortSetting.InterfaceID == IPv6PortSettings.InterfaceIDType.Manual)
                        this.ipv6IntfIdValueLabel.Text += (": " + ipv6PortSetting.ManualInterfaceID);

                    // Populate IPv6 addresses for the selected IPv6 interface.
                    foreach (IPv6PortSettings.AddressInfo ipv6AddrInfo in ipv6PortSetting.CurrentAddressInfo)
                        this.ipv6AddrsListBox.Items.Add(ipv6AddrInfo.IPAddr.ToString());

                    this.ipv6PrimDnsAddrLabel.Text = ipv6PortSetting.CurrentPrimaryDNS.ToString();
                    this.ipv6SecDnsAddrLabel.Text = ipv6PortSetting.CurrentSecondaryDNS.ToString();
                    this.ipv6DefRouterAddrLabel.Text = ipv6PortSetting.CurrentDefaultRouter.ToString();

                    break;
                }
            } // END foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)

            if (this.ipv6AddrsListBox.Items.Count == 0)
                this.ipv6AddrsListBox.Items.Add(ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA);
            else
                this.ipv6AddrsListBox.SelectedIndex = 0;

            if (computer.Network.IsIpv6Enabled(selectedInterfaceID))
                this.amtAddrFamilyComboBox.Enabled = true;
            else
                this.amtAddrFamilyComboBox.Enabled = false;
        }

        private void ipv6AddrsListBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Assuming that only AMT6 and later has support for IPv6.
            System.Diagnostics.Debug.Assert(computer.CoreVersion >= 6.0);

            string selectedIpv6Addr = (string)this.ipv6AddrsListBox.SelectedItem;

            // Update the address type for the selected IPv6 address.
            this.ipv6AddrTypeValueLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;

            IPv6PortSettings[] ipv6PortSettings = computer.Network.GetIPv6Settings();
            System.Diagnostics.Debug.Assert(ipv6PortSettings != null);

            // Find the selected IPv6 interface in the available collection.
            foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)
            {
                if (String.Compare(ipv6PortSetting.InstanceID, (string)this.ipv6InterfacesComboBox.SelectedItem, true) == 0)
                {
                    // Find the matching IPv6 address information to the user selected item.
                    foreach (IPv6PortSettings.AddressInfo ipv6AddrInfo in ipv6PortSetting.CurrentAddressInfo)
                    {
                        if (String.Compare(ipv6AddrInfo.IPAddr.ToString(), selectedIpv6Addr, true) == 0)
                        {
                            this.ipv6AddrTypeValueLabel.Text = ipv6AddrInfo.Type.ToString();
                            break;
                        }
                    } // END foreach (IPv6PortSettings.AddressInfo ipv6AddrInfo in ipv6PortSetting.CurrentAddressInfo)
                }
            } // END foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)
        }

        private void ipv6EditButton_Click(object sender, EventArgs e)
        {
            // Assuming that only AMT6 and later has support for IPv6.
            System.Diagnostics.Debug.Assert(computer.CoreVersion >= 6.0);

            string selectedInterfaceID = (string)this.ipv6InterfacesComboBox.SelectedItem;
            IPv6PortSettings[] ipv6PortSettings = computer.Network.GetIPv6Settings();
            if (ipv6PortSettings == null)
            {
                MessageBox.Show("No IPv6 interface configuration settings exist to edit.",
                    "Missing IPv6 Interface Information", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return; // Nothing to edit.
            }

            foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)
            {
                if (String.Compare(ipv6PortSetting.InstanceID, selectedInterfaceID, true) == 0)
                {
                    ManagedIpv6InterfaceEditForm form = new ManagedIpv6InterfaceEditForm(computer, ipv6PortSetting);
                    if (form.ShowDialog(this) == DialogResult.OK)
                    {
                        UpdateIpv6Info();
                        break;
                    }
                }
            } // END foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)
        }

        private void amtAddrFamilyComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Assuming that only AMT6 and later has support for IPv6.
            System.Diagnostics.Debug.Assert(computer.CoreVersion >= 6.0);

            AmtGeneralInfo.AmtIPAddressFamily amtAddrFamily = (AmtGeneralInfo.AmtIPAddressFamily)Enum.Parse(typeof(AmtGeneralInfo.AmtIPAddressFamily), (string)this.amtAddrFamilyComboBox.SelectedItem);

            // Only apply changes if the IPv6 interface is enabled.
            string selectedInterfaceID = (string)this.ipv6InterfacesComboBox.SelectedItem;
            if (computer.Network.IsIpv6Enabled(selectedInterfaceID))
            {
                AmtCallStatus r = computer.Info.SetPreferredAddressFamily(amtAddrFamily);
                if (r == AmtCallStatus.NO_WSMAN)
                {
                    this.amtAddrFamilyComboBox.SelectedItem = computer.Info.GetPreferredAddressFamily().ToString();
                    this.amtAddrFamilyComboBox.Enabled = false;
                    return;
                }
                else if (r != AmtCallStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "SetPreferredAddressFamily", r);
                    return;
                }
            }
        }

        private void UpdateIpv6Info()
        {
            if (computer.State != AmtSystemObjState.Connected)
                return; // Nothing to do.

            // Assuming that only AMT6 and later has support for IPv6.
            System.Diagnostics.Debug.Assert(computer.CoreVersion >= 6.0);

            // Refresh the network settings.
            computer.Network.ClearCache();
            computer.Network.FetchCache();

            string selectedInterfaceID = (string)this.ipv6InterfacesComboBox.SelectedItem;

            // Update the displayed IPv6 addresses for the updated IPv6 interface.
            this.ipv6AddrsListBox.Items.Clear();

            IPv6PortSettings[] ipv6PortSettings = computer.Network.GetIPv6Settings();
            if (ipv6PortSettings == null)
            {
                this.ipv6AddrTypeValueLabel.Text = String.Empty;
                this.ipv6PrimDnsAddrLabel.Text = String.Empty;
                this.ipv6SecDnsAddrLabel.Text = String.Empty;
                this.ipv6DefRouterAddrLabel.Text = String.Empty;
                return; // Nothing to do.
            }

            // Populate the available IPv6 Interface names and select the first match.
            foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)
            {
                if (String.Compare(ipv6PortSetting.InstanceID, selectedInterfaceID, true) == 0)
                {
                    // Update the interface ID information.
                    this.ipv6IntfIdValueLabel.Text = ipv6PortSetting.InterfaceID.ToString();
                    if (ipv6PortSetting.InterfaceID == IPv6PortSettings.InterfaceIDType.Manual)
                        this.ipv6IntfIdValueLabel.Text += (": " + ipv6PortSetting.ManualInterfaceID);

                    // Populate IPv6 addresses for the selected IPv6 interface.
                    foreach (IPv6PortSettings.AddressInfo ipv6AddrInfo in ipv6PortSetting.CurrentAddressInfo)
                        this.ipv6AddrsListBox.Items.Add(ipv6AddrInfo.IPAddr.ToString());

                    this.ipv6PrimDnsAddrLabel.Text = ipv6PortSetting.CurrentPrimaryDNS.ToString();
                    this.ipv6SecDnsAddrLabel.Text = ipv6PortSetting.CurrentSecondaryDNS.ToString();
                    this.ipv6DefRouterAddrLabel.Text = ipv6PortSetting.CurrentDefaultRouter.ToString();

                    break;
                }
            } // END foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)

            if (this.ipv6AddrsListBox.Items.Count == 0)
                this.ipv6AddrsListBox.Items.Add(ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA);
            else
                this.ipv6AddrsListBox.SelectedIndex = 0;

            if (computer.Network.IsIpv6Enabled(selectedInterfaceID))
                this.amtAddrFamilyComboBox.Enabled = true;
            else
                this.amtAddrFamilyComboBox.Enabled = false;
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //
            // Get the list of selected IP addresses and copy them to the system clipboard.
            //
            string ipAddrsToCopy = String.Empty;
            foreach (string selectedItem in this.ipv6AddrsListBox.SelectedItems)
                ipAddrsToCopy += String.Format("{0}\r\n", selectedItem);
            ipAddrsToCopy.TrimEnd(new char[] { '\r', '\n' });
            Clipboard.SetData(DataFormats.StringFormat, ipAddrsToCopy);
        }

        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            for (int index = 0; index < this.ipv6AddrsListBox.Items.Count; index++)
                this.ipv6AddrsListBox.SetSelected(index, true);
        }

        private void ipv6RefreshButton_Click(object sender, EventArgs e)
        {
            UpdateIpv6Info();
        }
    }
}
