﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Net;
using System.Text;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class ManagedIpv6InterfaceEditForm : Form
    {
        #region Private Members

        /// <summary>
        /// The Intel(R) AMT System that contains the IPv6 interface to be 
        /// modified.
        /// </summary>
        private AmtSystem _computer;

        /// <summary>
        /// The IPv6 interface and its settings to be modified.
        /// </summary>
        private IPv6PortSettings _ipv6PortSettings;

        #endregion

        #region Constructors

        public ManagedIpv6InterfaceEditForm()
        {
            InitializeComponent();
        }

        public ManagedIpv6InterfaceEditForm(AmtSystem computer, IPv6PortSettings ipv6PortSettigns)
        {
            this.Computer = computer;
            this.IPv6PortSettings = ipv6PortSettigns;

            InitializeComponent();
        }

        #endregion

        #region Public Properties

        /// <summary>
        /// Gets or sets the Intel(R) AMT System that contains the IPv6 
        /// interface to be edited.
        /// </summary>
        public AmtSystem Computer
        {
            get { return this._computer; }
            set { this._computer = value; }
        }

        /// <summary>
        /// Gets or sets the IPv6 port settings to be edited.
        /// </summary>
        public IPv6PortSettings IPv6PortSettings
        {
            get { return this._ipv6PortSettings; }
            set { this._ipv6PortSettings = value; }
        }

        #endregion

        #region Private Methods

        private void ManagedIpv6InterfaceEditForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;

            //
            // Initialize the controls of the dialog as appropriate.
            //
            this.ipv6EnableCheckBox.Checked = this.Computer.Network.IsIpv6Enabled(this.IPv6PortSettings.InstanceID);
            this.ipv6EnableCheckBox.Text += String.Format(" on {0}", this.IPv6PortSettings.InstanceID);

            this.ipv6IntfIdComboBox.SelectedItem = this.IPv6PortSettings.InterfaceID.ToString();
            this.ipv6IntfIdManualTextBox.Text = this.IPv6PortSettings.ManualInterfaceID;

            this.ipv6Dhcpv6TimeoutNumericUpDown.Value = this.Computer.Info.GetDHCPv6ConfigurationTimeout();

            // Determine if this is the wireless interface and update the 
            // remaining dialog controls accordingly.
            // NOTE: "Intel(r) AMT Ethernet Port 1" is assumed/documented as 
            // the identifier for the Intel(R) AMT wireless interface.
            if (String.Compare(this.IPv6PortSettings.InstanceID, "Intel(r) AMT Ethernet Port 1", true) == 0)
            {
                // Disable the static IPv6 address controls.
                this.ipv6StaticAddrTextBox.Enabled = false;
                this.ipv6StaticPrimDnsTextBox.Enabled = false;
                this.ipv6StaticSecDnsTextBox.Enabled = false;
                this.ipv6StaticRouterTextBox.Enabled = false;
            }
            else
            {
                // Determine if a static IPv6 address exists.
                foreach (IPv6PortSettings.AddressInfo ipv6AddrInfo in this.IPv6PortSettings.CurrentAddressInfo)
                {
                    if (ipv6AddrInfo.Type == IPv6PortSettings.AddressType.Static)
                    {
                        this.ipv6StaticAddrTextBox.Text = this.IPv6PortSettings.IPv6Address.ToString();
                        this.ipv6StaticPrimDnsTextBox.Text = this.IPv6PortSettings.PrimaryDNS.ToString();
                        this.ipv6StaticSecDnsTextBox.Text = this.IPv6PortSettings.SecondaryDNS.ToString();
                        this.ipv6StaticRouterTextBox.Text = this.IPv6PortSettings.DefaultRouter.ToString();
                        break;
                    }
                } // END foreach (IPv6PortSettings.AddressInfo ipv6AddrInfo in this.IPv6PortSettings.CurrentAddressInfo)
            }
        }

        private void ipv6EnableCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (this.ipv6EnableCheckBox.Checked == true)
            {
                this.ipv6IntfIdComboBox.Enabled = true;
                if (String.Compare((string)this.ipv6IntfIdComboBox.SelectedItem,
                                   IPv6PortSettings.InterfaceIDType.Manual.ToString(), true) == 0)
                {
                    this.ipv6IntfIdManualTextBox.Enabled = true;
                }
                else
                {
                    this.ipv6IntfIdManualTextBox.Enabled = false;
                }

                this.ipv6Dhcpv6TimeoutNumericUpDown.Enabled = true;

                // Determine if this is the wireless interface and update the 
                // remaining dialog controls accordingly.
                // NOTE: "Intel(r) AMT Ethernet Port 1" is assumed/documented as 
                // the identifier for the Intel(R) AMT wireless interface.
                if (String.Compare(this.IPv6PortSettings.InstanceID, "Intel(r) AMT Ethernet Port 1", true) != 0)
                {
                    this.ipv6StaticAddrTextBox.Enabled = true;
                    this.ipv6StaticPrimDnsTextBox.Enabled = true;
                    this.ipv6StaticSecDnsTextBox.Enabled = true;
                    this.ipv6StaticRouterTextBox.Enabled = true;
                }
            }
            else
            {
                this.ipv6IntfIdComboBox.Enabled = false;
                this.ipv6IntfIdManualTextBox.Enabled = false;
                this.ipv6Dhcpv6TimeoutNumericUpDown.Enabled = false;
                this.ipv6StaticAddrTextBox.Enabled = false;
                this.ipv6StaticPrimDnsTextBox.Enabled = false;
                this.ipv6StaticSecDnsTextBox.Enabled = false;
                this.ipv6StaticRouterTextBox.Enabled = false;
            }
        }

        private void ipv6IntfIdComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (String.Compare((string)this.ipv6IntfIdComboBox.SelectedItem,
                               IPv6PortSettings.InterfaceIDType.Manual.ToString(), true) == 0)
            {
                this.ipv6IntfIdManualTextBox.Enabled = true;
            }
            else
            {
                this.ipv6IntfIdManualTextBox.Enabled = false;
            }
        }

        private void ipv6IntfIdManualTextBox_TextChanged(object sender, EventArgs e)
        {
            //
            // Ensure that only hexadecimal characters are used.
            //
            uint newValue = 0;
            Regex hexNumRegex = new Regex("[0-9a-fA-F]+");

            try
            {
                newValue = uint.Parse(this.ipv6IntfIdManualTextBox.Text, System.Globalization.NumberStyles.HexNumber);
            }
            catch (ArgumentNullException)
            {
                this.ipv6IntfIdManualTextBox.Text = String.Format("{0:x}", newValue);
                return;
            }
            catch (FormatException)
            {
                Match hexNumRegexMatch = hexNumRegex.Match(this.ipv6IntfIdManualTextBox.Text);
                if (hexNumRegexMatch.Success)
                    newValue = uint.Parse(hexNumRegexMatch.Value, System.Globalization.NumberStyles.HexNumber);
            }
            catch (Exception)
            {
                return;
            }

            this.ipv6IntfIdManualTextBox.Text = String.Format("{0:x}", newValue);
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus r = AmtCallStatus.SUCCESS;

            if (this.Computer.HostNameIdentity.ForceIPOption != ForceIPOption.UseIPv4)
            {
                DialogResult userResp = MessageBox.Show("Any changes will force a disconnect from the system. Would you like to continue?",
                    this.Text, MessageBoxButtons.YesNo, MessageBoxIcon.Question);
                if (userResp == DialogResult.No) return; // Nothing to do.
            }

            //
            // Update the enabled state of the IPv6 interface.
            //
            r = this.Computer.Network.EnableIpv6(this.IPv6PortSettings.InstanceID, this.ipv6EnableCheckBox.Checked);
            if (r != AmtCallStatus.SUCCESS)
            {
                Utils.ErrorMessageBox(this, "EnableIpv6", r);
                this.DialogResult = DialogResult.Cancel;
                this.Close();
                return;
            }
            else
            {
                if (this.ipv6EnableCheckBox.Checked == false)
                {
                    // Depending on the connection type for this system, we may 
                    // need to force a disconnect from the system to avoid issue 
                    // when performing a refresh of the IPv6 network settings.
                    if (this.Computer.HostNameIdentity.ForceIPOption != ForceIPOption.UseIPv4)
                        this.Computer.Disconnect();
                    this.DialogResult = DialogResult.OK;
                    this.Close();
                    return;
                }
            }

            //
            // Update the remainder of the IPv6 interface settings.
            //
            this.IPv6PortSettings.InterfaceID = (IPv6PortSettings.InterfaceIDType)Enum.Parse(typeof(IPv6PortSettings.InterfaceIDType), (string)this.ipv6IntfIdComboBox.SelectedItem);
            this.IPv6PortSettings.ManualInterfaceID = String.Format("{0:X8}", Convert.ToUInt32(this.ipv6IntfIdManualTextBox.Text));

            try
            {
                if (String.IsNullOrEmpty(this.ipv6StaticAddrTextBox.Text))
                    this.IPv6PortSettings.IPv6Address = IPAddress.Parse("::0");
                else
                    this.IPv6PortSettings.IPv6Address = IPAddress.Parse(this.ipv6StaticAddrTextBox.Text);
            }
            catch (Exception excep)
            {
                Debug.Print("EXCEPTION: {0} (type {1})", excep.Message, excep.GetType());
                Debug.Print("EXCEPTION STACK: {0}", excep.StackTrace);

                this.IPv6PortSettings.IPv6Address = IPAddress.Parse("::0");
            }

            try
            {
                if (String.IsNullOrEmpty(this.ipv6StaticPrimDnsTextBox.Text))
                    this.IPv6PortSettings.PrimaryDNS = IPAddress.Parse("::0");
                else
                    this.IPv6PortSettings.PrimaryDNS = IPAddress.Parse(this.ipv6StaticPrimDnsTextBox.Text);
            }
            catch (Exception excep)
            {
                Debug.Print("EXCEPTION: {0} (type {1})", excep.Message, excep.GetType());
                Debug.Print("EXCEPTION STACK: {0}", excep.StackTrace);

                this.IPv6PortSettings.PrimaryDNS = IPAddress.Parse("::0");
            }

            try
            {
                if (String.IsNullOrEmpty(this.ipv6StaticSecDnsTextBox.Text))
                    this.IPv6PortSettings.SecondaryDNS = IPAddress.Parse("::0");
                else
                    this.IPv6PortSettings.SecondaryDNS = IPAddress.Parse(this.ipv6StaticSecDnsTextBox.Text);
            }
            catch (Exception excep)
            {
                Debug.Print("EXCEPTION: {0} (type {1})", excep.Message, excep.GetType());
                Debug.Print("EXCEPTION STACK: {0}", excep.StackTrace);

                this.IPv6PortSettings.SecondaryDNS = IPAddress.Parse("::0");
            }

            try
            {
                if (String.IsNullOrEmpty(this.ipv6StaticRouterTextBox.Text))
                    this.IPv6PortSettings.DefaultRouter = IPAddress.Parse("::0");
                else
                    this.IPv6PortSettings.DefaultRouter = IPAddress.Parse(this.ipv6StaticRouterTextBox.Text);
            }
            catch (Exception excep)
            {
                Debug.Print("EXCEPTION: {0} (type {1})", excep.Message, excep.GetType());
                Debug.Print("EXCEPTION STACK: {0}", excep.StackTrace);

                this.IPv6PortSettings.DefaultRouter = IPAddress.Parse("::0");
            }

            r = this.Computer.Info.SetDHCPv6ConfigurationTimeout((ushort)this.ipv6Dhcpv6TimeoutNumericUpDown.Value);
            if (r != AmtCallStatus.SUCCESS) Utils.ErrorMessageBox(this, "SetDHCPv6ConfigurationTimeout", r);

            r = this.Computer.Network.SetIpv6Settings(this.IPv6PortSettings);
            if (r != AmtCallStatus.SUCCESS) Utils.ErrorMessageBox(this, "SetIpv6Settings", r);

            if (this.Computer.HostNameIdentity.ForceIPOption != ForceIPOption.UseIPv4)
                this.Computer.Disconnect();

            this.DialogResult = DialogResult.OK;
            this.Close();
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        #endregion
    }
}
