// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class ComputerPanel : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtSystem computer = null;

        public ComputerPanel()
        {
            InitializeComponent();
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.computer = (AmtSystem)obj1;
        }

        public void Cleanup()
        {
            computer = null;
        }

        public void PerformAction(string command, object[] args)
        {
        }

        public void UpdatePanel()
        {
            if (computer == null)
                return;

            networkSettingsRefreshButton.Enabled = true;

            // String setup
            string EnabledText = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_Enabled;
            string DisabledText = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_Disabled;

            // State & cache independent information
            iamtConnectGroupBox.Visible = true;
            hostnameTextBox.Text = computer.HostNameIdentity.ConnectedDisplayName;
            usernameTextBox.Text = computer.Username;
            passwordTextBox.Text = computer.Password;
            rememberPasswordCheckBox.Checked = computer.RememberPassword;

            if (computer.HostNameIdentity.ForceIPOption == ForceIPOption.UseDefault)
                webLinkLabel.Text = computer.WebURL;
            else
            {
                // Force IP Option is set for this computer.
                // If it is in a disconnected state, just use the display name
                if (computer.State == AmtSystemObjState.Disconnected)
                {
                    IPAddress ipaddress = null;
                    string Name = computer.HostNameIdentity.DisplayName;
                    if (IPAddress.TryParse(Name, out ipaddress) == true)
                    {
                        if (ipaddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6)
                            Name = "[" + Name + "]";
                    }
                    if (computer.UseTls == true)
                        Name = "https://" + Name + ":16993";
                    else
                        Name = "http://" + Name + ":16992";
                    webLinkLabel.Text = Name;
                }
                else
                {
                    // If we are connected or "connecting" and have an IP Address, display the IP Address URL
                    if (computer.HostNameIdentity.IPAddress != null)
                    {
                        if (computer.UseTls == true)
                            webLinkLabel.Text = computer.HostNameIdentity.ToWebHttpsURL(16993);
                        else
                            webLinkLabel.Text = computer.HostNameIdentity.ToWebHttpURL(16992);
                    }
                }
            }

            webUiEnabledLabel.Enabled = webUiToggleButton.Enabled = (computer.AmtRev != 0);
            wirelessGroupBox.Visible = computer.IsWirelessSupported;

            if (computer.State == AmtSystemObjState.Connected)
            {
                #region Connection Tab

                usernameTextBox.ReadOnly = true; 

                if (computer.UseTls == true)
                {
                    // Set the padlock icon
                    padlockPictureBox.Visible = true;
                    hostnameTextBox.Width = connectButton.Left - hostnameTextBox.Left - 34;
                    usernameTextBox.Width = connectButton.Left - usernameTextBox.Left - 34;
                }
                else
                {
                    // Clear the padlock icon
                    padlockPictureBox.Visible = false;
                    hostnameTextBox.Width = connectButton.Left - hostnameTextBox.Left - 3;
                    usernameTextBox.Width = connectButton.Left - usernameTextBox.Left - 3;
                }

                // Show CIRA disconnection button
                disconnectCiraButton.Visible = (computer.CoreVersion >= 4.0 && computer.RemoteAccessAdmin != null && computer.HttpProxy != null);

                // Cache independent 
                connectButton.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_Discon;
                passwordTextBox.Enabled = false;
                rememberPasswordCheckBox.Enabled = false;
                meTimeButton.Enabled = (computer.NetworkTime != null);
                unprovisionButton.Enabled = true;

                if (mainform.Simplified == false)
                {
                    connectionWarningGroupBox.Visible = (computer.Warnings != 0);
                    connectionWarningTextBox.Text = "";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.NameMismatch) != 0 && computer.UseTls == true)
                        connectionWarningTextBox.Text += Properties.Resources.ConnectionNameDnsnameCertificateMismatch + "\r\n";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.NameMismatch) != 0 && computer.UseTls == false)
                        connectionWarningTextBox.Text += Properties.Resources.ConnectionNameDnsMismatch + "\r\n";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.CertificateExpired) != 0)
                        connectionWarningTextBox.Text += Properties.Resources.ComputerCertificateExpired + "\r\n";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.CertificateInvalid) != 0)
                        connectionWarningTextBox.Text += string.Format(Properties.Resources.ComputerCertificateInvalid + "\r\n", computer.CertificateError.ToString());
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.CertificateUntrusted) != 0)
                        connectionWarningTextBox.Text += Properties.Resources.ComputerCertificateUntrusted + "\r\n";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.AuthPrivateKeyNotExportable) != 0)
                        connectionWarningTextBox.Text += Properties.Resources.AuthCertificatePrivateKeyNoExport + "\r\n";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.OpenSslNotAvailable) != 0)
                        connectionWarningTextBox.Text += Properties.Resources.OpenSslNotAvailable + "\r\n";
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.BadRedirectionPort) != 0)
                        connectionWarningTextBox.Text += Properties.Resources.BadRedirectionPort + "\r\n";
                }
                #endregion

                #region Remote Control Tab

                // Redirection Service
                bool redirectState = (computer.CoreVersion == 0);
                if (computer.Redirection != null)
                    redirectState = computer.Redirection.GetRedirectionListenerState();
                
                // Get KVM state (enabled/disabled)
                if (computer.KVM.KVMSupported.Value)
                {
                    ManageabilityStack.AmtKVM.EnabledStateValues KVMState = computer.KVM.KVMEnabled.Value;
                    if (KVMState == AmtKVM.EnabledStateValues.ENABLED)
                    {
                        if (computer.KVM.KVMPort5900Enabled.Value)
                            KVMStandardButton.Enabled = true;
                        else
                            KVMStandardButton.Enabled = false;

                        if ((computer.UseTls == true && computer.Warnings != 0))
                        {
                            KVMRedirButton.Enabled = false;
                        }
                        else if (computer.KVM.KVMRedirectionPortEnabled.Value)
                            KVMRedirButton.Enabled = true;
                        else
                            KVMRedirButton.Enabled = false;
                    }
                    else
                    {
                        KVMStandardButton.Enabled = false;
                        KVMRedirButton.Enabled = false;
                    }
                }
                else
                {
                    KVMStandardButton.Enabled = false;
                    KVMRedirButton.Enabled = false;
                }
 
                if (computer.SecurityAdmin == null)
                {
                    takeControlButton.Enabled = true;
                }
                else
                {
                    if ((computer.UseTls == true && computer.Warnings != 0) )
                    {
                        takeControlButton.Enabled = false;
                    }
                    else
                    {
                        takeControlButton.Enabled = redirectState;
                    }
                }

                #endregion

                #region Management Engine Tab

                // SecurityAdmin Service
                if (computer.AmtRev != 1 && computer.SecurityAdmin != null && computer.SecurityAdmin.IsCached())
                {
                    solToggleButton.Enabled = iderToggleButton.Enabled = redirectToggleButton.Enabled = (computer.AmtRev != 0);
                    if (computer.SecurityAdmin.SerialOverLanEnabled)
                        serialOverLanLabel.Text = EnabledText;
                    else
                        serialOverLanLabel.Text = DisabledText;
                    if (computer.SecurityAdmin.IdeRedirectEnabled)
                        ideRedirectLabel.Text = EnabledText;
                    else
                        ideRedirectLabel.Text = DisabledText;
                    if (redirectState)
                        redirectionPortLabel.Text = EnabledText;
                    else
                        redirectionPortLabel.Text = DisabledText;


                    kvmLabel_Update();
                    if (computer.KVM.KVMState.Value == AmtKVM.KVMStates.ERROR_UNKNOWN_STATE)
                    {
                        kvmSettingButton.Enabled = false;
                    }
                    else
                    {
                        kvmSettingButton.Enabled = true;
                    }

                    UserConsentLabel_Update();
                    
                    if (computer.UserConsent.OptInSupported.Value == true)
                    {
                        userConsentButton.Enabled = true;
                    }
                    else
                    {
                        userConsentButton.Enabled = false;
                    }


                    if (computer.SecurityAdmin.WebUiEnabled)
                    {
                        webUiEnabledLabel.Text = EnabledText;
                    }
                    else
                    {
                        webUiEnabledLabel.Text = DisabledText;
                    }
                    webUiToggleButton.Enabled = (computer.AmtRev != 0);
                }
                else
                {
                    string StatusUnknown = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_StaUnk;
                    userConsentButton.Enabled = kvmSettingButton.Enabled = solToggleButton.Enabled = iderToggleButton.Enabled = redirectToggleButton.Enabled = false;
                    serialOverLanLabel.Text = StatusUnknown;
                    ideRedirectLabel.Text = StatusUnknown;
                    redirectionPortLabel.Text = StatusUnknown;
                    kvmLabel.Text = StatusUnknown;
                    UserConsentLabel.Text = StatusUnknown; 
                    webUiEnabledLabel.Text = StatusUnknown;
                    webUiToggleButton.Enabled = false;
                }

                // Hardware asset service
                string s = ManageabilityCommanderTool.Properties.Resources.Unknown;
                if (computer.Assets != null && computer.Assets.IsCached() && computer.Assets.GetBios() != null)
                    s = computer.Assets.GetBios().Version;
                else if (computer.Info != null && computer.Info.IsCached())
                    s = computer.Info.GetFirmwareVersion();
                biosVersionLabel.Text = s;

                // Remote Access Setup
                if (computer.RemoteAccessAdmin != null)
                {
                    if (computer.RemoteAccessAdmin.IsCached())
                    {
                        AmtCallStatus r1, r2;
                        remoteAccessSetupButton.Enabled = true;
                        AmtRemoteAccessAdmin.AmtRemoteAccessPolicy[] policies;
                        r1 = computer.RemoteAccessAdmin.GetRemoteAccessPolicies(out policies);
                        AmtRemoteAccessAdmin.AmtRemoteAccessServer[] servers;
                        r2 = computer.RemoteAccessAdmin.GetRemoteServers(out servers);

                        if (r1 == AmtCallStatus.SUCCESS && r2 == AmtCallStatus.SUCCESS)
                        {
                            if (policies == null || servers == null)
                            {
                                remoteAccessSetupLabel.Text = Properties.Resources.Unknown;
                            }
                            else
                            {
                                if (policies.Length == 0 || servers.Length == 0)
                                {
                                    remoteAccessSetupLabel.Text = "Not Setup";
                                }
                                else
                                {
                                    s = "";
                                    if (servers.Length == 1)
                                        s += "1 server";
                                    else
                                        s += string.Format("{0} servers", servers.Length);
                                    if (policies.Length == 1)
                                        s += ", 1 policy";
                                    else
                                        s += string.Format(", {0} policies", policies.Length);
                                    remoteAccessSetupLabel.Text = s;
                                }
                            }
                        }
                        else
                        {
                            remoteAccessSetupLabel.Text = Properties.Resources.Error;
                            remoteAccessSetupButton.Enabled = false;
                        }
                    }
                    else
                    {
                        remoteAccessSetupLabel.Text = Properties.Resources.Loading;
                        remoteAccessSetupButton.Enabled = false;
                    }
                }
                else
                {
                    remoteAccessSetupLabel.Text = Properties.Resources.Unsupported;
                    remoteAccessSetupButton.Enabled = false;
                }

                if (computer.AmtRev > 0 && computer.SecurityAdmin != null && computer.SecurityAdmin.IsCached())
                {
                    iamtVersionLabel.Text = string.Format("{0}.{1}", computer.SecurityAdmin.GetCoreVersion(), computer.SecurityAdmin.GetBuildNum());
                    int activeStateAC;
                    int wakeOnNetAccessThresholdAC;
                    int wakeOnNetAccessSleepTimer;
                    computer.SecurityAdmin.GetPowerSavingOptions(out activeStateAC, out wakeOnNetAccessThresholdAC, out wakeOnNetAccessSleepTimer);
                    setUserAccountsButton.Enabled = true;

                    AmtSecurityAdmin.AmtUserAcl[] accounts;
                    AmtCallStatus r = computer.SecurityAdmin.GetUserAclEntries(out accounts);
                    int userAccountsCount = 0;
                    if (r == AmtCallStatus.SUCCESS && accounts != null)
                        userAccountsCount = accounts.Length;

                    if (userAccountsCount == 0)
                        userAccountsLabel.Text = ManageabilityCommanderTool.Properties.Resources.Account_Num_None;
                    else if (userAccountsCount == 1)
                        userAccountsLabel.Text = ManageabilityCommanderTool.Properties.Resources.Account_Num_One;
                    else
                        userAccountsLabel.Text = userAccountsCount.ToString() + ManageabilityCommanderTool.Properties.Resources.Account_Num_Multiple;

                    mePowerSaveLabel.Text = computer.SecurityAdmin.GetPowerSavingOptionsStr();
                    setMePowerSaveButton.Enabled = (mePowerSaveLabel.Text.Length > 0);

                    if (wakeOnNetAccessSleepTimer == 1)
                    {
                        mePowerSaveLabel.Text += String.Format(ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_IdleTimeOut_01_Format, wakeOnNetAccessSleepTimer);
                    }
                    else if (wakeOnNetAccessSleepTimer > 1)
                    {
                        mePowerSaveLabel.Text += String.Format(ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_IdleTimeOut_02_Format, wakeOnNetAccessSleepTimer);
                    }
                    // Look at the Kerberos* setup
                    if (computer.AmtRev <= 1)
                    {
                        kerberosSetupLabel.Text = Properties.Resources.Unsupported;
                        kerberosSetupButton.Enabled = false;
                    }
                    else
                    {
                        KerberosOptionsType k;
                        r = computer.SecurityAdmin.GetKerberosOptions(out k);
                        if (r != AmtCallStatus.SUCCESS || k == null)
                        {
                            kerberosSetupLabel.Text = Properties.Resources.Disabled;
                        }
                        else
                        {
                            kerberosSetupLabel.Text = string.Format(Properties.Resources.KerberosRealm, k.KerberosRealmName);
                        }
                        kerberosSetupButton.Enabled = true;
                    }

                    if (computer.CoreVersion >= (float)2.5)
                    {
                        certificateStoreButton.Enabled = true;
                        AmtSecurityAdmin.AmtStoredCert[] certs;
                        computer.SecurityAdmin.CertStoreGetTrustedCertificates(out certs);
                        int rootCertCount = certs.Length;

                        if (computer.SecurityAdmin.CertStoreGetCertificates(out certs) == AmtCallStatus.SUCCESS)
                        {
                            certificateStoreLabel.Text = string.Format(Properties.Resources.AmtCertCounts, certs.Length, rootCertCount);
                        }
                        else
                        {
                            certificateStoreLabel.Text = Properties.Resources.Unknown;
                        }
                    }
                    else
                    {
                        certificateStoreButton.Enabled = false;
                        certificateStoreLabel.Text = Properties.Resources.Unsupported;
                    }
                }
                else
                {
                    if (computer.Info != null)
                        iamtVersionLabel.Text = string.Format("{0}.{1}", computer.Info.GetCoreVersion(), computer.Info.GetBuildNum());

                    mePowerSaveLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                    setMePowerSaveButton.Enabled = false;

                    userAccountsLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                    setUserAccountsButton.Enabled = false;

                    kerberosSetupLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                    kerberosSetupButton.Enabled = false;

                    certificateStoreButton.Enabled = false;
                    certificateStoreLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                }

                if (computer.WsMan == null)
                {
                    interactionTypeLabel.Text = Properties.Resources.EoiOnly;
                }
                else
                {
                    if (computer.WsManOnly == true)
                    {
                        interactionTypeLabel.Text = Properties.Resources.WsmanOnly;
                    }
                    else
                    {
                        interactionTypeLabel.Text = Properties.Resources.EoiAndWsman;
                    }
                }
                wsManSettingButton.Enabled = (computer.WsMan != null && computer.LimitedFeatures == false && computer.SecurityAdmin != null);

                // NetworkAdmin Service
                if (computer.Network != null && computer.Network.IsCached())
                {
                    computerNameDomainLabel.Text = computer.Network.GetHostName();
                    if (computer.Network.GetDomainName() != null)
                        computerNameDomainLabel.Text += "." + computer.Network.GetDomainName();
                    if ((computer.Warnings & AmtSystem.AmtConnectWarnings.NameMismatch) == 0)
                    {
                        editComputerNameButton.BackColor = editWirelessProfilesButton.BackColor;
                    }
                    else
                    {
                        editComputerNameButton.BackColor = Color.LightCoral;
                    }
                    editComputerNameButton.Enabled = true;
                }
                else if (computer.Info != null && computer.Info.IsCached())
                {
                    computerNameDomainLabel.Text = computer.Info.GetHostName();
                    editComputerNameButton.Enabled = false;
                    editComputerNameButton.BackColor = editWirelessProfilesButton.BackColor;
                }
                else
                {
                    computerNameDomainLabel.Text = ManageabilityCommanderTool.Properties.Resources.Unknown;
                    editComputerNameButton.Enabled = false;
                    editComputerNameButton.BackColor = editWirelessProfilesButton.BackColor;
                }

                // TODO: This logic check really should be done in the underlying stack.
                if (computer.CoreVersion >= (float)5.1)
                {
                    // Enable the controls for configuring the Alarm Clock settings.
                    AmtRemoteControl.AmtAlarmWakeupState state;
                    DateTime nextTime;
                    TimeSpan interval;
                    AmtCallStatus r = computer.Remote.GetAlarmWakeUp(out state, out nextTime, out interval);
                    if (r == AmtCallStatus.SUCCESS)
                    {
                        alarmClockLabel.Text = (state == AmtRemoteControl.AmtAlarmWakeupState.WakeUpIsArmed) ? "Armed" : "Stopped";
                        alarmClockButton.Enabled = true;
                    }
                    else
                    {
                        alarmClockLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                        alarmClockButton.Enabled = false;
                    }
                }
                else
                {
                    alarmClockLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_Not_Supported;
                    alarmClockButton.Enabled = false;
                }



                if (computer.SecurityAdmin != null)
                {
                    if (computer.SecurityAdmin.GetProvisioningMode() == AmtSecurityAdmin.ProvisioningMode.Enterprise)
                    {

                        TlsAthenticationType local;
                        TlsAthenticationType remote;

                        if (computer.SecurityAdmin.GetTlsOptions(out local, out remote) == AmtCallStatus.SUCCESS)
                        {
                            tlsStateLabel.Text = String.Format("Local: {0}, Remote: {1}",
                                local.ToString(), remote.ToString());
                            tlsStateButton.Enabled = true;
                        }
                        else
                        {
                            tlsStateLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                            tlsStateButton.Enabled = false;
                        }
                    }
                    else
                    {
                        //TLS is not supported in non-enterprise mode
                        tlsStateLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_Not_Supported;
                        tlsStateButton.Enabled = false;
                    }
                }

                AmtUserConsent.ControlModes ControlMode = computer.UserConsent.CurrentControlMode.Value;

                if (ControlMode == AmtUserConsent.ControlModes.CLIENT_CONTROL_MODE)
                {
                    controlModeStateLable.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_ClientControlMode;
                    controlModeButton.Enabled = true;
                }
                else if (ControlMode == AmtUserConsent.ControlModes.ADMIN_CONTROL_MODE)
                {
                    controlModeStateLable.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_AdminMode;
                    controlModeButton.Enabled = true;
                }
                else
                {
                    controlModeStateLable.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_Not_Supported;
                    controlModeButton.Enabled = false;
                }

                #endregion

                #region Networking Tab

                if (computer.Network != null)
                {
                    string interfacenames = "";
                    string interfaceIp = "";
                    string interfaceDhcp = "";

                    foreach (AmtNetworkAdminInterface intf in computer.Network.interfaces)
                    {
                        InterfaceDescriptorGetType infd = intf.GetInterfaceSettings();
                        interfacenames += ", \"" + infd.HardwareAddressDescription + "\"";

                        int a = (int)infd.IPv4Parameters.LocalAddress;
                        uint b = (uint)System.Net.IPAddress.NetworkToHostOrder((int)a);

                        interfaceIp += ", " + new IPAddress(b).ToString();
                        interfaceDhcp += ", " + infd.DhcpEnabled.ToString();
                    }

                    if (interfacenames.Length > 2)
                        interfacenames = interfacenames.Substring(2);
                    if (interfaceIp.Length > 2)
                        interfaceIp = interfaceIp.Substring(2);
                    if (interfaceDhcp.Length > 2)
                        interfaceDhcp = interfaceDhcp.Substring(2);

                    meNetworkInterfacesLabel.Text = interfacenames;
                    meIpAddressLabel.Text = interfaceIp;
                    meDhcpModeLabel.Text = interfaceDhcp;
                    changeNetworkSettingsButton.Enabled = true;

                    #region IPv6 Network Settings

                    // Assuming that only AMT6 and later has support for IPv6.
                    if (computer.CoreVersion >= 6.0)
                    {
                        this.ipv6InterfacesComboBox.Enabled = true;
                        this.ipv6AddrsListBox.Enabled = true;
                        this.ipv6InterfacesComboBox.Items.Clear();
                        this.ipv6AddrsListBox.Items.Clear();

                        IPv6PortSettings[] ipv6PortSettings = computer.Network.GetIPv6Settings();
                        if (ipv6PortSettings != null)
                        {
                            // Populate the available IPv6 Interface names and select the first.
                            foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)
                                this.ipv6InterfacesComboBox.Items.Add(ipv6PortSetting.InstanceID);
                            this.ipv6InterfacesComboBox.SelectedIndex = 0;

                            // The IPv6 addresses for the first IPv6 interface 
                            // will automatically be populated by updating the 
                            // selected index for the ComboBox control.

                            if (this.ipv6InterfacesComboBox.Items.Count <= 1)
                                this.ipv6InterfacesComboBox.Enabled = false;
                        }
                        else
                        {
                            this.ipv6InterfacesComboBox.Enabled = false;
                            this.ipv6AddrsListBox.Enabled = false;
                            this.ipv6InterfacesComboBox.Items.Clear();
                            this.ipv6AddrsListBox.Items.Clear();
                            this.ipv6InterfacesComboBox.Items.Add(ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA);
                            this.ipv6AddrsListBox.Items.Add(ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA);
                        }
                    }
                    else
                    {
                        this.ipv6InterfacesComboBox.Enabled = false;
                        this.ipv6AddrsListBox.Enabled = false;
                        this.ipv6InterfacesComboBox.Items.Clear();
                        this.ipv6AddrsListBox.Items.Clear();
                        this.ipv6InterfacesComboBox.Items.Add(ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA);
                        this.ipv6AddrsListBox.Items.Add(ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA);
                    }

                    #endregion
                }
                else
                {
                    meNetworkInterfacesLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                    ;
                    meIpAddressLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                    ;
                    meDhcpModeLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                    ;
                    changeNetworkSettingsButton.Enabled = false;
                }

                editWirelessProfilesButton.Enabled = (computer.Wireless != null);
                if (computer.Wireless != null)
                {
                    bool radioEnabled = false;
                    computer.Wireless.IsRadioOn(out radioEnabled);
                    if (radioEnabled == true)
                    {
                        wirelessRadio.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_RadioOn;
                    }
                    else
                    {
                        wirelessRadio.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_RadioOff;
                    }

                    wirelessCapability.Text = computer.Wireless.GetWirelessCapabilitiesStr();

                    int profileCount = computer.Wireless.GetWirelessProfiles().Length;
                    if (profileCount == 0)
                    {
                        wirelessSecurityLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NoProf;
                    }
                    else
                    {
                        wirelessSecurityLabel.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_Prof_Format, profileCount.ToString());
                    }

                    wirelessManagementButton.Enabled = true;

                    //if (computer.Wireless.GetWirelessPreference() == AmtWireless.LinkPreference.HOST)

                    linkPreferenceLabel.Text = computer.Wireless.GetWirelessPreference().ToString();

                    InterfaceDescriptorGetType interfaceDescriptor = null;
                    if (computer.Network != null && computer.Network.Wireless != null)
                        interfaceDescriptor = computer.Network.Wireless.GetInterfaceSettings();
                    if (interfaceDescriptor != null)
                    {
                        string WirelessLinkDesc = "";

                        if (computer.CoreVersion < (float)4.0)
                        {
                            // AMT 2.5 - This is a bit overkill, 2.5 does not support much.
                            if (interfaceDescriptor.LinkPolicy == 0x11)
                                WirelessLinkDesc = Properties.Resources.Enabled;
                            else if (interfaceDescriptor.LinkPolicy == 0x00)
                                WirelessLinkDesc = Properties.Resources.Disabled;
                            else
                            {
                                // AC status
                                if ((interfaceDescriptor.LinkPolicy & 0x0F) != 0)
                                {
                                    if ((interfaceDescriptor.LinkPolicy & 0x0F) == 0x0F)
                                        WirelessLinkDesc += ", " + Properties.Resources.LinkPolicy_AcAlways;
                                    else if ((interfaceDescriptor.LinkPolicy & 0x0F) == 0x0E)
                                        WirelessLinkDesc += ", " + Properties.Resources.LinkPolicy_AcSxOnly;
                                    else if ((interfaceDescriptor.LinkPolicy & 0x0F) == 0x01)
                                        WirelessLinkDesc += ", " + Properties.Resources.LinkPolicyAcS0Only;
                                    else
                                        WirelessLinkDesc += ", " + Properties.Resources.LinkPolicyAcUnknown;
                                }

                                // DC (Battery) status
                                if ((interfaceDescriptor.LinkPolicy & 0xF0) != 0)
                                {
                                    if ((interfaceDescriptor.LinkPolicy & 0xF0) == 0xF0)
                                        WirelessLinkDesc += ", " + Properties.Resources.LinkPolicy_DcAlways;
                                    else if ((interfaceDescriptor.LinkPolicy & 0xF0) == 0xE0)
                                        WirelessLinkDesc += ", " + Properties.Resources.LinkPolicy_DcSxOnly;
                                    else if ((interfaceDescriptor.LinkPolicy & 0xF0) == 0x10)
                                        WirelessLinkDesc += ", " + Properties.Resources.LinkPolicyDcS0Only;
                                    else
                                        WirelessLinkDesc += ", " + Properties.Resources.LinkPolicyDcUnknown;
                                }

                                if (WirelessLinkDesc.Length > 2)
                                {
                                    WirelessLinkDesc = string.Format(Properties.Resources.WirelessLinkEnabledOn, WirelessLinkDesc.Substring(2));
                                }
                                else
                                {
                                    WirelessLinkDesc = Properties.Resources.WirelessLinkAlwaysOff;
                                }
                            }
                        }
                        else
                        {
                            // AMT 4.0
                            if (interfaceDescriptor.LinkPolicy == 0)
                                WirelessLinkDesc = Properties.Resources.Disabled;
                            else if (interfaceDescriptor.LinkPolicy == 17)
                                WirelessLinkDesc = "Enabled in S0 only";
                            else if (interfaceDescriptor.LinkPolicy == 31)
                                WirelessLinkDesc = "Enabled in Sx";
                        }

                        wirelessManagementLabel.Text = WirelessLinkDesc;
                    }
                    else
                    {
                        wirelessManagementLabel.Text = Properties.Resources.Node_AMTSys_NA;
                    }
                }
                else
                {
                    editWirelessProfilesButton.Enabled = false;
                    wirelessRadio.Text = Properties.Resources.Node_AMTSys_UnsuppFeat;
                    wirelessCapability.Text = Properties.Resources.Node_AMTSys_NA;
                    wirelessSecurityLabel.Text = Properties.Resources.Node_AMTSys_NA;
                }

                #endregion
            }
            else
            {
                // Computer is disconnected
                meTimeButton.Enabled = false;
                unprovisionButton.Enabled = false;
                disconnectCiraButton.Visible = false;
                networkSettingsRefreshButton.Enabled = false;

                // Clear the padlock icon
                padlockPictureBox.Visible = false;
                hostnameTextBox.Width = connectButton.Left - hostnameTextBox.Left - 3;
                usernameTextBox.Width = connectButton.Left - usernameTextBox.Left - 3;

                userConsentButton.Enabled = kvmSettingButton.Enabled = solToggleButton.Enabled = iderToggleButton.Enabled = redirectToggleButton.Enabled = false;
                if (computer.State == AmtSystemObjState.Connecting)
                {
                    connectButton.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_AbortConn;
                    usernameTextBox.ReadOnly = true; 
                }
                else
                {
                    connectButton.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_Conn;

                    usernameTextBox.ReadOnly = false;
                }

                connectionWarningGroupBox.Visible = false;
                connectionWarningTextBox.Text = "";
                string NAString = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                passwordTextBox.Enabled = true;
                rememberPasswordCheckBox.Enabled = true;
                editComputerNameButton.BackColor = editWirelessProfilesButton.BackColor;
                takeControlButton.Enabled = false;
                KVMRedirButton.Enabled = false;
                KVMStandardButton.Enabled = false; 
                kvmLabel.Text = NAString;
                UserConsentLabel.Text = NAString; 
                serialOverLanLabel.Text = NAString;
                ideRedirectLabel.Text = NAString;
                redirectionPortLabel.Text = NAString;
                webUiEnabledLabel.Text = NAString;
                webUiEnabledLabel.Enabled = true;
                webUiToggleButton.Enabled = false;
                userAccountsLabel.Text = NAString;
                interactionTypeLabel.Text = NAString;
                wsManSettingButton.Enabled = false;
                certificateStoreLabel.Text = NAString;
                certificateStoreButton.Enabled = false;
                kerberosSetupLabel.Text = NAString;
                kerberosSetupButton.Enabled = false;
                remoteAccessSetupLabel.Text = NAString;
                remoteAccessSetupButton.Enabled = false;

                setMePowerSaveButton.Enabled = false;
                setUserAccountsButton.Enabled = false;
                computerNameDomainLabel.Text = NAString;
                editComputerNameButton.Enabled = false;
                iamtVersionLabel.Text = NAString;
                biosVersionLabel.Text = NAString;
                mePowerSaveLabel.Text = NAString;

                editWirelessProfilesButton.Enabled = false;
                wirelessManagementButton.Enabled = false;
                wirelessRadio.Text = NAString;
                wirelessCapability.Text = NAString;
                wirelessSecurityLabel.Text = NAString;
                wirelessManagementLabel.Text = NAString;

                meNetworkInterfacesLabel.Text = NAString;
                meIpAddressLabel.Text = NAString;
                meDhcpModeLabel.Text = NAString;
                changeNetworkSettingsButton.Enabled = false;

                alarmClockLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                alarmClockButton.Enabled = false;

                ipv6InterfacesComboBox.Enabled = false;
                ipv6InterfacesComboBox.Items.Clear();
                ipv6InterfacesComboBox.Items.Add(ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA);
                ipv6AddrsListBox.Enabled = false;
                ipv6AddrsListBox.Items.Clear();
                ipv6AddrsListBox.Items.Add(ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA);

                tlsStateLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                tlsStateButton.Enabled = false;

                controlModeStateLable.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                controlModeButton.Enabled = false; 
            }
        }

        public void ForceUpdatePanel()
        {
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                if (computer.State == AmtSystemObjState.Connected || computer.State == AmtSystemObjState.Connecting)
                {
                    ManageabilityControls.TerminalForm.DisposeRemotingForm(computer);
                    computer.Disconnect();
                    hostnameTextBox.Text = computer.HostNameIdentity.ConnectedDisplayName;
                }
                else
                {
                    // First, update the password & save it if needed.
                    if (computer.Password != passwordTextBox.Text)
                    {
                        computer.Password = passwordTextBox.Text;
                        
                        if (computer.RememberPassword == true)
                            mainform.SaveSettings();
                    }
                    
                    if (computer.Username != usernameTextBox.Text)
                    {
                        computer.Username = usernameTextBox.Text;
                        mainform.SaveSettings();
                    }

                    //check, if user name is entered, then password must be entered
                    //(no username no password = local user kerberous authentication)
                    if (( computer.Password == null || computer.Password == string.Empty)
                        && (computer.Username != null && computer.Username != string.Empty))
                    {
                        MessageBox.Show(
                            "A password is required if username is specified", "Invalid Password",
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                        return;
                    }



                    // Perform DNS Query to get IP address if a force IP Address option is specified
                    if ((computer.HostNameIdentity.ForceIPOption == ForceIPOption.UseIPv4) ||
                        (computer.HostNameIdentity.ForceIPOption == ForceIPOption.UseIPv6))
                    {
                        computer.HostNameIdentity.ResolveIPAddress();
                        if (computer.HostNameIdentity.IPAddress == null)
                        {
                            MessageBox.Show("Cannot resolve IP Address of the correct family.");
                            return;
                        }
                        hostnameTextBox.Text = computer.HostNameIdentity.DefaultName;
                    }

                    // Setup WSMAN support
                    computer.WsManSupport = Properties.Settings.Default.WsmanSupport;
                    // Then, connect to the computer.
                    computer.Connect();
                }

                if (mainTreeView.SelectedNode == systemNode)
                    mainform.UpdateTreeView();
            }
        }

        private void webLinkLabel_Click(object sender, EventArgs e)
        {
            try
            {
                System.Diagnostics.Process.Start(webLinkLabel.Text);
            }
            catch (Exception)
            {
            }
        }

        private void takeControlButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                if (ManageabilityStack.AmtRedirectorWrapper.CheckRedirectionWrapper() == false)
                {
                    MessageBox.Show("IMRSDK.dll file not found.", Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                if (computer.MyComputer)
                {
                    MessageBox.Show(this,
                        ManageabilityCommanderTool.Properties.Resources.MainForm_Terminal_MyComputer,
                        ManageabilityCommanderTool.Properties.Resources.MainForm_Terminal,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                }
                else if (computer.CoreVersion == 0 || (computer.Redirection != null && computer.Redirection.GetRedirectionListenerState() == true))
                {
                    ManageabilityControls.TerminalForm.ShowRemotingForm(computer, this, mainform.Simplified);
                }
                else
                {
                    MessageBox.Show(this,
                        ManageabilityCommanderTool.Properties.Resources.MainForm_PortDisable_ErrStr,
                        ManageabilityCommanderTool.Properties.Resources.MainForm_Terminal,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }

        private void serialOverLanLabel_DoubleClick(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                if (computer.State == AmtSystemObjState.Connected)
                {
                    AmtSecurityAdmin.EnabledInterfaces eif = computer.SecurityAdmin.GetEnabledInterfaces();
                    eif.SerialOverLAN = !eif.SerialOverLAN;
                    computer.SecurityAdmin.SetEnabledInterfaces(eif);
                }

                if (mainTreeView.SelectedNode == systemNode)
                    mainform.UpdateTreeView();
            }
        }

        private void ideRedirectLabel_DoubleClick(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                if (computer.State == AmtSystemObjState.Connected)
                {
                    AmtSecurityAdmin.EnabledInterfaces eif = computer.SecurityAdmin.GetEnabledInterfaces();
                    eif.IdeRedirection = !eif.IdeRedirection;
                    computer.SecurityAdmin.SetEnabledInterfaces(eif);
                }

                if (mainTreeView.SelectedNode == systemNode)
                    mainform.UpdateTreeView();
            }
        }

        private void redirectionPortLabel_DoubleClick(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                if (computer.Redirection == null)
                    return;

                if (computer.State == AmtSystemObjState.Connected)
                {
                    bool state;
                    try
                    {
                        state = computer.Redirection.GetRedirectionListenerState();
                    }
                    catch (WebException)
                    {
                        return;
                    }

                    computer.Redirection.SetRedirectionListenerState(!state);
                }

                if (mainTreeView.SelectedNode == systemNode)
                    mainform.UpdateTreeView();
            }
        }

        private void kvmSettingButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                new ManageabilityControls.EditKVMForm(computer).ShowDialog(this);                
                
                mainform.UpdateTreeView();
            }
        }

        private void userConsentButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                new ManageabilityControls.UserConsentForm(computer).ShowDialog(this);                

                mainform.UpdateTreeView();
            }
        }

        private void changeNetworkSettingsButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (computer.Network != null)
                {
                    computer.Network.ClearCache();
                    if (computer.Network.Wireless != null)
                        computer.Network.Wireless.ClearCache();
                }
                new ManagedInterfacesForm(computer).ShowDialog(this);
            }
        }

        private void wirelessManagementButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                InterfaceDescriptorGetType interfaceDescriptor = null;
                if (computer.Network.Wireless != null)
                {
                    computer.Network.Wireless.ClearCache();
                    interfaceDescriptor = computer.Network.Wireless.GetInterfaceSettings();
                }
                if (interfaceDescriptor != null)
                {
                    if (computer.CoreVersion < (float)4.0)
                    {
                        // AMT 2.5
                        bool wm = (interfaceDescriptor.LinkPolicy == 0x11);

                        InterfaceDescriptorSetType st = new InterfaceDescriptorSetType();
                        st.IPv4Parameters = null;
                        st.InterfaceMode = interfaceDescriptor.InterfaceMode;
                        if (wm == false)
                        {
                            st.LinkPolicy = 0x11;
                        }
                        else
                        {
                            st.LinkPolicy = 0x00;
                        }
                        AmtCallStatus r = computer.Network.Wireless.SetInterfaceSettings(st);
                        if (r != AmtCallStatus.SUCCESS)
                        {
                            MessageBox.Show(this, string.Format(Properties.Resources.SetInterfaceSettingsError, r.ToString()), Properties.Resources.InterfaceError, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        else
                        {
                            if (computer.Network != null)
                            {
                                computer.Network.ClearCache();
                                if (computer.Network.Wireless != null)
                                    computer.Network.Wireless.ClearCache();
                            }
                            mainform.UpdateTreeView();
                        }
                    }
                    else
                    {
                        // AMT 4.0
                        InterfaceDescriptorSetType st = new InterfaceDescriptorSetType();
                        st.IPv4Parameters = null;
                        st.InterfaceMode = interfaceDescriptor.InterfaceMode;
                        if (interfaceDescriptor.LinkPolicy == 0)
                            st.LinkPolicy = 17;
                        else if (interfaceDescriptor.LinkPolicy == 17)
                            st.LinkPolicy = 31;
                        else if (interfaceDescriptor.LinkPolicy == 31)
                            st.LinkPolicy = 0;

                        AmtCallStatus r = computer.Network.Wireless.SetInterfaceSettings(st);
                        if (r != AmtCallStatus.SUCCESS)
                        {
                            MessageBox.Show(this, string.Format(Properties.Resources.SetInterfaceSettingsError, r.ToString()), Properties.Resources.InterfaceError, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        else
                        {
                            if (computer.Network != null)
                            {
                                computer.Network.ClearCache();
                                if (computer.Network.Wireless != null)
                                    computer.Network.Wireless.ClearCache();
                            }
                            mainform.UpdateTreeView();
                        }
                    }
                }
            }
        }

        private void editWirelessProfilesButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                new EditWirelessProfilesForm(computer).ShowDialog(this);
                mainform.UpdateTreeView();
            }
        }

        private void editComputerNameButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                new EditComputerNameForm(computer).ShowDialog(this);
                mainform.UpdateTreeView();
            }
        }

        private void setMePowerSaveButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (computer.State == AmtSystemObjState.Connected)
                {
                    if (computer.AmtRev == 2)
                    {
                        if (computer.AmtRevMin < 5)
                        {
                            new Me2PowerSaverForm(computer).ShowDialog(this);
                        }
                        else
                        {
                            new Me3PowerSaverForm(computer).ShowDialog(this);
                        }
                    }
                    else if (computer.AmtRev >= 3)
                    {
                        new Me3PowerSaverForm(computer).ShowDialog(this);
                    }
                }
                if (mainTreeView.SelectedNode == systemNode)
                    mainform.UpdateTreeView();
            }
        }

        private void setUserAccountsButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (computer.State == AmtSystemObjState.Connected)
                {
                    AmtSecurityAdmin.AmtUserAcl[] accounts;
                    AmtCallStatus r = computer.SecurityAdmin.GetUserAclEntries(out accounts);
                    if (r == AmtCallStatus.SUCCESS && accounts != null)
                    {
                        AccessControlSetupForm form = new AccessControlSetupForm(accounts, computer);
                        if (form.ShowDialog(this) == DialogResult.OK)
                        {
                            r = computer.SecurityAdmin.ResetUserAclEntries(form.aclArray);
                            if (r != AmtCallStatus.SUCCESS)
                                Utils.ErrorMessageBox(this, "ResetUserAclEntries", r);
                        }
                    }
                }
                mainform.UpdateTreeView();
            }
        }

        private void certificateStoreButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (computer.State == AmtSystemObjState.Connected)
                {
                    if (new CertificateStoreForm(computer).ShowDialog(this) == DialogResult.OK)
                    {
                        mainform.UpdateTreeView();
                    }
                }
            }
        }

        private void kerberosSetupButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (computer.State == AmtSystemObjState.Connected)
                {
                    if (new KerberosConfigForm(computer, false).ShowDialog(this) == DialogResult.OK)
                    {
                        mainform.UpdateTreeView();
                    }
                }
            }
        }

        private void remoteAccessSetupButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (computer.State == AmtSystemObjState.Connected)
                {
                    new RemoteAccessSetupForm(computer).ShowDialog(this);
                    mainform.UpdateTreeView();
                }
            }
        }

        private void rememberPasswordCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                computer.RememberPassword = rememberPasswordCheckBox.Checked;
                computer.Password = passwordTextBox.Text;
                mainform.SaveSettings();

                if (mainTreeView.SelectedNode == systemNode)
                    mainform.UpdateTreeView();
            }
        }

        private void saveCertificateToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            X509Certificate2[] certs = null;

            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                certs = computer.GetTlsCertificates();
            }


            if (certs != null && certs.Length > 0)
            {
                saveFileDialog.DefaultExt = Properties.Resources.CertificateExtension;
                saveFileDialog.Filter = Properties.Resources.CertificateFileFilter;
                saveFileDialog.Title = Properties.Resources.SaveCertificateTitle;
                if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    byte[] certBin = certs[0].Export(X509ContentType.Cert);
                    FileStream f = new FileStream(saveFileDialog.FileName, FileMode.Create);
                    f.Write(certBin, 0, certBin.Length);
                    f.Close();
                }
            }
        }

        private void padlockPictureBox_DoubleClick(object sender, EventArgs e)
        {
            X509Certificate2[] certs = null;
            if (mainTreeView.SelectedNode == null)
                return;

            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                certs = computer.GetTlsCertificates();
            }

            if (certs != null && certs.Length > 0)
                X509Certificate2UI.DisplayCertificate(certs[0], Handle);
        }

        private void meTimeButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (computer.State == AmtSystemObjState.Connected)
                {
                    new ManageabilityControls.SetAmtTimeForm(computer).ShowDialog(this);
                }
            }
        }

        private void webUiToggleButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                if (computer.State == AmtSystemObjState.Connected)
                {
                    AmtSecurityAdmin.EnabledInterfaces eif = computer.SecurityAdmin.GetEnabledInterfaces();
                    eif.WebUI = !eif.WebUI;
                    computer.SecurityAdmin.SetEnabledInterfaces(eif);
                }

                if (mainTreeView.SelectedNode == systemNode)
                    mainform.UpdateTreeView();
            }
        }

        private void networkSettingsRefreshButton_Click(object sender, EventArgs e)
        {
            if (computer.Network != null)
            {
                computer.Network.ClearCache();
                if (computer.Network.Wireless != null)
                    computer.Network.Wireless.ClearCache();
            }
            mainform.UpdateTreeView();
        }

        private void wsManSettingButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                if (computer.State == AmtSystemObjState.Connected)
                    new WsmanSettingForm(computer).ShowDialog(this);
            }
        }

        private void disconnectCiraButton_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(this, "Disconnect CIRA Connection?", "Remote Management", MessageBoxButtons.OKCancel, MessageBoxIcon.Question) == DialogResult.OK)
            {
                AmtCallStatus r = computer.RemoteAccessAdmin.CloseRemoteAccessConnection();
                if (r != AmtCallStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "CloseRemoteAccessConnection", r);
                }
                else
                {
                    computer.Disconnect();
                }
            }
        }

        private void alarmClockButton_Click(object sender, EventArgs e)
        {
            // Create new dialog that allows the user to edit alarm clock settings.
            AlarmClockForm alarmClockDialog = new AlarmClockForm();
            alarmClockDialog.CurrentPowerSetting = computer.SecurityAdmin.GetPowerSavingOptionsStr();

            AmtRemoteControl.AmtAlarmWakeupState state;
            DateTime nextTime;
            TimeSpan interval;
            AmtCallStatus r = computer.Remote.GetAlarmWakeUp(out state, out nextTime, out interval);
            if (r != AmtCallStatus.SUCCESS)
                Utils.ErrorMessageBox(this, "GetAlarmWakeUp", r);

            alarmClockDialog.Computer = this.computer;
            alarmClockDialog.Enabled = (state == AmtRemoteControl.AmtAlarmWakeupState.WakeUpIsArmed);

            if (computer.SecurityAdmin.GetProvisioningMode() == AmtSecurityAdmin.ProvisioningMode.Enterprise)
                alarmClockDialog.NextTime = new DateTime(nextTime.ToBinary(), DateTimeKind.Utc);
            else
                alarmClockDialog.NextTime = nextTime;

            alarmClockDialog.Interval = interval;
            if ((alarmClockDialog.ShowDialog() == DialogResult.OK) &&
                (alarmClockDialog.Modified))
            {
                AmtRemoteControl.AmtAlarmWakeupCommand command;
                command = (alarmClockDialog.Enabled) ?
                    AmtRemoteControl.AmtAlarmWakeupCommand.SetAlarmWake :
                    AmtRemoteControl.AmtAlarmWakeupCommand.ClearAlarmWake;
                DateTime startTime = alarmClockDialog.StartTime;
                System.Diagnostics.Debug.WriteLine("About to set alarm");
                System.Diagnostics.Debug.Print("startTime: {0}", startTime.ToString());
                interval = alarmClockDialog.Interval;
                System.Diagnostics.Debug.Print("Interval Timespan: {0}", interval.ToString());
                r = computer.Remote.SetAlarmWakeUp(command, startTime, interval);
                if (r == AmtCallStatus.SUCCESS)
                {
                    alarmClockLabel.Text = (command == AmtRemoteControl.AmtAlarmWakeupCommand.SetAlarmWake) ? "Armed" : "Stopped";
                }
                else
                {
                    alarmClockLabel.Text = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA;
                    Utils.ErrorMessageBox(this, "SetAlarmWakeUp", r);
                }
            }
        }

        private void controlModeButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null)
                return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                new ManageabilityControls.ControlModeForm(computer).ShowDialog(this);

                mainform.UpdateTreeView();
            }
        }

        private void unprovisionButton_Click(object sender, EventArgs e)
        {
            mainform.Unprovision(sender, e);
        }

        private void ipv6InterfacesComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Assuming that only AMT6 and later has support for IPv6.
            System.Diagnostics.Debug.Assert(computer.CoreVersion >= 6.0);

            string selectedInterfaceID = (string)this.ipv6InterfacesComboBox.SelectedItem;

            // Update the displayed IPv6 addresses for the newly selected IPv6 interface.
            this.ipv6AddrsListBox.Items.Clear();

            IPv6PortSettings[] ipv6PortSettings = computer.Network.GetIPv6Settings();
            System.Diagnostics.Debug.Assert(ipv6PortSettings != null);

            // Populate the available IPv6 Interface names and select the first match.
            foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)
            {
                if (String.Compare(ipv6PortSetting.InstanceID, selectedInterfaceID, true) == 0)
                {
                    // Populate IPv6 addresses for the selected IPv6 interface.
                    foreach (IPv6PortSettings.AddressInfo ipv6AddrInfo in ipv6PortSetting.CurrentAddressInfo)
                        this.ipv6AddrsListBox.Items.Add(ipv6AddrInfo.IPAddr.ToString());
                    break;
                }
            } // END foreach (IPv6PortSettings ipv6PortSetting in ipv6PortSettings)

            if (this.ipv6AddrsListBox.Items.Count == 0)
                this.ipv6AddrsListBox.Items.Add(ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_NA);
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            //
            // Get the list of selected IP addresses and copy them to the system clipboard.
            //
            string ipAddrsToCopy = String.Empty;
            foreach (string selectedItem in this.ipv6AddrsListBox.SelectedItems)
                ipAddrsToCopy += String.Format("{0}\r\n", selectedItem);
            ipAddrsToCopy.TrimEnd(new char[] { '\r', '\n' });
            Clipboard.SetData(DataFormats.StringFormat, ipAddrsToCopy);
        }

        private void selectAllToolStripMenuItem_Click(object sender, EventArgs e)
        {
            for (int index = 0; index < this.ipv6AddrsListBox.Items.Count; index++)
                this.ipv6AddrsListBox.SetSelected(index, true);
        }

        private void tlsStateButton_Click(object sender, EventArgs e)
        {
            // Display the dialog that will allow the user to modify the 
            // Enterprise security settings and certificate used therein.
            EditTLSSettingsForm editTlsStateForm = new EditTLSSettingsForm(computer);
            if ((editTlsStateForm.ShowDialog() == DialogResult.OK) && editTlsStateForm.IsUpdated)
            {
                //
                // Apply the updated settings.
                //
                AmtCallStatus r = AmtCallStatus.SUCCESS;
                if ((editTlsStateForm.UseTlsPrev != editTlsStateForm.UseTls) &&
                    editTlsStateForm.UseTls)
                {
                    //
                    // Before we can enable TLS communication, we need to ensure 
                    // that the system time has been sync'd with the management 
                    // console (Commander).
                    //
                    DateTime n = computer.SecurityAdmin.GetProvisioningMode() == AmtSecurityAdmin.ProvisioningMode.Enterprise ? DateTime.UtcNow : DateTime.Now;
                    computer.NetworkTime.SetHighAccuracyTimeSynch(n); // Ignore the error code for now.

                    // Enable TLS communication.
                    r = computer.SecurityAdmin.EnableTls(
                        (editTlsStateForm.UseLocalAuth ? TlsAthenticationType.MutualAuth : TlsAthenticationType.ServerAuth),
                        (editTlsStateForm.UseRemoteAuth ? TlsAthenticationType.MutualAuth : TlsAthenticationType.ServerAuth),
                        editTlsStateForm.TrustedCNs, editTlsStateForm.TlsCert.Handle, true,
                        editTlsStateForm.AcceptNonSecureConnections);
                    if (r != AmtCallStatus.SUCCESS)
                    {
                        Utils.ErrorMessageBox(this, "SetTlsSettings", r);
                        return;
                    }

                    computer.UseTls = true;
                    if ((editTlsStateForm.UseLocalAuth) || (editTlsStateForm.UseRemoteAuth))
                        computer.UseMutualTls = true;
                }
                else if (editTlsStateForm.UseTls)
                {
                    // Update TLS communication settings.
                    r = computer.SecurityAdmin.UpdateTlsSettings(
                        (editTlsStateForm.UseLocalAuth ? TlsAthenticationType.MutualAuth : TlsAthenticationType.ServerAuth),
                        (editTlsStateForm.UseRemoteAuth ? TlsAthenticationType.MutualAuth : TlsAthenticationType.ServerAuth),
                        editTlsStateForm.TrustedCNs, editTlsStateForm.AcceptNonSecureConnections);
                    if (r != AmtCallStatus.SUCCESS)
                    {
                        Utils.ErrorMessageBox(this, "SetTlsSettings", r);
                        return;
                    }

                    computer.UseTls = true;
                    if ((editTlsStateForm.UseLocalAuth) || (editTlsStateForm.UseRemoteAuth))
                        computer.UseMutualTls = true;
                }
                else
                {
                    // Disable TLS communication settings.
                    r = computer.SecurityAdmin.DisableTls();
                    if (r != AmtCallStatus.SUCCESS)
                    {
                        Utils.ErrorMessageBox(this, "SetTlsSettings", r);
                        return;
                    }

                    computer.UseTls = false;
                    computer.UseMutualTls = false;
                }

                // Re-connect to use the new TLS settings.
                computer.Disconnect();
                //computer.Connect(); // TODO: Resolve how to re-connect
            }
        }

        private void kvmLabel_Update()
        {
            if (mainTreeView.SelectedNode == null)
                return;

            // String setup
            string EnabledText = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_Enabled;
            string DisabledText = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_Disabled;
            string UnsupportedText = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_Not_Supported;


            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                AmtKVM.KVMStates KvmState = computer.KVM.KVMState.Value;

                if (computer.State == AmtSystemObjState.Connected)
                {
                    if (KvmState == AmtKVM.KVMStates.ENABLED_STANDARD_PORT_ONLY)
                        kvmLabel.Text = "Standard Port " + EnabledText;
                    else if (KvmState == AmtKVM.KVMStates.ENABLED_REDIR_PORT_ONLY)
                        kvmLabel.Text = "Redirection Port " + EnabledText;
                    else if (KvmState == AmtKVM.KVMStates.ENABLED_BOTH_PORTS)
                        kvmLabel.Text = "All ports " + EnabledText;
                    else if (KvmState == AmtKVM.KVMStates.ERROR_UNKNOWN_STATE)
                        kvmLabel.Text = UnsupportedText;
                    else
                        kvmLabel.Text = DisabledText;
                }
            }
        }

        private void UserConsentLabel_Update()
        {
            if (mainTreeView.SelectedNode == null)
                return;

            // String setup            
            string UnsupportedText = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_Not_Supported;
            string NoneText = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_None;
            string KVMOnlyText = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_KVM_Only;
            string AllRedirText = ManageabilityCommanderTool.Properties.Resources.Node_AMTSys_All_Redir; 


            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                TreeNode systemNode = mainTreeView.SelectedNode;
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;
                switch (computer.UserConsent.OptInRequired)
                {
                    case AmtUserConsent.OptInRequiredValues.None:
                        //else if state == none
                        UserConsentLabel.Text = NoneText;
                        break;

                    //else if policy == KVM Only
                    case AmtUserConsent.OptInRequiredValues.KVM:
                        //else if state == KVM Only (will need to check KVM Opt in also??)
                        UserConsentLabel.Text = KVMOnlyText;
                        break;

                    //else if policy == all redir apps
                    case AmtUserConsent.OptInRequiredValues.All:
                        //else if state == all redir apps
                        UserConsentLabel.Text = AllRedirText;
                        break;
                }

                // TODO: detect unsupported.
                //Get User Consent state
                //if State == not supported (pre 6.0)
                //UserConsentLabel.Text = UnsupportedText;

                //else error case?
                
            }
        }

        private void linkPreferenceButton_Click(object sender, EventArgs e)
        {
            AmtWireless.LinkPreference lp = (computer.Wireless.GetWirelessPreference() == AmtWireless.LinkPreference.HOST) ?
                AmtWireless.LinkPreference.ME : AmtWireless.LinkPreference.HOST;
            computer.Wireless.SetWirelessPreference(lp);
            UpdatePanel();
        }


        private void KVMViewerButton_Click(object sender, EventArgs e)
        {
            LaunchKVMViewer(ManageabilityControls.KVMViewer.ConnectionType.Redirection); 
        }

        private void KVMStandardButton_Click(object sender, EventArgs e)
        {
            LaunchKVMViewer(ManageabilityControls.KVMViewer.ConnectionType.Standard); 
        }


        private void LaunchKVMViewer(ManageabilityControls.KVMViewer.ConnectionType type)
        {
            string Password = ""; 

            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtSystem))
            {
                AmtSystem computer = (AmtSystem)mainTreeView.SelectedNode.Tag;

                //check KVM Dlls?

                if (computer.MyComputer)
                {
                    MessageBox.Show(this,
                        ManageabilityCommanderTool.Properties.Resources.MainForm_Terminal_MyComputer,
                        ManageabilityCommanderTool.Properties.Resources.Mainform_KVM,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                else if (computer.CoreVersion == 0 || (computer.Redirection != null && computer.Redirection.GetRedirectionListenerState() == true))                
                {
                    if (type == ManageabilityControls.KVMViewer.ConnectionType.Redirection && !ManageabilityControls.KVMViewer.RedirectFormAvailable)
                    {
                        if (computer.KVMForm != null)
                        {
                            ManageabilityControls.KVMViewer Viewer = (ManageabilityControls.KVMViewer)computer.KVMForm;

                            if (Viewer.connectionType != type && Viewer.connectionType != ManageabilityControls.KVMViewer.ConnectionType.Disconnected)
                            {
                                MessageBox.Show(this,
                                    ManageabilityCommanderTool.Properties.Resources.MainForm_KVM_RedirOpen,
                                    ManageabilityCommanderTool.Properties.Resources.Mainform_KVM,
                                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                                return;
                            }
                        }
                        else
                        {
                            MessageBox.Show(this,
                                ManageabilityCommanderTool.Properties.Resources.MainForm_KVM_RedirOpen,
                                ManageabilityCommanderTool.Properties.Resources.Mainform_KVM,
                                MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                            return;
                        }
                    }

                    if (computer.KVMForm != null)
                    {
                        //KVM Viwerer is already open, so check if the same type 
                        //of viewer is being request.
                        ManageabilityControls.KVMViewer Viewer = (ManageabilityControls.KVMViewer)computer.KVMForm;

                        if (Viewer.connectionType != type && Viewer.connectionType != ManageabilityControls.KVMViewer.ConnectionType.Disconnected)
                        {
                            MessageBox.Show(this,
                                ManageabilityCommanderTool.Properties.Resources.MainForm_KVM_AlreadyOpen,
                                ManageabilityCommanderTool.Properties.Resources.Mainform_KVM,
                                MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                            return;
                        }
                    }

                    if (type == ManageabilityControls.KVMViewer.ConnectionType.Standard)
                    {
                        //need to make sure have RFB password
                        if (computer.KVMRfbPassword == string.Empty)
                        {
                            AskKVMPasswordForm KVMPassForm = new AskKVMPasswordForm();
                            if (KVMPassForm.ShowDialog() == DialogResult.OK)
                                Password = KVMPassForm.Password;
                            else
                                return;  //don't continue without password
                        }
                        else
                            Password = computer.KVMRfbPassword;
                    }
                    else
                        Password = computer.KVMRfbPassword; //for redir port dont really care if password is empty


                    ManageabilityControls.KVMViewer.ShowKVMForm(computer, type, Password); 
                }
                else
                {
                    MessageBox.Show(this,
                        ManageabilityCommanderTool.Properties.Resources.MainForm_PortDisable_ErrStr,
                        ManageabilityCommanderTool.Properties.Resources.Mainform_KVM,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }
    }
}
