// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Data;
using System.Text;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool.Panels
{
    public partial class StorageBlockPanel : UserControl, PanelInterface
    {
        private MainForm mainform;
        private TreeView mainTreeView;
        private AmtSystem computer = null;
        private AmtStorageBlock block;
        private AmtApplication app;

        public StorageBlockPanel()
        {
            InitializeComponent();           
        }

        public void Setup(MainForm mainform, object obj1, object obj2)
        {
            this.mainform = mainform;
            this.mainTreeView = mainform.GetTreeView();
            this.computer = (AmtSystem)obj1;
            object[] args = (object[])obj2;
            app = (AmtApplication)args[0];
            block = (AmtStorageBlock)args[1];
            storagePictureBox.AllowDrop = true;

            if (Properties.Settings.Default.StorageViewType == 0) viewHEXToolStripMenuItem_Click(this, null);
            if (Properties.Settings.Default.StorageViewType == 1) viewUTF8ToolStripMenuItem_Click(this, null);
            if (Properties.Settings.Default.StorageViewType == 2) viewAsImageToolStripMenuItem_Click(this, null);
            if (Properties.Settings.Default.StorageViewType == 3) viewAsHTMLToolStripMenuItem_Click(this, null);
        }

        public void Cleanup()
        {
            computer = null;
            block = null;
            app = null;
        }

        public void PerformAction(string command, object[] args) { }

        public void UpdatePanel()
        {
            if (computer == null) return;

            blockOwnerLabel.Text = app.VendorName + Properties.Resources.SlashSeperator + app.ApplicationName;
            if (block.Hidden == true)
            {
                blockVisibleLabel.Text = Properties.Resources.Hidden;
            }
            else
            {
                blockVisibleLabel.Text = Properties.Resources.Visible;
            }
            blockSizeLabel.Text = String.Format(ManageabilityCommanderTool.Properties.Resources.Block_Size, block.Size.ToString());

            openInWMIViewerToolStripMenuItem.Enabled = block.Name.StartsWith("IntelComDiag") || block.Name.StartsWith("DiagWriter");

            if (block.IsCached() == false)
            {
                storageDataTextBox.Visible = true;
                storagePictureBox.Visible = false;
                storageWebBrowser.Visible = false;
                storageDataTextBox.Text = Properties.Resources.Loading;
                new Thread(new ParameterizedThreadStart(FetchStorageBlockEx)).Start(new object[] { computer, block });
            }
            else
            {
                lock (block)
                {
                    byte[] buf = block.ReadBlock();
                    if (buf != null)
                    {
                        if (viewHEXToolStripMenuItem.Checked == true)
                        {
                            storageDataTextBox.Visible = true;
                            storagePictureBox.Visible = false;
                            storageWebBrowser.Visible = false;
                            storageDataTextBox.Text = Utils.BytesToHex(buf, 4);
                        }
                        else if (viewUTF8ToolStripMenuItem.Checked == true)
                        {
                            storageDataTextBox.Visible = true;
                            storagePictureBox.Visible = false;
                            storageWebBrowser.Visible = false;
                            storageDataTextBox.Text = UTF8Encoding.UTF8.GetString(buf);
                        }
                        else if (viewAsImageToolStripMenuItem.Checked == true)
                        {
                            try
                            {
                                storageDataTextBox.Visible = false;
                                storagePictureBox.Visible = true;
                                storageWebBrowser.Visible = false;
                                storagePictureBox.Image = Bitmap.FromStream(new MemoryStream(buf));
                            }
                            catch (Exception)
                            {
                                storageDataTextBox.Visible = true;
                                storagePictureBox.Visible = false;
                                storageWebBrowser.Visible = false;
                                storageDataTextBox.Text = Properties.Resources.InvalidImageData;
                            }
                        }
                        else if (viewAsHTMLToolStripMenuItem.Checked == true)
                        {
                            storageDataTextBox.Visible = false;
                            storagePictureBox.Visible = false;
                            storageWebBrowser.Visible = true;
                            // Save buffer to HTML file
                            FileStream f = new FileStream(Application.StartupPath + "\\StorageTemp.html", FileMode.Create);
                            f.Write(buf, 0, buf.Length);
                            f.Close();
                            // Point the web browser to the new file
                            storageWebBrowser.Url = new Uri("file://" + Application.StartupPath + "\\StorageTemp.html");
                            storageWebBrowser.Refresh();
                        }
                    }
                    else
                    {
                        storageDataTextBox.Clear();
                    }
                }
            }
        }

        public void ForceUpdatePanel()
        {
        }

        private void storageBlockVisibilityButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                TreeNode n = mainTreeView.SelectedNode;
                AmtStorageBlock block = (AmtStorageBlock)n.Tag;

                AmtStorageWrapper.AmtStorageStatus r = block.SetVisibility(!block.Hidden);
                if (r != AmtStorageWrapper.AmtStorageStatus.SUCCESS)
                {
                    MessageBox.Show(this, string.Format(Properties.Resources.BlockRemovalError, r.ToString()), Properties.Resources.StorageBlocksTitle, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                else
                {
                    string hiddenStr = "";
                    if (block.Hidden == true) hiddenStr = " " + Properties.Resources.Hidden2;
                    n.Text = block.Name + hiddenStr;
                    mainform.UpdateTreeView();
                }
            }
        }

        private void storagePermissionsButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                AmtStorageBlock block = (AmtStorageBlock)mainTreeView.SelectedNode.Tag;
                new ManageabilityControls.StoragePermissionsEditForm(null, null, block).ShowDialog(this);
            }
        }

        private void storageClearBlockButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                AmtStorageBlock block = (AmtStorageBlock)mainTreeView.SelectedNode.Tag;

                if (MessageBox.Show(this, Properties.Resources.ClearBlockQuestion, Properties.Resources.DataStorageTitle, MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.Yes)
                {
                    byte[] buf = new byte[block.Size];
                    block.WriteBlock(buf);
                    mainform.UpdateTreeView();
                }
            }
        }

        private void loadBlockButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                AmtStorageBlock block = (AmtStorageBlock)mainTreeView.SelectedNode.Tag;

                openFileDialog.Filter = ManageabilityCommanderTool.Properties.Resources.MainForm_FD_SaveBin;
                openFileDialog.FileName = (string)Application.UserAppDataRegistry.GetValue("OpenStorageFile", "");
                if (openFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    FileStream f;
                    try
                    {
                        f = new FileStream(openFileDialog.FileName, FileMode.Open);
                    }
                    catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                    Application.UserAppDataRegistry.SetValue("OpenStorageFile", openFileDialog.FileName);
                    if (f.Length > block.Size)
                    {
                        MessageBox.Show(this,
                            ManageabilityCommanderTool.Properties.Resources.MainForm_FSize_Exceed_ErrStr,
                            ManageabilityCommanderTool.Properties.Resources.MainForm_NVRAM_Storage,
                            MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                    else
                    {
                        byte[] buf = new byte[f.Length];
                        f.Read(buf, 0, (int)f.Length);
                        f.Close();
                        block.WriteBlock(buf);
                        mainform.UpdateTreeView();
                    }
                }
            }
        }

        private void saveBlockButton_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                AmtStorageBlock block = (AmtStorageBlock)mainTreeView.SelectedNode.Tag;

                saveFileDialog.Filter = ManageabilityCommanderTool.Properties.Resources.MainForm_FD_SaveBin;
                saveFileDialog.FileName = (string)Application.UserAppDataRegistry.GetValue("SaveStorageFile", Properties.Resources.StorageBlockDefaultFilename);
                if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    byte[] buf = block.ReadBlock();
                    if (buf == null || buf.Length == 0)
                    {
                        MessageBox.Show(this, Properties.Resources.ReadBlockReturnNull, Properties.Resources.BlockSaveError, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                    Application.UserAppDataRegistry.SetValue("SaveStorageFile", saveFileDialog.FileName);
                    FileStream f = new FileStream(saveFileDialog.FileName, FileMode.Create);
                    f.Write(buf, 0, buf.Length);
                    f.Close();
                }
            }
        }

        private void viewHEXToolStripMenuItem_Click(object sender, EventArgs e)
        {
            viewHEXToolStripMenuItem.Checked = true;
            viewUTF8ToolStripMenuItem.Checked = false;
            viewAsImageToolStripMenuItem.Checked = false;
            viewAsHTMLToolStripMenuItem.Checked = false;
            Properties.Settings.Default.StorageViewType = 0;
            UpdatePanel();
        }

        private void viewUTF8ToolStripMenuItem_Click(object sender, EventArgs e)
        {
            viewHEXToolStripMenuItem.Checked = false;
            viewUTF8ToolStripMenuItem.Checked = true;
            viewAsImageToolStripMenuItem.Checked = false;
            viewAsHTMLToolStripMenuItem.Checked = false;
            Properties.Settings.Default.StorageViewType = 1;
            UpdatePanel();
        }

        private void viewAsImageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            viewHEXToolStripMenuItem.Checked = false;
            viewUTF8ToolStripMenuItem.Checked = false;
            viewAsImageToolStripMenuItem.Checked = true;
            viewAsHTMLToolStripMenuItem.Checked = false;
            Properties.Settings.Default.StorageViewType = 2;
            UpdatePanel();
        }

        private void viewAsHTMLToolStripMenuItem_Click(object sender, EventArgs e)
        {
            viewHEXToolStripMenuItem.Checked = false;
            viewUTF8ToolStripMenuItem.Checked = false;
            viewAsImageToolStripMenuItem.Checked = false;
            viewAsHTMLToolStripMenuItem.Checked = true;
            Properties.Settings.Default.StorageViewType = 3;
            UpdatePanel();
        }

        private void openInWMIViewerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                AmtStorageBlock block = (AmtStorageBlock)mainTreeView.SelectedNode.Tag;
                byte[] buf = block.ReadBlock();
                if (buf != null && buf.Length > 0) new ManageabilityControls.DiagnosticViewerForm(buf).Show();
            }
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                AmtStorageBlock block = (AmtStorageBlock)mainTreeView.SelectedNode.Tag;
                block.ClearCache();
                mainform.UpdateTreeView();
            }
        }

        private void storagePictureBox_DragDrop(object sender, DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);

            if (files == null || files.Length == 0 || mainTreeView.SelectedNode == null) return;
            if (mainTreeView.SelectedNode.Tag != null && mainTreeView.SelectedNode.Tag.GetType() == typeof(AmtStorageBlock))
            {
                AmtStorageBlock block = (AmtStorageBlock)mainTreeView.SelectedNode.Tag;

                FileStream f;
                try
                {
                    f = new FileStream(files[0], FileMode.Open);
                }
                catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                if (f.Length > block.Size)
                {
                    MessageBox.Show(this,
                        ManageabilityCommanderTool.Properties.Resources.MainForm_FSize_Exceed_ErrStr,
                        ManageabilityCommanderTool.Properties.Resources.MainForm_NVRAM_Storage,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
                else
                {
                    byte[] buf = new byte[f.Length];
                    f.Read(buf, 0, (int)f.Length);
                    f.Close();
                    block.WriteBlock(buf);
                    mainform.UpdateTreeView();
                }
            }
        }

        private void storagePictureBox_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == true) e.Effect = DragDropEffects.All;
        }

        private void FetchStorageBlockEx(object param)
        {
            object[] ps = (object[])param;
            AmtSystem computer = (AmtSystem)ps[0];
            AmtStorageBlock block = (AmtStorageBlock)ps[1];
            if (computer.WsMan == null) AmtStorageWrapper.Storage_InitializeCOMinThread();
            lock (block) { block.ReadBlock(); }
            if (computer.WsMan == null) AmtStorageWrapper.Storage_UninitializeCOMinThread();
            this.Invoke(new ParameterizedThreadStart(FetchStorageBlockEx2), new object[] { null });
        }

        private void FetchStorageBlockEx2(object param)
        {
            UpdatePanel();
        }

        private void storageDataTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.A)
            {
                e.Handled = true;
                storageDataTextBox.SelectAll();
            }
        }

    }
}
