// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class RemoteAccessPolicyForm : Form
    {
        private AmtSystem system;
        private AmtRemoteAccessAdmin.AmtRemoteAccessPolicy editPolicy = null;
        private bool Loaded = false;

        public RemoteAccessPolicyForm(AmtSystem system)
        {
            this.system = system;
            InitializeComponent();
        }

        private void RemoteAccessPolicyForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            system.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);

            if (editPolicy == null)
            {
                triggerTypeComboBox.SelectedIndex = 0;
                UpdateServers();
            }
            else
            {
                triggerTypeComboBox.Enabled = false;
            }

            Loaded = true;
            UpdateInfo();
        }

        private void SystemStateChangedHandler(AmtSystem computer)
        {
            if (computer.State != AmtSystemObjState.Connected) DialogResult = DialogResult.Cancel;
        }

        private void UpdateInfo()
        {
            if (Loaded == false) return;

            periodicNumericUpDown.Enabled = (triggerTypeComboBox.SelectedIndex == 2);

            bool ok = true;
            ok &= ((remoteServerListView.CheckedItems.Count != 0) && (remoteServerListView.CheckedItems.Count <= 2));
            okButton.Enabled = ok;
        }

        private void UpdateServers()
        {
            AmtCallStatus r;
            remoteServerListView.BeginUpdate();
            remoteServerListView.Items.Clear();
            AmtRemoteAccessAdmin.AmtRemoteAccessServer[] servers;
            r = system.RemoteAccessAdmin.GetRemoteServers(out servers);
            if (r == AmtCallStatus.SUCCESS && servers != null)
            {
                foreach (AmtRemoteAccessAdmin.AmtRemoteAccessServer server in servers)
                {
                    // Fill name postion
                    string name = server.ServerName;
                    if (name == null) name = server.ServerAddress + " / " + server.ServerCommonName;
                    ListViewItem l = new ListViewItem(new string[] { name }, 0);
                    l.Tag = server;
                    remoteServerListView.Items.Add(l);
                }
            }
            remoteServerListView.EndUpdate();
        }

        private void triggerTypeComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            uint[] servers = new uint[remoteServerListView.CheckedItems.Count];
            int i = 0;
            foreach (ListViewItem l in remoteServerListView.CheckedItems)
            {
                AmtRemoteAccessAdmin.AmtRemoteAccessServer server = (AmtRemoteAccessAdmin.AmtRemoteAccessServer)l.Tag;
                servers[i++] = server.Handle;
            }

            byte[] data = null;
            if (triggerTypeComboBox.SelectedIndex == 2) data = BitConverter.GetBytes((uint)System.Net.IPAddress.HostToNetworkOrder((int)periodicNumericUpDown.Value));

            AmtRemoteAccessAdmin.AmtRemoteAccessPolicy policy = new AmtRemoteAccessAdmin.AmtRemoteAccessPolicy(0, (uint)tunnelLifetimeNumericUpDown.Value, (AmtRemoteAccessAdmin.AmtRemoteAccessTrigger)triggerTypeComboBox.SelectedIndex, servers, data);

            if (editPolicy == null)
            {
                AmtCallStatus r = system.RemoteAccessAdmin.AddRemoteAccessPolicy(policy);
                if (r == AmtCallStatus.SUCCESS) DialogResult = DialogResult.OK;
                else Utils.ErrorMessageBox(this, "AddRemoteAccessPolicy", r);
            }
            else
            {
                policy.Handle = editPolicy.Handle;
                policy.WsmanRef = editPolicy.WsmanRef;
                AmtCallStatus r = system.RemoteAccessAdmin.UpdateRemoteAccessPolicy(policy);
                if (r == AmtCallStatus.SUCCESS) DialogResult = DialogResult.OK;
                else Utils.ErrorMessageBox(this, "UpdateRemoteAccessPolicy", r);
            }
        }

        private void remoteServerListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void remoteServerListView_ItemChecked(object sender, ItemCheckedEventArgs e)
        {
            if (this.remoteServerListView.CheckedItems.Count > 2)
            {
                // The AMT SDK spec defines the current limit of '2' for the 
                // number of MPS servers that can be enabled for a given policy.
                this.okButton.Enabled = false;

                MessageBox.Show("Unable to select more than two (2) servers for a policy.\n" +
                    "Please update your selections before applying these changes.",
                    "Policy Server Limit Exceeded", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
                this.okButton.Enabled = true;

            UpdateInfo();
        }

        public void SetPolicy(AmtRemoteAccessAdmin.AmtRemoteAccessPolicy policy)
        {
            editPolicy = policy;
            UpdateServers();

            triggerTypeComboBox.SelectedIndex = (int)policy.Trigger;

            if (policy.Servers != null)
            {
                foreach (uint h in policy.Servers)
                {
                    foreach (ListViewItem l in remoteServerListView.Items)
                    {
                        AmtRemoteAccessAdmin.AmtRemoteAccessServer server = (AmtRemoteAccessAdmin.AmtRemoteAccessServer)l.Tag;
                        if (h == server.Handle) l.Checked = true;
                    }
                }
            }

            tunnelLifetimeNumericUpDown.Value = policy.TunnelLifeTime;
            if (policy.Trigger == AmtRemoteAccessAdmin.AmtRemoteAccessTrigger.PERIODIC) periodicNumericUpDown.Value = policy.PeriodicTimer;
        }

        private void RemoteAccessPolicyForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            system.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

    }
}
