// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityCommanderTool
{
    public partial class RemoteAccessSetupForm : Form
    {
        private AmtSystem system;

        public RemoteAccessSetupForm(AmtSystem system)
        {
            this.system = system;
            InitializeComponent();
        }

        private void RemoteAccessSetupForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;
            this.mainToolTip.Active = Properties.Settings.Default.ToolTips;
            system.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);

            UpdateServers();
            UpdatePolicies();
            UpdateuserInitiated();
        }

        private void SystemStateChangedHandler(AmtSystem computer)
        {
            if (computer.State != AmtSystemObjState.Connected) DialogResult = DialogResult.Cancel;
        }

        private string GetServerName(uint handle)
        {
            AmtRemoteAccessAdmin.AmtRemoteAccessServer[] servers;
            AmtCallStatus r = system.RemoteAccessAdmin.GetRemoteServers(out servers);
            if (r != AmtCallStatus.SUCCESS) return Properties.Resources.Unknown;
            foreach (AmtRemoteAccessAdmin.AmtRemoteAccessServer server in servers)
            {
                if (server.Handle == handle)
                {
                    if (server.ServerName != null) return server.ServerName;
                    return server.ServerAddress.ToString();
                }
            }
            return Properties.Resources.Unknown;
        }

        private void UpdateuserInitiated()
        {
            AmtCallStatus r;
            bool e;

            // BIOS
            r = system.RemoteAccessAdmin.GetUserInitiatedInterfaceConfiguration(0, out e);
            if (r == AmtCallStatus.SUCCESS) { if (e == true) biosLabel.Text = Properties.Resources.Enabled; else biosLabel.Text = Properties.Resources.Disabled; }
            biosButton.Enabled = (r == AmtCallStatus.SUCCESS);

            // OS agent
            r = system.RemoteAccessAdmin.GetUserInitiatedInterfaceConfiguration(1, out e);
            if (r == AmtCallStatus.SUCCESS) { if (e == true) osLabel.Text = Properties.Resources.Enabled; else osLabel.Text = Properties.Resources.Disabled; }
            osButton.Enabled = (r == AmtCallStatus.SUCCESS);
        }

        private void UpdateServers()
        {
            AmtCallStatus r;
            remoteServerListView.BeginUpdate();
            remoteServerListView.Items.Clear();
            AmtRemoteAccessAdmin.AmtRemoteAccessServer[] servers;
            r = system.RemoteAccessAdmin.GetRemoteServers(out servers);
            if (r == AmtCallStatus.SUCCESS && servers != null)
            {
                foreach (AmtRemoteAccessAdmin.AmtRemoteAccessServer server in servers)
                {
                    // Fill name postion
                    string name = server.ServerName;
                    if (name == null) name = server.ServerAddress + " / " + server.ServerCommonName;

                    // Fill authentication postion
                    string auth = null;
                    if (server.Username != null) auth = string.Format("Username: {0}", server.Username, server.Password);
                    else
                    {
                        AmtSecurityAdmin.AmtStoredCert[] certs;
                        if (system.SecurityAdmin.CertStoreGetCertificates(out certs) == AmtCallStatus.SUCCESS)
                        {
                            foreach (AmtSecurityAdmin.AmtStoredCert cert in certs)
                            {
                                if (cert.Handle == server.MutualAuthCertHandle) auth = string.Format("Certificate: {0}", cert.ToString());
                            }
                        }

                        if (auth == null) auth = string.Format("Certificate Handle #{0}", server.MutualAuthCertHandle);
                    }
                    ListViewItem l = new ListViewItem(new string[] { name, auth }, 0);
                    l.Tag = server;
                    remoteServerListView.Items.Add(l);
                }
            }
            remoteServerListView.EndUpdate();
            remoteServerListView_SelectedIndexChanged(this, null);
        }

        private void UpdatePolicies()
        {
            AmtCallStatus r;
            policyListView.BeginUpdate();
            policyListView.Items.Clear();
            AmtRemoteAccessAdmin.AmtRemoteAccessPolicy[] policies;
            r = system.RemoteAccessAdmin.GetRemoteAccessPolicies(out policies);
            if (r == AmtCallStatus.SUCCESS && policies != null)
            {
                foreach (AmtRemoteAccessAdmin.AmtRemoteAccessPolicy policy in policies)
                {
                    string servers = "";
                    if (policy.Servers != null)
                    {
                        foreach (uint h in policy.Servers)
                        {
                            if (servers.Length != 0) servers += " or ";
                            servers += "\"" + GetServerName(h) + "\"";
                        }
                    }
                    else
                    {
                        servers = Properties.Resources.Unknown;
                    }

                    string name = "";
                    if (policy.TunnelLifeTime != 0)
                    {
                        if (policy.Trigger == AmtRemoteAccessAdmin.AmtRemoteAccessTrigger.USERINITIATED) name = string.Format(Properties.Resources.CiraUserTimeout, servers, policy.TunnelLifeTime);
                        if (policy.Trigger == AmtRemoteAccessAdmin.AmtRemoteAccessTrigger.ALERT) name = string.Format(Properties.Resources.CiraAlertTimeout, servers, policy.TunnelLifeTime);
                        if (policy.Trigger == AmtRemoteAccessAdmin.AmtRemoteAccessTrigger.PERIODIC) name = string.Format(Properties.Resources.CiraPeriodicTimeout, policy.PeriodicTimer, servers, policy.TunnelLifeTime);
                    }
                    else
                    {
                        if (policy.Trigger == AmtRemoteAccessAdmin.AmtRemoteAccessTrigger.USERINITIATED) name = string.Format(Properties.Resources.CiraUserNoTimeout, servers);
                        if (policy.Trigger == AmtRemoteAccessAdmin.AmtRemoteAccessTrigger.ALERT) name = string.Format(Properties.Resources.CiraAlertNoTimeout, servers);
                        if (policy.Trigger == AmtRemoteAccessAdmin.AmtRemoteAccessTrigger.PERIODIC) name = string.Format(Properties.Resources.CiraPeriodicNoTimeout, policy.PeriodicTimer, servers);
                    }

                    ListViewItem l = new ListViewItem(new string[] { name }, 6);
                    l.Tag = policy;
                    policyListView.Items.Add(l);
                }
            }
            policyListView.EndUpdate();
            policyListView_SelectedIndexChanged(this, null);
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            system.RemoteAccessAdmin.EnableUserInitiatedInterface(0, true);
        }

        private void disconnectButton_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show(this, "Disconnect remote connection if present?", "Remote Connection", MessageBoxButtons.YesNo, MessageBoxIcon.Asterisk) == DialogResult.Yes)
            {
                AmtCallStatus r = system.RemoteAccessAdmin.CloseRemoteAccessConnection();
                if (r != AmtCallStatus.SUCCESS)
                {
                    Utils.ErrorMessageBox(this, "CloseRemoteAccessConnection", r);
                }
                else
                {
                    // Disconnect from the system.
                    system.Disconnect();
                    this.Close();
                }
            }
        }

        private void remoteServerListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            removeServerButton.Enabled = editServerButton.Enabled = (remoteServerListView.SelectedItems.Count != 0);
        }

        private void removeServerButton_Click(object sender, EventArgs e)
        {
            if (remoteServerListView.SelectedItems.Count == 0) return;
            AmtRemoteAccessAdmin.AmtRemoteAccessServer server = (AmtRemoteAccessAdmin.AmtRemoteAccessServer)(remoteServerListView.SelectedItems[0].Tag);
            system.RemoteAccessAdmin.RemoveRemoteServer(server);
            UpdateServers();
        }

        private void addServerButton_Click(object sender, EventArgs e)
        {
            RemoteAccessServerForm form = new RemoteAccessServerForm(system);
            if (form.ShowDialog(this) == DialogResult.OK) UpdateServers();
            if (system.State != AmtSystemObjState.Connected) Close();
        }

        private void editServerButton_Click(object sender, EventArgs e)
        {
            if (remoteServerListView.SelectedItems.Count == 0) return;
            AmtRemoteAccessAdmin.AmtRemoteAccessServer server = (AmtRemoteAccessAdmin.AmtRemoteAccessServer)(remoteServerListView.SelectedItems[0].Tag);
            RemoteAccessServerForm form = new RemoteAccessServerForm(system);
            form.SetServer(server);
            if (form.ShowDialog(this) == DialogResult.OK) UpdateServers();
            if (system.State != AmtSystemObjState.Connected) Close();
        }

        private void remoteServerListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) { removeServerButton_Click(this, null); e.Handled = true; }
        }

        private void policyListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            removeTunnelButton.Enabled = editTunnelButton.Enabled = (policyListView.SelectedItems.Count != 0);
        }

        private void removeTunnelButton_Click(object sender, EventArgs e)
        {
            if (policyListView.SelectedItems.Count == 0) return;
            AmtRemoteAccessAdmin.AmtRemoteAccessPolicy policy = (AmtRemoteAccessAdmin.AmtRemoteAccessPolicy)(policyListView.SelectedItems[0].Tag);
            system.RemoteAccessAdmin.RemoveRemoteAccessPolicy(policy);
            UpdatePolicies();
        }

        private void addTunnelButton_Click(object sender, EventArgs e)
        {
            RemoteAccessPolicyForm form = new RemoteAccessPolicyForm(system);
            if (form.ShowDialog(this) == DialogResult.OK) UpdatePolicies();
            if (system.State != AmtSystemObjState.Connected) Close();
        }

        private void editTunnelButton_Click(object sender, EventArgs e)
        {
            if (policyListView.SelectedItems.Count == 0) return;
            AmtRemoteAccessAdmin.AmtRemoteAccessPolicy policy = (AmtRemoteAccessAdmin.AmtRemoteAccessPolicy)(policyListView.SelectedItems[0].Tag);
            RemoteAccessPolicyForm form = new RemoteAccessPolicyForm(system);
            form.SetPolicy(policy);
            if (system.State != AmtSystemObjState.Connected) Close();
            if (form.ShowDialog(this) == DialogResult.OK) UpdatePolicies();
            if (system.State != AmtSystemObjState.Connected) Close();
        }

        private void policyListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete) { removeTunnelButton_Click(this, null); e.Handled = true; }
        }

        private void biosButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus r;
            bool ee;
            r = system.RemoteAccessAdmin.GetUserInitiatedInterfaceConfiguration(0, out ee);
            if (r != AmtCallStatus.SUCCESS) { Utils.ErrorMessageBox(this, "GetUserInitiatedInterfaceConfiguration", r); return; }
            r = system.RemoteAccessAdmin.EnableUserInitiatedInterface(0, !ee);
            if (r != AmtCallStatus.SUCCESS) { Utils.ErrorMessageBox(this, "EnableUserInitiatedInterface", r); return; }
            UpdateuserInitiated();
        }

        private void osButton_Click(object sender, EventArgs e)
        {
            AmtCallStatus r;
            bool ee;
            r = system.RemoteAccessAdmin.GetUserInitiatedInterfaceConfiguration(1, out ee);
            if (r != AmtCallStatus.SUCCESS) { Utils.ErrorMessageBox(this, "GetUserInitiatedInterfaceConfiguration", r); return; }
            r = system.RemoteAccessAdmin.EnableUserInitiatedInterface(1, !ee);
            if (r != AmtCallStatus.SUCCESS) { Utils.ErrorMessageBox(this, "EnableUserInitiatedInterface", r); return; }
            UpdateuserInitiated();
        }

        private void RemoteAccessSetupForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            system.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

    }
}
