// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Data;
using System.Text;
using System.Drawing;
using Microsoft.Win32;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography.X509Certificates;
using ManageabilityControls;
using ManageabilityStack;

namespace ManageabilityConnectorTool
{
    public partial class MainForm : Form
    {
        private HeciWrapper ManagementEngine;
        private HeciBridgeClass HeciBridge = null;
        private string HeciBridgeStatusStr = null;
        private int HeciBridgeCount = 0;
        private string RemoteBridgeHostname = null;
        private ushort RemoteBridgePort = 10000;
        private string ProductTitle;
        private Point Position1, Position2;
        private DateTime ConnectionTime = DateTime.MinValue;
        private DateTime DisconnectionTime = DateTime.MinValue;

        private enum ConnectionIcon
        {
            Unknown,
            Disconnected,
            Connected,
            Direct,
        }

        private ConnectionIcon HardIcon = 0;
        private ConnectionIcon SoftIcon = 0;

        public MainForm(string[] args)
        {
            InitializeComponent();

            foreach (string arg in args)
            {
                if (arg.Length > 5 && string.Compare(arg.Substring(0, 8), "-bridge:", true) == 0)
                {
                    string[] str = arg.Substring(8).Split(':');
                    if (str.Length == 1) RemoteBridgeHostname = str[0];
                    else if (str.Length == 2)
                    {
                        RemoteBridgeHostname = str[0];
                        ushort.TryParse(str[1], out RemoteBridgePort);
                    }
                }
            }
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            Position1 = label6.Location;
            Position2 = softwareLabel.Location;

            toolStripStatusLabel1.Text = AboutForm.VersionString;
            this.Text = ProductTitle = Utils.FilterStr(this.Text);
            HeciBridgeStatusStr = heciBridgeToolStripStatusLabel.Text;
            ServicePointManager.ServerCertificateValidationCallback = new System.Net.Security.RemoteCertificateValidationCallback(RemoteCertificateValidationCallbackSink);
            UpdateEngine();
            SystemEvents.PowerModeChanged += new PowerModeChangedEventHandler(SystemEvents_PowerModeChanged);

            UpdateUI();
        }

        private void UpdateUI()
        {
            if (ManagementEngine == null || ManagementEngine.ClientConnect == null)
            {
                // No hardware management, hide the UI's for it.
                label6.Location = label4.Location;
                label4.Visible = false;
                softwareLabel.Location = hardwareLabel.Location;
                hardwareLabel.Visible = false;

                // Remove non-applicable testing features. 
                automaticTestingToolStripMenuItem.Visible = false;
                resetToolStripMenuItem.Visible = false;
                toolStripMenuItem4.Visible = false;
            }
            else
            {
                // No hardware management, hide the UI's for it.
                label6.Location = Position1;
                label4.Visible = true;
                softwareLabel.Location = Position2;
                hardwareLabel.Visible = true;

                // Remove non-applicable testing features. 
                automaticTestingToolStripMenuItem.Visible = true;
                resetToolStripMenuItem.Visible = true;
                toolStripMenuItem4.Visible = true;
            }

            testingFeaturesToolStripMenuItem.Visible = (ManagementEngine != null);

            if (ManagementEngine == null || ManagementEngine.Platform == HeciWrapper.IntelPlatform.Unknown)
            {
                managementEngineStatusToolStripMenuItem.Visible = false;
            }
            else
            {
                managementEngineStatusToolStripMenuItem.Visible = true;
            }

            managementEngineStatusToolStripMenuItem.Enabled = (ManagementEngine != null && ManagementEngine.Versions != null);
        }

        private bool RemoteCertificateValidationCallbackSink(object sender, X509Certificate certificate, X509Chain chain, System.Net.Security.SslPolicyErrors sslPolicyErrors)
        {
            if (sender.GetType() == typeof(System.Net.HttpWebRequest))
            {
                System.Net.HttpWebRequest r = (System.Net.HttpWebRequest)sender;
                X509Certificate2[] certArray = new X509Certificate2[chain.ChainElements.Count];
                for (int i = 0; i < chain.ChainElements.Count; i++)
                {
                    certArray[i] = chain.ChainElements[i].Certificate;
                }
                if (r.ConnectionGroupName != null) AmtSystem.AddTlsCertificateCache(r.ConnectionGroupName, certArray);
            }
            return true; // Accept All Certificates
        }


        private void UpdateEngine()
        {
            if (ManagementEngine != null) ManagementEngine.Dispose();

            if (RemoteBridgeHostname == null)
            {
                this.Text = ProductTitle;
                ManagementEngine = new HeciWrapper();
            }
            else
            {
                this.Text = ProductTitle + " - " + RemoteBridgeHostname;
                ManagementEngine = new HeciWrapper(RemoteBridgeHostname, RemoteBridgePort);
            }

            if (HeciBridge != null) HeciBridge.Heci = ManagementEngine;

            // Display the version number
            if (ManagementEngine.Platform == HeciWrapper.IntelPlatform.IntelAMT) 
            {
                platformLabel.Text = string.Format(Properties.Resources.IntelAmtVersion, ManagementEngine.GetVersion().ToString());
                platformLabel.Enabled = true;
                startRemoteConfigurationToolStripMenuItem.Visible = true;
                startRemoteConfigurationToolStripMenuItem.Enabled = (ManagementEngine.major >= 3 && ManagementEngine.MeInfo.GetZeroTouchEnabled() && ManagementEngine.MeInfo.GetProvisioningState() != HeciMeInfo.ProvisioningStateEnum.Post);
            }
            else if (ManagementEngine.Platform == HeciWrapper.IntelPlatform.IntelConsumer)
            {
                enrollWithSupportProviderToolStripMenuItem.Visible = true;
                unenrollWithSupportProviderToolStripMenuItem.Visible = true;
                viewManagementProfilesToolStripMenuItem.Visible = true;
                platformLabel.Text = string.Format(Properties.Resources.IntelHomeVersion, ManagementEngine.GetVersion().ToString());
                platformLabel.Enabled = true;
                startRemoteConfigurationToolStripMenuItem.Visible = false;
            }
            else
            {
                return;
            }

            if (ManagementEngine.ClientConnect != null)
            {
                hardwareLabel.Enabled = true;
                if (ManagementEngine.Platform == HeciWrapper.IntelPlatform.IntelConsumer)
                {
                    MspProfile[] profiles = ManagementEngine.ClientConnect.GetProfiles();
                    if (profiles != null && profiles.Length > 0) viewManagementProfilesToolStripMenuItem.Enabled = true;
                }
                UpdateStatus();
                refreshTimer.Enabled = true;
            }

        }

        private void PerformAutomaticOperation(RemoteAccessStatus s)
        {
            if (Properties.Settings.Default.AutoDisconnect == true && s.Connection == RemoteAccessStatus.AMT_NETWORK_CONNECTION.OUTSIDE_ENTERPRISE && s.Status == RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_STATUS.CONNECTED && ConnectionTime != DateTime.MinValue && DisconnectionTime == DateTime.MinValue)
            {
                if (ConnectionTime.AddSeconds(Properties.Settings.Default.AutoDisconnectTimeout) < DateTime.Now)
                {
                    // Auto-closing connection
                    ManagementEngine.ClientConnect.CloseConnection();
                }
            }

            if (Properties.Settings.Default.AutoConnection == true && s.Connection == RemoteAccessStatus.AMT_NETWORK_CONNECTION.OUTSIDE_ENTERPRISE && s.Status == RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_STATUS.NOT_CONNECTED && ConnectionTime == DateTime.MinValue && DisconnectionTime != DateTime.MinValue)
            {
                if (DisconnectionTime.AddSeconds(Properties.Settings.Default.AutoConnectionTimeout) < DateTime.Now)
                {
                    // Auto-openning connection
                    ManagementEngine.ClientConnect.OpenConnection();
                }
            }
        }

        private void UpdateStatus()
        {
            enrollWithSupportProviderToolStripMenuItem.Enabled = false;
            unenrollWithSupportProviderToolStripMenuItem.Enabled = false;

            if (ManagementEngine == null || ManagementEngine.ClientConnect == null)
            {
                hardwareLabel.Text = Properties.Resources.Unsupported;
                if (SoftIcon == ConnectionIcon.Connected)
                {
                    connectButton.Text = Properties.Resources.DisconnectManagementConnection;
                    connectButton.Enabled = true;
                    connectToSupportProviderToolStripMenuItem.Enabled = false;
                    disconnectFromSupportProviderToolStripMenuItem.Enabled = true;
                }
                else if (SoftIcon == ConnectionIcon.Disconnected)
                {
                    connectButton.Text = Properties.Resources.RequestAssistance;
                    connectButton.Enabled = true;
                    connectToSupportProviderToolStripMenuItem.Enabled = true;
                    disconnectFromSupportProviderToolStripMenuItem.Enabled = false;
                }
                else
                {
                    connectButton.Text = Properties.Resources.RequestAssistance;
                    connectButton.Enabled = false;
                    connectToSupportProviderToolStripMenuItem.Enabled = false;
                    disconnectFromSupportProviderToolStripMenuItem.Enabled = false;
                }
                return;
            }

            string mspname = "";
            if (ManagementEngine.Platform == HeciWrapper.IntelPlatform.IntelConsumer)
            {
                EnrollementState es = ManagementEngine.ClientConnect.GetEnrollmentState();
                if (es != null && es.MspName != null && es.MspName.Length > 0) mspname = es.MspName + ", ";
                if (es == null && resetTimer.Enabled == true)
                {
                    hardwareLabel.Text = Properties.Resources.Working;
                    connectButton.Enabled = false;
                    connectButton.Text = Properties.Resources.PleaseWait;
                    connectToSupportProviderToolStripMenuItem.Enabled = false;
                    disconnectFromSupportProviderToolStripMenuItem.Enabled = false;
                    return;
                }
                if (es != null)
                {
                    switch (es.EnrollementStatus)
                    {
                        case EnrollementState.ENROLLMENT_STATE.ENROLLMENT_STATE_ENROLLMENT_PENDING:
                            hardwareLabel.Text = mspname + Properties.Resources.Enrolling;
                            connectButton.Enabled = false;
                            connectButton.Text = Properties.Resources.PleaseWait;
                            connectToSupportProviderToolStripMenuItem.Enabled = false;
                            disconnectFromSupportProviderToolStripMenuItem.Enabled = false;
                            return;
                        case EnrollementState.ENROLLMENT_STATE.ENROLLMENT_STATE_UNENROLLMENT_PENDING:
                            hardwareLabel.Text = Properties.Resources.Unenrolling;
                            connectButton.Enabled = false;
                            connectButton.Text = Properties.Resources.PleaseWait;
                            connectToSupportProviderToolStripMenuItem.Enabled = false;
                            disconnectFromSupportProviderToolStripMenuItem.Enabled = false;
                            return;
                        case EnrollementState.ENROLLMENT_STATE.ENROLLMENT_STATE_UNENROLLED:
                            hardwareLabel.Text = Properties.Resources.NotEnrolled;
                            connectButton.Enabled = true;
                            connectButton.Text = Properties.Resources.EnrollWithSupportProvider;
                            connectToSupportProviderToolStripMenuItem.Enabled = false;
                            disconnectFromSupportProviderToolStripMenuItem.Enabled = false;
                            enrollWithSupportProviderToolStripMenuItem.Enabled = true;
                            return;
                        case EnrollementState.ENROLLMENT_STATE.ENROLLMENT_STATE_ERROR:
                            hardwareLabel.Text = Properties.Resources.EnrollementError;
                            connectButton.Enabled = true;
                            connectButton.Text = Properties.Resources.EnrollWithSupportProvider;
                            connectToSupportProviderToolStripMenuItem.Enabled = false;
                            disconnectFromSupportProviderToolStripMenuItem.Enabled = false;
                            return;
                    }
                }
                unenrollWithSupportProviderToolStripMenuItem.Enabled = true;
            }

            RemoteAccessStatus s = ManagementEngine.ClientConnect.GetConnectionStatus();
            if (s == null)
            {
                hardwareLabel.Text = Properties.Resources.NoStatus;
                connectButton.Enabled = true;
                connectButton.Text = Properties.Resources.RequestAssistance;
                connectToSupportProviderToolStripMenuItem.Enabled = false;
                disconnectFromSupportProviderToolStripMenuItem.Enabled = false;
                ManagementEngine = null;

                // Set the connection state time
                ConnectionTime = DateTime.MinValue;
                DisconnectionTime = DateTime.MinValue;
            }
            else
            {
                switch (s.Status)
                {
                    case RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_STATUS.NOT_CONNECTED:
                        hardwareLabel.Text = mspname + Properties.Resources.NotConnected;
                        connectButton.Enabled = true;
                        connectButton.Text = Properties.Resources.RequestAssistance;
                        connectToSupportProviderToolStripMenuItem.Enabled = true;
                        disconnectFromSupportProviderToolStripMenuItem.Enabled = false;
                        HardIcon = ConnectionIcon.Disconnected;

                        // Set the connection state time
                        ConnectionTime = DateTime.MinValue;
                        if (DisconnectionTime == DateTime.MinValue) DisconnectionTime = DateTime.Now;
                        break;
                    case RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_STATUS.CONNECTING:
                        hardwareLabel.Text = mspname + Properties.Resources.Connecting;
                        connectButton.Enabled = false;
                        connectButton.Text = Properties.Resources.PleaseWait;
                        connectToSupportProviderToolStripMenuItem.Enabled = false;
                        disconnectFromSupportProviderToolStripMenuItem.Enabled = false;
                        HardIcon = ConnectionIcon.Disconnected;

                        // Set the connection state time
                        ConnectionTime = DateTime.MinValue;
                        DisconnectionTime = DateTime.MinValue;
                        break;
                    case RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_STATUS.CONNECTED:
                        if (s.Connection == RemoteAccessStatus.AMT_NETWORK_CONNECTION.DIRECT)
                        {
                            hardwareLabel.Text = Properties.Resources.DirectConnection;
                            connectButton.Enabled = true;
                            connectButton.Text = Properties.Resources.RequestAssistance;
                            connectToSupportProviderToolStripMenuItem.Enabled = true;
                            disconnectFromSupportProviderToolStripMenuItem.Enabled = false;
                            HardIcon = ConnectionIcon.Direct;

                            // Set the connection state time
                            ConnectionTime = DateTime.MinValue;
                            DisconnectionTime = DateTime.MinValue;
                        }
                        else
                        {
                            HardIcon = ConnectionIcon.Connected;
                            connectButton.Enabled = true;
                            if (s.Trigger == RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_TRIGGER.USER_INITIATED || ManagementEngine.Platform == HeciWrapper.IntelPlatform.IntelConsumer)
                            {
                                if (ManagementEngine.Platform == HeciWrapper.IntelPlatform.IntelConsumer) hardwareLabel.Text = mspname + Properties.Resources.Connected; else hardwareLabel.Text = Properties.Resources.UserInitiatedConnection;
                                connectButton.Text = Properties.Resources.DisconnectManagementConnection;
                                connectToSupportProviderToolStripMenuItem.Enabled = false;
                                disconnectFromSupportProviderToolStripMenuItem.Enabled = true;
                                //if (autoDisconnectToolStripMenuItem.Checked) connectButton_Click(this, null);
                            }
                            else
                            {
                                if (s.Trigger == RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_TRIGGER.TRIGGER_ALERT) hardwareLabel.Text = Properties.Resources.AlertInitiatedConnection;
                                if (s.Trigger == RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_TRIGGER.TRIGGER_PERIODIC) hardwareLabel.Text = Properties.Resources.TimerInitiatedConnection;
                                if (s.Trigger == RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_TRIGGER.TRIGGER_PROVISIONING) hardwareLabel.Text = Properties.Resources.ProvisioningConnection;
                                connectButton.Text = Properties.Resources.RequestAssistance;
                                connectToSupportProviderToolStripMenuItem.Enabled = false;
                                disconnectFromSupportProviderToolStripMenuItem.Enabled = false;
                            }

                            // Set the connection state time
                            if (ConnectionTime == DateTime.MinValue) ConnectionTime = DateTime.Now;
                            DisconnectionTime = DateTime.MinValue;
                        }
                        break;
                    case RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_STATUS.ERROR:
                        hardwareLabel.Text = Properties.Resources.Error + mspname;
                        connectButton.Enabled = true;
                        connectButton.Text = Properties.Resources.RequestAssistance;
                        connectToSupportProviderToolStripMenuItem.Enabled = true;
                        disconnectFromSupportProviderToolStripMenuItem.Enabled = false;

                        // Set the connection state time
                        ConnectionTime = DateTime.MinValue;
                        DisconnectionTime = DateTime.MinValue;
                        break;
                }
            }

            ConnectIcon();

            PerformAutomaticOperation(s);
        }

        private void ConnectIcon()
        {
            ConnectionIcon state = HardIcon;
            if (SoftIcon == ConnectionIcon.Connected && HardIcon != ConnectionIcon.Direct) state = ConnectionIcon.Connected;

            if (state == ConnectionIcon.Unknown && pictureBoxUnknown.Visible != true)
            {
                // Unknown State
                pictureBoxUnknown.Visible = true;
                pictureBoxDisconnected.Visible = false;
                pictureBoxConnected.Visible = false;
                pictureBoxLocal.Visible = false;
            }
            else if (state == ConnectionIcon.Disconnected && pictureBoxDisconnected.Visible != true)
            {
                // Disconnected State
                pictureBoxUnknown.Visible = false;
                pictureBoxDisconnected.Visible = true;
                pictureBoxConnected.Visible = false;
                pictureBoxLocal.Visible = false;
            }
            else if (state == ConnectionIcon.Connected && pictureBoxConnected.Visible != true)
            {
                // Connected State
                pictureBoxUnknown.Visible = false;
                pictureBoxDisconnected.Visible = false;
                pictureBoxConnected.Visible = true;
                pictureBoxLocal.Visible = false;
            }
            else if (state == ConnectionIcon.Direct && pictureBoxLocal.Visible != true)
            {
                // Direct Connection
                pictureBoxUnknown.Visible = false;
                pictureBoxDisconnected.Visible = false;
                pictureBoxConnected.Visible = false;
                pictureBoxLocal.Visible = true;
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Properties.Settings.Default.Save();
            if (ManagementEngine != null) ManagementEngine.Dispose();
        }

        private void refreshTimer_Tick(object sender, EventArgs e)
        {
            refreshTimer.Enabled = false;
            UpdateStatus();
            refreshTimer.Enabled = true; // if an exception occurs, this will stop the timer.
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            connectButton.Enabled = false;
            if (ManagementEngine == null) return;
            if (ManagementEngine.Platform == HeciWrapper.IntelPlatform.IntelConsumer)
            {
                EnrollementState es = ManagementEngine.ClientConnect.GetEnrollmentState();
                if (es == null) return;
                switch (es.EnrollementStatus)
                {
                    case EnrollementState.ENROLLMENT_STATE.ENROLLMENT_STATE_ENROLLMENT_PENDING:
                        return;
                    case EnrollementState.ENROLLMENT_STATE.ENROLLMENT_STATE_UNENROLLMENT_PENDING:
                        return;
                    case EnrollementState.ENROLLMENT_STATE.ENROLLMENT_STATE_UNENROLLED:
                        enrollWithSupportProviderToolStripMenuItem_Click(this, null);
                        return;
                    case EnrollementState.ENROLLMENT_STATE.ENROLLMENT_STATE_ERROR:
                        return;
                }
            }

            if (ManagementEngine.ClientConnect != null)
            {
                RemoteAccessStatus s = ManagementEngine.ClientConnect.GetConnectionStatus();
                switch (s.Status)
                {
                    case RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_STATUS.NOT_CONNECTED:
                        ManagementEngine.ClientConnect.OpenConnection();                        
                        break;
                    case RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_STATUS.CONNECTING:
                        break;
                    case RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_STATUS.CONNECTED:
                        if (s.Connection == RemoteAccessStatus.AMT_NETWORK_CONNECTION.DIRECT)
                        {
                            // Request help console event
                            ManagementEngine.ClientConnect.OpenConnection();
                        }
                        else if (s.Trigger == RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_TRIGGER.USER_INITIATED || ManagementEngine.Platform == HeciWrapper.IntelPlatform.IntelConsumer)
                        {
                            // Close a user initiated CIRA connection
                            ManagementEngine.ClientConnect.CloseConnection();                            
                        }
                        else
                        {
                            // Re-connect at the user's request
                            ManagementEngine.ClientConnect.OpenConnection();                            
                        }
                        break;
                    case RemoteAccessStatus.REMOTE_ACCESS_CONNECTION_STATUS.ERROR:
                        ManagementEngine.ClientConnect.OpenConnection();
                        break;
                }
            }

            UpdateStatus();
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void viewManagementProfilesToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ManagementEngine == null) return;
            MspProfile[] profiles = ManagementEngine.ClientConnect.GetProfiles();
            if (profiles.Length > 0) new ManagementProfilesForm(ManagementEngine, profiles).ShowDialog(this);
        }

        private void enrollWithSupportProviderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            MspProfile[] profiles = ManagementEngine.ClientConnect.GetProfiles();
            if (profiles != null && profiles.Length > 0)
            {
                EnrollementForm f = new EnrollementForm(profiles);
                if (f.ShowDialog(this) == DialogResult.OK)
                {
                    Application.DoEvents();

                    // Setup Hard-CIRA
                    connectButton.Enabled = false;
                    ManagementEngine.ClientConnect.SetActiveProfile(f.GetSelectedProfile(), f.GetUsername(), f.GetPassword());

                    Application.DoEvents();

                    Application.DoEvents();

                    UpdateStatus();
                }
            }
        }

        private void unenrollWithSupportProviderToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ManagementEngine.ClientConnect == null) return;
            connectButton.Enabled = false;
            resetToolStripProgressBar.Value = 0;
            resetTimer.Enabled = true;
            hardwareLabel.Text = Properties.Resources.Unenrolling;


            Properties.Settings.Default.ManagementEnabled = false;

            // Clear hard-CIRA configuration
            System.Threading.Thread t = new System.Threading.Thread(new System.Threading.ThreadStart(Unenroll));
            t.Start();
        }

        private void Unenroll()
        {
            ManagementEngine.ClientConnect.Unenroll();
            ManagementEngine.Dispose();
            ManagementEngine = null;
        }

        private void hardwareLabel_Click(object sender, EventArgs e)
        {
            UpdateStatus();
        }

        private void resetTimer_Tick(object sender, EventArgs e)
        {
            resetToolStripProgressBar.Visible = true;
            resetToolStripProgressBar.Value += 1;
            if (resetToolStripProgressBar.Value == resetToolStripProgressBar.Maximum)
            {
                resetTimer.Enabled = false;
                resetToolStripProgressBar.Visible = false;
            }
        }

        private void managementEngineStatusToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ManagementEngine != null) new MeInfoForm(ManagementEngine).ShowDialog(this);
        }

        private void ManageabilityDTKLatestNewsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AboutForm.GotoNews();
        }

        private void ManageabilityDTKDownloadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AboutForm.GotoDownloads();
        }

        private void ManageabilityDTKSupportForumsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AboutForm.GotoForums();
        }

        private void aboutManageabilityConnectorToolToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new AboutForm(this.Text, Properties.Resources.ProductSubName).ShowDialog(this);
        }

        void SystemEvents_PowerModeChanged(object sender, PowerModeChangedEventArgs e)
        {
            switch (e.Mode)
            {
                case PowerModes.Resume:
                    UpdateEngine();
                    break;
                case PowerModes.Suspend:
                    refreshTimer.Enabled = false;
                    break;
            }
        }

        private void showManageabilityStackDebugInfoToolStripMenuItem_Click(object sender, EventArgs e)
        {
            AmtDebugForm.ShowForm();
        }

        private void startRemoteConfigurationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ManagementEngine == null || ManagementEngine.MeInfo == null) return;
            bool r = ManagementEngine.MeInfo.StartConfiguration();
            if (r == true)
            {
                MessageBox.Show(this, Properties.Resources.RemoteConfigurationRequest, Properties.Resources.RemoteConfiguration, MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            else
            {
                MessageBox.Show(this, Properties.Resources.RemoteConfigurationFailed, Properties.Resources.RemoteConfiguration, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void resetToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateEngine();
        }

        private void tCPtoHECIBridgeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (HeciBridge != null)
            {
                HeciBridge.Stop();
                HeciBridge = null;
                tCPtoHECIBridgeToolStripMenuItem.Checked = false;
                heciBridgeToolStripStatusLabel.Visible = false;
                heciBridgeToolStripStatusLabel.Text = HeciBridgeStatusStr;
            }
            else
            {
                HeciBridgeForm form = new HeciBridgeForm();
                if (form.ShowDialog() == DialogResult.OK)
                {
                    HeciBridge = new HeciBridgeClass(ManagementEngine, form.Port);
                    bool ok = HeciBridge.Start();
                    if (ok == false)
                    {
                        HeciBridge = null;
                        MessageBox.Show(this, "Error binding to local TCP port.", "TCP-to-HECI bridge", MessageBoxButtons.OK, MessageBoxIcon.Asterisk);
                    }
                    else
                    {
                        HeciBridge.HeciMessageEvent += new HeciBridgeClass.HeciMessageHandler(HeciMessageSink);
                    }
                    tCPtoHECIBridgeToolStripMenuItem.Checked = heciBridgeToolStripStatusLabel.Visible = ok;
                    heciBridgeToolStripStatusLabel.Text = HeciBridgeStatusStr;
                    HeciBridgeCount = 0;
                }
            }
        }

        private void HeciMessageSink(HeciBridgeClass bridge, System.Net.Sockets.TcpClient client, byte[] buf, int buflen)
        {
            this.Invoke(new HeciBridgeClass.HeciMessageHandler(HeciMessageSinkEx), new object[] { bridge, client, buf, buflen });
        }

        private void HeciMessageSinkEx(HeciBridgeClass bridge, System.Net.Sockets.TcpClient client, byte[] buf, int buflen)
        {
            HeciBridgeCount++;
            heciBridgeToolStripStatusLabel.Text = HeciBridgeStatusStr + " (" + HeciBridgeCount.ToString() + ")";
        }

        private void connectToHECIBridgeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (RemoteBridgeHostname == null)
            {
                HeciBridgeConnectionForm form = new HeciBridgeConnectionForm();
                if (form.ShowDialog(this) == DialogResult.OK)
                {
                    RemoteBridgeHostname = form.Hostname;
                    RemoteBridgePort = form.Port;
                    connectToHECIBridgeToolStripMenuItem.Checked = true;
                    UpdateEngine();
                    UpdateUI();
                }
            }
            else
            {
                RemoteBridgeHostname = null;
                connectToHECIBridgeToolStripMenuItem.Checked = false;
                UpdateEngine();
                UpdateUI();
            }
        }

        private void automaticTestingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            new AutoTestingForm().ShowDialog(this);
        }

        private void testingFeaturesToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            automaticTestingToolStripMenuItem.Checked = (Properties.Settings.Default.AutoConnection || Properties.Settings.Default.AutoDisconnect);
        }

    }
}
