// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Net;
using System.Text;
using System.Windows.Forms;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class AddMpsForm : Form
    {
        #region Private Fields

        private WaitForm _ResolveForm;
        private delegate void AsyncCallback(IAsyncResult ar);
        private delegate void ReenableButtonsCallback();
        private IAsyncResult _DnsLookupAsyncResult = null;
        private IPAddress _IPAddress = null;

        #endregion

        #region Constructors

        public AddMpsForm()
        {
            InitializeComponent();
        }

        #endregion

        #region Public Properties

        public string Hostname
        {
            get { return hostnameTextBox.Text.Trim(); }
        }

        public ushort HttpPort
        {
            get { return (ushort)httpNumericUpDown.Value; }
        }

        public ushort SocksPort
        {
            get { return (ushort)socksNumericUpDown.Value; }
        }

        public IPAddress IPAddress
        {
            get { return _IPAddress; }
            set { _IPAddress = value; }
        }

        public ForceIPOption ForceIPOption
        {
            get
            {
                if (useDefaultRadioButton.Checked == true)
                    return ForceIPOption.UseDefault;
                else if (forceIPv4RadioButton.Checked == true)
                    return ForceIPOption.UseIPv4;
                else if (forceIPv6RadioButton.Checked == true)
                    return ForceIPOption.UseIPv6;
                System.Diagnostics.Debug.Assert(false, "Radio button state is invalid!");
                throw new ApplicationException();
            }
            set
            {
                if (value == ForceIPOption.UseDefault)
                    useDefaultRadioButton.Checked = true;
                else if (value == ForceIPOption.UseIPv4)
                    forceIPv4RadioButton.Checked = true;
                else if (value == ForceIPOption.UseIPv6)
                    forceIPv6RadioButton.Checked = true;
            }
        }

        #endregion

        private void AddMpsForm_Load(object sender, EventArgs e)
        {
            hostnameTextBox.Text = Properties.Settings.Default.MpsHost;
            httpNumericUpDown.Value = (decimal)Properties.Settings.Default.MpsHttpPort;
            socksNumericUpDown.Value = (decimal)Properties.Settings.Default.MpsSocksPort;
            UpdateInfo();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.MpsHost = Hostname;
            Properties.Settings.Default.MpsHttpPort = HttpPort;
            Properties.Settings.Default.MpsSocksPort = SocksPort;
            Properties.Settings.Default.Save();
            DialogResult = DialogResult.OK;

            okButton.Enabled = false;
            cancelButton.Enabled = false;
            hostnameTextBox.Enabled = false;
            httpNumericUpDown.Enabled = false;
            socksNumericUpDown.Enabled = false;

            IPAddress tmpAddress;
            bool FixedIPAddress = IPAddress.TryParse(Hostname, out tmpAddress);

            // CASE 1:  Use entered fixed IP Address
            if (FixedIPAddress == true)
            {
                IPAddress = tmpAddress;
                DialogResult = DialogResult.OK;
            }
            else // CASE 2
            {
                // If the user entered a hostname/FQDN and chose to force IPv4 or IPv6, we need to at least ensure
                // that we can resolve the name to an IPAddress of that type before we continue.
                if ((forceIPv4RadioButton.Checked == true) || (forceIPv6RadioButton.Checked == true))
                {
                    // Show "resolving DNS" window and start a new DNS lookup
                    _ResolveForm = new WaitForm(this, "Resolving DNS...");
                    _ResolveForm.Owner = this;
                    _ResolveForm.Show(this);

                    _DnsLookupAsyncResult = Dns.BeginGetHostEntry(Hostname,
                        GetHostAddressesCallback, null);
                }
                else // CASE 3: Hostname/FQDN but do not force IP Address
                {
                    DialogResult = DialogResult.OK;
                }
            }
        }

        private void cancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
        }

        private void UpdateInfo()
        {
            hostnameTextBox.Text = hostnameTextBox.Text.Trim();

            // Does the entry look like an ipaddress?
            IPAddress tmpAddress;
            bool LooksLikeIPAddress = IPAddress.TryParse(Hostname, out tmpAddress);
            forceIPv4RadioButton.Enabled = !LooksLikeIPAddress;
            forceIPv6RadioButton.Enabled = !LooksLikeIPAddress;
            useDefaultRadioButton.Enabled = !LooksLikeIPAddress;

            if (LooksLikeIPAddress == true)
            {
                if (tmpAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
                    forceIPv4RadioButton.Checked = true;
                else if (tmpAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6)
                    forceIPv6RadioButton.Checked = true;
            }
            else
            {
                useDefaultRadioButton.Checked = true;
            }

            okButton.Enabled = Hostname.Length > 0;
        }

        private void hostnameTextBox_TextChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        /// <summary>
        /// Delegate method for re-enabling grayed buttons if the user needs to correct some of their inputs
        /// </summary>
        private void ReenableButtons()
        {
            if (InvokeRequired == true)
            {
                ReenableButtonsCallback reenable = new ReenableButtonsCallback(ReenableButtons);
                Invoke(reenable);
                return;
            }

            okButton.Enabled = true;
            cancelButton.Enabled = true;
            hostnameTextBox.Enabled = true;
            httpNumericUpDown.Enabled = true;
            socksNumericUpDown.Enabled = true;
        }

        /// <summary>
        /// Callback that gets executed once the entered hostname is resolved (after the user clicks OK). This
        /// method parses the DnsHostEntry to determine if the correct IP Addresses were returned corresponding
        /// to the selected 'force' option.
        /// </summary>
        /// <param name="ar">IAsyncResult object from the BeginGetHostEntry(...) method call</param>
        private void GetHostAddressesCallback(IAsyncResult ar)
        {
            if (ar.IsCompleted == true)
            {
                _ResolveForm.Hide();
                int NumMatches = 0;
                try
                {
                    // Need to ensure that the right IP Address was found corresponding to the 'force' radio button
                    // option that the user selected.
                    IPHostEntry DnsEntry = Dns.EndGetHostEntry(ar);
                    foreach (IPAddress ipAddress in DnsEntry.AddressList)
                    {
                        if ((ipAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork) &&
                           (forceIPv4RadioButton.Checked == true))
                        {
                            IPAddress = ipAddress;
                            NumMatches++;
                            continue;
                        }
                        if ((ipAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6) &&
                           (forceIPv6RadioButton.Checked == true))
                        {
                            IPAddress = ipAddress;
                            NumMatches++;
                            continue;
                        }
                    }
                }
                catch (Exception)
                {
                }

                // If no IP address was found to match the selected "force" option, then display an error message
                // and re-display the form for the user to fix.
                if (NumMatches == 0)
                {
                    MessageBox.Show("The hostname could not be resolved to an IPv" + (forceIPv4RadioButton.Checked ? "4" : "6") +
                        " address.\nEither fix the hostname or choose a different 'force IP' option.",
                        "Dns Error", MessageBoxButtons.OK);

                    ReenableButtons();
                    return;
                }

                // TODO: Need to handle the case where the user needs to pick between 2+ matching IP Address types
                //       E.G. Dual NIC system with 2 IPv4 IP addresses
                if (NumMatches > 1)
                {
                    MessageBox.Show("Multiple matching IP Addresses found. Using the 1st one only.",
                        "Warning",
                        MessageBoxButtons.OK);
                }

                DialogResult = DialogResult.OK;
            }
        }
    }
}
