// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Text;
using System.Data;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class AddSystemForm : Form
    {
        #region Private Fields
        private WaitForm _ResolveForm;
        private delegate void AsyncCallback(IAsyncResult ar);
        private delegate void ReenableButtonsCallback();
        private IAsyncResult _DnsLookupAsyncResult = null;
        private IPAddress _IPAddress = null;
        #endregion

        #region Constructors
        public AddSystemForm(bool askTls)
        {
            InitializeComponent();
            hostnameTextBox.Text = Properties.Settings.Default.AddSystemDefaultHost;
            usernameTextBox.Text = Properties.Settings.Default.AddSystemDefaultUser;
            passwordControl.Text = Properties.Settings.Default.AddSystemDefaultPass;
            rememberPasswordCheckBox.Checked = Properties.Settings.Default.AddSystemRememberPassword;
            tlsInfoLabel.Visible = true;
            tlsCheckBox.Visible = false;
            rememberPasswordCheckBox.Visible = !askTls;
        }

        public AddSystemForm(string hostname)
        {
            InitializeComponent();
            hostnameTextBox.Text = hostname;
            usernameTextBox.Text = Properties.Settings.Default.AddSystemDefaultUser;
            passwordControl.Text = Properties.Settings.Default.AddSystemDefaultPass;
            rememberPasswordCheckBox.Checked = Properties.Settings.Default.AddSystemRememberPassword;
            tlsInfoLabel.Visible = true;
            tlsCheckBox.Visible = false;
        }

        public AddSystemForm(bool askTls, bool viaMps)
        {
            InitializeComponent();
            hostnameTextBox.Text = Properties.Settings.Default.AddSystemDefaultHost;
            usernameTextBox.Text = Properties.Settings.Default.AddSystemDefaultUser;
            passwordControl.Text = Properties.Settings.Default.AddSystemDefaultPass;
            rememberPasswordCheckBox.Checked = Properties.Settings.Default.AddSystemRememberPassword;
            tlsInfoLabel.Visible = true;
            tlsCheckBox.Visible = false;
            rememberPasswordCheckBox.Visible = !askTls;
            forceIPv4RadioButton.Enabled = forceIPv4RadioButton.Visible = false;
            forceIPv6RadioButton.Enabled = forceIPv6RadioButton.Visible = false;
            useDefaultRadioButton.Enabled = useDefaultRadioButton.Visible = false;
        }
        #endregion

        #region Public Properties
        /// <summary>
        /// The hostname of the Intel AMT system to add
        /// </summary>
        public string Hostname
        {
            get { return hostnameTextBox.Text; }
            set { hostnameTextBox.Text = value; }
        }
        
        /// <summary>
        /// The username for connection to the Intel AMT system
        /// </summary>
        public string Username
        {
            get { return usernameTextBox.Text; }
            set { usernameTextBox.Text = value; }
        }

        /// <summary>
        /// The password for the connection to the Intel AMT system
        /// </summary>
        public string Password
        {
            get { return passwordControl.Text; }
            set { passwordControl.Text = value; }
        }

        /// <summary>
        /// Flag indicating whether the user password should be remembered between sessions
        /// </summary>
        public bool RememberPassword
        {
            get { return rememberPasswordCheckBox.Checked; }
            set { rememberPasswordCheckBox.Checked = value; }
        }

        /// <summary>
        /// Flag indicating whether to use TLS for connection to the Intel AMT system
        /// </summary>
        public bool UseTls
        {
            get { return tlsCheckBox.Checked; }
            set { tlsCheckBox.Checked = value; }
        }

        public IPAddress IPAddress
        {
            get { return _IPAddress; }
            set { _IPAddress = value; }
        }

        public ForceIPOption ForceIPOption
        {
            get
            {
                if (useDefaultRadioButton.Checked == true)
                    return ForceIPOption.UseDefault;
                else if (forceIPv4RadioButton.Checked == true)
                    return ForceIPOption.UseIPv4;
                else if (forceIPv6RadioButton.Checked == true)
                    return ForceIPOption.UseIPv6;
                System.Diagnostics.Debug.Assert(false, "Radio button state is invalid!");
                throw new ApplicationException();
            }
            set
            {
                if (value == ForceIPOption.UseDefault)
                    useDefaultRadioButton.Checked = true;
                else if (value == ForceIPOption.UseIPv4)
                    forceIPv4RadioButton.Checked = true;
                else if (value == ForceIPOption.UseIPv6)
                    forceIPv6RadioButton.Checked = true;
            }
        }
        #endregion

        #region Event Handlers
        /// <summary>
        /// Initializes the form with an icon
        /// </summary>
        /// <param name="sender">N/A</param>
        /// <param name="e">N/A</param>
        private void AddSystemForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null)
                Icon = Owner.Icon;
        }

        /// <summary>
        /// Processes the hostname and attempts to resolve IP addresses when applicable once the user clicks OK
        /// </summary>
        /// <param name="sender">N/A</param>
        /// <param name="e">N/A</param>
        private void okButton_Click(object sender, EventArgs e)
        {
            Properties.Settings.Default.AddSystemDefaultHost = hostnameTextBox.Text;
            Properties.Settings.Default.AddSystemDefaultUser = usernameTextBox.Text;
            Properties.Settings.Default.AddSystemDefaultPass = passwordControl.Text;
            Properties.Settings.Default.AddSystemRememberPassword = rememberPasswordCheckBox.Checked;
            Properties.Settings.Default.Save();

            IPAddress tmpAddress;
            bool FixedIPAddress = IPAddress.TryParse(hostnameTextBox.Text, out tmpAddress);

            // CASE 1:  Use entered fixed IP Address
            if (FixedIPAddress == true)
            {
                IPAddress = tmpAddress;
                DialogResult = DialogResult.OK;
            }
            else // CASE 2
            // If the user entered a hostname/FQDN and chose to force IPv4 or IPv6, we need to at least ensure
            // that we can resolve the name to an IPAddress of that type before we continue.
            if ((forceIPv4RadioButton.Checked == true) || (forceIPv6RadioButton.Checked == true))
            {
                //disable buttons since _DnsLookupAsyncResult can take a while
                okButton.Enabled = false;
                cancelButton.Enabled = false;
                hostnameTextBox.Enabled = false;
                usernameTextBox.Enabled = false;
                passwordControl.Enabled = false;
                rememberPasswordCheckBox.Enabled = false;

                // Show "resolving DNS" window and start a new DNS lookup
                _ResolveForm = new WaitForm(this, "Resolving DNS...");
                _ResolveForm.Owner = this;
                _ResolveForm.Show(this);

                _DnsLookupAsyncResult = Dns.BeginGetHostEntry(
                    hostnameTextBox.Text,
                    GetHostAddressesCallback,
                    null);
            }
            else // CASE 3: Hostname/FQDN but do not force IP Address
            {
                DialogResult = DialogResult.OK;
            }
        }

        /// <summary>
        /// Dynamically updates control grayed-ness depending on the validness of the typed hostname as an IP Address
        /// </summary>
        /// <param name="sender">N/A</param>
        /// <param name="e">N/A</param>
        private void hostnameTextBox_TextChanged(object sender, EventArgs e)
        {
            // Does the entry look like an ipaddress?
            IPAddress tmpAddress;
            bool LooksLikeIPAddress = IPAddress.TryParse(hostnameTextBox.Text, out tmpAddress);
            forceIPv4RadioButton.Enabled = !LooksLikeIPAddress;
            forceIPv6RadioButton.Enabled = !LooksLikeIPAddress;
            useDefaultRadioButton.Enabled = !LooksLikeIPAddress;

            if (LooksLikeIPAddress == true)
            {
                if (tmpAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
                    forceIPv4RadioButton.Checked = true;
                else if (tmpAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6)
                    forceIPv6RadioButton.Checked = true;
            }
            else
            {
                useDefaultRadioButton.Checked = true;
            }
        }

        /// <summary>
        /// Delegate method for re-enabling grayed buttons if the user needs to correct some of their inputs
        /// </summary>
        private void ReenableButtons()
        {
            if (InvokeRequired == true)
            {
                ReenableButtonsCallback reenable = new ReenableButtonsCallback(ReenableButtons);
                Invoke(reenable);
                return;
            }

            okButton.Enabled = true;
            cancelButton.Enabled = true;
            hostnameTextBox.Enabled = true;
            usernameTextBox.Enabled = true;
            passwordControl.Enabled = true;
            rememberPasswordCheckBox.Enabled = true;
        }

        /// <summary>
        /// Callback that gets executed once the entered hostname is resolved (after the user clicks OK). This
        /// method parses the DnsHostEntry to determine if the correct IP Addresses were returned corresponding
        /// to the selected 'force' option.
        /// </summary>
        /// <param name="ar">IAsyncResult object from the BeginGetHostEntry(...) method call</param>
        private void GetHostAddressesCallback(IAsyncResult ar)
        {
            if (ar.IsCompleted == true)
            {
                _ResolveForm.Hide();
                int NumMatches = 0;
                try
                {
                    // Need to ensure that the right IP Address was found corresponding to the 'force' radio button
                    // option that the user selected.
                    IPHostEntry DnsEntry = Dns.EndGetHostEntry(ar);
                    foreach (IPAddress ipAddress in DnsEntry.AddressList)
                    {
                        if ((ipAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork) &&
                           (forceIPv4RadioButton.Checked == true))
                        {
                            IPAddress = ipAddress;
                            NumMatches++;
                            continue;
                        }
                        if ((ipAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6) &&
                           (forceIPv6RadioButton.Checked == true))
                        {
                            IPAddress = ipAddress;
                            NumMatches++;
                            continue;
                        }
                    }
                }
                catch (Exception)
                {
                }

                // If no IP address was found to match the selected "force" option, then display an error message
                // and re-display the form for the user to fix.
                if (NumMatches == 0)
                {
                    MessageBox.Show("The hostname could not be resolved to an IPv" + (forceIPv4RadioButton.Checked ? "4" : "6") +
                        " address.\nEither fix the hostname or choose a different 'force IP' option.",
                        "Dns Error", MessageBoxButtons.OK);

                    ReenableButtons();
                    return;
                }

                // TODO: Need to handle the case where the user needs to pick between 2+ matching IP Address types
                //       E.G. Dual NIC system with 2 IPv4 IP addresses
                if (NumMatches > 1)
                {
                    MessageBox.Show("Multiple matching IP Addresses found. Using the 1st one only.", 
                        "Warning",
                        MessageBoxButtons.OK);
                }

                DialogResult = DialogResult.OK;
            }
        }

        private void flushDnsCacheButton_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process proc;
            string path = Environment.GetFolderPath(Environment.SpecialFolder.System) + "\\ipconfig.exe";
            proc = System.Diagnostics.Process.Start(path, "/flushdns");
            if (proc.WaitForExit(2000) == false)
            {
                MessageBox.Show("Flush DNS command did not complete within 1 second", "Error");
                proc.Kill();
            }
            
            System.Diagnostics.Debug.Print("'IPConfig /flushdns' returned exit code " + proc.ExitCode);
        }
        #endregion
    }
}
