﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class EditKVMForm : Form
    {
        private AmtSystem system;

        public EditKVMForm(AmtSystem System)
        {
            this.system = System;
            InitializeComponent();
            UpdateFields();
        }

        enum KvmStateComboBoxItems
        {
            DISABLED =0,
            REDIR_ONLY =1,
            STANDARD_ONLY =2,
            BOTH =3
        }

        private void UpdateFields()
        {
            //initilize the form 

            //first set the KVMState in the combo box
            AmtKVM.KVMStates KVMState = system.KVM.KVMState.Value;
            if (KVMState == AmtKVM.KVMStates.DISABLED_MEBX)
            {
                //if KVM is disabled in MEBx set to 'disabled' state            
                KvmStateComboBox.SelectedItem = KvmStateComboBox.Items[(int)KvmStateComboBoxItems.DISABLED];

                //also display info warning message               
                KVMMEBxStateLabel.ForeColor = Color.Red;
                KVMMEBxStateLabel.Text = "KVM is disabled in MEBx";
                KVMMEBxStateLabel.Visible = true; 

                // disable combo box
                KvmStateComboBox.Enabled = false;  
            }
            else
            {
                //KVM state can be changed so enable the combo box
                KvmStateComboBox.Enabled = true;
                KVMMEBxStateLabel.Visible = false; 

                //set combo box to display the current state
                if (KVMState == AmtKVM.KVMStates.DISABLED)
                    KvmStateComboBox.SelectedItem = KvmStateComboBox.Items[(int)KvmStateComboBoxItems.DISABLED];
                else if (KVMState == AmtKVM.KVMStates.ENABLED_REDIR_PORT_ONLY)
                    KvmStateComboBox.SelectedItem = KvmStateComboBox.Items[(int)KvmStateComboBoxItems.REDIR_ONLY];
                else if (KVMState == AmtKVM.KVMStates.ENABLED_STANDARD_PORT_ONLY)
                    KvmStateComboBox.SelectedItem = KvmStateComboBox.Items[(int)KvmStateComboBoxItems.STANDARD_ONLY];
                else if (KVMState == AmtKVM.KVMStates.ENABLED_BOTH_PORTS)
                    KvmStateComboBox.SelectedItem = KvmStateComboBox.Items[(int)KvmStateComboBoxItems.BOTH];
                else
                    KvmStateComboBox.SelectedItem = KvmStateComboBox.Items[(int)KvmStateComboBoxItems.DISABLED];
            }


            //if opt-in (user consent) is currently enabled check the checkbox
            if (system.KVM.KVMOptInRequired.Value)
            {
                OptInCheckBox.Checked = true;

                //set the opt-in (user consent) sprite timeout value.
                //(valid values are 10-4095)
                ushort OptInTimeout = system.KVM.KVMOptInTimeout.Value;
                numericUpDownOptInTimeout.Value = (decimal)OptInTimeout;
                numericUpDownOptInTimeout.Enabled = true;
                OptInTimeoutLabel.Enabled = true;
            }
            else
            {
                OptInCheckBox.Checked = false;
                numericUpDownOptInTimeout.Enabled = false;
                OptInTimeoutLabel.Enabled = false;
            }

            //if system is in client control mode can't change opt in setting
            if (system.UserConsent.CurrentControlMode.Value == AmtUserConsent.ControlModes.CLIENT_CONTROL_MODE)
            {
                OptInCheckBox.Enabled = false;
                UserConsentLabel.Enabled = false; 
            }
            else
            {
                OptInCheckBox.Enabled = true;
                UserConsentLabel.Enabled = true; 
            }

 

            //set the KMV monitor
            //monitor0 = primary screen, monitor1 = secondary screen
            ushort Monitor = system.KVM.KVMDefaultScreen.Value; 
            if (Monitor == 0)
                Monitor0RadiotButton.Checked = true;
            else
                Monitor1RadioButton.Checked = true;

            //set the TCPSessionTimout
            //(valid values are 0-255)
            ushort TCPSessionTimeout = system.KVM.KVMSessionTimeout.Value; 
            numericUpDownTCPSessionTimeout.Value = TCPSessionTimeout;

            //set the RFB password (if one is saved with this system info)
            if(passwordControl.Text == string.Empty)
                passwordControl.Text = system.KVMRfbPassword;
        }


        /// <summary>
        /// function to set the KVM State (called when OK button is clicked)
        /// </summary>
        /// <returns>true if able to set to desired state</returns>
        private bool setKVMState()
        {
            bool status = true;

            AmtKVM.KVMStates DesiredState = AmtKVM.KVMStates.ERROR_UNKNOWN_STATE; 
                      
            if (KvmStateComboBox.SelectedItem == KvmStateComboBox.Items[(int)KvmStateComboBoxItems.DISABLED])
            {
                //User select to Disable KVM
                system.KVM.EnableKVMPort(AmtKVM.EnabledStateValues.DISABLED);

                DesiredState = AmtKVM.KVMStates.DISABLED; 
            }
            else
            {
                //User selected one of the enable KVM states

                //Enable KVM first
                system.KVM.EnableKVMPort(AmtKVM.EnabledStateValues.ENABLED);

                //Then enable the port based on user selection
                //  Redirection Port selected
                if (KvmStateComboBox.SelectedItem == KvmStateComboBox.Items[(int)KvmStateComboBoxItems.REDIR_ONLY])
                {
                    system.KVM.KVMRedirectionPortEnabled = true;
                    system.KVM.KVMPort5900Enabled = false; 
                    DesiredState = AmtKVM.KVMStates.ENABLED_REDIR_PORT_ONLY; 
                }
                // Standard Port selected
                else if (KvmStateComboBox.SelectedItem == KvmStateComboBox.Items[(int)KvmStateComboBoxItems.STANDARD_ONLY])
                {
                    system.KVM.KVMRedirectionPortEnabled = false;
                    system.KVM.KVMPort5900Enabled = true; 
                    DesiredState = AmtKVM.KVMStates.ENABLED_STANDARD_PORT_ONLY; 
                }
                // Both Ports selected
                else if (KvmStateComboBox.SelectedItem == KvmStateComboBox.Items[(int)KvmStateComboBoxItems.BOTH])
                {
                    system.KVM.KVMRedirectionPortEnabled = true;
                    system.KVM.KVMPort5900Enabled = true;
                    DesiredState = AmtKVM.KVMStates.ENABLED_BOTH_PORTS; 
                }
            }

            if (system.KVM.KVMState != DesiredState)
            {
                string errorMsg = string.Format(ManageabilityControls.Properties.Resources.EditKvmForm_ErrorStateChange,
                    KvmStateComboBox.SelectedItem.ToString(), Environment.NewLine, Environment.NewLine); 

                MessageBox.Show(this,
                    errorMsg,
                    ManageabilityControls.Properties.Resources.EditKvmForm,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

            }

            return status; 
        }

        /// <summary>
        /// function to set the opt in state.  called when 'ok' button is clicked.
        /// Note: logic for setting opt in changes slightly with AMT 6.1, may
        /// get errors trying to change the state for AMT 6.1 system.
        /// </summary>
        /// <returns></returns>
        private bool setOptIn()
        {

            try
            {
                if (OptInCheckBox.CheckState == CheckState.Checked)
                {
                    //call code to set opt in
                    system.KVM.KVMOptInRequired = true;
                }
                else
                {
                    //call code to disable opt in
                    system.KVM.KVMOptInRequired = false;
                }

                return true; 
            }
            catch
            {
                MessageBox.Show(this,
                        ManageabilityControls.Properties.Resources.EditKvmForm_ErrorOptIn,
                        ManageabilityControls.Properties.Resources.EditKvmForm,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                return false; 
            }
        }

        /// <summary>
        /// set opt in timeout value.  this function is called when 'ok' is clicked
        /// </summary>
        /// <returns></returns>
        private bool setOptInTimeOut()
        {
            try
            {
                system.KVM.KVMOptInTimeout = (ushort) numericUpDownOptInTimeout.Value; 
                return true;
            }
            catch
            {
                MessageBox.Show(this,
                    ManageabilityControls.Properties.Resources.EditKvmForm_ErrorTimeout,
                    ManageabilityControls.Properties.Resources.EditKvmForm,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                return false;
            }
        }

        /// <summary>
        /// set the default monitor.  this function is called when 'ok' is clicked
        /// </summary>
        /// <returns></returns>
        private bool setDefaultMonitor()
        {
            try
            {
                if (Monitor0RadiotButton.Checked)
                    system.KVM.KVMDefaultScreen = 0;
                else if (Monitor1RadioButton.Checked)
                    system.KVM.KVMDefaultScreen = 1; 

                return true; 

            }
            catch
            {
                MessageBox.Show(this,
                    ManageabilityControls.Properties.Resources.EditKvmForm_ErrorMonitor,
                    ManageabilityControls.Properties.Resources.EditKvmForm,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                return false;
            }
        }

        /// <summary>
        /// set the TCP session timeout value.  this function is called when 'ok' is clicked
        /// </summary>
        /// <returns></returns>
        private bool setTCPSessionTimeOut()
        {
            try
            {
                system.KVM.KVMSessionTimeout = (ushort)numericUpDownTCPSessionTimeout.Value; 
                return true;
            }

            catch
            {
                MessageBox.Show(this,
                    ManageabilityControls.Properties.Resources.EditKvmForm_ErrorTimeout,
                    ManageabilityControls.Properties.Resources.EditKvmForm,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                return false; 
            }
        }


        private bool setRFBPassword()
        {
            if (passwordControl.Text.Length > 0)
            {

                if (system.KVM.SetKVMRfbPassword(passwordControl.Text))
                {
                    if (rememberPasswordCheckBox.Checked)
                        system.KVMRfbPassword = passwordControl.Text;

                    return true;
                }
                else
                {
                    string error = String.Format(ManageabilityControls.Properties.Resources.EditKvmForm_ErrorRFBPassword,
                        Environment.NewLine, Environment.NewLine);

                    //error message on password setting
                    MessageBox.Show(this,
                        error,
                        ManageabilityControls.Properties.Resources.EditKvmForm,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                    return false;
                }
            }
            else
            {
                return true;
            }
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            bool doneSatus = true;
            bool status = true;

            this.Cursor = Cursors.WaitCursor; 

            status = setRFBPassword();
            if (doneSatus)
                doneSatus = status; 
                        
            status = setKVMState();
            if (doneSatus)
                doneSatus = status; 

            status = setOptIn();
            if (doneSatus)
                doneSatus = status; 

            status = setOptInTimeOut();
            if (doneSatus)
                doneSatus = status; 

            status = setDefaultMonitor();
            if (doneSatus)
                doneSatus = status; 

            status = setTCPSessionTimeOut();
            if (doneSatus)
                doneSatus = status;

            this.Cursor = Cursors.Default; 

            if (doneSatus)
            {
                this.Close();
            }
            else
            {
                UpdateFields();
            }
        }

        private void OptInCheckBox_Checked(object sender, EventArgs e)
        {
            if (OptInCheckBox.Checked)
            {
                numericUpDownOptInTimeout.Enabled = true;
                OptInTimeoutLabel.Enabled = true;
            }
            else
            {
                numericUpDownOptInTimeout.Enabled = false;
                OptInTimeoutLabel.Enabled = false;
            }

        }

    }
}
