﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.IO;
using System.Threading;
using System.Drawing.Imaging; 

using RealVNC.VNCViewerControl;
using RealVNC.VNCViewerSDK;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class KVMViewer : Form
    {
        public enum ConnectionType
        {
            Redirection = 0,
            Standard,
            Disconnected
        }

        /// <summary>
        /// Various KVM states
        /// </summary>
        public enum KVM_VIEWER_STATES
        {
            /// <summary>
            /// KVM is disconnected
            /// </summary>
            KVM_VIEWER_READY,
            /// <summary>
            /// KVM connected
            /// </summary>
            KVM_VIEWER_CONNECTED,
            /// <summary>
            /// KVM is during connection
            /// </summary>
            KVM_VIEWER_CONNECTING
        };

        /// <summary>
        /// Color level define various types for color level
        /// </summary>
        public enum COLOR_LEVEL
        {       
            COLOR_LEVEL_FULL = System.Drawing.Imaging.PixelFormat.Format16bppRgb565,     
            COLOR_LEVEL_MEDIUM = System.Drawing.Imaging.PixelFormat.Format16bppRgb555,
            COLOR_LEVEL_VERY_LOW = System.Drawing.Imaging.PixelFormat.Format8bppIndexed,
        };

        private AmtSystem _system;
        private ConnectionType _connectionType;
        private KVM_VIEWER_STATES _lastState;
        private string _RFBPassword;

        private static bool _RedirectFormAvailable = true;

        public static bool RedirectFormAvailable
        {
            get { return _RedirectFormAvailable; }
        }

        public ConnectionType connectionType
        {
            get { return _connectionType; }
        }

        public static void ShowKVMForm(AmtSystem system, ConnectionType type, string Password)
        {
            if (system.KVMForm == null)
            {
                system.KVMForm = new KVMViewer(system, type, Password);
                ((KVMViewer)system.KVMForm).Show();
            }
            else
            {
                try
                {
                    ((KVMViewer)system.KVMForm).Show();
                }
                catch (ObjectDisposedException)
                {
                    system.KVMForm = new KVMViewer(system, type, Password);
                    ((KVMViewer)system.KVMForm).Show();
                }
                ((KVMViewer)system.KVMForm).Activate();
            }
        }

        public static KVMViewer GetKVMForm(AmtSystem system, ConnectionType type, string Password)
        {
            if (system.KVMForm == null) 
                system.KVMForm =  new KVMViewer(system, type, Password);
            return (KVMViewer)(system.KVMForm);
        }

        public static void DisposeKVMForm(AmtSystem system)
        {
            if (system.KVMForm != null)
            {
                ((KVMViewer)system.KVMForm).Close();
                system.KVMForm = null;
            }
        }


        public KVMViewer(AmtSystem system, ConnectionType type, string Password)
        {
            if (type == ConnectionType.Redirection && !_RedirectFormAvailable)
            {
                throw new Exception("Redirection KVM Viewer already opened");
            }

            _system = system;
            _connectionType = type;
            _RFBPassword = Password;

            if (type == ConnectionType.Redirection)
                _RedirectFormAvailable = false; 

            InitializeComponent();

            InitViewer();
            UpdateUI();

            // this line is VERY IMPORTANT
            // it enforce creation of windows handle
            // the handle should be exists before the Start function will be called
            IntPtr p = Handle;
        }

        private void UpdateUI()
        {
            this.Text = "Remote KVM: " + _system.HostNameIdentity.DefaultName;

            switch (_connectionType)
            {
                case ConnectionType.Standard:
                    lbConnectionMode.Text = "Connection Mode: Standard Port";
                    TLSButton.Visible = false;
                    break;

                case ConnectionType.Redirection:
                    lbConnectionMode.Text = "Connection Mode: Redirection Port";
                    TLSButton.Visible = _system.UseTls;
                    break;
            }
        }

        private void InitViewer()
        {
            if(_connectionType == ConnectionType.Redirection)
                _system.KVMWrapper.InitializeKVMLib();

            _system.KVMWrapper.InitializeVNCControl();

            // those are the 3 main events
            _system.KVMWrapper.ViewerPanel.ViewerEvents.OnServerInitCallback += ServerInitHandler;
            _system.KVMWrapper.ViewerPanel.ViewerEvents.OnStatusCallback += ViewerEvents_OnStatusCallback;
            _system.KVMWrapper.ViewerPanel.ViewerEvents.OnCredentialsCallback += CredentialsHandler;
            // register for notification about a dektop resize
            _system.KVMWrapper.ViewerPanel.ViewerEvents.OnDesktopResizeCallback += ViewerEvents_OnDesktopResizeCallback;
            _lastState = KVM_VIEWER_STATES.KVM_VIEWER_READY;
            this.panelRemote.Controls.Add(_system.KVMWrapper.ViewerPanel);
        }

        #region Viewer Control Events
        // if this function was called it means that the connection succeeded
        private delegate void ServerInitHandler_D(IVNCViewerEvents sender, Size desktopSize,
                                       String desktopName, VNCPixelFormat serverNativePixelFormat);
        /// <summary>
        /// Called when the session is fully established.
        /// </summary>
        /// <param name="sender">the object that generate the event</param>
        /// <param name="desktopSize">the size of the remote desktop</param>
        /// <param name="desktopName">the name of the remote name</param>
        /// <param name="serverNativePixelFormat">the native pixel format of the server</param>
        private void ServerInitHandler(IVNCViewerEvents sender, Size desktopSize,
                                       String desktopName, VNCPixelFormat serverNativePixelFormat)
        {

            if (InvokeRequired)
            {
                BeginInvoke(new ServerInitHandler_D(ServerInitHandler), new object[] { sender, desktopSize, desktopName, serverNativePixelFormat });
            }
            else
            {
                ChangedState(KVM_VIEWER_STATES.KVM_VIEWER_CONNECTED, VNCViewerStatus.None, string.Empty);
            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender">the object that generate the event</param>
        /// <param name="usernameRequired">indicate if a username required (Currently Intel(r) AMT does not support a username for the RFB session )</param>
        /// <param name="username">the username (Currently Intel(r) AMT does not support a username for the RFB session )</param>
        /// <param name="passwordRequired">indicate if a password is required</param>
        /// <param name="password">the RFB password</param>
        private void CredentialsHandler(IVNCViewerEvents sender,
                         bool usernameRequired,
                         out string username,
                         bool passwordRequired,
                         out string password)
        {
            // no username require in order to access Intel(r) AMT
            username = (usernameRequired) ? string.Empty : string.Empty;
            password = (passwordRequired) ? _RFBPassword : string.Empty;
        }

        private delegate void onDesktopResizeCallback_D(IVNCViewerEvents sender, Size size);
        /// <summary>
        /// Called when the server desktop has been resized (e.g. they changed
        /// the display resolution).  This operation is similar to
        /// CreatePixelBuffer() but we have to make sure to copy the contents
        /// of the server desktop into the new PixelBuffer first!
        /// </summary>
        /// <param name="sender">the object that generate the event</param>
        /// <param name="size">the new size of the remote machine</param>
        private void ViewerEvents_OnDesktopResizeCallback(IVNCViewerEvents sender, Size size)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new onDesktopResizeCallback_D(ViewerEvents_OnDesktopResizeCallback), new object[] { sender, size });
            }
            else
            {
                _system.KVMWrapper.ViewerPanel.Size = size;
            }
        }

        private delegate void ViewerEvents_OnStatusCallback_D(IVNCViewerEvents sender, VNCViewerStatus status, string closeReason);
        /// <summary>
        /// This function will be notified when a VNC session was closed
        /// </summary>
        /// <param name="sender">the object that generate the event</param>
        /// <param name="status">the reason why the connection was closed</param>
        /// <param name="closeReason">string that contain error message explaining the close reason</param>
        private void ViewerEvents_OnStatusCallback(IVNCViewerEvents sender, VNCViewerStatus status, string closeReason)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new ViewerEvents_OnStatusCallback_D(ViewerEvents_OnStatusCallback), new object[] { sender, status, closeReason });
            }
            else
            {
                Stop();
                ChangedState(KVM_VIEWER_STATES.KVM_VIEWER_READY, status, closeReason);
            }
        }

        #endregion


        /// <summary>
        /// Send a command to refresh all of the screen
        /// </summary>
        public void RefreshScreen(object sender, EventArgs e)
        {
            if(_lastState == KVM_VIEWER_STATES.KVM_VIEWER_CONNECTED)
                _system.KVMWrapper.ViewerPanel.RefreshScreen();
        }

        private void Start()
        {
            PositionMouseOnPanel();
            this.panelRemote.Visible = false;

            int status = 0;

            // if using redirection ports, start the proxy and connect the VNCViewerControl to the proxy
            if (_connectionType == ConnectionType.Redirection)
            {
                status = _system.KVMWrapper.StartKVM(AmtKVMWrapper.KVMConnectionType.Redirection);
            }
            // if using default port connect to intel(r) AMT on the default port (5900)
            else if (_connectionType == ConnectionType.Standard)
            {
                status = _system.KVMWrapper.StartKVM(AmtKVMWrapper.KVMConnectionType.Standard);
            }

            // notify about the status changed
            ChangedState(KVM_VIEWER_STATES.KVM_VIEWER_CONNECTING, VNCViewerStatus.None, string.Empty);
            //return status;
        }

        /// <summary>
        /// Start the KVM session
        /// </summary>
        public void Start(object sender, EventArgs e)
        {
            Start(); 
        }

        /// <summary>
        /// Position the mouse on the remote panel
        /// </summary>
        private void PositionMouseOnPanel()
        {
            int X = this.Left + this.Width / 2;
            int Y = this.Top + this.Height / 2;
            Cursor.Position = new Point(X, Y);
        }


        /// <summary>
        /// Stop the KVM session
        /// </summary>
        public void Stop(object sender, EventArgs e)
        {
            Stop(); 
        }

        private void Stop()
        {
            this.panelRemote.Visible = false;
            if (_connectionType == ConnectionType.Redirection)
                _system.KVMWrapper.StopKVM();

            _system.KVMWrapper.ViewerPanel.Disconnect(); 
        }


        private delegate void ChangedState_D(KVM_VIEWER_STATES new_state, VNCViewerStatus status, string closeReason);
        /// <summary>
        /// This function update the GUI settings based on the state of the KVM Viewer
        /// </summary>
        /// <param name="new_state">the new state of the KVM viewer</param>
        /// <param name="status">the status of the VNCViewerControl</param>
        /// <param name="closeReason">if session was closed this contain the close reason</param>
        private void ChangedState(KVM_VIEWER_STATES new_state, VNCViewerStatus status, string closeReason)
        {
            StartButton.Enabled = false;
            StopButton.Enabled = false;
            UpdateUI();

            if (InvokeRequired)
            {
                BeginInvoke(new ChangedState_D(ChangedState), new object[] { new_state, status, closeReason });
            }
            else
            {
                switch (new_state)
                {
                    case KVM_VIEWER_STATES.KVM_VIEWER_READY:
                        StartButton.Enabled = true;
                        changeMouse(Cursors.Default);
                        string reason = (closeReason == null || closeReason == string.Empty) ? string.Empty : String.Format(ManageabilityControls.Properties.Resources.KVMViewer_Reason, closeReason);
                        if (_lastState == KVM_VIEWER_STATES.KVM_VIEWER_CONNECTING)
                        {
                            MessageBox.Show(
                                String.Format(ManageabilityControls.Properties.Resources.KVMViewer_Connection_Errror_Mesg, reason),
                                 ManageabilityControls.Properties.Resources.KVMViewer_Connection_Error,
                                 MessageBoxButtons.OK, MessageBoxIcon.Error);
                        }
                        else if (_lastState == KVM_VIEWER_STATES.KVM_VIEWER_CONNECTED)
                        {
                            MessageBox.Show(
                                String.Format(ManageabilityControls.Properties.Resources.KVMViewer_Connection_Warning_Mesg, reason),
                                 ManageabilityControls.Properties.Resources.KVMViewer_Connection_Warning,
                                 MessageBoxButtons.OK, MessageBoxIcon.Warning);
                        }

                        //            EnableDisableConnectionMenu(false);

                        lbStatus.Text = ManageabilityControls.Properties.Resources.KVMViewer_Status_Disconnected;
                        this.panelRemote.Visible = true;

                        break;
                    case KVM_VIEWER_STATES.KVM_VIEWER_CONNECTING:

                        changeMouse(Cursors.WaitCursor);

                        lbStatus.Text = string.Format(ManageabilityControls.Properties.Resources.KVMViewer_Status_Connecting, _system.HostNameIdentity.DefaultName);
                        this.panelRemote.Visible = false;
                        break;
                    case KVM_VIEWER_STATES.KVM_VIEWER_CONNECTED:
                        StopButton.Enabled = true;
                        changeMouse(Cursors.Default);

                        //            EnableDisableConnectionMenu(true);

                        lbStatus.Text = string.Format(ManageabilityControls.Properties.Resources.KVMViewer_Status_Connected, _system.HostNameIdentity.DefaultName);
                        this.panelRemote.Visible = true;

                        break;
                }

                _lastState = new_state;
            }
        }

        /// <summary>
        /// Delegate change the mouse state
        /// </summary>
        /// <param name="state">The new mouse state</param>
        private delegate void changeMouse_D(Cursor state);
        /// <summary>
        /// Change the mouse state
        /// </summary>
        /// <param name="state">The new mouse state</param>
        private void changeMouse(Cursor state)
        {
            if (InvokeRequired)
            {
                BeginInvoke(new changeMouse_D(changeMouse), new object[] { state });
            }
            else
            {
                this.Cursor = state;
            }
        }

        private void KVMViewer_FormClosed(object sender, FormClosedEventArgs e)
        {
            //make sure that the KVM Viewer is stopped if the form is closing
            Stop();
            
            //add a short delay so that any remaining VNC callbacks can complete
            //without this delay will ocassionaly get an 'invoke' error thrown
            //from the VNC library
            Thread.Sleep(500);

            if (_connectionType == ConnectionType.Redirection)
                _RedirectFormAvailable = true;

            _connectionType = ConnectionType.Disconnected; 
        }

        private void KVMViewer_Shown(object sender, EventArgs e)
        {
            Start();
            scaleing(true);
            SetDefaultMonitorChecked();
            SetColorChecked(COLOR_LEVEL.COLOR_LEVEL_FULL);
        }

        private void scaleingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            scaleing(((ToolStripMenuItem)sender).Checked); 
        }

        private void scaleing(bool scale)
        {
            this.panelCenter.Visible = false;
            if (scale)
            {
                _system.KVMWrapper.ViewerPanel.AutoScroll = false;
                _system.KVMWrapper.ViewerPanel.Dock = DockStyle.Fill;
            }
            else
            {
                _system.KVMWrapper.ViewerPanel.AutoScroll = true;
                _system.KVMWrapper.ViewerPanel.Dock = DockStyle.None;
            }
            this.panelCenter.Visible = true;
        }

        private void KVMViewer_ResizeBegin(object sender, EventArgs e)
        {
            this.panelCenter.Visible = false;
        }

        private void KVMViewer_ResizeEnd(object sender, EventArgs e)
        {
            this.panelCenter.Visible = true;
        }

        private void AltCtrDelButton_Click(object sender, EventArgs e)
        {
            _system.KVMWrapper.ViewerPanel.Viewer.SendKeyEvent(RealVNC.VNCViewerSDK.VNCKeyEventType.Down, RealVNC.VNCViewerSDK.VNCViewerSDK.SpecialKeyToXKeySymbol((uint)Keys.ControlKey));
            _system.KVMWrapper.ViewerPanel.Viewer.SendKeyEvent(RealVNC.VNCViewerSDK.VNCKeyEventType.Down, RealVNC.VNCViewerSDK.VNCViewerSDK.SpecialKeyToXKeySymbol((uint)Keys.Menu));
            _system.KVMWrapper.ViewerPanel.Viewer.SendKeyEvent(RealVNC.VNCViewerSDK.VNCKeyEventType.Down, RealVNC.VNCViewerSDK.VNCViewerSDK.SpecialKeyToXKeySymbol((uint)Keys.Delete));
            _system.KVMWrapper.ViewerPanel.Viewer.SendKeyEvent(RealVNC.VNCViewerSDK.VNCKeyEventType.Up, RealVNC.VNCViewerSDK.VNCViewerSDK.SpecialKeyToXKeySymbol((uint)Keys.ControlKey));
            _system.KVMWrapper.ViewerPanel.Viewer.SendKeyEvent(RealVNC.VNCViewerSDK.VNCKeyEventType.Up, RealVNC.VNCViewerSDK.VNCViewerSDK.SpecialKeyToXKeySymbol((uint)Keys.Menu));
            _system.KVMWrapper.ViewerPanel.Viewer.SendKeyEvent(RealVNC.VNCViewerSDK.VNCKeyEventType.Up, RealVNC.VNCViewerSDK.VNCViewerSDK.SpecialKeyToXKeySymbol((uint)Keys.Delete));    
        }

        private void primaryToolStripMenuItem_click(object sender, EventArgs e)
        {
            _system.KVM.KVMDefaultScreen = 0;
            SetDefaultMonitorChecked(); 
        }

        private void secondaryToolStripMenuItem_click(object sender, EventArgs e)
        {
            _system.KVM.KVMDefaultScreen = 1;
            SetDefaultMonitorChecked(); 
        }

        private void SetDefaultMonitorChecked()
        {
            if (_system.KVM.KVMDefaultScreen.Value == 0)
            {
                primaryToolStripMenuItem.Checked = true;
                secondaryToolStripMenuItem.Checked = false;
            }
            else
            {
                primaryToolStripMenuItem.Checked = false;
                secondaryToolStripMenuItem.Checked = true;
            }
        }

        private void SetColorChecked(COLOR_LEVEL level)
        {
            ViewerPixelFormat format = (ViewerPixelFormat)level;
            _system.KVMWrapper.ViewerPanel.SetPixelFormat(format); 

            if (level == COLOR_LEVEL.COLOR_LEVEL_FULL)
            {
                fullAllAvailableColorsToolStripMenuItem.Checked = true;
                medium32768ColorsToolStripMenuItem.Checked = false;
                veryLow256ColorsToolStripMenuItem.Checked = false;
            }
            else if (level == COLOR_LEVEL.COLOR_LEVEL_MEDIUM)
            {
                fullAllAvailableColorsToolStripMenuItem.Checked = false;
                medium32768ColorsToolStripMenuItem.Checked = true;
                veryLow256ColorsToolStripMenuItem.Checked = false;
            }
            else
            {
                fullAllAvailableColorsToolStripMenuItem.Checked = false;
                medium32768ColorsToolStripMenuItem.Checked = false;
                veryLow256ColorsToolStripMenuItem.Checked = true;
            }
        }

        private void FullColor_Click(object sender, EventArgs e)
        {
            SetColorChecked(COLOR_LEVEL.COLOR_LEVEL_FULL);
        }

        private void MediumColor_Click(object sender, EventArgs e)
        {
            SetColorChecked(COLOR_LEVEL.COLOR_LEVEL_MEDIUM);
        }

        private void LowColor_Click(object sender, EventArgs e)
        {
            SetColorChecked(COLOR_LEVEL.COLOR_LEVEL_VERY_LOW); 
        }

        private void TLSButton_Click(object sender, EventArgs e)
        {
            if (_system.UseTls == false) return;
            X509Certificate2[] certs = _system.GetTlsCertificates();
            if (certs != null && certs.Length > 0) X509Certificate2UI.DisplayCertificate(certs[0], Handle);
        }
    }
}
