// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Xml;
using System.Text;
using System.Management;
using System.Collections;
using System.IO.Compression;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public class ManagementData
    {
        public string Query;
        public DateTime Time;
        public Hashtable[] Data;

        public ManagementData(string query, DateTime time, Hashtable[] data)
        {
            Query = query;
            Time = time;
            Data = data;
        }

        public static string ToXml(ManagementData[] managementData)
        {
            StringBuilder o = new StringBuilder("<?xml version=\"1.0\" encoding=\"utf-8\" ?><DiagnosticInfo>");
            foreach (ManagementData m in managementData) o.Append(m.ToXmlNoHeader());
            o.Append("</DiagnosticInfo>");
            return o.ToString();
        }

        public static byte[] ToBinary(ManagementData[] managementData)
        {
            return Utils.Compress(UTF8Encoding.UTF8.GetBytes(ToXml(managementData)));
        }

        public static ManagementData[] GetFromBinary(byte[] data)
        {
            byte[] buf = Utils.Decompress(data);
            if (buf == null) return null;
            return GetFromXml(UTF8Encoding.UTF8.GetString(buf));
        }

        public static ManagementData[] GetFromXml(string xml)
        {
            ArrayList l = new ArrayList();

            XmlDocument xdoc = new XmlDocument();
            xdoc.LoadXml(xml);

            XmlNodeList nodes = xdoc.GetElementsByTagName("query");
            if (nodes.Count == 0) return new ManagementData[0];

            // For each query
            foreach (XmlNode n1 in nodes)
            {
                // For each table
                ArrayList data = new ArrayList();
                foreach (XmlNode n2 in n1.ChildNodes)
                {
                    // For each row
                    Hashtable t = new Hashtable();
                    foreach (XmlNode n3 in n2.ChildNodes)
                    {
                        t[n3.Attributes["name"].InnerText] = n3.InnerText;
                    }
                    data.Add(t);                    
                }
                l.Add(new ManagementData(n1.Attributes["name"].InnerText, DateTime.Parse(n1.Attributes["timeStamp"].InnerText), (Hashtable[])data.ToArray(typeof(Hashtable))));
            }

            return (ManagementData[])l.ToArray(typeof(ManagementData));
        }

        public string QueryType
        {
            get
            {
                string[] queryDecode = Query.Split('|');
                if (queryDecode.Length < 1) return null;
                return queryDecode[0];
            }
        }

        public string QueryDetail
        {
            get
            {
                string[] queryDecode = Query.Split('|');
                if (queryDecode.Length < 1) return null;
                return queryDecode[1];
            }
        }

        public string WimScope
        {
            get
            {
                string[] queryDecode = Query.Split('|');
                if (queryDecode.Length != 3 || queryDecode[0] != "WMI") return null;
                return queryDecode[1];
            }
        }

        public string WmiQuery
        {
            get
            {
                string[] queryDecode = Query.Split('|');
                if (queryDecode.Length != 3 || queryDecode[0] != "WMI") return null;
                return queryDecode[2];
            }
        }

        public byte[] ToBinary()
        {
            return Utils.Compress(UTF8Encoding.UTF8.GetBytes(ToXml()));
        }

        public string ToXml()
        {
            return string.Format("<?xml version=\"1.0\" encoding=\"utf-8\" ?><DiagnosticInfo>{0}</DiagnosticInfo>", ToXmlNoHeader());
        }

        private string ToXmlNoHeader()
        {
            StringBuilder output = new StringBuilder();
            output.AppendFormat("<query name=\"{0}\" timeStamp=\"{1}\">", Query, Time.ToLocalTime());
            foreach (Hashtable h in Data)
            {
                output.Append("<valueArray>");
                foreach (string k in h.Keys) { output.AppendFormat("<value name=\"{0}\">{1}</value>", System.Security.SecurityElement.Escape(k), System.Security.SecurityElement.Escape(h[k].ToString())); }
                output.Append("</valueArray>");
            }
            output.Append("</query>");
            return output.ToString();
        }

        public static ManagementData GetWmiQuery(string query)
        {
            try
            {
                string[] queryDecode = query.Split('|');
                if (queryDecode.Length != 3 || queryDecode[0] != "WMI") return null;

                ManagementObjectCollection c = new ManagementObjectSearcher(queryDecode[1], queryDecode[2]).Get();
                ArrayList l = new ArrayList();
                foreach (ManagementBaseObject obj in c)
                {
                    Hashtable h = new Hashtable();
                    foreach (PropertyData p in obj.Properties) if (p.Name != null && p.Value != null) { h[p.Name] = p.Value.ToString(); }
                    l.Add(h);
                }
                return new ManagementData(query, DateTime.Now, (Hashtable[])l.ToArray(typeof(Hashtable)));
            }
            catch (ManagementException)
            {
                return null;
            }
        }

    }


}
