// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class ScriptingForm : Form
    {
        private TerminalForm terminal;
        private AmtSystem computer = null;
        private char[] screen = null;
        private int runIndex = -2;
        private bool running = false;
        private int runPause = 500;
        private string formTitle = null;
        private FileInfo scriptInfo = null;
        private Hashtable jumpcounters = new Hashtable();

        [Serializable]
        public enum ScriptCommands
        {
            Nop,
            Label,
            Wait,
            Jump,
            Reset,
            IfOnScreen,
            WaitFor,
            Send,
            ExitScript,
            Message,
        }

        [Serializable]
        public enum ResetArgs
        {
            Normal,
            Bios,
            RemotedFloppy,
            RemotedCdrom,
            PowerDown,
            PowerUp,
        }
        [Serializable]
        public class ScriptCommand
        {
            public ScriptCommands command = ScriptCommands.Nop;
            public string arg1 = null;
            public string arg2 = null;
            public int arg3 = int.MinValue;
            public int arg4 = int.MinValue;

            public ScriptCommand() { }

            public ScriptCommand(ScriptCommands cmd, string a1, string a2, int a3, int a4)
            {
                command = cmd;
                arg1 = a1;
                arg2 = a2;
                arg3 = a3;
                arg4 = a4;
            }

            public override string ToString()
            {
                switch (command)
                {
                    case ScriptCommands.Nop:
                        return "NOP";
                    case ScriptCommands.Label:
                        return "LABEL \"" + arg1 + "\"";
                    case ScriptCommands.Wait:
                        return "WAIT " + arg3.ToString() + " seconds";
                    case ScriptCommands.Jump:
                        if (arg3 == 0)
                        {
                            return "JUMP to label \"" + arg1 + "\"";
                        }
                        else
                        {
                            return "JUMP to label \"" + arg1 + "\" " + arg3.ToString() + " time(s)";
                        }
                    case ScriptCommands.Reset:
                        return "RESET " + ((ResetArgs)arg3).ToString();
                    case ScriptCommands.IfOnScreen:
                        return "IF ON SCREEN...";
                    case ScriptCommands.WaitFor:
                        return "WAIT FOR \"" + arg1 + "\"";
                    case ScriptCommands.Send:
                        return "SEND \"" + arg1.Replace("\\q", "?").Replace("\\w", "?").Replace("\\a", "?").Replace("\\z", "?") + "\"";
                    case ScriptCommands.ExitScript:
                        return "EXIT SCRIPT";
                    case ScriptCommands.Message:
                        return "MESSAGE \"" + arg1 + "\"";
                }
                return "UNKNOWN";
            }

            public int GetIconIndex()
            {
                if (command == ScriptCommands.Reset || command == ScriptCommands.ExitScript) return 3;
                return 0;
            }
        }

        public ScriptingForm(TerminalForm terminal)
        {
            this.terminal = terminal;
            computer = terminal.computer;
            screen = terminal.screen;
            InitializeComponent();
        }

        private void ScriptingForm_Load(object sender, EventArgs e)
        {
            if (terminal.Silent == true)
            {
                this.Left = 20000;
                this.ShowInTaskbar = false;
            }

            if (this.Owner != null) Icon = Owner.Icon;
            this.Text = Utils.FilterStrEx(this.Text);
            formTitle = this.Text;
            changeRunIndex(-1);
            skipCriticalCommandsToolStripMenuItem.Checked = ((int)(Application.UserAppDataRegistry.GetValue("SkipCriticalCommands", 0)) == 0);
        }

        public void RunScript(string scriptFileName)
        {
            OpenScript(new FileInfo(scriptFileName));
            runButton_Click(this, null);
        }

        private void closeToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void AddCommand(ScriptCommand cmd)
        {
            ListViewItem l = new ListViewItem(cmd.ToString(), cmd.GetIconIndex());
            l.Tag = cmd;
            scriptingListView.Items.Add(l);
        }

        private void AddCommand(ScriptCommand cmd, int index)
        {
            ListViewItem l = new ListViewItem(cmd.ToString(), cmd.GetIconIndex());
            l.Tag = cmd;
            scriptingListView.Items.Insert(index, l);
        }

        private void newScriptToolStripMenuItem_Click(object sender, EventArgs e)
        {
            changeRunIndex(-1);
            scriptingListView.Items.Clear();
            scriptInfo = null;
            this.Text = formTitle;
        }

        private void openScriptToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                FileInfo fi = new FileInfo(openFileDialog.FileName);
                if (OpenScript(fi) == true)
                {
                    scriptInfo = fi;
                    this.Text = formTitle + " - " + scriptInfo.Name;
                }
            }
        }

        private bool OpenScript(FileInfo fi)
        {
            if (fi.Exists == false) return false;
            if (string.Compare(fi.Extension, ".scp", true) == 0)
            {
                // Open script in BINARY mode
                FileStream s;
                try
                {
                    s = new FileStream(fi.FullName, FileMode.Open);
                }
                catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return false; }
                byte[] buf = new byte[s.Length];
                s.Read(buf, 0, (int)(s.Length));
                s.Close();
                Hashtable t = (Hashtable)Utils.UnserializeBinary(buf);
                if (t == null)
                {
                    MessageBox.Show(this, Properties.Resources.ErrorLoadingScript, Properties.Resources.Script, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return false;
                }
                ArrayList cmds = (ArrayList)(t["commands"]);
                scriptingListView.BeginUpdate();
                scriptingListView.Items.Clear();
                changeRunIndex(-1);
                foreach (ScriptCommand sc in cmds)
                {
                    AddCommand(sc);
                }
                scriptingListView.EndUpdate();

                scriptInfo = fi;
                this.Text = formTitle + " - " + scriptInfo.Name;
            }
            else if (string.Compare(fi.Extension, ".txt", true) == 0)
            {
                // Open script in TEXT mode
                FileStream s;
                try
                {
                    s = new FileStream(fi.FullName, FileMode.Open);
                }
                catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return false; }
                byte[] buf = new byte[s.Length];
                s.Read(buf, 0, (int)(s.Length));
                s.Close();
                string[] sfile = UTF8Encoding.UTF8.GetString(buf).Replace("\n", "").Split('\r');

                scriptingListView.Items.Clear();
                changeRunIndex(-1);
                foreach (string scmd in sfile)
                {
                    if (scmd.Length == 0) continue;
                    string sscmd = scmd.Trim();
                    if (sscmd[0] == ':')
                    {
                        // This is a label
                        AddCommand(new ScriptCommand(ScriptCommands.Label, sscmd.Substring(1), null, 0, 0));
                        continue;
                    }
                    int i = sscmd.IndexOf(" ");
                    string cmd = null;
                    string[] args = null;
                    if (i < 0)
                    {
                        cmd = sscmd;
                    }
                    else
                    {
                        cmd = sscmd.Substring(0, i);
                        args = SplitQuoatsCommand(sscmd.Substring(i + 1));
                    }

                    if (args == null || args.Length == 0)
                    {
                        args = new string[2];
                        args[0] = "";
                        args[1] = "";
                    }
                    else if (args.Length == 1)
                    {
                        string[] args2 = new string[2];
                        args2[0] = args[0];
                        args2[1] = "";
                        args = args2;
                    }

                    if (string.Compare(cmd, ScriptCommands.ExitScript.ToString(), true) == 0) AddCommand(new ScriptCommand(ScriptCommands.ExitScript, null, null, int.MinValue, int.MinValue));
                    if (string.Compare(cmd, ScriptCommands.IfOnScreen.ToString(), true) == 0) AddCommand(new ScriptCommand(ScriptCommands.IfOnScreen, args[0], null, int.MinValue, int.MinValue));
                    if (string.Compare(cmd, ScriptCommands.Jump.ToString(), true) == 0) AddCommand(new ScriptCommand(ScriptCommands.Jump, args[0], null, GetInArgValue(args[1]), int.MinValue));
                    if (string.Compare(cmd, ScriptCommands.Label.ToString(), true) == 0) AddCommand(new ScriptCommand(ScriptCommands.Label, args[0], null, int.MinValue, int.MinValue));
                    if (string.Compare(cmd, ScriptCommands.Message.ToString(), true) == 0) AddCommand(new ScriptCommand(ScriptCommands.Message, args[0], null, int.MinValue, int.MinValue));
                    if (string.Compare(cmd, ScriptCommands.Nop.ToString(), true) == 0) AddCommand(new ScriptCommand(ScriptCommands.Nop, null, null, int.MinValue, int.MinValue));
                    if (string.Compare(cmd, ScriptCommands.Reset.ToString(), true) == 0) AddCommand(new ScriptCommand(ScriptCommands.Reset, null, null, GetInArgValue(args[0]), int.MinValue));
                    if (string.Compare(cmd, ScriptCommands.Send.ToString(), true) == 0) AddCommand(new ScriptCommand(ScriptCommands.Send, args[0], null, int.MinValue, int.MinValue));
                    if (string.Compare(cmd, ScriptCommands.Wait.ToString(), true) == 0) AddCommand(new ScriptCommand(ScriptCommands.Wait, null, null, GetInArgValue(args[0]), int.MinValue));
                    if (string.Compare(cmd, ScriptCommands.WaitFor.ToString(), true) == 0) AddCommand(new ScriptCommand(ScriptCommands.WaitFor, args[0], null, int.MinValue, int.MinValue));
                }
            }
            return true;
        }

        private int GetInArgValue(string arg)
        {
            if (arg.Length == 0) return 0;
            int r;
            if (int.TryParse(arg, out r) == false) return 0;
            return r;
        }

        public static string[] SplitQuoatsCommand(string str)
        {
            ArrayList list = new ArrayList();

            bool quotes = false;
            string arg = "";

            for (int i=0;i<str.Length;i++)
            {
                if (str[i] == '\"')
                {
                    if (quotes == false)
                    {
                        quotes = true;
                    }
                    else
                    {
                        list.Add(arg);
                        quotes = false;
                        arg = "";
                    }
                    continue;
                }
                if (str[i] == '\\')
                {
                    arg += str[i];
                    arg += str[i+1];
                    i++;
                    continue;
                }
                if (str[i] == ',' && quotes == false)
                {
                    if (arg.Length > 0) list.Add(arg);
                    arg = "";
                    continue;
                }
                arg += str[i];
            }
            if (arg.Length > 0) list.Add(arg);

            return (string[])list.ToArray(typeof(string));
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (scriptInfo == null) return;
            SaveScript(scriptInfo);
        }

        private void saveScriptToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (scriptInfo != null) saveFileDialog.FileName = scriptInfo.FullName;
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                FileInfo fi = new FileInfo(saveFileDialog.FileName);
                SaveScript(fi);
                this.Text = formTitle + " - " + fi.Name;
                scriptInfo = fi;
            }
        }

        private void SaveScript(FileInfo fi)
        {
            if (string.Compare(fi.Extension, ".scp", true) == 0)
            {
                // Save script in XML mode
                ArrayList cmds = new ArrayList();
                foreach (ListViewItem l in scriptingListView.Items) cmds.Add(l.Tag);
                Hashtable t = new Hashtable();
                t["commands"] = cmds;
                byte[] buf = Utils.SerializeBinary(t);
                FileStream s;
                try
                {
                    s = new FileStream(fi.FullName, FileMode.Create);
                }
                catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                s.Write(buf, 0, buf.Length);
                s.Close();
            }
            else if (string.Compare(fi.Extension, ".txt", true) == 0)
            {
                // Save script in TEXT mode
                ArrayList cmds = new ArrayList();
                foreach (ListViewItem l in scriptingListView.Items) cmds.Add(l.Tag);
                string output = "";
                foreach (ScriptCommand sc in cmds)
                {
                    if (sc.command == ScriptCommands.Label)
                    {
                        output += ":" + sc.arg1 + "\r\n";
                    }
                    else
                    {
                        output += sc.command.ToString();
                        string args = "";
                        if (sc.arg1 != null && sc.arg1.Length > 0) args += ",\"" + sc.arg1 + "\"";
                        if (sc.arg2 != null && sc.arg2.Length > 0) args += ",\"" + sc.arg2 + "\"";
                        if (sc.arg3 != int.MinValue) args += "," + sc.arg3;
                        if (sc.arg4 != int.MinValue) args += "," + sc.arg4;
                        if (args.Length > 0) args = args.Substring(1);
                        output += " " + args + "\r\n";
                    }
                }

                byte[] buf = UTF8Encoding.UTF8.GetBytes(output);
                FileStream s;
                try
                {
                    s = new FileStream(fi.FullName, FileMode.Create);
                }
                catch (Exception ex) { Utils.FileErrorMessageBox(this, ex); return; }
                s.Write(buf, 0, buf.Length);
                s.Close();
            }
        }

        private void editToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            addCommandToolStripMenuItem.Enabled = (running == false);
            removeSelectedToolStripMenuItem.Enabled = (running == false && scriptingListView.SelectedItems.Count > 0);
            copyToolStripMenuItem.Enabled = cutToolStripMenuItem.Enabled = (running == false && scriptingListView.SelectedItems.Count > 0);
            pasteToolStripMenuItem.Enabled = (Utils.UnSerializeFromClipboard(typeof(ScriptCommand[])) != null);
        }

        private void removeSelectedToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (running) return;
            changeRunIndex(-1);
            scriptingListView.BeginUpdate();
            while (scriptingListView.SelectedItems.Count > 0)
            {
                scriptingListView.Items.Remove(scriptingListView.SelectedItems[0]);
            }
            scriptingListView.EndUpdate();
        }

        private void scriptingListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                removeSelectedToolStripMenuItem_Click(this, null);
                e.Handled = true;
            }
        }

        private void scriptContextMenuStrip_Opening(object sender, CancelEventArgs e)
        {
            if (scriptingListView.SelectedItems.Count == 0)
            {
                editCommandToolStripMenuItem.Visible = false;
                setExecutionPointHereToolStripMenuItem.Visible = false;
                toolStripMenuItem2.Visible = false;
            }
            else
            {
                editCommandToolStripMenuItem.Visible = true;
                setExecutionPointHereToolStripMenuItem.Visible = true;
                toolStripMenuItem2.Visible = true;
            }
        }

        private void setExecutionPointHereToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (scriptingListView.SelectedItems.Count == 0) return;
            changeRunIndex(scriptingListView.SelectedIndices[0]);
        }

        private void changeRunIndex(int newIndex)
        {
            if (newIndex == runIndex) return;

            if (newIndex >= scriptingListView.Items.Count)
            {
                // Out of bounds
                newIndex = -1;
                if (terminal.Silent == true)
                {
                    for (int i = 0; i < 10; i++)
                    {
                        Application.DoEvents();
                        System.Threading.Thread.Sleep(50);
                    }
                    terminal.WriteSilentStatus("Script Completed");
                    terminal.Close();
                }
            }

            bool updateState = (newIndex < 0 || runIndex < 0);
            if (runIndex >= 0)
            {
                ScriptCommand cmd = (ScriptCommand)scriptingListView.Items[runIndex].Tag;
                scriptingListView.Items[runIndex].ImageIndex = cmd.GetIconIndex();
            }
            runIndex = newIndex;
            if (runIndex >= 0) scriptingListView.Items[runIndex].ImageIndex = 1;
            if (updateState == true) SetButtonState();
        }

        private void SetButtonState()
        {
            if (running == true)
            {
                runButton.ImageIndex = 5;
                pauseButton.ImageIndex = 2;
                stopButton.ImageIndex = 10;
                stepButton.ImageIndex = 0;
            }
            else
            {
                if (runIndex >= 0)
                {
                    runButton.ImageIndex = 4;
                    pauseButton.ImageIndex = 3;
                    stopButton.ImageIndex = 10;
                    stepButton.ImageIndex = 0;
                }
                else
                {
                    runButton.ImageIndex = 4;
                    pauseButton.ImageIndex = 2;
                    stopButton.ImageIndex = 11;
                    stepButton.ImageIndex = 0;
                }
            }
        }

        private void stopButton_Click(object sender, EventArgs e)
        {
            mainTimer.Enabled = false;
            running = false;
            changeRunIndex(-1);
        }

        private void pauseButton_Click(object sender, EventArgs e)
        {
            mainTimer.Enabled = false;
            running = false;
            SetButtonState();
        }

        private void stepButton_Click(object sender, EventArgs e)
        {
            if (running == true) return;
            RunNextCommand();
        }

        private void RunNextCommand()
        {
            if (runIndex < 0)
            {
                // Start the first command
                if (scriptingListView.Items.Count == 0) return;
                changeRunIndex(0);
            }

            if (runIndex >= scriptingListView.Items.Count)
            {
                // Out of bounds
                changeRunIndex(-1);
                return;
            }

            ScriptCommand cmd = (ScriptCommand)scriptingListView.Items[runIndex].Tag;

            // Perform the command
            switch (cmd.command)
            {
                case ScriptCommands.Nop:
                    changeRunIndex(runIndex + 1);
                    SetTimer(0);
                    break;
                case ScriptCommands.Label:
                    changeRunIndex(runIndex + 1);
                    SetTimer(0);
                    break;
                case ScriptCommands.Wait:
                    changeRunIndex(runIndex + 1);
                    SetTimer(cmd.arg3 * 1000);
                    break;
                case ScriptCommands.Jump:
                    if (cmd.arg3 > 0)
                    {
                        // Jump # times
                        if (!jumpcounters.Contains(cmd)) jumpcounters[cmd] = cmd.arg3;
                        int jc = (int)jumpcounters[cmd];
                        if (jc > 0)
                        {
                            // Perform the jump
                            jumpcounters[cmd] = jc - 1;
                            changeRunIndex(GetIndexOfLabel(cmd.arg1));
                        }
                        else
                        {
                            // Don't jump
                            changeRunIndex(runIndex + 1);
                        }
                    }
                    else
                    {
                        // Jump always
                        changeRunIndex(GetIndexOfLabel(cmd.arg1));
                    }
                    SetTimer(0);
                    break;
                case ScriptCommands.Reset:
                    if (skipCriticalCommandsToolStripMenuItem.Checked == false || terminal.TcpMode == false)
                    {
                        // Perform a reset
                        switch ((ResetArgs)cmd.arg3)
                        {
                            case ResetArgs.Normal:
                                terminal.remotedNormalBootToolStripMenuItem_Click(this, null);
                                break;
                            case ResetArgs.Bios:
                                terminal.remoteBIOSSetupToolStripMenuItem_Click(this, null);
                                break;
                            case ResetArgs.RemotedFloppy:
                                terminal.remoteBootToRedirectFloppyToolStripMenuItem_Click(this, null);
                                break;
                            case ResetArgs.RemotedCdrom:
                                terminal.remoteBootToRedirectCDToolStripMenuItem_Click(this, null);
                                break;
                            case ResetArgs.PowerDown:
                                terminal.powerDownToolStripMenuItem_Click(this, null);
                                break;
                            case ResetArgs.PowerUp:
                                terminal.powerUpToolStripMenuItem_Click(this, null);
                                break;
                        }
                    }
                    changeRunIndex(runIndex + 1);
                    SetTimer(0);
                    break;
                case ScriptCommands.IfOnScreen:
                    if (new string(screen).IndexOf(cmd.arg1) >= 0)
                    {
                        changeRunIndex(GetIndexOfLabel(cmd.arg2));
                    }
                    else
                    {
                        changeRunIndex(runIndex + 1);
                    }
                    SetTimer(0);
                    break;
                case ScriptCommands.WaitFor:
                    if (new string(screen).IndexOf(cmd.arg1) >= 0) changeRunIndex(runIndex + 1);
                    SetTimer(0);
                    break;
                case ScriptCommands.Send:
                    bool esc = false;
                    foreach (char c in cmd.arg1)
                    {
                        if (esc == true)
                        {
                            switch (c)
                            {
                                case 'r':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.Return));
                                    break;
                                case 'n':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.LineFeed));
                                    break;
                                case 't':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.Tab));
                                    break;
                                case '1':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.F1));
                                    break;
                                case '2':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.F2));
                                    break;
                                case '3':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.F3));
                                    break;
                                case '4':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.F4));
                                    break;
                                case '5':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.F5));
                                    break;
                                case '6':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.F6));
                                    break;
                                case '7':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.F7));
                                    break;
                                case '8':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.F8));
                                    break;
                                case '9':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.F9));
                                    break;
                                case '0':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.F10));
                                    break;
                                case '-':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.F11));
                                    break;
                                case '=':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.F12));
                                    break;
                                case 'q':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.Left));
                                    break;
                                case 'w':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.Right));
                                    break;
                                case 'a':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.Up));
                                    break;
                                case 'z':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.Down));
                                    break;
                                case 'x':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.Escape));
                                    break;
                                case 'i':
                                    terminal.TerminalForm_KeyDown(this, new KeyEventArgs(Keys.Control | Keys.I));
                                    break;
                                case '\\':
                                    terminal.TerminalForm_KeyPress(this, new KeyPressEventArgs('\\'));
                                    break;
                                default:
                                    terminal.TerminalForm_KeyPress(this, new KeyPressEventArgs(c));
                                    break;
                            }
                            esc = false;
                        }
                        else
                        {
                            switch (c)
                            {
                                case '\\':
                                    esc = true;
                                    break;
                                default:
                                    terminal.TerminalForm_KeyPress(this, new KeyPressEventArgs(c));
                                    break;
                            }
                        }
                    }
                    
                    changeRunIndex(runIndex + 1);
                    SetTimer(0);
                    break;
                case ScriptCommands.ExitScript:
                    stopButton_Click(this, null);
                    if (skipCriticalCommandsToolStripMenuItem.Checked == false)
                    {
                        if (terminal.Silent == true)
                        {
                            terminal.WriteSilentStatus("Script Completed");
                            terminal.Close();
                        }
                        else
                        {
                            Close();
                        }
                    }
                    break;
                case ScriptCommands.Message:
                    if (terminal.Silent == false)
                    {
                        MessageBox.Show(this, cmd.arg1, Properties.Resources.ScriptMessageTitle, MessageBoxButtons.OK);
                    }
                    else
                    {
                        terminal.WriteSilentStatus(cmd.arg1);
                    }
                    changeRunIndex(runIndex + 1);
                    SetTimer(0);
                    break;
            }
        }

        private int GetIndexOfLabel(string labelName)
        {
            int i = 0;
            foreach (ListViewItem l in scriptingListView.Items)
            {
                ScriptCommand cmd = (ScriptCommand)l.Tag;
                if (cmd.command == ScriptCommands.Label && cmd.arg1 == labelName) return i;
                i++;
            }
            return -1;
        }

        private void SetTimer(int milliseconds)
        {
            if (running == false) return;
            if (runIndex < 0)
            {
                running = false;
                SetButtonState();
                return;
            }
            mainTimer.Interval = milliseconds + runPause;
            mainTimer.Enabled = true;
        }

        private void mainTimer_Tick(object sender, EventArgs e)
        {
            mainTimer.Enabled = false;
            RunNextCommand();
        }

        private void runButton_Click(object sender, EventArgs e)
        {
            if (running == true) return;
            terminal.WriteSilentStatus("Starting Script");
            jumpcounters.Clear();

            if (runIndex < 0) changeRunIndex(0);
            running = true;
            SetTimer(0);
            SetButtonState();
        }

        private void fileToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            newScriptToolStripMenuItem.Enabled = (running == false);
            openScriptToolStripMenuItem.Enabled = (running == false);
            saveToolStripMenuItem.Enabled = (scriptInfo != null);
        }

        private void scriptingListView_DoubleClick(object sender, EventArgs e)
        {
            if (scriptingListView.SelectedItems.Count != 1) return;
            ListViewItem l = scriptingListView.SelectedItems[0];
            ScriptCommand cmd = (ScriptCommand)(l.Tag);
            ScriptCommandForm form = new ScriptCommandForm();
            form.command = cmd;
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                l.Tag = form.command;
                l.Text = form.command.ToString();
            }
        }

        private void addCommandToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            if (running) return;
            ScriptCommandForm form = new ScriptCommandForm();
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                changeRunIndex(-1);
                if (scriptingListView.SelectedItems.Count == 0)
                {
                    AddCommand(form.command);
                }
                else
                {
                    AddCommand(form.command, scriptingListView.SelectedIndices[0]);
                }
            }
        }

        private void ScriptingForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            stopButton_Click(this, null);
        }

        private void skipCriticalCommandsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            int x = 0;
            if (skipCriticalCommandsToolStripMenuItem.Checked == true) x = 1;
            Application.UserAppDataRegistry.SetValue("SkipCriticalCommands", x);
        }

        private void cutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            copyToolStripMenuItem_Click(this, null);
            removeSelectedToolStripMenuItem_Click(this, null);
        }

        private void copyToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ArrayList cmds = new ArrayList();
            foreach (ListViewItem l in scriptingListView.SelectedItems) cmds.Add(l.Tag);
            Utils.SerializeToClipboard((ScriptCommand[])cmds.ToArray(typeof(ScriptCommand)), typeof(ScriptCommand[]));
        }

        private void pasteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (running) return;
            ScriptCommand[] cmds = (ScriptCommand[])Utils.UnSerializeFromClipboard(typeof(ScriptCommand[]));
            if (cmds != null)
            {
                changeRunIndex(-1);
                if (scriptingListView.SelectedItems.Count == 0)
                {
                    foreach (ScriptCommand cmd in cmds) AddCommand(cmd);
                }
                else
                {
                    int i = scriptingListView.SelectedIndices[0];
                    foreach (ScriptCommand cmd in cmds)
                    {
                        AddCommand(cmd, i);
                        i++;
                    }
                }
            }
        }

        private void scriptingListView_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.None;
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false) == false) return;
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
            if (files == null || files.Length != 1) return;
            FileInfo fi = new FileInfo(files[0]);
            if (string.Compare(fi.Extension, ".xml", true) != 0 && string.Compare(fi.Extension, ".txt", true) != 0) return;
            e.Effect = DragDropEffects.All;
        }

        private void scriptingListView_DragDrop(object sender, DragEventArgs e)
        {
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop);
            if (files == null || files.Length != 1) return;
            OpenScript(new FileInfo(files[0]));
        }


    }
}
