﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ManageabilityStack;
using System.Text.RegularExpressions;

namespace ManageabilityControls
{
    public partial class UserConsentRequestForm : Form
    {
        private AmtSystem _system;
        private Regex _codeRegEx = new Regex(@"^\d{6}$");

        private bool _timeElapsed = false;

        private int _spriteCounter = -1;

        private System.Threading.Thread _pollingThread = null;

        private bool _dirty = true;
        private AmtUserConsent.OptInRequiredValues? _optInRequired = null;
        private AmtUserConsent.OptInStates? _optInState = null;

        private UserConsentRequestForm()
        {
            InitializeComponent();
        }

        public UserConsentRequestForm(AmtSystem system) : this(system, true)
        {
        }

        public UserConsentRequestForm(AmtSystem system, bool standAlone) : this()
        {
            _system = system;
            if (!standAlone)
            {
                string originalInstruction = instructionsLabel.Text;
                instructionsLabel.Text = "You initiated an action that requires user consent.\n\n" + originalInstruction;
            }

            _pollingThread = new System.Threading.Thread(new System.Threading.ParameterizedThreadStart(pollState));
            _pollingThread.Name = "OptIn Poll State";
            _pollingThread.Start(null);

            _refreshTimer.Start();

            updateForm();
        }

        public static bool RequestUserConsentIfNeeded(IWin32Window owner, AmtSystem system, AmtUserConsent.OptInRequiredValues level)
        {
            // If it's not supported by this version of AMT, or if the set level above what's needed, just let them go ahead.
            if (system.UserConsent.OptInRequired == null || system.UserConsent.OptInRequired < level)
            {
                //They don't need user consent for this action.
                return true;
            }
            if (!(system.UserConsent.OptInState == ManageabilityStack.AmtUserConsent.OptInStates.Received ||
                system.UserConsent.OptInState == ManageabilityStack.AmtUserConsent.OptInStates.InSession))
            {
                new UserConsentRequestForm(system, false).ShowDialog(owner);
            }

            if (!(system.UserConsent.OptInState == ManageabilityStack.AmtUserConsent.OptInStates.Received ||
                system.UserConsent.OptInState == ManageabilityStack.AmtUserConsent.OptInStates.InSession))
            {
                DialogResult result = MessageBox.Show(owner, "User consent is required for this action, but has not been obtained.  Continuing will produce errors and unpredictable results.  Are you sure you wish to continue?", "User Consent Needed", MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button2);
                return result == DialogResult.Yes;
            }
            else
            {
                // They must have gotten consent.
                return true;
            }
        }

        /// <summary>
        /// Override to disable X close button.
        /// </summary>
        protected override CreateParams CreateParams
        {
            get
            {
                CreateParams createParams = base.CreateParams;
                createParams.ClassStyle |= 0x200;  //CS_NOCLOSE
                return createParams;
            }
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
        }

        private void requestUserConsentButton_Click(object sender, EventArgs e)
        {
            _system.UserConsent.StartOptIn();
            _dirty = true;

            updateForm();

            _timeElapsed = false;
            _spriteCounter = 0;

            closeButton.Enabled = false;
        }

        //private delegate void _considerRebootD();
        private void considerReboot()
        {
            new UserConsentReboot(_system).ShowDialog(this);
        }

        private void updateForm()
        {
            if (_pollingThread != null)
            {
                // If we are polling and we know our copy is dirty, wait for the poll thread to update it.
                while (_dirty == true) { System.Threading.Thread.Sleep(0); }
            }
            else
            {
                // Looks like we're not polling.  So, refresh the state explicitly.
                getState();
            }
            AmtUserConsent.OptInStates? state = null;
            AmtUserConsent.OptInRequiredValues? optInRequired = null;
            // Get a local copy of the state.
            lock (this)
            {
                state = _optInState;
                optInRequired = _optInRequired;
            }
            if (state == null || optInRequired == null)
            {
                return;
            }
            // Optin State: Update the label.
            userConsentStateLabel.Text = lookupStateString(state);
            // userConsentPictureBox: Update the image.
            if (optInRequired == AmtUserConsent.OptInRequiredValues.None)
            {
                userConsentPictureBox.Visible = false;
            }
            else
            {
                userConsentPictureBox.Visible = true;
                switch (state)
                {
                    case AmtUserConsent.OptInStates.NotStarted:
                        userConsentPictureBox.BackgroundImage = ManageabilityControls.Properties.Resources.Consent_required;
                        break;
                    case AmtUserConsent.OptInStates.Requested:
                    case AmtUserConsent.OptInStates.Displayed:
                        userConsentPictureBox.BackgroundImage = ManageabilityControls.Properties.Resources.Consent_in_progress;
                        break;
                    case AmtUserConsent.OptInStates.Received:
                    case AmtUserConsent.OptInStates.InSession:
                        userConsentPictureBox.BackgroundImage = ManageabilityControls.Properties.Resources.Consent_granted;
                        break;
                }
            }
            // Request Button: If consent is not required because of policy or because it is already obtained, disable it.
            requestUserConsentButton.Enabled = (optInRequired != AmtUserConsent.OptInRequiredValues.None)
                && (state == AmtUserConsent.OptInStates.NotStarted);
            // Cancel User Consent Button:  Only enabled if a request is in progress or accepted.
            CancelUserConsentButton.Enabled = (state != AmtUserConsent.OptInStates.NotStarted);
            if (state == AmtUserConsent.OptInStates.Received || state == AmtUserConsent.OptInStates.InSession)
            {
                CancelUserConsentButton.Text = "Release";
            }
            else
            {
                CancelUserConsentButton.Text = "Cancel Request";
            }
            // Submit code field.
            consentCodeTextBox.Enabled = state == AmtUserConsent.OptInStates.Displayed;
            // Submit code button.  Set by key entry handler, so do nothing here.

            // Switch monitors button.  Display same as code text box.
            switchMonitorButton.Enabled = state == AmtUserConsent.OptInStates.Displayed;

            // Close Button: Only allowed when we're just starting or finished.  Otherwise, make them cancel.
            closeButton.Enabled = (state != AmtUserConsent.OptInStates.Requested && state != AmtUserConsent.OptInStates.Displayed);

        }

        private void consentCodeTextBox_TextChanged(object sender, EventArgs e)
        {
            submitCodeButton.Enabled = _codeRegEx.IsMatch(consentCodeTextBox.Text);
        }

        private void CancelUserConsentButton_Click(object sender, EventArgs e)
        {
            if (!_system.UserConsent.CancelOptIn())
            {
                MessageBox.Show(this, "Cannot release consent at this time.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
            {
                _dirty = true;
                consentCodeTextBox.Text = string.Empty;
                updateForm();
            }
        }

        private void refreshTimer_Tick(object sender, EventArgs e)
        {
            if (_spriteCounter >= 0)
            {
                if (_spriteCounter < 15 || _timeElapsed)
                {
                    _spriteCounter = (_spriteCounter + 1) % 20;
                }
                else // _spriteCounter >= 15
                {
                    _timeElapsed = true;
                    considerReboot();
                }
            }
            updateForm();
            if (_optInState == AmtUserConsent.OptInStates.Requested )
            {
                string ellipsis = new string('.', _spriteCounter / 2);
                userConsentStateLabel.Text += ellipsis;
            }
        }

        private void submitCodeButton_Click(object sender, EventArgs e)
        {
            uint code = 0;
            try
            {
                code = Convert.ToUInt32(consentCodeTextBox.Text);
            }
            catch (FormatException)
            {
                MessageBox.Show(this, "Code was not a six-digit number.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                return;
            }
            if (!_system.UserConsent.SendUserCode(code))
            {
                MessageBox.Show(this, "Code not accepted.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                // Still update the form, because if it was the third attempt the state goes back to NotStarted.
            }
            _dirty = true;
            submitCodeButton.Enabled = false;
            consentCodeTextBox.Text = string.Empty;
            updateForm();
        }

        private void switchMonitorButton_Click(object sender, EventArgs e)
        {
            _system.UserConsent.SwitchSpriteMonitor();
        }

        private void pollState(object x)
        {
            try
            {
                while (true)
                {
                    getState();
                    System.Threading.Thread.Sleep(200);
                }
            }
            catch (System.Threading.ThreadInterruptedException)
            {
                //Normal exit.  Let it go.
            }
        }
        private void getState()
        {
            AmtUserConsent userConsent = _system.UserConsent;
            AmtUserConsent.OptInRequiredValues? optInRequired;
            AmtUserConsent.OptInStates? optInState;
            lock (userConsent)
            {
                userConsent.ClearCache();
                optInRequired = userConsent.OptInRequired;
                optInState = userConsent.OptInState;
            }
            lock (this)
            {
                _optInRequired = optInRequired;
                _optInState = optInState;
                _dirty = false;
            }
        }

        private void UserConsentRequestForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (_pollingThread != null)
            {
                _pollingThread.Interrupt();
                _pollingThread = null;
            }
            if (_refreshTimer.Enabled)
            {
                _refreshTimer.Stop();
            }
        }

        private static string lookupStateString(AmtUserConsent.OptInStates? state)
        {
            string retVal = string.Empty;
            switch (state)
            {
                case AmtUserConsent.OptInStates.NotStarted:
                    retVal = "Press \"Request\" to begin.";
                    break;
                case AmtUserConsent.OptInStates.Requested:
                    retVal = "Requesting.";
                    break;
                case AmtUserConsent.OptInStates.Displayed:
                    retVal = "A code is now displayed on the user's system.\nEnter this code below.";
                    break;
                case AmtUserConsent.OptInStates.Received:
                case AmtUserConsent.OptInStates.InSession:
                    retVal = "Request granted.";
                    break;
            }
            return retVal;
        }
    }
}
