// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Data;
using System.Text;
using System.Drawing;
using System.ComponentModel;
using System.Windows.Forms;
using System.Collections.Generic;
using ManageabilityStack;

namespace ManageabilityControls
{
    public partial class WatchdogMonitorForm : Form
    {
        private SerialHighDriver serial;
        private bool updating = false;
        private AmtSystem computer;

        private class agentInfo
        {
            public string Name;
            public Guid ID;
            public string Condition;
            public bool Active;
        }

        public WatchdogMonitorForm(SerialHighDriver serial, AmtSystem computer)
        {
            this.computer = computer;
            this.serial = serial;
            InitializeComponent();

            // If the computer is disconnected, close this form
            this.computer = computer;
            if (computer != null) computer.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        private void WatchdogMonitorForm_Load(object sender, EventArgs e)
        {
            if (this.Owner != null) Icon = Owner.Icon;

            // Get the watchdog list
            serial.ProcessDataBlockEvent += new SerialHighDriver.ProcessDataBlockHandler(WatchdogDataBlockHandler);
            serial.WriteDataBlock("WGRQ", null);
        }

        private void SystemStateChangedHandler(AmtSystem computer) { try { this.Invoke(new AmtSystem.ObjectStateHandler(SystemStateChangedHandlerEx), new object[] { computer }); } catch (Exception) { } }
        private void SystemStateChangedHandlerEx(AmtSystem computer) { if (computer.State != AmtSystemObjState.Connected) Close(); }

        private void WatchdogDataBlockHandler(SerialHighDriver sender, string name, byte[] data)
        {
            // Speed up by context switching only if this is a message we process.
            if (name.Length == 4 && name.CompareTo("WGRS") == 0)
            {
                object[] p = new object[3];
                p[0] = sender;
                p[1] = name;
                p[2] = data;
                this.Invoke(new SerialHighDriver.ProcessDataBlockHandler(WatchdogDataBlockHandlerEx), p);
            }
        }

        private void WatchdogDataBlockHandlerEx(SerialHighDriver sender, string name, byte[] data)
        {
            if (name.CompareTo("WGRS") == 0)
            {
                string s = UTF8Encoding.UTF8.GetString(data);
                if (s.Length < 5) return;

                updating = true;
                watchDogListView.BeginUpdate();

                Guid selectedItem = Guid.Empty;
                bool selectedItemEx = false;
                if (watchDogListView.SelectedItems.Count == 1)
                {
                    selectedItem = ((agentInfo)watchDogListView.SelectedItems[0].Tag).ID;
                    selectedItemEx = true;
                }

                watchDogListView.Items.Clear();
                string[] ss = s.Split('~');
                for (int i = 0; i < (ss.Length - 1); i += 5)
                {
                    agentInfo a = new agentInfo();
                    a.ID = new Guid(ss[i]);
                    a.Name = ss[i + 1];
                    if (ss[i + 2].Length > 0) a.Condition = ss[i + 2];
                    a.Active = ss[i + 3].Equals("1");
                    string state = ss[i + 4];
                    string conditionStr = "";
                    if (a.Condition != null && a.Condition.Length > 0) conditionStr = " (" + a.Condition + ")";
                    ListViewItem l = new ListViewItem(new String[] { a.Name + conditionStr, state });
                    l.Tag = a;
                    l.Checked = a.Active;
                    watchDogListView.Items.Add(l);
                    if (selectedItemEx == true && watchDogListView.SelectedItems.Count == 0 && selectedItem == a.ID) l.Selected = true;
                }

                watchDogListView.EndUpdate();
                updating = false;
                removeButton.Enabled = (watchDogListView.SelectedItems.Count == 1);

                refreshTimer.Enabled = autoRefreshCheckBox.Checked;
            }
        }

        private void refreshToolStripMenuItem_Click(object sender, EventArgs e)
        {
            serial.WriteDataBlock("WGRQ", null);
        }

        private void refreshTimer_Tick(object sender, EventArgs e)
        {
            refreshTimer.Enabled = false;
            serial.WriteDataBlock("WGRQ", null);
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void WatchdogMonitorForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            serial.ProcessDataBlockEvent -= new SerialHighDriver.ProcessDataBlockHandler(WatchdogDataBlockHandler);
            serial = null;
            if (computer != null) computer.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
        }

        private void autoRefreshCheckBox_CheckedChanged(object sender, EventArgs e)
        {
            refreshTimer.Enabled = autoRefreshCheckBox.Checked;
        }

        private void watchDogListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            removeButton.Enabled = (watchDogListView.SelectedItems.Count == 1);
        }

        private void removeButton_Click(object sender, EventArgs e)
        {
            if (watchDogListView.SelectedItems.Count != 1) return;
            agentInfo a = (agentInfo)watchDogListView.SelectedItems[0].Tag;
            serial.WriteDataBlock("WGRM", a.ID.ToByteArray());
            serial.WriteDataBlock("WGRQ", null);
        }

        private void addButton_Click(object sender, EventArgs e)
        {
            ManageabilityControls.AddWatchdogForm form = new ManageabilityControls.AddWatchdogForm();
            if (watchDogListView.SelectedItems.Count == 1)
            {
                agentInfo a = (agentInfo)watchDogListView.SelectedItems[0].Tag;
                form.AgentName = a.Name;
                form.AgentProcessCondition = a.Condition;
                form.AgentID = a.ID;
            }
            if (form.ShowDialog(this) == DialogResult.OK)
            {
                string condition = "";
                if (form.AgentProcessCondition != null) condition = form.AgentProcessCondition;
                serial.WriteDataBlock("WGAD", UTF8Encoding.UTF8.GetBytes(form.AgentID.ToString() + "~" + form.AgentName + "~" + condition + "~1~"));
                serial.WriteDataBlock("WGRQ", null);
            }
        }

        private void watchDogListView_ItemChecked(object sender, ItemCheckedEventArgs e)
        {
            if (updating == true) return;
            agentInfo a = (agentInfo)e.Item.Tag;
            string active = "0";
            if (e.Item.Checked == true) active = "1";
            string condition = "";
            if (a.Condition != null) condition = a.Condition;
            serial.WriteDataBlock("WGAD", UTF8Encoding.UTF8.GetBytes(a.ID.ToString() + "~" + a.Name + "~" + condition + "~" + active + "~"));
            serial.WriteDataBlock("WGRQ", null);
        }

        private void watchDogListView_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Delete)
            {
                removeButton_Click(this, null);
                e.Handled = true;
            }
        }

    }
}
