// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Net.Sockets;
using System.Collections;
using System.Collections.Generic;

namespace ManageabilityDirectorTool
{
    public class AmtHelloServer
    {
        private TcpListener tcpListener;
        private ArrayList tcpClients = new ArrayList();
        private Hashtable tcpBufList = new Hashtable();
        private Hashtable tcpEpList = new Hashtable();
        private Hashtable tcpHelloList = new Hashtable();
        private ushort port;

        public delegate void StateChangeHandler(AmtHelloServer sender);
        public event StateChangeHandler StateChangeEvent;

        public delegate void HelloHandler(AmtHelloServer sender, AmtHello msg);
        public event HelloHandler HelloEvent;

        public AmtHelloServer()
        {
            port = 9971;
        }

        public AmtHelloServer(ushort port)
        {
            this.port = port;
        }

        public bool IsListening {get { return tcpListener != null; }}
        public int Connections { get { return tcpClients.Count; } }
        public ushort ListenPort { get { return port; } }

        public void Start()
        {
            if (tcpListener != null) return;

            tcpListener = new TcpListener(IPAddress.Any, port);
            try
            {
                tcpListener.Start();
                tcpListener.BeginAcceptTcpClient(new AsyncCallback(ListenerEvent), this);
            }
            catch (SocketException)
            {
                tcpListener = null;
            }

            if (StateChangeEvent != null) StateChangeEvent(this);
        }

        public void Stop()
        {
            if (tcpListener != null)
            {
                tcpListener.Stop();
                tcpListener = null;
            }

            foreach (TcpClient client in tcpClients) { client.Close(); }

            tcpClients.Clear();
            tcpBufList.Clear();

            if (StateChangeEvent != null) StateChangeEvent(this);
        }

        private void ListenerEvent(IAsyncResult r)
        {
            if (tcpListener == null) return;

            TcpClient client;
            try
            {
                client = tcpListener.EndAcceptTcpClient(r);
            }
            catch (ObjectDisposedException) {return;}

            tcpEpList.Add(client, (IPEndPoint)client.Client.RemoteEndPoint);
            tcpClients.Add(client);
            tcpListener.BeginAcceptTcpClient(new AsyncCallback(ListenerEvent), this);
            if (StateChangeEvent != null) StateChangeEvent(this);

            byte[] buf = new byte[1024];
            tcpBufList[client] = buf;
            client.GetStream().BeginRead(buf, 0, 1024, new AsyncCallback(ReadEvent), client);
        }

        private void ReadEvent(IAsyncResult r)
        {
            TcpClient client = (TcpClient)r.AsyncState;
            byte[] buf = (byte[])tcpBufList[client];
            if (client == null) return;

            int len;
            try
            {
                len = client.GetStream().EndRead(r);
            }
            catch (InvalidOperationException)
            {
                CloseTcp(client);
                return;
            }
            catch (IOException)
            {
                CloseTcp(client);
                return;
            }

            if (len == 0)
            {
                CloseTcp(client);
                return;
            }

            // Add the data to the accumulated data
            if (tcpHelloList.ContainsKey(client) == false)
            {
                byte[] b = new byte[len];
                Array.Copy(buf, b, len);
                tcpHelloList[client] = b;
            }
            else
            {
                byte[] b1 = (byte[])tcpHelloList[client];
                byte[] b = new byte[b1.Length + len];
                Array.Copy(b1, b, b1.Length);
                Array.Copy(buf, 0, b, b1.Length, len);
                tcpHelloList[client] = b;
            }

            buf = new byte[1024];
            tcpBufList[client] = buf;
            try
            {
                client.GetStream().BeginRead(buf, 0, 1024, new AsyncCallback(ReadEvent), client);
            }
            catch (ObjectDisposedException) { }
            catch (InvalidOperationException) {CloseTcp(client);}
            catch (IOException) {CloseTcp(client);}
        }

        private void CloseTcp(TcpClient client)
        {
            client.Close();
            tcpClients.Remove(client);
            tcpBufList.Remove(client);

            byte[] hello = (byte[])tcpHelloList[client];
            if (hello != null && hello.Length > 0)
            {
                ManageabilityStack.AmtDebugForm.LogEvent(this, ManageabilityStack.AmtDebugForm.EventType.Information, "Got HELLO data", "HEX: " + ManageabilityStack.Utils.BytesToHex(hello) + "\r\n\r\nUTF: " + UTF8Encoding.UTF8.GetString(hello));
                if (HelloEvent != null) HelloEvent(this, new AmtHello(hello, (IPEndPoint)tcpEpList[client]));
            }

            tcpEpList.Remove(client);
            tcpHelloList.Remove(client);
            if (StateChangeEvent != null) StateChangeEvent(this);
        }

        public void InjectFakeHello(AmtHello msg)
        {
            if (HelloEvent != null) HelloEvent(this, new AmtHello(msg.Data, msg.RemoteEndPoint));
        }

        [Serializable]
        public class AmtHello
        {
            public byte[] Data;
            public string Pid;
            public byte[][] CertHash;
            public DateTime ReceivedTime;
            public IPEndPoint RemoteEndPoint;
            public int Version;

            public AmtHello(byte[] buf, IPEndPoint ep)
            {
                Data = buf;
                ReceivedTime = DateTime.Now;
                RemoteEndPoint = ep;
                Version = buf[2];
                if (buf.Length == 32) // One Touch PID
                {
                    byte[] b = new byte[8];
                    Array.Copy(buf,24,b,0,8);
                    Pid = UTF8Encoding.UTF8.GetString(b);
                    if (Pid.Length == 8) Pid = Pid.Substring(0, 4) + "-" + Pid.Substring(4, 4);
                }
                if (Version == 3) // Zero-Touch Key Hash
                {
                    int hashCount = buf[24];
                    CertHash = new byte[hashCount][];
                    int ptr = 26;
                    for (int i = 0; i < hashCount; i++)
                    {
                        CertHash[i] = new byte[buf[ptr]];
                        Array.Copy(buf, ptr + 1, CertHash[i], 0, buf[ptr]);
                        ptr += (buf[ptr] + 2);
                    }
                }
            }

            public bool NetworkPasswordChanged
            {
                get {return BitConverter.ToInt16(Data, 0) != 0;}
            }

            public Guid GetGuid()
            {
                if (Data.Length < 24) return Guid.Empty;
                byte[] b = new byte[16];
                Array.Copy(Data, 8, b, 0, 16);
                return new Guid(b);
            }

            public float GetVersion()
            {
                if (Data.Length < 4) return 0;
                return (float)BitConverter.ToInt16(Data, 2);
            }
        }

    }
}
