// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Text;
using System.Collections;
using System.Collections.Generic;

namespace ManageabilityDirectorTool
{
    public class AmtProvisioningKeyStore
    {
        private Microsoft.Win32.RegistryKey reg;
        public Hashtable Keys = new Hashtable();

        public AmtProvisioningKeyStore(Microsoft.Win32.RegistryKey reg)
        {
            this.reg = reg;
            Refresh();
        }

        public void Refresh()
        {
            Keys.Clear();
            AmtProvisioningKey[] k = GetKeysFromReg();
            if (k != null) foreach (AmtProvisioningKey v in k) {Keys.Add(v.Pid, v);}
        }

        public void Flush()
        {
            ArrayList l = new ArrayList();
            foreach (AmtProvisioningKey v in Keys.Values) { l.Add(v); }
            SetKeysInReg((AmtProvisioningKey[])l.ToArray(typeof(AmtProvisioningKey)));
        }

        public AmtProvisioningKey GetNewKey()
        {
            AmtProvisioningKey key = AmtProvisioningKey.GetNextKey(reg);
            Keys.Add(key.Pid, key);
            return key;
        }

        public void AddKey(AmtProvisioningKey key)
        {
            Keys[key.Pid] = key;
        }

        public void RemoveKey(AmtProvisioningKey key)
        {
            Keys.Remove(key.Pid);
        }

        private AmtProvisioningKey[] GetKeysFromReg()
        {
            try
            {
                return (AmtProvisioningKey[])ManageabilityStack.Utils.UnserializeBinary((byte[])reg.GetValue("ProvisioningKeys", null));
            }
            catch (Exception)
            {
                return new AmtProvisioningKey[0];
            }
        }

        private void SetKeysInReg(AmtProvisioningKey[] keys)
        {
            reg.SetValue("ProvisioningKeys", ManageabilityStack.Utils.SerializeBinary(keys));
        }

    }


    
    [Serializable]
    public class AmtProvisioningKey
    {
        public enum AmtProvisioningKeyState
        {
            Creation,
            Pending,
            Used
        }

        public string Pid;
        public string Pps;
        public AmtProvisioningKeyState State;
        public DateTime GenTime;
        public int HelloRequests;
        public int CompletedRequests;
        public string AdminPassword;
        public string ConfigurationProfile;
        public string SecurityTemplate;

        public AmtProvisioningKey(string pid, string pps, AmtProvisioningKeyState state, DateTime genTime)
        {
            Pid = pid;
            Pps = pps;
            State = state;
            GenTime = genTime;
            HelloRequests = 0;
            CompletedRequests = 0;
        }

        private static string alphabet = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
        private static Random randomGenerator = new Random();

        private static char ComputeChecksum(string s)
        {
	        int A=0;
            for (int i = 0; i < s.Length; i++) A += s[i];
            A %= alphabet.Length;
	        return alphabet[A];
        }

        private static string GenerateRandomPid()
        {
            string pid = "";
            for (int i = 0; i < 7; i++) pid += alphabet[randomGenerator.Next(alphabet.Length)];
            pid += ComputeChecksum(pid);
            return pid.Substring(0, 4) + "-" + pid.Substring(4, 4);
        }

        private static string GenerateUserPid(string userPid)
        {
            if (userPid.Length != 7) return null;
            userPid += ComputeChecksum(userPid);
            return userPid.Substring(0, 4) + "-" + userPid.Substring(4, 4);
        }

        private static string GenerateNextPid(string pid)
        {
            pid = pid.Replace("-","").Trim();
            if (pid.Length != 8) return null;
            pid = pid.Substring(0,7);

            char[] p = pid.ToCharArray();

            bool carry = true;
            for (int i = 6; i >= 0; i--)
            {
                if (carry)
                {
                    carry = false;
                    p[i] = (char)(((byte)p[i]) + 1);
                    if (p[i] == ('9' + 1)) p[i] = 'A';
                    if (p[i] > 'Z')
                    {
                        carry = true;
                        p[i] = '0';
                    }
                }
            }

            pid = new string(p);
            pid += ComputeChecksum(pid);
            return pid.Substring(0, 4) + "-" + pid.Substring(4, 4);
        }

        private static string GetNextPid(Microsoft.Win32.RegistryKey reg)
        {
            string keyid = (string)reg.GetValue("NextKeyPid", "0000-0000");
            if (keyid == null) keyid = "0000-0000";
            keyid = GenerateNextPid(keyid);
            if (keyid == null) keyid = GenerateNextPid("0000-0000");
            reg.SetValue("NextKeyPid", keyid);
            return keyid;
        }

        private static string GenerateRandomPps()
        {
            string pps = "";
            string s;
            for (int j = 0; j < 8; j++)
            {
                s = "";
                for (int i = 0; i < 3; i++) s += alphabet[randomGenerator.Next(alphabet.Length)];
                s += ComputeChecksum(s);
                pps += s;
                if (j < 7) pps += "-";
            }
            return pps;
        }

        private static string GenerateUserPps(string userPps)
        {
            if (userPps.Length != 24) return null;

            string pps = "";
            string s;
            for (int j = 0; j < 8; j++)
            {
                s = userPps.Substring(j*3,3);
                s += ComputeChecksum(s);
                pps += s;
                if (j < 7) pps += "-";
            }
            return pps;
        }

        public static AmtProvisioningKey GetNextKey(Microsoft.Win32.RegistryKey reg)
        {
            return new AmtProvisioningKey(GetNextPid(reg), GenerateRandomPps(), AmtProvisioningKeyState.Pending, DateTime.Now);
        }

        public static AmtProvisioningKey GetNullKey()
        {
            return new AmtProvisioningKey(GenerateUserPid("0000000"), GenerateUserPps("000000000000000000000000"), AmtProvisioningKeyState.Pending, DateTime.Now);
        }

        public static string GetFullPps()
        {
            return GenerateRandomPps();
        }

        public static string GetWeakPps()
        {
            string s1 = GenerateRandomPps();
            string s2 = GenerateUserPps("000000000000000000000000");
            return s2.Substring(0, 30) + s1.Substring(30, 9);
        }

        public static string GetNullPps()
        {
            return GenerateUserPps("000000000000000000000000");
        }

    }

}
