// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Text;
using System.Collections;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityDirectorTool
{
    public interface SecurityTemplateChange
    {
        AmtCallStatus PerformAction(MainForm form, AmtSystem system);
    }

    public class SecurityTemplateReset : SecurityTemplateChange
    {
        public bool FullReset = false;
        public AmtSecurityAdmin.ProvisioningMode Mode = AmtSecurityAdmin.ProvisioningMode.Current;

        public SecurityTemplateReset(bool fullReset, AmtSecurityAdmin.ProvisioningMode mode)
        {
            Mode = mode;
            FullReset = fullReset;
        }

        public AmtCallStatus PerformAction(MainForm form, AmtSystem system)
        {
            AmtCallStatus r;
            if (FullReset)
            {
                // Issue Full Reset
                r = system.SecurityAdmin.Unprovision(Mode);
                if (r != AmtCallStatus.SUCCESS)
                {
                    System.Windows.Forms.MessageBox.Show(form, 
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_Unprov_Err + r.ToString(), 
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_Unprov_Err_Title,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            else
            {
                r = system.SecurityAdmin.PartialUnprovision();
                if (r != AmtCallStatus.SUCCESS)
                {
                    System.Windows.Forms.MessageBox.Show(form, 
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_PartUnprov_Err + r.ToString(), 
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_PartUnprov_Err_Title,
                        MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }

            system.Disconnect();
            return r;
        }
    }

    public class SecurityTemplateAdminChange : SecurityTemplateChange
    {
        public string Username;
        public string Password;

        public SecurityTemplateAdminChange(string username, string password)
        {
            Username = username;
            Password = password;
        }

        public AmtCallStatus PerformAction(MainForm form, AmtSystem system)
        {
            AmtCallStatus r = AmtCallStatus.SUCCESS;

            form.AddProvisioningServerLog(String.Format(ManageabilityDirectorTool.Properties.Resources.Config_With_Account, system.HostNameIdentity.DisplayName));

            if (Password != null)
            {
                r = system.SecurityAdmin.SetAdminAclEntryEx(Username, Password);
                if (r != AmtCallStatus.SUCCESS)
                {
                    System.Windows.Forms.MessageBox.Show(form, 
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_SetACLEx_Err + r.ToString(), 
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_SetACLEx_Err_Title,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return r;
                }

                system.Disconnect();
                system.Username = Username;
                system.Password = Password;
                form.SaveSettings();

                // Update the tree view name in case the admin account name changed.
                TreeNode n = Utils.GetNodeWithTag(form.rootNode, system);
                n.Text = system.HostNameIdentity.DisplayName + " / " + system.Username;
            }
            return r;
        }
    }

    [Serializable]
    public class SecurityTemplate : SecurityTemplateChange
    {
        public string Name;

        public AmtSecurityAdmin.EnabledInterfaces EnabledInterfaces = new AmtSecurityAdmin.EnabledInterfaces();
        public CrlType[] Crls = null;
        public string[] TrustedFqdnCN = null;
        public string TLSCertificateIssuerTP = null;
        public string[] TrustedRootCertificatesTP = null;
        public string IssueOrganisation = Properties.Resources.SampleOrganisation;
        public string IssueCountryCode = Properties.Resources.SampleCertificateCountryCode;
        public int IssueKeySize = 1024;
        public TlsAthenticationType TlsAthenticationLocal = TlsAthenticationType.NoAuth;
        public TlsAthenticationType TlsAthenticationRemote = TlsAthenticationType.NoAuth;
        public KerberosOptionsType KerberosOptions = null;
        public string AdminAclEntry = null;
        public string AdminAclPassword = null;
        public AmtSecurityAdmin.AmtUserAcl[] UserAclEntries = null;
        public bool RedirectionPort = false;

        // Power Saving
        public int ActivePowerPackage = 0;
        public int ActiveStateAC = 0;
        public int WakeOnNetAccessThresholdAC = 0;
        public int WakeOnNetAccessSleepTimer = 0;

        public SecurityTemplate(string name)
        {
            Name = name;
        }

        public AmtCallStatus PerformAction(MainForm form, AmtSystem system)
        {
            if (system.AmtRev == 1) return AmtCallStatus.CALL_SKIPPED;

            AmtCallStatus r = AmtCallStatus.SUCCESS;

            form.AddProvisioningServerLog(String.Format(ManageabilityDirectorTool.Properties.Resources.Config_With_Profile, system.HostNameIdentity.DisplayName, Name));

            system.SecurityAdmin.SetTrustedFqdnCN(TrustedFqdnCN);
            system.SecurityAdmin.SetEnabledInterfaces(EnabledInterfaces);
            system.Redirection.SetRedirectionListenerState(RedirectionPort);
            system.SecurityAdmin.SetRngKey();
            r = system.SecurityAdmin.ResetUserAclEntriesEx(UserAclEntries);
            if (r != AmtCallStatus.SUCCESS)
            {
                System.Windows.Forms.MessageBox.Show(form, 
                    ManageabilityDirectorTool.Properties.Resources.MsgBox_SetACL_Err + r.ToString(),
                    ManageabilityDirectorTool.Properties.Resources.MsgBox_SetACL_Err_Title,
                    MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }

            // Set Trusted Certificates
            system.SecurityAdmin.DeleteAllTrustedRootCertificates();
            if (TrustedRootCertificatesTP != null && TrustedRootCertificatesTP.Length != 0)
            {
                ArrayList certslist = new ArrayList();
                foreach (string tp in TrustedRootCertificatesTP)
                {
                    X509Certificate2 c = form.GetCertFromThumpprint(tp);
                    if (c != null) certslist.Add(c);
                }
                X509Certificate2[] certs = (X509Certificate2[])certslist.ToArray(typeof(X509Certificate2));
                system.SecurityAdmin.SetTrustedRootCertificates(certs);
            }

            // Issue a certificate
            if (TLSCertificateIssuerTP == null)
            {
                // system.SecurityAdmin.SetTLSCertificate(null);
                if (TlsAthenticationLocal == TlsAthenticationType.NoAuth && TlsAthenticationRemote == TlsAthenticationType.NoAuth)
                {
                    system.SecurityAdmin.SetTlsOptions(TlsAthenticationLocal, TlsAthenticationRemote);

                    if (system.CoreVersion >= 2.5)
                    {
                        // Get the TLS certificate handle
                        uint tlsHandle;
                        bool tlsHandleSpecified;
                        r = system.SecurityAdmin.GetTLSCredentials(out tlsHandle, out tlsHandleSpecified);

                        if (r == AmtCallStatus.SUCCESS && tlsHandleSpecified == true)
                        {
                            // Remove the old certificate if any
                            r = system.SecurityAdmin.SetTlsCredentials(0, false);
                            r = system.SecurityAdmin.CertStoreRemoveCertificate(tlsHandle);
                        }
                    }
                }
            }
            else
            {
                // Issue a new certificate
                string dnsname = system.GetDnsName();

                ArrayList certnames = new ArrayList();
                if (dnsname == null)
                {
                    // Issue a cert with IP address
                    if (system.IPAddress != null)
                    {
                        certnames.Add(system.IPAddress.ToString());
                    }
                    else
                    {
                        try
                        {
                            certnames.Add(IPAddress.Parse(system.HostNameIdentity.DisplayName).ToString());
                        }
                        catch (Exception) { }
                        if (certnames.Count == 0)
                        {
                            try
                            {
                                certnames.Add(System.Net.Dns.GetHostEntry(system.HostNameIdentity.DisplayName).AddressList[0].ToString());
                            }
                            catch (Exception)
                            {
                                certnames.Add(Properties.Resources.Unknown);
                            }
                        }
                    }
                }
                else
                {
                    // Issue a cert with FQDN address
                    certnames.Add(dnsname);
                    string shortname = dnsname;
                    int i = dnsname.IndexOf(".");
                    if (i >= 0)
                    {
                        shortname = shortname.Substring(0, i);
                        certnames.Add(shortname);
                    }
                    certnames.Add(shortname + "$iME");
                }

                // Try to add device GUID to certificate
                if (!system.PlatformId.Equals(Guid.Empty))
                {
                    certnames.Add("Device (UUID: " + system.PlatformId.ToString() + ")");
                    //certnames.Add(system.PlatformId.ToString());
                }

                X509Certificate2 issuingCert = null;
                if (TLSCertificateIssuerTP != null && TLSCertificateIssuerTP.Length > 0)
                {
                    issuingCert = form.GetCertFromThumpprint(TLSCertificateIssuerTP);
                }

                // Can't issue a new certificate, no issuing certificate
                if (issuingCert == null)
                {
                    MessageBox.Show(form,
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_SetTLSCert_No_Setup,
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_SetTLSKeyCert_Err,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return AmtCallStatus.CALL_SKIPPED;
                }

                // Create a certificate
                X509Certificate2 newcert = ManageabilityStack.CertificateOperations.IssueSignedCert(form.GetCertFromThumpprint(TLSCertificateIssuerTP), (string[])certnames.ToArray(typeof(string)), IssueOrganisation, IssueCountryCode, IssueKeySize, ManageabilityStack.CertificateOperations.IssueCertType.TlsServer);
                if (newcert == null) return AmtCallStatus.FAILED_WEB_CALL;

                // Set the certificate
                if (system.CoreVersion < 2.5)
                {
                    // Build a certificate chain
                    X509Chain chain = new X509Chain();
                    chain.ChainPolicy = new X509ChainPolicy();
                    chain.ChainPolicy.RevocationMode = X509RevocationMode.NoCheck;
                    chain.Build(newcert); // Cert #0 is AMT, Cert #N is Root.
                    ArrayList certChainlist = new ArrayList();
                    foreach (X509ChainElement certChainElement in chain.ChainElements)
                    {
                        certChainlist.Add(certChainElement.Certificate);
                    }
                    X509Certificate2[] certChain = (X509Certificate2[])certChainlist.ToArray(typeof(X509Certificate2));

                    // Get the private key
                    byte[] privatekey = ManageabilityStack.CertificateOperations.GetPkcs10PrivateKeyFromCert(newcert);

                    // Older way to put in a certificate in AMT
                    r = system.SecurityAdmin.SetTLSKeyAndCertificate(privatekey, certChain);
                }
                else
                {
                    // Get the TLS certificate handle
                    uint tlsHandle;
                    bool tlsHandleSpecified;
                    system.SecurityAdmin.GetTLSCredentials(out tlsHandle, out tlsHandleSpecified);

                    // Get the private key
                    byte[] privatekey = ManageabilityStack.CertificateOperations.GetPkcs1PrivateKeyFromCert(newcert);
                    
                    // New way to put a certificate in AMT
                    uint h;
                    r = system.SecurityAdmin.CertStoreAddCertificate(newcert, privatekey, out h);
                    if (r == AmtCallStatus.SUCCESS) r = system.SecurityAdmin.SetTlsCredentials(h, true);

                    // Remove the old certificate if any
                    if (tlsHandleSpecified == true) system.SecurityAdmin.CertStoreRemoveCertificate(tlsHandle);
                }

                if (r != AmtCallStatus.SUCCESS)
                {
                    MessageBox.Show(form,
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_SetTLSKeyCert + r.ToString(),
                        ManageabilityDirectorTool.Properties.Resources.MsgBox_SetTLSKeyCert_Err,
                        MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return AmtCallStatus.CALL_SKIPPED;
                }

                system.SecurityAdmin.SetTlsOptions(TlsAthenticationLocal, TlsAthenticationRemote);
            }

            if (system.SecurityAdmin.CommitChanges() == AmtCallStatus.SUCCESS)
            {
                // Handle a possible switch in security connection

                if (system.UseTls == true && TlsAthenticationRemote == TlsAthenticationType.NoAuth)
                {
                    // Switch down to no auth.
                    system.Disconnect();
                    system.UseTls = false;
                    form.SaveSettings();
                }
                else if (system.UseTls == false && TlsAthenticationRemote != TlsAthenticationType.NoAuth)
                {
                    // Switch up to auth.
                    system.Disconnect();
                    system.UseTls = true;
                    form.SaveSettings();
                }
            }

            return r;
        }
    }
}
