// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Text;
using System.Data;
using ManageabilityOutpostTool;
using Microsoft.Win32;
using System.Diagnostics;
using System.ServiceProcess;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography.X509Certificates;
using ManageabilityStack;

namespace ManageabilityOutpostToolService
{
    public partial class OutpostService : ServiceBase
    {
        private AmtSystem system = null;
        private AmtSerialAgent serialAgent = null;
        private CryptProtectClass cp = new CryptProtectClass(CryptProtectClass.cpStore.USE_MACHINE_STORE);

        public OutpostService()
        {
            InitializeComponent();
        }

        public void DebugStart(string[] args) { OnStart(args); }
        public void DebugStop() { OnStop(); }
        public void DebugShutdown() { OnShutdown(); }
        public void DebugCustomCommand(int command) { OnCustomCommand(command); }

        protected override void OnStart(string[] args)
        {
            if (RegistryCtlClass.OutpostDataRegistry.GetValue("SerialAgent", bool.FalseString).ToString().CompareTo(bool.TrueString) == 0)
            {
                EnableSerialAgent(true);
            }

            if (RegistryCtlClass.OutpostDataRegistry.GetValue("ConnectAutomatic", bool.FalseString).ToString().CompareTo(bool.TrueString) == 0)
            {
                ConnectLocalAgent(true);
            }

            SystemEvents.PowerModeChanged += new PowerModeChangedEventHandler(SystemEvents_PowerModeChanged);
        }

        protected override void OnStop()
        {
            SystemEvents.PowerModeChanged -= new PowerModeChangedEventHandler(SystemEvents_PowerModeChanged);
            EnableSerialAgent(false);
            ConnectLocalAgent(false);
        }

        protected override void OnShutdown()
        {
            SystemEvents.PowerModeChanged -= new PowerModeChangedEventHandler(SystemEvents_PowerModeChanged);
            EnableSerialAgent(false);
            ConnectLocalAgent(false);
        }

        private bool DisconnectedLocal = false;
        private bool DisconnectedSerial = false;
        void SystemEvents_PowerModeChanged(object sender, PowerModeChangedEventArgs e)
        {
            switch (e.Mode)
            {
                case PowerModes.Resume:
                    if (DisconnectedLocal)
                    {
                        ConnectLocalAgent(true);
                        DisconnectedLocal = false;
                    }
                    if (DisconnectedSerial)
                    {
                        EnableSerialAgent(true);
                        DisconnectedSerial = false;
                    }
                    break;
                case PowerModes.Suspend:
                    if (system != null)
                    {
                        ConnectLocalAgent(false);
                        DisconnectedLocal = true;
                    }
                    if (serialAgent != null)
                    {
                        EnableSerialAgent(false);
                        DisconnectedSerial = true;
                    }
                    break;
            }
        }

        private int CustomCommandCount = 0;
        protected override void OnCustomCommand(int command)
        {
            RegistryCtlClass.OutpostDataRegistry.SetValue("CustomCommandCount", CustomCommandCount++);

            switch (command)
            { 
                case ServiceCtlClass.CC_SERIALAGENT_ENABLE:
                    if (EnableSerialAgent(true))
                    {
                        RegistryCtlClass.OutpostDataRegistry.SetValue("SerialAgent", bool.TrueString);
                    }
                    break;
                case ServiceCtlClass.CC_SERIALAGENT_DISABLE:
                    if (EnableSerialAgent(false))
                    {
                        RegistryCtlClass.OutpostDataRegistry.SetValue("SerialAgent", bool.FalseString);
                    }
                    break;
                case ServiceCtlClass.CC_AMTLOCAL_CONNECT:
                    ConnectLocalAgent(true);
                    break;
                case ServiceCtlClass.CC_AMTLOCAL_DISCONNECT:
                    ConnectLocalAgent(false);
                    break;
                case ServiceCtlClass.CC_PERMISSION_CHANGED:
                    SerialAgentPermissionCtl.SyncFromRegistry();
                    break;
                case ServiceCtlClass.CC_WATCHDOG_ADD:
                case ServiceCtlClass.CC_WATCHDOG_EDIT:
                case ServiceCtlClass.CC_WATCHDOG_REMOVE:
                    SyncWdFromRegistry();
                    break;
            }
        }

        private bool EnableSerialAgent(bool bEnable)
        {
            if (bEnable)
            {
                if (serialAgent == null)
                {
                    string SOLPortName = DevDriCtlClass.GetSOLPortName();
                    if (null == SOLPortName) {return false;}

                    try
                    {
                        serialAgent = new AmtSerialAgent(SOLPortName, "Manageability Outpost Tool Agent Service");
                        serialAgent.ChangedWatchdogState += new AmtSerialAgent.ChangedWatchdogStateHandler(ChangedWatchdogStateSink);
                        serialAgent.system = system;
                    }
                    catch (UnauthorizedAccessException)
                    {
                    }

                }
            }
            else
            {
                if (serialAgent != null)
                {
                    serialAgent.ChangedWatchdogState -= new AmtSerialAgent.ChangedWatchdogStateHandler(ChangedWatchdogStateSink);
                    serialAgent.Dispose();
                    serialAgent = null;
                }
            }

            return true;
        }

        private bool RemoteCertificateValidationCallbackSink(object sender, X509Certificate certificate, X509Chain chain, System.Net.Security.SslPolicyErrors sslPolicyErrors)
        {
            if (sender.GetType() == typeof(System.Net.HttpWebRequest))
            {
                System.Net.HttpWebRequest r = (System.Net.HttpWebRequest)sender;
                X509Certificate2[] certArray = new X509Certificate2[chain.ChainElements.Count];
                for (int i = 0; i < chain.ChainElements.Count; i++)
                {
                    certArray[i] = chain.ChainElements[i].Certificate;
                }
                if (r.ConnectionGroupName != null) AmtSystem.AddTlsCertificateCache(r.ConnectionGroupName, certArray);
            }
            return true; // Accept All Certificates
        }

        private void SystemStateChangedHandler(AmtSystem computer)
        {
            switch (computer.State)
            {
                case AmtSystemObjState.Connecting:
                    break;
                case AmtSystemObjState.Disconnected:
                    // Connection failed.
                    RegistryCtlClass.OutpostDataRegistry.SetValue("Connected", bool.FalseString);
                    if (serialAgent != null) serialAgent.system = null;
                    system.WatchdogLocal.Notification -= new AmtWatchdogLocal.NotificationHandler(NotificationSink);
                    system.WatchdogLocal.Stop();
                    system.Dispose();
                    system = null;
                    break;
                case AmtSystemObjState.Connected:
                    RegistryCtlClass.OutpostDataRegistry.SetValue("Connected", bool.TrueString);
                    RegistryCtlClass.OutpostDataRegistry.SetValue("UseTlsFirst", system.UseTls);
                    SyncWdFromRegistry();
                    system.WatchdogLocal.Notification += new AmtWatchdogLocal.NotificationHandler(NotificationSink);
                    system.WatchdogLocal.Start();
                    if (serialAgent != null) serialAgent.system = system;
                    break;
            }
        }

        private bool ConnectLocalAgent(bool bConnect)
        {
            // TODO: Don't always accept all certificates???
            if (ServicePointManager.ServerCertificateValidationCallback == null)
            {
                ServicePointManager.ServerCertificateValidationCallback = new System.Net.Security.RemoteCertificateValidationCallback(RemoteCertificateValidationCallbackSink);
            }

            if (bConnect)
            {
                if (system == null)
                {
                    byte[] bPassword = (byte[])RegistryCtlClass.OutpostDataRegistry.GetValue("PasswordBin", new byte[0],
                        RegistryValueOptions.DoNotExpandEnvironmentNames);
                    if (bPassword.Length == 0) return false;
                    string sPassword = cp.ByteArrayToStr(cp.Decrypt(bPassword, null));
                    string sUsername = (string)RegistryCtlClass.OutpostDataRegistry.GetValue("Username", "admin");
                    bool useTlsFirst = (bool)RegistryCtlClass.OutpostDataRegistry.GetValue("UseTlsFirst", bool.FalseString).ToString().Equals(bool.TrueString);

                    // Connect a new system object
                    system = new AmtSystem(Utils.GetLocalFqdn(), 16992, sUsername, sPassword, false, false);
                    system.UseTls = useTlsFirst;
                    system.LocalConnection = true;
                    system.AutoFetchCache = false;
                    system.OnStateChanged += new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                    system.Connect();
                }
            }
            else
            {
                if (system != null)
                {
                    RegistryCtlClass.OutpostDataRegistry.SetValue("Connected", bool.FalseString);
                    system.OnStateChanged -= new AmtSystem.ObjectStateHandler(SystemStateChangedHandler);
                    system.WatchdogLocal.Notification -= new AmtWatchdogLocal.NotificationHandler(NotificationSink);
                    system.WatchdogLocal.Stop();
                    system.WatchdogLocal.RemoveAllWatchdogs();
                    system.Disconnect();
                    system.Dispose();
                    system = null;
                }
            }

            return true;
        }

        private void NotificationSink(AmtSystem system, int notificationCode)
        {
            string msg = null;
            switch (notificationCode)
            {
                case 100:
                    ShutdownClass.DoExitWin(ShutdownClass.SLEEP, false, ref msg); // Suggest goto sleep
                    break;
                case 101:
                    ShutdownClass.DoExitWin(ShutdownClass.EWX_SHUTDOWN, false, ref msg); // Suggest shutdown
                    break;
                case 102:
                    ShutdownClass.DoExitWin(ShutdownClass.EWX_LOGOFF, false, ref msg); // Suggest logoff
                    break;
                case 103:
                    ShutdownClass.DoExitWin(ShutdownClass.EWX_REBOOT, false, ref msg); // Suggest reboot
                    break;
            }
        }

        private void SyncWdFromRegistry()
        {
            if (system == null) return;
            system.WatchdogLocal.RemoveAllWatchdogs();
            string data = (string)RegistryCtlClass.OutpostDataRegistry.GetValue("WatchdogsEx", "");
            if (data.Length < 3) return;
            string[] datas = data.Split('~');
            for (int i = 0; i < (datas.Length - 1); i += 4)
            {
                try
                {
                    string active = datas[i + 3];
                    if (active.Equals("1"))
                    {
                        string condition = datas[i + 2];
                        string name = datas[i + 1];
                        string ids = datas[i];
                        Guid id = new Guid(ids);
                        AmtWatchdogLocal.AmtWatchdog wg = new AmtWatchdogLocal.AmtWatchdog(name, id);
                        if (condition.Length > 0) wg.processCondition = condition;
                        system.WatchdogLocal.AddWatchdog(wg);
                    }
                }
                catch (Exception) {}
            }
        }

        private void SyncWdToRegistry()
        {
            string data = "";
            AmtWatchdogLocal.AmtWatchdog[] wgs = system.WatchdogLocal.GetWatchdogs();
            foreach (AmtWatchdogLocal.AmtWatchdog wg in wgs)
            {
                string active = "0";
                if (wg.beat == true) active = "1";
                data += wg.agentID.ToString() + "~" + wg.name + "~" + wg.processCondition + "~" + active + "~";
            }
            RegistryCtlClass.OutpostDataRegistry.SetValue("WatchdogsEx", data);
        }

        private void ChangedWatchdogStateSink()
        {
            SyncWdToRegistry();
        }

    }
}
