// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Text;
using System.Threading;
using System.Collections;
using System.Windows.Forms;
using System.Collections.Generic;

namespace ManageabilityOutpostTool
{
    public class SerialHighDriver
    {
        private const int LOOP_TIMEOUT = 50;
        private const int PACKET_SIZE = 512;
        private const int WRITEBUFFER_WATERMARK = 1024;

        private bool exitting = false;
        private SerialLowDriver serial;
        private Thread serialThread;
        private ArrayList PendingBlocks = new ArrayList();
        private TransferBlock CurrentBlock = null;
        private int CurrentBlockPtr = 0;
        private int CurrentBlockSize = 0;
        private string inboundBlockName = null;
        private int inboundBlockSize = 0;
        private MemoryStream inboundBlock = null;

        public delegate void SerialOutputHandler(SerialHighDriver sender, byte[] buf, int offset, int length);
        public event SerialOutputHandler SerialOutputEvent;

        public delegate void ProcessCharHandler(SerialHighDriver sender, byte c);
        public event ProcessCharHandler ProcessCharEvent;

        public delegate void ProcessEscapeHandler(SerialHighDriver sender, EscCode code, int[] args);
        public event ProcessEscapeHandler ProcessEscapeEvent;
        
        public delegate void ProcessDataBlockProgressHandler(SerialHighDriver sender, string name, int total, int current);
        public event ProcessDataBlockProgressHandler ProcessDataBlockProgressEvent;
        
        public delegate void ProcessDataBlockHandler(SerialHighDriver sender, string name, byte[] data);
        public event ProcessDataBlockHandler ProcessDataBlockEvent;

        public enum EscCode
        {
            ClearScreen = 'J',
            MoveCursor = 'H',
            ScreenAttribs = 'm',
            KeyUp = 65,
            KeyDown = 66,
            KeyRight = 67,
            KeyLeft = 68
        }

        private class TransferBlock
        {
            public string name;
            public byte[] data;
            public int offset;
            public int length;
        }

        public SerialHighDriver(string port)
        {
            serial = new SerialLowDriver(port);
            serial.ProcessCharEvent += new SerialLowDriver.ProcessCharHandler(ProcessCharEventSink);
            serial.ProcessEscapeEvent += new SerialLowDriver.ProcessEscapeHandler(ProcessEscapeEventSink);
            serial.ProcessPacketEvent += new SerialLowDriver.ProcessPacketHandler(ProcessPacketEventSink);
            serial.SerialOutputEvent += new SerialLowDriver.SerialOutputHandler(SerialOutputSink);

            serialThread = new Thread(new ThreadStart(SerialLoop));
            serialThread.Start();
        }

        private void SerialOutputSink(SerialLowDriver sender, byte[] buf, int offset, int length)
        {
            if (SerialOutputEvent != null) SerialOutputEvent(this, buf, offset, length);
        }

        public void Dispose()
        {
            if (serial != null)
            {
                exitting = true;
                WritePacket(5, null, 0, 0);
                while (serialThread.IsAlive) Thread.Sleep(10);
                serialThread = null;
                serial.Dispose();
                serial = null;
            }
        }

        public int BytesToWrite
        {
            get {return serial.BytesToWrite;}
        }

        private void SerialLoop()
        {
            try
            {
                while (exitting == false)
                {
                    if (serial.BytesToWrite < WRITEBUFFER_WATERMARK)
                    {
                        if (CurrentBlock == null)
                        {
                            // Block Setup
                            lock (PendingBlocks)
                            {
                                if (PendingBlocks.Count > 0)
                                {
                                    CurrentBlock = (TransferBlock)PendingBlocks[0];
                                    PendingBlocks.RemoveAt(0);
                                    CurrentBlockPtr = CurrentBlock.offset;
                                    CurrentBlockSize = CurrentBlock.length;

                                    // Send Name
                                    ////////////////////////////////////////////////////////////////
                                    byte[] b = UTF8Encoding.UTF8.GetBytes(CurrentBlock.name);
                                    WritePacket(1, b, 0, b.Length);

                                    // Send Size
                                    b = UTF8Encoding.UTF8.GetBytes(CurrentBlockSize.ToString());
                                    WritePacket(2, b, 0, b.Length);

                                    if (CurrentBlockSize == 0)
                                    {
                                        // End of writes
                                        WritePacket(4, null, 0, 0);
                                        CurrentBlock = null;
                                        CurrentBlockPtr = 0;
                                    }
                                }
                                else
                                {
                                    if (exitting == true) break;
                                    Thread.Sleep(LOOP_TIMEOUT);
                                }
                            }
                        }
                        if (CurrentBlock != null)
                        {
                            // Block Write
                            int sendsize = Math.Min(PACKET_SIZE, CurrentBlockSize);
                            WritePacket(3, CurrentBlock.data, CurrentBlockPtr, sendsize);
                            CurrentBlockPtr += sendsize;
                            CurrentBlockSize -= sendsize;
                            if (CurrentBlockSize == 0)
                            {
                                // End of writes
                                WritePacket(4, null, 0, 0);
                                CurrentBlock = null;
                                CurrentBlockPtr = 0;
                            }
                        }
                    }
                    else
                    {
                        if (exitting == true) break;
                        Thread.Sleep(LOOP_TIMEOUT);
                    }
                }
            }
            catch (ThreadInterruptedException) { }
            catch (ThreadAbortException) { }
        }

        private void ProcessCharEventSink(SerialLowDriver sender, byte c)
        {
            if (ProcessCharEvent != null) ProcessCharEvent(this, c);
        }

        private void ProcessEscapeEventSink(SerialLowDriver sender, byte code, int[] args)
        {
            if (code == 'f') code = (byte)'H';
            if (ProcessEscapeEvent != null) ProcessEscapeEvent(this, (EscCode)code, args);
        }

        private void ProcessPacketEventSink(SerialLowDriver sender, int command, byte[] data)
        {
            switch (command)
            {
                case 1: // Get name

                    //////////////////////////////////////////////////////
                    inboundBlockName = UTF8Encoding.UTF8.GetString(data);
                    break;
                case 2: // Get size
                    //////////////////////////////////////////////////////
                    inboundBlockSize = int.Parse(UTF8Encoding.UTF8.GetString(data));
                    inboundBlock = new MemoryStream();
                    if (ProcessDataBlockProgressEvent != null) ProcessDataBlockProgressEvent(this, inboundBlockName, inboundBlockSize, (int)inboundBlock.Length);
                    break;
                case 3: // Get data
                    inboundBlock.Write(data,0,data.Length);
                    if (ProcessDataBlockProgressEvent != null) ProcessDataBlockProgressEvent(this, inboundBlockName, inboundBlockSize, (int)inboundBlock.Length);
                    break;
                case 4: // Finish
                    if (inboundBlockName != null && ProcessDataBlockEvent != null) ProcessDataBlockEvent(this, inboundBlockName, inboundBlock.ToArray());
                    inboundBlock = null;
                    inboundBlockSize = 0;
                    break;
                case 5: // Exit
                    if (ProcessDataBlockEvent != null) ProcessDataBlockEvent(this, "EXIT", null);
                    inboundBlock = null;
                    inboundBlockSize = 0;
                    break;
                case 6: // Ping
                    if (ProcessDataBlockEvent != null) ProcessDataBlockEvent(this, "PING", null);
                    WritePacket(7, null, 0, 0);
                    break;
                case 7: // Pong
                    if (ProcessDataBlockEvent != null) ProcessDataBlockEvent(this, "PONG", null);
                    break;
            }
        }

        public void Ping()
        {
            WritePacket(6, null, 0, 0);
        }

        private void WritePacket(int command, byte[] data, int offset, int length)
        {
            lock (serial)
            {
                WriteEsc((byte)'@', command, length);
                if (data != null && length != 0) Write(data, offset, length);
            }
        }

        public void Write(byte text) { lock (serial) { serial.Write(text); } }
        public void Write(string text) { lock (serial) { serial.Write(text); } }
        public void Write(byte[] buffer) { lock (serial) { serial.Write(buffer); } }
        public void Write(byte[] buffer, int offset, int length) { lock (serial) { serial.Write(buffer, offset, length); } }
        public void WriteEsc(byte code) { lock (serial) { serial.Write(new byte[3] { 27, 91, code }); } }
        public void WriteEsc(byte code, string args) { lock (serial) { serial.Write("" + (char)27 + (char)91 + args + (char)code); } }
        public void WriteEsc(byte code, int arg) { lock (serial) { serial.Write("" + (char)27 + (char)91 + arg.ToString() + (char)code); } }
        public void WriteEsc(byte code, int arg1, int arg2) { lock (serial) { serial.Write("" + (char)27 + (char)91 + arg1.ToString() + ";" + arg2.ToString() + (char)code); } }
        public void WriteEsc(byte code, int arg1, int arg2, int arg3) { lock (serial) { serial.Write("" + (char)27 + (char)91 + arg1.ToString() + ";" + arg2.ToString() + ";" + arg3.ToString() + (char)code); } }
        public void WriteClearScreen() { WriteEsc((byte)'J', 2); }
        public void WriteKeyUp() { serial.Write(new byte[3] { 27, 91, 65 }); }
        public void WriteKeyDown() { serial.Write(new byte[3] { 27, 91, 66 }); }
        public void WriteKeyRight() { serial.Write(new byte[3] { 27, 91, 67 }); }
        public void WriteKeyLeft() { serial.Write(new byte[3] { 27, 91, 68 }); }
        public void WriteCursorMove(byte x, byte y) { WriteEsc((byte)'H', y, x); }

        public void WriteDataBlock(string name, byte[] data)
        {
            TransferBlock t = new TransferBlock();
            t.name = name;
            t.data = data;
            t.offset = 0;
            if (data != null) { t.length = data.Length; } else { t.length = 0; }
            lock (PendingBlocks) { PendingBlocks.Add(t); }
        }

        public void WriteDataBlock(string name, byte[] data, int offset, int length)
        {
            TransferBlock t = new TransferBlock();
            t.name = name;
            t.data = data;
            t.offset = offset;
            t.length = length;
            lock (PendingBlocks) { PendingBlocks.Add(t); }
        }

    }
}
