// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Text;
using System.IO.Ports;
using System.Threading;
using System.Windows.Forms;
using System.Collections.Generic;

namespace ManageabilityOutpostTool
{
    public class SerialLowDriver
    {
        private const int EXIT_TIMEOUT = 30;
        private const int LOOP_TIMEOUT = 50;

        private bool exitting = false;
        private string port;
        private SerialPort serial;
        private Thread serialThread = null;
        private int termstate = 0;
        private int[] escNumber = new int[32];
        private int escNumberPtr = 0;
        private byte[] packetAccumulator = null;
        private int packetAccumulatorPtr = 0;

        public delegate void SerialOutputHandler(SerialLowDriver sender, byte[] buf, int offset, int length);
        public event SerialOutputHandler SerialOutputEvent;
        public delegate void ProcessCharHandler(SerialLowDriver sender, byte c);
        public event ProcessCharHandler ProcessCharEvent;
        public delegate void ProcessEscapeHandler(SerialLowDriver sender, byte code, int[] args);
        public event ProcessEscapeHandler ProcessEscapeEvent;
        public delegate void ProcessPacketHandler(SerialLowDriver sender, int command, byte[] data);
        public event ProcessPacketHandler ProcessPacketEvent;

        public SerialLowDriver(string port)
        {
            this.port = port;
            serial = new SerialPort(port, 115200, Parity.None, 8, StopBits.One);
            serial.Open();
            serial.DtrEnable = true; 

            // Flush the read buffer
            serial.ReadTimeout = 0;
            char[] buf = new char[1024];
            try
            {
                serial.Read(buf, 0, 1024);
            }
            catch (TimeoutException) { }

            // Start the loop thread
            serial.ReadTimeout = SerialPort.InfiniteTimeout;
            serialThread = new Thread(new ThreadStart(SerialLoop));
            serialThread.Start();
        }

        public void Dispose()
        {
            if (serial != null)
            {
                exitting = true;
                while (serialThread.IsAlive) Thread.Sleep(10);
                serialThread = null;
                serial.Close();
                serial.Dispose();
                serial = null;
            }
        }

        public int BytesToWrite
        {
            get
            {
                if (serial == null)
                {
                    return 0;
                }
                else
                {
                    try
                    {
                        return serial.BytesToWrite;
                    }
                    catch (System.IO.IOException)
                    {
                        return 0;
                    }
                }
            }
        }

        private void SerialLoop()
        {
            serial.ReadTimeout = 50;
            try
            {
                byte[] buffer = new byte[200];
                while (exitting == false)
                {
                    int len = 0;
                    try
                    {
                        len = serial.Read(buffer, 0, 200);
                    }
                    catch (TimeoutException) { }
                    if (len != 0)
                    {
                        for (int i = 0; i < len; i++)
                        {
                            if (termstate == 3)
                            {
                                // FAST PATH: We are accumulating bytes, lets handle a block
                                int s = Math.Min(packetAccumulator.Length - packetAccumulatorPtr, len - i);
                                Array.Copy(buffer, i, packetAccumulator, packetAccumulatorPtr, s);
                                packetAccumulatorPtr += s;
                                i += (s - 1);
                                if (packetAccumulatorPtr == packetAccumulator.Length)
                                {
                                    // Packet Completed
                                    if (ProcessPacketEvent != null) ProcessPacketEvent(this, escNumber[0], packetAccumulator);
                                    packetAccumulator = null;
                                    termstate = 0;
                                }
                            }
                            else
                            {
                                // Process this single byte
                                ProcessByte(buffer[i]);
                            }
                        }

                        if (ProcessEscapeEvent != null) ProcessEscapeEvent(this, (byte)'~', null);

                    }
                }
            }
            catch (ThreadInterruptedException) { }
            catch (ThreadAbortException) { }
        }

        private void ProcessByte(byte b)
        {
            switch (termstate)
            {
                case 0: // Normal Term State
                    switch (b)
                    {
                        case 27: // ESC
                            termstate = 1;
                            break;
                        default:
                            // Process a single char
                            if (ProcessCharEvent != null) ProcessCharEvent(this, b);
                            break;
                    }
                    break;
                case 1:
                    switch (b)
                    {
                        case 91:
                            escNumberPtr = 0;
                            Array.Clear(escNumber, 0, 32);
                            termstate = 2;
                            break;
                        default:
                            termstate = 0;
                            break;
                    }
                    break;
                case 2:
                    if (b >= '0' && b <= '9')
                    {
                        // This is a number
                        escNumber[escNumberPtr] = (escNumber[escNumberPtr] * 10) + (b - '0');
                        break;
                    }
                    else if (b == ';')
                    {
                        // New number
                        escNumberPtr++;
                        break;
                    }
                    else if (b == '@')
                    {
                        // New Packet
                        if (escNumber[1] == 0)
                        {
                            // Empty Command
                            if (ProcessPacketEvent != null) ProcessPacketEvent(this, escNumber[0], null);
                            termstate = 0;
                        }
                        else
                        {
                            packetAccumulatorPtr = 0;
                            packetAccumulator = new byte[escNumber[1]];
                            termstate = 3;
                        }
                        break;
                    }
                    else
                    {
                        // Process Escape Sequence
                        int[] vals = new int[escNumberPtr+1];
                        Array.Copy(escNumber, vals, escNumberPtr + 1);
                        if (ProcessEscapeEvent != null) ProcessEscapeEvent(this, b, vals);
                        termstate = 0;
                    }
                    break;
                case 3: // Packet Accumulator
                    packetAccumulator[packetAccumulatorPtr] = b;
                    if (packetAccumulatorPtr == packetAccumulator.Length - 1)
                    {
                        // Packet Completed
                        if (ProcessPacketEvent != null) ProcessPacketEvent(this, escNumber[0], packetAccumulator);
                        packetAccumulator = null;
                        termstate = 0;
                    }
                    else
                    {
                        // More to come
                        packetAccumulatorPtr++;
                    }
                    break;
            }
        }

        public void Write(byte text) { lock (serial) { serial.Write(new byte[] { text }, 0, 1); if (SerialOutputEvent != null) SerialOutputEvent(this, new byte[] { text }, 0, 1); } }
        public void Write(string text) { lock (serial) { serial.Write(text); if (SerialOutputEvent != null) {byte[] buf = UTF8Encoding.UTF8.GetBytes(text); SerialOutputEvent(this, buf, 0, buf.Length); } } }
        public void Write(byte[] buffer) { lock (serial) { serial.Write(buffer, 0, buffer.Length); if (SerialOutputEvent != null) SerialOutputEvent(this, buffer, 0, buffer.Length); } }
        public void Write(byte[] buffer, int offset, int length) { lock (serial) { serial.Write(buffer, offset, length); if (SerialOutputEvent != null) SerialOutputEvent(this, buffer, offset, length); } }
        public void WriteEsc(byte code) { lock (serial) { serial.Write(new byte[3] { 27, 91, code }, 0, 3); } }
        public void WriteEsc(byte code, string args) { lock (serial) { serial.Write("" + (char)27 + (char)91 + args + (char)code); } }
        public void WriteEsc(byte code, int arg) { lock (serial) { serial.Write("" + (char)27 + (char)91 + arg.ToString() + (char)code); } }
        public void WriteEsc(byte code, int arg1, int arg2) { lock (serial) { serial.Write("" + (char)27 + (char)91 + arg1.ToString() + ";" + arg2.ToString() + (char)code); } }
        public void WriteEsc(byte code, int arg1, int arg2, int arg3) { lock (serial) { serial.Write("" + (char)27 + (char)91 + arg1.ToString() + ";" + arg2.ToString() + ";" + arg3.ToString() + (char)code); } }
        public void WritePacket(int command, byte[] data) { WriteEsc((byte)'@', data.Length); Write(data); }

    }
}
