// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Diagnostics;
using System.Net;
using System.Text;
using Intel.Manageability.Cim; 
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.Cim.Untyped;
using Intel.Manageability.Exceptions;
using Intel.Manageability.WSManagement;

namespace ManageabilityStack
{
    public class AmtAuditLog
    {
        private AmtSystem system;
        private AuditLogService service;

        public AmtAuditLog(AmtSystem system, AuditLogService service)
        {
            this.system = system;
            this.service = service;
        }

        public void Dispose()
        {
            if (service != null) service.Dispose();
            service = null;
        }

        public void ClearCache()
        {
        }

        public void FetchCache()
        {
        }

        public enum AmtAuditEventName
        {
            AMTProvisioningStarted = 16000,
            AMTProvisioningCompleted = 16001,
            ACLEntryAdded = 16002,
            ACLEntryModified = 16003,
            ACLEntryRemoved = 16004,
            ACLAccessWithInvalidCredentials = 16005,
            ACLEntryEnabled = 16006,
            TLSStateChanged = 16007,
            TLSServerCertificateSet = 16008,
            TLSServerCertificateRemove = 16009,
            TLSTrustedRootCertificateAdded = 16010,
            TLSTrustedRootCertificateRemoved = 16011,
            TLSPresharedKeySet = 16012,
            KerberosSettingsModified = 16013,
            KerberosMasterKeyModified = 16014,
            FlashWearOutCountersReset = 16015,
            PowerPackageModified = 16016,
            SetRealmAuthenticationMode = 16017,
            PerformedPowerUp = 17000,
            PerformedPowerDown = 17001,
            PerformedPowerCycle = 17002,
            PerformedReset = 17003,
            SetBootOptions = 17004,
            IDERSessionOpened = 18000,
            IDERSessionClosed = 18001,
            IDEREnabled = 18002,
            IDERDisabled = 18003,
            SoLSessionOpened = 18004,
            SoLSessionClosed = 18005,
            SoLEnabled = 18006,
            SoLDisabled = 18007,
            KVMSessionStarted = 18008,
            KVMSessionEnded = 18009,
            KVMEnabled = 18010,
            KVMDisabled = 18011,
            VNCPasswordFailedThreeTimes = 18012,
            FirmwareUpdated = 19000,
            FirmwareUpdatedFailed = 19001,
            SecurityAuditLogCleared = 20000,
            SecurityAuditPolicyModified = 20001,
            SecurityAuditLogDisabled = 20002,
            SecurityAuditLogEnabled = 20003,
            SecurityAuditLogExported = 20004,
            SecurityAuditLogRecovered = 20005,
            AMTTimeSet = 21000,
            TCPIPParametersSet = 22000,
            HostNameSet = 22001,
            DomainNameSet = 22002,
            VLANParametersSet = 22003,
            LinkPolicySet = 22004,
            IPv6ParametersSet = 22005,
            GlobalStorageAttributesSet = 23000,
            StorageEACLModified = 23001,
            StorageFPACLModified = 23002,
            StorageWriteOperation = 23003,
            AlertSubscribed = 24000,
            AlertUnsubscribed = 24001,
            EventLogCleared = 24002,
            EventLogFrozen = 24003,
            CBFilterAdded = 25000,
            CBFilterRemoved = 25001,
            CBPolicyAdded = 25002,
            CBPolicyRemoved = 25003,
            CBDefaultPolicySet = 25004,
            CBHeuristicsOptionSet = 25005,
            CBheuristicsStateCleared = 25006,
            AgentWatchdogAdded = 26000,
            AgentWatchdogRemoved = 26001,
            AgentWatchdogActionSet = 26002,
            WirelessProfileAdded = 27000,
            WirelessProfileRemoved = 27001,
            WirelessProfileUpdated = 27002,
            EACPostureSignerSET = 28000,
            EACEnabled = 28001,
            EACDisabled = 28002,
            EACPostureState = 28003,
            EACSetOptions = 28004,
            KVMOptInEnabled = 29000,
            KVMOptInDisabled = 29001,
            KVMPasswordChanged = 29002,
            KVMConsentSucceeded = 29003,
            KVMConsentFailed = 29004,
        }

        public enum AmtAuditApplicationName
        {
            SecurityAdmin = 16,
            RCO = 17,
            RedirectionManager = 18,
            FirmwareUpdateManager = 19,
            SecurityAuditLog = 20,
            NetworkTime = 21,
            NetworkAdministration = 22,
            StorageAdministration = 23,
            EventManager = 24,
            CircuitBreakerManager = 25,
            AgentPresenceManager = 26,
            WirelessConfiguration = 27,
            EAC = 28,
            KVM = 29,
        }

        /// <summary>
        /// Enumerates the available audit log records storage policy options.
        /// </summary>
        public enum AmtStoragePolicyType
        {
            /// <summary>
            /// No roll-over in audit log storage. Old events will not be
            /// overwritten. When the log is full, critical audit event will be
            /// blocked until a user with audit privileges clears the log.
            /// </summary>
            NO_ROLL_OVER,
            /// <summary>
            /// Full rollover. When the log is full, a new auditable event
            /// will overwrite the oldest record in the log.
            /// </summary>
            ROLL_OVER,
            /// <summary>
            /// Partial (restricted) rollover  only events older than the
            /// threshold set using SetStoragePolicy will be overwritten.
            /// </summary>
            RESTRICTED_ROLL_OVER,
        }

        public AmtCallStatus ClearAuditLog()
        {
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_AuditLog auditLog = new AMT_AuditLog(system.WsMan);
                    return (AmtCallStatus)auditLog.ClearLog();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.ClearAuditLog WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.ClearAuditLog CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.ClearAuditLog error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r;
                try
                {
                    r = (AmtCallStatus)service.ClearAuditLog();
                }
                catch (WebException e)
                {
                    if (e.ToString().Contains("401")) r = AmtCallStatus.NOT_PERMITTED; else r = AmtCallStatus.FAILED_WEB_CALL;
                }
                return r;
            }
        }

        public AmtCallStatus EnableAuditing(bool enabled)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_AuditLog auditLog = new AMT_AuditLog(system.WsMan);
                    ushort requestedState;
                    if (enabled) requestedState = 2; else requestedState = 3;
                    CimReference job;
                    return (AmtCallStatus)auditLog.RequestStateChange(requestedState, null, out job);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.EnableAuditing WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.EnableAuditing CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.EnableAuditing error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r;
                try
                {
                    r = (AmtCallStatus)service.EnableAuditing(enabled);
                }
                catch (WebException e)
                {
                    if (e.ToString().Contains("401")) r = AmtCallStatus.NOT_PERMITTED; else r = AmtCallStatus.FAILED_WEB_CALL;
                }
                return r;
            }
        }

        public AmtCallStatus ExportAuditLogSignature(uint SigningMechanisum, out AuditSignatureType auditSignature)
        {
            auditSignature = null;
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    CimDateTime startLogTime, endLogTime, generationTime;
                    uint totalRecordCount, signatureMechanism;
                    string uuid, fqdn;
                    byte[] signature, certificates;
                    ushort[] lengthOfCertificates;
                    AMT_AuditLog auditLog = new AMT_AuditLog(system.WsMan);
                    AmtCallStatus r = (AmtCallStatus)auditLog.ExportAuditLogSignature(SigningMechanisum,
                        out totalRecordCount, out startLogTime, out endLogTime, out generationTime, out uuid,
                        out fqdn, out signatureMechanism, out signature, out lengthOfCertificates, out certificates);
                    if (r != AmtCallStatus.SUCCESS) return r;

                    auditSignature = new AuditSignatureType();
                    auditSignature.TotalRecordCount = totalRecordCount;
                    auditSignature.UUID = Encoding.Default.GetBytes(uuid);
                    auditSignature.FQDN = fqdn;
                    auditSignature.SignatureMechanism = signatureMechanism;
                    auditSignature.Signature = signature;
                    auditSignature.LengthOfCertificates = lengthOfCertificates;
                    auditSignature.Certificates = certificates;
                    //auditSignature.StartLogTime = DateTime.Parse(startLogTime.ToString());
                    //auditSignature.EndLogTime = DateTime.Parse(endLogTime.ToString());
                    //auditSignature.GenerationTime = DateTime.Parse(generationTime.ToString());

                    return r;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.ExportAuditLogSignature WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.ExportAuditLogSignature CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.ExportAuditLogSignature error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r;
                try
                {
                    r = (AmtCallStatus)service.ExportAuditLogSignature(SigningMechanisum, out auditSignature);
                }
                catch (WebException e)
                {
                    auditSignature = null;
                    if (e.ToString().Contains("401")) r = AmtCallStatus.NOT_PERMITTED; else r = AmtCallStatus.FAILED_WEB_CALL;
                }
                return r;
            }
        }

        public AmtCallStatus LockAuditLog(uint lockTimeinSeconds, out uint handle)
        {
            handle = 0;
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_AuditLog auditLog = new AMT_AuditLog(system.WsMan);
                    return (AmtCallStatus)auditLog.SetAuditLock(lockTimeinSeconds, 0, null, out handle);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.LockAuditLog WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.LockAuditLog CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.LockAuditLog error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                uint h = 0;
                bool hs = false;
                AmtCallStatus r;
                try
                {
                    r = (AmtCallStatus)service.LockAuditLog(lockTimeinSeconds, true, 0, ref h, ref hs);
                }
                catch (WebException e)
                {
                    if (e.ToString().Contains("401")) r = AmtCallStatus.NOT_PERMITTED; else r = AmtCallStatus.FAILED_WEB_CALL;
                }
                handle = h;
                return r;
            }
        }

        public AmtCallStatus LockAuditLogUnprovisioning(uint lockTimeinSeconds, out uint handle)
        {
            handle = 0;
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_AuditLog auditLog = new AMT_AuditLog(system.WsMan);
                    return (AmtCallStatus)auditLog.SetAuditLock(lockTimeinSeconds, 1, null, out handle);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.LockAuditLogUnprovisioning WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.LockAuditLogUnprovisioning CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.LockAuditLogUnprovisioning error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                uint h = 0;
                bool hs = false;
                AmtCallStatus r;
                try
                {
                    r = (AmtCallStatus)service.LockAuditLog(lockTimeinSeconds, true, 1, ref h, ref hs);
                }
                catch (WebException e)
                {
                    if (e.ToString().Contains("401")) r = AmtCallStatus.NOT_PERMITTED; else r = AmtCallStatus.FAILED_WEB_CALL;
                }
                handle = h;
                return r;
            }
        }

        public AmtCallStatus UnLockAuditLog(uint handle)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_AuditLog auditLog = new AMT_AuditLog(system.WsMan);
                    uint outHandle;
                    return (AmtCallStatus)auditLog.SetAuditLock(null, 2, handle, out outHandle);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.UnLockAuditLog WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.UnLockAuditLog CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.UnLockAuditLog error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                uint h = handle;
                bool hs = true;
                AmtCallStatus r;
                try
                {
                    r = (AmtCallStatus)service.LockAuditLog(0, false, 2, ref h, ref hs);
                }
                catch (WebException e)
                {
                    if (e.ToString().Contains("401")) r = AmtCallStatus.NOT_PERMITTED; else r = AmtCallStatus.FAILED_WEB_CALL;
                }
                return r;
            }
        }

        public AmtCallStatus SetSigningKeyingMaterial(AuditKeyingMaterialType keyAndCert)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_AuditLog auditLog = new AMT_AuditLog(system.WsMan);
                    return (AmtCallStatus)auditLog.SetSigningKeyMaterial(keyAndCert.SignatureMechanism,
                        keyAndCert.SigningKey.Item.DERKey, keyAndCert.LengthOfCertificates, keyAndCert.Certificates);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.SetSigningKeyingMaterial WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.SetSigningKeyingMaterial CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.SetSigningKeyingMaterial error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r;
                try
                {
                    r = (AmtCallStatus)service.SetSigningKeyingMaterial(keyAndCert);
                }
                catch (WebException e)
                {
                    if (e.ToString().Contains("401")) r = AmtCallStatus.NOT_PERMITTED; else r = AmtCallStatus.FAILED_WEB_CALL;
                }
                return r;
            }
        }

        public AmtCallStatus SetAuditPolicy(bool enable, AuditPolicyType auditPolicy)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_AuditPolicyRule auditPolicyRule = new AMT_AuditPolicyRule(system.WsMan);
                    return (AmtCallStatus)auditPolicyRule.SetAuditPolicy(enable, auditPolicy.AuditedAppID, auditPolicy.EventId, auditPolicy.Flag);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.SetAuditPolicy WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.SetAuditPolicy CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.SetAuditPolicy error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r;
                try
                {
                    r = (AmtCallStatus)service.SetAuditPolicy(enable, auditPolicy);
                }
                catch (WebException e)
                {
                    if (e.ToString().Contains("401")) r = AmtCallStatus.NOT_PERMITTED; else r = AmtCallStatus.FAILED_WEB_CALL;
                }
                return r;
            }
        }

        public AmtCallStatus GetStoragePolicy(out AmtStoragePolicyType policy, out UInt32 threshold)
        {
            policy = AmtStoragePolicyType.NO_ROLL_OVER;
            threshold = 1;

            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_AuditLog auditLog = new AMT_AuditLog(system.WsMan);
                    auditLog.Get();

                    if (auditLog.StoragePolicyExist)
                    {
                        policy = (AmtStoragePolicyType)auditLog.StoragePolicy;
                    }
                    if (auditLog.MinDaysToKeepExist)
                    {
                        threshold = auditLog.MinDaysToKeep;
                    }

                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.GetStoragePolicy WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.GetStoragePolicy CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "GetStoragePolicy error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r;
                uint storagePolicy;
                bool thresholdSpecified;
                try
                {
                    r = (AmtCallStatus)service.GetAuditLogStoragePolicy(out storagePolicy, out threshold, out thresholdSpecified);
                    if (r == AmtCallStatus.SUCCESS)
                        policy = (AmtStoragePolicyType)storagePolicy;
                }
                catch (WebException e)
                {
                    if (e.ToString().Contains("401")) r = AmtCallStatus.NOT_PERMITTED; else r = AmtCallStatus.FAILED_WEB_CALL;
                }
                return r;
            }
        }

        public AmtCallStatus SetStoragePolicy(AmtStoragePolicyType policy, UInt32 threshold)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_AuditLog auditLog = new AMT_AuditLog(system.WsMan);
                    auditLog.Get();
                    auditLog.StoragePolicy = (byte)policy;
                    auditLog.MinDaysToKeep = (byte)threshold;
                    auditLog.Put();
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.SetStoragePolicy WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.SetStoragePolicy CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAuditLog.SetStoragePolicy error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                return (AmtCallStatus)service.SetAuditLogStoragePolicy((uint)policy, (uint)threshold, (policy == AmtStoragePolicyType.RESTRICTED_ROLL_OVER) ? true : false);
            }
        }
    }
}
