// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Diagnostics;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;

namespace ManageabilityStack
{
    public class AmtCircuitBreaker
    {
        private AmtSystem system;
        private CircuitBreakerService service;

        public AmtCircuitBreaker(CircuitBreakerService service, AmtSystem system)
        {
            this.service = service;
            this.system = system;
        }

        public void Dispose()
        {
            service = null;
        }

        public void FetchCache()
        {
            try
            {
                uint a, b;
                GetCircuitBreakerCapabilities(out a, out b);
                GetCircuitBreakerFilters();
                GetCircuitBreakerPolicies();
                GetCircuitBreakerEnabledPolicies();
                GetHcbOptions(0);
                GetHcbState(0);
            }
            catch (Exception e)
            {
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Failure to retrieve the Circuit Breaker data.",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
            }
        }

        public void ClearCache()
        {
            CircuitBreakerCapabilitiesCache = null;
            CircuitBreakerFiltersCache = null;
            CircuitBreakerPoliciesCache = null;
            CircuitBreakerEnabledPolicies = null;
            HcbOptionsCache.Clear();
            HcbStateCache.Clear();
        }

        public void ClearFiltersCache()
        {
            CircuitBreakerFiltersCache = null;
        }

        public void ClearPoliciesCache()
        {
            CircuitBreakerEnabledPolicies = null;
            CircuitBreakerPoliciesCache = null;
        }

        public void ClearHcbCache()
        {
            HcbOptionsCache.Clear();
            HcbStateCache.Clear();
        }

        private CircuitBreakerCapabilitiesType[] CircuitBreakerCapabilitiesCache = null;
        private uint MaxSupportedPoliciesCache = 0;
        private uint MaxSupportedFiltersCache = 0;
        public CircuitBreakerCapabilitiesType[] GetCircuitBreakerCapabilities(out uint MaxSupportedPolicies, out uint MaxSupportedFilters)
        {
            MaxSupportedPolicies = 0;
            MaxSupportedFilters = 0;
            if (CircuitBreakerCapabilitiesCache != null)
            {
                MaxSupportedPolicies = MaxSupportedPoliciesCache;
                MaxSupportedFilters = MaxSupportedFiltersCache;
                return CircuitBreakerCapabilitiesCache;
            }

            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AMT_GeneralSystemDefenseCapabilities s1 = new AMT_GeneralSystemDefenseCapabilities(system.WsMan);
                    s1.Get();
                    MaxSupportedPolicies = MaxSupportedPoliciesCache = s1.GlobalMaxSupportedPolicies;
                    MaxSupportedFilters = MaxSupportedFiltersCache = s1.GlobalMaxSupportedFilters;

                    Collection<AMT_NetworkPortSystemDefenseCapabilities> s2 = AMT_NetworkPortSystemDefenseCapabilities.Enumerate(system.WsMan);

                    CircuitBreakerCapabilitiesCache = new CircuitBreakerCapabilitiesType[s2.Count];
                    uint i = 0;
                    foreach (AMT_NetworkPortSystemDefenseCapabilities s in s2)
                    {
                        CircuitBreakerCapabilitiesCache[i] = new CircuitBreakerCapabilitiesType();
                        if (i == 0) CircuitBreakerCapabilitiesCache[i].HardwareDescription = "Wired"; // TODO
                        if (i == 1) CircuitBreakerCapabilitiesCache[i].HardwareDescription = "Wireless"; // TODO
                        CircuitBreakerCapabilitiesCache[i].HardwareID = i; // TODO
                        CircuitBreakerCapabilitiesCache[i].IPv4_AntiSpoofingCapable = s.IPv4_AntiSpoofingCapable;
                        CircuitBreakerCapabilitiesCache[i].IPv4_MaxRxCounters = s.IPv4_MaxRxCounters;
                        CircuitBreakerCapabilitiesCache[i].IPv4_MaxRxFilters = s.IPv4_MaxRxFilters;
                        CircuitBreakerCapabilitiesCache[i].IPv4_MaxTxCounters = s.IPv4_MaxTxCounters;
                        CircuitBreakerCapabilitiesCache[i].IPv4_MaxTxFilters = s.IPv4_MaxTxFilters;
                        CircuitBreakerCapabilitiesCache[i].IPv6_AntiSpoofingCapable = s.IPv6_AntiSpoofingCapable;
                        CircuitBreakerCapabilitiesCache[i].IPv6_MaxRxCounters = s.IPv6_MaxRxCounters;
                        CircuitBreakerCapabilitiesCache[i].IPv6_MaxRxFilters = s.IPv6_MaxRxFilters;
                        CircuitBreakerCapabilitiesCache[i].IPv6_MaxTxCounters = s.IPv6_MaxTxCounters;
                        CircuitBreakerCapabilitiesCache[i].IPv6_MaxTxFilters = s.IPv6_MaxTxFilters;
                        CircuitBreakerCapabilitiesCache[i].IPv6_To_IPv4_Ratio = s.IPv6_to_IPv4_Ratio;
                        i++;
                    }
                    return CircuitBreakerCapabilitiesCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtCircuitBreaker.GetCircuitBreakerCapabilities WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtCircuitBreaker.GetCircuitBreakerCapabilities CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtCircuitBreaker.GetCircuitBreakerCapabilities error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    CircuitBreakerCapabilitiesCache = null;
                    return null;
                }
            }
            else
            {
                // Use EOI
                CircuitBreakerCapabilitiesType[] Capabilities;
                MaxSupportedPolicies = 0;
                MaxSupportedFilters = 0;

                AmtCallStatus r = (AmtCallStatus)service.CbQueryCapabilities(out MaxSupportedPolicies, out MaxSupportedFilters, out Capabilities);

                MaxSupportedPoliciesCache = MaxSupportedPolicies;
                MaxSupportedFiltersCache = MaxSupportedFilters;
                CircuitBreakerCapabilitiesCache = Capabilities;
                return CircuitBreakerCapabilitiesCache;
            }
        }

        private CircuitBreakerFilterInfoType[] CircuitBreakerFiltersCache = null;
        public CircuitBreakerFilterInfoType[] GetCircuitBreakerFilters()
        {
            if (CircuitBreakerFiltersCache != null) return CircuitBreakerFiltersCache;
            
            AmtCallStatus r;
            /*
            if (system.WsMan != null)
            {
                // USE WSMAN
                ArrayList l = new ArrayList();

                // Fetch Ethernet Filters
                AMT_Hdr8021FilterType[] filters = (AMT_Hdr8021FilterType[])system.WsMan.Enumerate(typeof(AMT_Hdr8021FilterType));
                if (filters != null && filters.Length > 0)
                {
                    foreach (AMT_Hdr8021FilterType filter in filters)
                    {
                        CircuitBreakerFilterInfoType t = new CircuitBreakerFilterInfoType();
                        t.FilterCreationHandle = filter.InstanceID;
                        t.Filter = new CircuitBreakerFilterType();
                        t.Filter.ActionEventOnMatch = filter.ActionEventOnMatch;
                        t.Filter.FilterDirection = filter.FilterDirection;
                        t.Filter.FilterName = filter.Name;
                        t.Filter.FilterProfile = filter.FilterProfile;
                        t.Filter.FilterProfileData = filter.FilterProfileData;
                        t.Filter.FilterPacket = new CircuitBreakerPacketType();
                        t.Filter.FilterPacket.Item = new CircuitBreakerPacketETHType();
                        ((CircuitBreakerPacketETHType)t.Filter.FilterPacket.Item).EthernetFrameType = filter.HdrProtocolID8021;
                        l.Add(t);
                    }
                }

                // Fetch IP Header Filters
                AMT_IPHeadersFilterType[] filters2 = (AMT_IPHeadersFilterType[])system.WsMan.Enumerate(typeof(AMT_IPHeadersFilterType));
                if (filters2 != null && filters2.Length > 0)
                {
                    foreach (AMT_IPHeadersFilterType filter in filters2)
                    {
                        CircuitBreakerFilterInfoType t = new CircuitBreakerFilterInfoType();
                        t.FilterCreationHandle = filter.InstanceID;
                        t.Filter = new CircuitBreakerFilterType();
                        t.Filter.ActionEventOnMatch = filter.ActionEventOnMatch;
                        t.Filter.FilterDirection = filter.FilterDirection;
                        t.Filter.FilterName = filter.Name;
                        t.Filter.FilterProfile = filter.FilterProfile;
                        t.Filter.FilterProfileData = filter.FilterProfileData;
                        t.Filter.FilterPacket = new CircuitBreakerPacketType();
                        t.Filter.FilterPacket.Item = filter;
                        l.Add(t);
                    }
                }

                CircuitBreakerFiltersCache = (CircuitBreakerFilterInfoType[])l.ToArray(typeof(CircuitBreakerFilterInfoType));
            }
            else
            {
            */
                // Use SOAP
                r = (AmtCallStatus)service.CbFilterEnumerate(out CircuitBreakerFiltersCache);
            //}

            return CircuitBreakerFiltersCache;
        }

        public bool IsHeuristicSupported()
        {
            return (system.AmtRev == 3 || system.AmtRev == 5);
        }

        public string GetFilterName(uint handle)
        {
            if (handle == 4294967293)
            {
                return ManageabilityStack.Properties.Resources.CircuitBreakerAllRecPac;
                //return "All Received Packets";
            }
            else if (handle == 4294967292)
            {
                return ManageabilityStack.Properties.Resources.CircuitBreakerAllSentPac;
                //return "All Sent Packets";
            }
            else if (handle == 4294967290)
            {
                return ManageabilityStack.Properties.Resources.CircuitBreakerSpoofedPac;
                //return "Spoofed Packets";
            }
            else
            {
                Hashtable t = GetCircuitBreakerFiltersTable();
                if (t.ContainsKey(handle))
                {
                    return (string)(t[handle]);
                }
                else
                {
                    return ManageabilityStack.Properties.Resources.CircuitBreakerNotPre;
                    //return "Not Present";
                }
            }
        }

        public Hashtable GetCircuitBreakerFiltersTable()
        {
            if (CircuitBreakerFiltersCache == null) GetCircuitBreakerFilters();
            if (CircuitBreakerFiltersCache == null) return null;
            Hashtable t = new Hashtable();
            foreach (CircuitBreakerFilterInfoType f in CircuitBreakerFiltersCache)
            {
                t.Add(f.FilterCreationHandle, f.Filter.FilterName);
            }
            return t;
        }

        [Serializable]
        public class TotalCircuitBreakerPolicy
        {
            public CircuitBreakerPolicyType Policy;
            public ArrayList FilterNames;

            public TotalCircuitBreakerPolicy()
            {
                Policy = null;
                FilterNames = new ArrayList();
            }

            public TotalCircuitBreakerPolicy(AmtSystem computer, CircuitBreakerPolicyType policy)
            {
                Policy = policy;
                FilterNames = new ArrayList();
                if (policy.FilterCreationHandles != null)
                {
                    foreach (uint fh in policy.FilterCreationHandles)
                    {
                        FilterNames.Add(computer.CircuitBreaker.GetFilterName(fh));
                    }
                }
            }

            public uint[] GetFilterhandles(AmtSystem computer)
            {
                CircuitBreakerFilterInfoType[] filters = computer.CircuitBreaker.GetCircuitBreakerFilters();
                ArrayList l = new ArrayList();
                foreach (CircuitBreakerFilterInfoType filter in filters)
                {
                    if (FilterNames.Contains(filter.Filter.FilterName)) l.Add(filter.FilterCreationHandle);
                }
                return (uint[])l.ToArray(typeof(uint));
            }
        }

        private CircuitBreakerPolicyInfoType[] CircuitBreakerPoliciesCache = null;
        public CircuitBreakerPolicyInfoType[] GetCircuitBreakerPolicies()
        {
            if (CircuitBreakerPoliciesCache != null) return CircuitBreakerPoliciesCache;
            CircuitBreakerPolicyInfoType[] r;
            service.CbPolicyEnumerate(out r);
            CircuitBreakerPoliciesCache = r;
            return CircuitBreakerPoliciesCache;
        }

        public CircuitBreakerPolicyInfoType GetCircuitBreakerPolicy(uint handle)
        {
            CircuitBreakerPolicyInfoType[] policies = GetCircuitBreakerPolicies();
            if (policies == null) return null;
            foreach (CircuitBreakerPolicyInfoType policy in policies)
            {
                if (policy.PolicyCreationHandle == handle) return policy;
            }
            return null;
        }

        private static CircuitBreakerHardwarePolicyType[] CircuitBreakerEnabledPolicies = null;
        private CircuitBreakerHardwarePolicyType[] GetCircuitBreakerEnabledPolicies()
        {
            if (CircuitBreakerEnabledPolicies != null) return CircuitBreakerEnabledPolicies;
            CircuitBreakerHardwarePolicyType[] r;
            service.CbPolicyGetEnabled(0, false, out r);
            CircuitBreakerEnabledPolicies = r;
            return CircuitBreakerEnabledPolicies;
        }

        // Return the list of HardwareID on which a policy handle is active
        public uint[] IsPolicyActive(uint policyHandle)
        {
            if (CircuitBreakerEnabledPolicies == null) CircuitBreakerEnabledPolicies = GetCircuitBreakerEnabledPolicies();
            if (CircuitBreakerEnabledPolicies == null) return null;

            uint[] l = new uint[10];
            uint len = 0;
            foreach (CircuitBreakerHardwarePolicyType ep in CircuitBreakerEnabledPolicies)
            {
                if (ep.PolicyCreationHandle == policyHandle)
                {
                    l[len] = ep.HardwareID;
                    len++;
                }
            }

            if (len == 0) return null;

            uint[] l2 = new uint[len];
            for (int i = 0; i < len; i++) { l2[i] = l[i]; }

            return l2;
        }

        // Return true if this policy is active on that hardware
        public bool IsPolicyActive(uint policyHandle, uint hardwareid)
        {
            if (CircuitBreakerEnabledPolicies == null) CircuitBreakerEnabledPolicies = GetCircuitBreakerEnabledPolicies();
            if (CircuitBreakerEnabledPolicies == null) return false;
            foreach (CircuitBreakerHardwarePolicyType ep in CircuitBreakerEnabledPolicies)
            {
                if (ep.PolicyCreationHandle == policyHandle && ep.HardwareID == hardwareid) return true;
            }
            return false;
        }

        public void CircuitBreakerEnablePolicies(CircuitBreakerHardwarePolicyType[] enablePolicies)
        {
            CircuitBreakerHardwarePolicyType[] rr;
            AmtCallStatus r = (AmtCallStatus)service.CbPolicyEnable(enablePolicies, out rr);
            CircuitBreakerEnabledPolicies = enablePolicies;
        }

        public void CircuitBreakerEnablePolicy(uint policyHandle, uint hardwareId)
        {
            CircuitBreakerHardwarePolicyType[] r;
            CircuitBreakerHardwarePolicyType[] s = new CircuitBreakerHardwarePolicyType[1];
            s[0] = new CircuitBreakerHardwarePolicyType();
            s[0].HardwareID = hardwareId;
            s[0].PolicyCreationHandle = policyHandle;

            service.CbPolicyEnable(s, out r);
            CircuitBreakerEnabledPolicies = r;
        }

        public void CircuitBreakerDisablePolicy()
        {
            CircuitBreakerEnabledPolicies = null;
            service.CbPolicyDisable(0, false);
            return;
        }

        public void CircuitBreakerDisablePolicy(uint hardwareId)
        {
            CircuitBreakerEnabledPolicies = null;
            service.CbPolicyDisable(hardwareId, true);
            return;
        }

        public uint CircuitBreakerCreateFilter(CircuitBreakerFilterType filter)
        {
            CircuitBreakerFiltersCache = null;
            uint filterhandle = 0;
            service.CbFilterCreate(filter, out filterhandle);
            return filterhandle;
        }

        public void CircuitBreakerRemoveFilter(uint handle)
        {
            CircuitBreakerFiltersCache = null;
            service.CbFilterDelete(handle);
        }

        public uint CircuitBreakerCreatePolicy(CircuitBreakerPolicyType policy)
        {
            CircuitBreakerEnabledPolicies = null;
            CircuitBreakerPoliciesCache = null;
            uint policyhandle = uint.MaxValue;
            service.CbPolicyCreate(policy, out policyhandle);
            return policyhandle;
        }

        public void CircuitBreakerRemovePolicy(uint handle)
        {
            CircuitBreakerEnabledPolicies = null;
            CircuitBreakerPoliciesCache = null;
            service.CbPolicyDelete(handle);
        }

        public CircuitBreakerFilterStatisticsType[] CircuitBreakerGetActiveStatistics(uint hardwareid, bool reset, out uint PolicyCreationHandle, out uint ActivationTime, out uint LastResetTime)
        {
            /*
            if (system.WsMan != null)
            {
                AMT_ActiveFilterStatisticsType[] Statistics = (AMT_ActiveFilterStatisticsType[])system.WsMan.Enumerate(typeof(AMT_ActiveFilterStatisticsType));
                //Statistics[0].

            }
            else
            {
            */

            PolicyCreationHandle = 0;
            ActivationTime = 0;
            LastResetTime = 0;
            try
            {
                CircuitBreakerFilterStatisticsType[] Statistics;

                service.CbPolicyGetActiveStatistics(hardwareid, reset, out PolicyCreationHandle, out ActivationTime, out LastResetTime, out Statistics);
                return Statistics;
            }
            catch (System.Net.WebException)
            {
                system.Disconnect();
                return null;
            }
        }

        private Hashtable HcbOptionsCache = new Hashtable();
        public HcbOptionsType GetHcbOptions(uint InterfaceHandle)
        {
            if (!IsHeuristicSupported()) return null;
            if (HcbOptionsCache.ContainsKey(InterfaceHandle)) return (HcbOptionsType)HcbOptionsCache[InterfaceHandle];
            HcbOptionsType HcbOption;
            AmtCallStatus r = (AmtCallStatus)service.GetHcbOptions(InterfaceHandle, out HcbOption);
            HcbOptionsCache[InterfaceHandle] = HcbOption;
            return HcbOption;
        }

        public AmtCallStatus SetHcbOptions(uint InterfaceHandle, HcbOptionsType HcbOption)
        {
            if (!IsHeuristicSupported()) return AmtCallStatus.NOP;
            AmtCallStatus r = (AmtCallStatus)service.SetHcbOptions(InterfaceHandle, HcbOption);
            ClearHcbCache();
            return r;
        }

        public class AmtHcbState
        {
            public uint InterfaceHandle;
            public ushort HcbState;
            public bool BlockedAll;
            public BlockedPortInfoType BlockedPortInfo;
            public uint EnabledCbPolicy;
            public bool EnabledCbPolicySpecified;
            public uint TimeStamp;
            public bool TimeStampSpecified;
            public ushort HcbTriggerReason;
            public bool HcbTriggerReasonSpecified;
        }

        private Hashtable HcbStateCache = new Hashtable();
        public AmtHcbState GetHcbState(uint InterfaceHandle)
        {
            if (!IsHeuristicSupported()) return null;
            if (HcbStateCache.ContainsKey(InterfaceHandle)) return (AmtHcbState)HcbStateCache[InterfaceHandle];
            AmtHcbState state = new AmtHcbState();
            state.InterfaceHandle = InterfaceHandle;
            AmtCallStatus r = (AmtCallStatus)service.GetHcbState(state.InterfaceHandle, out state.HcbState, out state.BlockedAll, out state.TimeStamp, out state.TimeStampSpecified, out state.HcbTriggerReason, out state.HcbTriggerReasonSpecified, out state.BlockedPortInfo, out state.EnabledCbPolicy, out state.EnabledCbPolicySpecified);
            HcbStateCache[InterfaceHandle] = state;
            return state;
        }

        public AmtCallStatus ClearHcbState(uint InterfaceHandle)
        {
            if (!IsHeuristicSupported()) return AmtCallStatus.NOP;
            HcbStateCache.Remove(InterfaceHandle);
            AmtCallStatus r = (AmtCallStatus)service.ClearHcbState(InterfaceHandle);
            return r;
        }

    }
}
