// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Xml;
using System.Data;
using System.Text;
using System.Drawing;
using System.Reflection;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;

namespace ManageabilityStack
{
    public partial class AmtDebugForm : Form
    {
        private static ArrayList events = new ArrayList();
        private delegate void EventHandler(LogEventClass e);
        private static event EventHandler OnLogEvent;
        private static AmtDebugForm amtDebugForm;
        private static FileStream OutputFileStream = null;

        public enum EventType
        {
            Information,
            Event,
            Warning,
            Error,
            CallAttempt,
            CallCompleted,
            CallWarning,
            CallFailed,
        }

        public class LogEventClass
        {
            public DateTime time;
            public EventType type;
            public string header;
            public string body;

            public LogEventClass(EventType eventType, string header, string body)
            {
                this.time = DateTime.Now;
                this.type = eventType;
                this.header = header;
                this.body = body;
            }

            public string ToLogFileString()
            {
                if (body == null)
                {
                    return string.Format("-> [{0}] {1}: {2}\r\n",
                        time.ToLongTimeString(), type.ToString(), header);
                }
                else
                {
                    return string.Format("-> [{0}] {1}: {2}\r\n{3}\r\n",
                        time.ToLongTimeString(), type.ToString(), header, body);
                }
            }

            private static int[] iconarray = new int[] { 0, 0, 1, 2, 4, 0, 1, 2 };
            public int GetIcon()
            {
                return iconarray[(int)type];
            }
        }

        public static void GenerateExceptionFile(string filename, string exception, string versionInfo)
        {
            FileStream fs = new FileStream(filename, FileMode.Append);
            StringBuilder s = new StringBuilder();
            if (fs.Length == 0) s.Append(Properties.Resources.Exception_Sink).Append("\r\n\r\n");
            s.Append("--- BEGIN LOG ENTRY ---\r\n\r\n");
            s.AppendFormat("{0}\r\nVersion: {1}\r\n\r\n", DateTime.Now.ToString(), versionInfo);
            s.AppendFormat("ERROR: {0}\r\n\r\n", exception);
            s.AppendFormat("RECENT EVENTS\r\n-------------\r\n\r\n{0}\r\n", GetLastEvents(10));
            s.Append("--- END LOG ENTRY ---\r\n\r\n");
            byte[] bytes = UTF8Encoding.UTF8.GetBytes(s.ToString());
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();
            MessageBox.Show(string.Format(Properties.Resources.ExceptionLoggedIntoFile.Replace("\\r\\n", "\r\n"), fs.Name, Properties.Resources.Exception_Sink), Properties.Resources.ApplicationException, MessageBoxButtons.OK, MessageBoxIcon.Error);
        }

        public static string GetLastEvents(int count)
        {
            StringBuilder s = new StringBuilder();
            lock (events)
            {
                int x = Math.Min(count, events.Count);
                for (int i = 0; i < x; i++)
                {
                    s.Append(((LogEventClass)events[i]).ToLogFileString());
                }
            }
            return s.ToString();
        }

        public static void LogEvent(object obj, EventType eventType, string header)
        {
            LogEvent(obj, eventType, header, null);
        }

        public static void LogEvent(object obj, EventType eventType, string header, string body)
        {
            LogEventClass e = new LogEventClass(eventType, header, body);

            lock (events)
            {
                events.Insert(0, e);
                while (events.Count > 200) events.RemoveAt(200);
            }

            try
            {
                if (OnLogEvent != null) OnLogEvent(e);
            }
            catch (Exception) { }

            if (OutputFileStream != null)
            {
                byte[] bytes = UTF8Encoding.UTF8.GetBytes(e.ToLogFileString());
                OutputFileStream.Write(bytes, 0, bytes.Length);
                OutputFileStream.Flush(); // Must flush each time since we may crash at anytime during logging.
            }
        }

        public static void LogEventWithIntrospection(object obj, EventType eventType, string header, object introspection)
        {
            LogEvent(obj, eventType, header, ObjectToString("", introspection));
        }

        public static void LogHttpCall(EventType type, string url, string body)
        {
            LogEvent(null, type, url, body);
        }

        public static void LogWsManCall(object sender, string call, object o)
        {
            if (o == null) return;
            Type t = o.GetType();
            if (t.IsArray == true)
            {
                StringBuilder a = new StringBuilder("ARRAY\r\n{\r\n");
                Array o2 = (Array)o;
                for (int i = 0; i < o2.GetLength(0); i++)
                {
                    a.Append(WsManObjectToString("  ", o2.GetValue(i)));
                    a.Append("\r\n");
                }
                a.Append("}");
                LogEvent(null, EventType.CallCompleted, call, a.ToString());
            }
            else
            {
                LogEvent(null, EventType.CallCompleted, call, string.Format("{0}\r\n", WsManObjectToString("", o)));
            }
        }
    
        public static void LogWebCall(string url, string call, object[] callparams, object[] returnparams)
        {
            EventType t = EventType.CallCompleted;

            if (returnparams != null && returnparams.Length > 0 && returnparams[0].GetType() == typeof(uint))
            {
                returnparams = (object[])(returnparams.Clone());
                returnparams[0] = (AmtCallStatus)((uint)returnparams[0]);
                call += " - " + ((AmtCallStatus)returnparams[0]).ToString();
            }

            if (returnparams == null)
            {
                t = EventType.CallAttempt;
            }

            string body = "";

            int i;
            if (callparams != null)
            {
                i = 1;
                foreach (object o in callparams) body += string.Format("Call Param {0}: {1}\r\n", i++, ObjectToString("", o));
            }
            if (returnparams != null)
            {
                i = 1;
                foreach (object o in returnparams) body += string.Format("Return Param {0}: {1}\r\n", i++, ObjectToString("", o));
            }

            LogEvent(null, t, call, body);
        }

        public static string ObjectToString(string ident, object o)
        {
            if (o == null) return "Null";
            Type t = o.GetType();
            if (t.IsGenericType == true) return o.ToString();
            if (t.IsPrimitive == true) return o.ToString();
            if (t.IsEnum == true) return o.ToString();
            if (t == typeof(string)) return "\"" + o.ToString() + "\"";

            if (t == typeof(byte[]) || t == typeof(int[]) || t == typeof(uint[]))
            {
                string a = "ARRAY { ";
                Array o2 = (Array)o;
                for (int i = 0; i < o2.GetLength(0); i++)
                {
                    if (i != 0) a += ",";
                    a += o2.GetValue(i).ToString();
                }
                a += " }";
                return a;
            }

            if (t.IsArray == true)
            {
                string a = "ARRAY\r\n";
                a += ident + "{\r\n";
                Array o2 = (Array)o;
                for (int i = 0;i < o2.GetLength(0); i ++)
                {
                    a += ObjectToString(ident + "  ", o2.GetValue(i)) + "\r\n";
                }
                a += ident + "}";
                return a;
            }

            StringBuilder s = new StringBuilder();
            s.AppendFormat("CLASS \"{0}\"\r\n{1}", t.Name, ident);
            s.Append("{\r\n");
            FieldInfo[] fields = t.GetFields();
            foreach (FieldInfo f in fields) s.AppendFormat("{0}  {1} = {2}\r\n", ident, f.Name, ObjectToString(ident + "  ", f.GetValue(o)));
            PropertyInfo[] props = t.GetProperties();
            foreach (PropertyInfo p in props)
            {
                try
                {
                    s.AppendFormat("{0}  {1} = {2}\r\n", ident, p.Name, ObjectToString(ident + "  ", p.GetValue(o, null)));
                }
                catch (System.Reflection.TargetInvocationException)
                {
                    s.AppendFormat("{0}  {1} = (???)\r\n", ident, p.Name);
                }
            }
            s.Append(ident).Append("}");
            return s.ToString();
        }

        private static string WsManObjectToString(string ident, object o)
        {
            FieldInfo[] fields = o.GetType().GetFields();
            StringBuilder sb = new StringBuilder();
            foreach (FieldInfo f in fields)
            {
                if (f.Name != null)
                {
                    if (f.Name.EndsWith("Specified") == false)
                    {
                        string value = Properties.Resources.Null;
                        if (f.GetValue(o) != null) value = f.GetValue(o).ToString();

                        foreach (FieldInfo ff in fields)
                        {
                            if (ff.Name != null && ff.Name.Equals(f.Name + "Specified") && f.GetValue(o) != null && f.GetValue(o).ToString().Equals(bool.FalseString))
                            {
                                value = Properties.Resources.NotSpecified;
                            }
                        }

                        if (value.Equals("System.Byte[]"))
                        {
                            byte[] buf = (byte[])f.GetValue(o);
                            value = string.Format(Properties.Resources.DisplaySizeAndHex, buf.Length.ToString(), Utils.BytesToHex(buf, 0, buf.Length));
                        }
                        else if (value.Equals("System.UInt16[]"))
                        {
                            UInt16[] nums = (UInt16[])f.GetValue(o);
                            value = "";
                            foreach (UInt16 num in nums) value += ", " + num.ToString();
                            if (value.Length > 2) value = value.Substring(2);
                        }
                        else if (value.Equals("System.UInt32[]"))
                        {
                            UInt32[] nums = (UInt32[])f.GetValue(o);
                            value = "";
                            foreach (UInt32 num in nums) value += ", " + num.ToString();
                            if (value.Length > 2) value = value.Substring(2);
                        }
                        //else if (value.Equals("MOF_CLASSES.EndpointReferenceType"))
                        //{
                        //    MOF_CLASSES.EndpointReferenceType rt = (MOF_CLASSES.EndpointReferenceType)f.GetValue(o);
                        //    string x = Properties.Resources.WsManEndpointRef;
                        //    int i = 0;
                        //    foreach (XmlElement xx in rt.ReferenceParameters.Any)
                        //    {
                        //        if (i != 0) x += "; ";
                        //        i++;
                        //        x += xx.InnerText;
                        //    }
                        //    value = x;
                        //}

                        sb.AppendFormat("{0}{1}: {2}\r\n", ident, f.Name, value);
                    }
                }
            }
            return sb.ToString();
        }

        public static void LogException(object sender, Exception ex)
        {
            LogEvent(null, EventType.Error, ex.GetType().ToString(), ex.ToString());
        }

        public static void ShowForm()
        {
            if (amtDebugForm != null)
            {
                amtDebugForm.Activate();
            }
            else
            {
                amtDebugForm = new AmtDebugForm();
                amtDebugForm.Show();
            }
        }

        private AmtDebugForm()
        {
            InitializeComponent();
        }

        private void okButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        public void RefreshData()
        {
            eventListView.BeginUpdate();
            eventListView.Items.Clear();
            ArrayList ll = new ArrayList();
            lock (events)
            {
                foreach (LogEventClass e in events)
                {
                    ListViewItem l = new ListViewItem(new string[] { e.time.ToLongTimeString(), e.type.ToString(), e.header }, e.GetIcon());
                    l.Tag = e;
                    ll.Add(l);
                }
            }
            eventListView.Items.AddRange((ListViewItem[])ll.ToArray(typeof(ListViewItem)));
            eventListView.EndUpdate();
        }

        public void InfoLogEventSink(LogEventClass e)
        {
            this.Invoke(new EventHandler(InfoLogEventSinkEx), new object[] { e });
        }

        public void InfoLogEventSinkEx(LogEventClass e)
        {
            try
            {
                lock (eventListView)
                {
                    ArrayList ll = new ArrayList();
                    ListViewItem l = new ListViewItem(new string[] { e.time.ToLongTimeString(), e.type.ToString(), e.header }, e.GetIcon());
                    l.Tag = e;
                    eventListView.Items.Insert(0, l);
                    while (eventListView.Items.Count > 200) eventListView.Items.RemoveAt(200);
                }
            }
            catch (System.InvalidOperationException) { } // In some cases, the form may no longer exist, if that's the case, just skip this.
        }

        private void AmtDebugForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            OnLogEvent -= new EventHandler(InfoLogEventSink);
            amtDebugForm = null;
        }

        private void AmtDebugForm_Load(object sender, EventArgs e)
        {
            this.Text = Utils.FilterStr(this.Text);
            RefreshData();
            OnLogEvent += new EventHandler(InfoLogEventSink);
        }

        private void clearEventsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            eventListView.Items.Clear();
            events.Clear();
            eventTextBox.Clear();
        }

        public static void StartDebugLog(string filename)
        {
            if (OutputFileStream != null)
            {
                OutputFileStream.Close();
                OutputFileStream = null;
            }
            OutputFileStream = new FileStream(filename, FileMode.Append);
        }

        private void streamToFileToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (OutputFileStream != null)
            {
                OutputFileStream.Close();
                OutputFileStream = null;
            }
            else
            {
                if (saveFileDialog.ShowDialog(this) == DialogResult.OK)
                {
                    OutputFileStream = new FileStream(saveFileDialog.FileName, FileMode.Create);
                }
            }
        }

        private void fileToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            streamToFileToolStripMenuItem.Checked = (OutputFileStream != null);
        }

        private void eventListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            eventTextBox.Clear();
            if (eventListView.SelectedItems.Count == 0) return;

            LogEventClass ee = (LogEventClass)eventListView.SelectedItems[0].Tag;

            if (ee.body == null)
            {
                eventTextBox.Text = ee.header;
            }
            else
            {
                eventTextBox.Text = ee.body;
            }
        }

        private void eventTextBox_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Control && e.KeyCode == Keys.A)
            {
                e.Handled = true;
                eventTextBox.SelectAll();
            }
        }
    }
}
