// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Collections.Generic;

namespace ManageabilityStack
{
    public class AmtDiscovery
    {
        private IPAddress start;
        private IPAddress end;
        private IPAddress current;
        private const ushort port = 16992;
        private Hashtable socketTable = new Hashtable();
        private Thread timerThread;
        private Thread mainThread;
        private const int timeout = 4000;
        private const int threadCap = 3;

        public const int PingCount = 2;
        private const int pingPeriod = 3000;

        private Dictionary<IPAddress, AmtDiscoveredComputer> reportedComputers = new Dictionary<IPAddress, AmtDiscoveredComputer>();

        public enum AmtDiscoveryEvent
        {
            LaunchingDiscovery = 1,
            RmcpPing,
            RmcpPong,
            StartingTcpSweep,
            MachineFound,
            MachineNotFound,
            DiscoveryFinished,
        }

        public enum AmtDiscoveryMethod
        {
            None = 0,
            TCP,
            RMCP
        }

        public enum ProvisionState
        {
            None = 0,
            NonTLS,
            TLS
        }

        private struct DiscoverySocketInfo
        {
            public Socket s;
            public IPEndPoint ep;
            public IAsyncResult async;
            public DateTime t;
            public bool stopped;
        }

        public class AmtDiscoveredComputer
        {
            public string Hostname;
            public IPEndPoint EndPoint;
            public string Info;
            public ProvisionState State;
            public IPHostEntry Entry;
            public uint AmtRev = 0;
            public uint AmtRevMin = 0;
            public AmtDiscoveryMethod Method = AmtDiscoveryMethod.None;

            public AmtDiscoveredComputer(string hostname, IPEndPoint endPoint, string info, ProvisionState provisionState, IPHostEntry entry, uint amtRev, uint amtRevMin, AmtDiscoveryMethod method)
            {
                Hostname = hostname;
                EndPoint = endPoint;
                Info = info;
                State = provisionState;
                Entry = entry;
                AmtRev = amtRev;
                AmtRevMin = amtRevMin;
                Method = method;
            }

            /// <summary>
            /// Returns true if this is a mobile platform
            /// </summary>
            public bool IsMobile
            {
                get
                {
                    return (AmtRev == 2 && AmtRevMin >= 5);
                }
            }
        }

        public delegate void AmtDiscoveryEventHandler(AmtDiscovery amtDiscovery, IPEndPoint ep, AmtDiscoveryEvent e, AmtDiscoveredComputer computer);
        public event AmtDiscoveryEventHandler OnDiscoveryEvent;

        public AmtDiscovery()
        {
        }

        public bool IsRunning
        {
            get
            {
                return (mainThread != null);
            }
        }

        #region address math
        // Get the next IP address - Works with IPv4 & IPv6
        private static IPAddress GetNextAddress(IPAddress addr)
        {
            byte[] ip = addr.GetAddressBytes();
            ip[ip.Length-1]++;
            bool carry = false;
            carry = (ip[ip.Length - 1] == 0);
            for (int i = ip.Length-1; i > 0; i--)
            {
                if (carry)
                {
                    ip[i - 1]++;
                    carry = (ip[i - 1] == 0);
                }
            }
            return new IPAddress(ip) ;
        }

        private static bool IsSameAddress(IPAddress a1, IPAddress a2)
        {
            byte[] b1 = a1.GetAddressBytes();
            byte[] b2 = a2.GetAddressBytes();
            if (b1.Length != b2.Length) return false;
            for (int i = 0; i < b1.Length; i++) if (b1[i] != b2[i]) return false;
            return true;
        }

        public static long GetAddressDiff(IPAddress a1, IPAddress a2)
        {
            byte[] b1 = a1.GetAddressBytes();
            byte[] b2 = a2.GetAddressBytes();
            if (b1.Length != b2.Length) return -1;
            int carry = 0;
            long total = 0;
            int l = b1.Length - 1;
            for (int i = l ; i >= 0; i--)
            {
                int d = b2[i] - carry - b1[i];
                if (d < 0)
                {
                    carry = 1;
                    total += (d << ((l - i) * 8));
                }
                else
                {
                    carry = 0;
                    total += (d << ((l - i) * 8));
                }
            }
            return total + 1;
        }
        #endregion address math

        private IPAddress GetNextAddressToRMCPScan(IPAddress addr)
        {
            IPAddress current = addr;
            lock (reportedComputers)
            {
                do
                {
                    current = GetNextAddress(current);
                }
                while (reportedComputers.ContainsKey(current));
           }
            return current;
        }
        private IPAddress GetNextAddressToTCPScan(IPAddress addr)
        {
            IPAddress current = addr;
            lock (reportedComputers)
            {
                bool shouldScan;
                do
                {
                    current = GetNextAddress(current);
                    shouldScan = true;
                    AmtDiscoveredComputer previousDiscovery;
                    if (reportedComputers.TryGetValue(current, out previousDiscovery))
                    {
                        // If we want to rescan clients that don't give version via RMCP, we
                        // would need to avoid skipping them with this "if".
                        //if (previousDiscovery != null && previousDiscovery.AmtRev > 0)
                        //{
                            shouldScan = false;
                        //}
                    }
                }
                while (!shouldScan/* && !IsSameAddress(current, end)*/);
                return current;
            }
        }
        private IPAddress GetFirstAddress()
        {
            IPAddress addr = start;
            lock (reportedComputers)
            {
                while (reportedComputers.ContainsKey(addr))
                {
                    addr = GetNextAddress(addr);
                }
            }
            return addr;
        }

        public void Dispose()
        {
            Stop();
        }

        public void Start(IPAddress start, IPAddress end)
        {
            if (mainThread != null) return;

            this.start = start;
            this.end = end;
            current = start;
            reportedComputers.Clear();

            mainThread = new Thread(new ParameterizedThreadStart(MainThread));
            mainThread.Name = "Discovery Main Thread";
            mainThread.Start(this);

        }

        private void PingSweep(UdpClient RMCPClient)
        {
            lock (this)
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "Starting RMCP Ping Discovery Sweep.", string.Empty);
                if (OnDiscoveryEvent != null) OnDiscoveryEvent(this, null, AmtDiscoveryEvent.RmcpPing, null);

                IPAddress endPlusOne = AmtDiscovery.GetNextAddress(end);
                for (current = GetFirstAddress(); !AmtDiscovery.IsSameAddress(current, endPlusOne); current = GetNextAddressToRMCPScan(current))
                {
                    lock (RMCPClient)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "RMCP Ping Discovery Sweep: " + current.ToString(), string.Empty);

                        try
                        {
                            RMCPClient.Send(pingMessage, pingMessage.Length, new IPEndPoint(current, 623));
                        }
                        catch
                        {
                        }

                    }
                }
            }
        }

        //private void PingCallback(IAsyncResult result)
        //{
        //    byte[] receiveBytes = null;
        //    IPEndPoint ep = null;// ((IPEndPoint)(s.Client.RemoteEndPoint));
        //    // Get the result
        //    lock (this)
        //    {
        //        try
        //        {
        //            if (RMCPClient.Available > 0)
        //            {
        //                receiveBytes = RMCPClient.EndReceive(result, ref ep);
        //            }
        //            else
        //            {
        //                return;
        //            }
        //        }
        //        catch (SocketException ex)
        //        {
        //            if (ex.ErrorCode == 10054)
        //            {
        //                // User probably included the local machine in the range.  Report as not found.
        //                //if (OnDiscoveryEvent != null) OnDiscoveryEvent(this, ep, AmtDiscoveryEvent.MachineNotFound, null);
        //                return;
        //            }
        //            throw;
        //        }
        //        finally
        //        {
        //            //RMCPClient.BeginReceive(new AsyncCallback(PingCallback), null);
        //            StartRmcpListener();
        //        }
        //    }
        //    processPong(receiveBytes, ep);
        //}
        //private void StartRmcpListener()
        //{
        //    bool tryAgain = true;
        //    while (tryAgain)
        //    {
        //        lock (this)
        //        {
        //            try
        //            {
        //                tryAgain = false;
        //                RMCPClient.BeginReceive(new AsyncCallback(PingCallback), null);
        //            }
        //            catch (SocketException ex)
        //            {
        //                if (ex.ErrorCode == 10054)
        //                {
        //                    //// User probably included the local machine in the range.  Report as not found.
        //                    //if (OnDiscoveryEvent != null) OnDiscoveryEvent(this, null, AmtDiscoveryEvent.MachineNotFound, null);
        //                }
        //                tryAgain = true;
        //            }
        //        }
        //    }
        //}

        private void TimerThread(object x)
        {
            try
            {
                ArrayList l = new ArrayList();
                DateTime n;
                while (true)
                {
                    Thread.Sleep(500);
                    n = DateTime.Now;
                    lock (socketTable)
                    {
                        foreach (Socket s in socketTable.Keys)
                        {
                            DiscoverySocketInfo inf = (DiscoverySocketInfo)socketTable[s];
                            int c = inf.t.CompareTo(n);
                            if (c < 0)
                            {
                                // Break the socket
                                try
                                {
                                    s.Close();
                                }
                                catch (SocketException) { }
                                inf.stopped = true;
                                l.Add(inf);
                            }
                        }
                        /*
                        foreach (DiscoverySocketInfo inf in l)
                        {
                            socketTable.Remove(inf.s);
                        }
                        */
                    }
                    /*
                    foreach (DiscoverySocketInfo inf in l)
                    {
                        if (OnDiscoveryEvent != null) OnDiscoveryEvent(this, inf.ep, AmtDiscoveryEvent.MachineNotFound, null);
                        LaunchNextSocket();
                    }
                    */
                    l.Clear();
                    if (socketTable.Count == 0 && GetAddressDiff(current, end) < 0)
                    {
                        timerThread = null;
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "TCP discovery complete.", string.Empty);
                        if (OnDiscoveryEvent != null) OnDiscoveryEvent(this, null, AmtDiscoveryEvent.DiscoveryFinished, null);
                        break;
                    }
                }
            }
            catch (ThreadInterruptedException) {}
            timerThread = null;
        }

        private void MainThread(object x)
        {
            Thread listenerThread = null;
            try
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "Launching discovery.", string.Empty);
                if (OnDiscoveryEvent != null) OnDiscoveryEvent(this, null, AmtDiscoveryEvent.LaunchingDiscovery, null);
                using (UdpClient RMCPClient = new UdpClient())
                {
                    listenerThread = new Thread(new ParameterizedThreadStart(ListenerThread));
                    listenerThread.Name = "RMCP Listener Thread";
                    listenerThread.Start(RMCPClient);
                    for (int i = 0; i < PingCount; ++i)
                    {
                        PingSweep(RMCPClient);
                        Thread.Sleep(pingPeriod);
                    }
                    listenerThread.Abort();
                    listenerThread = null;
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "RMCP ping sweep complete.  Starting TCP connection sweep.", string.Empty);
                if (OnDiscoveryEvent != null) OnDiscoveryEvent(this, null, AmtDiscoveryEvent.StartingTcpSweep, null);

                current = GetFirstAddress();
                LaunchOneSocket(new IPEndPoint(current, port));
                for (int i = 0; i < threadCap; i++) LaunchNextSocket();

                timerThread = new Thread(new ParameterizedThreadStart(TimerThread));
                timerThread.Name = "TCP Discovery Thread";
                timerThread.Start(this);
                timerThread.Join();
            }
            catch (ThreadInterruptedException)
            {
                if (listenerThread != null)
                {
                    listenerThread.Abort();
                    listenerThread = null;
                }
                if (timerThread != null)
                {
                    timerThread.Interrupt();
                    timerThread = null;
                }
            }
            mainThread = null;
        }

        private void processPong(byte[] receiveBytes, IPEndPoint ep)
        {
            // Process the result
            if (receiveBytes != null && ep != null)
            {
                if (receiveBytes[3] == 0x86)
                {
                    // This is just the ack.  Drop it and wait again for the pong.
                    return;
                }
                    //Check 1) if it's an ASF call, 2) if it's a pong.
                else if ((receiveBytes[3] == 0x06) && (receiveBytes[8] == 0x40))
                {
                    //Make sure it's an Intel one.
                    uint iana = BitConverter.ToUInt32(receiveBytes, 4);
                    if (iana != 0x57010000)
                    {
                        // Not an Intel device.
                        return;
                    }
                    //DiscoverySocketInfo inf;
                    ushort amtPort;
                    ushort amtRev;
                    ushort amtRevMin;
                    AmtDiscoveredComputer discoveredComputer = null;
                    ProvisionState provisionState = ProvisionState.None;
                    // This must be the pong.
                    if (OnDiscoveryEvent != null)
                    {
                        // Try to get the Reverse DNS name
                        string hostname = ep.Address.ToString();
                        IPHostEntry hostEntry = null;
                        try
                        {
                            hostEntry = System.Net.Dns.GetHostEntry(ep.Address);
                        }
                        catch (Exception) { }
                        if (hostEntry != null) hostname = hostEntry.HostName;

                        if (receiveBytes.Length > 19)
                        {
                            amtPort = ((ushort)((receiveBytes[16] << 8) | receiveBytes[17]));
                            amtRev = (ushort)((receiveBytes[18]) >> 4);
                            amtRevMin = (ushort)((receiveBytes[18]) & 0xF);

                            if (amtPort == 0)
                            {
                                provisionState = ProvisionState.None;
                            }
                            else if (amtPort == 16993)
                            {
                                provisionState = ProvisionState.TLS;
                            }
                            else
                            {
                                provisionState = ProvisionState.NonTLS;
                            }

                            string version = "< 6.0";
                            if (amtRev > 0)
                            {
                                version = string.Format("{0}.{1}", amtRev, amtRevMin);
                            }
                            string str = string.Format("Intel(R) Active Management Technology {0}", version);
                            str = Utils.FilterStr(str);
                            discoveredComputer = new AmtDiscoveredComputer(hostname, ep, str, provisionState, hostEntry, amtRev, amtRevMin, AmtDiscoveryMethod.RMCP);
                        }
                        else
                        {
                            discoveredComputer = new AmtDiscoveredComputer(hostname, ep, "Unknown AMT version", ProvisionState.None, null, 0, 0, AmtDiscoveryMethod.RMCP);
                        }
                        if (discoveredComputer != null)
                        {
                            // Make sure we haven't already reported this one.
                            lock (reportedComputers)
                            {
                                if (!reportedComputers.ContainsKey(ep.Address))
                                {
                                    reportedComputers.Add(ep.Address, discoveredComputer);
                                }
                                else
                                {
                                    // We've seen it before.  Log a warning and drop it. (return)
                                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Warning, "Unexpected second response to discovery ping.",
                                        string.Format("IP Address: {0}.", ep.Address));
                                    return;
                                }
                            }
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "RMCP ping discovered an AMT system.", 
                                string.Format("System name: {0}", discoveredComputer.Hostname));
                            OnDiscoveryEvent(this, ep, AmtDiscoveryEvent.RmcpPong, discoveredComputer);
                        }
                    }
                }
            }
        }
        private void ListenerThread(object x)
        {
            UdpClient listener = (UdpClient)x;
            try
            {
                while (true)
                {
                    try
                    {
                        byte[] bytes = null;
                        IPEndPoint ep = null;
                        lock (listener)
                        {
                            if (listener.Available > 0)
                            {
                                bytes = listener.Receive(ref ep);
                            }
                            else
                            {
                                continue;
                            }
                        }
                        if (bytes != null)
                        {
                            processPong(bytes, ep);
                        }
                    }
                    catch (SocketException ex)
                    {
                        if (ex.ErrorCode == 10054)
                        {
                            // Client forcibly closed connection.
                            continue;
                        }
                        throw;
                    }
                }
            }
            catch (ThreadAbortException)
            {
                // Let the caller do this.
                //listener.Close();
                //listener = null;
            }
            catch (ObjectDisposedException)
            {
                // If the master thread stopped using the listener, it would be disposed.
                // Just return in this case.
            }
        }
        private static byte[] pingMessage = new byte[] 
        { 
            6,
            0,
            0,
            6,
            0,
            0,
            0x11,
            0xBE,
            0x80,
            0,
            0,
            0
        };
        
        private void LaunchOneSocket(IPEndPoint ep)
        {
            Socket s = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);
            s.Ttl = 2;
            s.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.DontLinger, true);
            s.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);

            DiscoverySocketInfo inf = new DiscoverySocketInfo();
            inf.ep = ep;
            inf.s = s;
            inf.t = DateTime.Now.AddMilliseconds(timeout);

            lock (socketTable)
            {
                socketTable.Add(s, inf);
            }

            inf.async = s.BeginConnect(ep, new AsyncCallback(SocketConnectCallback), s);
            
        }

        private void SocketConnectCallback(IAsyncResult result)
        {
            bool success = true;
            DiscoverySocketInfo inf;
            Socket s = (Socket)result.AsyncState;
            lock (socketTable)
            {
                if (socketTable[s] == null) return;
                inf = (DiscoverySocketInfo)socketTable[s];
                socketTable.Remove(s);

                try
                {
                    s.EndConnect(result);
                }
                catch (SocketException)
                {
                    success = false;
                }
                catch (ObjectDisposedException)
                {
                    success = false;
                }
            }
            if (OnDiscoveryEvent != null)
            {
                AmtDiscoveredComputer discoveredComputer = null;
                if (success == true)
                {
                    uint AmtRev = 0;
                    uint AmtRevMin = 0;
                    IPEndPoint ip = (IPEndPoint)s.RemoteEndPoint;

                    // Try a manual TCP Client GET request
                    string str = null;
                    using (TcpClient client = new TcpClient())
                    {
                        try
                        {
                            client.Connect(ip);
                        }
                        catch (Exception)
                        {
                            str = "Active refusal of TCP connection.";
                        }

                        if (str == null)
                        {
                            NetworkStream ns = client.GetStream();
                            DateTime failTime = DateTime.Now.AddSeconds(5);
                            byte[] b = UTF8Encoding.UTF8.GetBytes("HEAD / HTTP/1.1\r\nHost: AmtDiscovery\r\n\r\n");
                            ns.Write(b, 0, b.Length);
                            byte[] buf = new byte[2000];
                            int ptr = 0;
                            string head = "";
                            try
                            {
                                while (DateTime.Now.CompareTo(failTime) < 0 && head.IndexOf("\r\n\r\n") < 0)
                                {
                                    Thread.Sleep(100);
                                    ptr = ns.Read(buf, ptr, 2000 - ptr);
                                    head = UTF8Encoding.UTF8.GetString(buf, 0, ptr);
                                }
                            }
                            catch (Exception) { }
                            client.Close();

                            if (head.IndexOf("\r\n\r\n") > 0)
                            {
                                int i = head.IndexOf("\r\nServer: ");
                                if (i >= 0)
                                {
                                    str = head.Substring(i + 10);
                                    i = str.IndexOf("\r\n");
                                    str = str.Substring(0, i);
                                }
                                if (str == null)
                                {
                                    str = "Unknown Network Node";
                                }
                                else if (str.Equals("Intel(R) Active Management Technology") == true)
                                {
                                    str = "Intel(R) Active Management Technology 1.0";
                                    AmtRev = 1;
                                    AmtRevMin = 0;
                                }
                                else if (str.StartsWith("Intel(R) Active Management Technology 2.") == true)
                                {
                                    AmtRev = 2;
                                    try
                                    {
                                        AmtRevMin = uint.Parse(str.Substring(40, 1));
                                    }
                                    catch (Exception) { }
                                }
                                else if (str.StartsWith("Intel(R) Active Management Technology 3.") == true)
                                {
                                    AmtRev = 3;
                                    try
                                    {
                                        AmtRevMin = uint.Parse(str.Substring(40, 1));
                                    }
                                    catch (Exception) { }
                                }
                                else if (str.StartsWith("Intel(R) Active Management Technology 4.") == true)
                                {
                                    AmtRev = 4;
                                    try
                                    {
                                        AmtRevMin = uint.Parse(str.Substring(40, 1));
                                    }
                                    catch (Exception) { }
                                }
                            }
                        }
                    }

                    bool tls = false;
                    if (str == null)
                    {
                        // Just for kicks, lets try to connect to the TLS port and see what happens.
                        using (TcpClient client = new TcpClient())
                        {
                            try
                            {
                                client.Connect(ip.Address, 16993);
                                tls = true;
                                client.Close();
                            }
                            catch (Exception) { }
                        }
                        if (tls) str = "Intel(R) AMT with TLS or not configured";
                    }

                    if (str == null)
                    {
                        str = ManageabilityStack.Properties.Resources.Discovery_DiscoveryEvent;
                    }
                    else
                    {
                        str = Utils.FilterStr(str);
                    }

                    // Try to get the Reverse DNS name
                    string hostname = inf.ep.Address.ToString();
                    IPHostEntry hostEntry = null;
                    try
                    {
                        hostEntry = System.Net.Dns.GetHostEntry(inf.ep.Address);
                    }
                    catch (Exception) {}
                    if (hostEntry != null) hostname = hostEntry.HostName;
                    ProvisionState provisionState = (tls) ? ProvisionState.TLS : ProvisionState.NonTLS;
                    discoveredComputer = new AmtDiscoveredComputer(hostname, inf.ep, str, provisionState, hostEntry, AmtRev, AmtRevMin, AmtDiscoveryMethod.TCP);
                }
                else // if (success == false)
                {
                    // Just for kicks, lets try to connect to the TLS port and see what happens.
                    string str = null;
                    bool tls = false;
                    TcpClient client = new TcpClient();
                    try
                    {
                        IPEndPoint ip = (IPEndPoint)inf.ep;
                        client.Connect(ip.Address, 16993);
                        tls = true;
                        client.Close();
                    }
                    catch (Exception) { }
                    if (tls)
                    {
                        str = "Intel(R) AMT with TLS or not configured";
                        client = null;

                        if (str == null)
                        {
                            str = ManageabilityStack.Properties.Resources.Discovery_DiscoveryEvent;
                        }
                        else
                        {
                            str = Utils.FilterStr(str);
                        }

                        // Try to get the Reverse DNS name
                        string hostname = inf.ep.Address.ToString();
                        IPHostEntry hostEntry = null;
                        try
                        {
                            hostEntry = System.Net.Dns.GetHostEntry(inf.ep.Address);
                        }
                        catch (Exception) { }
                        if (hostEntry != null) hostname = hostEntry.HostName;
                        ProvisionState provisionState = (tls) ? ProvisionState.TLS : ProvisionState.NonTLS;
                        discoveredComputer = new AmtDiscoveredComputer(hostname, inf.ep, str, provisionState, hostEntry, 0, 0, AmtDiscoveryMethod.TCP);
                    }
                    else
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "TCP Connection attempt found nothing.",
                            string.Format("{0} did not respond to TCP connect attempt.", inf.ep.Address.ToString()));
                        OnDiscoveryEvent(this, inf.ep, AmtDiscoveryEvent.MachineNotFound, null);
                    }
                }
                if (discoveredComputer != null)
                {
                    lock (reportedComputers)
                    {
                        if (!reportedComputers.ContainsKey(inf.ep.Address))
                        {
                            reportedComputers.Add(inf.ep.Address, discoveredComputer);
                        }
                        else
                        {
                            // We've seen it before.  Log a warning and drop it. (return)
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Warning, "Unexpected second discovery using TCP connection.",
                                string.Format("IP Address: {0}.", inf.ep.Address));
                        }
                    }
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "AMT computer discovered by TCP connection.",
                        string.Format("Computer Name: {0}", discoveredComputer.Hostname));
                    OnDiscoveryEvent(this, inf.ep, AmtDiscoveryEvent.MachineFound, discoveredComputer);
                }
            }
            LaunchNextSocket();
        }

        private void LaunchNextSocket()
        {
            if (current == null) return;
            
            lock (socketTable)
            {
                if (socketTable.Count > threadCap) return;
            }

            if (GetAddressDiff(current, end) > 0)//(!IsSameAddress(current,end))
            {
                current = GetNextAddressToTCPScan(current);
                if (GetAddressDiff(current, end) >= 0)
                {
                    LaunchOneSocket(new IPEndPoint(current, port));
                }
            }
        }

        public void Stop()
        {
            if (mainThread != null) mainThread.Interrupt();
            mainThread = null;
            lock (socketTable)
            {
                foreach (Socket s in socketTable.Keys)
                {
                    DiscoverySocketInfo inf = (DiscoverySocketInfo)socketTable[s];
                    // Break the socket
                    try
                    {
                        inf.stopped = true;
                        s.Close();
                    }
                    catch (SocketException) { }
                }
                socketTable.Clear();
            }
        }

    }
}


/*
        [DllImport("iphlpapi.dll", ExactSpelling = true)]
        private static extern int SendARP(int DestIP, int SrcIP, byte[] pMacAddr, ref int PhyAddrLen);

        private List<int> list = new List<int>();
        private int finishedThreads = 0;

        static void Main(string[] args)
        {
            if (!ParseCmdLine(args)) return;
            new ScanLAN(args[0], 1, 254);
        }

        public ScanLAN(string subnet, int first, int last)
        {
            for (int i = first; i <= last; i++)
            {
                new Thread(this.ScanAddressThread).Start(subnet + "." + i);
            }
            while (this.finishedThreads < last - first + 1) Thread.Sleep(100);

            this.list.Sort();
            foreach (int i in this.list)
            {
                Console.WriteLine(subnet + "." + i);
            }
        }

        private void ScanAddressThread(object ipString)
        {
            string ip = (string) ipString;
            int dest = BitConverter.ToInt32(IPAddress.Parse(ip).GetAddressBytes(), 0);
            byte[] mac = new byte[6];
            int length = 6;

            SendARP(dest, 0, mac, ref length);
            if (length > 0)
            {
                this.list.Add(Int32.Parse(ip.Substring(ip.LastIndexOf(".") + 1)));
            }
            Interlocked.Increment(ref this.finishedThreads);
        }
*/
