// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Net;
using System.Net.Sockets;
using System.Text;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;

namespace ManageabilityStack
{
    public class AmtEventManager
    {
        private EventManagerService service;
        public delegate void AmtNetworkEventHandler(AmtEventManager source, AmtSystem system, AmtAlert alert);
        public event AmtNetworkEventHandler OnAlert;
        public ArrayList AlertList = new ArrayList();
        private AmtSystem parent = null;
        private bool local = false;
        public bool LocalAccessOnly { get { return local; } }

        public AmtEventManager(EventManagerService service, AmtSystem parent, bool local)
        {
            this.service = service;
            this.parent = parent;
            this.local = local;
            AmtTrapListener.TrapListener.AmtNetworkEvent += new AmtTrapListener.AmtNetworkEventHandler(AmtNetworkEventSink);
        }

        public void Dispose()
        {
            AmtTrapListener.TrapListener.AmtNetworkEvent -= new AmtTrapListener.AmtNetworkEventHandler(AmtNetworkEventSink);
            service = null;
        }

        public void ClearCache()
        {
            EventLogCache = null;
            SubscriptionsCache = null;
            EventFiltersCache = null;
        }

        public bool IsCached()
        {
            if (local)
            {
                return (EventLogCache != null);
            }
            else
            {
                return (SensorsCache != null && EventLogCache != null && /* SubscriptionsCache != null && */ EventFiltersCache != null);
            }
        }

        public void ClearSubscriptionsCache()
        {
            SubscriptionsCache = null;
        }

        public void ClearFiltersCache()
        {
            EventFiltersCache = null;
        }

        public void FetchCache()
        {
            try
            {
                GetSensors();
                GetEventLog();
                GetAlertSubscriptions();
                GetEventFilters();
            }
            catch (Exception e)
            {
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Failure to retrieve the Event Manager data.",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
            }
        }

        private void AmtNetworkEventSink(IPEndPoint endpoint, AmtAlert alert)
        {
            lock (AlertList)
            {
                // First, check if we got this alert already in the list.
                foreach (AmtAlert a in AlertList)
                {
                    if (a.SequenceNumber == alert.SequenceNumber) return;
                }

                // We don't have this alert, so, lets log it and fire and event
                AlertList.Insert(0, alert);
            }
            if (OnAlert != null) OnAlert(this, parent, alert);
        }

        public AmtAlert[] GetAlerts()
        {
            return (AmtAlert[])AlertList.ToArray(typeof(AmtAlert));
        }

        public void ClearAlerts()
        {
            lock (AlertList)
            {
                AlertList.Clear();
            }
        }

        public AmtCallStatus ClearLog()
        {
            if (parent.WsMan != null)
            {
                try
                {
                    EventLogCache = null;
                    AMT_MessageLog msgLog = new AMT_MessageLog(parent.WsMan);
                    AmtCallStatus status;
                    status = (AmtCallStatus)msgLog.ClearLog();
                    return status;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.ClearLog WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.ClearLog CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.ClearLog error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                AmtCallStatus r = (AmtCallStatus)service.ClearEventLog();
                if (r == AmtCallStatus.SUCCESS) EventLogCache = null;
                return r;
            }
        }

        public AmtCallStatus SetAlertCommunityString(string str)
        {
            AlertCommunityStringCache = null;
            if (parent.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_EventManagerService s = new AMT_EventManagerService(parent.WsMan);
                    s.DefaultCommunityString = str;
                    s.Put();
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.SetAlertCommunityString WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.SetAlertCommunityString CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.SetAlertCommunityString error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                byte[] s = UTF8Encoding.UTF8.GetBytes(str.Substring(0, 200));
                return (AmtCallStatus)service.SetAlertCommunityString((byte)s.Length, s);
            }
        }

        private string AlertCommunityStringCache = null;
        public string GetAlertCommunityString()
        {
            if (AlertCommunityStringCache != null) return AlertCommunityStringCache;

            if (parent.WsMan != null)
            {
                try
                {
                    AMT_EventManagerService s = new AMT_EventManagerService(parent.WsMan);
                    s.Get();
                    AlertCommunityStringCache = s.DefaultCommunityString;
                    return AlertCommunityStringCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetAlertCommunityString WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetAlertCommunityString CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetAlertCommunityString error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    throw;
                }
            }
            else
            {
                byte[] s;
                byte sl;
                AmtCallStatus r = (AmtCallStatus)service.GetAlertCommunityString(out sl, out s);
                return UTF8Encoding.UTF8.GetString(s);
            }
        }

        public byte[] GetAlertPolicieHandles()
        {
            uint totalPolicyCount;
            uint policyReturned;
            byte[] policyHandles;
            service.EnumerateAlertPolicies(0, out totalPolicyCount, out policyReturned, out policyHandles);
            return policyHandles;
        }

        public uint[] GetAlertSubscriptionHandles()
        {
            uint totalSubscriptionCount;
            uint subscriptionsReturned;
            uint[] subscriptionHandles;
            service.EnumerateAlertSubscriptions(0, 0, false, out totalSubscriptionCount, out subscriptionsReturned, out subscriptionHandles);
            return subscriptionHandles;
        }

        private AmtEventSensor[] SensorsCache = null;
        public AmtEventSensor[] GetSensors()
        {
            if (SensorsCache != null) return SensorsCache;

            if (parent.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    Collection<AMT_PETCapabilities> r = AMT_PETCapabilities.Enumerate(parent.WsMan);
                    ArrayList l = new ArrayList();
                    foreach (AMT_PETCapabilities s in r)
                    {
                        SensorAttributesType sat = new SensorAttributesType();
                        sat.Entity = s.Entity;
                        sat.EntityInstance = s.EntityInstance;
                        sat.EventOffset = s.EventOffset;
                        sat.EventSensorType = s.EventSensorType;
                        sat.EventSeverity = s.EventSeverity;
                        sat.EventSourceType = s.EventSourceType;
                        sat.EventType = s.EventType;
                        sat.Index = s.Index;
                        sat.SensorNumber = s.SensorNumber;
                        sat.AssertionEvent = s.AssertionEvent;
                        sat.DeassertionEvent = s.DeassertionEvent;
                        sat.DeviceAddress = s.DeviceAddress;
                        l.Add(new AmtEventSensor(sat));
                    }
                    SensorsCache = (AmtEventSensor[])l.ToArray(typeof(AmtEventSensor));
                    return SensorsCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetSensors WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetSensors CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetSensors error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    SensorsCache = null;
                    return SensorsCache;
                }
            }
            else
            {
                // Use SOAP
                uint totalCount;
                uint returned;
                uint[] handles;
                service.EnumerateSensors(1, out totalCount, out returned, out handles);

                AmtEventSensor[] sensorTable = new AmtEventSensor[handles.Length];
                for (int i = 0; i < handles.Length; i++)
                {
                    SensorAttributesType s;
                    service.GetSensorAttributes(handles[i], out s);
                    sensorTable[i] = new AmtEventSensor(s);
                }
                SensorsCache = sensorTable;
                return SensorsCache;
            }
        }

        public AmtCallStatus FreezeEventLog(bool state)
        {
            return (AmtCallStatus)service.FreezeEventLog(state);
        }

        public AmtCallStatus GetEventLogStatus(out uint numberOfFreeRecords, out uint numberOfRecords, out uint time, out bool isFrozen)
        {
            return (AmtCallStatus)service.GetEventLogStatus(out numberOfFreeRecords, out numberOfRecords, out time, out isFrozen);
        }

        private class AmtEventComparerClass : IComparer
        {
            int IComparer.Compare(Object x, Object y)
            {
                AmtEvent e1 = (AmtEvent)x;
                AmtEvent e2 = (AmtEvent)y;
                return (e2.TimeStamp.CompareTo(e1.TimeStamp));
            }
        }

        private AmtEvent[] EventLogCache = null;
        public AmtEvent[] GetEventLog() { return GetEventLog(false); }
        public AmtEvent[] GetEventLog(bool forceupdate)
        {
            if (EventLogCache != null && forceupdate == false) return EventLogCache;

            if (parent.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    string iterationIdentifier;
                    string newIterationIdentifier;
                    string[] recordArray = new string[0];
                    bool noMoreRecords = false;
                    ArrayList list = new ArrayList();

                    AMT_MessageLog messageLog = new AMT_MessageLog(parent.WsMan);
                    messageLog.Get();
                    ulong curNumRecords = (messageLog.CurrentNumberOfRecordsExist ? messageLog.CurrentNumberOfRecords : 0);
                    messageLog.PositionToFirstRecord(out iterationIdentifier);
                    if (iterationIdentifier != null)
                    {
                        while (noMoreRecords != true)
                        {
                            messageLog.GetRecords(iterationIdentifier, (uint)curNumRecords,
                                out newIterationIdentifier, out noMoreRecords, out recordArray);

                            foreach (string str in recordArray)
                            {
                                byte[] byteBuffer = Convert.FromBase64String(str);
                                list.Add(new AmtEvent(byteBuffer));
                            }

                            iterationIdentifier = newIterationIdentifier;
                        } // END while (noMoreRecords != true)
                    }

                    list.Sort(new AmtEventComparerClass());
                    EventLogCache = (AmtEvent[])list.ToArray(typeof(AmtEvent));
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetEventLog WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetEventLog CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetEventLog error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return null;
                }
            }
            else
            {
                // Use SOAP
                uint totalRecordCount;
                uint recordsReturned;
                uint ptr = 1;
                ArrayList list = new ArrayList();
                do
                {
                    EventLogRecordType[] events;
                    if (service == null) return new AmtEvent[0];
                    try // This is a patch in case the firmware can't handle event log
                    {
                        service.ReadEventLogRecords(ptr, out totalRecordCount, out recordsReturned, out events);
                    }
                    catch (System.Net.WebException) { return new AmtEvent[0]; }
                    catch (System.InvalidOperationException) { return new AmtEvent[0]; }
                    foreach (EventLogRecordType e in events) list.Add(new AmtEvent(e));
                    ptr += recordsReturned;
                }
                while (ptr < totalRecordCount);
                EventLogCache = (AmtEvent[])list.ToArray(typeof(AmtEvent));
            }

            return EventLogCache;
        }

        public System.DateTime GetClock()
        {
            uint time;
            service.GetEventLogTimestampClock(out time);
            return Utils.ClockToDate(time);
        }

        public AmtCallStatus SetClock(System.DateTime time)
        {
            return (AmtCallStatus)service.SetEventLogTimestampClock(Utils.DateToClock(time));
        }

        private AmtCallStatus SubscribeForAlert(AlertSubscriptionGeneralType subscription, out uint subscriptionHandle)
        {
            subscriptionHandle = 0;

            if (parent.CoreVersion >= 3.0)
            {
                return (AmtCallStatus)service.SubscribeForGeneralAlert(subscription, out subscriptionHandle);
            }
            else
            {
                AlertSubscriptionType t = new AlertSubscriptionType();
                t.AlertSubscriptionPolicyID = subscription.PolicyID;
                if (subscription.Item.GetType() != typeof(AlertSubscriptionSNMPType)) return AmtCallStatus.CALL_SKIPPED; 
                AlertSubscriptionSNMPType t2 = (AlertSubscriptionSNMPType)subscription.Item;
                if (t2.Address.ItemElementName != EventItemChoiceType.IPv4Address) return AmtCallStatus.CALL_SKIPPED;
                IPAddress a = IPAddress.Parse(t2.Address.Item);
                byte[] b = a.GetAddressBytes();
                t.DestinationAddress = (uint)((b[0] << 24) + (b[1] << 16) + (b[2] << 8) + b[3]);
                SubscriptionsCache = null;
                return (AmtCallStatus)service.SubscribeForAlert(t, out subscriptionHandle);
            }
        }

        public bool CancelAlertSubscription(uint handle)
        {
            if (parent.WsMan != null)
            {
                // WSMAN
                AmtAlertSubscription[] subs = GetAlertSubscriptions();
                if (subs == null) return false;
                foreach (AmtAlertSubscription sub in subs) CancelAlertSubscription(sub); // TODO: We could optimize this with a WSMAN delete all.
                return true;
            }
            else
            {
                // SOAP
                SubscriptionsCache = null;
                service.CancelAlertSubscription(handle);
                return true;
            }
        }

        public AmtAlertSubscription SubscribeForAlerts(string hostname, byte policyId)
        {
            // First, try to parse the hostname for an IPv4 or IPv6 address
            IPAddress addr = null; ;
            IPAddress.TryParse(hostname, out addr);

            if (parent.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_SNMPEventSubscriber sub = new AMT_SNMPEventSubscriber(parent.WsMan);
                    sub.AccessInfo = hostname;
                    sub.CommunityString = "public";
                    sub.CreationClassName = "AMT_SNMPEventSubscriber";
                    sub.Name = "Intel(r) AMT PET Receiver";
                    sub.PolicyID = policyId;
                    sub.SystemCreationClassName = "CIM_ComputerSystem";
                    sub.SystemName = "Intel(r) AMT";

                    // Handle case where IP Address is IPv6
                    IPAddress address;
                    IPAddress.TryParse(hostname, out address);
                    if(address.AddressFamily == AddressFamily.InterNetworkV6)
                        sub.InfoFormat = 4;

                    sub.Create();
                    SubscriptionsCache = null;

                    if (addr != null)
                        return new AmtAlertSubscription(0, addr, policyId);
                    return new AmtAlertSubscription(0, hostname, policyId);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.SubscribeForAlerts WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.SubscribeForAlerts CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.SubscribeForAlerts error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return null;
                }
            }
            else
            {
                // SOAP
                if (parent.CoreVersion >= 3.0)
                {
                    AlertSubscriptionGeneralType a = new AlertSubscriptionGeneralType();
                    a.PolicyID = policyId;
                    AlertSubscriptionSNMPType b = new AlertSubscriptionSNMPType();
                    b.Address = new NodeAddressType();

                    if (addr != null)
                    {
                        // IPv4 or IPv6 address
                        if (addr.AddressFamily == AddressFamily.InterNetwork) b.Address.ItemElementName = EventItemChoiceType.IPv4Address;
                        if (addr.AddressFamily == AddressFamily.InterNetworkV6) b.Address.ItemElementName = EventItemChoiceType.IPv6Address;
                        b.Address.Item = addr.ToString();
                    }
                    else
                    {
                        // Hostname
                        b.Address.ItemElementName = EventItemChoiceType.HostName;
                        b.Address.Item = hostname;
                    }

                    b.CommunityString = "Public";
                    a.Item = b;
                    uint handle;
                    AmtCallStatus r = (AmtCallStatus)service.SubscribeForGeneralAlert(a, out handle);
                    if (r != AmtCallStatus.SUCCESS) return null;
                    SubscriptionsCache = null;
                    return new AmtAlertSubscription(handle, addr, policyId);
                }
                else
                {
                    uint handle;
                    AlertSubscriptionType subscriptionInfo = new AlertSubscriptionType();
                    subscriptionInfo.AlertSubscriptionPolicyID = policyId;

                    byte[] addrbytes = addr.GetAddressBytes();
                    if (addrbytes.Length != 4) return null; // Not an IPv4 address
                    subscriptionInfo.DestinationAddress = (uint)((addrbytes[0] << 24) | (addrbytes[1] << 16) | (addrbytes[2] << 8) | (addrbytes[3]));
                    SubscriptionsCache = null;
                    AmtCallStatus r = (AmtCallStatus)service.SubscribeForAlert(subscriptionInfo, out handle);
                    if (r != AmtCallStatus.SUCCESS) return null;
                    SubscriptionsCache = null;
                    return new AmtAlertSubscription(handle, addr, policyId);
                }
            }
        }

        public AmtCallStatus CancelAlertSubscription(AmtAlertSubscription subscription)
        {
            if (parent.WsMan != null)
            {
                // WSMAN
                try
                {
                    parent.WsMan.Delete(subscription.Wsman.Reference);
                    SubscriptionsCache = null;
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.CancelAlertSubscription WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.CancelAlertSubscription CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.CancelAlertSubscription error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                SubscriptionsCache = null;
                return (AmtCallStatus)service.CancelAlertSubscription(subscription.Handle);
            }
        }

        private AmtAlertSubscription[] SubscriptionsCache = null;
        public AmtAlertSubscription[] GetAlertSubscriptions()
        {
            if (service == null) return null;
            if (SubscriptionsCache != null) return SubscriptionsCache;

            if (parent.WsMan != null)
            {
                // WSMAN
                try
                {
                    Collection<AMT_SNMPEventSubscriber> subs = AMT_SNMPEventSubscriber.Enumerate(parent.WsMan);
                    if (subs.Count == 0) return new AmtAlertSubscription[0];
                    ArrayList l = new ArrayList();
                    foreach (AMT_SNMPEventSubscriber sub in subs)
                    {
                        IPAddress addr = null;
                        IPAddress.TryParse(sub.AccessInfo, out addr);
                        AmtAlertSubscription s;
                        if (addr != null)
                            s = new AmtAlertSubscription(0, addr, sub.PolicyID);
                        else
                            s = new AmtAlertSubscription(0, sub.AccessInfo, sub.PolicyID);
                        s.Wsman = sub;
                        l.Add(s);
                    }
                    SubscriptionsCache = (AmtAlertSubscription[])l.ToArray(typeof(AmtAlertSubscription));
                    return SubscriptionsCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetAlertSubscriptions WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetAlertSubscriptions CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetAlertSubscriptions error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    SubscriptionsCache = new AmtAlertSubscription[0];
                    return SubscriptionsCache;
                }
            }
            else
            {
                // SOAP
                uint totalSubcriptionCount = 0;
                uint subscriptionReturned = 0;
                uint[] subscriptionHandles = null;
                AmtCallStatus r;
                if (parent.CoreVersion >= 3.0)
                {
                    r = (AmtCallStatus)service.EnumerateGeneralAlertSubscriptions(0, false, out subscriptionHandles);
                    if (subscriptionHandles != null) subscriptionReturned = totalSubcriptionCount = (uint)subscriptionHandles.Length;
                }
                else
                {
                    r = (AmtCallStatus)service.EnumerateAlertSubscriptions(1, 0, false, out totalSubcriptionCount, out subscriptionReturned, out subscriptionHandles);
                }

                ArrayList list = new ArrayList();
                foreach (uint handle in subscriptionHandles)
                {
                    if (parent.CoreVersion >= 3.0)
                    {
                        AlertSubscriptionGeneralType subscriptionInfo;
                        service.GetGeneralAlertSubscription(handle, out subscriptionInfo);
                        if (subscriptionInfo.Item.GetType() == typeof(AlertSubscriptionSNMPType))
                        {
                            AlertSubscriptionSNMPType t = (AlertSubscriptionSNMPType)subscriptionInfo.Item;
                            if (t.Address.ItemElementName == EventItemChoiceType.IPv4Address || t.Address.ItemElementName == EventItemChoiceType.IPv6Address)
                            {
                                // IP Address
                                list.Add(new AmtAlertSubscription(handle, IPAddress.Parse(t.Address.Item), subscriptionInfo.PolicyID));
                            }
                            else
                            {
                                // Hostname
                                list.Add(new AmtAlertSubscription(handle, t.Address.Item, subscriptionInfo.PolicyID));
                            }
                        }
                        else if (subscriptionInfo.Item.GetType() == typeof(AlertSubscriptionSoapType))
                        {
                            // TODO
                        }
                    }
                    else
                    {
                        AlertSubscriptionType subscriptionInfo;
                        service.GetAlertSubscription(handle, out subscriptionInfo);
                        uint l = (uint)subscriptionInfo.DestinationAddress;
                        l = (uint)IPAddress.NetworkToHostOrder((int)l);
                        list.Add(new AmtAlertSubscription(handle, new IPAddress((long)l), subscriptionInfo.AlertSubscriptionPolicyID));
                    }
                }

                SubscriptionsCache = (AmtAlertSubscription[])list.ToArray(typeof(AmtAlertSubscription));
                return SubscriptionsCache;
            }
        }

        private class AmtEventFilterComparerClass : IComparer
        {
            int IComparer.Compare(Object x, Object y)
            {
                AmtEventFilter e1 = (AmtEventFilter)x;
                AmtEventFilter e2 = (AmtEventFilter)y;
                return (e1.Filter.AlertSubscriptionPolicyID.CompareTo(e2.Filter.AlertSubscriptionPolicyID));
            }
        }

        private AmtEventFilter[] EventFiltersCache = null;
        public AmtEventFilter[] GetEventFilters()
        {
            if (local) return new AmtEventFilter[0];

            if (parent.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    Collection<AMT_PETFilterSetting> filters = AMT_PETFilterSetting.Enumerate(parent.WsMan);

                    ArrayList list = new ArrayList();
                    foreach (AMT_PETFilterSetting filter in filters)
                    {
                        list.Add(new AmtEventFilter(filter.PolicyID, filter));
                    }
                    list.Sort(new AmtEventFilterComparerClass());
                    EventFiltersCache = (AmtEventFilter[])list.ToArray(typeof(AmtEventFilter));
                    return EventFiltersCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetEventFilters WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetEventFilters CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.GetEventFilters error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    EventFiltersCache = new AmtEventFilter[0];
                    return EventFiltersCache;
                }
            }
            else
            {
                // USE EOI
                if (EventFiltersCache != null) return EventFiltersCache;
                if (service == null) return new AmtEventFilter[0];

                uint totalEventFilterCount = 0;
                uint filterReturned = 0;
                uint[] filtersHandles = null;
                AmtCallStatus r = (AmtCallStatus)service.EnumerateEventFilters(1, 0, false, out totalEventFilterCount, out filterReturned, out filtersHandles);

                AmtEventFilter[] filters = new AmtEventFilter[filterReturned];

                int i = 0;
                foreach (uint handle in filtersHandles)
                {
                    EventFilterType filterType;
                    if (service == null) return new AmtEventFilter[0];
                    service.GetEventFilter(handle, out filterType);
                    filters[i] = new AmtEventFilter(handle, filterType);
                    i++;
                }

                EventFiltersCache = filters;
                return EventFiltersCache;
            }
        }

        public AmtCallStatus AddEventFilter(EventFilterType filter)
        {
            EventFiltersCache = null;
            if (parent.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_PETFilterSetting Filter = new AMT_PETFilterSetting(parent.WsMan);
                    Filter.PolicyID = filter.AlertSubscriptionPolicyID;
                    Filter.DeviceAddress = filter.DeviceAddress;
                    Filter.Entity = filter.Entity;
                    Filter.EntityInstance = filter.EntityInstance;
                    Filter.EventOffset = filter.EventOffset;
                    Filter.EventSensorType = filter.EventSensorType;
                    Filter.EventSeverity = filter.EventSeverity;
                    Filter.EventSourceType = filter.EventSourceType;
                    Filter.EventType = filter.EventType;
                    if ((filter.FilterAction & 0x01) != 0) { Filter.SendAlert = true; } else { Filter.SendAlert = false; }
                    if ((filter.FilterAction & 0x80) != 0) { Filter.LogOnEvent = true; } else { Filter.LogOnEvent = false; }
                    if ((filter.FilterConfiguration & 0x80) != 0) { Filter.EnableFilter = true; } else { Filter.EnableFilter = false; }
                    if ((filter.FilterConfiguration & 0x40) != 0) { Filter.OEMFilter = true; } else { Filter.OEMFilter = false; }
                    Filter.SensorNumber = filter.SensorNumber;
                    Filter.ElementName = "Intel(r) AMT PET Filter " + Filter.PolicyID.ToString();
                    Filter.InstanceID = Filter.PolicyID.ToString();

                    Filter.Create();
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.AddEventFilter WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.AddEventFilter CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.AddEventFilter error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                uint handle;
                return (AmtCallStatus)service.AddEventFilter(filter, out handle);
            }
        }

        public AmtCallStatus RemoveEventFilter(AmtEventFilter filter)
        {
            EventFiltersCache = null;
            if (parent.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    parent.WsMan.Delete(filter.Wsman.Reference);
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.RemoveEventFilter WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.RemoveEventFilter CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtEventManager.RemoveEventFilter error",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                return (AmtCallStatus)service.RemoveEventFilter(filter.Handle);
            }
        }

    }

    [Serializable]
    public class AmtEventFilter
    {
        public uint Handle;
        public EventFilterType Filter;

        [NonSerialized]
        public AMT_PETFilterSetting Wsman;

        public AmtEventFilter() {}

        public AmtEventFilter(uint handle, EventFilterType filter)
        {
            Handle = handle;
            Filter = filter;
            Wsman = null;
        }

        public AmtEventFilter(uint handle, AMT_PETFilterSetting filter)
        {
            Handle = handle;
            Wsman = filter;
            Filter = new EventFilterType();
            Filter.AlertSubscriptionPolicyID = filter.PolicyID;
            Filter.DeviceAddress = filter.DeviceAddress;
            Filter.Entity = filter.Entity;
            Filter.EntityInstance = filter.EntityInstance;
            Filter.EventOffset = filter.EventOffset;
            Filter.EventSensorType = filter.EventSensorType;
            Filter.EventSeverity = filter.EventSeverity;
            Filter.EventSourceType = filter.EventSourceType;
            Filter.EventType = filter.EventType;
            Filter.FilterAction = 0;
            if (filter.SendAlert == true) Filter.FilterAction += 0x01;
            if (filter.LogOnEvent == true) Filter.FilterAction += 0x80;
            Filter.FilterConfiguration = 0;
            if (filter.EnableFilter == true) Filter.FilterConfiguration += 0x80;
            if (filter.OEMFilter == true) Filter.FilterConfiguration += 0x40; else Filter.FilterConfiguration += 0x20;
            Filter.SensorNumber = filter.SensorNumber;
        }

        public string GetFilterString()
        {
            string ev = "";
            if (Filter.Entity == 0)
            {
                ev += ManageabilityStack.Properties.Resources.EventManager_ANYENTITY;
            }
            else
            {
                ev += AmtEventData.GetEntityTypeStr(Filter.Entity);
            }

            if (Filter.EventType == 0xFF)
            {
                ev += ManageabilityStack.Properties.Resources.EventManager_ANYTYPE;
            }
            else
            {
                ev += ", " + AmtEventData.GetEventTypeStr(Filter.EventType);
            }

            if (Filter.EventSensorType == 0xFF)
            {
                ev += ManageabilityStack.Properties.Resources.EventManager_ANYSENSOR;
            }
            else
            {
                ev += ", " + AmtEventData.GetEventSensorTypeStr(Filter.EventSensorType);
                if (Filter.EventOffset > 127)
                {
                    ev += string.Format(" (Offset {0}, {1})", Filter.EventOffset - 128, Properties.Resources.Deassert);
                }
                else if (Filter.EventOffset != 0x0F)
                {
                    ev += string.Format(" (Offset {0})", Filter.EventOffset);
                }
            }
            return ev;
        }

        public string GetActionString()
        {
            string ac = "";
            if ((Filter.FilterConfiguration & 0x80) == 0)
            {
                ac = ManageabilityStack.Properties.Resources.EventManager_DISABLEDFLT;
                //ac = "Disabled Filter";
            }
            else
            {
                if (((Filter.FilterAction & 0x80) == 0) & ((Filter.FilterAction & 0x01) == 0)) ac = ManageabilityStack.Properties.Resources.EventManager_DONOTHING;  /*"Do Nothing";*/
                if (((Filter.FilterAction & 0x80) != 0) & ((Filter.FilterAction & 0x01) == 0)) ac = ManageabilityStack.Properties.Resources.EventManager_LOGEVT;   /*"Log Event";*/
                if (((Filter.FilterAction & 0x80) == 0) & ((Filter.FilterAction & 0x01) != 0)) ac = ManageabilityStack.Properties.Resources.EventManager_ALTEVT; /*"Alert Event";*/
                if (((Filter.FilterAction & 0x80) != 0) & ((Filter.FilterAction & 0x01) != 0)) ac = ManageabilityStack.Properties.Resources.EventManager_LOGALTEVT;  /*"Log & Alert Event";*/
            }
            return ac;
        }
    }

    public class AmtAlertSubscription
    {
        public uint Handle;
        public IPAddress Address;
        public string Hostname;
        public byte FilterPolicyID;
        public AMT_SNMPEventSubscriber Wsman;

        public AmtAlertSubscription(uint handle, IPAddress address, byte filterPolicyID)
        {
            Handle = handle;
            Address = address;
            FilterPolicyID = filterPolicyID;
        }

        public AmtAlertSubscription(uint handle, string hostname, byte filterPolicyID)
        {
            Handle = handle;
            Hostname = hostname;
            FilterPolicyID = filterPolicyID;
        }
    }

    public class AmtEventData
    {
        private static string[] SystemFirmwareError =
        {
            "Unspecified.",
            "No system memory is physically installed in the system.",
            "No usable system memory, all installed memory has experienced an unrecoverable failure.",
            "Unrecoverable hard-disk/ATAPI/IDE device failure.",
            "Unrecoverable system-board failure.",
            "Unrecoverable diskette subsystem failure.",
            "Unrecoverable hard-disk controller failure.",
            "Unrecoverable PS/2 or USB keyboard failure.",
            "Removable boot media not found",
            "Unrecoverable video controller failure",
            "No video device detected",
            "Firmware (BIOS) ROM corruption detected",
            "CPU voltage mismatch (processors that share same supply have mismatched voltage requirements)",
            "CPU speed matching failure"
        };
        private static string[] m_SystemFirmwareProgress = 
        {
            "Unspecified.",
            "Memory initialization.",
            "Hard-disk initialization",
            "Secondary processor(s) initialization",
            "User authentication",
            "User-initiated system setup",
            "USB resource configuration",
            "PCI resource configuration",
            "Option ROM initialization",
            "Video initialization",
            "Cache initialization",
            "SM Bus initialization",
            "Keyboard controller initialization",
            "Embedded controller/management controller initialization",
            "Docking station attachment",
            "Enabling docking station",
            "Docking station ejection",
            "Disabling docking station",
            "Calling operating system wake-up vector",
            "Starting operating system boot process",
            "Baseboard or motherboard initialization",
            "reserved",
            "Floppy initialization",
            "Keyboard test",
            "Pointing device test",
            "Primary processor initialization"
        };

/*
TODO: DTK, RDK and Web Interface are returning different descriptions for the same event log record. 

For example,  for eventlog has following information:

Event Sensor Type: 35(watchdog 2)
Event Type: 111(sensor specific)
Event Offset: 0
RDK display description as: Watchdog. Entity as: Unspecified
Web Interface display description as: System boot failure, Source as: Intel@ AMT
DTK display description as: Unknown Event Type. Entity as: Unspecified.

For this specific eventlog, what should the description and entity be?
*/

        public static string GetEventDetailStr(byte eventSensorType, byte eventOffset, byte[] eventDataField, byte entity)
        {
            if (eventSensorType == 15) // System firmware progress
            {
                if (eventDataField.Length == 1 && eventDataField[0] == 235) return "Invalid Data";
                try
                {
                    if (eventOffset == 0)
                        return SystemFirmwareError[eventDataField[1]];
                    return m_SystemFirmwareProgress[eventDataField[1]];
                }
                catch (IndexOutOfRangeException) { }
            }

            if (eventSensorType == 18) // System event
            {
                if (eventDataField[0] == 0xAA) // Watchdog
                {
                    byte[] agent1 = new byte[4];
                    byte[] agent2 = new byte[2];
                    Array.Copy(eventDataField, 1, agent1, 0, 4);
                    Array.Copy(eventDataField, 5, agent2, 0, 2);
                    return string.Format("Agent {0}-{1}-... changed to {2}", Utils.BytesToHex(agent1), Utils.BytesToHex(agent2), AmtWatchdogRemote.TransitionStateToString((ushort)eventDataField[7]));
                }
            }

            if (eventSensorType == 5 && eventOffset == 0) // System chassis
            {
                return "Case intrusion";
            }

            if (eventSensorType == 192 && eventOffset == 0 && eventDataField[0] == 170 && eventDataField[1] == 48)
            {
                if (eventDataField[2] == 0) return "A remote Serial Over LAN session was established.";
                if (eventDataField[2] == 1) return "Remote Serial Over LAN session finished. User control was restored.";
                if (eventDataField[2] == 2) return "A remote IDE-Redirection session was established.";
                if (eventDataField[2] == 3) return "Remote IDE-Redirection session finished. User control was restored.";
            }

            if (eventSensorType == 36)
            {
                long handle = ((long)(eventDataField[1]) << 24) + ((long)(eventDataField[2]) << 16) + ((long)(eventDataField[3]) << 8) + (long)(eventDataField[4]);
                string nic = string.Format("#{0}", eventDataField[0]);
                if (eventDataField[0] == 0xAA) nic = "wired"; // TODO: Add wireless *****
                //if (eventDataField[0] == 0xAA) nic = "wireless";

                if (handle == 4294967293)
                {
                    return string.Format("All received packet filter was matched on {0} interface.", nic);
                    //return "All Received Packets";
                }
                else if (handle == 4294967292)
                {
                    return string.Format("All outbound packet filter was matched on {0} interface.", nic);
                    //return "All Sent Packets";
                }
                else if (handle == 4294967290)
                {
                    return string.Format("Spoofed packet filter was matched on {0} interface.", nic);
                    //return "Spoofed Packets";
                }

                return string.Format("Filter {0} was matched on {1} interface.", handle, nic);
            }

            if (eventSensorType == 37)
            {
                return "System firmware started (at least one CPU is properly executing).";
            }

            if (eventSensorType == 192)
            {
                if (eventDataField[2] == 0) return "Security policy invoked. Some or all network traffic (TX) was stopped.";
                if (eventDataField[2] == 2) return "Security policy invoked. Some or all network traffic (RX) was stopped.";
                return "Security policy invoked.";
            }

            if (eventSensorType == 193)
            {
                if (eventDataField[0] == 0xAA && eventDataField[1] == 0x30 && eventDataField[2] == 0x00 && eventDataField[3] == 0x00)
                {
                    return "User request for remote connection.";
                }
                else if (eventDataField[0] == 0xAA && eventDataField[1] == 0x20 && eventDataField[2] == 0x03 && eventDataField[3] == 0x01)
                {
                    return "EAC error: attempt to get posture while NAC in Intel AMT is disabled."; // eventDataField = 0xAA20030100000000
                }
            }

            if (eventSensorType == 6)
            {
                return string.Format("Authentication failed {0} times. The system may be under attack.", (long)eventDataField[1] + ((long)eventDataField[2] << 8));
            }

            if (eventSensorType == 35)
            {
                return "System boot failure";
            }

            if (eventSensorType == 32)
            {
                return "Operating system lockup or power interrupt";
            }

            return string.Format("Unknown Sensor Type #{0}", eventSensorType); ;
        }
       
        public static string[] EventTypes = {

		    ManageabilityStack.Properties.Resources.EventManager_EvtType_0,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_1,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_2,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_3,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_4,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_5,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_6,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_7,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_8,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_9,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_10,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_11,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_12,
            ManageabilityStack.Properties.Resources.EventManager_EvtType_13

            /*
		    "Unspecified",
		    "Threshold",
		    "Discrete usage",
		    "Discrete state",
		    "Discrete predictive failure",
		    "Discrete limit exceeded",
		    "Discrete performance met",
		    "Discrete severity",
		    "Discrete device presence",
		    "Discrete device enable",
		    "Discrete availability",
		    "Discrete redundancy",
		    "Discrete ACPI power",
		    "Sensor specific"*/
	    };

        public static string GetEventTypeStr(ushort eventType)
        {
            if (eventType == 112) return "Intel Specific"; // TODO
            try
            {
                return EventTypes[eventType];
            }
            catch (IndexOutOfRangeException)
            {
                return ManageabilityStack.Properties.Resources.EventManager_EGetEvtType;
                //return "Unknown Event Type";
            }
        }

        public static string[] EventSensorTypes = {

		    ManageabilityStack.Properties.Resources.EventManager_ESensorType_0,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_1,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_2,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_3,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_4,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_5,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_6,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_7,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_8,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_9,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_10,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_11,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_12,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_13,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_14,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_15,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_16,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_17,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_18,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_19,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_20,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_21,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_22,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_23,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_24,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_25,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_26,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_27,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_28,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_29,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_30,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_31,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_32,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_33,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_34,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_35,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_36,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_37,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_38,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_39,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_40,
            ManageabilityStack.Properties.Resources.EventManager_ESensorType_41

            /*
		    "Unspecified",
		    "Temperature",
		    "Voltage",
		    "Current",
		    "Fan",
		    "Physical security",
		    "Platform security",
		    "Processor",
		    "Power supply",
		    "Power unit",
		    "Cooling device",
		    "Other units based sensor",
		    "Memory",
		    "Drive slot",
		    "Power memory resize",
		    "System firmware progress",
		    "Event logging disabled",
		    "Watchdog 1",
		    "System event",
		    "Critical interrupt",
		    "Button",
		    "Module board",
		    "Microcontroller coprocessor",
		    "Add in card",
		    "Chassis",
		    "Chip set",
		    "Other FRU",
		    "Cable interconnect",
		    "Terminator",
		    "System boot initiated",
		    "Boot error",
		    "OS boot",
		    "OS critical stop",
		    "Slot connector",
		    "System ACPI power state",
		    "Watchdog 2",
		    "Platform alert",
		    "Entity presence",
		    "Monitor ASIC IC",
		    "LAN",
		    "Management subsystem health",
		    "Battery",*/
	    };

        public static string GetEventSensorTypeStr(ushort eventSensorType)
        {
            if (eventSensorType == 0xFF) return ManageabilityStack.Properties.Resources.EventManager_EGetSensorType_0;
            if (eventSensorType == 192) return "Intel Specific"; // TODO: Figure out what this value is.
            if (eventSensorType == 193) return "Intel AMT diagnostics";
            try
            {
                return EventSensorTypes[eventSensorType];
            }
            catch (IndexOutOfRangeException)
            {
                return ManageabilityStack.Properties.Resources.EventManager_EGetSensorType_1;
            }
        }

        public static string[] EntityTypes = {
            ManageabilityStack.Properties.Resources.EventManager_EnType_0,
            ManageabilityStack.Properties.Resources.EventManager_EnType_1,
            ManageabilityStack.Properties.Resources.EventManager_EnType_2,
            ManageabilityStack.Properties.Resources.EventManager_EnType_3,
            ManageabilityStack.Properties.Resources.EventManager_EnType_4,
            ManageabilityStack.Properties.Resources.EventManager_EnType_5,
            ManageabilityStack.Properties.Resources.EventManager_EnType_6,
            ManageabilityStack.Properties.Resources.EventManager_EnType_7,
            ManageabilityStack.Properties.Resources.EventManager_EnType_8,
            ManageabilityStack.Properties.Resources.EventManager_EnType_9,
            ManageabilityStack.Properties.Resources.EventManager_EnType_10,
            ManageabilityStack.Properties.Resources.EventManager_EnType_11,
            ManageabilityStack.Properties.Resources.EventManager_EnType_12,
            ManageabilityStack.Properties.Resources.EventManager_EnType_13,
            ManageabilityStack.Properties.Resources.EventManager_EnType_14,
            ManageabilityStack.Properties.Resources.EventManager_EnType_15,
            ManageabilityStack.Properties.Resources.EventManager_EnType_16,
            ManageabilityStack.Properties.Resources.EventManager_EnType_17,
            ManageabilityStack.Properties.Resources.EventManager_EnType_18,
            ManageabilityStack.Properties.Resources.EventManager_EnType_19,
            ManageabilityStack.Properties.Resources.EventManager_EnType_20,
            ManageabilityStack.Properties.Resources.EventManager_EnType_21,
            ManageabilityStack.Properties.Resources.EventManager_EnType_22,
            ManageabilityStack.Properties.Resources.EventManager_EnType_23,
            ManageabilityStack.Properties.Resources.EventManager_EnType_24,
            ManageabilityStack.Properties.Resources.EventManager_EnType_25,
            ManageabilityStack.Properties.Resources.EventManager_EnType_26,
            ManageabilityStack.Properties.Resources.EventManager_EnType_27,
            ManageabilityStack.Properties.Resources.EventManager_EnType_28,
            ManageabilityStack.Properties.Resources.EventManager_EnType_29,
            ManageabilityStack.Properties.Resources.EventManager_EnType_30,
            ManageabilityStack.Properties.Resources.EventManager_EnType_31,
            ManageabilityStack.Properties.Resources.EventManager_EnType_32,
            ManageabilityStack.Properties.Resources.EventManager_EnType_33,
            ManageabilityStack.Properties.Resources.EventManager_EnType_34,
            ManageabilityStack.Properties.Resources.EventManager_EnType_35,
            ManageabilityStack.Properties.Resources.EventManager_EnType_36,
            ManageabilityStack.Properties.Resources.EventManager_EnType_37,
            ManageabilityStack.Properties.Resources.EventManager_EnType_38,
            ManageabilityStack.Properties.Resources.EventManager_EnType_39,
            ManageabilityStack.Properties.Resources.EventManager_EnType_40,
            ManageabilityStack.Properties.Resources.EventManager_EnType_41,
            ManageabilityStack.Properties.Resources.EventManager_EnType_42,
            ManageabilityStack.Properties.Resources.EventManager_EnType_43,
            ManageabilityStack.Properties.Resources.EventManager_EnType_44,
            ManageabilityStack.Properties.Resources.EventManager_EnType_45,
            ManageabilityStack.Properties.Resources.EventManager_EnType_46,
            ManageabilityStack.Properties.Resources.EventManager_EnType_47,
            ManageabilityStack.Properties.Resources.EventManager_EnType_48,
            ManageabilityStack.Properties.Resources.EventManager_EnType_49,
            ManageabilityStack.Properties.Resources.EventManager_EnType_50,
            ManageabilityStack.Properties.Resources.EventManager_EnType_51,
            ManageabilityStack.Properties.Resources.EventManager_EnType_52

            /*
		    "Unspecified",                
		    "Other",
		    "Unknown",
		    "Processor",
		    "Disk",
		    "Peripheral",
		    "System management module",
		    "System board",
		    "Memory module",
		    "Processor module",
		    "Power supply",               
		    "Add in card",
		    "Front panel board",
		    "Back panel board",
		    "Power system board",
		    "Drive backplane",
		    "System internal expansion board",
		    "Other system board",
		    "Processor board",
		    "Power unit",
		    "Power module",                
		    "Power management board",
		    "Chassis back panel board",
		    "System chassis",
		    "Sub chassis",
		    "Other chassis board",
		    "Disk drive bay",
		    "Peripheral bay",
		    "Device bay",
		    "Fan cooling",
		    "Cooling unit",                 
		    "Cable interconnect",
		    "Memory device",
		    "System management software",
		    "BIOS",
		    "Operating system",
		    "System bus",
		    "Group",
		    "Remote mgmt comm device",
		    "External environment",
		    "Battery",                      
		    "Processing blade",
		    "Connectivity switch",
		    "Processor/memory module",
		    "I/O module",
		    "Processor I/O module",
		    "Management controller firmware",
		    "IPMI channel",
		    "PCI bus",
		    "PCI express bus",
		    "SCSI bus",                     
		    "SATA/SAS bus",
		    "Processor front side bus",*/
	    };

        public static string GetEntityTypeStr(ushort entityType)
        {
            if (entityType == 0xFF) return ManageabilityStack.Properties.Resources.EventManager_EGetETTType_0;
            try
            {
                return EntityTypes[entityType];
            }
            catch (IndexOutOfRangeException)
            {
                return ManageabilityStack.Properties.Resources.EventManager_EGetETTType_1;
            }
        }

        public static string[] EventTrapSourceTypes = {
		    ManageabilityStack.Properties.Resources.EventManager_ETrapSrcType_0,
            ManageabilityStack.Properties.Resources.EventManager_ETrapSrcType_1,
            ManageabilityStack.Properties.Resources.EventManager_ETrapSrcType_2,
            ManageabilityStack.Properties.Resources.EventManager_ETrapSrcType_3,
            ManageabilityStack.Properties.Resources.EventManager_ETrapSrcType_4,
            ManageabilityStack.Properties.Resources.EventManager_ETrapSrcType_5,
            ManageabilityStack.Properties.Resources.EventManager_ETrapSrcType_6,
            ManageabilityStack.Properties.Resources.EventManager_ETrapSrcType_7,
            ManageabilityStack.Properties.Resources.EventManager_ETrapSrcType_8,
            ManageabilityStack.Properties.Resources.EventManager_ETrapSrcType_9,
            ManageabilityStack.Properties.Resources.EventManager_ETrapSrcType_10,
            ManageabilityStack.Properties.Resources.EventManager_ETrapSrcType_11
            /*
		    "Platform firmware (e.g. BIOS)",
		    "SMI handler",
		    "ISV system management software",
		    "Alert ASIC",
		    "IPMI",
		    "BIOS vendor",
		    "System board set vendor",
		    "System integrator",
		    "Third party add-in",
		    "OSV",
		    "NIC",
		    "System management card"
             */
	    };

        public static string GetEventTrapSourceStr(ushort eventTrapSource)
        {
            if (eventTrapSource == 0xFF) return ManageabilityStack.Properties.Resources.EventManager_ETrapSrc_0;
            if (eventTrapSource == 88) return "Intel Specific"; // TODO: Figure out what this value is.
            try
            {
                return EventTrapSourceTypes[eventTrapSource];
            }
            catch (IndexOutOfRangeException)
            {
                return ManageabilityStack.Properties.Resources.EventManager_ETrapSrc_1;
            }
        }

        public static string GetEventSeverityString(byte ev)
        {
            string s = "";
            if ((ev & 0x01) != 0) s += ManageabilityStack.Properties.Resources.EventManager_ESvt_0;
            if ((ev & 0x02) != 0) s += ManageabilityStack.Properties.Resources.EventManager_ESvt_1;
            if ((ev & 0x04) != 0) s += ManageabilityStack.Properties.Resources.EventManager_ESvt_2;
            if ((ev & 0x08) != 0) s += ManageabilityStack.Properties.Resources.EventManager_ESvt_3;
            if ((ev & 0x10) != 0) s += ManageabilityStack.Properties.Resources.EventManager_ESvt_4;
            if ((ev & 0x20) != 0) s += ManageabilityStack.Properties.Resources.EventManager_ESvt_5;
            if (s.Length > 2)
            {
                s = s.Substring(2);
            }
            else
            {
                s = ManageabilityStack.Properties.Resources.EventManager_ESvt_6;
            }
            return s;
        }

        public enum EventTypesEnum
        {
            Unspecified,
            Threshold,
            Discrete_usage,
            Discrete_state,
            Discrete_predictive_failure,
            Discrete_limit_exceeded,
            Discrete_performance_met,
            Discrete_severity,
            Discrete_device_presence,
            Discrete_device_enable,
            Discrete_availability,
            Discrete_redundancy,
            Discrete_acpi_power,
            Sensor_specific,
            Intel_specific = 112,
            MatchAll = 255
        };

        public static string[] GetEventOffsets(byte eventType)
        {
            if (eventType == (int)EventTypesEnum.Threshold) return new string[] { 
                ManageabilityStack.Properties.Resources.EventManager_EOThr_0,
                ManageabilityStack.Properties.Resources.EventManager_EOThr_1,
                ManageabilityStack.Properties.Resources.EventManager_EOThr_2,
                ManageabilityStack.Properties.Resources.EventManager_EOThr_3,
                ManageabilityStack.Properties.Resources.EventManager_EOThr_4,
                ManageabilityStack.Properties.Resources.EventManager_EOThr_5,
                ManageabilityStack.Properties.Resources.EventManager_EOThr_6,
                ManageabilityStack.Properties.Resources.EventManager_EOThr_7,
                ManageabilityStack.Properties.Resources.EventManager_EOThr_8,
                ManageabilityStack.Properties.Resources.EventManager_EOThr_9,
                ManageabilityStack.Properties.Resources.EventManager_EOThr_10,
                ManageabilityStack.Properties.Resources.EventManager_EOThr_11};

            /*
        "Lower Non-critical - going low", 
        "Lower Non-critical - going high", 
        "Lower Critical - going low", 
        "Lower Critical - going high", 
        "Lower Non-recoverable - going low", 
        "Lower Non-recoverable - going high", 
        "Upper Non-critical - going low",
        "Upper Non-critical - going high", 
        "Upper Critical - going low",
        "Upper Critical - going high", 
        "Upper Non-recoverable - going low",
        "Upper Non-recoverable - going high" };*/

            if (eventType == (int)EventTypesEnum.Discrete_usage) return new string[] { 
                ManageabilityStack.Properties.Resources.EventManager_EOUsg_0,
                ManageabilityStack.Properties.Resources.EventManager_EOUsg_1, 
                ManageabilityStack.Properties.Resources.EventManager_EOUsg_2 };

            if (eventType == (int)EventTypesEnum.Discrete_state) return new string[] { 
                ManageabilityStack.Properties.Resources.EventManager_EOStt_0, 
                ManageabilityStack.Properties.Resources.EventManager_EOStt_1 };
            if (eventType == (int)EventTypesEnum.Discrete_predictive_failure) return new string[] { 
                ManageabilityStack.Properties.Resources.EventManager_EOPrf_0,
                ManageabilityStack.Properties.Resources.EventManager_EOPrf_1 };
            if (eventType == (int)EventTypesEnum.Discrete_limit_exceeded) return new string[] { 
                ManageabilityStack.Properties.Resources.EventManager_EOLmt_0,
                ManageabilityStack.Properties.Resources.EventManager_EOLmt_1 };
            if (eventType == (int)EventTypesEnum.Discrete_performance_met) return new string[] { 
                ManageabilityStack.Properties.Resources.EventManager_EOPfm_0,
                ManageabilityStack.Properties.Resources.EventManager_EOPfm_1 };

            if (eventType == (int)EventTypesEnum.Discrete_severity) return new string[] { 
                ManageabilityStack.Properties.Resources.EventManager_EOSvt_0,
                ManageabilityStack.Properties.Resources.EventManager_EOSvt_1,
                ManageabilityStack.Properties.Resources.EventManager_EOSvt_2,
                ManageabilityStack.Properties.Resources.EventManager_EOSvt_3,
                ManageabilityStack.Properties.Resources.EventManager_EOSvt_4,
                ManageabilityStack.Properties.Resources.EventManager_EOSvt_5,
                ManageabilityStack.Properties.Resources.EventManager_EOSvt_6,
                ManageabilityStack.Properties.Resources.EventManager_EOSvt_7,
                ManageabilityStack.Properties.Resources.EventManager_EOSvt_8};
            /*
            "Transition to OK", 
            "Transition to Non-Critical from OK", 
            "Transition to Critical from Less Severe", 
            "Transition to Non-recoverable from Less Severe",
            "Transition to Non-Critical from More Severe",
            "Transition to Critical from Non-recoverable",
            "Transition to Non-recoverable", 
            "Monitor", 
            "Informational" };*/

            if (eventType == (int)EventTypesEnum.Discrete_device_presence) return new string[] {
                ManageabilityStack.Properties.Resources.EventManager_EOSDpr_0,
                ManageabilityStack.Properties.Resources.EventManager_EOSDpr_0 };
            if (eventType == (int)EventTypesEnum.Discrete_device_enable) return new string[] { 
                ManageabilityStack.Properties.Resources.EventManager_EOSDen_0, 
                ManageabilityStack.Properties.Resources.EventManager_EOSDen_0 };
            if (eventType == (int)EventTypesEnum.Discrete_availability) return new string[] { 
                ManageabilityStack.Properties.Resources.EventManager_EOSAva_0,
                ManageabilityStack.Properties.Resources.EventManager_EOSAva_1,
                ManageabilityStack.Properties.Resources.EventManager_EOSAva_2,
                ManageabilityStack.Properties.Resources.EventManager_EOSAva_3,
                ManageabilityStack.Properties.Resources.EventManager_EOSAva_4,
                ManageabilityStack.Properties.Resources.EventManager_EOSAva_5,
                ManageabilityStack.Properties.Resources.EventManager_EOSAva_6,
                ManageabilityStack.Properties.Resources.EventManager_EOSAva_7,
                ManageabilityStack.Properties.Resources.EventManager_EOSAva_8};

            /*
            "Transition to Running", 
            "Transition to In Test", 
            "Transition to Power Off",
            "Transition to On Line",
            "Transition to Off Line", 
            "Transition to Off Duty", 
            "Transition to Degraded", 
            "Transition to Power Save", 
            "Install Error" };*/

            if (eventType == (int)EventTypesEnum.Discrete_redundancy) return new string[] { 
                ManageabilityStack.Properties.Resources.EventManager_EOSRdd_0, 
                ManageabilityStack.Properties.Resources.EventManager_EOSRdd_1, 
                ManageabilityStack.Properties.Resources.EventManager_EOSRdd_2 };
            if (eventType == (int)EventTypesEnum.Discrete_acpi_power) return new string[] { 
                ManageabilityStack.Properties.Resources.EventManager_EOSPwr_0, 
                ManageabilityStack.Properties.Resources.EventManager_EOSPwr_1,
                ManageabilityStack.Properties.Resources.EventManager_EOSPwr_2, 
                ManageabilityStack.Properties.Resources.EventManager_EOSPwr_3 };

            if (eventType == (int)EventTypesEnum.Intel_specific) return new string[] { 
                "A remote Serial Over LAN session was established." };  // TODO: Figure out what this value is.

            if (eventType == (int)EventTypesEnum.Sensor_specific)
            {
                string[] sensor_specific = new string[256];
                for (int i = 0; i < 256; i++)
                {
                    sensor_specific[i] = ManageabilityStack.Properties.Resources.EventManager_EOSSpc + i.ToString();
                }
                return sensor_specific;
            }

            if (eventType == (int)EventTypesEnum.MatchAll) return new string[] { 
                "0",
                "1",
                "2",
                "3",
                "4",
                "5",
                "6",
                "7",
                "8",
                "9",
                "10",
                "11",
                "12",
                "13",
                "14",
                "Match All"};

            return null;
        }

        public static string GetEventOffsetStr(byte eventType, byte offset)
        {
            string[] offests = GetEventOffsets(eventType);
            try
            {
                return offests[offset];
            }
            catch (Exception)
            {
                return null;
            }
        }

    }

    [Serializable]
    public class AmtEvent
    {
        private EventLogRecordType evt;
        private string eventSourceStr = null;
        private string eventDetailStr = null;

        public System.DateTime TimeStamp = System.DateTime.MinValue;
        public byte DeviceAddress { get { return evt.DeviceAddress; } }
        public string EventSeverityStr { get { return AmtEventData.GetEventSeverityString(evt.EventSeverity); } }
        public byte EventSeverity { get { return evt.EventSeverity; } }
        public string EntityTypeStr { get { return AmtEventData.GetEntityTypeStr(evt.Entity); } }
        public string EventSensorTypeStr { get { return AmtEventData.GetEventSensorTypeStr(evt.EventSensorType); } }
        public string EventTypesStr { get { return AmtEventData.GetEventTypeStr(evt.EventType); } }
        public string EventOffsetStr { get { return AmtEventData.GetEventOffsetStr(evt.EventType, evt.EventOffset); } }
        public string EventSourceStr { get { if (eventSourceStr == null) return AmtEventData.GetEventTrapSourceStr(evt.EventSourceType); else return eventSourceStr; } }
        public byte SensorDevice { get { return evt.SensorNumber; } }
        public string EventDetailStr
        {
            get
            {
                if (eventDetailStr == null)
                    return AmtEventData.GetEventDetailStr(
                        evt.EventSensorType,
                        evt.EventOffset,
                        evt.EventData,
                        evt.Entity);
                else
                    return eventDetailStr;
            }
        }
        public string EventDataStr
        {
            get
            {
                string o = "0x";
                foreach (byte b in evt.EventData) {o += b.ToString("X2");}
                return o;
            } 
        }
        
        public AmtEvent(EventLogRecordType evt)
        {
            this.evt = evt;
            TimeStamp = Utils.ClockToDate(evt.TimeStamp);
        }

        public AmtEvent(byte[] bytes)
        {
            evt = new EventLogRecordType();
            evt.TimeStamp = (((uint)bytes[3]) << 24) +
                             (((uint)bytes[2]) << 16) +
                             (((uint)bytes[1]) << 8) +
                             (uint)bytes[0];
            evt.DeviceAddress = bytes[4];
            evt.EventSensorType = bytes[5];
            evt.EventType = bytes[6];
            evt.EventOffset = bytes[7];
            evt.EventSourceType = bytes[8];
            evt.EventSeverity = bytes[9];
            evt.SensorNumber = bytes[10];
            evt.Entity = bytes[11];
            evt.EntityInstance = bytes[12];
            evt.EventData = new byte[8];
            for (int j = 0; j < 8; j++)
                evt.EventData[j] = bytes[13 + j];

            TimeStamp = Utils.ClockToDate(evt.TimeStamp);
        }
    }

    public class AmtEventSensor
    {
        private SensorAttributesType sensor;
        public bool AssertionEvent      { get { return sensor.AssertionEvent; } }
        public bool DeassertionEvent    { get { return sensor.DeassertionEvent; } }
        public byte DeviceAddress       { get { return sensor.DeviceAddress; } }
        public byte Entity              { get { return sensor.Entity; } }
        public string EntityStr         { get { return AmtEventData.GetEntityTypeStr(sensor.Entity); } }
        public byte EntityInstance      { get { return sensor.EntityInstance; } }
        public byte EventOffset         { get { return sensor.EventOffset; } }
        public byte EventSensorType     { get { return sensor.EventSensorType; } }
        public string EventSensorTypeStr{ get { return AmtEventData.GetEventSensorTypeStr(sensor.EventSensorType); } }
        public byte EventSeverity       { get { return sensor.EventSeverity; } }
        public byte EventSourceType     { get { return sensor.EventSourceType; } }
        public byte EventType           { get { return sensor.EventType; } }
        public byte Index               { get { return sensor.Index; } }
        public bool IsLegacySensor      { get { return sensor.IsLegacySensor; } }
        public byte SensorNumber        { get { return sensor.SensorNumber; } }

        public AmtEventSensor(SensorAttributesType sensor)
        {
            this.sensor = sensor;
        }

        public AmtEventSensor(AMT_PETCapabilities s)
        {
            sensor = new SensorAttributesType();
            // TODO
        }
    }
}
