// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Net;
using System.Text;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.Exceptions;
using Intel.Manageability.WSManagement;

namespace ManageabilityStack
{
    public class AmtGeneralInfo
    {
        private AmtSystem system;
        private GeneralInfoService generalInfoService;
        private bool cached = false;
        public bool IsCached() { return cached; }

        public AmtGeneralInfo(AmtSystem system, GeneralInfoService service)
        {
            this.system = system;
            generalInfoService = service;
        }
        
        public void Dispose()
        {
            if (generalInfoService != null) generalInfoService.Dispose();
            generalInfoService = null;
        }

        public void ClearCache()
        {
            cached = false;
            CoreVersionCache = null;
            FirmwareVersionCache = null;
            ProvisioningStateCacheSet = false;
            GetProvisioningModeCache = uint.MaxValue;
            ReadAuditLogCache = null;
            AuditLogStatusCache = null;
        }

        public void FetchCache()
        {
            try
            {
                //CapabilityDataType[] capabilities;
                //GetMeCapabilities(out capabilities);

                GetCoreVersion();
                GetProvisioningState();
                GetProvisioningMode();
                GetVersions();
                GetHostName();

                AmtAuditLogRecord[] records;
                ReadAuditLog(out records);

                AmtAuditLogStatus auditLogStatus;
                GetAuditLogStatus(out auditLogStatus);

                Guid uniqueIdentifier;
                GetUuid(out uniqueIdentifier);
                cached = true;
            }
            catch (Exception e)
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "Failure to retrieve the General Information data",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
            }
        }

        public ushort GetDHCPv6ConfigurationTimeout()
        {
            if (system.WsMan == null) return 0;

            try
            {
                AMT_GeneralSettings generalSettings = new AMT_GeneralSettings(system.WsMan);
                generalSettings.Get();

                if (generalSettings.DHCPv6ConfigurationTimeoutExist == true)
                    return generalSettings.DHCPv6ConfigurationTimeout;
                else
                    return 0;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetDHCPv6ConfigurationTimeout WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetDHCPv6ConfigurationTimeout CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetDHCPv6ConfigurationTimeout",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return 0;
            }
        }
        public AmtCallStatus SetDHCPv6ConfigurationTimeout(ushort timeout)
        {
            if (system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_GeneralSettings generalSettings = new AMT_GeneralSettings(system.WsMan);
                generalSettings.Get();
                generalSettings.DHCPv6ConfigurationTimeout = timeout;
                generalSettings.Put();
                return AmtCallStatus.SUCCESS;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.SetDHCPv6ConfigurationTimeout WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.SetDHCPv6ConfigurationTimeout CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.SetDHCPv6ConfigurationTimeout",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        public enum AmtIPAddressFamily
        {
            IPv4 = 0,
            IPv6 = 1,
        }
        public AmtIPAddressFamily GetPreferredAddressFamily()
        {
            if (system.WsMan == null) return AmtIPAddressFamily.IPv4;

            try
            {
                AMT_GeneralSettings generalSettings = new AMT_GeneralSettings(system.WsMan);
                generalSettings.Get();

                if (generalSettings.PreferredAddressFamilyExist == true)
                    return (AmtIPAddressFamily)Enum.Parse(typeof(AmtIPAddressFamily), generalSettings.PreferredAddressFamily.ToString());
                else
                    return AmtIPAddressFamily.IPv4;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetPreferredAddressFamily WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetPreferredAddressFamily CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetPreferredAddressFamily",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtIPAddressFamily.IPv4;
            }
        }
        public AmtCallStatus SetPreferredAddressFamily(AmtIPAddressFamily preferredAddrFamily)
        {
            if (system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_GeneralSettings generalSettings = new AMT_GeneralSettings(system.WsMan);
                generalSettings.Get();
                generalSettings.PreferredAddressFamily = (uint)preferredAddrFamily;
                generalSettings.Put();
                return AmtCallStatus.SUCCESS;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.SetPreferredAddressFamily WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.SetPreferredAddressFamily CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.SetPreferredAddressFamily",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        public bool GetSharedFQDN()
        {
            if (system.WsMan == null) return true;

            try
            {
                AMT_GeneralSettings generalSettings = new AMT_GeneralSettings(system.WsMan);
                generalSettings.Get();

                if (generalSettings.SharedFQDNExist == true)
                    return generalSettings.SharedFQDN;
                else
                    return true;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetSharedFQDN WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetSharedFQDN CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetSharedFQDN",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return true;
            }
        }
        public AmtCallStatus SetSharedFQDN(bool enable, string hostname, string domainName)
        {
            if (system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_GeneralSettings generalSettings = new AMT_GeneralSettings(system.WsMan);
                generalSettings.Get();
                generalSettings.SharedFQDN = enable;
                generalSettings.HostName = hostname;
                generalSettings.DomainName = domainName;
                generalSettings.Put();
                return AmtCallStatus.SUCCESS;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.SetSharedFQDN WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.SetSharedFQDN CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.SetSharedFQDN",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        public AmtCallStatus GetDDNSSettings(out bool DDNSUpdateEnabled, out uint DDNSPeriodicUpdateInterval, out uint DDNSTTL)
        {
            DDNSUpdateEnabled = false;
            DDNSPeriodicUpdateInterval = 0;
            DDNSTTL = 0;

            if (system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_GeneralSettings generalSettings = new AMT_GeneralSettings(system.WsMan);
                generalSettings.Get();

                if (generalSettings.DDNSUpdateEnabledExist == true)
                    DDNSUpdateEnabled = generalSettings.DDNSUpdateEnabled;

                if (generalSettings.DDNSPeriodicUpdateIntervalExist == true)
                    DDNSPeriodicUpdateInterval = generalSettings.DDNSPeriodicUpdateInterval;

                if (generalSettings.DDNSTTLExist == true)
                    DDNSTTL = generalSettings.DDNSTTL;

                return AmtCallStatus.SUCCESS;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetDDNSSettings WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetDDNSSettings CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetDDNSSettings",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }
        public AmtCallStatus SetDDNSSettings(bool DDNSUpdateEnabled, uint DDNSPeriodicUpdateInterval, uint DDNSTTL)
        {
            if (system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                AMT_GeneralSettings generalSettings = new AMT_GeneralSettings(system.WsMan);
                generalSettings.Get();
                generalSettings.DDNSUpdateEnabled = DDNSUpdateEnabled;
                generalSettings.DDNSPeriodicUpdateInterval = DDNSPeriodicUpdateInterval;
                generalSettings.DDNSTTL = DDNSTTL;
                generalSettings.Put();
                return AmtCallStatus.SUCCESS;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.SetDDNSSettings WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.SetDDNSSettings CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.SetDDNSSettings",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        private string CoreVersionCache = null;
        public string GetCoreVersion()
        {
            if (CoreVersionCache != null) return CoreVersionCache;
            if (system.WsMan != null) CoreVersionCache = GetCoreVersionWSMAN(); else CoreVersionCache = GetCoreVersionSOAP();
            return CoreVersionCache;
        }

        public string GetCoreVersionWSMAN()
        {
            if (system.WsMan == null) return "";
            try
            {
                Collection<CIM_SoftwareIdentity> si = CIM_SoftwareIdentity.Enumerate(system.WsMan);
                if (si.Count == 0) return null;
                CoreVersionCache = (si[0].VersionStringExist ? si[0].VersionString : String.Empty);
                return CoreVersionCache;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetCoreVersionWSMAN WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetCoreVersionWSMAN CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetCoreVersionWSMAN",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return null;
            }
        }

        public string GetCoreVersionSOAP()
        {
            generalInfoService.GetCoreVersion(out CoreVersionCache);
            return CoreVersionCache;
        }

        private Guid UuidCache = Guid.Empty;
        public AmtCallStatus GetUuid(out Guid uniqueIdentifier)
        {
            uniqueIdentifier = UuidCache;
            if (!UuidCache.Equals(Guid.Empty)) return AmtCallStatus.SUCCESS;
            if (system.CoreVersion < 4) return AmtCallStatus.NOT_PERMITTED;

            if (system.WsMan != null)
            {
                try
                {
                    byte[] UUID;
                    AMT_SetupAndConfigurationService sc = new AMT_SetupAndConfigurationService(system.WsMan);
                    AmtCallStatus r = (AmtCallStatus)sc.GetUuid(out UUID);
                    if (r != AmtCallStatus.SUCCESS) return r;
                    UuidCache = uniqueIdentifier = new Guid(UUID);
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetUuid WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetUuid CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetUuid",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                byte[] g;
                AmtCallStatus r = (AmtCallStatus)generalInfoService.GetUuid(out g);
                if (r != AmtCallStatus.SUCCESS) return r;
                uniqueIdentifier = UuidCache = new Guid(g);
                return r;
            }
        }

        private CapabilityDataType[] MeCapabilitiesCache = null;
        public AmtCallStatus GetMeCapabilities(out CapabilityDataType[] capabilities)
        {
            if (system.CoreVersion < 5 || system.WsManOnly == true)
            {
                capabilities = null;
                return AmtCallStatus.NOT_PERMITTED;
            }
            if (MeCapabilitiesCache != null)
            {
                capabilities = MeCapabilitiesCache;
                return AmtCallStatus.SUCCESS;
            }
            AmtCallStatus r = (AmtCallStatus)generalInfoService.GetMeCapabilities(out MeCapabilitiesCache);
            capabilities = MeCapabilitiesCache;
            return r;
        }

        private string FirmwareVersionCache = null;
        private FirmwareVersionType[] VersionsCache = null;
        public FirmwareVersionType[] GetVersions()
        {
            if (FirmwareVersionCache != null) return VersionsCache;
            if (system.WsMan != null)
            {
                // Get Versions
                try
                {
                    Collection<CIM_SoftwareIdentity> si = CIM_SoftwareIdentity.Enumerate(system.WsMan);
                    ArrayList l = new ArrayList();
                    foreach (CIM_SoftwareIdentity s in si)
                    {
                        FirmwareVersionType t = new FirmwareVersionType();
                        t.Description = s.InstanceID;// (s.DescriptionExist ? s.Description : String.Empty); ;
                        t.Version = (s.VersionStringExist ? s.VersionString : String.Empty);
                        l.Add(t);
                    }
                    VersionsCache = (FirmwareVersionType[])l.ToArray(typeof(FirmwareVersionType));

                    // Get BIOS version
                    Collection<CIM_BIOSElement> bs = CIM_BIOSElement.Enumerate(system.WsMan);
                    if (bs.Count > 0)
                    {
                        FirmwareVersionCache = bs[0].Version;
                    }

                    return VersionsCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetVersions WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetVersions CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetVersions",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                    if (VersionsCache != null)
                    {
                        return VersionsCache;
                    }
                    else
                    {
                        FirmwareVersionCache = ""; 
                        return null;
                    }

                }
            }
            else
            {
                generalInfoService.GetCodeVersions(out FirmwareVersionCache, out VersionsCache);
                return VersionsCache;
            }
        }

        public string GetFirmwareVersion()
        {
            if (FirmwareVersionCache == null) GetVersions();
            return FirmwareVersionCache;
        }

        public bool GetStandardManagibilitySKU()
        {
            bool ReturnVal = false; 

            FirmwareVersionType[] Versions = GetVersions();

            if (Versions == null)
                return false; 

            foreach (FirmwareVersionType version in Versions)
            {
                if(version.Description.Contains("Sku") )
                {
                    int SkuValue =0;

                    if (int.TryParse(version.Version, out SkuValue))
                    {
                        //flag for Intel Standard Manageability SKU
                        //see Network Interface Guide, GeneralInfoInterface for more info
                        if ((SkuValue & 0x00000010) == 0x00000010)
                        {
                            ReturnVal = true;
                        }
                    }

                    break;
                }
            }

            return ReturnVal; 
        }

        public string GetBuildNum()
        {
            string retVal = null; 

            FirmwareVersionType[] Versions = GetVersions();

            foreach (FirmwareVersionType version in Versions)
            {
                if (version.Description.Contains("Build Number"))
                {
                    retVal = version.Version;
                }
            }

            return retVal;
        }


        public string GetHostName()
        {
            if (system.CoreVersion == 0) return system.HostNameIdentity.DefaultName;

            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AMT_GeneralSettings s = new AMT_GeneralSettings(system.WsMan);
                    s.Get();
                    return s.HostName;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetHostName WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetHostName CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetHostName",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return null;
                }
            }
            else
            {
                // USE EOI
                string hostname;
                AmtCallStatus r = (AmtCallStatus)generalInfoService.GetHostName(out hostname);
                return hostname;
            }
        }

        public bool GetNetworkState()
        {
            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AMT_GeneralSettings s = new AMT_GeneralSettings(system.WsMan);
                    s.Get();
                    if (s.NetworkInterfaceEnabledExist == false) return false;
                    return s.NetworkInterfaceEnabled;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetNetworkState WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetNetworkState CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetNetworkState",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return false;
                }
            }
            else
            {
                // USE EOI
                bool enabled;
                AmtCallStatus r = (AmtCallStatus)generalInfoService.GetNetworkState(out enabled);
                return enabled;
            }
        }

        public bool SerialOverLanEnabled { get { return GetEnabledInterfaces().SerialOverLAN; } }
        public bool IdeRedirectEnabled { get { return GetEnabledInterfaces().IdeRedirection; } }
        public bool WebUiEnabled { get { return GetEnabledInterfaces().WebUI; } }

        private AmtSecurityAdmin.EnabledInterfaces EnabledInterfacesCache = null;
        public AmtSecurityAdmin.EnabledInterfaces GetEnabledInterfaces()
        {
            if (EnabledInterfacesCache != null) return EnabledInterfacesCache;
            AmtSecurityAdmin.EnabledInterfaces eif = new AmtSecurityAdmin.EnabledInterfaces();
            eif.IdeRedirection = false;
            eif.SerialOverLAN = false;
            eif.WebUI = false;

            if (system.AmtRev == 0)
            {
                // Emulator, enable SOL but not IDE-R
                eif.IdeRedirection = false;
                eif.SerialOverLAN = true;
                eif.WebUI = false;
                EnabledInterfacesCache = eif;
                return EnabledInterfacesCache;
            }

            if (system.LimitedFeatures == true)
            {
                // Limited features, Only WebUI and SOL work, IDE-R may work.
                eif.IdeRedirection = false;
                if (system.WsMan != null)
                {
                    try
                    {
                        AMT_BootCapabilities BootCapabilitiesObj = new AMT_BootCapabilities(system.WsMan);
                        BootCapabilitiesObj.Get();
                        eif.IdeRedirection = BootCapabilitiesObj.IDER;
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        if (wsManException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetEnabledInterfaces WSMAN error",
                            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                        }
                        CimException cimException = e as CimException;
                        if (cimException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetEnabledInterfaces CIM error",
                            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                        }

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetEnabledInterfaces",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    }
                }

                eif.SerialOverLAN = true;
                eif.WebUI = true;
                EnabledInterfacesCache = eif;
                return EnabledInterfacesCache;
            }

            if (system.AmtRev == 1)
            {
                // Intel AMT 1.0 does not support this method (Assume it's all active)
                eif.IdeRedirection = true;
                eif.SerialOverLAN = true;
                eif.WebUI = true;
                EnabledInterfacesCache = eif;
                return EnabledInterfacesCache;
            }

            if (system.WsMan != null)
            {
                // WSMAN - Get Web UI status
                try
                {
                    AMT_WebUIService wst = new AMT_WebUIService(system.WsMan);
                    wst.Get();
                    eif.WebUI = (wst.EnabledState == 2);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetEnabledInterfaces WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetEnabledInterfaces CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetEnabledInterfaces",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return null;
                }

                // Get SOL & IDE-R state
                try
                {
                    AMT_RedirectionService rs = new AMT_RedirectionService(system.WsMan);
                    rs.Get();
                    if (rs.EnabledState == 2 || rs.EnabledState == 32771)
                    {
                        eif.SerialOverLAN = true;
                        eif.IdeRedirection = true;
                    }
                    else if (rs.EnabledState == 3 || rs.EnabledState == 32768)
                    {
                        eif.SerialOverLAN = false;
                        eif.IdeRedirection = false;
                    }
                    else if (rs.EnabledState == 32769)
                    {
                        eif.SerialOverLAN = false;
                        eif.IdeRedirection = true;
                    }
                    else if (rs.EnabledState == 32770)
                    {
                        eif.SerialOverLAN = true;
                        eif.IdeRedirection = false;
                    }

                    EnabledInterfacesCache = eif;
                    return EnabledInterfacesCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetEnabledInterfaces WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetEnabledInterfaces CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetEnabledInterfaces",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return null;
                }
            }
            else
            {
                // SOAP
                EnabledInterfacesType[] ei;
                AmtCallStatus r = (AmtCallStatus)generalInfoService.GetEnabledInterfaces(out ei);
                foreach (EnabledInterfacesType i in ei)
                {
                    if (i == EnabledInterfacesType.IdeRedirection) eif.IdeRedirection = true;
                    if (i == EnabledInterfacesType.SerialOverLAN) eif.SerialOverLAN = true;
                    if (i == EnabledInterfacesType.WebUI) eif.WebUI = true;
                }
                EnabledInterfacesCache = eif;
                return EnabledInterfacesCache;
            }
        }

        public AmtCallStatus GetVlanParameters(out bool VLanMode, out ushort VLanTag)
        {
            AmtCallStatus r = AmtCallStatus.NOT_PERMITTED;
            VLanMode = false;
            VLanTag = 0;
            if (system.WsMan != null)
            {
                try
                {
                    Collection<AMT_EthernetPortSettings> eps = AMT_EthernetPortSettings.Enumerate(system.WsMan);
                    if (eps.Count > 0)
                    {
                        VLanMode = eps[0].VLANTagExist;
                        VLanTag = (eps[0].VLANTagExist ? eps[0].VLANTag : (ushort)0);
                        return AmtCallStatus.SUCCESS;
                    }
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetVlanParameters WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetVlanParameters CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetVlanParameters",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                try
                {
                    r = (AmtCallStatus)generalInfoService.GetVlanParameters(0, false, out VLanMode, out VLanTag);
                }
                catch (Exception) { }
                return r;
            }
        }

        public PasswordModelType GetPasswordModel()
        {
            if (system.WsMan != null)
            {
                try
                {
                    AMT_SetupAndConfigurationService sct = new AMT_SetupAndConfigurationService(system.WsMan);
                    sct.Get();
                    return (PasswordModelType)sct.PasswordModel;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetPasswordModel WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetPasswordModel CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetPasswordModel",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return PasswordModelType.PasswordModelSeperate;
                }
            }
            else
            {
                PasswordModelType p;
                AmtCallStatus r = (AmtCallStatus)generalInfoService.GetPasswordModel(out p);
                return p;
            }
        }

        public enum ProvisioningMode
        {
            Current = 0,
            Enterprise = 1,
            SmallBusiness = 2
        }

        private uint GetProvisioningModeCache = uint.MaxValue;
        public ProvisioningMode GetProvisioningMode()
        {
            if (GetProvisioningModeCache != uint.MaxValue) return (ProvisioningMode)GetProvisioningModeCache;

            if (system.WsMan != null)
            {
                try
                {
                    AMT_SetupAndConfigurationService sct = new AMT_SetupAndConfigurationService(system.WsMan);
                    sct.Get();
                    if (sct.ProvisioningMode == 1) GetProvisioningModeCache = (uint)ProvisioningMode.Enterprise;
                    if (sct.ProvisioningMode == 2) GetProvisioningModeCache = (uint)ProvisioningMode.SmallBusiness;
                    return (ProvisioningMode)GetProvisioningModeCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetProvisioningMode WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetProvisioningMode CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetProvisioningMode",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return (ProvisioningMode)GetProvisioningModeCache;
                }
            }
            else
            {
                AmtCallStatus r = (AmtCallStatus)generalInfoService.GetProvisioningMode(out GetProvisioningModeCache);
                return (ProvisioningMode)GetProvisioningModeCache;
            }
        }

        private ProvisioningStateType ProvisioningStateCache = ProvisioningStateType.ProvisioningStatePost;
        private bool ProvisioningStateCacheSet = false;
        public ProvisioningStateType GetProvisioningState()
        {
            if (ProvisioningStateCacheSet == true) return ProvisioningStateCache;

            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_SetupAndConfigurationService sct = new AMT_SetupAndConfigurationService(system.WsMan);
                    sct.Get();
                    ProvisioningStateCache = (ProvisioningStateType)sct.ProvisioningState;
                    ProvisioningStateCacheSet = true;
                    return ProvisioningStateCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetProvisioningState WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetProvisioningState CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetProvisioningState",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    ProvisioningStateCacheSet = false;
                    return ProvisioningStateType.ProvisioningStatePost;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r = (AmtCallStatus)generalInfoService.GetProvisioningState(out ProvisioningStateCache);
                ProvisioningStateCacheSet = true;
                return ProvisioningStateCache;
            }
        }

        public AmtCallStatus GetSecurityParameters(out bool EnterpriseMode, out bool TlsEnabled, out bool HwCryptoEnabled, out ProvisioningStateType ProvisioningState, out bool NetworkInterfaceEnabled, out bool SOLEnabled, out bool IDEREnabled, out bool FWUpdateEnabled, out bool LinkIsUp)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                AmtSecurityAdmin.EnabledInterfaces eif = GetEnabledInterfaces();
                SOLEnabled = eif.SerialOverLAN;
                IDEREnabled = eif.IdeRedirection;
                ProvisioningState = GetProvisioningState();
                EnterpriseMode = (GetProvisioningMode() == ProvisioningMode.Enterprise);

                // TODO!!!!
                TlsEnabled = false;
                HwCryptoEnabled = false;
                NetworkInterfaceEnabled = true;
                FWUpdateEnabled = true;
                LinkIsUp = true;

                return AmtCallStatus.SUCCESS;
            }
            else
            {
                // SOAP
                return (AmtCallStatus)generalInfoService.GetSecurityParameters(out EnterpriseMode, out TlsEnabled, out HwCryptoEnabled, out ProvisioningState, out NetworkInterfaceEnabled, out SOLEnabled, out IDEREnabled, out FWUpdateEnabled, out LinkIsUp);
            }
        }

        AmtRedirectEvent[] IderSessionLogCache = null;
        public AmtRedirectEvent[] GetIderSessionLog() { return GetIderSessionLog(false); }
        public AmtRedirectEvent[] GetIderSessionLog(bool nocache)
        {
            if (system.AmtRev == 1) return null;
            if (IderSessionLogCache != null && nocache == false) return IderSessionLogCache;

            if (system.WsMan != null)
            {
                try
                {
                    AMT_RedirectionService rs = new AMT_RedirectionService(system.WsMan);
                    rs.Get();
                    if (rs.AccessLogExist == false || rs.AccessLog.Length == 0)
                    {
                        IderSessionLogCache = new AmtRedirectEvent[0];
                        return IderSessionLogCache;
                    }

                    IderSessionLogCache = new AmtRedirectEvent[rs.AccessLog.Length];
                    for (int i = 0; i < rs.AccessLog.Length; i++)
                    {
                        IderSessionLogCache[i] = new AmtRedirectEvent(rs.AccessLog[i]);
                    }
                    return IderSessionLogCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetIderSessionLog WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetIderSessionLog CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }


                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetIderSessionLog",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    IderSessionLogCache = null;
                    return null;
                }
            }
            else
            {
                GenInfoIderSessionLogEntryType[] logData;
                AmtCallStatus r = (AmtCallStatus)generalInfoService.GetIderSessionLog(out logData);

                if (logData == null)
                {
                    IderSessionLogCache = new AmtRedirectEvent[0];
                    return IderSessionLogCache;
                }

                IderSessionLogCache = new AmtRedirectEvent[logData.Length];
                for (int i = 0; i < logData.Length; i++)
                {
                    IderSessionLogCache[i] = new AmtRedirectEvent(logData[i]);
                }
                return IderSessionLogCache;
            }
        }

        public enum AmtAuditLogStateMask
        {
            DISABLED = 0x01,
            LOCKED = 0x02,
            ALMOST_FULL = 0x04,
            FULL = 0x08,
            NO_KEY = 0x10
        }


        public class AmtAuditLogStatus
        {
            public uint PercentageFree;
            public uint NumberOfRecords;
            public System.DateTime TimeOfLastRecord;
            public bool Disabled;
            public bool Locked;
            public bool AlmostFull;
            public bool Full;
            public bool NoKey;
            public uint MaxAllowedAuditors;
        }

        public void ClearAuditLogStatusCache()
        {
            AuditLogStatusCache = null;
        }

        private AmtAuditLogStatus AuditLogStatusCache = null;
        public AmtCallStatus GetAuditLogStatus(out AmtAuditLogStatus auditLogStatus)
        {
            auditLogStatus = null;
            if (system.CoreVersion < 4 || system.LimitedFeatures == true)
            {
                auditLogStatus = null;
                return AmtCallStatus.NOT_PERMITTED;
            }
            if (AuditLogStatusCache != null)
            {
                auditLogStatus = AuditLogStatusCache;
                return AmtCallStatus.SUCCESS;
            }

            if (system.WsMan != null)
            {
                try
                {
                    AMT_AuditLog al = new AMT_AuditLog(system.WsMan);
                    al.Get();
                    auditLogStatus = new AmtAuditLogStatus();
                    auditLogStatus.AlmostFull = ((al.AuditState & 0x04) != 0);
                    auditLogStatus.Disabled = ((al.AuditState & 0x01) != 0);
                    auditLogStatus.Full = ((al.AuditState & 0x08) != 0);
                    auditLogStatus.Locked = ((al.AuditState & 0x02) != 0);
                    auditLogStatus.MaxAllowedAuditors = al.MaxAllowedAuditors;
                    auditLogStatus.NoKey = ((al.AuditState & 0x10) != 0);
                    auditLogStatus.NumberOfRecords = (uint)al.CurrentNumberOfRecords;
                    auditLogStatus.PercentageFree = al.PercentageFree;
                    if (al.TimeOfLastRecordExist)
                    {
                        try
                        {
                            auditLogStatus.TimeOfLastRecord = System.DateTime.ParseExact(al.TimeOfLastRecord.StringValue(), "yyyy-MM-ddTHH:mm:ssZ", null);
                        }
                        catch (Exception)
                        {
                            // TODO: Determine why the TimeOfLastRecord property doesn't exist when it says it does on AMT5.
                            auditLogStatus.TimeOfLastRecord = DateTime.MinValue;
                        }
                    }
                    AuditLogStatusCache = auditLogStatus;
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetAuditLogStatus WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetAuditLogStatus CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetAuditLogStatus",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                uint state;
                uint timeoflast;
                auditLogStatus = new AmtAuditLogStatus();
                AmtCallStatus r = (AmtCallStatus)generalInfoService.GetAuditLogStatus(out auditLogStatus.PercentageFree, out auditLogStatus.NumberOfRecords, out timeoflast, out state, out auditLogStatus.MaxAllowedAuditors);
                if (r != AmtCallStatus.SUCCESS) { auditLogStatus = null; return r; }
                auditLogStatus.Disabled = (state & (uint)AmtAuditLogStateMask.DISABLED) != 0;
                auditLogStatus.Locked = (state & (uint)AmtAuditLogStateMask.LOCKED) != 0;
                auditLogStatus.AlmostFull = (state & (uint)AmtAuditLogStateMask.ALMOST_FULL) != 0;
                auditLogStatus.Full = (state & (uint)AmtAuditLogStateMask.FULL) != 0;
                auditLogStatus.NoKey = (state & (uint)AmtAuditLogStateMask.NO_KEY) != 0;
                auditLogStatus.TimeOfLastRecord = Utils.ClockToDate(timeoflast);
                AuditLogStatusCache = auditLogStatus;
                return r;
            }
        }

        public AmtCallStatus GetAuditPolicy(out AuditPolicyType[] auditPolicyRecords)
        {
            if (system.CoreVersion < 4 || system.LimitedFeatures == true)
            {
                auditPolicyRecords = null;
                return AmtCallStatus.NOT_PERMITTED;
            }

            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_AuditPolicyRule ap = new AMT_AuditPolicyRule(system.WsMan);
                    ap.Get();
                    auditPolicyRecords = new AuditPolicyType[ap.AuditApplicationEventID.Length];
                    for (int i = 0; i < ap.AuditApplicationEventID.Length; i++)
                    {
                        auditPolicyRecords[i] = new AuditPolicyType();
                        auditPolicyRecords[i].AuditedAppID = (ushort)(ap.AuditApplicationEventID[i] >> 16);
                        auditPolicyRecords[i].EventId = (ushort)(ap.AuditApplicationEventID[i] & 0xFFFF);
                        auditPolicyRecords[i].Flag = ap.PolicyType[i];
                        auditPolicyRecords[i].FlagSpecified = true;
                    }
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetAuditPolicy WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetAuditPolicy CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.GetAuditPolicy",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    auditPolicyRecords = null;
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r = (AmtCallStatus)generalInfoService.GetAuditPolicy(out auditPolicyRecords);
                return r;
            }
        }

        public enum AmtAuditlogInitiator
        {
            HttpDigest = 0,
            Kerberos = 1,
            Local = 2
        }

        public enum AmtAuditLogMCLocation
        {
            IPv4Address,
            IPv6Address,
            HostName
        }

        public class AmtAuditLogRecord
        {
            public AmtAuditLog.AmtAuditApplicationName AuditAppID;
            public AmtAuditLog.AmtAuditEventName EventID;
            public AmtAuditlogInitiator Initiator;
            public string HttpDigestUsername = null;
            public uint KerberosUserInDomain = 0;
            public string KerberosDomain = null;
            public System.DateTime TimeStamp;
            public AmtAuditLogMCLocation ConsoleLocation;
            public string Address = null;
            public byte[] ExtendedData;
    
            public AmtAuditLogRecord(byte[] buf)
            {
                int ptr = 0;
                AuditAppID = (AmtAuditLog.AmtAuditApplicationName)((buf[ptr++] << 8) + buf[ptr++]);
                EventID = (AmtAuditLog.AmtAuditEventName)(((buf[ptr++] << 8) + buf[ptr++]) + ((uint)AuditAppID * 1000));
                Initiator = (AmtAuditlogInitiator)buf[ptr++];
                if (Initiator == AmtAuditlogInitiator.HttpDigest)
                {
                    byte username_size = buf[ptr++];
                    HttpDigestUsername = UTF8Encoding.UTF8.GetString(buf, ptr, username_size);
                    ptr += username_size;
                    if (HttpDigestUsername.EndsWith("\0")) HttpDigestUsername = HttpDigestUsername.Substring(0, HttpDigestUsername.Length - 1);
                }
                else if (Initiator == AmtAuditlogInitiator.Kerberos)
                {
                    KerberosUserInDomain = BitConverter.ToUInt32(buf, ptr);
                    ptr += 4;
                    byte domain_size = buf[ptr++];
                    KerberosDomain = UTF8Encoding.UTF8.GetString(buf, ptr, domain_size);
                    ptr += domain_size;
                }
                uint time = (((uint)buf[ptr++]) << 24) + ((uint)(buf[ptr++] << 16)) + ((uint)(buf[ptr++] << 8)) + ((uint)(buf[ptr++]));
                TimeStamp = Utils.ClockToDate(time);
                ConsoleLocation = (AmtAuditLogMCLocation)buf[ptr++];
                
                byte address_size = buf[ptr++];
                Address = UTF8Encoding.UTF8.GetString(buf, ptr, address_size);
                if (Address.EndsWith("\0")) Address = Address.Substring(0, Address.Length - 1);
                if (ConsoleLocation == AmtAuditLogMCLocation.IPv4Address)
                {
                    string[] splt = Address.Split('.');
                    Address = string.Format("{0}.{1}.{2}.{3}", int.Parse(splt[0]), int.Parse(splt[1]), int.Parse(splt[2]), int.Parse(splt[3]));
                }
                else if (ConsoleLocation == AmtAuditLogMCLocation.IPv6Address) Address = IPAddress.Parse(Address).ToString();

                ptr += address_size;

                byte data_size = buf[ptr++];
                ExtendedData = new byte[data_size];
                Array.Copy(buf, ptr, ExtendedData, 0, data_size);
                ptr += data_size;
            }
        }

        public void ClearAuditLogCache() { ReadAuditLogCache = null; AuditLogStatusCache = null; }
        public bool IsAuditLogCached() { return ReadAuditLogCache != null; }
        private AmtAuditLogRecord[] ReadAuditLogCache = null;
        public AmtCallStatus ReadAuditLog(out AmtAuditLogRecord[] records)
        {
            records = null;
            if (system.CoreVersion < 4 || system.LimitedFeatures == true) return AmtCallStatus.NOT_PERMITTED;
            if (ReadAuditLogCache != null)
            {
                records = ReadAuditLogCache;
                return AmtCallStatus.SUCCESS;
            }

            if (system.WsMan != null)
            {
                ArrayList recs = new ArrayList();
                uint totalRecordCount = 0;
                uint count = 1;
                AmtCallStatus r = AmtCallStatus.SUCCESS;
                do
                {
                    try
                    {
                        AMT_AuditLog auditLog = new AMT_AuditLog(system.WsMan);
                        uint recordsReturned;
                        string[] eventRecords;
                        r = (AmtCallStatus)auditLog.ReadRecords(count, out totalRecordCount, out recordsReturned, out eventRecords);
                        if (r != AmtCallStatus.SUCCESS) break;
                        count += recordsReturned;
                        foreach (string b in eventRecords) recs.Add(new AmtAuditLogRecord(Convert.FromBase64String(b)));
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        if (wsManException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.ReadAuditLog WSMAN error",
                            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                        }
                        CimException cimException = e as CimException;
                        if (cimException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.ReadAuditLog CIM error",
                            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                        }

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.ReadAuditLog",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                        r = AmtCallStatus.FAILED_WEB_CALL;
                        break;
                    }
                }
                while (count < totalRecordCount || r != AmtCallStatus.FAILED_WEB_CALL);

                recs.Sort(new AmtAuditLogComparerClass());
                ReadAuditLogCache = (AmtAuditLogRecord[])recs.ToArray(typeof(AmtAuditLogRecord));
                records = ReadAuditLogCache;
                return AmtCallStatus.SUCCESS;
            }
            else
            {
                ArrayList recs = new ArrayList();
                uint totalRecordCount = 0;
                uint count = 1;
                do
                {
                    byte[][] buf;
                    AmtCallStatus r = (AmtCallStatus)generalInfoService.ReadAuditLogRecords(count, out totalRecordCount, out buf);
                    if (r != AmtCallStatus.SUCCESS) return r;
                    if (buf != null)
                    {
                        count += (uint)(buf.Length);
                        foreach (byte[] b in buf) recs.Add(new AmtAuditLogRecord(b));
                    }
                }
                while (count < totalRecordCount);

                recs.Sort(new AmtAuditLogComparerClass());
                ReadAuditLogCache = (AmtAuditLogRecord[])recs.ToArray(typeof(AmtAuditLogRecord));
                records = ReadAuditLogCache;
                return AmtCallStatus.SUCCESS;
            }
        }

        private class AmtAuditLogComparerClass : IComparer
        {
            int IComparer.Compare(Object x, Object y)
            {
                AmtAuditLogRecord e1 = (AmtAuditLogRecord)x;
                AmtAuditLogRecord e2 = (AmtAuditLogRecord)y;
                return (e2.TimeStamp.CompareTo(e1.TimeStamp));
            }
        }

    }
}
