// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Text;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;
using Intel.Manageability.Utils; 

namespace ManageabilityStack
{

    public class AmtAssetManager
    {
        private HardwareAssetService service;
        private AmtSystem system;
        private bool cached = false;
        public bool IsCached() { return cached; }

        public AmtAssetManager(AmtSystem system, HardwareAssetService service)
        {
            this.system = system;
            this.service = service;
        }

        public void Dispose()
        {
            ClearCache();
            service = null;
        }

        /// <summary>
        /// Clear all of the data cached in this object. This will force the web service to be called next time asset data is needed.
        /// </summary>
        public void ClearCache()
        {
            cached = false;
            ComputerSystemCache = null;
            BaseBoardCache = null;
            BiosCache = null;
            BiosCacheSet = false;
            MemoryModulesCache = null;
            HardwareModulesCache = null;
            MediaDevicesCache = null;
            ProcessorsCache = null;
            BatteriesCache = null;
        }

        public void FetchCache()
        {
            try
            {
                GetComputerSystem();
                GetBaseBoard();
                GetBios();
                GetMemoryModules();
                GetHardwareModules();
                GetMediaDevices();
                GetProcessors();
                if (system.IsMobile) GetBatteries();
                cached = true;
            }
            catch (Exception e)
            {
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Failure to retrieve the Hardware data.",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
            }
        }

        private AssetDataType[] GetAssetOfType(uint AssetType)
        {
            if (AssetType == 9 && system.IsMobile == false) return new AssetDataType[0];
            uint count;
            AssetDataType[] assetData;
            try {service.GetAssetData(AssetType, out count, out assetData);} catch (Exception) {return null;}
            return assetData;
        }

        private AmtHardwareComputerSystem ComputerSystemCache = null;
        public AmtHardwareComputerSystem GetComputerSystem()
        {
            if (ComputerSystemCache != null) return ComputerSystemCache;
            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    CIM_Chassis r1 = new CIM_Chassis(system.WsMan);
                    r1.Get();
                    CIM_ComputerSystemPackage r2 = new CIM_ComputerSystemPackage(system.WsMan);
                    r2.Get();
                    ComputerSystemCache = new AmtHardwareComputerSystem(r1, r2);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.ReadAuditLog WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.ReadAuditLog CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtGeneralInformation.ReadAuditLog",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return null;
                }
            }
            else
            {
                // USE SOAP
                AssetDataType[] r = GetAssetOfType(AmtHardwareComputerSystem.AssetType);
                if (r == null || r.Length == 0) { return null; }
                ComputerSystemCache = new AmtHardwareComputerSystem(r[0].AssetData, (int)(r[0].AssetSize));
            }
            return ComputerSystemCache;
        }

        private AmtHardwareBaseBoard BaseBoardCache = null;
        public AmtHardwareBaseBoard GetBaseBoard()
        {
            if (BaseBoardCache != null) return BaseBoardCache;
            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    CIM_Card r = new CIM_Card(system.WsMan);
                    r.Get();
                    BaseBoardCache = new AmtHardwareBaseBoard(r);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetBaseBoard WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetBaseBoard CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetBaseBoard",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return null;
                }
            }
            else
            {
                // Use SOAP
                AssetDataType[] r = GetAssetOfType(AmtHardwareBaseBoard.AssetType);
                if (r == null || r.Length == 0) { return null; }
                BaseBoardCache = new AmtHardwareBaseBoard(r[0].AssetData, (int)(r[0].AssetSize));
            }
            return BaseBoardCache;
        }

        private AmtHardwareBios BiosCache = null;
        private bool BiosCacheSet = false;
        public AmtHardwareBios GetBios()
        {
            if (BiosCacheSet == true) return BiosCache;

            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    CIM_BIOSElement r = new CIM_BIOSElement(system.WsMan);
                    r.Get();

                    // Getting the CIM_BIOSFeature instance using the CIM_BIOSElement as EPR
                    CIM_BIOSFeature biosFeature = (CIM_BIOSFeature)AssociationTraversalTypedUtils.GetAssociated(system.WsMan,
                        r.Reference, typeof(CIM_BIOSFeature), typeof(CIM_BIOSFeatureBIOSElements));

                    BiosCache = new AmtHardwareBios(r, biosFeature);
                    BiosCacheSet = true;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetBios WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetBios CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetBios",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    BiosCache = null;
                    BiosCacheSet = false;
                    return null;
                }
            }
            else
            {
                // USE SOAP
                AssetDataType[] r = GetAssetOfType(AmtHardwareBios.AssetType);
                if (r == null || r.Length == 0) {return null;}
                BiosCache = new AmtHardwareBios(r[0].AssetData, (int)(r[0].AssetSize));
                BiosCacheSet = true;
            }
            return BiosCache;
        }

        private AmtHardwareMemoryModule[] MemoryModulesCache = null;
        public AmtHardwareMemoryModule[] GetMemoryModules()
        {
            if (MemoryModulesCache != null) return MemoryModulesCache;

            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    Collection<CIM_PhysicalMemory> r = CIM_PhysicalMemory.Enumerate(system.WsMan);
                    MemoryModulesCache = new AmtHardwareMemoryModule[r.Count];
                    for (int i = 0; i < r.Count; i++)
                    {
                        MemoryModulesCache[i] = new AmtHardwareMemoryModule(r[i]);
                    }
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetMemoryModules WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetMemoryModules CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetMemoryModules",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    MemoryModulesCache = null;
                    return null;
                }
            }
            else
            {
                // USE SOAP
                AssetDataType[] r = GetAssetOfType(AmtHardwareMemoryModule.AssetType);
                if (r == null || r.Length == 0) { MemoryModulesCache = new AmtHardwareMemoryModule[0]; return MemoryModulesCache; }
                MemoryModulesCache = new AmtHardwareMemoryModule[r.Length];
                for (int i = 0; i < r.Length; i++)
                {
                    MemoryModulesCache[i] = new AmtHardwareMemoryModule(r[i].AssetData, (int)(r[i].AssetSize));
                }
            }
            return MemoryModulesCache;
        }

        private AmtHardwareModule[] HardwareModulesCache = null;
        public AmtHardwareModule[] GetHardwareModules()
        {
            if (HardwareModulesCache != null) return HardwareModulesCache;

            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AmtHardwareModule[] HardwareModulesList;
                    Collection<AMT_PCIDevice> r = AMT_PCIDevice.Enumerate(system.WsMan);
                    HardwareModulesList = new AmtHardwareModule[r.Count];
                    for (int i = 0; i < r.Count; i++)
                    {
                        HardwareModulesList[i] = new AmtHardwareModule(r[i]);
                    }
                    HardwareModulesCache = HardwareModulesList;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetHardwareModules WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetHardwareModules CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetHardwareModules",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    HardwareModulesCache = null;
                    return null;
                }
            }
            else
            {
                // USE SOAP
                AssetDataType[] r = GetAssetOfType(AmtHardwareModule.AssetType);
                if (r == null || r.Length == 0) { HardwareModulesCache = new AmtHardwareModule[0]; return HardwareModulesCache; }
                ArrayList HardwareModulesList = new ArrayList();
                for (int i = 0; i < r.Length; i++)
                {
                    try
                    {
                        HardwareModulesList.Add(new AmtHardwareModule(r[i].AssetData, (int)(r[i].AssetSize)));
                    }
                    catch (Exception ex)
                    {
                        AmtDebugForm.LogException(this, ex);
                    }
                }
                HardwareModulesCache = (AmtHardwareModule[])HardwareModulesList.ToArray(typeof(AmtHardwareModule));
            }
            return HardwareModulesCache;
        }

        private AmtHardwareMediaDevice[] MediaDevicesCache = null;
        public AmtHardwareMediaDevice[] GetMediaDevices()
        {
            if (MediaDevicesCache != null) return MediaDevicesCache;

            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AmtHardwareMediaDevice[] MediaDeviceslist;
                    Collection<CIM_MediaAccessDevice> r1 = CIM_MediaAccessDevice.Enumerate(system.WsMan);
                    MediaDeviceslist = new AmtHardwareMediaDevice[r1.Count];
                    for (int i = 0; i < r1.Count; i++)
                    {
                        Collection<CimBase> enumResultList = AssociationTraversalTypedUtils.EnumerateAssociated(system.WsMan,
                             r1[i].Reference, typeof(CIM_PhysicalPackage), typeof(CIM_Realizes), "Dependent", null);

                        CIM_PhysicalPackage package = (CIM_PhysicalPackage)enumResultList[0];

                        MediaDeviceslist[i] = new AmtHardwareMediaDevice(r1[i], package);
                    }
                    MediaDevicesCache = MediaDeviceslist;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetMediaDevices WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetMediaDevices CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetMediaDevices",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    MediaDevicesCache = null;
                    return null;
                }
            }
            else
            {
                // USE SOAP
                AssetDataType[] r = GetAssetOfType(AmtHardwareMediaDevice.AssetType);
                if (r == null || r.Length == 0) { MediaDevicesCache = new AmtHardwareMediaDevice[0]; return null; }
                MediaDevicesCache = new AmtHardwareMediaDevice[r.Length];
                ArrayList MediaDeviceslist = new ArrayList();
                for (int i = 0; i < r.Length; i++)
                {
                    try
                    {
                        MediaDeviceslist.Add(new AmtHardwareMediaDevice(r[i].AssetData, (int)(r[i].AssetSize), system.AmtRev));
                    }
                    catch (Exception ex)
                    {
                        AmtDebugForm.LogException(this, ex);
                    }
                }
                MediaDevicesCache = (AmtHardwareMediaDevice[])MediaDeviceslist.ToArray(typeof(AmtHardwareMediaDevice));
            }
            return MediaDevicesCache;
        }

        private AmtHardwareProcessor[] ProcessorsCache = null;
        public AmtHardwareProcessor[] GetProcessors()
        {
            if (ProcessorsCache != null) return ProcessorsCache;
            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    Collection<CIM_Processor> r = CIM_Processor.Enumerate(system.WsMan);
                    ProcessorsCache = new AmtHardwareProcessor[r.Count];
                    for (int i = 0; i < r.Count; i++)
                    {
                       // Traversing the CIM_Chip insatnce using the processor as reference
                        CIM_Chip chip = (CIM_Chip)AssociationTraversalTypedUtils.GetAssociated(system.WsMan, 
                            r[i].Reference, typeof(CIM_Chip), typeof(CIM_Realizes), "Dependent", null);

                        // TODO: Use CIM_Realize to match a CIM_Chip to this CIM_Processor
                        ProcessorsCache[i] = new AmtHardwareProcessor(r[i], chip);
                                                                         
                    }
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetProcessors WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetProcessors CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtAssetManager.GetProcessors",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    ProcessorsCache = null;
                    return null;
                }
            }
            else
            {
                // USE SOAP
                AssetDataType[] r = GetAssetOfType(AmtHardwareProcessor.AssetType);
                if (r == null || r.Length == 0) { ProcessorsCache = new AmtHardwareProcessor[0]; return null; }
                ProcessorsCache = new AmtHardwareProcessor[r.Length];
                for (int i = 0; i < r.Length; i++)
                {
                    ProcessorsCache[i] = new AmtHardwareProcessor(r[i].AssetData, (int)(r[i].AssetSize));
                }
            }
            return ProcessorsCache;
        }

        private AmtHardwareBattery[] BatteriesCache = null;
        public AmtHardwareBattery[] GetBatteries()
        {
            if (system.AmtRev == 1 || system.WsManOnly == true) return null;
            if (BatteriesCache != null) return BatteriesCache;
            AssetDataType[] r = GetAssetOfType(AmtHardwareBattery.AssetType);
            if (r == null || r.Length == 0) { BatteriesCache = new AmtHardwareBattery[0]; return null; }
            BatteriesCache = new AmtHardwareBattery[r.Length];
            for (int i = 0; i < r.Length; i++)
            {
                BatteriesCache[i] = new AmtHardwareBattery(r[i].AssetData, (int)(r[i].AssetSize));
            }
            return BatteriesCache;
        }

    }            // AMT Asset Manager

    public class AmtHardwareBios               // Type 2
    {
        public static string GetCharacteristicsString(ushort[] Characteristics)
        {
            StringBuilder sbCharacteristics = new StringBuilder();

            foreach (ushort characteristic in Characteristics)
            {
                switch (characteristic)
                {
                    case 1:
                        sbCharacteristics.Append("   Other");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 2:
                        sbCharacteristics.Append("   Unknown");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 3:
                        sbCharacteristics.Append("   Undefined");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 4:
                        sbCharacteristics.Append("   ISA Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 5:
                        sbCharacteristics.Append("   MCA Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 6:
                        sbCharacteristics.Append("   EISA Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 7:
                        sbCharacteristics.Append("   PCI Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 8:
                        sbCharacteristics.Append("   PCMCIA Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 9:
                        sbCharacteristics.Append("   PnP Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 10:
                        sbCharacteristics.Append("   APM Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 11:
                        sbCharacteristics.Append("   Upgradeable BIOS");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 12:
                        sbCharacteristics.Append("   BIOS Shadowing Allowed");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 13:
                        sbCharacteristics.Append("   VL VESA Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 14:
                        sbCharacteristics.Append("   ESCD Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 15:
                        sbCharacteristics.Append("   LS-120 Boot Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 16:
                        sbCharacteristics.Append("   ACPI Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 17:
                        sbCharacteristics.Append("   I2O Boot Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 18:
                        sbCharacteristics.Append("   USB Legacy Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 19:
                        sbCharacteristics.Append("   AGP Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 20:
                        sbCharacteristics.Append("   PC Card");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 21:
                        sbCharacteristics.Append("   IR");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 22:
                        sbCharacteristics.Append("   1394");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 23:
                        sbCharacteristics.Append("   I2C");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 24:
                        sbCharacteristics.Append("   Smart Battery");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 25:
                        sbCharacteristics.Append("   ATAPI ZIP Drive Boot Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 26:
                        sbCharacteristics.Append("   1394 Boot Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 27:
                        sbCharacteristics.Append("   Boot from CD");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 28:
                        sbCharacteristics.Append("   Selectable Boot");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 29:
                        sbCharacteristics.Append("   BIOS ROM is Socketed");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 30:
                        sbCharacteristics.Append("   Boot from PCMCIA");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 31:
                        sbCharacteristics.Append("   EDD Specification Support");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 32:
                        sbCharacteristics.Append("   Enable Targeted Content Distribution");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    case 160:
                        sbCharacteristics.Append("   PC-98");
                        sbCharacteristics.Append(Environment.NewLine); 
                        break;
                    default:                        
                        break;
                }
            }

            return sbCharacteristics.ToString(); 
        }


        public static uint AssetType = 2;

        private uint StructureVersion;
	    public string Vendor;
	    public string Version;
        public string ReleaseDate;
        public ushort[] Characteristics; 
        //public uint Characteristics;

        public AmtHardwareBios(byte[] data, int size)
        {
            StructureVersion        = BitConverter.ToUInt32(data, 0);
            Vendor                  = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4, 65));
            Version                 = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4 + 65, 65));
            ReleaseDate             = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4 + 65 + 65, 65));
            //Characteristics         = BitConverter.ToUInt32(data, 4 + 65 + 65 + 65 + 1);
            Characteristics         = new ushort[]{}; 
        }

        public AmtHardwareBios(CIM_BIOSElement res,  CIM_BIOSFeature biosFeature)
        {
            Vendor = Utils.FilterStr((res.ManufacturerExist ? res.Manufacturer : String.Empty));
            Version = Utils.FilterStr(res.Version);
            if (res.ReleaseDateExist)
            {
                CimDateTime ReleaseDateTime = res.ReleaseDate;
                ReleaseDate = Utils.FilterStr(ReleaseDateTime.StringValue());
            }
            //Characteristics = 0;
            Characteristics = biosFeature.Characteristics; 
        }

        public override string ToString()
        {
            return ManageabilityStack.Properties.Resources.HW_BIOS_ToString + Vendor;
            // return "BIOS - " + Vendor;
        }
    }

    public class AmtHardwareComputerSystem     // Type 3
    {
        public static uint AssetType = 3;
   
        private uint StructureVersion;
        public string Manufacturer;
        public string Product;
        public string Version;
        public string SerialNumber;
        public Guid UUID;

        public AmtHardwareComputerSystem(CIM_Chassis chassis, CIM_ComputerSystemPackage package)
        {
            Manufacturer = Utils.FilterStr((chassis.ManufacturerExist ? chassis.Manufacturer : String.Empty));
            Product = Utils.FilterStr((chassis.ModelExist ? chassis.Model : String.Empty));
            Version = Utils.FilterStr((chassis.VersionExist ? chassis.Version : String.Empty));
            SerialNumber = Utils.FilterStr((chassis.SerialNumberExist ? chassis.SerialNumber : String.Empty));
            if (package.PlatformGUIDExist)
                UUID = new Guid(package.PlatformGUID);
            else
                UUID = Guid.Empty;
        }

        public AmtHardwareComputerSystem(byte[] data, int size)
        {
            StructureVersion        = BitConverter.ToUInt32(data, 0);
            Manufacturer            = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4, 65));
            Product                 = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4 + 65, 65));
            Version                 = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4 + 65 + 65, 65));
            SerialNumber            = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4 + 65 + 65 + 65, 65));
            byte[] Uuidbytes = new byte[16];
            for (int i=0;i<16;i++)
            {
                Uuidbytes[i] = data[4 + 65 + 65 + 65 + 65 + i];
                UUID = new Guid(Uuidbytes);
            }
        }

        public override string ToString()
        {
            if (Manufacturer.Length == 0)
            {
                return ManageabilityStack.Properties.Resources.HW_CS_ToString_0;
                //return "Computer";
            }
            else
            {
                return ManageabilityStack.Properties.Resources.HW_CS_ToString_1 + Manufacturer;
                // return "Computer - " + Manufacturer;
            }
        }
    }

    public class AmtHardwareBaseBoard          // Type 4
    {
        public static uint AssetType = 4;

        private uint StructureVersion;
        public string Manufacturer;
        public string Product;
        public string Version;
        public string SerialNumber;
        public string AssetTag;
        public bool	Replaceable;
        public bool ReplaceableSpecified;

        public AmtHardwareBaseBoard(CIM_Card card)
        {
            Manufacturer = Utils.FilterStr((card.ManufacturerExist ? card.Manufacturer : String.Empty));
            Product = Utils.FilterStr((card.ModelExist ? card.Model : String.Empty));
            Version = Utils.FilterStr((card.VersionExist ? card.Version : String.Empty));
            SerialNumber = Utils.FilterStr((card.SerialNumberExist ? card.SerialNumber : String.Empty));
            AssetTag = Utils.FilterStr(card.Tag);
            Replaceable = (card.ReplaceableExist ? card.Replaceable : card.ReplaceableExist);
            ReplaceableSpecified = card.ReplaceableExist;
        }

        public AmtHardwareBaseBoard(byte[] data, int size)
        {
            StructureVersion        = BitConverter.ToUInt32(data, 0);
            Manufacturer            = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4, 65));
            Product                 = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4 + 65, 65));
            Version                 = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4 + 65 + 65, 65));
            SerialNumber            = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4 + 65 + 65 + 65, 65));
            AssetTag                = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4 + 65 + 65 + 65 + 65, 65));
            Replaceable             = (data[4 + 65 + 65 + 65 + 65 + 65] != 0);
            ReplaceableSpecified    = true;
        }

        public override string ToString()
        {
            return ManageabilityStack.Properties.Resources.HW_BB_ToString + Manufacturer;
        }
    }

    public class AmtHardwareProcessor          // Type 5
    {
        
        public enum AtmProcessorStatus
        {
            UNKNOWN = 0x0,
            ENABLED = 0x1,
            DISABLED_BY_USER = 0x2,
            DISABLED_BY_BIOS = 0x3,
            WAITING_TO_BE_ENABLED = 0x4,
            OTHER = 0x7
        }

        public enum AtmProcessorType
        {
            OTHER = 1,
            UNKNOWN = 2,
            CENTRAL = 3,
            MATH = 4,
            DSP = 5,
            VIDEO = 6
        }

        public enum AtmProcessorFamily
        {
            OTHER = 0x01,
            UNKNOWN = 0x02,
            I8086 = 0x03,
            I80286 = 0x04,
            INTEL_386 = 0x05,
            INTEL_486 = 0x06,
            I8087 = 0x07,
            I80287 = 0x08,
            I80387 = 0x09,
            I80487 = 0x0A,
            PENTIUM = 0x0B,
            PENTIUM_PRO = 0x0C,
            PENTIUM_II = 0x0D,
            PENTIUM_MMX_TECHNOLOGY = 0x0E,
            CELERON = 0x0F,
            PENTIUM_II_XEON = 0x10,
            PENTIUM_III = 0x11,
            ITANIUM = 0x82,
            PENTIUM_III_XEON = 0xB0,
            PENTIUM_III_SPEEDSTEP_TECHNOLOGY = 0xB1,
            PENTIUM_4 = 0xB2,
            XEON = 0xB3,
            XEON_MP = 0xB5,
            ITANIUM_2 = 0xB8,  
            PENTIUM_M = 0xB9,
            CELERON_D = 0xBA,
            PENTIUM_D = 0xBB,
            PENTIUM_EE = 0xBC,
            CORE_SOLO = 0xBD,
        };

        public enum AtmProcessorUpgrade
        {
            OTHER = 0x01,
            UNKNOWN = 0x02,
            DAUGHTER_BOARD = 0x03,
            ZIF_SOCKET = 0x04,
            REPLACEABLE_PIGGYBACK = 0x05,
            NONE = 0x06,
            LIF_SOCKET = 0x07,
            SLOT_1 = 0x08,
            SLOT_2 = 0x09,
            SOCKET_370 = 0x0A,
            SLOT_M = 0x0C,
            SOCKET_423 = 0x0D,
            SOCKET_478 = 0x0F
        }

        public static uint AssetType = 5;

        private uint StructureVersion;
        public ulong ID;
        public ushort MaxSocketSpeed;
        public ushort CurrentSpeed;
        public AtmProcessorStatus Status;
        public AtmProcessorType Type;
        public AtmProcessorFamily Family;
        public AtmProcessorUpgrade UpgradeInformation;
        public bool SocketPopulated;
        public string SocketDesignation;
        public string Manufacturer;
        public string Version;

        // ADDED prnz -
        // Added Role (a free-form string)
        // Role replaces Type in AMT4.0/CP
        public string Role;
        
        public AmtHardwareProcessor(CIM_Processor proc, CIM_Chip chip)
        {
            // Fill in fields from CIM_Processor
            ID = proc.UniqueIDExist ? Convert.ToUInt64(proc.UniqueID) : (ulong)0;
            MaxSocketSpeed = proc.MaxClockSpeedExist ? (ushort)proc.MaxClockSpeed : (ushort)0;
            CurrentSpeed = proc.CurrentClockSpeedExist ? (ushort)proc.CurrentClockSpeed : (ushort)0;
            Status = proc.CPUStatusExist ? (AtmProcessorStatus)proc.CPUStatus : AtmProcessorStatus.UNKNOWN;
            Type = AtmProcessorType.UNKNOWN;
            Role = proc.RoleExist ? proc.Role : null;
            Family = proc.FamilyExist ? (AtmProcessorFamily)proc.Family : AtmProcessorFamily.UNKNOWN;
            UpgradeInformation = proc.UpgradeMethodExist ? (AtmProcessorUpgrade)proc.UpgradeMethod : AtmProcessorUpgrade.UNKNOWN;
            SocketPopulated = true;
            SocketDesignation = null;
            
            // Manufacturer and Version are from CIM_Chip
            if(chip != null)
            {
                Manufacturer = chip.ManufacturerExist ? chip.Manufacturer : null;
                Version = chip.VersionExist ? chip.Version : null;
            }
        }

        public AmtHardwareProcessor(byte[] data, int size)
        {
            StructureVersion = BitConverter.ToUInt32(data, 0);
            ID = BitConverter.ToUInt64(data, 4);
            MaxSocketSpeed = BitConverter.ToUInt16(data, 12);
            CurrentSpeed = BitConverter.ToUInt16(data, 14);
            Status = (AtmProcessorStatus)data[16];
            Type = (AtmProcessorType)data[17];
            Family = (AtmProcessorFamily)data[18];
            UpgradeInformation = (AtmProcessorUpgrade)data[19];
            SocketPopulated = (data[20] != 0);
            SocketDesignation = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 21, 65));
            Manufacturer = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 86, 65));
            Version = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 151, 65));
        }

        public override string ToString()
        {
            return ManageabilityStack.Properties.Resources.HW_Processor_ToString + Manufacturer;
            //return "Processor - " + Manufacturer;
        }
    }

    public class AmtHardwareMemoryModule       // Type 6
    {
       
        public enum AmtMemoryFormFactor
        {
            OTHER = 0x01,
            UNKNOWN = 0x02,
            SIMM = 0x03,
            SIP = 0x04,
            CHIP = 0x05,
            DIP = 0x06,
            ZIP = 0x07,
            PROPRIETARY = 0x08,
            DIMM = 0x09,
            TSOP = 0x0A,
            ROW_OF_CHIPS = 0x0B,
            RIMM = 0x0C,
            SODIMM = 0x0D,
            SRIMM = 0x0E
        };

        public enum AmtMemoryType
        {
            UNKNOWN = 0,
            OTHER = 1,
            DRAM = 2,
            SynchronousDRAM = 3,
            CacheDRAM = 4, 
            EDO = 5,
            EDRAM = 6,
            VRAM = 7,
            SRAM = 8,
            RAM = 9,
            ROM = 10, 
            FLASH = 11,
            EEPROM = 12,
            FEPROM = 13,
            EPROM = 14,
            CDRAM = 15,
            ThreeDRAM = 16,
            SDRAM = 17,
            SGRAM = 18,
            RDRAM = 19,
            DDR = 20,
            DDR2 = 21,
            BRAM = 22,
            FBDIMM = 23,
            DDR3 = 24,
            FBD2 = 25
        };

        public enum AmtMemoryTypeSoap
        {
            OTHER = 0x01,
            UNKNOWN = 0x02,
            DRAM = 0x03,
            EDRAM = 0x04,
            VRAM = 0x05,
            SRAM = 0x06,
            RAM = 0x07,
            ROM = 0x08,
            FLASH = 0x09,
            EEPROM = 0x0A,
            FEPROM = 0x0B,
            EPROM = 0x0C,
            CDRAM = 0x0D,
            ThreeDRAM = 0x0E,
            SDRAM = 0x0F,
            SGRAM = 0x10,
            RDRAM = 0x11,
            DDR = 0x12,
            DDR2 = 0x13
        };

        public enum AmtMemoryStatus
        {
            UNKNOWN = 0,
            OTHER = 1,
            OK = 2,
            DEGRADED = 3,
            STRESSED = 4,
            PREDICTIVE_FAILURE = 5,
            ERROR = 6,
            NONRECOVERABLE_ERROR = 7,
            STARTING = 8,
            STOPPING = 9,
            STOPPED = 10,
            IN_SERVICE = 11,
            NO_CONTACT = 12,
            LOST_COMMUNICATION = 13,
            ABORTED = 14,
            DORMANT = 15,
            SUPPORTING_ENTITY_IN_ERROR = 16,
            COMPLETED = 17,
            POWER_MODE = 18
        };
        
        private enum MemoryTypeDetail
        {
            Other = 0x2,
            Unknown = 0x4,
            FastPaged = 0x8,
            StaticColumn = 0x10,
            PseudoStatic = 0x20,
            RAMBUS = 0x40,
            Synchronous = 0x80,
            CMOS = 0x100,
            EDO = 0x200,
            WindowDRAM = 0x400,
            CacheDRAM = 0x800,
            NonVolatile = 0x1000
        };
        

        
        public static uint AssetType = 6;

        private uint   StructureVersion;
        private ushort ReportedSize;
        public ulong   Size;
        public AmtMemoryFormFactor FormFactor;
        public AmtMemoryType Type;
        public ushort TypeDetail;
        public string TypeDetailStr;
        public uint Speed;
        public string Manufacturer;
        public string SerialNumber;
        public string AssetTag;
        public string PartNumber;
        public AmtMemoryStatus[] Status;

        public AmtHardwareMemoryModule(CIM_PhysicalMemory memory)
        {
            Size = memory.Capacity / 1024;
            FormFactor = (AmtMemoryFormFactor)memory.FormFactor;
            Type = (AmtMemoryType)memory.MemoryType;
            TypeDetail = 0;
            Speed = memory.Speed;     
            Manufacturer = Utils.FilterStr((memory.ManufacturerExist ? memory.Manufacturer : String.Empty));
            SerialNumber = Utils.FilterStr((memory.SerialNumberExist ? memory.SerialNumber : String.Empty));
            AssetTag = Utils.FilterStr(memory.Tag);
            PartNumber = Utils.FilterStr((memory.PartNumberExist ? memory.PartNumber : String.Empty));

            TypeDetailStr = "";
            for (int i = 1; ; i = i * 2)
            {
                if ((TypeDetail & i) != 0)
                {
                    MemoryTypeDetail d = (MemoryTypeDetail)i;
                    if (TypeDetailStr.Length != 0) TypeDetailStr += ", ";
                    TypeDetailStr += d.ToString();
                }
                if (i > 30000) break;
            }
            
            ArrayList statusList = new ArrayList();
            if(memory.OperationalStatusExist)
            {
                foreach(ushort statusItem in memory.OperationalStatus)
                {
                    statusList.Add((AmtMemoryStatus)statusItem);
                }
                Status = (AmtMemoryStatus[])statusList.ToArray(typeof(AmtMemoryStatus));
            }
        }

        public AmtHardwareMemoryModule(byte[] data, int size)
        {
            StructureVersion    = BitConverter.ToUInt32(data, 0);
            ReportedSize        = BitConverter.ToUInt16(data, 4);
            FormFactor          = (AmtMemoryFormFactor)(int)data[6];
            AmtMemoryTypeSoap SType = (AmtMemoryTypeSoap)(int)data[7];
            TypeDetail          = BitConverter.ToUInt16(data, 8);
            Speed               = BitConverter.ToUInt16(data, 10);
            Manufacturer        = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 12, 65));
            SerialNumber        = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 12 + 65, 65));
            AssetTag            = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 12 + 65 + 65, 65));
            PartNumber          = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 12 + 65 + 65 + 65, 65));

            Type = AmtMemoryType.UNKNOWN;
            if ((int)SType >= 0x04 && (int)SType <= 0x13) Type = (AmtMemoryType)((int)SType + 2);
            else if (SType == AmtMemoryTypeSoap.OTHER) Type = AmtMemoryType.OTHER;
            else if (SType == AmtMemoryTypeSoap.DRAM) Type = AmtMemoryType.DRAM;

            if ((ReportedSize & 0x8000) != 0)
            {
                // If the MSB is 1, this is KB's.
                Size = (ulong)(ReportedSize & 0x7FFF);
            }
            else
            {
                // If the MSB is 0, this is MB's.
                Size = ((ulong)(ReportedSize & 0x7FFF)) * 1024;
            }

            TypeDetailStr = "";
            for (int i = 1;; i = i * 2)
            {
                if ((TypeDetail & i) != 0)
                {
                    MemoryTypeDetail d = (MemoryTypeDetail)i;
                    if (TypeDetailStr.Length != 0) TypeDetailStr += ", ";
                    TypeDetailStr += d.ToString();
                }
                if (i > 30000) break;
            }
        }

        public string MaxMediaSizeLStr
        {
            get
            {
                uint m = (uint)Size;
                if (m == 0) return ManageabilityStack.Properties.Resources.HW_Memory_LString_NONE;
                if (m < 2048) return "" + m + ManageabilityStack.Properties.Resources.HW_Memory_LString_KB;
                m = m / 1024;
                if (m < 2048) return "" + m + ManageabilityStack.Properties.Resources.HW_Memory_LString_MB;
                m = m / 1024;
                if (m < 2048) return "" + m + ManageabilityStack.Properties.Resources.HW_Memory_LString_GB;
                m = m / 1024;
                return "" + m + ManageabilityStack.Properties.Resources.HW_Memory_LString_TB;
            }
        }

        public string MaxMediaSizeSStr
        {
            get
            {
                uint m = (uint)Size;
                if (m == 0) return ManageabilityStack.Properties.Resources.HW_Memory_SString_NONE;
                if (m < 2048) return "" + m + ManageabilityStack.Properties.Resources.HW_Memory_SString_KB;
                m = m / 1024;
                if (m < 2048) return "" + m + ManageabilityStack.Properties.Resources.HW_Memory_SString_MB;
                m = m / 1024;
                if (m < 2048) return "" + m + ManageabilityStack.Properties.Resources.HW_Memory_SString_GB;
                m = m / 1024;
                return "" + m + ManageabilityStack.Properties.Resources.HW_Memory_SString_TB;
            }
        }

        public override string ToString()
        {
            if (Size == 0)
            {
                return ManageabilityStack.Properties.Resources.HW_Memory_ToString_0;
            }
            else
            {
                return ManageabilityStack.Properties.Resources.HW_Memory_ToString_1 + MaxMediaSizeSStr + " " + Type.ToString();
            }
        }
    }

    public class AmtHardwareModule             // Type 7
    {
        private string[] PciDeviceTypeStrings =
        {
            ManageabilityStack.Properties.Resources.HW_PCI_Type_0,
            ManageabilityStack.Properties.Resources.HW_PCI_Type_1,
            ManageabilityStack.Properties.Resources.HW_PCI_Type_2,
            ManageabilityStack.Properties.Resources.HW_PCI_Type_3,
            ManageabilityStack.Properties.Resources.HW_PCI_Type_4,
            ManageabilityStack.Properties.Resources.HW_PCI_Type_5,
            ManageabilityStack.Properties.Resources.HW_PCI_Type_6,
            ManageabilityStack.Properties.Resources.HW_PCI_Type_7,
            ManageabilityStack.Properties.Resources.HW_PCI_Type_8,
            ManageabilityStack.Properties.Resources.HW_PCI_Type_9,
            ManageabilityStack.Properties.Resources.HW_PCI_Type_10,
            ManageabilityStack.Properties.Resources.HW_PCI_Type_11,
            ManageabilityStack.Properties.Resources.HW_PCI_Type_12
            /*
            "Pre-2.0 PCI Specification Device",
            "Mass Storage Controller",
            "Network Controller",
            "Display Controller",
            "Multimedia Device",
            "Memory Controller",
            "Bridge Device",
            "Simple Communications Controller",
            "Base Systems Peripheral",
            "Input Device",
            "Docking Station",
            "Processor",
            "Serial Bus Controller"*/
        };

        public static uint AssetType = 7;

        private uint StructureVersion;
        public ushort VendorID;
        public string VendorIdStr;
        public ushort DeviceID;
        public string DeviceIdStr;
        public bool DeviceIdStrFound;
        public byte RevisionID;
        public byte ProgIf = 0;
        public byte Subclass = 0;
        public byte Baseclass = 0;
        public ushort SubvendorID;
        public ushort SubsystemID;
        public ushort DeviceLocation = 0;

        public AmtHardwareModule(AMT_PCIDevice device)
        {
            VendorID = device.VendorID;
            //DeviceID = device.DeviceID; // ****************************************************************************
            RevisionID = device.RevisionID;
            ProgIf = device.ProgIf;
            Subclass = device.Subclass;
            Baseclass = device.ClassCode;
            SubvendorID = device.SubsystemID;
            SubsystemID = device.SubsystemVendorID;
            DeviceLocation = device.DeviceLocation;
            DeviceIdStrFound = Utils.GetPciDevice(VendorID, DeviceID, out VendorIdStr, out DeviceIdStr);
        }

        public AmtHardwareModule(byte[] data, int size)
        {
            StructureVersion = BitConverter.ToUInt32(data, 0);
            VendorID = BitConverter.ToUInt16(data, 4);
            DeviceID = BitConverter.ToUInt16(data, 6);
            RevisionID = data[8];
            ProgIf = data[9];
            Subclass = data[10];
            Baseclass = data[11];
            SubvendorID = BitConverter.ToUInt16(data, 12);
            SubsystemID = BitConverter.ToUInt16(data, 14);
            DeviceLocation = BitConverter.ToUInt16(data, 16);
            DeviceIdStrFound = Utils.GetPciDevice(VendorID, DeviceID, out VendorIdStr, out DeviceIdStr);
        }

        public override string ToString()
        {
            if (DeviceIdStrFound)
            {
                return VendorIdStr + " - " + DeviceIdStr;
            }
            else
            {
                return ManageabilityStack.Properties.Resources.HW_Module_ToString + VendorIdStr;
            }
        }
    }

    public class AmtHardwareMediaDevice        // Type 8
    {
       
        private string[] MediaCapabilitiesByte1 =
        {
            /*
	        "SMART feature set",
	        "Security Mode feature set",
	        "Removable Media feature set",
	        "mandatory Power Management feature set",
	        "write cache",
	        "look-ahead",
	        "release interrupt",
	        "SERVICE interrupt",
	        "DEVICE RESET command",
	        "Host Protected Area feature set",
	        "WRITE BUFFER command",
	        "READ BUFFER command",
	        "NOP command",*/
            ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_0,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_1,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_2,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_3,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_4,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_5,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_6,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_7,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_8,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_9,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_10,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_11,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B1_12,
            "",
            "",
            "",
        };

        private string[] MediaCapabilitiesByte2 =
        {
            ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_0,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_1,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_2,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_3,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_4,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_5,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_6,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_7,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_8,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_9,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_10,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_11,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B2_12,
            /*
	        "DOWNLOAD MICROCODE command",
	        "READ/WRITE DMA QUEUED",
	        "CFA feature set",
	        "Advanced Power Management feature set",
	        "Removable Media Status Notification feature set",
	        "Power-Up In Standby feature set",
	        "SET FEATURES subcommand required to spin-up after power-up",
	        "SET MAX security extension",
	        "Automatic Acoustic Management feature set",
	        "48-bit Address feature set",
	        "Device Configuration Overlay feature set",
	        "mandatory FLUSH CACHE command",
	        "FLUSH CACHE EXT command",*/
            "",
            "",
            "",
        };

        private string[] MediaCapabilitiesByte3 =
        {
            ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B3_0,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B3_1,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B3_2,
	        ManageabilityStack.Properties.Resources.HW_MediaDev_Cap_B3_3,
            /*
	        "SMART error logging",
	        "SMART self-test",
	        "Media serial number",
	        "Media Card Pass Through Command feature set",*/
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
            "",
        };

        public static uint AssetType = 8;

        private uint StructureVersion;
        public string ModelNumber;
        public string SerialNumber;
        public ushort Capabilities1;
        public ushort Capabilities2;
        public ushort Capabilities3;
        public string CapabilitiesStr1;
        public string CapabilitiesStr2;
        public string CapabilitiesStr3;
        public ulong MaxMediaSize;
        public string MaxMediaSizeLStr
        {
            get
            {
                // The maximum media size value is only reduced by 1000 in 
                // accordance with how media sizes are reported.
                ulong m = MaxMediaSize;
                if (m == 0) return ManageabilityStack.Properties.Resources.HW_MaxMediaSizeLStr_0;
                if (m < 2000) return "" + m + ManageabilityStack.Properties.Resources.HW_MaxMediaSizeLStr_2;
                m = m / 1000;
                if (m < 2000) return "" + m + ManageabilityStack.Properties.Resources.HW_MaxMediaSizeLStr_3;
                m = m / 1000;
                if (m < 2000) return "" + m + ManageabilityStack.Properties.Resources.HW_MaxMediaSizeLStr_4;
                m = m / 1000;
                return "" + m + ManageabilityStack.Properties.Resources.HW_MaxMediaSizeLStr_5;
            }
        }
        
        public string MaxMediaSizeSStr
        {
            get
            {
                ulong m = MaxMediaSize;
                if (m == 0) return ManageabilityStack.Properties.Resources.HW_MaxMediaSizeSStr_0;
                if (m < 2048) return "" + m + ManageabilityStack.Properties.Resources.HW_MaxMediaSizeSStr_1;
                m = m / 1024;
                if (m < 2048) return "" + m + ManageabilityStack.Properties.Resources.HW_MaxMediaSizeSStr_2;
                m = m / 1024;
                if (m < 2048) return "" + m + ManageabilityStack.Properties.Resources.HW_MaxMediaSizeSStr_3;
                m = m / 1024;
                if (m < 2048) return "" + m + ManageabilityStack.Properties.Resources.HW_MaxMediaSizeSStr_4;
                m = m / 1024;
                return "" + m + ManageabilityStack.Properties.Resources.HW_MaxMediaSizeSStr_5;
            }
        }


        
        public AmtHardwareMediaDevice(CIM_MediaAccessDevice media, CIM_PhysicalPackage package)
        {
            ModelNumber = package.Model;
            SerialNumber = package.SerialNumber; 

            if (media.CapabilitiesExist)
            {
                if (media.Capabilities.Length > 0) Capabilities1 = media.Capabilities[0];
                if (media.Capabilities.Length > 1) Capabilities2 = media.Capabilities[1];
                if (media.Capabilities.Length > 2) Capabilities3 = media.Capabilities[2];
            }
            MaxMediaSize = media.MaxMediaSize;

            CapabilitiesStr1 = "";
            int j = 0;
            for (int i = 1; ; i = i * 2)
            {
                if ((Capabilities1 & i) != 0)
                {
                    if (CapabilitiesStr1.Length != 0 && MediaCapabilitiesByte1[j].Length != 0) CapabilitiesStr1 += ", ";
                    CapabilitiesStr1 += MediaCapabilitiesByte1[j];
                }
                if (i > 30000) break;
                j++;
            }

            CapabilitiesStr2 = "";
            j = 0;
            for (int i = 1; ; i = i * 2)
            {
                if ((Capabilities2 & i) != 0)
                {
                    if (CapabilitiesStr2.Length != 0 && MediaCapabilitiesByte2[j].Length != 0) CapabilitiesStr2 += ", ";
                    CapabilitiesStr2 += MediaCapabilitiesByte2[j];
                }
                if (i > 30000) break;
                j++;
            }

            CapabilitiesStr3 = "";
            j = 0;
            for (int i = 1; ; i = i * 2)
            {
                if ((Capabilities3 & i) != 0)
                {
                    if (CapabilitiesStr3.Length != 0 && MediaCapabilitiesByte3[j].Length != 0) CapabilitiesStr3 += ", ";
                    CapabilitiesStr3 += MediaCapabilitiesByte3[j];
                }
                if (i > 30000) break;
                j++;
            }
        }

        public AmtHardwareMediaDevice(byte[] data, int size, uint rev)
        {
            StructureVersion = BitConverter.ToUInt32(data, 0);
            ModelNumber = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 4, 40));
            SerialNumber = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 44, 20));
            Capabilities1 = BitConverter.ToUInt16(data, 64);
            Capabilities2 = BitConverter.ToUInt16(data, 66);
            Capabilities3 = BitConverter.ToUInt16(data, 68);
            MaxMediaSize = BitConverter.ToUInt64(data, 70);

            // This value needs to be reduced to report the media size in KB, 
            // which is consistent with WSMAN interface value.
            // Additionally, the value is only reduced by 1000 in accordance 
            // with how media sizes are reported (base 10, not base 2).
            MaxMediaSize = MaxMediaSize/1000;

            if (rev == 1)
            {
                MaxMediaSize = MaxMediaSize * 1024 * 1024;
            }

            CapabilitiesStr1 = "";
            int j = 0;
            for (int i = 1; ; i = i * 2)
            {
                if ((Capabilities1 & i) != 0)
                {
                    if (CapabilitiesStr1.Length != 0 && MediaCapabilitiesByte1[j].Length != 0) CapabilitiesStr1 += ", ";
                    CapabilitiesStr1 += MediaCapabilitiesByte1[j];
                }
                if (i > 30000) break;
                j++;
            }

            CapabilitiesStr2 = "";
            j = 0;
            for (int i = 1; ; i = i * 2)
            {
                if ((Capabilities2 & i) != 0)
                {
                    if (CapabilitiesStr2.Length != 0 && MediaCapabilitiesByte2[j].Length != 0) CapabilitiesStr2 += ", ";
                    CapabilitiesStr2 += MediaCapabilitiesByte2[j];
                }
                if (i > 30000) break;
                j++;
            }

            CapabilitiesStr3 = "";
            j = 0;
            for (int i = 1; ; i = i * 2)
            {
                if ((Capabilities3 & i) != 0)
                {
                    if (CapabilitiesStr3.Length != 0 && MediaCapabilitiesByte3[j].Length != 0) CapabilitiesStr3 += ", ";
                    CapabilitiesStr3 += MediaCapabilitiesByte3[j];
                }
                if (i > 30000) break;
                j++;
            }
        }

        public override string ToString()
        {
            return "Media - " + ModelNumber;            
        }
    }

    public class AmtHardwareBattery        // Type 9
    {
        
        public static uint AssetType = 9;

        public uint StructuredVersion;
        public uint OEMSpecific;
        public System.DateTime SBDSManufactureDate;
        public int DesignCapacity;
        public ushort DesignVoltage;
        public ushort SBDSSerialNumber;
        private byte DeviceChemistryX;
        public byte MaximumErrorInBatteryData;
        public string SerialNumber;
        public string SBDSVersionNumber;
        public string Location;
        public string Manufacturer;
        public string ManufactureDate;
        public string DeviceName;
        public string SBDSDeviceChemistry;

        /*
            typedef struct _PT_PORTABLE_BATTERY
            {
                uint32 StructuredVersion;
                uint32 OEMSpecific;
                uint32 SBDSManufactureDate;
                uint16 DesignCapacity;
                uint16 DesignVoltage;
                uint16 SBDSSerialNumber;
                uint8 DesignCapacityMultiplier;
                uint8 DeviceChemistry;
                uint8 MaximumErrorInBatteryData;
                uint8 SerialNumber[65];
                uint8 SBDSVersionNumber[65];
                uint8 Location[65];
                uint8 Manufacturer[65];
                uint8 ManufactureDate[65];
                uint8 DeviceName[65];
                uint8 SBDSDeviceChemistry[65];
            } PT_PORTABLE_BATTERY;

         */

        private string[] AmtBatteryDeviceChemistry = 
        {
            "Invalid Value 0",
            ManageabilityStack.Properties.Resources.HW_BatteryDevChem_0,
            ManageabilityStack.Properties.Resources.HW_BatteryDevChem_1,
            ManageabilityStack.Properties.Resources.HW_BatteryDevChem_2,
            ManageabilityStack.Properties.Resources.HW_BatteryDevChem_3,
            ManageabilityStack.Properties.Resources.HW_BatteryDevChem_4,
            ManageabilityStack.Properties.Resources.HW_BatteryDevChem_5,
            ManageabilityStack.Properties.Resources.HW_BatteryDevChem_6,
            ManageabilityStack.Properties.Resources.HW_BatteryDevChem_7
        };

        public AmtHardwareBattery(byte[] data, int size)
        {
            StructuredVersion = BitConverter.ToUInt32(data, 0);
            OEMSpecific = BitConverter.ToUInt32(data, 4);
            uint SBDSManufactureDateX = BitConverter.ToUInt32(data, 8);
            DesignCapacity = BitConverter.ToUInt16(data, 12);
            DesignVoltage = BitConverter.ToUInt16(data, 14);
            SBDSSerialNumber = BitConverter.ToUInt16(data, 16);
            byte DesignCapacityMultiplier = data[18];
            DeviceChemistryX = data[19];
            MaximumErrorInBatteryData = data[20];
            SerialNumber = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 21, 65));
            SBDSVersionNumber = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 21 + 65, 65));
            Location = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 21 + 65 + 65, 65));
            Manufacturer = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 21 + 65 + 65 + 65, 65));
            ManufactureDate = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 21 + 65 + 65 + 65 + 65, 65));
            DeviceName = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 21 + 65 + 65 + 65 + 65 + 65, 65));
            SBDSDeviceChemistry = Utils.FilterStr(UTF8Encoding.UTF8.GetString(data, 21 + 65 + 65 + 65 + 65 + 65 + 65, 65));

            DesignCapacity = DesignCapacity * DesignCapacityMultiplier;

            if (SBDSManufactureDateX != 0)
            {
                int year = (int)((SBDSManufactureDateX >> 8) + 1980);
                int month = (int)((SBDSManufactureDateX & 0x00F0) >> 4);
                int day = (int)(SBDSManufactureDateX & 0x000F);
                try
                {
                    SBDSManufactureDate = new System.DateTime(year, month, day);
                }
                catch (Exception)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, 
                        String.Format(ManageabilityStack.Properties.Resources.Log_Battery_Info ,year.ToString() , month.ToString() ,day.ToString()) );
                    SBDSManufactureDate = System.DateTime.MinValue;
                }
            }
            else
            {
                SBDSManufactureDate = System.DateTime.MinValue;
            }
        }

        public string DeviceChemistry
        {
            get
            {
                if (DeviceChemistryX >= AmtBatteryDeviceChemistry.Length) return AmtBatteryDeviceChemistry[1];
                return AmtBatteryDeviceChemistry[DeviceChemistryX];
            }
        }

        public override string ToString()
        {
            return ManageabilityStack.Properties.Resources.HW_Battery_ToString + Manufacturer;
            //return "Battery - " + Manufacturer;
        }
    }

}
