// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================

using System;
using System.Collections.Generic;
using System.Text;
using System.Text.RegularExpressions;
using System.Diagnostics;

using Intel.Manageability.Cim;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.Exceptions;
using Intel.Manageability.WSManagement;

namespace ManageabilityStack
{
    public class AmtKVM
    {
        public enum KVMStates
        {
            DISABLED_MEBX = 0,    //means you can not change to enable
            DISABLED,
            ENABLED_REDIR_PORT_ONLY,
            ENABLED_STANDARD_PORT_ONLY,
            ENABLED_BOTH_PORTS,
            ERROR_UNKNOWN_STATE
        }

        public enum EnabledStateValues
        {
            ENABLED = 2,
            DISABLED = 3,
            ENABLED_OFFLINE = 6
        }

        /// <summary>
        /// Visible Screen see more info in the KVM guide
        /// </summary>
        public enum SCREEN
        {

            VISIBLE_SCREEN_DEFAULT_SCREEN = 0,
            VISIBLE_SCREEN_EXTERNAL_SCREEN = 1
        };


        private bool _cached = false;


        private AmtSystem system;
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="system"></param>
        public AmtKVM(AmtSystem system)
        {
            this.system = system;
            fetchCache(); 
        }

        public void ClearCache()
        {
            _cached = false;
            _KVMEnabledInMEBx = null; 
            _KVMPort5900Enabled = null;             
            _KVMOptInRequired = null; 
            _KVMOptInTimeout = null; 
            _KVMSessionTimeout = null; 
            _KVMDefaultScreen = null;
            _KVMEnabled = null;
            _KVMSupported = null; 
        }

        private void fetchCache()
        {
            if (!_cached)
            {
                try
                {
                    IPS_KVMRedirectionSettingData KVMSetting = new IPS_KVMRedirectionSettingData(system.WsMan);
                    KVMSetting.Get(); 
                    _KVMEnabledInMEBx = KVMSetting.EnabledByMEBx;
                    _KVMPort5900Enabled = KVMSetting.Is5900PortEnabled; 
                    _KVMOptInRequired = KVMSetting.OptInPolicy; 
                    if(_KVMOptInRequired.Value)
                        _KVMOptInTimeout = KVMSetting.OptInPolicyTimeout;                    
                    _KVMSessionTimeout = KVMSetting.SessionTimeout; 
                    _KVMDefaultScreen = KVMSetting.DefaultScreen;
                    _KVMSupported = true; 

                
                    CIM_RedirectionService RedirService = new CIM_RedirectionService(system.WsMan);
                    RedirService.Get();
                    _KVMEnabled = (EnabledStateValues)RedirService.EnabledState;
                    
                }
                #region catch block
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.fetchCache WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.fetchCache CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.fetchCache",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                    _KVMSupported = false; 
                    _KVMEnabledInMEBx = false;
                    _KVMPort5900Enabled = false;
                    _KVMOptInRequired = false;
                    _KVMOptInTimeout = 0;
                    _KVMSessionTimeout = 0;
                    _KVMDefaultScreen = 0; 

                    return;
                }
                #endregion
                finally
                {
                    _cached = true;
                }
            }

        }

        /// <summary>
        /// Check if KVM is supported
        /// </summary>
        private bool? _KVMSupported = null;
        public bool? KVMSupported
        {
            get
            {
                fetchCache();
                return _KVMSupported;
            }
        }

        /// <summary>
        /// Check if KVM is enabled in MEBx
        /// </summary>
        private bool? _KVMEnabledInMEBx = null; 
        public bool? KVMEnabledInMEBx
        {
            get
            {
                fetchCache();
                return _KVMEnabledInMEBx; 
            }
        }

        /// <summary>
        /// Check if KVM is enabled for default port 5900
        /// </summary>
        private bool? _KVMPort5900Enabled = null; 
        public bool? KVMPort5900Enabled
        {
            get
            {
                fetchCache();
                return _KVMPort5900Enabled; 
            }
            set
            {
                try
                {
                    //check if enabled in MEBx is true.  this value is a READONLY value,
                    //and can not be changed
                    IPS_KVMRedirectionSettingData KVMSetting = new IPS_KVMRedirectionSettingData(system.WsMan);
                    KVMSetting.Get();
                    KVMSetting.Is5900PortEnabled = value.Value;
                    KVMSetting.Put();

                    ClearCache(); 

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information,
                        "SetKVMPort5900Enabled() set Is5900PortEnabled as " + value.Value);
                }
                #region catch block
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMPort5900Enabled WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMPort5900Enabled CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMPort5900Enabled",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                }
                #endregion
            }


        }


        /// <summary>
        /// Get the AMT_RedirectionService.ListenerEnabled state.  Getting this from the 
        /// AmtRedirect class, which already contains the functionailty for this.
        /// </summary>
        /// <returns></returns>
        public bool? KVMRedirectionPortEnabled
        {
            get
            {
                if (system.Redirection != null)
                    return system.Redirection.GetRedirectionListenerState();
                else
                    return false; 
            }
            set
            {
                if (system.Redirection == null)
                    return;

                //dont need try/catch, it is handled by Redirection class object
                AmtCallStatus returnStatus = system.Redirection.SetRedirectionListenerState(value.Value);

                ClearCache(); 
            }
        }


        /// <summary>
        /// If KVM is enabled in MEBx, this function will set KVM state
        /// </summary>
        /// <param name="desiredState"></param>
        ///    ENABLED=2,
        ///    DISABLED=3,
        ///    ENABLED_OFFLINE=6 //do we need this value?
        /// <returns>KVMState
        ///     DISABLED_MEBX=0,    
        ///     DISABLED,
        ///     ENABLED_REDIR_PORT_ONLY,
        ///     ENABLED_STANDARD_PORT_ONLY,
        ///     ENABLED_BOTH_PORTS,
        ///     ERROR_UNKNOWN_STATE</returns>
        public KVMStates EnableKVMPort(EnabledStateValues desiredState)
        {
            if (KVMEnabledInMEBx == true)
            {
                try
                {
                    //can change other enabled state

                    CIM_RedirectionService RedirService = new CIM_RedirectionService(system.WsMan);
                    RedirService.Get();

                    EnabledStateValues CurrentState = (EnabledStateValues)RedirService.EnabledState;

                    if (CurrentState != desiredState)
                    {
                        //change state
                        CIM_KVMRedirectionSAP kvmSap = new CIM_KVMRedirectionSAP(system.WsMan);
                        kvmSap.Get();
                        CimReference job;
                        uint status = kvmSap.RequestStateChange((ushort)desiredState, null, out job);
                        if (status != 0)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed,
                                "AmtKVM.EnabledStateValues CIM_KVMRedirectionSAP.RequestStateChange "
                                + "returned status error: " + status);


                            return KVMStates.ERROR_UNKNOWN_STATE;
                        }
                    }

                    return KVMState.Value;
                }
                #region catch block
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.EnabledStateValues WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.EnabledStateValues CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.EnabledStateValues",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                    return KVMStates.ERROR_UNKNOWN_STATE;
                }
                #endregion


            }
            else
            {
                //will not change other enabled states since KVM is 
                //disabled in MEBx... 

                return KVMStates.DISABLED_MEBX;
            }

            //delete this later
            //return KVMState.DISABLED; 
        }

        private EnabledStateValues? _KVMEnabled = null;
        public EnabledStateValues? KVMEnabled
        {
            get
            {
                fetchCache();
                return _KVMEnabled;
            }
        }

        /// <summary>
        /// This function return the current KVM state
        /// </summary>
        /// <returns>KVMState</returns>
        ///     DISABLED_MEBX=0,    
        ///     DISABLED,
        ///     ENABLED_REDIR_PORT_ONLY,
        ///     ENABLED_STANDARD_PORT_ONLY,
        ///     ENABLED_BOTH_PORTS,
        ///     ERROR_UNKNOWN_STATE
        public KVMStates? KVMState
        {
            get
            {
                if (KVMEnabled == null)
                {
                    return KVMStates.ERROR_UNKNOWN_STATE;
                }            
                else if (KVMEnabled.Value == EnabledStateValues.DISABLED)
                {
                    return KVMStates.DISABLED;
                }
                else
                {

                    if (KVMRedirectionPortEnabled.Value && KVMPort5900Enabled.Value)
                    {
                        return KVMStates.ENABLED_BOTH_PORTS;
                    }
                    else if (KVMRedirectionPortEnabled.Value)
                    {
                        return KVMStates.ENABLED_REDIR_PORT_ONLY;
                    }
                    else if (KVMPort5900Enabled.Value)
                    {
                        return KVMStates.ENABLED_STANDARD_PORT_ONLY;
                    }
                    else
                    {
                        return KVMStates.DISABLED; //KVM is enabled in MEBx & network interface but no ports are enabled
                    }
                }


            }
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        private bool? _KVMOptInRequired = null; 
        public bool? KVMOptInRequired
        {
            get
            {
                fetchCache(); 
                return _KVMOptInRequired;
            }
            set
            {
                try
                {
                    IPS_KVMRedirectionSettingData KVMSetting = new IPS_KVMRedirectionSettingData(system.WsMan);
                    KVMSetting.Get();
                    KVMSetting.OptInPolicy = value.Value;
                    KVMSetting.Put();

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information,
                        "SetKVMOptInRequired() set OptInPolicy as " + value.Value);

                    ClearCache(); 
                }
                #region catch block
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMOptInRequired WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMOptInRequired CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMOptInRequired",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                }
                #endregion
            }

        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        private UInt16? _KVMOptInTimeout = null; 
        public UInt16? KVMOptInTimeout
        {
            get
            {
                fetchCache();
                return _KVMOptInTimeout; 
            }
            set
            {
                try
                {
                    //because input value is a uint16 it will be between 0-65535

                    if ((value.Value >= 0 && value.Value <= 9) || (value.Value >= 4069))
                    {
                        //error, invalid value
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed,
                            "AmtKVM.SetKVMOptInTimeout, invalid Timeout value used, will not set new Timeout value."
                            + " Valid range is between 10-4095.");
                    }
                    //else value is in acceptable range

                    IPS_KVMRedirectionSettingData KVMSetting = new IPS_KVMRedirectionSettingData(system.WsMan);
                    KVMSetting.Get();
                    KVMSetting.OptInPolicyTimeout = value.Value;
                    KVMSetting.Put();

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information,
                        "SetKVMOptInTimeout() set OptInPolicyTimeout as " + value.Value);

                    ClearCache();
                }
                #region catch block
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMOptInTimeout WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMOptInTimeout CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMOptInTimeout",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                }
                #endregion
            }
        }


        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        private UInt16? _KVMSessionTimeout = null; 
        public UInt16? KVMSessionTimeout
        {
            get
            {
                fetchCache();
                return _KVMSessionTimeout; 
            }
            set
            {
                try
                {
                    //because input value is a uint16 it will be between 0-65535
                    if (value.Value > 255)
                    {
                        //error, invalid value
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed,
                            "AmtKVM.SetKVMSessionTimeout, invalid Timeout value used, will not set new Timeout value."
                            + " Valid range is between 0-255.");
                    }
                    //else value is in acceptable range

                    IPS_KVMRedirectionSettingData KVMSetting = new IPS_KVMRedirectionSettingData(system.WsMan);
                    KVMSetting.Get();

                    KVMSetting.SessionTimeout = value.Value;

                    KVMSetting.Put();

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information,
                        "SetKVMSessionTimeout() set SessionTimeout as " + value.Value);

                    ClearCache(); 
                }
                #region catch block
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMSessionTimeout WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMSessionTimeout CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMSessionTimeout",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                }
                #endregion
            }
        }




        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public bool SetKVMRfbPassword(string password)
        {
            try
            {
                bool ValidPassword = true;

                //string lenth < 8
                if (password.Length < 8)
                    ValidPassword = false;

                //contains capitol letter
                if (ValidPassword)
                {
                    foreach (char currentChar in password.ToCharArray())
                    {
                        if (char.IsUpper(currentChar))
                        {
                            ValidPassword = true;
                            break;
                        }

                        ValidPassword = false;
                    }
                }

                //lower case
                if (ValidPassword)
                {
                    foreach (char currentChar in password.ToCharArray())
                    {
                        if (char.IsLower(currentChar))
                        {
                            ValidPassword = true;
                            break;
                        }

                        ValidPassword = false;
                    }

                }

                //special character
                if (ValidPassword)
                {
                    Regex SpecialChar = new Regex("[!@#$%^&*()]");
                    ValidPassword = SpecialChar.IsMatch(password);
                }

                //digit
                if (ValidPassword)
                {
                    foreach (char currentChar in password.ToCharArray())
                    {
                        if (char.IsDigit(currentChar))
                        {
                            ValidPassword = true;
                            break;
                        }

                        ValidPassword = false;
                    }

                }

                if (!ValidPassword)
                {
                    //error, invalid value
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed,
                        "AmtKVM.SetKVMRfbPassword, invalid RFB Password specified."
                        + " Password must be 8 characters long and contain at least one "
                        + " capital letter, one regular letter, one digit, and one special"
                        + " character.");

                    return false;
                }

                //else value is in acceptable range

                IPS_KVMRedirectionSettingData KVMSetting = new IPS_KVMRedirectionSettingData(system.WsMan);
                KVMSetting.Get();

                KVMSetting.RFBPassword = password;

                KVMSetting.Put();

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information,
                    "SetKVMRfbPassword() set RFBPassword as " + password);

                return true;
            }
            #region catch block
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMRfbPassword WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMRfbPassword CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMRfbPassword",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                return false;
            }
            #endregion
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        private ushort? _KVMDefaultScreen = null; 
        public ushort? KVMDefaultScreen
        {
            get
            {
                fetchCache();
                return _KVMDefaultScreen; 
            }
            set
            {
                try
                {
                    //because input value is a uint16 it will be between 0-65535

                    if (value.Value > 1)
                    {
                        //error, invalid value
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed,
                            "AmtKVM.SetKVMDefaultScreen, invalid Sreen value used, will not set new Sreen value."
                            + " Valid range is between 0-1.");
                    }
                    //else value is in acceptable range

                    IPS_KVMRedirectionSettingData KVMSetting = new IPS_KVMRedirectionSettingData(system.WsMan);
                    KVMSetting.Get();
                    KVMSetting.DefaultScreen = (byte)value.Value;
                    KVMSetting.Put();

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information,
                        "SetKVMDefaultScreen() set DefaultScreen as " + value.Value);

                    ClearCache(); 
                }
                #region catch block
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMDefaultScreen WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMDefaultScreen CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtKVM.SetKVMDefaultScreen",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                }
                #endregion
            }
        }

    }
}
