﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================

using System;
using System.IO;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using System.Runtime.InteropServices;

using RealVNC.VNCViewerControl;
using RealVNC.VNCViewerSDK;


namespace ManageabilityStack
{
    public class AmtKVMWrapper
    {
        private enum PROXY_TYPE
        {
            PROXY_NO_PROXY,
            PROXY_SOCKS_V5
        };

        private string ip;
        private string username;
        private string password;
        private string domain; 
        private bool UseTls;
        private X509Certificate2 Certificate = null;
        private AmtRedirectorWrapper.SOCKSProxy proxy;
        private static ConnectionInfo info; 

        private X509Certificate2[] serverCerts = null;
        public X509Certificate2[] ServerCerts
        {
            get { return this.serverCerts; }
            set { this.serverCerts = value; }
        }

        private VNCViewerControl _viewerPanel;

        public VNCViewerControl ViewerPanel
        {
            get { return this._viewerPanel; }
            set { this._viewerPanel = value; }
        }

        public AmtKVMWrapper(string ip, string username, string password, bool tls, X509Certificate2 cert)
        {
            this.ip = ip;
            this.username = username;
            this.password = password;
            this.Certificate = cert;
            UseTls = tls;
            this.proxy = null;
            this.domain = null; 

            info = new ConnectionInfo();
            info.host = ip;
            info.host_port = (ushort)(tls ? AMT_PORTS.SECURE : AMT_PORTS.NON_SECURE);

            if (username.Contains("\\"))
            {
                //split domain/username 

                this.domain = username.Substring(0, username.IndexOf("\\"));
                this.username = username.Substring(username.IndexOf("\\") + 1); 
            }

            info.proxy = string.Empty;
            info.proxy_port = 8090;
            info.proxy_user = string.Empty;
            info.proxy_pwd = string.Empty;
        }

        public AmtKVMWrapper(string ip, string username, string password, bool tls, X509Certificate2 cert, AmtRedirectorWrapper.SOCKSProxy proxy)
        {
            this.ip = ip;
            this.username = username;
            this.password = password;
            this.Certificate = cert;
            UseTls = tls;
            this.proxy = proxy;
            this.domain = null; 

            info = new ConnectionInfo();
            info.host = ip;
            info.host_port = (ushort)(tls ? AMT_PORTS.SECURE : AMT_PORTS.NON_SECURE);

            if (username.Contains("\\"))
            {
                //split domain/username 
                this.domain = username.Substring(0, username.IndexOf("\\"));
                this.username = username.Substring(username.IndexOf("\\") + 1);
            }

            if (proxy != null)
            {
                info.proxy = proxy.server;
                info.proxy_port = proxy.port;
                info.proxy_user = proxy.user;
                info.proxy_pwd = proxy.password;
            }
            else
            {
                info.proxy = string.Empty;
                info.proxy_port = 8090;
                info.proxy_user = string.Empty;
                info.proxy_pwd = string.Empty;
            }
        }

        /// </summary>
        public void InitializeKVMLib()
        {
            int status = 0; 
          

            //setup the TLS and MTLS certificates needed for initilizing the KVMLib
            SetupCertificates();

            if (UseTls)
            {
                if (Certificate != null)
                {
                    //MTLS - need both root and private certificate PEM files
                    status = KVMInit("Trusted Root Certificates.pem",
                                    Application.StartupPath + "\\PrivateCert.pem",
                                    "");
                }
                else
                {
                    //TLS - need just root certificate PEM file
                    status = KVMInit("Trusted Root Certificates.pem", "", "");
                }

            }
            else 
            {
                //noTLS
                status = KVMInit("", "", "");
            }


            // notify about the status changed
            if (0 != status)
            {
                //not able to initilize the KVM library
                throw new Exception("Could not initialize the KVMLib");
            }
        }

        /// <summary>
        /// Initiazlie the VNCViewerControl and register the relevant events
        /// </summary>
        public void InitializeVNCControl()
        {
            _viewerPanel = new VNCViewerControl();
            _viewerPanel.Dock = DockStyle.Fill;
            _viewerPanel.Anchor = AnchorStyles.Bottom | AnchorStyles.Left | AnchorStyles.Right | AnchorStyles.Top;
            _viewerPanel.AutoSize = false;
            _viewerPanel.AutoSizeMode = AutoSizeMode.GrowAndShrink;

        }


        private void SetupCertificates()
        {
            if (UseTls)
            {
                if (!File.Exists("Trusted Root Certificates.pem"))
                {
                    // Attempt to create the trusted root certificates file.
                    CertificateOperations.AllTrustedRootsToPemFile("Trusted Root Certificates.pem");
                }
                if (this.ServerCerts != null)
                {
                    // Use the given server certificates (from the client Intel(R) 
                    // AMT system to generate the certificate chain for SOL.
                    // NOTE: This is a work around for when Sub-CA certs are not 
                    // available locally to the management system (where Commander 
                    // is running).
                    CertificateOperations.AllRootsToPemFile("Trusted Root Certificates.pem", this.ServerCerts);
                }

                if (File.Exists("Trusted Root Certificates.pem"))
                {

                    if (Certificate != null)
                    {
                        //setup for MTLS
                        // Convert the certificate to PEM
                        CertificateOperations.CertAndKeyChainToPemFile(Certificate, "PrivateCert.pem", null);
                    }
                }
            }
        }

        public enum KVMConnectionType
        {
            Redirection =0, //AMT port 16994/16995
            Standard =1     //RFB port 5900
        }

        public int StartKVM(KVMConnectionType KVMType)
        {
            int status = 0;

            // if using redirection ports, start the proxy and connect the VNCViewerControl to the proxy           
            if(KVMType == KVMConnectionType.Redirection)
            {
                if (UseTls)
                {
                    if (Certificate != null)
                    {
                        //MTLS - need both root and private certificate PEM files
                        KVMSetCertificates("Trusted Root Certificates.pem",
                                        Application.StartupPath + "\\PrivateCert.pem",
                                        "");
                    }
                    else
                    {
                        //TLS - need just root certificate PEM file
                        KVMSetCertificates("Trusted Root Certificates.pem", "", "");
                    }

                }
                else 
                {
                    //noTLS
                    KVMSetCertificates("", "", "");
                }

                
                //TODO: check if any CIRA stuff need to do here
                //in SDK example not doing anything here....

                //I think that the CIRA stuff get set in the KVMInit function with 
                //the getConnectionInfo() function 

                // update cira socks proxy
                _viewerPanel.SetParameter("ProxyServer", string.Empty);
                _viewerPanel.SetParameter("ProxyType", string.Empty);
                ushort port = 0;
                status = KVMStart(ref port);
                if (0 != status)
                {
                    return status;
                }
                _viewerPanel.Connect(IPAddress.Loopback.ToString() + ":" + port);
            }
            // if using default port connect to intel(r) AMT on the default port (5900)
            else if(KVMType == KVMConnectionType.Standard)
            {
                if (proxy!=null)
                {
                    string proxystring = proxy.server + ":" + proxy.port; 

                    // update cira socks proxy
                    _viewerPanel.SetParameter("ProxyServer", proxystring); 
                    _viewerPanel.SetParameter("ProxyType", "socks");
                }
                else
                {
                    _viewerPanel.SetParameter("ProxyServer", string.Empty);
                    _viewerPanel.SetParameter("ProxyType", string.Empty);
                }
                _viewerPanel.Connect(ip);
            }
           
            return status;        
        }

        public void StopKVM()
        {
            _viewerPanel.Disconnect();

            //this call will only succeed if the KVMLib was started.
            //will return an error code otherwise
            //ignoring return code
            KVMStop();
        }

        #region KVMLib

        private const string KVMSharedlibrary = "KVMLib.dll";

        private enum AMT_PORTS
        {
            NON_SECURE = 16994,
            SECURE = 16995
        };

        private const int MAX_STRING_FIELD_SIZE = 256;

        /// <summary>
        /// This struct should be filled with platform connection information
        /// when a new viewer connects to the kvmlib
        /// </summary>
        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        public struct ConnectionInfo
        {
            /// <summary>
            /// Intel(r) AMT Hostname / IP Address
            /// </summary>
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_STRING_FIELD_SIZE)]
            public string host;
            /// <summary>
            /// 16994 for non-TLS connections, 16995 for TLS connections
            /// </summary>
            public ushort host_port;
            /// <summary>
            /// Username of Intel(r) AMT user
            /// </summary>
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_STRING_FIELD_SIZE)]
            public string user;
            /// <summary>
            /// Password of Intel(r) AMT user
            /// </summary>
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_STRING_FIELD_SIZE)]
            public string pwd;
            /// <summary>
            /// SOCKS proxy Hostname
            /// </summary>
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_STRING_FIELD_SIZE)]
            public string proxy;
            /// <summary>
            ///  SOCKS proxy Port
            /// </summary>
            public ushort proxy_port;
            /// <summary>
            /// Username of the SOCKS proxy user
            /// </summary>
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_STRING_FIELD_SIZE)]
            public string proxy_user;
            /// <summary>
            /// Password of the SOCKS proxy user
            /// </summary>
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_STRING_FIELD_SIZE)]
            public string proxy_pwd;
            /// <summary>
            /// The clientID will be returned from the KVM library 
            /// </summary>
            public uint clientID;
            /// <summary>
            /// Domain name of Intel(r) AMT user name (used for kerberos authentication)
            /// </summary>
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MAX_STRING_FIELD_SIZE)]
            public string domain;
        };
        /// <summary>
        /// Declaration of a callback function; this function will be passed to
        /// the KVM_init function
        /// </summary>
        /// <param name="s">the structure filled by the callback function</param>
        public delegate void getConnectionInfo(ref ConnectionInfo s);

        /// <summary>
        /// This function is called when a new VNC connection is
        /// initiated
        /// </summary>
        /// <param name="info">A structure containing the latest configuration settings</param>
        protected static void doGetConnectionInfo(ref ConnectionInfo ininfo)
        {
            
            // receive the details to connect to the Intel AMT machine
            ininfo = info; 
        }

        // c function that accepts a callback as param
        [DllImport(KVMSharedlibrary)]
        private static extern int KVM_Init(string root_cert, string perosnal_cert, string personal_pwd, getConnectionInfo p);

        /// <summary>
        /// Frees all the resources acquired by the library
        /// as a result of an Init request and subsequent calls
        /// </summary>
        /// <returns>0 upon success</returns>
        [DllImport(KVMSharedlibrary)]
        private static extern int KVM_Deinit();

        /// <summary>
        /// Allows changing the certificates after initialization of the library
        /// </summary>
        /// <param name="root_cert">path to the root certificate file, otherwise an empty string</param>
        /// <param name="perosnal_cert">path to the personal certificate file, otherwise an empty string</param>
        /// <param name="personal_pwd">password of the personal certificate file, otherwise an empty string</param>
        /// <returns></returns>
        [DllImport(KVMSharedlibrary)]
        private static extern int KVM_SetCertificates(string root_cert, string perosnal_cert, string personal_pwd);

        /// <summary>
        /// This function starts the KVMlib listening to the specified port.
        /// if the specified port is 0 the KVMLib will choose a free port
        /// randomly and will assign it to the variable port.
        /// </summary>
        /// <param name="port">
        /// 0 - KVMlib will choose available free port and assign it to the variable port
        /// 1-65595 - the port that the KVMLib will use
        /// </param>
        /// <returns>0 upon success</returns>
        [DllImport(KVMSharedlibrary)]
        private static extern int KVM_Start(ref ushort port);

        /// <summary>
        /// This function stops the KVMLib listening to the designated port 
        /// </summary>
        /// <returns>0 upon success</returns>
        [DllImport(KVMSharedlibrary)]
        private static extern int KVM_Stop();

        private static getConnectionInfo _info;
        /// <summary>
        /// KVM_init initializes the KVM library. It should be called only when the application is loaded
        /// </summary>
        /// <param name="root_cert">path to the root certificate file. Required for TLS connections, 
        /// used to validate Intel AMT server certificate. If TLS is not required the string should be empty.</param>
        /// <param name="perosnal_cert">path to the client certificate file. Required for TLS Mutual Authentication.
        /// If Mutual Authentication is not required the string should be empty.</param>
        /// <param name="personal_pwd">password of the client certificate, if the certificate is password-protected.
        /// If the certificate is not password protected, the string should be empty.</param>
        /// <returns>0 upon success</returns>
        public static int KVMInit(string root_cert, string perosnal_cert, string personal_pwd)
        {
            int status = -1;
            _info = new getConnectionInfo(doGetConnectionInfo);
            status = KVM_Init(root_cert, perosnal_cert, personal_pwd, _info);
            return status;
        }

        /// <summary>
        /// Frees all the resources used by the library
        /// </summary>
        /// <returns>0 upon success</returns>
        public static int KVMDeinit()
        {
            int status = -1;
            status = KVM_Deinit();
            return status;
        }

        /// <summary>
        /// Allows changing certificate information after initialization of the library
        /// </summary>
        /// <param name="root_cert">path to the root certificate file. Required for TLS connections, 
        /// used to validate Intel AMT server certificate. If TLS is not required the string should be empty.</param>
        /// <param name="perosnal_cert">path to the client certificate file. Required for TLS Mutual Authentication.
        /// If Mutual Authentication is not required the string should be empty.</param>
        /// <param name="personal_pwd">password of the client certificate, if the certificate is password-protected.
        /// If the certificate is not password protected, the string should be empty.</param>
        /// <returns></returns>
        public static int KVMSetCertificates(string root_cert, string perosnal_cert, string personal_pwd)
        {
            int status = -1;
            status = KVM_SetCertificates(root_cert, perosnal_cert, personal_pwd);
            return status;
        }

        /// <summary>
        /// This function starts the KVMlib on the specified port.
        /// if the specified port is 0 the KVMLib will randomly choose
        /// a free port and assign it to the variable port.
        /// </summary>
        /// <param name="port">
        /// 0 means that the KVMlib will choose an available free port and assigned it to the variable port
        /// 1-65595 - the port that the KVMLib will listen to
        /// </param>
        /// <returns>0 upon success</returns>
        public static int KVMStart(ref ushort port)
        {
            int status = -1;
            status = KVM_Start(ref port);
            return status;
        }

        /// <summary>
        /// This function will stops the KVMLib listening to the designated port
        /// </summary>
        /// <returns>0 upon success</returns>
        public static int KVMStop()
        {
            int status = -1;
            status = KVM_Stop();
            return status;
        }
    }

    #endregion
}
