// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Net;
using System.Text.RegularExpressions;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;

namespace ManageabilityStack
{
    public class AmtNetworkAdmin
    {
        private NetworkAdministrationService service;
        private AmtSystem system;
        public AmtNetworkAdminInterface[] interfaces = null;
        public AmtNetworkAdminInterface Wired = null;
        public AmtNetworkAdminInterface Wireless = null;
        private bool cached = false;
        public bool IsCached() { return cached; }

        public AmtNetworkAdmin(AmtSystem system, NetworkAdministrationService service)
        {
            this.service = service;
            this.system = system;
            bool setup = false;

            if (system.AmtRev >= 3)
            {
                try
                {
                    // Use the new interface system
                    uint[] interfaceHandles;
                    AmtCallStatus r = EnumerateInterfaces(out interfaceHandles);

                    if (r == AmtCallStatus.SUCCESS)
                    {
                        interfaces = new AmtNetworkAdminInterface[interfaceHandles.Length];
                        for (int i = 0; i < interfaces.Length; i++)
                        {
                            interfaces[i] = new AmtNetworkAdminInterface(system, service, this, interfaceHandles[i]);
                            if (interfaceHandles[i] == 0) Wired = interfaces[i];
                            if (interfaceHandles[i] == 1) Wireless = interfaces[i];
                        }
                    }
                    setup = true;
                }
                catch (System.Web.Services.Protocols.SoapHeaderException) {}
            }
            
            if (setup == false)
            {
                // Use the older system
                interfaces = new AmtNetworkAdminInterface[1];
                interfaces[0] = new AmtNetworkAdminInterface(system, service, this, 0);
                Wired = interfaces[0];
            }
        }

        public void Dispose()
        {
            ClearCache();
            service = null;
        }

        public void ClearCache()
        {
            cached = false;
            HostNameCache = null;
            DomainNameCache = null;
            GetPingResponseCacheSet = false;
            DhcpModeCache = 0;
            StaticIPv4ParametersCache = null;
            Get8021XActiveS0CacheSet = false;
            Get8021XPxeTimeoutCacheSet = false;
            IPv6InfoCache = null;

            for (int i = 0; i < interfaces.Length; i++) interfaces[i].ClearCache();
        }

        public void FetchCache()
        {
            try
            {
                GetHostName();
                GetDomainName();
                GetPingResponse();
                GetIPv6Settings();

                bool resp;
                Get8021XActiveS0(out resp);
                uint timeout;
                Get8021XPxeTimeout(out timeout);

                for (int i = 0; i < interfaces.Length; i++) interfaces[i].FetchCache();
                cached = true;
            }
            catch (Exception e)
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Failure to retrieve the Network Administration data.",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
            }
        }

        private IPv6PortSettings[] IPv6InfoCache = null;
        public IPv6PortSettings[] GetIPv6Settings()
        {
            if (IPv6InfoCache != null)
                return IPv6InfoCache;

            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    if (system.AmtRev >= 6)
                    {
                        Collection<IPS_IPv6PortSettings> ipv6Ports = IPS_IPv6PortSettings.Enumerate(system.WsMan);
                        IPv6InfoCache = new IPv6PortSettings[ipv6Ports.Count];
                        for (int i = 0; i < ipv6Ports.Count; i++)
                            IPv6InfoCache[i] = new IPv6PortSettings(ipv6Ports[i]);
                    }

                    return IPv6InfoCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.GetIPv6Settings WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.GetIPv6Settings CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.GetIPv6Settings",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    IPv6InfoCache = null;
                    return IPv6InfoCache;
                }
            }
            else
                return null;
        }

        public AmtCallStatus SetIpv6Settings(IPv6PortSettings newIpv6PortSettings)
        {
            if (system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            try
            {
                //
                // Find the matching IPv6 Port Settings instance to update.
                //
                Collection<IPS_IPv6PortSettings> ipv6Ports = IPS_IPv6PortSettings.Enumerate(system.WsMan);
                foreach (IPS_IPv6PortSettings ipv6Port in ipv6Ports)
                {
                    if (ipv6Port.InstanceIDExist &&
                        String.Compare(ipv6Port.InstanceID, newIpv6PortSettings.InstanceID, true) == 0)
                    {

                        ipv6Port.InterfaceIDType = (uint)newIpv6PortSettings.InterfaceID;
                        if (newIpv6PortSettings.InterfaceID == IPv6PortSettings.InterfaceIDType.Manual)
                            ipv6Port.ManualInterfaceID = newIpv6PortSettings.ManualInterfaceID;

                        ipv6Port.IPv6Address = newIpv6PortSettings.IPv6Address.ToString();
                        ipv6Port.PrimaryDNS = newIpv6PortSettings.PrimaryDNS.ToString();
                        ipv6Port.SecondaryDNS = newIpv6PortSettings.SecondaryDNS.ToString();
                        ipv6Port.DefaultRouter = newIpv6PortSettings.DefaultRouter.ToString();

                        ipv6Port.Put();
                        return AmtCallStatus.SUCCESS;
                    }
                } // END foreach (IPS_IPv6PortSettings ipv6Port in ipv6Port)

                return AmtCallStatus.INVALID_PARAMETER;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.SetIpv6Settings WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.SetIpv6Settings CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.SetIpv6Settings",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        public bool IsIpv6Enabled(string instanceId)
        {
            if (system.WsMan == null) return false;

            IPS_IPv6PortSettings ipv6PortSettings = null;
            try
            {
                Collection<IPS_IPv6PortSettings> ipv6Ports = IPS_IPv6PortSettings.Enumerate(system.WsMan);
                foreach (IPS_IPv6PortSettings ipv6Port in ipv6Ports)
                {
                    if (String.Compare(ipv6Port.InstanceID, instanceId, true) == 0)
                    {
                        ipv6PortSettings = ipv6Port;
                        break;
                    }
                } // END foreach (IPS_IPv6PortSettings ipv6Port in ipv6Ports)
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6 WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6 CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return false; // Unable to find matching interface ID.
            }

            if (ipv6PortSettings == null) return false; // Unable to find a matching interface ID.

            Collection<CIM_ElementSettingData> elements = null;
            try
            {
                elements = CIM_ElementSettingData.Enumerate(system.WsMan);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6 WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6 CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return false;
            }
            if (elements == null) return false;

            foreach (CIM_ElementSettingData element in elements)
            {
                if ((String.Compare(element.SettingData.referenceParameters.ResourceURI,
                                    ipv6PortSettings.Reference.referenceParameters.ResourceURI, true) == 0) &&
                    (String.Compare(element.SettingData.referenceParameters.keys[0].Name,
                                    ipv6PortSettings.Reference.referenceParameters.keys[0].Name, true) == 0))
                {
                    if (element.IsCurrentExist == true)
                        return (element.IsCurrent == 1 ? true : false);
                    else
                        return false;
                }
            } // END foreach (CIM_ElementSettingData element in elements)

            return false;
        }

        public AmtCallStatus EnableIpv6(string instanceId, bool enable)
        {
            if (system.WsMan == null) return AmtCallStatus.NO_WSMAN;

            IPS_IPv6PortSettings ipv6PortSettings = null;
            try
            {
                Collection<IPS_IPv6PortSettings> ipv6Ports = IPS_IPv6PortSettings.Enumerate(system.WsMan);
                foreach (IPS_IPv6PortSettings ipv6Port in ipv6Ports)
                {
                    if (String.Compare(ipv6Port.InstanceID, instanceId, true) == 0)
                    {
                        ipv6PortSettings = ipv6Port;
                        break;
                    }
                } // END foreach (IPS_IPv6PortSettings ipv6Port in ipv6Ports)
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6 WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6 CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.INVALID_PARAMETER; // Unable to find matching interface ID.
            }

            if (ipv6PortSettings == null) return AmtCallStatus.INVALID_PARAMETER; // Unable to find a matching interface ID.

            Collection<CIM_ElementSettingData> elements = null;
            try
            {
                elements = CIM_ElementSettingData.Enumerate(system.WsMan);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6 WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6 CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
            if (elements == null) return AmtCallStatus.DATA_MISSING;

            foreach (CIM_ElementSettingData element in elements)
            {
                if ((String.Compare(element.SettingData.referenceParameters.ResourceURI,
                                    ipv6PortSettings.Reference.referenceParameters.ResourceURI, true) == 0) &&
                    (String.Compare(element.SettingData.referenceParameters.keys[0].Value,
                                    ipv6PortSettings.Reference.referenceParameters.keys[0].Value, true) == 0))
                {
                    try
                    {
                        // Enable (1 = IS_CURRENT) or Disable (2 = IS_NOT_CURRENT)
                        element.IsCurrent = (ushort)(enable ? 1 : 2);
                        element.Put();
                        return AmtCallStatus.SUCCESS;
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        if (wsManException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6 WSMAN error",
                            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                        }
                        CimException cimException = e as CimException;
                        if (cimException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6 CIM error",
                            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                        }

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.EnableIpv6",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                        return AmtCallStatus.FAILED_WEB_CALL;
                    }
                }
            } // END foreach (CIM_ElementSettingData element in elements)

            return AmtCallStatus.DATA_MISSING;
        }

        private string HostNameCache = null;
        public string GetHostName()
        {
            if (system.WsMan != null)
            {
                if (HostNameCache != null) return HostNameCache;
                // USE WSMAN
                try
                {
                    AMT_GeneralSettings s = new AMT_GeneralSettings(system.WsMan);
                    s.Get();
                    HostNameCache = (s.HostNameExist ? s.HostName : null);
                    return HostNameCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.GetHostName WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.GetHostName CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.GetHostName",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    throw;
                }
            }
            else
            {
                // Use EOI
                if (HostNameCache != null) return HostNameCache;
                string hostName;
                service.GetHostName(out hostName);
                HostNameCache = hostName;
                return HostNameCache;
            }
        }

        private string DomainNameCache = null;
        public string GetDomainName()
        {
            if (system.WsMan != null)
            {
                if (DomainNameCache != null) return DomainNameCache;
                // USE WSMAN
                try
                {
                    AMT_GeneralSettings s = new AMT_GeneralSettings(system.WsMan);
                    s.Get();
                    DomainNameCache = (s.DomainNameExist ? s.DomainName : null);
                    return DomainNameCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.GetDomainName WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.GetDomainName CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.GetDomainName",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    throw;
                }
            }
            else
            {
                // USE EOI
                if (DomainNameCache != null) return DomainNameCache;
                if (system.AmtRev < 2) return null;
                string domainName;
                service.GetDomainName(out domainName);
                DomainNameCache = domainName;
                return DomainNameCache;
            }
        }

        public AmtCallStatus SetHostDomainName(string hostname, string domainName)
        {
            AmtCallStatus r;
            HostNameCache = null;
            DomainNameCache = null;
            r = (AmtCallStatus)service.SetHostName(hostname);
            if (r != AmtCallStatus.SUCCESS) return r;
            if (system.AmtRev >= 2)
            {
                r = (AmtCallStatus)service.SetDomainName(domainName);
                if (r != AmtCallStatus.SUCCESS) return r;
            }
            r = system.SecurityAdmin.CommitChanges();
            return r;
        }

        private bool GetPingResponseCache = false;
        private bool GetPingResponseCacheSet = false;
        public bool GetPingResponse()
        {
            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AMT_GeneralSettings s = new AMT_GeneralSettings(system.WsMan);
                    s.Get();
                    return s.PingResponseEnabled;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.GetPingResponse WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.GetPingResponse CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkAdmin.GetPingResponse",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    throw;
                }
            }
            else
            {
                // USE EOI
                if (GetPingResponseCacheSet == true) return GetPingResponseCache;
                AmtCallStatus r = (AmtCallStatus)service.GetPingResponse(out GetPingResponseCache);
                if (r == AmtCallStatus.SUCCESS) GetPingResponseCacheSet = true;
                return GetPingResponseCache;
            }
        }

        public AmtCallStatus SetPingResponse(bool enabled)
        {
            GetPingResponseCacheSet = false;
            return (AmtCallStatus)service.SetPingResponse(enabled);
        }

        public enum DhcpMode
        {
            DhcpModeInvalid = 0,
            DhcpModeDisabled = 1,
            DhcpModeEnabled = 2
        };

        private DhcpMode DhcpModeCache = 0;
        private StaticIPv4ParametersType StaticIPv4ParametersCache = null;
        public AmtCallStatus GetTcpIpParameters(out DhcpMode DhcpMode, out StaticIPv4ParametersType StaticIPv4Parameters)
        {
            if (StaticIPv4ParametersCache != null)
            {
                DhcpMode = DhcpModeCache;
                StaticIPv4Parameters = StaticIPv4ParametersCache;
                return AmtCallStatus.SUCCESS;
            }

            uint DhcpModeInt = 0;
            AmtCallStatus r = (AmtCallStatus)service.GetTcpIpParameters(out DhcpModeInt, out StaticIPv4Parameters);
            DhcpMode = (DhcpMode)DhcpModeInt;
            if (r == AmtCallStatus.SUCCESS)
            {
                DhcpModeCache = DhcpMode;
                StaticIPv4ParametersCache = StaticIPv4Parameters;
            }
            return r;
        }

        public AmtCallStatus SetTcpIpParameters(DhcpMode DhcpMode, StaticIPv4ParametersType StaticIPv4Parameters)
        {
            StaticIPv4ParametersCache = null;
            AmtCallStatus r = (AmtCallStatus)service.SetTcpIpParameters((uint)DhcpMode, StaticIPv4Parameters);
            return r;
        }

        private uint[] InterfaceHandlesCache = null;
        private AmtCallStatus EnumerateInterfaces(out uint[] interfaceHandles)
        {
            if (InterfaceHandlesCache != null)
            {
                interfaceHandles = InterfaceHandlesCache;
                return AmtCallStatus.SUCCESS;
            }
            AmtCallStatus r = (AmtCallStatus)service.EnumerateInterfaces(out interfaceHandles);
            if (r == AmtCallStatus.SUCCESS) InterfaceHandlesCache = interfaceHandles;
            return r;
        }

        private bool Get8021XActiveS0Cache = false;
        private bool Get8021XActiveS0CacheSet = false;
        public AmtCallStatus Get8021XActiveS0(out bool active)
        {
            active = false;
            if (system.AmtRev < 3 || (system.AmtRev == 2 && system.AmtRevMin < 6)) return AmtCallStatus.NOT_PERMITTED;
            if (system.Info.GetCoreVersion().Equals("3.0.2")) return AmtCallStatus.NOT_PERMITTED;
            if (Get8021XActiveS0CacheSet == true)
            {
                active = Get8021XActiveS0Cache;
                return AmtCallStatus.SUCCESS;
            }
            AmtCallStatus r;
            try
            {
                 r = (AmtCallStatus)service.Get8021XActiveS0(out active);
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                Get8021XActiveS0CacheSet = true;
                Get8021XActiveS0Cache = false;
                r = AmtCallStatus.FAILED_WEB_CALL;
            }
            if (r == AmtCallStatus.SUCCESS)
            {
                Get8021XActiveS0CacheSet = true;
                Get8021XActiveS0Cache = active;
            }
            return r;
        }

        public AmtCallStatus Set8021XActiveS0(bool active)
        {
            if (system.AmtRev < 3 || (system.AmtRev == 2 && system.AmtRevMin < 6)) return AmtCallStatus.NOT_PERMITTED;
            if (system.Info.GetCoreVersion().Equals("3.0.2")) return AmtCallStatus.NOT_PERMITTED;
            AmtCallStatus r = (AmtCallStatus)service.Set8021XActiveS0(active);
            if (r == AmtCallStatus.SUCCESS) Get8021XActiveS0CacheSet = false;
            return r;
        }

        private uint Get8021XPxeTimeoutCache = 0;
        private bool Get8021XPxeTimeoutCacheSet = false;
        public AmtCallStatus Get8021XPxeTimeout(out uint timeout)
        {
            timeout = 0;
            if (system.AmtRev != 2 || system.AmtRevMin < 6) return AmtCallStatus.NOT_PERMITTED;
            if (system.Info.GetCoreVersion().Equals("3.0.2")) return AmtCallStatus.NOT_PERMITTED;
            if (Get8021XPxeTimeoutCacheSet == true)
            {
                timeout = Get8021XPxeTimeoutCache;
                return AmtCallStatus.SUCCESS;
            }
            AmtCallStatus r = (AmtCallStatus)service.Get8021XPxeTimeout(out timeout);
            if (r == AmtCallStatus.SUCCESS)
            {
                Get8021XPxeTimeoutCacheSet = true;
                Get8021XPxeTimeoutCache = timeout;
            }
            return r;
        }

        public AmtCallStatus Set8021XPxeTimeout(uint timeout)
        {
            if (system.AmtRev != 2 || system.AmtRevMin < 6) return AmtCallStatus.NOT_PERMITTED;
            if (system.Info.GetCoreVersion().Equals("3.0.2")) return AmtCallStatus.NOT_PERMITTED;
            AmtCallStatus r = (AmtCallStatus)service.Set8021XPxeTimeout(timeout, true);
            if (r == AmtCallStatus.SUCCESS) Get8021XPxeTimeoutCacheSet = false;
            return r;
        }

    }

    public class AmtNetworkAdminInterface
    {
        private NetworkAdministrationService service;
        private AmtSystem system;
        private AmtNetworkAdmin parent;
        private uint handle;
        public uint Handle { get { return handle; } }

        public AmtNetworkAdminInterface(AmtSystem system, NetworkAdministrationService service, AmtNetworkAdmin parent, uint interfaceHandle)
        {
            this.service = service;
            this.system = system;
            this.parent = parent;
            this.handle = interfaceHandle;
        }

        public void ClearCache()
        {
            GetInterfaceSettingsCache = null;
        }

        public void FetchCache()
        {
            try
            {
                GetInterfaceSettings();
            }
            catch (Exception e)
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Failure to retrieve the Network Administration Interface data.",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
            }
        }

        private InterfaceDescriptorGetType GetInterfaceSettingsCache = null;
        public InterfaceDescriptorGetType GetInterfaceSettings()
        {
            if (GetInterfaceSettingsCache != null) return GetInterfaceSettingsCache;

            if (system.IsMobile == false && system.AmtRev <= 2)
            {
                // Old System
                AmtNetworkAdmin.DhcpMode DhcpMode;
                StaticIPv4ParametersType StaticIPv4Parameters;
                AmtCallStatus r = parent.GetTcpIpParameters(out DhcpMode, out StaticIPv4Parameters);
                if (r != AmtCallStatus.SUCCESS) return null;

                InterfaceDescriptorGetType i = new InterfaceDescriptorGetType();
                i.DhcpEnabled = (DhcpMode == AmtNetworkAdmin.DhcpMode.DhcpModeEnabled);
                i.IPv4Parameters = new IPv4ParametersType();
                i.IPv4Parameters.DefaultGatewayAddress = StaticIPv4Parameters.DefaultGatewayAddress;
                i.IPv4Parameters.LocalAddress = StaticIPv4Parameters.LocalAddress;
                i.IPv4Parameters.PrimaryDnsAddress = StaticIPv4Parameters.PrimaryDnsAddress;
                i.IPv4Parameters.SecondaryDnsAddress = StaticIPv4Parameters.SecondaryDnsAddress;
                i.IPv4Parameters.SubnetMask = StaticIPv4Parameters.SubnetMask;
                i.HardwareAddressDescription = Properties.Resources.WiredInterface;
                if (i.DhcpEnabled == true) { i.InterfaceMode = InterfaceModeType.SHARED_MAC_ADDRESS; } else { i.InterfaceMode = InterfaceModeType.SEPARATE_MAC_ADDRESS; }

                GetInterfaceSettingsCache = i;
                return GetInterfaceSettingsCache;
            }
            else
            {
                // New System
                InterfaceDescriptorGetType interfaceDescriptorGetType;
                AmtCallStatus r = (AmtCallStatus)service.GetInterfaceSettings(handle, out interfaceDescriptorGetType);
                if (r != AmtCallStatus.SUCCESS) return null;
                GetInterfaceSettingsCache = interfaceDescriptorGetType;
                return GetInterfaceSettingsCache;
            }
        }

        public AmtCallStatus SetInterfaceSettings(InterfaceDescriptorSetType descriptor)
        {
            if (system.AmtRev >= 3 || (system.AmtRev == 2 && system.AmtRevMin >= 5))
            {
                // New system
                return SetInterfaceSettingsNew(descriptor);
            }
            else
            {
                // Old system
                uint DhcpMode = 2; // DHCP Enabled
                StaticIPv4ParametersType ipparams = new StaticIPv4ParametersType();

                if (descriptor.IPv4Parameters != null)
                {
                    DhcpMode = 1; // DHCP Disabled
                    ipparams.LocalAddress = descriptor.IPv4Parameters.LocalAddress;
                    ipparams.DefaultGatewayAddress = descriptor.IPv4Parameters.DefaultGatewayAddress;
                    ipparams.PrimaryDnsAddress = descriptor.IPv4Parameters.PrimaryDnsAddress;
                    ipparams.SecondaryDnsAddress = descriptor.IPv4Parameters.SecondaryDnsAddress;
                    ipparams.SubnetMask = descriptor.IPv4Parameters.SubnetMask;
                    ipparams.DomainName = system.Network.GetDomainName();
                }

                return SetInterfaceSettingsOld(DhcpMode, ipparams);
            }
        }

        private AmtCallStatus SetInterfaceSettingsNew(InterfaceDescriptorSetType descriptor)
        {
            if (system.AmtRev < 2 || (system.AmtRev == 2 && system.AmtRevMin < 5)) return AmtCallStatus.NOT_PERMITTED;
            AmtCallStatus r = (AmtCallStatus)service.SetInterfaceSettings(handle, descriptor);
            if (r == AmtCallStatus.SUCCESS)
            {
                GetInterfaceSettingsCache = null;
                r = (AmtCallStatus)system.SecurityAdmin.CommitChanges();
            }
            return r;
        }

        private AmtCallStatus SetInterfaceSettingsOld(uint DhcpMode, StaticIPv4ParametersType ipparams)
        {
            if (system.AmtRev >= 3 || (system.AmtRev == 2 && system.AmtRevMin >= 5)) return AmtCallStatus.NOT_PERMITTED;
            if (DhcpMode == 2) ipparams = null;
            AmtCallStatus r = (AmtCallStatus)service.SetTcpIpParameters(DhcpMode, ipparams);
            if (r == AmtCallStatus.SUCCESS)
            {
                GetInterfaceSettingsCache = null;
                r = (AmtCallStatus)system.SecurityAdmin.CommitChanges();
            }
            return r;
        }

        public AmtCallStatus GetVlanParameters(out bool VLanMode, out ushort VLanTag)
        {
            AmtCallStatus r = AmtCallStatus.NOT_PERMITTED;
            VLanMode = false;
            VLanTag = 0;
            try
            {
                r = (AmtCallStatus)service.GetVlanParameters(handle, true, out VLanMode, out VLanTag);
            }
            catch (Exception) { }
            return r;
        }

        public AmtCallStatus SetVlanParameters(bool VLanMode, ushort VLanTag)
        {
            if (handle != 0) return AmtCallStatus.NOT_PERMITTED;
            AmtCallStatus r = (AmtCallStatus)service.SetVlanParameters(handle, false, VLanMode, VLanTag);
            return r;
        }

        private XProfileType Get8021XWiredProfileCache = null;
        public AmtCallStatus Get8021XWiredProfile(out XProfileType xprofile)
        {
            if (system.AmtRev < 3 && system.IsMobile == false) {xprofile = null;return AmtCallStatus.NOT_PERMITTED;}
            if (system.Info.GetCoreVersion().Equals("3.0.2")) { xprofile = null; return AmtCallStatus.NOT_PERMITTED; }
            if (Get8021XWiredProfileCache != null)
            {
                xprofile = Get8021XWiredProfileCache;
                return AmtCallStatus.SUCCESS;
            }
            AmtCallStatus r = (AmtCallStatus)service.Get8021XWiredProfile(handle, out xprofile);
            if (r == AmtCallStatus.SUCCESS) Get8021XWiredProfileCache = null;
            return r;
        }

        public AmtCallStatus Set8021XWiredProfile(XProfileType xprofile)
        {
            if (system.AmtRev < 3 && system.IsMobile == false) { xprofile = null; return AmtCallStatus.NOT_PERMITTED; }
            if (system.Info.GetCoreVersion().Equals("3.0.2")) return AmtCallStatus.NOT_PERMITTED;
            AmtCallStatus r = (AmtCallStatus)service.Set8021XWiredProfile(handle, xprofile);
            if (r == AmtCallStatus.SUCCESS) Get8021XWiredProfileCache = xprofile;
            return r;
        }

    }

    public class IPv6PortSettings
    {
        #region Public Definitions (structs, enums, etc.)

        public enum InterfaceIDType
        {
            /// <summary>
            /// The auto configured address will be based on random interface 
            /// ID (RFC: 3041).
            /// </summary>
            Random,
            /// <summary>
            /// The auto configured address will be based on an interface ID 
            /// generated by appending 0x8086 to the interface MAC address.
            /// </summary>
            IntelID,
            /// <summary>
            /// the auto configured address will be based on user set interface 
            /// ID.
            /// </summary>
            Manual,
        }

        public enum AddressType
        {
            /// <summary>
            /// Address generated from: fe80::/10 + Interface ID
            /// </summary>
            UniqueLocal = 0,
            /// <summary>
            /// Address generated from: [Router advertised prefix] + [Interface ID]
            /// </summary>
            RouterAdvertised = 1,
            /// <summary>
            /// Global IPv6 address obtained from DHCPv6 Server
            /// </summary>
            DHCPv6 = 2,
            /// <summary>
            /// IPv6 unicast address configured by the user
            /// </summary>
            Static = 3,
            /// <summary>
            /// Returned by FW for IPv6 addresses that are not owned by FW (such as IPv6 router address, configuration server address etc.)
            /// </summary>
            Unowned = 4,
        }

        public struct AddressInfo
        {
            /// <summary>
            /// IPv6 address for the Intel(R) AMT client.
            /// </summary>
            public IPAddress IPAddr;

            /// <summary>
            /// Possible 'type' values:
            ///    0 - Address generated from: fe80::/10 + Interface ID
            ///    1 - Address generated from: [Router advertised prefix] + [Interface ID]
            ///    2 - Global IPv6 address obtained from DHCPv6 Server
            ///    3 - IPv6 unicast address configured by the user
            ///    4 - Returned by FW for IPv6 addresses that are not owned by FW (such as IPv6 router address, configuration server address etc.)
            /// </summary>
            public AddressType Type;

            /// <summary>
            /// Possible 'state' values:
            ///    0 - DAD is still in process for this address
            ///    1 - Address is valid and may be used for new communication
            ///    2 - Address is deprecated and should not be used for new communication
            ///    3 - Covers both the preferred and deprecated states
            ///    4 - The valid lifetime of the address has expired
            ///    5 - An interface ID collision has been detected for this address when performing DAD
            ///    6 - Returned by FW for IPv6 addresses that are not owned by FW (such as IPv6 router address, configuration server address etc.)
            /// </summary>
            public ushort State;
        }

        #endregion

        #region Private Members

        /// <summary>
        /// "Intel(R) IPS IPv6 Settings 'd'", where 'd' is incremented for every existing instance.
        /// </summary>
        private string _instanceId;
        /// <summary>
        /// A collection of configured IPv6 addresses for the IPv6 port.
        /// </summary>
        private Collection<AddressInfo> _currentAddressInfo = new Collection<AddressInfo>();
        /// <summary>
        /// The type of the interface ID for the IPv6 port.
        /// </summary>
        private InterfaceIDType _interfaceId;
        /// <summary>
        /// Represents the manually set InterfaceID and not the currently 
        /// configured interface ID unless the InterfaceIDType is set to Manual. 
        /// The currently configured interface ID is apparent in the interface's 
        /// link local address FE80::<Interface ID>.
        /// </summary>
        private String _manualInterfaceId;
        /// <summary>
        /// Used when failing to perform DHCPv6 and auto-configuration.
        /// Unset this parameter by passing "::0".
        /// </summary>
        private IPAddress _ipv6Address;
        /// <summary>
        /// Used to describe the currently used primary DNS Address (IPv6).
        /// </summary>
        private IPAddress _currentPrimaryDNS;
        /// <summary>
        /// Used to describe a static Primary DNS Address (IPv6).
        /// Used only if the DNS IPv6 addresses were not configured by DHCPv6.
        /// Relevant only for wired interface.
        /// </summary>
        private IPAddress _primaryDNS;
        /// <summary>
        /// Used to describe the currently used secondary DNS Address (IPv6).
        /// </summary>
        private IPAddress _currentSecondaryDNS;
        /// <summary>
        /// Used to describe a static Secondary DNS Address (IPv6).
        /// Used only if the DNS IPv6 addresses were not configured by DHCPv6.
        /// Relevant only for wired interface.
        /// </summary>
        private IPAddress _secondaryDNS;
        /// <summary>
        /// Used to describe the currently used default router Address (IPv6).
        /// </summary>
        private IPAddress _currentDefaultRouter;
        /// <summary>
        /// Used to describe the Default router Address (IPv6).
        /// Used only if default router was not auto-configured via router advertisements.
        /// Relevant only for wired interface.
        /// </summary>
        private IPAddress _defaultRouter;

        #endregion

        #region Constructors

        public IPv6PortSettings()
        {
        }

        public IPv6PortSettings(IPS_IPv6PortSettings portSettings)
        {
            this.InstanceID = (portSettings.InstanceIDExist ? portSettings.InstanceID : String.Empty);

            if (portSettings.CurrentAddressInfoExist)
            {
                this.CurrentAddressInfo = new Collection<AddressInfo>();
                foreach (string addrInfo in portSettings.CurrentAddressInfo)
                {
                    try
                    {
                        Regex regex = new Regex(@"(?<IPv6Addr>.*),(?<AddrType>\d),(?<AddrState>\d)");
                        Match addrInfoMatch = regex.Match(addrInfo);
                        if (!addrInfoMatch.Success)
                            continue; // Nothing to do here. Move onto the next address info.

                        AddressInfo newAddrInfo = new AddressInfo();
                        newAddrInfo.IPAddr = IPAddress.Parse(addrInfoMatch.Groups["IPv6Addr"].Value);
                        newAddrInfo.Type = (AddressType) Enum.Parse(typeof(AddressType), addrInfoMatch.Groups["AddrType"].Value);
                        newAddrInfo.State = Convert.ToUInt16(addrInfoMatch.Groups["AddrState"].Value);
                        this.CurrentAddressInfo.Add(newAddrInfo);
                    }
                    catch (Exception e)
                    {
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                        continue;
                    }
                } // END foreach (string addrInfo in portSettings.CurrentAddressInfo)
            }

            this.InterfaceID = (portSettings.InterfaceIDTypeExist ? (InterfaceIDType) portSettings.InterfaceIDType : InterfaceIDType.Manual);
            this.ManualInterfaceID = (portSettings.ManualInterfaceIDExist ? portSettings.ManualInterfaceID : String.Empty);
            
            this.IPv6Address = IPAddress.IPv6None;
            if (portSettings.IPv6AddressExist)
                IPAddress.TryParse(portSettings.IPv6Address, out this._ipv6Address);

            this.CurrentPrimaryDNS = IPAddress.IPv6None;
            if (portSettings.CurrentPrimaryDNSExist)
                IPAddress.TryParse(portSettings.CurrentPrimaryDNS, out this._currentPrimaryDNS);

            this.PrimaryDNS = IPAddress.IPv6None;
            if (portSettings.PrimaryDNSExist)
                IPAddress.TryParse(portSettings.PrimaryDNS, out this._primaryDNS);

            this.CurrentSecondaryDNS = IPAddress.IPv6None;
            if (portSettings.CurrentSecondaryDNSExist)
                IPAddress.TryParse(portSettings.CurrentSecondaryDNS, out this._currentSecondaryDNS);

            this.SecondaryDNS = IPAddress.IPv6None;
            if (portSettings.SecondaryDNSExist)
                IPAddress.TryParse(portSettings.SecondaryDNS, out this._secondaryDNS);

            this.CurrentDefaultRouter = IPAddress.IPv6None;
            if (portSettings.CurrentDefaultRouterExist)
                IPAddress.TryParse(portSettings.CurrentDefaultRouter, out this._currentDefaultRouter);

            this.DefaultRouter = IPAddress.IPv6None;
            if (portSettings.DefaultRouterExist)
                IPAddress.TryParse(portSettings.DefaultRouter, out this._defaultRouter);
        }

        #endregion

        #region Public Properties

        /// <summary>
        /// Gets or sets the ID for this instance of the IPv6 port settings.
        /// </summary>
        public string InstanceID
        {
            get { return this._instanceId; }
            set { this._instanceId = value; }
        }

        /// <summary>
        /// Gets or sets the collection of configured IPv6 addresses for the IPv6 port.
        /// </summary>
        public Collection<AddressInfo> CurrentAddressInfo
        {
            get { return this._currentAddressInfo; }
            set { this._currentAddressInfo = value; }
        }

        /// <summary>
        /// Gets or sets the type of the interface ID for the IPv6 port.
        /// </summary>
        public InterfaceIDType InterfaceID
        {
            get { return this._interfaceId; }
            set { this._interfaceId = value; }
        }

        /// <summary>
        /// Gets or sets the manually set ID for the interface.
        /// </summary>
        public String ManualInterfaceID
        {
            get { return this._manualInterfaceId; }
            set { this._manualInterfaceId = value; }
        }

        /// <summary>
        /// Gets or sets the IPv6 address used when failing to perform DHCPv6 
        /// and auto-configuration.
        /// </summary>
        public IPAddress IPv6Address
        {
            get { return this._ipv6Address; }
            set { this._ipv6Address = value; }
        }

        /// <summary>
        /// Gets or sets the currently used primary DNS Address (IPv6).
        /// </summary>
        public IPAddress CurrentPrimaryDNS
        {
            get { return this._currentPrimaryDNS; }
            set { this._currentPrimaryDNS = value; }
        }

        /// <summary>
        /// Gets or sets the static Primary DNS Address (IPv6).
        /// </summary>
        /// <remarks>
        /// Used only if the DNS IPv6 addresses were not configured by DHCPv6.
        /// Relevant only for wired interface.
        /// </remarks>
        public IPAddress PrimaryDNS
        {
            get { return this._primaryDNS; }
            set { this._primaryDNS = value; }
        }

        /// <summary>
        /// Gets or sets the currently used secondary DNS Address (IPv6).
        /// </summary>
        public IPAddress CurrentSecondaryDNS
        {
            get { return this._currentSecondaryDNS; }
            set { this._currentSecondaryDNS = value; }
        }

        /// <summary>
        /// Gets or sets the static Secondary DNS Address (IPv6).
        /// </summary>
        /// <remarks>
        /// Used only if the DNS IPv6 addresses were not configured by DHCPv6.
        /// Relevant only for wired interface.
        /// </remarks>
        public IPAddress SecondaryDNS
        {
            get { return this._secondaryDNS; }
            set { this._secondaryDNS = value; }
        }

        /// <summary>
        /// Gets or sets the currently used default router Address (IPv6).
        /// </summary>
        public IPAddress CurrentDefaultRouter
        {
            get { return this._currentDefaultRouter; }
            set { this._currentDefaultRouter = value; }
        }

        /// <summary>
        /// Used to describe the Default router Address (IPv6).
        /// </summary>
        /// <remarks>
        /// Used only if default router was not auto-configured via router advertisements.
        /// Relevant only for wired interface.
        /// </remarks>
        public IPAddress DefaultRouter
        {
            get { return this._defaultRouter; }
            set { this._defaultRouter = value; }
        }

        #endregion

        #region Public Methods

        #endregion
    }

}
