// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Diagnostics;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;

namespace ManageabilityStack
{
    public class AmtNetworkTime
    {
        private NetworkTimeService service;
        private AmtSystem system;
        public bool IsCached() { return true; }

        public AmtNetworkTime(NetworkTimeService service, AmtSystem system)
        {
            if (service != null) this.service = service;
            this.system = system;
        }

        public void Dispose()
        {
            service = null;
        }

        public void ClearCache()
        {
        }

        public void FetchCache()
        {
        }

        public AmtCallStatus GetLowAccuracyTimeSynch(out System.DateTime time)
        {
            time = System.DateTime.MinValue;
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_TimeSynchronizationService ts = new AMT_TimeSynchronizationService(system.WsMan);
                    uint Ta0;
                    AmtCallStatus r = (AmtCallStatus)ts.GetLowAccuracyTimeSynch(out Ta0);
                    if (r != AmtCallStatus.SUCCESS)
                        return r;
                    Debug.Print("GetLowAccuracyTimeSynch() returned: {0}", Ta0.ToString());
                    time = Utils.ClockToDate(Ta0);
                    Debug.Print("Converted Time: {0}", time.ToString());
                    return r;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkTime.GetLowAccuracyTimeSynch WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkTime.GetLowAccuracyTimeSynch CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkTime.GetLowAccuracyTimeSynch",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                uint ta0;
                time = System.DateTime.MinValue;
                AmtCallStatus r = (AmtCallStatus)service.GetLowAccuracyTimeSynch(out ta0);
                if (r != AmtCallStatus.SUCCESS)
                    return r;
                Debug.Print("GetLowAccuracyTimeSynch() returned: {0}", ta0.ToString());
                time = Utils.ClockToDate(ta0);
                Debug.Print("Converted Time: {0}", time.ToString());
                return r;
            }
        }

        public AmtCallStatus SetHighAccuracyTimeSynch(System.DateTime newtime)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_TimeSynchronizationService ts = new AMT_TimeSynchronizationService(system.WsMan);

                    // Get current time
                    uint Ta0;
                    AmtCallStatus r = (AmtCallStatus)ts.GetLowAccuracyTimeSynch(out Ta0);
                    if (r != AmtCallStatus.SUCCESS) return r;

                    // Set new time
                    r = (AmtCallStatus)ts.SetHighAccuracyTimeSynch(Ta0, Utils.DateToClock(newtime), Utils.DateToClock(newtime));
                    return r;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkTime.SetHighAccuracyTimeSynch WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkTime.SetHighAccuracyTimeSynch CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtNetworkTime.SetHighAccuracyTimeSynch",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                uint ta0;
                AmtCallStatus r = (AmtCallStatus)service.GetLowAccuracyTimeSynch(out ta0);
                uint tm1 = Utils.DateToClock(newtime);
                return (AmtCallStatus)service.SetHighAccuracyTimeSynch(ta0, tm1, tm1);
            }
        }

    }
}
