// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Diagnostics;
using System.Net;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;

namespace ManageabilityStack
{
    public class AmtRedirect
    {
        private RedirectionService service;
        private AmtSystem system;
        private bool cached = false;
        public bool IsCached() { return cached; }

        public AmtRedirect(RedirectionService service, AmtSystem system)
        {
            this.service = service;
            this.system = system;
        }

        public void Dispose()
        {
            service = null;
        }

        public void ClearCache()
        {
            cached = false;
            IderSessionLogCache = null;
            RedirectionListenerStateCache = false;
        }

        public void FetchCache()
        {
            try
            {
                GetIderSessionLog();
                GetRedirectionListenerState();
                cached = true;
            }
            catch (Exception e)
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Failure to retrieve the Redirection data.",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
            }
        }

        AmtRedirectEvent[] IderSessionLogCache = null;
        public AmtRedirectEvent[] GetIderSessionLog() { return GetIderSessionLog(false);  }
        public AmtRedirectEvent[] GetIderSessionLog(bool nocache)
        {
            if (system.AmtRev == 1) return null;
            if (IderSessionLogCache != null && nocache == false) return IderSessionLogCache;

            if (system.WsMan != null)
            {
                AMT_RedirectionService rs = null;
                try
                {
                    rs = new AMT_RedirectionService(system.WsMan);
                    rs.Get();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRedirect.GetIderSessionLog WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRedirect.GetIderSessionLog CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRedirect.GetIderSessionLog",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return IderSessionLogCache;
                }
                if (rs == null || !rs.AccessLogExist)
                {
                    IderSessionLogCache = new AmtRedirectEvent[0];
                    return IderSessionLogCache;
                }

                IderSessionLogCache = new AmtRedirectEvent[rs.AccessLog.Length];
                for (int i = 0; i < rs.AccessLog.Length; i++)
                {
                    try
                    {
                        IderSessionLogCache[i] = new AmtRedirectEvent(rs.AccessLog[i]);
                    }
                    catch (Exception)
                    {
                        IderSessionLogCache[i] = null;
                    }
                }
                return IderSessionLogCache;
            }
            else
            {
                IderSessionLogEntryType[] logData;
                AmtCallStatus r = (AmtCallStatus)service.GetIderSessionLog(out logData);

                if (logData == null)
                {
                    IderSessionLogCache = new AmtRedirectEvent[0];
                    return IderSessionLogCache;
                }

                IderSessionLogCache = new AmtRedirectEvent[logData.Length];
                for (int i = 0; i < logData.Length; i++)
                {
                    IderSessionLogCache[i] = new AmtRedirectEvent(logData[i]);
                }
                return IderSessionLogCache;
            }
        }

        private bool RedirectionListenerStateCache = false;
        private bool RedirectionListenerStateCacheValue;
        public bool GetRedirectionListenerState()
        {
            if (system.AmtRev <= 1) return true;
            if (RedirectionListenerStateCache) return RedirectionListenerStateCacheValue;

            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_RedirectionService rs = new AMT_RedirectionService(system.WsMan);
                    rs.Get();
                    RedirectionListenerStateCacheValue = rs.ListenerEnabled;
                    RedirectionListenerStateCache = true;
                    return RedirectionListenerStateCacheValue;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRedirect.GetRedirectionListenerState WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRedirect.GetRedirectionListenerState CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRedirect.GetRedirectionListenerState",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    RedirectionListenerStateCacheValue = false;
                    RedirectionListenerStateCache = false;
                    return RedirectionListenerStateCacheValue;
                }
            }
            else
            {
                AmtCallStatus r;
                try
                {
                    r = (AmtCallStatus)service.GetRedirectionListenerState(out RedirectionListenerStateCacheValue);
                }
                catch (WebException)
                {
                    system.Disconnect();
                }
                RedirectionListenerStateCache = true;
                return RedirectionListenerStateCacheValue;
            }
        }

        public AmtCallStatus SetRedirectionListenerState(bool enabled)
        {
            if (system.AmtRev <= 1) return AmtCallStatus.SUCCESS;
            RedirectionListenerStateCache = false;

            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_RedirectionService rs = new AMT_RedirectionService(system.WsMan);
                    rs.Get();
                    rs.ListenerEnabled = enabled;
                    rs.Put();
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRedirect.SetRedirectionListenerState WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRedirect.SetRedirectionListenerState CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRedirect.SetRedirectionListenerState",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    throw;
                }
            }
            else
            {
                return (AmtCallStatus)service.SetRedirectionListenerState(enabled);
            }
        }
    }

    public class AmtRedirectEvent
    {
        private static System.Globalization.CultureInfo EnglishCultureInfo = new System.Globalization.CultureInfo("en-US", true);
        public IPEndPoint ConsoleAddress;
        public System.DateTime TimeStamp;

        public AmtRedirectEvent(string logentry)
        {
            // 01/26/2008, 23:41:56, 192.168.5.90:52594
            int i = logentry.IndexOf(", ", 12);
            if (i >= 0)
            {
                string date = logentry.Substring(0, i);
                TimeStamp = System.DateTime.ParseExact(date, "MM/dd/yyyy, HH:mm:ss", null);
                logentry = logentry.Substring(i+2);
                i = logentry.LastIndexOf(":");
                if (i >= 0)
                {
                    IPAddress a = IPAddress.Parse(logentry.Substring(0, i));
                    int port = int.Parse(logentry.Substring(i + 1));
                    ConsoleAddress = new IPEndPoint(a, port);
                }
            }
        }

        public AmtRedirectEvent(IderSessionLogEntryType logentry)
        {
            ConsoleAddress = new IPEndPoint((long)logentry.ConsoleAddress, logentry.Port);
            TimeStamp = Utils.ClockToDate(logentry.TimeStamp);
        }

        public AmtRedirectEvent(GenInfoIderSessionLogEntryType logentry)
        {
            ConsoleAddress = new IPEndPoint((long)logentry.ConsoleAddress, logentry.Port);
            TimeStamp = Utils.ClockToDate(logentry.TimeStamp);
        }
        
    }
}
