// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Net;
using Intel.Manageability.Cim; 
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.Cim.Untyped;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;

namespace ManageabilityStack
{
    public class AmtRemoteAccessAdmin
    {
        private AmtSystem system;
        private RemoteAccessAdminService service;
        private bool cached = false;
        public bool IsCached() { return cached; }

        public AmtRemoteAccessAdmin(AmtSystem system, RemoteAccessAdminService service)
        {
            this.system = system;
            this.service = service;
        }

        public void Dispose()
        {
            if (service != null) service.Dispose();
            service = null;
        }

        public void ClearCache()
        {
            cached = false;
            RemoteServersCache = null;
            RemoteAccessPoliciesCache = null;
            UserInitiatedInterfaceConfigurationCache = new bool[2];
            UserInitiatedInterfaceConfigurationCacheSet = new bool[2];
        }

        public void FetchCache()
        {
            try
            {
                AmtRemoteAccessServer[] RemoteServers;
                GetRemoteServers(out RemoteServers);
                AmtRemoteAccessPolicy[] RemoteAccessPolicies;
                GetRemoteAccessPolicies(out RemoteAccessPolicies);
                bool e;
                GetUserInitiatedInterfaceConfiguration(0, out e);
                GetUserInitiatedInterfaceConfiguration(1, out e);
                cached = true;
            }
            catch (Exception e)
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Failure to retrieve the Remote Access Administration data.",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
            }
        }

        [Serializable]
        public class AmtRemoteAccessServer
        {
            public uint Handle;
            public string ServerName;
            public IPAddress ServerAddress;
            public string ServerCommonName;
            public ushort Port;
            public uint MutualAuthCertHandle;
            public AmtSecurityAdmin.AmtStoredCert MutualAuthCert;
            public string Username;
            public string Password;
            public AMT_RemoteAccessCredentialContext wsmanContext;

            public AmtRemoteAccessServer()
            {
            }
    
            public AmtRemoteAccessServer(uint handle, MpsAddressType address, ushort port, AuthType auth)
            {
                this.Handle = handle;

                // Handle address
                if (address.Item.GetType() == typeof(string)) ServerName = (string)address.Item;
                if (address.Item.GetType() == typeof(IpAddressType))
                {
                    IpAddressType sa = (IpAddressType)address.Item;
                    ServerAddress = IPAddress.Parse((string)sa.Ip.Item);
                    ServerCommonName = sa.CN;
                }

                // Handle authentication
                if (auth.Item.GetType() == typeof(MutualAuthType))
                {
                    MutualAuthType MutualAuth = (MutualAuthType)(auth.Item);
                    MutualAuthCertHandle = MutualAuth.TlsCertHandle;
                }
                else if (auth.Item.GetType() == typeof(UserNamePwdType))
                {
                    UserNamePwdType UserNamePwd = (UserNamePwdType)(auth.Item);
                    Username = UserNamePwd.UserName;
                    Password = UserNamePwd.Password;
                }

                this.Port = port;
            }

            public AmtRemoteAccessServer(uint handle, string serverName, ushort port, string username, string password)
            {
                this.Handle = handle;
                this.ServerName = serverName;
                this.Port = port;
                this.Username = username;
                this.Password = password;
            }

            public AmtRemoteAccessServer(uint handle, IPAddress serverAddress, string commonName, ushort port, string username, string password)
            {
                this.Handle = handle;
                this.ServerAddress = serverAddress;
                this.ServerCommonName = commonName;
                this.Port = port;
                this.Username = username;
                this.Password = password;
            }

            public AmtRemoteAccessServer(uint handle, string serverName, ushort port, uint mutualAuthCertHandle)
            {
                this.Handle = handle;
                this.ServerName = serverName;
                this.Port = port;
                this.MutualAuthCertHandle = mutualAuthCertHandle;
            }

            public AmtRemoteAccessServer(uint handle, IPAddress serverAddress, string commonName, ushort port, uint mutualAuthCertHandle)
            {
                this.Handle = handle;
                this.ServerAddress = serverAddress;
                this.ServerCommonName = commonName;
                this.Port = port;
                this.MutualAuthCertHandle = mutualAuthCertHandle;
            }

            public MpsAddressType Address
            {
                get
                {
                    if (ServerAddress != null)
                    {
                        IpAddressType ad = new IpAddressType();
                        ad.Ip = new NodeAddressType();
                        ad.Ip.Item = ServerAddress.ToString();
                        ad.Ip.ItemElementName = EventItemChoiceType.IPv4Address;
                        ad.CN = ServerCommonName;
                        MpsAddressType a = new MpsAddressType();
                        a.Item = ad;
                        return a;
                    }
                    else
                    {
                        MpsAddressType a = new MpsAddressType();
                        a.Item = ServerName;
                        return a;
                    }
                }
            }

            public AuthType Authentication
            {
                get
                {
                    if (Username != null && Password != null)
                    {
                        UserNamePwdType p = new UserNamePwdType();
                        p.UserName = Username;
                        p.Password = Password;
                        AuthType a = new AuthType();
                        a.Item = p;
                        return a;
                    }
                    else
                    {
                        MutualAuthType m = new MutualAuthType();
                        m.TlsCertHandle = MutualAuthCertHandle;
                        AuthType a = new AuthType();
                        a.Item = m;
                        return a;
                    }
                }
            }
        }

        private AmtRemoteAccessServer[] RemoteServersCache = null;
        public AmtCallStatus GetRemoteServers(out AmtRemoteAccessServer[] remoteServers)
        {
            if (RemoteServersCache != null)
            {
                remoteServers = RemoteServersCache;
                return AmtCallStatus.SUCCESS;
            }
            if (system.LimitedFeatures == true)
            {
                RemoteServersCache = remoteServers = new AmtRemoteAccessServer[0];
                return AmtCallStatus.SUCCESS;
            }
            remoteServers = null;

            if (system.WsMan != null)
            {
                Collection<AMT_RemoteAccessCredentialContext> ws_credential_map = null;
                try
                {
                    ws_credential_map = AMT_RemoteAccessCredentialContext.Enumerate(system.WsMan);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetRemoteServers WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetRemoteServers CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetRemoteServers",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }

                ArrayList servers = new ArrayList();
                if (ws_credential_map != null && ws_credential_map.Count > 0)
                {
                    foreach (AMT_RemoteAccessCredentialContext cm in ws_credential_map)
                    {
                        AmtRemoteAccessServer remAccessServer = null;
                        AMT_ManagementPresenceRemoteSAP server = null;
                        AMT_MPSUsernamePassword user = null;
                        AMT_PublicKeyCertificate cert = null;

                        try
                        {
                            server = new AMT_ManagementPresenceRemoteSAP(system.WsMan);
                            server.Get(cm.ElementProvidingContext);

                            if (cm.ElementInContext.referenceParameters.ResourceURI.Contains("AMT_MPSUsernamePassword"))
                            {
                                user = new AMT_MPSUsernamePassword(system.WsMan);
                                user.Get(cm.ElementInContext);
                            }
                            else if (cm.ElementInContext.referenceParameters.ResourceURI.Contains("AMT_PublicKeyCertificate")) 
                            {
                                cert = new AMT_PublicKeyCertificate(system.WsMan);
                                cert.Get(cm.ElementInContext);
                            }
                        }
                        catch (Exception e)
                        {
                            WSManException wsManException = e as WSManException;
                            if (wsManException != null)
                            {
                                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetRemoteServers WSMAN error",
                                String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                                wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                            }
                            CimException cimException = e as CimException;
                            if (cimException != null)
                            {
                                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetRemoteServers CIM error",
                                String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                                cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                            }

                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetRemoteServers",
                                String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                            Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                            Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                            continue;
                        }

                        if (server.CNExist == false)
                        {
                            // Just the name specified
                            if (user == null)
                                remAccessServer = new AmtRemoteAccessServer(GetHandleFromString(server.Name),
                                server.AccessInfo, server.Port, GetHandleFromString(cert.InstanceID));
                            else
                                remAccessServer = new AmtRemoteAccessServer(GetHandleFromString(server.Name),
                                    server.AccessInfo, server.Port, (user.RemoteIDExist ? user.RemoteID : null),
                                    (user.SecretExist ? user.Secret : String.Empty));
                        }
                        else
                        {
                            // IP address and CN specified
                            if (user == null)
                                remAccessServer = new AmtRemoteAccessServer(GetHandleFromString(server.Name),
                                    IPAddress.Parse(server.AccessInfo), server.CN, server.Port, GetHandleFromString(cert.InstanceID));
                            else
                                remAccessServer = new AmtRemoteAccessServer(GetHandleFromString(server.Name),
                                    IPAddress.Parse(server.AccessInfo), server.CN, server.Port,
                                    (user.RemoteIDExist ? user.RemoteID : null), (user.SecretExist ? user.Secret : String.Empty));
                        }

                        if (remAccessServer != null)
                        {
                            remAccessServer.wsmanContext = cm;
                            servers.Add(remAccessServer);
                        }
                    }
                }

                RemoteServersCache = (AmtRemoteAccessServer[])servers.ToArray(typeof(AmtRemoteAccessServer));
                remoteServers = RemoteServersCache;
                return AmtCallStatus.SUCCESS;
            }
            else
            {
                uint[] handles;
                AmtCallStatus r = (AmtCallStatus)service.EnumerateMpServers(out handles);
                if (r != AmtCallStatus.SUCCESS) return r;

                ArrayList servers = new ArrayList();
                if (handles != null)
                {
                    foreach (uint handle in handles)
                    {
                        MpsAddressType address;
                        ushort port;
                        AuthType auth;

                        // Get the remote server
                        r = (AmtCallStatus)service.GetMpServer(handle, out address, out port, out auth);
                        if (r == AmtCallStatus.SUCCESS) servers.Add(new AmtRemoteAccessServer(handle, address, port, auth));
                    }
                }

                RemoteServersCache = (AmtRemoteAccessServer[])servers.ToArray(typeof(AmtRemoteAccessServer));
                remoteServers = RemoteServersCache;
                return AmtCallStatus.SUCCESS;
            }
        }

        public uint GetHandleFromString(string str)
        {
            int i = str.IndexOf(':');
            while (i >= 0)
            {
                str = str.Substring(i+1);
                i = str.IndexOf(':');
            }
            i = str.IndexOf(' ');
            while (i >= 0)
            {
                str = str.Substring(i+1);
                i = str.IndexOf(' ');
            }
            return uint.Parse(str);
        }

        public AmtCallStatus RemoveRemoteServer(AmtRemoteAccessServer server)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    system.WsMan.Delete(server.wsmanContext.ElementProvidingContext);
                    RemoteServersCache = null;
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.RemoveRemoteServer WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.RemoveRemoteServer CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.RemoveRemoteServer",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r = (AmtCallStatus)service.RemoveMpServer(server.Handle);
                if (r == AmtCallStatus.SUCCESS) RemoteServersCache = null;
                return r;
            }
        }

        public AmtCallStatus AddRemoteServer(AmtRemoteAccessServer server)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_RemoteAccessService remoteAccessService = new AMT_RemoteAccessService(system.WsMan);
                    string accessInfo = null;
                    ushort infoFormat;
                    uint authMethod;
                    CimReference cert = null;
                    string username = null;
                    string password = null;
                    string CN = null;
                    CimReference mpServer = null;

                    if (server.ServerCommonName != null && server.ServerCommonName.Length > 0)
                        CN = server.ServerCommonName;

                    if (server.Username != null && server.Username.Length > 0)
                    {
                        // Username & password
                        authMethod = 2;
                        username = server.Username;
                        password = server.Password;
                    }
                    else
                    {
                        // Certificate
                        authMethod = 1;
                        cert = server.MutualAuthCert.Endpoint;
                    }

                    if (server.ServerAddress != null)
                    {
                        infoFormat = 3; // IPv4
                        if (server.ServerAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6) infoFormat = 4; // IPv6
                        accessInfo = server.ServerAddress.ToString();
                    }
                    else
                    {
                        infoFormat = 201; // FQDN
                        accessInfo = server.ServerName;
                    }

                    AmtCallStatus r = (AmtCallStatus)remoteAccessService.AddMpServer(accessInfo, infoFormat,
                        server.Port, authMethod, cert, username, password, CN, out mpServer);

                    RemoteServersCache = null;
                    return r;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.AddRemoteServer WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.AddRemoteServer CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.AddRemoteServer",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r = (AmtCallStatus)service.AddMpServer(server.Address, server.Port, server.Authentication, out server.Handle);
                RemoteServersCache = null;
                return r;
            }
        }

        public AmtCallStatus UpdateRemoteServer(AmtRemoteAccessServer server)
        {
            if (system.WsMan != null)
            {
                Debug.Assert(server.wsmanContext != null);

                // Use WSMAN
                try
                {
                    //
                    // Configure the connection properties.
                    //
                    AMT_ManagementPresenceRemoteSAP mps = new AMT_ManagementPresenceRemoteSAP(system.WsMan);
                    mps.Get(server.wsmanContext.ElementProvidingContext);
                    if (!String.IsNullOrEmpty(server.ServerName))
                    {
                        // Use the FQDN of the MPS.
                        mps.InfoFormat = 201; // FQDN
                        mps.AccessInfo = server.ServerName;
                    }
                    else
                    {
                        Debug.Assert(!String.IsNullOrEmpty(server.ServerCommonName));

                        // Use an IP address and CN of the MPS.
                        mps.InfoFormat = 3; // IPv4
                        if (server.ServerAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6) mps.InfoFormat = 4; // IPv6
                        mps.AccessInfo = server.ServerAddress.ToString();
                        mps.CN = server.ServerCommonName;
                    }
                    mps.Port = server.Port;
                    mps.Put();

                    //
                    // Configure the authentication properties.
                    //
                    if (!String.IsNullOrEmpty(server.Username) &&
                        (server.wsmanContext.ElementInContext.GetReferenceType() == typeof(AMT_MPSUsernamePassword)))
                    {
                        // Update the username/password used for MPS authentication.
                        AMT_MPSUsernamePassword user = new AMT_MPSUsernamePassword(system.WsMan);
                        user.Get(server.wsmanContext.ElementInContext);
                        user.RemoteID = server.Username;
                        user.Secret = server.Password;
                        user.Put();
                    }
                    else if (String.IsNullOrEmpty(server.Username) &&
                             (server.wsmanContext.ElementInContext.GetReferenceType() == typeof(AMT_PublicKeyCertificate)))
                    {
                        // Update the certificate used for MPS authentication.
                        AMT_PublicKeyCertificate cert = new AMT_PublicKeyCertificate(system.WsMan);

                        // NOTE: To retrieve the previous authentication certificate:
                        //cert.Get(server.wsmanContext.ElementInContext);

                        cert.Get(server.MutualAuthCert.Endpoint);
                        server.wsmanContext.ElementInContext = cert.Reference;
                        server.wsmanContext.Put();
                    }
                    else
                    {
                        // Currently unable to switch authentication modes for 
                        // an existing MPS server. Report the error.
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Warning, "Unable to change MPS authentication modes.");
                        return AmtCallStatus.UNSUPPORTED;
                    }

                    RemoteServersCache = null;
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.UpdateRemoteServer WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.UpdateRemoteServer CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.UpdateRemoteServer",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use EOI/SOAP
                AmtCallStatus r = (AmtCallStatus)service.UpdateMpServer(server.Handle, server.Address, server.Port, server.Authentication);
                if (r == AmtCallStatus.SUCCESS) RemoteServersCache = null;
                return r;
            }
        }

        public enum AmtRemoteAccessTrigger
        {
            USERINITIATED = 0,
            ALERT = 1,
            PERIODIC = 2
        }

        public class AmtRemoteAccessPolicy
        {
            public uint Handle;
            public uint TunnelLifeTime;
            public AmtRemoteAccessTrigger Trigger;
            public uint[] Servers;
            public uint PeriodicTimer;
            public AMT_RemoteAccessPolicyRule WsmanRef;

            public AmtRemoteAccessPolicy(uint handle, uint tunnelLifeTime, AmtRemoteAccessTrigger trigger, uint[] servers, byte[] data)
            {
                this.Handle = handle;
                this.TunnelLifeTime = tunnelLifeTime;
                this.Trigger = trigger;
                this.Servers = servers;
                if (Trigger == AmtRemoteAccessTrigger.PERIODIC && data != null)
                {
                    if (data.Length == 4)
                    {
                        PeriodicTimer = (uint)IPAddress.NetworkToHostOrder(BitConverter.ToInt32(data, 0));
                    }
                    else if (data.Length == 8)
                    {
                        PeriodicTimer = (uint)IPAddress.NetworkToHostOrder(BitConverter.ToInt64(data, 0));
                    }
                }
            }

            public byte[] GetData()
            {
                if (Trigger == AmtRemoteAccessTrigger.PERIODIC) return BitConverter.GetBytes(IPAddress.HostToNetworkOrder(PeriodicTimer));
                return new byte[0];
            }
        }

        private AmtRemoteAccessPolicy[] RemoteAccessPoliciesCache = null;
        public AmtCallStatus GetRemoteAccessPolicies(out AmtRemoteAccessPolicy[] remotePolicies)
        {
            if (RemoteAccessPoliciesCache != null)
            {
                remotePolicies = RemoteAccessPoliciesCache;
                return AmtCallStatus.SUCCESS;
            }
            if (system.LimitedFeatures == true)
            {
                RemoteAccessPoliciesCache = remotePolicies = new AmtRemoteAccessPolicy[0];
                return AmtCallStatus.SUCCESS;
            }
            remotePolicies = null;

            ArrayList policies = new ArrayList();
            if (system.WsMan != null)
            {
                Collection<AMT_RemoteAccessPolicyAppliesToMPS> ws_policyrelations = null;
                // WSMAN
                try
                {
                    ws_policyrelations = AMT_RemoteAccessPolicyAppliesToMPS.Enumerate(system.WsMan);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetRemoteAccessPolicies WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetRemoteAccessPolicies CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetRemoteAccessPolicies",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
                if (ws_policyrelations != null && ws_policyrelations.Count > 0)
                {
                    foreach (AMT_RemoteAccessPolicyAppliesToMPS ws_policyrelation in ws_policyrelations)
                    {
                        AMT_RemoteAccessPolicyRule policy = null;
                        AMT_ManagementPresenceRemoteSAP server = null;
                        try
                        {
                            policy = new AMT_RemoteAccessPolicyRule(system.WsMan);
                            policy.Get(ws_policyrelation.PolicySet);

                            server = new AMT_ManagementPresenceRemoteSAP(system.WsMan);
                            server.Get(ws_policyrelation.ManagedElement);

                            int j = -1;
                            for (int i = 0; i < policies.Count; i++)
                            {
                                if (((AmtRemoteAccessPolicy)policies[i]).Handle == policy.Trigger) j = i;
                            }

                            if (j >= 0)
                            {
                                // Add the server to this existing policy object
                                AmtRemoteAccessPolicy p = (AmtRemoteAccessPolicy)policies[j];
                                uint[] x = new uint[p.Servers.Length + 1];
                                p.Servers.CopyTo(x, 0);
                                x[x.Length - 1] = GetHandleFromString(server.Name);
                                p.Servers = x;
                            }
                            else
                            {
                                // Create a new policy object
                                AmtRemoteAccessPolicy p = new AmtRemoteAccessPolicy(policy.Trigger, policy.TunnelLifeTime,
                                    (AmtRemoteAccessTrigger)policy.Trigger, new uint[] { GetHandleFromString(server.Name) },
                                    (policy.ExtendedDataExist ? policy.ExtendedData : new byte[0]));
                                p.WsmanRef = policy;
                                policies.Add(p);
                            }
                        }
                        catch (Exception e)
                        {
                            WSManException wsManException = e as WSManException;
                            if (wsManException != null)
                            {
                                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetRemoteAccessPolicies WSMAN error",
                                String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                                wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                            }
                            CimException cimException = e as CimException;
                            if (cimException != null)
                            {
                                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetRemoteAccessPolicies CIM error",
                                String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                                cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                            }

                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetRemoteAccessPolicies",
                                String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                            Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                            Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                            continue;
                        }
                    }
                }
            }
            else
            {
                // SOAP
                uint[] handles;
                AmtCallStatus r;
                r = (AmtCallStatus)service.EnumerateRemoteAccessPolicies(out handles);
                if (r != AmtCallStatus.SUCCESS) return r;

                foreach (uint handle in handles)
                {
                    uint tunnelLifeTime;
                    uint triger;
                    uint[] servers;
                    byte[] data;

                    // Get the remote policy
                    r = (AmtCallStatus)service.GetRemoteAccessPolicy(handle, out tunnelLifeTime, out triger, out servers, out data);
                    if (r == AmtCallStatus.SUCCESS) policies.Add(new AmtRemoteAccessPolicy(handle, tunnelLifeTime, (AmtRemoteAccessTrigger)triger, servers, data));
                }
            }

            RemoteAccessPoliciesCache = (AmtRemoteAccessPolicy[])policies.ToArray(typeof(AmtRemoteAccessPolicy));
            remotePolicies = RemoteAccessPoliciesCache;
            return AmtCallStatus.SUCCESS;
        }

        public AmtCallStatus RemoveRemoteAccessPolicy(AmtRemoteAccessPolicy policy)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    system.WsMan.Delete(policy.WsmanRef.Reference);
                    RemoteAccessPoliciesCache = null;
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.RemoveRemoteAccessPolicy WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.RemoveRemoteAccessPolicy CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.RemoveRemoteAccessPolicy",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r = (AmtCallStatus)service.RemoveRemoteAccessPolicy(policy.Handle);
                if (r == AmtCallStatus.SUCCESS) RemoteAccessPoliciesCache = null;
                return r;
            }
        }

        public AmtCallStatus AddRemoteAccessPolicy(AmtRemoteAccessPolicy policy)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AmtRemoteAccessServer[] remoteServers;
                    AmtCallStatus r = GetRemoteServers(out remoteServers);
                    if (r != AmtCallStatus.SUCCESS) return r;

                    ArrayList l = new ArrayList();
                    foreach (uint h in policy.Servers)
                    {
                        foreach (AmtRemoteAccessServer s in remoteServers) if (s.Handle == h) l.Add(s.wsmanContext.ElementProvidingContext);
                    }

                    AMT_RemoteAccessService remoteAccessService = new AMT_RemoteAccessService(system.WsMan);
                    CimReference policyRule;
                    r = (AmtCallStatus) remoteAccessService.AddRemoteAccessPolicyRule((ushort)policy.Trigger,
                        policy.TunnelLifeTime, policy.GetData(), (CimReference[])l.ToArray(typeof(CimReference)), out policyRule);
                    if (r != AmtCallStatus.SUCCESS) return r;
                    RemoteAccessPoliciesCache = null;
                    return r;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.AddRemoteAccessPolicy WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.AddRemoteAccessPolicy CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.AddRemoteAccessPolicy",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r = (AmtCallStatus)service.AddRemoteAccessPolicy((uint)policy.Trigger, policy.TunnelLifeTime, policy.Servers, policy.GetData(), out policy.Handle);
                if (r == AmtCallStatus.SUCCESS) RemoteAccessPoliciesCache = null;
                return r;
            }
        }

        public AmtCallStatus UpdateRemoteAccessPolicy(AmtRemoteAccessPolicy policy)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                AmtCallStatus r;
                r = RemoveRemoteAccessPolicy(policy);
                if (r != AmtCallStatus.SUCCESS) return r;
                r = AddRemoteAccessPolicy(policy);
                return r;
            }
            else
            {
                // SOAP
                AmtCallStatus r = (AmtCallStatus)service.UpdateRemoteAccessPolicy(policy.Handle, policy.TunnelLifeTime, policy.Servers, policy.GetData());
                // if (r == AmtCallStatus.SUCCESS) RemoteAccessPoliciesCache = null;
                RemoteAccessPoliciesCache = null;
                return r;
            }
        }

        public AmtCallStatus CloseRemoteAccessConnection()
        {
            AmtCallStatus r;
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_RemoteAccessService remoteAccessService = new AMT_RemoteAccessService(system.WsMan);
                    r = (AmtCallStatus)remoteAccessService.CloseRemoteAccessConnection();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.CloseRemoteAccessConnection WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.CloseRemoteAccessConnection CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.CloseRemoteAccessConnection",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // EOI
                r = (AmtCallStatus)service.CloseRemoteAccessConnection();
            }
            return r;
        }

        public AmtCallStatus EnableUserInitiatedInterface(uint source, bool enabled)
        {
            if (source >= 2 || system.LimitedFeatures == true) return AmtCallStatus.NOT_PERMITTED;

            if (system.WsMan != null)
            {
                // WSMAN
                bool ebios;
                bool eos;
                if (source == 0) ebios = enabled; else GetUserInitiatedInterfaceConfiguration(0, out ebios);
                if (source == 1) eos = enabled; else GetUserInitiatedInterfaceConfiguration(1, out eos);
                ushort val = 32768;
                if (ebios == true && eos == false) val = 32769;
                if (ebios == false && eos == true) val = 32770;
                if (ebios == true && eos == true) val = 32771;

                try
                {
                    CimReference job = null;
                    AMT_UserInitiatedConnectionService userInitConService = new AMT_UserInitiatedConnectionService(system.WsMan);
                    AmtCallStatus r = (AmtCallStatus)userInitConService.RequestStateChange(val, null, out job);
                    if (r != AmtCallStatus.SUCCESS) return r;
                    UserInitiatedInterfaceConfigurationCacheSet = new bool[] { false, true };
                    return r;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.EnableUserInitiatedInterface WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.EnableUserInitiatedInterface CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.EnableUserInitiatedInterface",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                AmtCallStatus r = (AmtCallStatus)service.EnableUserInitiatedInterface(source, enabled);
                if (r == AmtCallStatus.SUCCESS)
                {
                    UserInitiatedInterfaceConfigurationCache[source] = enabled;
                    UserInitiatedInterfaceConfigurationCacheSet[source] = true;
                }
                return r;
            }
        }

        private bool[] UserInitiatedInterfaceConfigurationCache = new bool[2];
        private bool[] UserInitiatedInterfaceConfigurationCacheSet = new bool[2];
        public AmtCallStatus GetUserInitiatedInterfaceConfiguration(uint source, out bool enabled)
        {
            if (UserInitiatedInterfaceConfigurationCacheSet[source] == true)
            {
                enabled = UserInitiatedInterfaceConfigurationCache[source];
                return AmtCallStatus.SUCCESS;
            }

            if (system.LimitedFeatures == true)
            {
                enabled = true;
                return AmtCallStatus.SUCCESS;
            }

            enabled = false;
            if (system.WsMan != null)
            {
                try
                {
                    Collection<AMT_UserInitiatedConnectionService> ws_userinitiated = AMT_UserInitiatedConnectionService.Enumerate(system.WsMan);
                    if (ws_userinitiated != null && ws_userinitiated.Count > 0)
                    {
                        if (ws_userinitiated[0].EnabledState == 32768) // Disabled
                        {
                            UserInitiatedInterfaceConfigurationCache = new bool[] { false, false };
                            UserInitiatedInterfaceConfigurationCacheSet = new bool[] { true, true };
                        }
                        else if (ws_userinitiated[0].EnabledState == 32769) // BIOS only
                        {
                            UserInitiatedInterfaceConfigurationCache = new bool[] { true, false };
                            UserInitiatedInterfaceConfigurationCacheSet = new bool[] { true, true };
                        }
                        else if (ws_userinitiated[0].EnabledState == 32770) // OS only
                        {
                            UserInitiatedInterfaceConfigurationCache = new bool[] { false, true };
                            UserInitiatedInterfaceConfigurationCacheSet = new bool[] { true, true };
                        }
                        else if (ws_userinitiated[0].EnabledState == 32771) // BIOS & OS enabled
                        {
                            UserInitiatedInterfaceConfigurationCache = new bool[] { true, true };
                            UserInitiatedInterfaceConfigurationCacheSet = new bool[] { true, true };
                        }

                        enabled = UserInitiatedInterfaceConfigurationCache[source];

                        return AmtCallStatus.SUCCESS;
                    }
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetUserInitiatedInterfaceConfiguration WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetUserInitiatedInterfaceConfiguration CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteAccessAdmin.GetUserInitiatedInterfaceConfiguration",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                if (source >= 2) return AmtCallStatus.NOT_PERMITTED;
                AmtCallStatus r = (AmtCallStatus)service.GetUserInitiatedInterfaceConfiguration(source, out enabled);
                if (r == AmtCallStatus.SUCCESS)
                {
                    UserInitiatedInterfaceConfigurationCache[source] = enabled;
                    UserInitiatedInterfaceConfigurationCacheSet[source] = true;
                }
                return r;
            }
        }

    }
}
