// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Text.RegularExpressions;
using System.Threading;
using Intel.Manageability.Cim; 
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.Cim.Untyped;
using Intel.Manageability.Exceptions;
using Intel.Manageability.WSManagement;

namespace ManageabilityStack
{
    public class AmtRemoteControl
    {
        #region CONSTANTS

        //  command line flags
        private const string POWER = "p";
        private const string CAPABILITIES = "c";
        private const string REMOTE = "r";
        private const string API_TEST = "api";
        private const string REDUCED_API_TEST = "a";
        private const string OPT_HOST = "host";
        private const string OPT_PASS = "pass";
        private const string OPT_USER = "user";
        private const string OPT_VERBOSE = "verbose";

        //  Remote control commands
        const int Reset = 10;
        const int PowerUp = 2;
        const int PowerDown = 8;
        const int PowerCycleReset = 5;
        const int SetBootOptions = 33; //0x21

        //Special commands
        const int NOP = 0; // 0x00
        const int ForcePxeBoot = 1; // 0x01
        const int ForceHardDriveBoot = 2; // 0x02
        const int ForceHardDriveSafeModeBoot = 3; // 0x03
        const int ForceDiagnosticsBoot = 4; // 0x04
        const int ForceCdOrDvdBoot = 5; // 0x05
        // 06h-0BFh Reserved for future ASF definition 
        const int IntelOemCommand = 193; // 0x0C1
        // 0C1h-0FFh Reserved FOR OEM 

        //Constants
        const int NO_VALUE = -1;
        const int PT_STATUS_SUCCESS = 0;
        const int PT_STATUS_NOT_SUPPORTED = 1;

        //OEM Parameters - Intel(R) AMT proprietary boot options
        const int UseSol = 1;

        //Reserved bits for checking
        //correctness of the Boot Options settings
        const int BootOptionsReservedBits = 1817;


        //Special Command Parameters:
        //Following boot options can be defined by using the bitwise OR operator. 
        //For example: 
        //unsigned short specialCommParam = UseIderCD | ReflashBios
        const int UseIderFloppy = 1;   // use floppy as IDER boot device
        const int ReflashBios = 4;   // 1 << 2
        const int BiosSetup = 8;  // 1 << 3
        const int BiosPause = 16; // 1 << 4
        const int UseIderCD = 257;// 1 | (1 << 8) use CD/DVD as IDER boot device

        // Standard boot options
        // Following boot options can be defined by using the bitwise OR operator. 
        // For example: 
        // unsigned short bootOptions = LockPowerButton | LockKeyboard | FirmwareVerbosityVerbose
        const int LockPowerButton = 2;	// 1 << 1
        const int LockResetButton = 4;	// 1 << 2
        const int LockKeyboard = 32;	// 1 << 5
        const int LockSleepButton = 64;	// 1 << 6
        const int UserPasswordBypass = 2048;	// 1 << 11 
        const int ForceProgressEvents = 4096;	// 1 << 12

        // only one from the Firmware verbosity options can be used
        const int FirmwareVerbositySystemDefault = 0;	 // system default
        const int FirmwareVerbosityQuiet = 8192;	 // 1 << 13 minimal screen activity
        const int FirmwareVerbosityVerbose = 16384; // 1 << 14 all messages appear on the screen
        const int FirmwareVerbosityScreen = 24576; // 3 << 13 blank, no messages appear on the screen.
        const int ConfigurationDataReset = 32768; // 1 << 14

        // Reserved bits for checking
        // correctness of the Special Parameters settings
        const int SpecialCommandParametersReservedBits = 65248;

        #endregion CONSTANTS

        private AmtSystem system;
        private RemoteControlService service;

        public AmtPowerStates PowerState = AmtPowerStates.UNKNOWN15;

        public AmtRemoteControl(AmtSystem system, RemoteControlService service)
        {
            this.system = system;
            this.service = service;
        }

        public void Dispose()
        {
            system = null;
            service = null;
        }

        private Thread execThread;
        private AmtCallStatus execThreadReturn = AmtCallStatus.NOP;
        public AmtCallStatus SendRemoteControl(byte Command, uint IanaOemNumber, byte SpecialCommand, ushort SpecialCommandParameter, ushort BootOptions, ushort OEMparameters)
        {
            if (execThread != null) return AmtCallStatus.FAILED_WEB_CALL;
            object[] p = new object[6];
            p[0] = Command;
            p[1] = IanaOemNumber;
            p[2] = SpecialCommand;
            p[3] = SpecialCommandParameter;
            p[4] = BootOptions;
            p[5] = OEMparameters;

            execThread = new Thread(new ParameterizedThreadStart(SendRemoteControlEx));

            execThreadReturn = AmtCallStatus.NOP;
            execThread.Start(p);
            
            // Wait for the thread to end or kill it!
            int rounds = Properties.Settings.Default.HttpTimeout;
            while (rounds-- != 0 && execThreadReturn == AmtCallStatus.NOP)
            {
                Thread.Sleep(1000);
            }
            if (execThreadReturn == AmtCallStatus.NOP)
            {
                execThread.Interrupt();
                execThreadReturn = AmtCallStatus.FAILED_WEB_CALL;
            }

            execThread = null;
            return execThreadReturn;
        }

        private void SendRemoteControlEx(object po)
        {
            object[] p = (object[])po;
            byte Command = (byte)p[0];
            uint IanaOemNumber = (uint)p[1];
            byte SpecialCommand = (byte)p[2];
            ushort SpecialCommandParameter = (ushort)p[3];
            ushort BootOptions = (ushort)p[4];
            ushort OEMparameters = (ushort)p[5];
            AmtCallStatus bootReturnStatus = AmtCallStatus.NOP;

            if (system.WsMan != null)
            {
                // USE WSMAN
                byte CommandEx = 0;
                if (Command == 0x0010) CommandEx = 10; // (Reset)
                if (Command == 0x0011) CommandEx = 2;  // (PowerUp)
                if (Command == 0x0012) CommandEx = 8;  // (PowerDown)
                if (Command == 0x0013) CommandEx = 5;  // (PowerCycleReset)
                if (Command == 0x0021) CommandEx = 33; // (SetBootOptions)

                if (system.UserConsent.OptInRequired != ManageabilityStack.AmtUserConsent.OptInRequiredValues.All
                || (system.UserConsent.OptInState == ManageabilityStack.AmtUserConsent.OptInStates.Received ||
                system.UserConsent.OptInState == ManageabilityStack.AmtUserConsent.OptInStates.InSession))
                {
                    try
                    {
                        CimReference req_BootSourceSettingEPR = null;
                        Get_req_BootSourceSettingEPR(SpecialCommand, ref req_BootSourceSettingEPR);
                        Invoke_changeBootOrder(req_BootSourceSettingEPR, true);
                        if (CommandEx != 8)
                            Put_BootSettingData(SpecialCommandParameter, OEMparameters, SpecialCommandParameter, BootOptions);

                        if (SpecialCommandParameter != 0 && SpecialCommandParameter != 193)
                            Invoke_changeBootOrder(req_BootSourceSettingEPR, true);

                        CimReference BootConfigSettingEPR = null;
                        Get_BootConfigSettingEPR(ref BootConfigSettingEPR);
                        CIM_BootService bootService = new CIM_BootService(system.WsMan);
                        CimReference job;
                        bootReturnStatus = (AmtCallStatus)bootService.SetBootConfigRole(BootConfigSettingEPR, 1, out job);
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        if (wsManException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.SendRemoteControlEx WSMAN error",
                            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                        }
                        CimException cimException = e as CimException;
                        if (cimException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.SendRemoteControlEx CIM error",
                            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                        }

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.SendRemoteControlEx",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                        execThreadReturn = AmtCallStatus.FAILED_WEB_CALL;
                        bootReturnStatus = AmtCallStatus.FAILED_WEB_CALL;

                        Utils.ErrorMessageBox(null, "SendRemoteControl", execThreadReturn + "\r\n" + e.Message);

                    }
                }
                else
                {
                    bootReturnStatus = AmtCallStatus.SUCCESS;
                }

                if (bootReturnStatus == AmtCallStatus.SUCCESS && CommandEx != 33)
                {
                    try
                    {
                        CIM_ComputerSystem.CimKeys keys = new CIM_ComputerSystem.CimKeys();
                        keys.Name = "ManagedSystem";
                        CIM_ComputerSystem managedSystem = new CIM_ComputerSystem(system.WsMan);
                        managedSystem.Get(keys);
                        CimReference managedSystemEPR = managedSystem.Reference;

                        //CimReference ComputerSystem_HostEPR = null;
                        //Get_ComputerSystem_HostEPR(ref ComputerSystem_HostEPR);
                        CIM_PowerManagementService powerMgmtService = new CIM_PowerManagementService(system.WsMan);
                        powerMgmtService.Get();
                        CimReference job;
                        execThreadReturn = (AmtCallStatus)powerMgmtService.RequestPowerStateChange((ushort)CommandEx, managedSystemEPR, null, null, out job);
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        if (wsManException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.SendRemoteControlEx WSMAN error",
                            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                        }
                        CimException cimException = e as CimException;
                        if (cimException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.SendRemoteControlEx CIM error",
                            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                        }

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.SendRemoteControlEx",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                        execThreadReturn = AmtCallStatus.FAILED_WEB_CALL;
                    }
                }
            }
            else
            {
                // USE EOI
                try
                {
                    execThreadReturn = (AmtCallStatus)service.RemoteControl(Command, IanaOemNumber, SpecialCommand, (SpecialCommand != 0), SpecialCommandParameter, (SpecialCommandParameter != 0), BootOptions, (BootOptions != 0), OEMparameters, (OEMparameters != 0));
                }
                catch (Exception) { }
            }

            return;
        }


        /// <summary>
        /// Get the requested EPR (End Point Reference) of CIM_BootConfifSetting
        /// </summary>
        private void Get_BootConfigSettingEPR(ref CimReference  BootConfigSettingEPR)
        {
            try
            {
                CIM_BootConfigSetting res = new CIM_BootConfigSetting(system.WsMan);
                res.Get();
                BootConfigSettingEPR = res.Reference;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Get_BootConfigSettingEPR WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Get_BootConfigSettingEPR CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Get_BootConfigSettingEPR",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return;
            }
        }

        /// <summary>
        /// Get the requested EPR (End Point Reference) of CIM_ComputerSystem instance of the Host
        /// </summary>
        private void Get_ComputerSystem_HostEPR(ref CimReference ComputerSystem_HostEPR)
        {
            Collection<CIM_ComputerSystem> enumComputerSystems = null;
            try
            {
                enumComputerSystems = CIM_ComputerSystem.Enumerate(system.WsMan);
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Get_ComputerSystem_HostEPR WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Get_ComputerSystem_HostEPR CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Get_ComputerSystem_HostEPR",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return;
            }

            foreach (CIM_ComputerSystem enumComputerSystem in enumComputerSystems)
            {
                if (enumComputerSystem.Name == "ManagedSystem") //looking for the Hosts instance
                {
                    ComputerSystem_HostEPR = enumComputerSystem.Reference;
                    return;
                }
            }
        }

        /// <summary>
        /// initialize AMT_BootSettibgData with all the user requests using WinRM.Put
        /// </summary>
        private void Put_BootSettingData(int specialCommand, int oemParameter, int specialCommandParameters, int bootOptions)
        {
            AMT_BootCapabilities BootCapabilitiesObj = new AMT_BootCapabilities(system.WsMan);
            AMT_BootSettingData BootSettingDataObj = new AMT_BootSettingData(system.WsMan);
            try
            {
                BootCapabilitiesObj.Get();
                BootSettingDataObj.Get();
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Put_BootSettingData WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Put_BootSettingData CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Put_BootSettingData",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return;
            }

            BootSettingDataObj.UseIDER = false;
            BootSettingDataObj.IDERBootDevice = 0;
            BootSettingDataObj.BootMediaIndex = 0;

            if (specialCommand == ForceHardDriveSafeModeBoot)
            {
                if (!BootCapabilitiesObj.ForceHardDriveSafeModeBoot)
                    throw new Exception("\nForceHardDriveSafeModeBoot not supported");
                BootSettingDataObj.UseSafeMode = true;
            }
            else
                BootSettingDataObj.UseSafeMode = false;

            if (oemParameter == UseSol)
            {
                if (!BootCapabilitiesObj.SOL)
                    throw new Exception("\nSol not supported");
                BootSettingDataObj.UseSOL = true;
            }
            else
                BootSettingDataObj.UseSOL = false;

            if (specialCommandParameters == UseIderFloppy)
            {
                if (!BootCapabilitiesObj.IDER)
                    throw new Exception("\nIDER not supported");
                BootSettingDataObj.UseIDER = true;
                BootSettingDataObj.IDERBootDevice = 0;
            }
            else if (specialCommandParameters == UseIderCD)
            {
                if (!BootCapabilitiesObj.IDER)
                    throw new Exception("\nIDER not supported");
                BootSettingDataObj.UseIDER = true;
                BootSettingDataObj.IDERBootDevice = 1;
            }
            if ((specialCommandParameters & ReflashBios) == ReflashBios)
            {
                if (!BootCapabilitiesObj.BIOSReflash)
                    throw new Exception("\nBIOSReflash not supported");
                BootSettingDataObj.ReflashBIOS = true;
            }
            else
                BootSettingDataObj.ReflashBIOS = false;

            if ((specialCommandParameters & BiosPause) == BiosPause)
            {
                if (!BootCapabilitiesObj.BIOSPause)
                    throw new Exception("\nBIOSPause not supported");
                BootSettingDataObj.BIOSPause = true;
            }
            else
                BootSettingDataObj.BIOSPause = false;

            if ((specialCommandParameters & BiosSetup) == BiosSetup)
            {
                if (!BootCapabilitiesObj.BIOSSetup)
                    throw new Exception("\nBIOSSetup not supported");
                BootSettingDataObj.BIOSSetup = true;
            }
            else
                BootSettingDataObj.BIOSSetup = false;

            if ((bootOptions & LockPowerButton) == LockPowerButton)
            {
                if (!BootCapabilitiesObj.PowerButtonLock)
                    throw new Exception("\nLockPowerButton not supported");
                BootSettingDataObj.LockPowerButton = true;
            }
            else
                BootSettingDataObj.LockPowerButton = false;

            if ((bootOptions & LockResetButton) == LockResetButton)
            {
                if (!BootCapabilitiesObj.ResetButtonLock)
                    throw new Exception("\nLockResetButton not supported");
                BootSettingDataObj.LockResetButton = true;
            }
            else
                BootSettingDataObj.LockResetButton = false;

            if ((bootOptions & LockKeyboard) == LockKeyboard)
            {
                if (!BootCapabilitiesObj.KeyboardLock)
                    throw new Exception("\nLockKeyboard not supported");
                BootSettingDataObj.LockKeyboard = true;
            }
            else
                BootSettingDataObj.LockKeyboard = false;

            if ((bootOptions & LockSleepButton) == LockSleepButton)
            {
                if (!BootCapabilitiesObj.SleepButtonLock)
                    throw new Exception("\nLockSleepButton not supported");
                BootSettingDataObj.LockSleepButton = true;
            }
            else
                BootSettingDataObj.LockSleepButton = false;

            if ((bootOptions & UserPasswordBypass) == UserPasswordBypass)
            {
                if (!BootCapabilitiesObj.UserPasswordBypass)
                    throw new Exception("\nUserPasswordBypass not supported");
                BootSettingDataObj.UserPasswordBypass = true;
            }
            else
                BootSettingDataObj.UserPasswordBypass = false;

            if ((bootOptions & ForceProgressEvents) == ForceProgressEvents)
            {
                if (!BootCapabilitiesObj.ForcedProgressEvents)
                    throw new Exception("\nForceProgressEvents not supported");
                BootSettingDataObj.ForcedProgressEvents = true;
            }
            else
                BootSettingDataObj.ForcedProgressEvents = false;

            if ((bootOptions & ConfigurationDataReset) == ConfigurationDataReset)
            {
                if (!BootCapabilitiesObj.ConfigurationDataReset)
                    throw new Exception("\nConfigurationDataReset not supported");
                BootSettingDataObj.ConfigurationDataReset = true;
            }
            else
                BootSettingDataObj.ConfigurationDataReset = false;

            while (true)
            {
                if ((bootOptions & FirmwareVerbosityQuiet) == FirmwareVerbosityQuiet)
                {
                    if (!BootCapabilitiesObj.VerbosityQuiet)
                        throw new Exception("\nFirmwareVerbosityQuiet not supported");
                    BootSettingDataObj.FirmwareVerbosity = 1;
                    break;
                }
                if ((bootOptions & FirmwareVerbosityVerbose) == FirmwareVerbosityVerbose)
                {
                    if (!BootCapabilitiesObj.VerbosityVerbose)
                        throw new Exception("\nFirmwareVerbosityVerbose not supported");
                    BootSettingDataObj.FirmwareVerbosity = 2;
                    break;
                }
                if ((bootOptions & FirmwareVerbosityScreen) == FirmwareVerbosityScreen)
                {
                    if (!BootCapabilitiesObj.VerbosityScreenBlank)
                        throw new Exception("\nFirmwareVerbosityScreen not supported");
                    BootSettingDataObj.FirmwareVerbosity = 3;
                    break;
                }
                BootSettingDataObj.FirmwareVerbosity = 0;
                break;
            }

            try
            {
                BootSettingDataObj.Put();
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Put_BootSettingData WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Put_BootSettingData CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Put_BootSettingData",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return;
            }
        }

        /// <summary>
        /// Get the requested EPR (End Point Reference) of the instance representing the requested special boot source
        /// </summary>
        private void Get_req_BootSourceSettingEPR(int specialCommand, ref CimReference req_BootSourceSettingEPR)
        {
            String BootSource_instanceID;
            switch (specialCommand)
            {
                case 1: //ForcePxeBoot:
                    BootSource_instanceID = "Intel(r) AMT: Force PXE Boot";
                    break;
                case 2: //ForceHardDriveBoot:
                    BootSource_instanceID = "Intel(r) AMT: Force Hard-drive Boot";
                    break;
                case 4: //ForceDiagnosticsBoot:
                    BootSource_instanceID = "Intel(r) AMT: Force Diagnostic Boot";
                    break;
                case 5: //ForceCdOrDvdBoot:
                    BootSource_instanceID = "Intel(r) AMT: Force CD/DVD Boot";
                    break;
                default:
                    BootSource_instanceID = "0";
                    break;
            }

            try
            {
                Collection<CIM_BootSourceSetting> enumBootSourceSettings = CIM_BootSourceSetting.Enumerate(system.WsMan);
                foreach (CIM_BootSourceSetting enumBootSourceSetting in enumBootSourceSettings)
                {
                    if (BootSource_instanceID.Equals(enumBootSourceSetting.InstanceID))
                    {
                        req_BootSourceSettingEPR = enumBootSourceSetting.Reference;
                        return;
                    }
                }
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Get_req_BootSourceSettingEPR WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Get_req_BootSourceSettingEPR CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Get_req_BootSourceSettingEPR",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return;
            }
        }

        private CimReference Invoke_SetBootConfigRole(CimReference BootConfigSettingEPR)
        {
            try
            {
                CIM_BootService bootService = new CIM_BootService(system.WsMan);
                CimReference job;
                AmtCallStatus r = (AmtCallStatus)bootService.SetBootConfigRole(BootConfigSettingEPR, 1, out job);
                if (r != AmtCallStatus.SUCCESS) return null;
                return job;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Invoke_SetBootConfigRole WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Invoke_SetBootConfigRole CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Invoke_SetBootConfigRole",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return null;
            }
        }

        private CimReference Invoke_changeBootOrder(CimReference req_BootSourceSettingEPR, bool exist_EPR)
        {
            try
            {
                CIM_BootConfigSetting bootConfigSetting = new CIM_BootConfigSetting(system.WsMan);
                bootConfigSetting.Get();

                AmtCallStatus r = AmtCallStatus.INTERNAL_ERROR;
                CimReference job;

                if (req_BootSourceSettingEPR == null)
                    r = (AmtCallStatus)bootConfigSetting.ChangeBootOrder(null, out job);
                else
                    r = (AmtCallStatus)bootConfigSetting.ChangeBootOrder(new CimReference[] { req_BootSourceSettingEPR }, out job);
                if (r != AmtCallStatus.SUCCESS) return null;

                return job;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Invoke_changeBootOrder WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Invoke_changeBootOrder CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Invoke_changeBootOrder",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return null;
            }
        }

        private CimReference Invoke_PowerStateChange(int command, CimReference ComputerSystem_HostEPR)
        {
            try
            {
                CIM_PowerManagementService powerMgmtService = new CIM_PowerManagementService(system.WsMan);
                CimReference job;
                AmtCallStatus r = (AmtCallStatus)powerMgmtService.RequestPowerStateChange((ushort)command, ComputerSystem_HostEPR, null, null, out job);
                if (r != AmtCallStatus.SUCCESS) return null;
                return job;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Invoke_PowerStateChange WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Invoke_PowerStateChange CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.Invoke_PowerStateChange",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return null;
            }
        }

        public class PowerManagementCapabilities
        {
            public bool PowerStateSettable;
            public bool PowerCyclingSupported;
            public bool TimedPowerOnSupported;
            public bool OffHardPowerCyclingSupported;
            public bool HWResetSupported;
        }

        public AMT_BootCapabilities GetBootCapabilities()
        {
            if (GetBootCapabilitiesCacheSet == false) FetchGetBootCapabilities();
            return GetBootCapabilitiesCache1;
        }

        public PowerManagementCapabilities GetPowerManagementCapabilities()
        {
            if (GetBootCapabilitiesCacheSet == false) FetchGetBootCapabilities();
            return GetBootCapabilitiesCache3;
        }

        private bool GetBootCapabilitiesCacheSet = false;
        private AMT_BootCapabilities GetBootCapabilitiesCache1 = null;
        private CIM_PowerManagementCapabilities GetBootCapabilitiesCache2 = null;
        private PowerManagementCapabilities GetBootCapabilitiesCache3 = null;
        public void FetchGetBootCapabilities()
        {
            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    GetBootCapabilitiesCache1 = new AMT_BootCapabilities(system.WsMan);
                    GetBootCapabilitiesCache1.Get();
                    GetBootCapabilitiesCache2 = new CIM_PowerManagementCapabilities(system.WsMan);
                    GetBootCapabilitiesCache2.Get();
                    GetBootCapabilitiesCache3 = new PowerManagementCapabilities();

                    if (GetBootCapabilitiesCache2 != null)
                    {
                        foreach (uint x in GetBootCapabilitiesCache2.PowerChangeCapabilities)
                        {
                            switch (x)
                            {
                                case 3: GetBootCapabilitiesCache3.PowerStateSettable = true; break;
                                case 4: GetBootCapabilitiesCache3.PowerCyclingSupported = true; break;
                                case 5: GetBootCapabilitiesCache3.TimedPowerOnSupported = true; break;
                                case 6: GetBootCapabilitiesCache3.OffHardPowerCyclingSupported = true; break;
                                case 7: GetBootCapabilitiesCache3.HWResetSupported = true; break;
                            }
                        }
                    }
                    GetBootCapabilitiesCacheSet = true;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.FetchGetBootCapabilities WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.FetchGetBootCapabilities CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.FetchGetBootCapabilities",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return;
                }
            }
            else
            {
                // USE EOI
                uint IanaOemNumber;
                uint OemDefinedCapabilities;
                ushort SpecialCommandsSupported;
                byte SystemCapabilitiesSupported;
                uint SystemFirmwareCapabilities;

                AmtCallStatus r = (AmtCallStatus)service.GetRemoteControlCapabilities(out IanaOemNumber, out OemDefinedCapabilities, out SpecialCommandsSupported, out SystemCapabilitiesSupported, out SystemFirmwareCapabilities);
                if (r != AmtCallStatus.SUCCESS) return;
                AMT_BootCapabilities o = new AMT_BootCapabilities();

                o.VerbosityScreenBlank = ((SystemFirmwareCapabilities & 0x0001) != 0);
                o.PowerButtonLock = ((SystemFirmwareCapabilities & 0x0002) != 0);
                o.ResetButtonLock = ((SystemFirmwareCapabilities & 0x0004) != 0);
                o.KeyboardLock = ((SystemFirmwareCapabilities & 0x0020) != 0);
                o.SleepButtonLock = ((SystemFirmwareCapabilities & 0x0040) != 0);
                o.UserPasswordBypass = ((SystemFirmwareCapabilities & 0x0800) != 0);
                o.ForcedProgressEvents = ((SystemFirmwareCapabilities & 0x1000) != 0);
                o.VerbosityVerbose = ((SystemFirmwareCapabilities & 0x2000) != 0);
                o.VerbosityQuiet = ((SystemFirmwareCapabilities & 0x4000) != 0);
                o.ConfigurationDataReset = ((SystemFirmwareCapabilities & 0x8000) != 0);

                o.SOL = ((OemDefinedCapabilities & 0x0002) != 0);
                o.IDER = ((OemDefinedCapabilities & 0x0001) != 0);

                o.BIOSReflash = ((OemDefinedCapabilities & 0x0004) != 0);
                o.BIOSSetup = ((OemDefinedCapabilities & 0x0008) != 0);

                o.ForcePXEBoot = ((SpecialCommandsSupported & 0x0100) != 0);
                o.ForceHardDriveBoot = ((SpecialCommandsSupported & 0x0200) != 0);
                o.ForceHardDriveSafeModeBoot = ((SpecialCommandsSupported & 0x0400) != 0);
                o.ForceDiagnosticBoot = ((SpecialCommandsSupported & 0x0800) != 0);
                o.ForceCDorDVDBoot = ((SpecialCommandsSupported & 0x1000) != 0);

                GetBootCapabilitiesCache1 = o;

                GetBootCapabilitiesCache3 = new PowerManagementCapabilities();
                GetBootCapabilitiesCache3.HWResetSupported = ((SystemCapabilitiesSupported & 0x08) != 0);
                GetBootCapabilitiesCache3.PowerStateSettable = ((SystemCapabilitiesSupported & 0x02) != 0) && ((SystemCapabilitiesSupported & 0x03) != 0);
                GetBootCapabilitiesCache3.PowerCyclingSupported = ((SystemCapabilitiesSupported & 0x01) != 0);

                GetBootCapabilitiesCacheSet = true;
            }
        }

        public enum AmtPowerStates
        {
            S0_POWERED = 0,
            S1_SLEEPING_WITH_CONTEXT = 1,
            S2_SLEEPING_WITHOUT_CONTEXT = 2,
            S3_SLEEPING_WITH_MEMORY_CONTEXT_ONLY = 3,
            S4_SUSPENDED_TO_DISK = 4,
            S5_SOFT_OFF = 5,
            S4_OR_S5 = 6,
            MECHANICAL_OFF = 7,
            S1_OR_S2_OR_S3 = 8,
            S1_OR_S2_OR_S3_OR_S4 = 9,
            S5_OVERRIDE = 10,
            LEGACY_ON = 11,
            LEGACY_OFF = 12,
            UNKNOWN13 = 13,
            UNKNOWN14 = 14,
            UNKNOWN15 = 15,
            ERROR_GETTING_STATE = 16,
            UNKNOWN = 254,
            DISCONNECTED = 255,
        }

        private AmtPowerStates ConvertPowerStateWsmanToEoi(ushort powerstate)
        {
            switch (powerstate)
            {
                case 1: return AmtPowerStates.UNKNOWN; // Other
                case 2: return AmtPowerStates.S0_POWERED; // On
                case 3: return AmtPowerStates.S1_SLEEPING_WITH_CONTEXT; // Sleep - Light
                case 4: return AmtPowerStates.S3_SLEEPING_WITH_MEMORY_CONTEXT_ONLY; // Sleep - Deep
                case 5: return AmtPowerStates.S5_SOFT_OFF; // Power Cycle (Off - Soft)
                case 6: return AmtPowerStates.MECHANICAL_OFF; // Off - Hard
                case 7: return AmtPowerStates.S4_SUSPENDED_TO_DISK; // Hibernate (Off - Soft)
                case 8: return AmtPowerStates.S5_SOFT_OFF; // Off - Soft
                case 9: return AmtPowerStates.MECHANICAL_OFF; // Power Cycle (Off-Hard)
                default: return AmtPowerStates.UNKNOWN;
            }
        }

        public AmtPowerStates GetSystemPowerState()
        {
            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    CIM_AssociatedPowerManagementService r = new CIM_AssociatedPowerManagementService(system.WsMan);
                    r.Get();
                    if (r.PowerStateExist == false) { PowerState = AmtPowerStates.UNKNOWN15; return AmtPowerStates.UNKNOWN; }
                    PowerState = ConvertPowerStateWsmanToEoi(r.PowerState);
                    return PowerState;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.GetSystemPowerState WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.GetSystemPowerState CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.GetSystemPowerState",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtPowerStates.ERROR_GETTING_STATE;
                }
            }
            else
            {
                // USE EOI
                AmtCallStatus r;
                uint systemPowerState;
                try
                {
                    r = (AmtCallStatus)service.GetSystemPowerState(out systemPowerState);
                }
                catch (Exception)
                {
                    PowerState = AmtPowerStates.UNKNOWN15;
                    return AmtPowerStates.ERROR_GETTING_STATE;
                }
                if (r != AmtCallStatus.SUCCESS) return AmtPowerStates.ERROR_GETTING_STATE;
                PowerState = (AmtPowerStates)(systemPowerState & (uint)0x000F);
                return PowerState;
            }
        }

        public AmtCallStatus GetSystemPowerStateEx(out AmtPowerStates powerState, out bool batteryPower, out bool osLockup)
        {
            powerState = 0;
            batteryPower = false;
            osLockup = false;

            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    CIM_AssociatedPowerManagementService rr = new CIM_AssociatedPowerManagementService(system.WsMan);
                    rr.Get();
                    if (rr.PowerStateExist == false) { powerState = AmtPowerStates.UNKNOWN15; return AmtCallStatus.SUCCESS; }
                    PowerState = powerState = ConvertPowerStateWsmanToEoi(rr.PowerState);
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.GetSystemPowerStateEx WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.GetSystemPowerStateEx CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.GetSystemPowerStateEx",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use EOI
                uint systemPowerState;
                AmtCallStatus r;

                try
                {
                    r = (AmtCallStatus)service.GetSystemPowerState(out systemPowerState);
                }
                catch (Exception)
                {
                    powerState = AmtPowerStates.UNKNOWN15;
                    return AmtCallStatus.FAILED_WEB_CALL;
                }

                if (r != AmtCallStatus.SUCCESS) return r;

                PowerState = powerState = (AmtPowerStates)(systemPowerState & (uint)0x000F);
                osLockup = ((systemPowerState & (uint)0x0100) != 0);
                batteryPower = ((systemPowerState & (uint)0x0200) != 0);
                return r;
            }
        }

        public enum AmtAlarmWakeupCommand
        {
            /// <summary>
            /// Disable previously defined host wake up.
            /// </summary>
            ClearAlarmWake,
            /// <summary>
            /// Enable host wake up.
            /// </summary>
            SetAlarmWake
        }

        public enum AmtAlarmWakeupState
        {
            /// <summary>
            /// Wake up alarm is not armed.
            /// </summary>
            WakeUpIsStopped,
            /// <summary>
            /// Wake up alarm is armed.
            /// </summary>
            WakeUpIsArmed
        }

        public AmtCallStatus GetAlarmWakeUp(out AmtAlarmWakeupState state, out System.DateTime nextTime, out TimeSpan interval)
        {
            AmtCallStatus r = AmtCallStatus.SUCCESS;
            state = AmtAlarmWakeupState.WakeUpIsStopped;
            nextTime = System.DateTime.MinValue;
            interval = TimeSpan.MinValue;

            if (system.WsMan != null)
            {
                // USE WSMAN
                AMT_AlarmClockService alarmClock = new AMT_AlarmClockService(system.WsMan);
                try
                {
                    alarmClock.Get();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.GetAlarmWakeUp WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.GetAlarmWakeUp CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.GetAlarmWakeUp",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }

                if (!alarmClock.NextAMTAlarmTimeExist)
                {
                    state = AmtAlarmWakeupState.WakeUpIsStopped;
                }
                else
                {
                    try
                    {
                        state = AmtAlarmWakeupState.WakeUpIsArmed;

                        CimDateTime alarmTime = alarmClock.NextAMTAlarmTime;
                        string alarmTimeString = alarmTime.ToString();
                        Debug.Print(alarmTimeString);
                        nextTime = new DateTime(alarmTime.Years, alarmTime.Months, alarmTime.Days, alarmTime.Hours,
                            alarmTime.Minutes, alarmTime.Seconds);
                        if (nextTime.Kind != DateTimeKind.Unspecified)
                            nextTime = new System.DateTime(nextTime.Ticks, DateTimeKind.Unspecified);

                        if (alarmClock.AMTAlarmClockIntervalExist)
                        {
                            Regex intervalRegex = new Regex(@"P((?<days>\d*)D)?T((?<hours>\d*)H)?((?<minutes>\d*)M)?");
                            string intervalString = alarmClock.AMTAlarmClockInterval.ToString();
                            Match match = intervalRegex.Match(intervalString);
                            if (match.Success)
                            {
                                int days = 0;
                                int hours = 0;
                                int minutes = 0;

                                if (match.Groups["days"].Success)
                                    days = Convert.ToInt32(match.Groups["days"].Value);
                                if (match.Groups["hours"].Success)
                                    hours = Convert.ToInt32(match.Groups["hours"].Value);
                                if (match.Groups["minutes"].Success)
                                    minutes = Convert.ToInt32(match.Groups["minutes"].Value);
                                interval = new TimeSpan(days, hours, minutes, 0);
                            }
                        }
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        if (wsManException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.GetAlarmWakeUp WSMAN error",
                            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                        }
                        CimException cimException = e as CimException;
                        if (cimException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.GetAlarmWakeUp CIM error",
                            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                        }

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.GetAlarmWakeUp",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                        return AmtCallStatus.FAILED_WEB_CALL;
                    }
                }
            }
            else
            {
                // USE EOI
                uint alarmWakeupState;
                DateTimeType alarmWakeupNextTime;
                DateTimeType alarmWakeupInterval;
                try
                {
                    r = (AmtCallStatus)service.GetAlarmWakeUp(out alarmWakeupState, out alarmWakeupNextTime, out alarmWakeupInterval);
                    if (r == AmtCallStatus.SUCCESS)
                    {
                        state = (AmtAlarmWakeupState)alarmWakeupState;
                        if (alarmWakeupNextTime == null)
                            nextTime = System.DateTime.MinValue;
                        else
                            nextTime = new System.DateTime(alarmWakeupNextTime.Date.Years, alarmWakeupNextTime.Date.Months,
                                    alarmWakeupNextTime.Date.Days, alarmWakeupNextTime.Time.Hours, alarmWakeupNextTime.Time.Minutes,
                                    alarmWakeupNextTime.Time.Seconds, DateTimeKind.Unspecified);

                        if (alarmWakeupInterval == null)
                            interval = TimeSpan.MinValue;
                        else
                            interval = new TimeSpan(alarmWakeupInterval.Date.Days, alarmWakeupInterval.Time.Hours,
                                alarmWakeupInterval.Time.Minutes, alarmWakeupInterval.Time.Seconds);
                    }
                }
                catch (Exception)
                {
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            return r;
        }

        public AmtCallStatus SetAlarmWakeUp(AmtAlarmWakeupCommand command, System.DateTime startTime, TimeSpan interval)
        {
            AmtCallStatus r = AmtCallStatus.SUCCESS;

            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AMT_AlarmClockService alarmClock = new AMT_AlarmClockService(system.WsMan);
                    alarmClock.Get();

                    switch (command)
                    {
                        case AmtAlarmWakeupCommand.ClearAlarmWake:
                            alarmClock.RemoveAMTAlarmClockInterval();
                            alarmClock.RemoveNextAMTAlarmTime();
                            break;
                        case AmtAlarmWakeupCommand.SetAlarmWake:
                            alarmClock.NextAMTAlarmTime = new CIMDateTimeAbsolute(startTime);
                            if (interval == TimeSpan.MinValue)
                                alarmClock.RemoveAMTAlarmClockInterval();
                            else
                            {
                                string intervalTimeString = String.Format("P{0}DT{1}H{2}M",
                                    interval.Days, interval.Hours, interval.Minutes);
                                alarmClock.AMTAlarmClockInterval = new CimDateTimeInterval(intervalTimeString);
                            }
                            break;
                    } // END switch (command)
                    alarmClock.Put();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.SetAlarmWakeUp WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.SetAlarmWakeUp CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtRemoteControl.SetAlarmWakeUp",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // USE EOI
                DateTimeType startTimeType = new DateTimeType();
                startTimeType.Date = new DateStructType();
                startTimeType.Time = new TimeStructType();
                DateTimeType intervalType = null;

                startTimeType.Date.Months = (ushort)startTime.Month;
                startTimeType.Date.Days = (ushort)startTime.Day;
                startTimeType.Date.Years = (ushort)startTime.Year;
                startTimeType.Time.Hours = (ushort)startTime.Hour;
                startTimeType.Time.Minutes = (ushort)startTime.Minute;

                if (interval != TimeSpan.MinValue)
                {
                    intervalType = new DateTimeType();
                    intervalType.Date = new DateStructType();
                    intervalType.Time = new TimeStructType();
                    intervalType.Date.Days = (ushort)interval.Days;
                    intervalType.Time.Hours = (ushort)interval.Hours;
                    intervalType.Time.Minutes = (ushort)interval.Minutes;
                }

                try
                {
                    r = (AmtCallStatus)service.SetAlarmWakeUp((uint)command, startTimeType, intervalType);
                }
                catch (Exception)
                {
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            return r;
        }
    }
}
