// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Threading;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.Exceptions;
using Intel.Manageability.WSManagement;

namespace ManageabilityStack
{
    /// <summary>
    /// Represents the PT_STATUS codes that are returned by the Intel(R) AMT 
    /// commands.
    /// </summary>
    /// <remarks>
    /// Refer the 'Network Interface Guide' document in the Intel(R) AMT SDK 
    /// for more information. Also refer to the StatusCodeDefinitions.h file 
    /// in the Intel(R) AMT SDK for the definition of these status codes.
    /// </remarks>
    /// TODO: Document each status code.
    public enum AmtCallStatus
    {
        SUCCESS = 0x0,
        INTERNAL_ERROR = 0x1,
        NOT_READY = 0x2,
        INVALID_PT_MODE = 0x3,
        INVALID_MESSAGE_LENGTH = 0x4,
        TABLE_FINGERPRINT_NOT_AVAILABLE = 0x5,
        INTEGRITY_CHECK_FAILED = 0x6,
        UNSUPPORTED_ISVS_VERSION = 0x7,
        APPLICATION_NOT_REGISTERED = 0x8,
        INVALID_REGISTRATION_DATA = 0x9,
        APPLICATION_DOES_NOT_EXIST = 0xA,
        NOT_ENOUGH_STORAGE = 0xB,
        INVALID_NAME = 0xC,
        BLOCK_DOES_NOT_EXIST = 0xD,
        INVALID_BYTE_OFFSET = 0xE,
        INVALID_BYTE_COUNT = 0xF,
        NOT_PERMITTED = 0x10,
        NOT_OWNER = 0x11,
        BLOCK_LOCKED_BY_OTHER = 0x12,
        BLOCK_NOT_LOCKED = 0x13,
        INVALID_GROUP_PERMISSIONS = 0x14,
        GROUP_DOES_NOT_EXIST = 0x15,
        INVALID_MEMBER_COUNT = 0x16,
        MAX_LIMIT_REACHED = 0x17,
        INVALID_AUTH_TYPE = 0x18,
        AUTHENTICATION_FAILURE = 0x19,
        INVALID_DHCP_MODE = 0x1A,
        INVALID_IP_ADDRESS = 0x1B,
        INVALID_DOMAIN_NAME = 0x1C,
        UNSUPPORTED_VERSION = 0x1D,
        REQUEST_UNEXPECTED = 0x1E,
        INVALID_TABLE_TYPE = 0x1F,
        INVALID_PROVISIONING_STATE = 0x20,
        UNSUPPORTED_OBJECT = 0x21,
        INVALID_TIME = 0x22,
        INVALID_INDEX = 0x23,
        INVALID_PARAMETER = 0x24,
        INVALID_NETMASK = 0x25,
        FLASH_WRITE_LIMIT_EXCEEDED = 0x26,
        INVALID_IMAGE_LENGTH = 0x27,
        INVALID_IMAGE_SIGNATURE = 0x28,
        PROPOSE_ANOTHER_VERSION = 0x29,
        INVALID_PID_FORMAT = 0x2A,
        INVALID_PPS_FORMAT = 0x2B,
        BIST_COMMAND_BLOCKED = 0x2C,
        CONNECTION_FAILED = 0x2D,
        CONNECTION_TOO_MANY = 0x2E,
        RNG_GENERATION_IN_PROGRESS = 0x2F,
        RNG_NOT_READY = 0x30,
        CERTIFICATE_NOT_READY = 0x31,
        DISABLED_BY_POLICY = 0x400,
        NETWORK_IF_ERROR_BASE = 0x800,
        UNSUPPORTED_OEM_NUMBER = 0x801,
        UNSUPPORTED_BOOT_OPTION = 0x802,
        INVALID_COMMAND = 0x803,
        INVALID_SPECIAL_COMMAND = 0x804,
        INVALID_HANDLE = 0x805,
        INVALID_PASSWORD = 0x806,
        INVALID_REALM = 0x807,
        STORAGE_ACL_ENTRY_IN_USE = 0x808,
        DATA_MISSING = 0x809,
        DUPLICATE = 0x80A,
        EVENTLOG_FROZEN = 0x80B,
        PKI_MISSING_KEYS = 0x80C,
        PKI_GENERATING_KEYS = 0x80D,
        INVALID_KEY = 0x80E,
        INVALID_CERT = 0x80F,
        CERT_KEY_NOT_MATCH = 0x810,
        MAX_KERB_DOMAIN_REACHED = 0x811,
        UNSUPPORTED = 0x812,
        INVALID_PRIORITY = 0x813,
        STATUS_NOT_FOUND = 0x814,
        INVALID_CREDENTIALS = 0x815,
        INVALID_PASSPHRASE = 0x816,
        NO_ASSOCIATION = 0x818,
        AUDIT_FAIL = 0x81B,
        STATUS_BLOCKING_COMPONENT = 0x81C,
        NO_WSMAN = 0xFFFB,
        NO_PROCESS = 0xFFFC,
        NOP = 0xFFFD,
        CALL_SKIPPED = 0xFFFE,
        FAILED_WEB_CALL = 0xFFFF
    };

    /// <summary>
    /// Represents the connected state of the system.
    /// </summary>
    public enum AmtSystemObjState
    {
        /// <summary>
        /// The system is disconnected.
        /// </summary>
        Disconnected,
        /// <summary>
        /// A connection attempt is in progress.
        /// </summary>
        Connecting,
        /// <summary>
        /// The system is connected.
        /// </summary>
        Connected,
        /// <summary>
        /// An error occurred during the connection attempt.
        /// </summary>
        Error
    }

    /// <summary>
    /// Represents an Intel(R) AMT system and the various services that it 
    /// provides.
    /// </summary>
    public class AmtSystem
    {
        /// <summary>
        /// Defines the possible warnings for secure connections.
        /// </summary>
        public enum AmtConnectWarnings
        {
            NameMismatch = 1,
            CertificateInvalid = 2,
            CertificateExpired = 4,
            CertificateUntrusted = 8,
            AuthPrivateKeyNotExportable = 16,
            OpenSslNotAvailable = 32,
            BadRedirectionPort = 64
        }

        #region Private Fields
        private bool GetDnsEntryCacheValid = false;
        private IPHostEntry GetDnsEntryCache = null;
        private AmtRedirectorWrapper redirectorWrapper = null;
        private Thread launchThread = null;
        private bool connectlock = false;
        private NetworkIdentity _HostNameIdentity;

        private bool _MyComputer = false; 

        /// TODO: Deprecate
        //      private string _NetworkName;
        #endregion

        #region Properties
        // Cache Fetch Event
        public delegate void FetchHandler(AmtSystem system, object obj);
        public event FetchHandler OnFetchCompleted;

        // System State Changed Event
        public delegate void ObjectStateHandler(AmtSystem system);
        public event ObjectStateHandler OnStateChanged;

        //
        // All Supported Services
        //

        /// <summary>
        /// The GeneralInfo service access object for this Intel(R) AMT system.
        /// </summary>
        private AmtGeneralInfo info = null;
        /// <summary>
        /// The GeneralInfo Interface provides general (read only) information 
        /// for various (local or network access) management applications.
        /// </summary>
        public AmtGeneralInfo Info
        {
            get
            {
                return info;
            }
        }

        /// <summary>
        /// The RemoteControl service access object for this Intel(R) AMT system.
        /// </summary>
        private AmtRemoteControl remote = null;
        /// <summary>
        /// The Remote Control Interface allows managing the power and booting 
        /// state of the Intel(R) AMT managed system.
        /// </summary>
        public AmtRemoteControl Remote
        {
            get
            {
                return remote;
            }
        }

        /// <summary>
        /// The Network Administration service object for this Intel(R) AMT 
        /// system.
        /// </summary>
        private AmtNetworkAdmin network = null;
        /// <summary>
        /// The Network Administration Interface enables administrators to 
        /// configure local network parameters.
        /// </summary>
        public AmtNetworkAdmin Network
        {
            get
            {
                return network;
            }
        }

        /// <summary>
        /// The Hardware Asset service object for this Intel(R) AMT system.
        /// </summary>
        private AmtAssetManager assets = null;
        /// <summary>
        /// The Hardware Asset Interface exposes operations that return 
        /// hardware asset data.
        /// </summary>
        public AmtAssetManager Assets
        {
            get
            {
                return assets;
            }
        }

        /// <summary>
        /// The Event Manager service object for this Intel(R) AMT system.
        /// </summary>
        private AmtEventManager events = null;
        /// <summary>
        /// The Event Manager interface includes operations that can be used 
        /// by a remote application to subscribe for events, set event filters 
        /// and manage the event log.
        /// </summary>
        public AmtEventManager Events
        {
            get
            {
                return events;
            }
        }

        /// <summary>
        /// The Redirection service object for this Intel(R) AMT system.
        /// </summary>
        private AmtRedirect redirection = null;
        /// <summary>
        /// The Redirection Interface is built for remote redirection 
        /// applications, such as management consoles or redirection utilities.
        /// </summary>
        public AmtRedirect Redirection
        {
            get
            {
                return redirection;
            }
        }

        /// <summary>
        /// The Storage service object for this Intel(R) AMT system.
        /// </summary>
        private AmtStorage storage = null;
        /// <summary>
        /// The Storage Interface gives third-party applications the ability 
        /// to allocate, use and manage blocks of Intel(R) AMT non-volatile 
        /// storage, which can be used to store information that is remotely 
        /// accessible in all power states of the Intel(R) AMT managed system.
        /// </summary>
        public AmtStorage Storage
        {
            get
            {
                return storage;
            }
        }

        /// <summary>
        /// The Remote Agent Presence object for this Intel(R) AMT system.
        /// </summary>
        private AmtWatchdogRemote watchdog = null;
        /// <summary>
        /// The Agent Presence Interface gives third-party applications the 
        /// ability to register to application Watchdog services of Intel AMT.
        /// </summary>
        public AmtWatchdogRemote Watchdog
        {
            get
            {
                return watchdog;
            }
        }

        /// <summary>
        /// The Local Agent Presence object for this Intel(R) AMT system.
        /// </summary>
        private AmtWatchdogLocal watchdogLocal = null;
        /// <summary>
        /// The Local Agent Presence interface is used by registered 
        /// applications (on the local host) to report the presence of the 
        /// associated application (over the Intel Management Engine Interface).
        /// </summary>
        public AmtWatchdogLocal WatchdogLocal
        {
            get
            {
                return watchdogLocal;
            }
        }

        /// <summary>
        /// The Security Administration service object for this Intel(R) AMT 
        /// system.
        /// </summary>
        private AmtSecurityAdmin securityAdmin = null;
        /// <summary>
        /// The Security Administration Interface exposes security 
        /// administration functionality to remote configuration applications.
        /// </summary>
        public AmtSecurityAdmin SecurityAdmin
        {
            get
            {
                return securityAdmin;
            }
        }

        /// <summary>
        /// The Circuit Breaker service object for this Intel(R) AMT system.
        /// </summary>
        private AmtCircuitBreaker circuitBreaker = null;
        /// <summary>
        /// The Circuit Breaker interface (also known as System Defense) is 
        /// used by remote or local applications (i.e. management consoles, 
        /// agent presence, environment detection, and also NAC / 802.1X) to 
        /// apply filters to the network traffic.
        /// </summary>
        public AmtCircuitBreaker CircuitBreaker
        {
            get
            {
                return circuitBreaker;
            }
        }

        /// <summary>
        /// The Wireless Configuration service object for this Intel(R) AMT 
        /// system.
        /// </summary>
        private AmtWireless wireless = null;
        /// <summary>
        /// The WirelessConfiguration interface is used for embedding wireless 
        /// profiles into Intel AMT as well as for reading general wireless 
        /// configuration settings.
        /// </summary>
        public AmtWireless Wireless
        {
            get
            {
                return wireless;
            }
        }

        /// <summary>
        /// The Network Access Control service object for this Intel(R) AMT 
        /// system.
        /// </summary>
        private AmtNetworkAccessControl networkAccessControl = null;
        /// <summary>
        /// This Network Access Control interface provides read-only access to 
        /// the Endpoint Access Control capabilities.
        /// </summary>
        public AmtNetworkAccessControl NetworkAccessControl
        {
            get
            {
                return networkAccessControl;
            }
        }

        /// <summary>
        /// The Network Access Control Admin service object for this Intel(R) 
        /// AMT system.
        /// </summary>
        private AmtNetworkAccessControlAdmin networkAccessControlAdmin = null;
        /// <summary>
        /// The EAC Admin interface is used to remotely configure and manage 
        /// the Endpoint Access Control Admin capabilities.
        /// </summary>
        public AmtNetworkAccessControlAdmin NetworkAccessControlAdmin
        {
            get
            {
                return networkAccessControlAdmin;
            }
        }

        /// <summary>
        /// The Remote Access Admin object for this Intel(R) AMT system.
        /// </summary>
        private AmtRemoteAccessAdmin remoteAccessAdmin = null;
        /// <summary>
        /// The Remote Access Interface enables administrators to configure 
        /// remote access parameters. A Management Presence Server (MPS) 
        /// enables management consoles located inside an enterprise to connect 
        /// to Intel AMT devices located outside the enterprise.
        /// </summary>
        public AmtRemoteAccessAdmin RemoteAccessAdmin
        {
            get
            {
                return remoteAccessAdmin;
            }
        }

        /// <summary>
        /// The Network Time service object for this Intel(R) AMT system.
        /// </summary>
        private AmtNetworkTime networkTime = null;
        /// <summary>
        /// The NetworkTime Interface is built for remote time configuration by 
        /// a management console.
        /// </summary>
        public AmtNetworkTime NetworkTime
        {
            get
            {
                return networkTime;
            }
        }

        /// <summary>
        /// The Audit Log object for this Intel(R) AMT system.
        /// </summary>
        private AmtAuditLog auditLog = null;
        /// <summary>
        /// The Audit interface enables an Auditor to audit actions initiated 
        /// by the administrators and other users on the system.
        /// </summary>
        public AmtAuditLog AuditLog
        {
            get
            {
                return auditLog;
            }
        }

        /// <summary>
        /// TODO
        /// </summary>
        private AmtWakeOnEvent wakeonevent = null;
        /// <summary>
        /// TODO
        /// </summary>
        public AmtWakeOnEvent WakeOnEvent
        {
            get
            {
                return wakeonevent;
            }
        }

        //
        // Connection State
        //

        /// <summary>
        /// The current connection of the Intel(R) AMT system.
        /// </summary>
        private AmtSystemObjState objState = AmtSystemObjState.Disconnected;
        /// <summary>
        /// Gets the current connection of the Intel(R) AMT system.
        /// </summary>
        public AmtSystemObjState State
        {
            get
            {
                return objState;
            }
        }

        /// <summary>
        /// Changes the connection state of the Intel(R) AMT system.
        /// </summary>
        /// <param name="newstate">The new connection state.</param>
        /// <remarks>
        /// Calls the OnStateChanged event handler.
        /// </remarks>
        private void ChangeState(AmtSystemObjState newstate)
        {
            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, this.HostNameIdentity.DefaultName + " changing state to " + newstate.ToString());
            objState = newstate;
            if (newstate == AmtSystemObjState.Disconnected)
            {
                HostNameIdentity.Disconnect();
            }
            if (OnStateChanged != null)
                OnStateChanged(this);
        }

        /// <summary>
        /// Gets or sets the timeout value for HTTP connections to the system.
        /// </summary>
        public static int HttpTimeout
        {
            get
            {
                return Properties.Settings.Default.HttpTimeout;
            }
            set
            {
                Properties.Settings.Default.HttpTimeout = value;
                Properties.Settings.Default.Save();
            }
        }

        //
        // TLS certificate cache
        //

        /// <summary>
        /// Collection of server certificates.
        /// </summary>
        private static Hashtable ServerCertificates = new Hashtable();

        /// <summary>
        /// Gets available certificates for TLS connections on the current network.
        /// </summary>
        /// <returns>
        /// An array of server certificates (X509Certificate2 objects).
        /// </returns>
        /// <remarks>
        /// The current network is defined by the NetworkName property.
        /// </remarks>
        public X509Certificate2[] GetTlsCertificates()
        {
            return (X509Certificate2[])ServerCertificates[HostNameIdentity.DefaultName];
        }

        /// <summary>
        /// Gets available certificates for TLS connections on the defined network.
        /// </summary>
        /// <param name="networkname">Network name to retrieve matching certificates.</param>
        /// <returns>
        /// An array of server certificates (X509Certificate2 objects).
        /// </returns>
        public static X509Certificate2[] GetTlsCertificates(string networkname)
        {
            return (X509Certificate2[])ServerCertificates[networkname];
        }

        /// <summary>
        /// Removes all server certificates for the current network.
        /// </summary>
        /// <remarks>
        /// The current network is defined by the NetworkName property.
        /// </remarks>
        public void ClearTlsCertificateCache()
        {
            ServerCertificates.Remove(HostNameIdentity.DefaultName);
        }

        /// <summary>
        /// Removes all server certificates for the defined network.
        /// </summary>
        /// <param name="networkname">Name of the network to remove matching certificates.</param>
        public static void ClearTlsCertificateCache(string networkname)
        {
            ServerCertificates.Remove(networkname);
        }

        /// <summary>
        /// Add the given certificates to the collection of server 
        /// certificates for TLS communications.
        /// </summary>
        /// <param name="networkname">Network name for the given certificates.</param>
        /// <param name="certArray">An array of server certificates (X509Certificate2 objects).</param>
        public static void AddTlsCertificateCache(string networkname, X509Certificate2[] certArray)
        {
            ServerCertificates[networkname] = certArray;
        }

        /// <summary>
        /// Port number used to communicate to the Intel(R) AMT system.
        /// </summary>
        public int Port;
        /// <summary>
        /// Flag indicating the use of TLS communication.
        /// </summary>
        public bool UseTls;
        /// <summary>
        /// Flag indicating the use of Mutual TLS communication.
        /// </summary>
        public bool UseMutualTls;
        /// <summary>
        /// Username to use when connecting to the Intel(R) AMT system.
        /// </summary>
        public string Username;
        /// <summary>
        /// Password for the account specified by Username.
        /// </summary>
        public string Password;
        /// <summary>
        /// Flag indicating the persistance of the Password.
        /// </summary>
        public bool RememberPassword;
        /// <summary>
        /// Thumbprint (hash) of the certificate used for TLS communication.
        /// </summary>
        public string TlsCertThumbprint;
        /// <summary>
        /// Flag indicating the use of the Web Services interfaces only.
        /// </summary>
        public bool WebServicesOnly;
        /// <summary>
        /// X.509 Certificate to use for mutual authentication (TLS).
        /// </summary>
        private X509Certificate2 authCertificate;
        /// <summary>
        /// Flag to indicate if the system's Intel(R) AMT Web Services cache 
        /// will be automatically fetched.
        /// </summary>
        public bool AutoFetchCache = true;
        /// <summary>
        /// TODO
        /// </summary>
        public bool RemoteManagement = false;
        /// <summary>
        /// Remote control (Terminal) instance for this system.
        /// </summary>
        public object ControlForm = null;

        /// <summary>
        /// Intel Remote KVM instance for this system.
        /// </summary>
        public object KVMForm = null;
        /// <summary>
        /// The AmtKVM access object for remote KVM.
        /// </summary>
        private AmtKVM kvm = null;
        /// <summary>
        /// Gets the KVM communication object for this system.
        /// </summary>
        public AmtKVM KVM { get { return kvm; } }


        public string KVMRfbPassword = "";


        /// <summary>
        /// TODO
        /// </summary>
        private AmtKVMWrapper _KvmWrapper = null;
        /// <summary>
        /// TODO
        /// </summary>
        public AmtKVMWrapper KVMWrapper
        {
            get
            {
                if (_KvmWrapper != null)
                    return _KvmWrapper;

                // If we're using (m)TLS for the connection, then we need 
                // to use the hostname/FQDN of the client in order to 
                // preserve the certificate authentication. Otherwise, the 
                // raw IP address will cause the redirection library to 
                // fail the connection attempt.
                if (UseTls)
                    _KvmWrapper = new AmtKVMWrapper(HostNameIdentity.DisplayName, Username, Password, UseTls, authCertificate, SocksProxy);
                else
                    _KvmWrapper = new AmtKVMWrapper(HostNameIdentity.DefaultName, Username, Password, UseTls, authCertificate, SocksProxy);

                _KvmWrapper.ServerCerts = this.GetTlsCertificates();
                return _KvmWrapper;
            }
        }


        /// <summary>
        /// Object used to track and get user consent.
        /// </summary>
        private AmtUserConsent userConsent = null;
        public AmtUserConsent UserConsent { get { return userConsent; } }

        /// <summary>
        /// IP address for the system.
        /// </summary>
        public IPAddress IPAddress = null;
        /// <summary>
        /// Connection warnings for this Intel(R) AMT system.
        /// </summary>
        public AmtConnectWarnings Warnings = 0;
        /// <summary>
        /// Error status of the certificate chain from the connection.
        /// </summary>
        public X509ChainStatusFlags CertificateError = X509ChainStatusFlags.NoError;
        /// <summary>
        /// Flag used to indicate if the Intel(R) AMT system supports the 
        /// WS-MAN communication protocol.
        /// </summary>
        public bool WsManSupport = true;
        private DotNetWSManClient wsMan = null;
        /// <summary>
        /// Gets the WS-MAN communication object for this system.
        /// </summary>
        public DotNetWSManClient WsMan { get { return wsMan; } }
        /// <summary>
        /// Flag used to indicate if only the WS-MAN communication protocol is 
        /// to be used.
        /// </summary>
        public bool WsManOnly = false;
        /// <summary>
        /// Flag to indicate a local connection to the Intel(R) AMT system.
        /// </summary>
        public bool LocalConnection = false;
        private X509Certificate2 mutualAuthCert = null;
        /// <summary>
        /// Gets the X.509 Certificate used for mutual authentication (TLS).
        /// </summary>
        public X509Certificate2 MutualAuthCert
        {
            get
            {
                return mutualAuthCert;
            }
        }
        /// <summary>
        /// The proxy object to use for HTTP communication.
        /// </summary>
        public WebProxy HttpProxy = null;
        /// <summary>
        /// The proxy object to use for SOCKS communication.
        /// </summary>
        /// <remarks>
        /// This is used for Intel(R) AMT redirection (SOL/IDER) communication.
        /// </remarks>
        public AmtRedirectorWrapper.SOCKSProxy SocksProxy = null;

        // Intel AMT Computer information
        /// <summary>
        /// Hostname of the Intel(R) AMT system.
        /// </summary>
        public NetworkIdentity HostNameIdentity
        {
            get
            {
                return _HostNameIdentity;
            }
            set
            {
                _HostNameIdentity = value;
            }
        }

        /// <summary>
        /// flag to track if this AMT instance is the 'local' computer
        /// </summary>
        public bool MyComputer
        {
            get
            {
                return _MyComputer;
            }
            set
            {
                _MyComputer = value;
            }
        }
        /// <summary>
        /// The UUID of the system.
        /// </summary>
        public Guid PlatformId = Guid.Empty;
        /// <summary>
        /// The Intel(R) AMT major version number.
        /// </summary>
        public uint AmtRev = 0;
        /// <summary>
        /// The Intel(R) AMT minor version number.
        /// </summary>
        public uint AmtRevMin = 0;
        /// <summary>
        /// The Major.Minor version number of the Intel(R) AMT ME.
        /// </summary>
        public float CoreVersion;
        /// <summary>
        /// User defined description of an Intel(R) AMT system connected 
        /// through the Manageability Switchbox Tool.
        /// </summary>
        public string Description;
        /// <summary>
        /// Flag used to indicate the limited availability of Intel(R) AMT features.
        /// </summary>
        /// <remarks>
        /// This can be used to represent a Standard Manageability SKU.
        /// </remarks>
        public bool LimitedFeatures = false;

        // Used by the user to tag this system with user data
        public object Tag = null;

        /// <summary>
        /// Gets the AmtRedirectorWrapper for the system.
        /// </summary>
        public AmtRedirectorWrapper RedirectorWrapper
        {
            get
            {
                if (redirectorWrapper != null)
                    return redirectorWrapper;

                // If we're using (m)TLS for the connection, then we need 
                // to use the hostname/FQDN of the client in order to 
                // preserve the certificate authentication. Otherwise, the 
                // raw IP address will cause the redirection library to 
                // fail the connection attempt.
                if (UseTls)
                    redirectorWrapper = new AmtRedirectorWrapper(HostNameIdentity.DisplayName, Username, Password, UseTls, authCertificate, SocksProxy);
                else
                    redirectorWrapper = new AmtRedirectorWrapper(HostNameIdentity.DefaultName, Username, Password, UseTls, authCertificate, SocksProxy);

                redirectorWrapper.ServerCerts = this.GetTlsCertificates();
                return redirectorWrapper;
            }
        }

        /// <summary>
        /// Returns true if this is a mobile platform
        /// </summary>
        public bool IsMobile
        {
            get
            {
                if (AmtRev == 2 && AmtRevMin >= 5) // Santa Rosa platforms
                    return true;
                else if ((AmtRev >= 3) && (wsMan != null) && (State == AmtSystemObjState.Connected))
                {
                    try
                    {
                        CIM_ComputerSystem.CimKeys keys = new CIM_ComputerSystem.CimKeys();
                        keys.Name = "ManagedSystem";
                        CIM_ComputerSystem compSys = new CIM_ComputerSystem(WsMan);
                        compSys.Get(keys);

                        if (!compSys.DedicatedExist)
                            return false;

                        if (compSys.Dedicated[0] == 32) // Desktop
                            return false;
                        else if (compSys.Dedicated[0] == 33) // Mobile
                            return true;
                        else
                            return false; // Truly an unexpected value for Intel(R) AMT platforms.
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        if (wsManException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSystem.IsMobile WSMAN error",
                            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                        }
                        CimException cimException = e as CimException;
                        if (cimException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSystem.IsMobile CIM error",
                            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                        }

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSystem.IsMobile",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                        return false;
                    }
                }
                else
                {
                    // Either the system doesn't support WSMAN or this 
                    // connection isn't configured to use WSMAN. Therefore, we 
                    // can only assume that this system isn't mobile.
                    // (At least until a better method for making this 
                    // determination is implemented.)
                    return false;
                }
            }
        }

        /// <summary>
        /// Local cached value of the IsWirelessSupported flag.
        /// </summary>
        private bool? IsWirelessSupportedCache = null;

        /// <summary>
        /// Determine if the Intel(R) AMT system supports wireless.
        /// </summary>
        /// <returns>
        /// Returns true if the system supports wireless. Otherwise, the return 
        /// is false.
        /// </returns>
        /// <remarks>
        /// This method depends on using the WS-MAN interface for communicating 
        /// with the targeted Intel(R) AMT system.
        /// </remarks>
        public bool IsWirelessSupported
        {
            get
            {
                if (IsWirelessSupportedCache != null) return (bool)IsWirelessSupportedCache;

                if (wsMan != null)
                {
                    try
                    {
                        CIM_EthernetPort eNetPort = new CIM_EthernetPort(wsMan);
                        CIM_EthernetPort.CimKeys keys = new CIM_EthernetPort.CimKeys();
                        // The WiFi Device ID string is documented in the AMT SDK 
                        // document "Intel AMT WS-Management Flows.pdf".
                        keys.DeviceID = "Intel(r) AMT Ethernet Port 1";
                        eNetPort.Get(keys);

                        // Assumed that Wireless is supported as this point.
                        IsWirelessSupportedCache = true;
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        if (wsManException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSystem.IsWirelessSupported WSMAN error",
                            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                        }
                        CimException cimException = e as CimException;
                        if (cimException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSystem.IsWirelessSupported CIM error",
                            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                        }

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSystem.IsWirelessSupported",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                        IsWirelessSupportedCache = false; // Wireless is assumed unsupported.
                    }

                    return (bool)IsWirelessSupportedCache;
                }
                else
                {
                    // Unable to determine without WSMAN support.
                    return false;
                }
            }
        }

        /// <summary>
        /// Gets the URL for the Intel(R) AMT system.
        /// </summary>
        public string WebURL
        {
            get
            {
                if (UseTls)
                    return "https://" + HostNameIdentity.DefaultName + ":16993";
                return "http://" + HostNameIdentity.DefaultName + ":16992";
            }
        }
        #endregion

        #region Constructors / Dispose
        /// <summary>
        /// Constructs a new Intel(R) AMT system object with the given properties.
        /// </summary>
        /// <param name="networkname">Name of the system on the network (IP address or FQDN).</param>
        /// <param name="port">Port number used to access the Intel(R) AMT ME.</param>
        /// <param name="username">Username of the Intel(R) AMT account to use.</param>
        /// <param name="password">Password for the Intel(R) AMT account specified by username.</param>
        /// <param name="rememberPassword">Flag indicating the permanence of the password.</param>
        /// <param name="webServicesOnly">Flag indicating the use of the Web Services interface only.</param>
        public AmtSystem(string networkname, int port, string username, string password, bool rememberPassword,
            bool webServicesOnly)
        {
            UseTls = false; // This is going to be auto-detected.
            objState = AmtSystemObjState.Disconnected;
            Port = port;
            _HostNameIdentity = new NetworkIdentity(networkname);
            Username = username;
            Password = password;
            RememberPassword = rememberPassword;
            WebServicesOnly = webServicesOnly;
        }

        /// <summary>
        /// Constructs a new Intel(R) AMT system object with the given properties.
        /// </summary>
        /// <param name="networkname">Name of the system on the network (IP address or FQDN).</param>
        /// <param name="port">Port number used to access the Intel(R) AMT ME.</param>
        /// <param name="username">Username of the Intel(R) AMT account to use.</param>
        /// <param name="password">Password for the Intel(R) AMT account specified by username.</param>
        /// <param name="rememberPassword">Flag indicating the permanence of the password.</param>
        /// <param name="webServicesOnly">Flag indicating the use of the Web Services interface only.</param>
        public AmtSystem(string networkname, int port, string username, string password, bool rememberPassword,
            bool webServicesOnly, bool myComputer)
        {
            UseTls = false; // This is going to be auto-detected.
            objState = AmtSystemObjState.Disconnected;
            Port = port;
            _HostNameIdentity = new NetworkIdentity(networkname);
            Username = username;
            Password = password;
            RememberPassword = rememberPassword;
            WebServicesOnly = webServicesOnly;
            MyComputer = myComputer; 
        }

        /// <summary>
        /// Constructs a new Intel(R) AMT system object with the given properties.
        /// </summary>
        /// <param name="networkname">Name of the system on the network (IP address or FQDN).</param>
        /// <param name="port">Port number used to access the Intel(R) AMT ME.</param>
        /// <param name="username">Username of the Intel(R) AMT account to use.</param>
        /// <param name="password">Password for the Intel(R) AMT account specified by username.</param>
        /// <param name="rememberPassword">Flag indicating the permanence of the password.</param>
        /// <param name="webServicesOnly">Flag indicating the use of the Web Services interface only.</param>
        /// <param name="forceIPOption">enumeration indicating which IPAddress mode to use</param>
        public AmtSystem(string networkname, int port, string username, string password, bool rememberPassword,
            bool webServicesOnly, ForceIPOption forceIPOption, IPAddress ipAddress)
        {
            UseTls = false; // This is going to be auto-detected.
            objState = AmtSystemObjState.Disconnected;
            Port = port;
            _HostNameIdentity = new NetworkIdentity(networkname, forceIPOption, ipAddress);
            Username = username;
            Password = password;
            RememberPassword = rememberPassword;
            WebServicesOnly = webServicesOnly;
        }


        /// <summary>
        /// Disposes of the system object, ensuring that open connections are 
        /// closed.
        /// </summary>
        public void Dispose()
        {
            KVMForm = null;
            ControlForm = null;
            Disconnect();
        }
        #endregion

        #region Public Connect / Disconnect methods
        /// <summary>
        /// Disconnects from the Intel(R) AMT system.
        /// </summary>
        public void Disconnect()
        {
            if (State == AmtSystemObjState.Disconnected)
                return;
            new Thread(new ThreadStart(DisconnectEx)).Start();
        }

        /// <summary>
        /// Disconnects from the Intel(R) AMT system.
        /// </summary>
        /// <remarks>
        /// Disposes of the Intel(R) AMT interface objects and their caches.
        /// </remarks>
        private void DisconnectEx()
        {
            if (State == AmtSystemObjState.Disconnected)
                return;
            ChangeState(AmtSystemObjState.Disconnected);
            if (launchThread != null)
            {
                try
                {
                    launchThread.Interrupt();
                    int i = 0;
                    while (launchThread != null && (i++ < 100))
                    {
                        Thread.Sleep(50);
                    }
                    launchThread = null;
                }
                catch (NullReferenceException)
                {
                }
            }
            KVMForm = null;
            ControlForm = null;
            if (redirectorWrapper != null)
                redirectorWrapper.Dispose();
            redirectorWrapper = null;
            if (Info != null)
                Info.Dispose();
            info = null;

            if (Remote != null)
                Remote.Dispose();
            if (CircuitBreaker != null)
                CircuitBreaker.Dispose();
            if (Network != null)
                Network.Dispose();
            if (Assets != null)
                Assets.Dispose();
            if (Events != null)
                Events.Dispose();
            if (Redirection != null)
                Redirection.Dispose();
            if (Watchdog != null)
                Watchdog.Dispose();
            if (NetworkAccessControlAdmin != null)
                NetworkAccessControlAdmin.Dispose();
            if (RemoteAccessAdmin != null)
                RemoteAccessAdmin.Dispose();
            if (AuditLog != null)
                AuditLog.Dispose();
            if (WakeOnEvent != null)
                WakeOnEvent.Dispose();
            if (Wireless != null)
                Wireless.Dispose();
            if (wsMan != null)
                wsMan = null;

            remote = null;
            circuitBreaker = null;
            network = null;
            assets = null;
            events = null;
            redirection = null;
            watchdog = null;
            networkAccessControlAdmin = null;
            remoteAccessAdmin = null;
            auditLog = null;
            wireless = null;
            wakeonevent = null;

            connectlock = false;
        }

        /// <summary>
        /// Connects to the Intel(R) AMT system.
        /// </summary>
        public void Connect()
        {
            if (connectlock == true || launchThread != null || State != AmtSystemObjState.Disconnected)
                return;
            ChangeState(AmtSystemObjState.Connecting);
            launchThread = new Thread(new ParameterizedThreadStart(ConnectEx));
            launchThread.Priority = ThreadPriority.Lowest;
            launchThread.Start(this);
        }
        #endregion

        #region Private Methods
        /// <summary>
        /// Used to fetch any document from this Intel(R) AMT computer, this is 
        /// useful for making test calls or getting the web page data.
        /// This call will authenticate properly and work thru a Switchbox if 
        /// needed.
        /// </summary>
        /// <param name="directiveObj"></param>
        /// <param name="postData"></param>
        /// <returns></returns>
        private string LoadDocument(string directiveObj, string postData)
        {
            HttpWebRequest r = null;

            int retrycount = 3;
            string url;

            if (UseTls == true)
            {
                url = HostNameIdentity.ToWebHttpsURL(Port) + "/" + directiveObj;
            }
            else
            {
                url = HostNameIdentity.ToWebHttpURL(Port) + "/" + directiveObj;
            }


            ManageabilityStack.AmtDebugForm.LogHttpCall(AmtDebugForm.EventType.CallAttempt, url, postData);

            while (retrycount > 0)
            {
                retrycount--;
                try
                {
                    r = (HttpWebRequest)HttpWebRequest.Create(url);
                    r.Timeout = Properties.Settings.Default.HttpTimeout * 1000;
                    ;
                    r.Proxy = HttpProxy;
                    r.PreAuthenticate = true;
                    r.ConnectionGroupName = HostNameIdentity.DefaultName;
                    r.Credentials = GetCredentials(url);

                    if (postData != null)
                    {
                        byte[] data = UTF8Encoding.UTF8.GetBytes(postData);
                        r.Method = "POST";
                        r.ContentType = "application/octet";
                        r.ContentLength = data.Length;

                        Stream s = r.GetRequestStream();
                        s.Write(data, 0, data.Length);
                        s.Close();
                    }

                    retrycount = -1;
                }
                catch (IOException)
                {
                }
            }
            if (retrycount == 0 || r == null)
            {
                ManageabilityStack.AmtDebugForm.LogHttpCall(AmtDebugForm.EventType.CallFailed, url, postData);
                return null;
            }

            HttpWebResponse rr = null;
            try
            {
                rr = (HttpWebResponse)r.GetResponse();
            }
            catch (WebException ex)
            {
                ManageabilityStack.AmtDebugForm.LogHttpCall(AmtDebugForm.EventType.CallFailed, url, postData);
                ManageabilityStack.AmtDebugForm.LogException(this, ex);
                return null;
            }
            if (rr == null)
            {
                ManageabilityStack.AmtDebugForm.LogHttpCall(AmtDebugForm.EventType.CallFailed, url, postData);
                return null;
            }

            byte[] buf = new byte[200];
            StringBuilder sb = new StringBuilder();
            int len;
            while ((len = rr.GetResponseStream().Read(buf, 0, 200)) != 0)
                sb.Append(UTF8Encoding.UTF8.GetString(buf, 0, len));
            return sb.ToString();
        }

        private CredentialCache GetCredentials(string url)
        {
            if (Password != null && Password.Length > 0)
            {
                int i = Username.IndexOf("\\");
                if (i > 0)
                {
                    // Using Kerberos*
                    string t_username = Username.Substring(i + 1);
                    string t_domain = Username.Substring(0, i);
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, string.Format("NetworkCredential Kerberos*: User={0}, Domain={1}, Pass={2}", t_username, t_domain, Password));

                    // Setup the authentication manager
                    if (AuthenticationManager.CustomTargetNameDictionary.ContainsKey(url) == false)
                    {
                        string SPN = "HTTP/" + HostNameIdentity.DefaultName + ":16993";
                        if (url.StartsWith("https", StringComparison.CurrentCultureIgnoreCase) == true)
                            SPN = "HTTP/" + HostNameIdentity.DefaultName + ":16992";
                        AuthenticationManager.CustomTargetNameDictionary.Add(url, SPN);
                    }

                    CredentialCache myCache = new CredentialCache();
                    myCache.Add(new System.Uri(url), "Negotiate", new NetworkCredential(t_username, Password, t_domain));
                    return myCache;
                }
                else
                {
                    // Using HTTP Digest
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "NetworkCredential HTTP Digest: " + Username + ", " + Password);
                    CredentialCache myCache = new CredentialCache();
                    myCache.Add(new System.Uri(url), "Digest", new NetworkCredential(Username, Password));
                    return myCache;
                }
            }

            if ((Password == null || Password.Length == 0) && (Username == null || Username.Length == 0))
            {
                // Use Kerberos* with local credentials

                // Setup the authentication manager
                if (AuthenticationManager.CustomTargetNameDictionary.ContainsKey(url) == false)
                {
                    string SPN = "HTTP/" + HostNameIdentity.DefaultName + ":16993";
                    if (url.StartsWith("https", StringComparison.CurrentCultureIgnoreCase) == true)
                        SPN = "HTTP/" + HostNameIdentity.DefaultName + ":16992";
                    AuthenticationManager.CustomTargetNameDictionary.Add(url, SPN);
                }

                CredentialCache myCache = new CredentialCache();
                myCache.Add(new System.Uri(url), "Negotiate", System.Net.CredentialCache.DefaultNetworkCredentials);
                return myCache;
            }

            return null;
        }

        private void ServiceSetup(System.Web.Services.Protocols.SoapHttpClientProtocol service)
        {

            if (UseTls)
            {
                service.Url = service.Url.Replace("http://", "https://");
                service.Url = service.Url.Replace("16992", "16993");
                if (Port == 16992)
                    Port = 16993;
            }
            else
            {
                service.Url = service.Url.Replace("https://", "http://");
                service.Url = service.Url.Replace("16993", "16992");
                if (Port == 16993)
                    Port = 16992;
            }

            service.Timeout = Properties.Settings.Default.HttpTimeout * 1000; // This will set the SOAP HTTP timeout for all services in milliseconds. (UI Timeout value * 1000)
            service.Proxy = HttpProxy; // Set the HTTP proxy, 
            service.ConnectionGroupName = HostNameIdentity.DefaultName;
            service.Credentials = GetCredentials(service.Url);
            service.PreAuthenticate = true;

            if (UseTls && authCertificate != null)
                service.ClientCertificates.Add(authCertificate);
            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "Service Setup: " + service.Url);
        }

        private bool TryConnect(bool tls, X509Certificate2 cert, out AmtSecurityAdmin secadmin, out AmtGeneralInfo geninfo, out AMT_GeneralSettings genSettings, out bool local)
        {
            UseTls = tls;
            authCertificate = cert;
            geninfo = null;
            secadmin = null;
            genSettings = null;
            local = false;
            WsManOnly = false;

            SecurityAdministrationService securityAdministrationService;
            if (LocalConnection == false)
            {
                securityAdministrationService = new SecurityAdministrationService(HostNameIdentity.ToWebURL(Port));
                ServiceSetup(securityAdministrationService);
                secadmin = new AmtSecurityAdmin(securityAdministrationService, this);
            }

            // Try to connect
            try
            {
                // Parse the version in decimal form
                string s = null;
                HttpStatusCode code = HttpStatusCode.Unauthorized;

                if (SecurityAdmin != null)
                    code = SecurityAdmin.GetCoreVersionEx(out s);
                if (SecurityAdmin == null || code == HttpStatusCode.Unauthorized)
                {
                    // This is a hint that we are connected to LMS local interface
                    secadmin = null;
                    GeneralInfoService generalInfo = new GeneralInfoService(HostNameIdentity.ToWebURL(Port));
                    ServiceSetup(generalInfo);
                    geninfo = new AmtGeneralInfo(this, generalInfo);
                    try
                    {
                        s = geninfo.GetCoreVersion();
                        local = true;
                    }
                    catch (WebException ex)
                    {
                        if (ex.Response != null && (((((System.Net.HttpWebResponse)ex.Response).StatusCode) == HttpStatusCode.Unauthorized) || (((System.Net.HttpWebResponse)ex.Response).StatusCode) == HttpStatusCode.ServiceUnavailable) || (ex.Status == WebExceptionStatus.ProtocolError))
                        {
                            // Hint that this could be WSMAN-only LMS connection
                            secadmin = null;
                            bool useKerberos = false;
                            if (Username.Contains("\\") || Username == string.Empty)
                                useKerberos = true;
                            if (cert != null)
                                // TODO: Can we only use the first Common Name for the cert?
                                wsMan = new DotNetWSManClient(HostNameIdentity.DefaultName, Username, Password, UseTls, useKerberos,
                                    CertificateOperations.GetCertificateCommonNames(cert)[0], HttpProxy);
                            else
                                wsMan = new DotNetWSManClient(HostNameIdentity.DefaultName, Username, Password, UseTls, useKerberos, String.Empty, HttpProxy);

                            if (wsMan != null)
                            {
                                local = true;
                                WsManOnly = true;
                                geninfo = null;
                                secadmin = null;
                                genSettings = null;

                                try
                                {
                                    Collection<CIM_SoftwareIdentity> si = CIM_SoftwareIdentity.Enumerate(WsMan);
                                    // Connect LMS connection.
                                    s = si[0].VersionString;
                                }
                                catch (Exception e)
                                {
                                    WSManException wsManException = e as WSManException;
                                    if (wsManException != null)
                                    {
                                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "CIM_SoftwareIdentity.Enumerate WSMAN error",
                                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                                    }
                                    CimException cimException = e as CimException;
                                    if (cimException != null)
                                    {
                                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "CIM_SoftwareIdentity.Enumerate CIM error",
                                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                                    }

                                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information,
                                        "Failed to enumerate the software identities. can not successfully connect to system.",
                                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);


                                    s = null;
                                    wsMan = null;
                                    genSettings = null;
                                    return false;
                                }
                            }
                        }
                    }
                }
                else if (SecurityAdmin == null || code == HttpStatusCode.NotFound)
                {
                    // This is a hint that we could be connected using WSMAN only
                    secadmin = null;

                    bool useKerberos = false;
                    if (Username.Contains("\\") || Username == string.Empty)
                        useKerberos = true;
                    if (cert != null)
                        // TODO: Can we only use the first Common Name for the cert?
                        wsMan = new DotNetWSManClient(HostNameIdentity.DefaultName, Username, Password, UseTls, useKerberos,
                            CertificateOperations.GetCertificateCommonNames(cert)[0], HttpProxy);
                    else
                        wsMan = new DotNetWSManClient(HostNameIdentity.DefaultName, Username, Password, UseTls, useKerberos, String.Empty, HttpProxy);

                    if (wsMan != null)
                    {
                        WsManOnly = true;
                        try
                        {
                            genSettings = new AMT_GeneralSettings(wsMan);
                            genSettings.Get();

                            Collection<CIM_SoftwareIdentity> si = CIM_SoftwareIdentity.Enumerate(wsMan);
                            if (si == null) return false;
                            s = (si[0].VersionStringExist ? si[0].VersionString : null);
                        }
                        catch (Exception e)
                        {
                            WSManException wsManException = e as WSManException;
                            if (wsManException != null)
                            {
                                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "CIM_SoftwareIdentity.Enumerate WSMAN error",
                                String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                                wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                            }
                            CimException cimException = e as CimException;
                            if (cimException != null)
                            {
                                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "CIM_SoftwareIdentity.Enumerate CIM error",
                                String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                                cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                            }

                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                                "Failed to enumerate the software identities.",
                                String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                            Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                            Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                            wsMan = null;
                            genSettings = null;
                            return false;
                        }
                    }
                }

                if (s == null)
                {
                    geninfo = null;
                    secadmin = null;
                    genSettings = null;
                    return false;
                }

                CoreVersion = 0;
                string[] vers = s.Split('.');
                if (vers.Length > 0)
                {
                    int v;
                    if (int.TryParse(vers[0], out v) == true)
                    {
                        AmtRev = (uint)v;
                        CoreVersion = v;
                    }
                }
                if (vers.Length > 1)
                {
                    int v;
                    if (int.TryParse(vers[1], out v) == true)
                    {
                        AmtRevMin = (uint)v;
                        CoreVersion += ((float)v / (float)(10.0));
                    }
                }
            }
            catch (WebException)
            {
                secadmin = null;
                geninfo = null;
                return false;
            }

            return true;
        }

        /// <summary>
        /// Connects to the Intel(R) AMT system.
        /// </summary>
        /// <param name="parameter">The Intel(R) AMT system to establish a connection with.</param>
        /// <remarks>
        /// First attempts to connect to the system without any security (i.e., 
        /// SMB provisioned + Digest Authentication). If that fails, then the 
        /// second connection attempt is with TLS (server authentication only).
        /// The final connection attempt with with mutual authentication TLS.
        /// Creates the Intel(R) AMT interface objects and fetches their caches.
        /// </remarks>
        private void ConnectEx(object parameter)
        {
            try
            {
                // Reset the connection warnings
                Warnings = 0;

                //
                // Build a connection attempt list in the following order:
                //    1. Local Mutual TLS, if previously used
                //    2. Remote Mutual TLS, if previously used
                //    2. TLS, if previously used
                //    3. Non-TLS
                //    4. TLS
                //    5. Remote Mutual TLS
                //    6. Local Mutual TLS
                //
                ArrayList connectionAttempts = new ArrayList();

                if (LocalConnection) // Local Mutual TLS
                    connectionAttempts.AddRange(CertificateOperations.GetValidAgentCertificates());
                else if (UseMutualTls) // Remote Mutual TLS
                    connectionAttempts.AddRange(CertificateOperations.GetValidConsoleCertificates());
                else if (UseTls) // TLS
                    connectionAttempts.Add(1);
                connectionAttempts.Add(0); // Non-TLS
                connectionAttempts.Add(1); // TLS
                connectionAttempts.AddRange(CertificateOperations.GetValidConsoleCertificates()); // Remote Mutual TLS
                connectionAttempts.AddRange(CertificateOperations.GetValidAgentCertificates()); // Local Mutual TLS
                
                AMT_GeneralSettings wsGeneralInfo = null;
                bool local = false;

                // Try each attempt one by one.
                int j = 0;
                while (j < connectionAttempts.Count)
                {
                    // Check for connection interruption
                    if (State == AmtSystemObjState.Disconnected)
                    {
                        launchThread = null;
                        return;
                    }

                    if (connectionAttempts[j].GetType() == typeof(int) && ((int)connectionAttempts[j]) == 0)
                    {
                        // Try no TLS
                        UseTls = false;
                        UseMutualTls = false;
                        mutualAuthCert = null;

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information,
                            String.Format("Attempting non-TLS connection to {0}", HostNameIdentity.DisplayName));
                        TryConnect(false, null, out securityAdmin, out info, out wsGeneralInfo, out local);
                    }
                    else if (connectionAttempts[j].GetType() == typeof(int) && ((int)connectionAttempts[j]) == 1)
                    {
                        // Try TLS
                        UseTls = true;
                        UseMutualTls = false;
                        mutualAuthCert = null;

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information,
                            String.Format("Attempting TLS connection to {0}", HostNameIdentity.DisplayName));
                        TryConnect(true, null, out securityAdmin, out info, out wsGeneralInfo, out local);
                    }
                    else if (connectionAttempts[j].GetType() == typeof(X509Certificate2))
                    {
                        // Try Mutual-Auth TLS
                        UseTls = true;
                        UseMutualTls = true;
                        mutualAuthCert = (X509Certificate2)(connectionAttempts[j]);

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information,
                            String.Format("Attempting mutual TLS connection to {0} with certificate {1}",
                            HostNameIdentity.DisplayName, CertificateOperations.GetCertificateString(mutualAuthCert)));
                        TryConnect(true, mutualAuthCert, out securityAdmin, out info, out wsGeneralInfo, out local);
                    }
                    if (SecurityAdmin != null || Info != null || wsGeneralInfo != null )
                        break;
                    j++;
                }

                if (SecurityAdmin == null && Info == null && wsGeneralInfo == null )
                {
                    // Connection Failed.
                    UseTls = false;
                    UseMutualTls = false;
                    mutualAuthCert = null;
                    ChangeState(AmtSystemObjState.Disconnected);
                    launchThread = null;
                    return;
                }

                // Check for connection interruption
                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }

                // If WSMAN not initialized yet, do it now.
                if (AmtRev >= 3 && WsManSupport == true && wsMan == null)
                {
                    bool useKerberos = false;
                    if (Username.Contains("\\") || Username == string.Empty)
                        useKerberos = true;
                    if (mutualAuthCert != null)
                        wsMan = new DotNetWSManClient(HostNameIdentity.DefaultName, Username, Password, UseTls,
                            useKerberos, CertificateOperations.GetCertificateCommonNames(mutualAuthCert)[0], HttpProxy);
                    else
                        wsMan = new DotNetWSManClient(HostNameIdentity.DefaultName, Username, Password,
                            UseTls, useKerberos, String.Empty, HttpProxy);
                }

                // Set is this connection is local or remote
                LocalConnection = local;


                // General Information
                if (info == null) 
                {
                    GeneralInfoService generalInfo = new GeneralInfoService(HostNameIdentity.ToWebURL(Port));
                    ServiceSetup(generalInfo);
                    info = new AmtGeneralInfo(this, generalInfo);
                }

                if (info.GetStandardManagibilitySKU())
                    LimitedFeatures = true;
                else
                    LimitedFeatures = false; 

                // Check for connection interruption
                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }

                // Setup local only interfaces
                if (LocalConnection) 
                {
                    // Local watchdog interface for local interface only
                    AgentWatchdogLocalService agentWatchdogLocalService = new AgentWatchdogLocalService(HostNameIdentity.ToWebURL(Port));
                    ServiceSetup(agentWatchdogLocalService);
                    watchdogLocal = new AmtWatchdogLocal(this, agentWatchdogLocalService);

                    if (CoreVersion >= 2.6)
                    {
                        // Event management for local events only
                        EventManagerService eventManagerService = new EventManagerService(HostNameIdentity.ToWebURL(Port));
                        ServiceSetup(eventManagerService);
                        events = new AmtEventManager(eventManagerService, this, true);
                    }

                    if (CoreVersion >= 2.5 && WsManOnly == false)
                    {
                        // Endpoint Access Control for local interface only
                        EndpointAccessControlService endpointAccessControlService = new EndpointAccessControlService(HostNameIdentity.ToWebURL(Port));
                        ServiceSetup(endpointAccessControlService);
                        networkAccessControl = new AmtNetworkAccessControl(this, endpointAccessControlService);
                    }
                }

                // Check for connection interruption
                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }

                // Setup remote only interfaces
                if (!LocalConnection && CoreVersion > 0.0)
                {
                    // Security Admin
                    SecurityAdministrationService securityAdministrationService = new SecurityAdministrationService(HostNameIdentity.ToWebURL(Port));
                    ServiceSetup(securityAdministrationService);
                    securityAdmin = new AmtSecurityAdmin(securityAdministrationService, this);

                    // Network Management
                    if (WsManOnly == false)
                    {
                        NetworkAdministrationService networkAdminService = new NetworkAdministrationService(HostNameIdentity.ToWebURL(Port));
                        ServiceSetup(networkAdminService);
                        try
                        {
                            network = new AmtNetworkAdmin(this, networkAdminService);
                        }
                        catch (Exception)
                        {
                            // Failed to connect
                            Dispose();
                            launchThread = null;
                            return;
                        }
                    }

                    // Hardware Asset Management (WSMAN enabled)
                    HardwareAssetService hardwareAssetService = new HardwareAssetService(HostNameIdentity.ToWebURL(Port));
                    ServiceSetup(hardwareAssetService);
                    assets = new AmtAssetManager(this, hardwareAssetService);
                    if (PlatformId.Equals(Guid.Empty) == true)
                    {
                        AmtHardwareComputerSystem computerSystemAsset = Assets.GetComputerSystem();
                        if (computerSystemAsset != null)
                            PlatformId = computerSystemAsset.UUID;
                    }

                    // Redirect Management (WSMAN enabled)
                    RedirectionService redirectionService = new RedirectionService(HostNameIdentity.ToWebURL(Port));
                    ServiceSetup(redirectionService);
                    redirection = new AmtRedirect(redirectionService, this);

                    // Time Management (WSMAN enabled)
                    NetworkTimeService networkTimeService = new NetworkTimeService(HostNameIdentity.ToWebURL(Port));
                    ServiceSetup(networkTimeService);
                    networkTime = new AmtNetworkTime(networkTimeService, this);

                    // Check for connection interruption
                    if (State == AmtSystemObjState.Disconnected)
                    {
                        launchThread = null;
                        return;
                    }

                    // Remote Control Management (WSMAN enabled)
                    RemoteControlService remoteControl = new RemoteControlService(HostNameIdentity.ToWebURL(Port));
                    ServiceSetup(remoteControl);
                    remote = new AmtRemoteControl(this, remoteControl);

                    // Event management for remote events (WSMAN enabled - PARTIALLY)
                    EventManagerService eventManagerService = new EventManagerService(HostNameIdentity.ToWebURL(Port));
                    ServiceSetup(eventManagerService);
                    events = new AmtEventManager(eventManagerService, this, false);

                    //Get User Consent states.
                    userConsent = new AmtUserConsent(this);

                    if (AmtRev >= 2 && WsManOnly == false && UserConsent.CurrentControlMode != AmtUserConsent.ControlModes.CLIENT_CONTROL_MODE)
                    {
                        CircuitBreakerService circuitBreakerService = new CircuitBreakerService(HostNameIdentity.ToWebURL(Port));
                        ServiceSetup(circuitBreakerService);
                        circuitBreaker = new AmtCircuitBreaker(circuitBreakerService, this);

                        // In some cases, Intel AMT does not support circuit breaker
                        try
                        {
                            circuitBreaker.GetCircuitBreakerFilters();
                        }
                        catch (WebException)
                        {
                            circuitBreaker = null;
                        }
                    }

                    // Check for connection interruption
                    if (State == AmtSystemObjState.Disconnected)
                    {
                        launchThread = null;
                        return;
                    }

                    // Watchdog Management
                    if (CircuitBreaker != null && WsManOnly == false)
                    {
                        AgentWatchdogRemoteService agentWatchdogRemoteService = new AgentWatchdogRemoteService(HostNameIdentity.ToWebURL(Port));
                        ServiceSetup(agentWatchdogRemoteService);
                        watchdog = new AmtWatchdogRemote(agentWatchdogRemoteService);
                    }

                    // Wireless Profiles Management (Mobile platforms only)
                    if (WsManOnly == false && Network != null && Network.Wireless != null)
                    {
                        WirelessConfigurationService wirelessService = new WirelessConfigurationService(HostNameIdentity.ToWebURL(Port));
                        ServiceSetup(wirelessService);
                        wireless = new AmtWireless(wirelessService, this);

                        // We have seen cases where Intel AMT will report 2.5 to 2.9, but the radio interface is missing (Desktop with Mobile parts???)
                        bool radioEnabled;
                        if (wireless.IsRadioOn(out radioEnabled) != AmtCallStatus.SUCCESS)
                            wireless = null;
                    }

                    // Setup Network Access Control
                    if (WsManOnly == false && (AmtRev > 2 || (AmtRev == 2 && AmtRevMin >= 5)))
                    {
                        //EndpointAccessControlService EAC = new EndpointAccessControlService(NetworkName + ":16992");
                        //ServiceSetup(EAC);
                        //NetworkAccessControl = new AmtNetworkAccessControl(this, EAC);
                        EndpointAccessControlAdminService EACA = new EndpointAccessControlAdminService(HostNameIdentity.ToWebURL(Port));
                        ServiceSetup(EACA);
                        networkAccessControlAdmin = new AmtNetworkAccessControlAdmin(this, EACA);
                    }

                    // Audit Log
                    if (AmtRev >= 4)
                    {
                        AuditLogService al = new AuditLogService(HostNameIdentity.ToWebURL(Port));
                        ServiceSetup(al);
                        auditLog = new AmtAuditLog(this, al);

                        // Check that Audit Log works
                        AmtCallStatus auditresponse = AmtCallStatus.FAILED_WEB_CALL;
                        try
                        {
                            AmtGeneralInfo.AmtAuditLogStatus auditStatus;
                            auditresponse = Info.GetAuditLogStatus(out auditStatus);
                        }
                        catch (Exception)
                        {
                        }
                        if (auditresponse != AmtCallStatus.SUCCESS)
                        {
                            auditLog = null;
                        }
                    }

                    // Storage Management (WSMAN enabled)
                    if (wsMan != null || AmtStorageWrapper.CheckStorageWrapper() == true)
                    {
                        StorageService storageService = new StorageService(HostNameIdentity.ToWebURL(Port));
                        StorageAdministrationService storageAdministrationService = new StorageAdministrationService(HostNameIdentity.ToWebURL(Port));
                        ServiceSetup(storageService);
                        ServiceSetup(storageAdministrationService);
                        storage = new AmtStorage(this, storageAdministrationService, storageService);
                    }

                    // Check for connection interruption
                    if (State == AmtSystemObjState.Disconnected)
                    {
                        launchThread = null;
                        return;
                    }

                    // Remote Access
                    if (AmtRev >= 4 || AmtRev == 0)
                    {
                        RemoteAccessAdminService raa = new RemoteAccessAdminService(HostNameIdentity.ToWebURL(Port));
                        ServiceSetup(raa);
                        remoteAccessAdmin = new AmtRemoteAccessAdmin(this, raa);

                        if (WsMan != null)
                        {
                            // Try to access the remote access policies, if fail, turn this feature off.
                            AmtRemoteAccessAdmin.AmtRemoteAccessPolicy[] remoteAccessPolicies;
                            AmtCallStatus r = AmtCallStatus.FAILED_WEB_CALL;
                            try
                            {
                                r = remoteAccessAdmin.GetRemoteAccessPolicies(out remoteAccessPolicies);
                            }
                            catch (Exception)
                            {
                            }
                            if (r != AmtCallStatus.SUCCESS)
                                remoteAccessAdmin = null;
                        }
                    }
                }

                // Check for connection interruption
                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }

                // Check for connection interruption
                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }

                if ( UseTls == true)
                {
                    // using TLS, lets check for warnings.

                    // If we are using mutual auth, we need to make sure OpenSSL is present
                    if (UseTls == true && mutualAuthCert != null)
                    {
                        Directory.SetCurrentDirectory(System.Windows.Forms.Application.StartupPath);
                        if (Directory.Exists(".\\OpenSSL") == false || File.Exists(".\\OpenSSL\\OpenSSL.exe") == false)
                            this.Warnings |= AmtConnectWarnings.OpenSslNotAvailable;
                    }

                    // Check mutual-auth cert has exportable key
                    if (mutualAuthCert != null && CertificateOperations.IsKeyExportable(mutualAuthCert) == false)
                    {
                        this.Warnings |= AmtConnectWarnings.AuthPrivateKeyNotExportable;
                    }

                    // Check for name mismatch
                    if (HostnameMatch(this) == false)
                    {
                        this.Warnings |= AmtConnectWarnings.NameMismatch;
                    }

                    // Check the certificate
                    X509Certificate2[] certs = null;
                    if (ServerCertificates[HostNameIdentity.DefaultName] != null)
                    {
                        certs = (X509Certificate2[])ServerCertificates[HostNameIdentity.DefaultName]; // Try to get actualy certificate chain used.
                    }
                    CertificateError = X509ChainStatusFlags.NoError;
                    if (certs == null || certs.Length == 0)
                    {
                        this.Warnings |= AmtConnectWarnings.CertificateInvalid;
                        CertificateError = X509ChainStatusFlags.PartialChain;
                    }
                    else
                    {
                        X509Chain chain = new X509Chain();
                        chain.ChainPolicy = new X509ChainPolicy();
                        chain.ChainPolicy.RevocationMode = X509RevocationMode.NoCheck;
                        for (int i = 1; i < certs.Length; i++)
                        {
                            chain.ChainPolicy.ExtraStore.Add(certs[i]);
                        }
                        if (chain.Build(certs[0]) == false)
                        {
                            foreach (X509ChainStatus s in chain.ChainStatus)
                            {
                                if (s.Status != X509ChainStatusFlags.NoError)
                                {
                                    CertificateError = s.Status;
                                    break;
                                }
                            }

                            bool errorSet = false;
                            if (chain.ChainStatus[0].Status == X509ChainStatusFlags.UntrustedRoot)
                            {
                                this.Warnings |= AmtConnectWarnings.CertificateUntrusted;
                                errorSet = true;
                            }
                            if (chain.ChainStatus[0].Status == X509ChainStatusFlags.NotTimeValid)
                            {
                                this.Warnings |= AmtConnectWarnings.CertificateExpired;
                                errorSet = true;
                            }

                            if (errorSet == false)
                                this.Warnings |= AmtConnectWarnings.CertificateInvalid;
                        }
                    }
                }


                // Check for connection interruption
                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }

                // Check for KVM Enabled
                kvm = new AmtKVM(this);

                ChangeState(AmtSystemObjState.Connected);
                connectlock = true;

                // Check for connection interruption
                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }

                if (AutoFetchCache == true)
                    FetchCache();
                if (OnFetchCompleted != null)
                    OnFetchCompleted(this, null);

                launchThread = null;
            }
            catch (ThreadInterruptedException)
            {
            }
        }
        #endregion

        #region Public Methods
        /// <summary>
        /// Clears the Web Service caches for the Intel(R) AMT interface objects.
        /// </summary>
        public void ClearCache()
        {
            if (Network != null)
                Network.ClearCache();
            if (Events != null)
                Events.ClearCache();
            if (Info != null)
                Info.ClearCache();
            if (Assets != null)
                Assets.ClearCache();
            if (Redirection != null)
                Redirection.ClearCache();
            if (SecurityAdmin != null)
                SecurityAdmin.ClearCache();
            if (Storage != null)
                Storage.ClearCache();
            if (Watchdog != null)
                Watchdog.ClearCache();
            if (CircuitBreaker != null)
                CircuitBreaker.ClearCache();
            if (Wireless != null)
                Wireless.ClearCache();
            if (NetworkAccessControl != null)
                NetworkAccessControl.ClearCache();
            if (NetworkAccessControlAdmin != null)
                NetworkAccessControlAdmin.ClearCache();
            if (AuditLog != null)
                AuditLog.ClearCache();
            if (NetworkTime != null)
                NetworkTime.ClearCache();
            if (RemoteAccessAdmin != null)
                RemoteAccessAdmin.ClearCache();

            IsWirelessSupportedCache = null;
        }

        /// <summary>
        /// Retrieves the DNS name for the system as defined by its hostname.
        /// </summary>
        public void FetchCacheDnsName()
        {
            if (State == AmtSystemObjState.Disconnected)
                return;
            GetDnsName();
        }

        /// <summary>
        /// Retrieves the Intel(R) AMT interface data into the local cache 
        /// objects.
        /// </summary>
        public void FetchCache()
        {
            new Thread(new ThreadStart(FetchCacheDnsName)).Start();
            try
            {
                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }
                if (Redirection != null)
                {
                    Redirection.FetchCache();
                    if (OnFetchCompleted != null && State != AmtSystemObjState.Disconnected)
                        OnFetchCompleted(this, Redirection);
                }

                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }
                if (Network != null)
                {
                    Network.FetchCache();
                    if (OnFetchCompleted != null && State != AmtSystemObjState.Disconnected)
                        OnFetchCompleted(this, Network);
                }

                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }
                if (Info != null)
                {
                    Info.FetchCache();
                    if (OnFetchCompleted != null && State != AmtSystemObjState.Disconnected)
                        OnFetchCompleted(this, Info);
                }

                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }
                if (SecurityAdmin != null)
                {
                    SecurityAdmin.FetchCache();
                    if (OnFetchCompleted != null && State != AmtSystemObjState.Disconnected)
                        OnFetchCompleted(this, SecurityAdmin);
                }

                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }
                if (Watchdog != null)
                {
                    Watchdog.FetchCache();
                    if (OnFetchCompleted != null && State != AmtSystemObjState.Disconnected)
                        OnFetchCompleted(this, Watchdog);
                }

                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }
                if (CircuitBreaker != null)
                {
                    CircuitBreaker.FetchCache();
                    if (OnFetchCompleted != null && State != AmtSystemObjState.Disconnected)
                        OnFetchCompleted(this, CircuitBreaker);
                }

                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }
                if (Wireless != null)
                {
                    Wireless.FetchCache();
                    if (OnFetchCompleted != null && State != AmtSystemObjState.Disconnected)
                        OnFetchCompleted(this, Wireless);
                }

                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }
                if (NetworkAccessControlAdmin != null)
                {
                    NetworkAccessControlAdmin.FetchCache();
                    if (OnFetchCompleted != null && State != AmtSystemObjState.Disconnected)
                        OnFetchCompleted(this, NetworkAccessControlAdmin);
                }

                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }
                if (RemoteAccessAdmin != null)
                {
                    RemoteAccessAdmin.FetchCache();
                    if (OnFetchCompleted != null && State != AmtSystemObjState.Disconnected)
                        OnFetchCompleted(this, RemoteAccessAdmin);
                }

                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }
                if (Assets != null)
                {
                    Assets.FetchCache();
                    if (OnFetchCompleted != null && State != AmtSystemObjState.Disconnected)
                        OnFetchCompleted(this, Assets);
                }

                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }
                if (Storage != null)
                {
                    Storage.FetchCache();
                    if (OnFetchCompleted != null && State != AmtSystemObjState.Disconnected)
                        OnFetchCompleted(this, Storage);
                }

                if (State == AmtSystemObjState.Disconnected)
                {
                    launchThread = null;
                    return;
                }
                if (Events != null)
                {
                    Events.FetchCache();
                    if (OnFetchCompleted != null && State != AmtSystemObjState.Disconnected)
                        OnFetchCompleted(this, Events);
                }
            }
            catch (Exception e)
            {
                Debug.Print("Exception while fetching system cache: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error, "Error fetching remote interface data for system: " + this.HostNameIdentity.DefaultName, e.Message);
            } // Any number of issues can comeup during cache load, disconnections are probably the most frequent.
        }

        /// <summary>
        /// This call pre-loads all of the Intel(R) AMT interface objects 
        /// needed to connect into memory. This will speed up the first 
        /// connection.
        /// </summary>
        public static void PreLoadModules()
        {
            AmtSystem computer = new AmtSystem("x", 1, "x", "x", false, true);
            AmtGeneralInfo info = new AmtGeneralInfo(computer, new GeneralInfoService("x"));
            AmtRemoteControl remote = new AmtRemoteControl(computer, new RemoteControlService("x"));
            AmtNetworkAdmin network = new AmtNetworkAdmin(computer, new NetworkAdministrationService("x"));
            AmtAssetManager assets = new AmtAssetManager(computer, new HardwareAssetService("x"));
            AmtEventManager events = new AmtEventManager(new EventManagerService("x"), computer, false);
            AmtRedirect redirection = new AmtRedirect(new RedirectionService("x"), computer);
            AmtStorage storage = new AmtStorage(computer, new StorageAdministrationService("x"), new StorageService("x"));
            AmtStorageWrapper storagewrapper = new AmtStorageWrapper(computer, "x", "x", "x", Guid.Empty);
            AmtWatchdogRemote watchdog = new AmtWatchdogRemote(new AgentWatchdogRemoteService("x"));
            AmtWatchdogLocal watchdogLocal = new AmtWatchdogLocal(computer, new AgentWatchdogLocalService("x"));
            AmtSecurityAdmin securityAdmin = new AmtSecurityAdmin(new SecurityAdministrationService("x"), computer);
            AmtCircuitBreaker circuitBreaker = new AmtCircuitBreaker(new CircuitBreakerService("x"), computer);
            AmtWireless wireless = new AmtWireless(new WirelessConfigurationService("x"), computer);
            AmtNetworkAccessControl networkAccessControl = new AmtNetworkAccessControl(computer, new EndpointAccessControlService("x"));
            AmtNetworkAccessControlAdmin networkAccessControlAdmin = new AmtNetworkAccessControlAdmin(computer, new EndpointAccessControlAdminService("x"));
            AmtRemoteAccessAdmin remoteAccessAdmin = new AmtRemoteAccessAdmin(computer, new RemoteAccessAdminService("x"));
            AmtNetworkTime networkTime = new AmtNetworkTime(new NetworkTimeService("x"), computer);
            AmtAuditLog auditLog = new AmtAuditLog(computer, new AuditLogService("x"));

            /*
            System.Reflection.Assembly myAssembly = System.Reflection.Assembly.GetAssembly(typeof(AMT_GeneralSettingsType));
            foreach (Type t in myAssembly.GetTypes())
            {
                if ((t.FullName.StartsWith("MOF_CLASSES.CIM_") || t.FullName.StartsWith("MOF_CLASSES.AMT_")) && t.FullName.EndsWith("Type"))
                {
                    t.GetConstructor(new Type[0]).Invoke(new object[0]);
                }
            }
            */
        }

        /// <summary>
        /// Retreives the X509 Certificate object with the given thumbprint.
        /// </summary>
        /// <param name="thumbprint">Thumbprint (hash) to match against.</param>
        /// <returns>If successful, returns the matching X509 Certificate object. 
        /// Otherwise, the return is null.</returns>
        public static X509Certificate2 GetCertificateFromThumbprint(string thumbprint)
        {
            if (thumbprint == null)
                return null;

            // Look in my store
            X509Store CertificateStore = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadOnly);
            X509Certificate2 certificate = null;
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (cert.Thumbprint.CompareTo(thumbprint) == 0)
                {
                    certificate = cert;
                    break;
                }
            }
            CertificateStore.Close();
            if (certificate != null)
                return certificate;

            // Look in root store
            CertificateStore = new X509Store(StoreName.Root, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadOnly);
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (cert.Thumbprint.CompareTo(thumbprint) == 0)
                {
                    certificate = cert;
                    break;
                }
            }
            CertificateStore.Close();
            return certificate;
        }

        /// <summary>
        /// Assigns the DNS name information to the local cache object of the 
        /// system.
        /// </summary>
        /// <param name="entry"></param>
        public void SetDnsEntryCache(IPHostEntry entry)
        {
            GetDnsEntryCache = entry;
            GetDnsEntryCacheValid = true;
        }

        /// <summary>
        /// Retrieves the DNS name information for the system.
        /// </summary>
        /// <returns></returns>
        public IPHostEntry GetDnsEntry()
        {
            if (GetDnsEntryCacheValid == true)
                return GetDnsEntryCache;
            IPHostEntry entry = null;
            try
            {
                entry = System.Net.Dns.GetHostEntry(HostNameIdentity.DefaultName);
            }
            catch (Exception)
            {
            }
            if (entry != null)
                GetDnsEntryCache = entry;
            else
                GetDnsEntryCache = null;
            GetDnsEntryCacheValid = true;
            return GetDnsEntryCache;
        }

        /// <summary>
        /// Retrieves the DNS hostname of the Intel(R) AMT system.
        /// </summary>
        /// <returns></returns>
        public string GetDnsName()
        {
            IPHostEntry e = GetDnsEntry();
            if (e == null)
                return "";
            return e.HostName;
        }

        /// <summary>
        /// Used to check if the hostname of a system matches correctly.
        /// </summary>
        /// <param name="system"></param>
        /// <returns></returns>
        public static bool HostnameMatch(AmtSystem system)
        {
            // If TLS is being used, get the certificate name.
            if (system.UseTls == true)
            {
                X509Certificate2[] certs = system.GetTlsCertificates();
                if (certs == null || certs.Length == 0)
                    return false;

                string[] certificateNames = CertificateOperations.GetCertificateCommonNames(certs[0]);
                foreach (string name in certificateNames)
                {
                    if (string.Compare(system.HostNameIdentity.DisplayName, name, true) == 0)
                        return true;
                }
                return false;
            }

            return true;
        }

        /// <summary>
        /// Defines the Management Presence Server (MPS) for the network.
        /// </summary>
        /// <param name="hostname">IP address or FQDN of the MPS.</param>
        /// <param name="httpport">HTTP port number to use when connecting to the MPS.</param>
        /// <param name="socksport">SOCKS port number to use when connecting to the MPS.</param>
        public void SetMPS(string hostname, ushort httpport, ushort socksport)
        {
            SetMPS(hostname, httpport, socksport, null, null);
        }

        /// <summary>
        /// Defines the Management Presence Server (MPS) for the network.
        /// </summary>
        /// <param name="hostname">IP address or FQDN of the MPS.</param>
        /// <param name="httpport">HTTP port number to use when connecting to the MPS.</param>
        /// <param name="socksport">SOCKS port number to use when connecting to the MPS.</param>
        /// <param name="username">Username for the connection.</param>
        /// <param name="password">Password for the account used to connect to the MPS.</param>
        public void SetMPS(string hostname, ushort httpport, ushort socksport, string username, string password)
        {
            NetworkCredential nc = null;
            if (username != null && password != null)
            {
                nc = new NetworkCredential(username, password);
            }
            HttpProxy = new WebProxy(hostname + ":" + httpport.ToString(), false, null, nc);

            SocksProxy = new AmtRedirectorWrapper.SOCKSProxy();
            SocksProxy.server = hostname;
            SocksProxy.port = socksport;
            SocksProxy.user = username;
            SocksProxy.password = password;
        }

        /// <summary>
        /// Removes the MPS proxy information.
        /// </summary>
        public void ClearMPS()
        {
            HttpProxy = null;
            SocksProxy = null;
        }
        #endregion
    }

    /// <summary>
    /// Represents the persistent state of an AmtSystem object that can be 
    /// serialized.
    /// </summary>
    [Serializable]
    public class AmtSerialSystemSettings
    {
        public string Hostname;
        public int Port;
        public string Username;
        public string Password;
        public string KVMRfbPassword;
        public bool Tls;
        public bool mTls;
        public Guid PlatformId;
        public uint AmtRev = 0;
        public uint AmtRevMin = 0;
        public string ProxyHost;
        public int ProxyPort;
        public ForceIPOption ForceIPOption = ForceIPOption.UseDefault;
        public AmtRedirectorWrapper.SOCKSProxy SocksProxy;
        public string IpAddr;
        public bool MPS; 
        public bool MyComputer;

        public AmtSerialSystemSettings()
        {
        }

        public AmtSerialSystemSettings(AmtSystem s)
        {
            MPS = false;
            Hostname = s.HostNameIdentity.DisplayName;
            Port = s.Port;
            Username = s.Username;
            Password = s.Password;
            KVMRfbPassword = s.KVMRfbPassword;
            MyComputer = s.MyComputer; 
            Tls = s.UseTls;
            mTls = s.UseMutualTls;
            PlatformId = s.PlatformId;
            AmtRev = s.AmtRev;
            AmtRevMin = s.AmtRevMin;
            ForceIPOption = s.HostNameIdentity.ForceIPOption;
            if (s.HttpProxy != null)
            {
                ProxyHost = s.HttpProxy.Address.Host;
                ProxyPort = s.HttpProxy.Address.Port;
            }
            SocksProxy = s.SocksProxy;
        }

        public AmtSerialSystemSettings(AmtMpsServer m)
        {
            MPS = true;
            Hostname = "~~MPS~~";
            Username = m.Serialize();
        }

        public AmtSystem GetSystem()
        {
            if (Password == null)
            {
                if (String.IsNullOrEmpty(IpAddr))
                    return new AmtSystem(Hostname, Port, Username, null, false, false, ForceIPOption, null);
                else
                    return new AmtSystem(Hostname, Port, Username, null, false, false, ForceIPOption, IPAddress.Parse(IpAddr));
            }

            AmtSystem s = null;
            if (String.IsNullOrEmpty(IpAddr))
                s = new AmtSystem(Hostname, Port, Username, Password, true, false, ForceIPOption, null);
            else
                s = new AmtSystem(Hostname, Port, Username, Password, true, false, ForceIPOption, IPAddress.Parse(IpAddr));
            s.PlatformId = PlatformId;
            s.AmtRev = AmtRev;
            s.AmtRevMin = AmtRevMin;
            s.UseTls = Tls;
            s.UseMutualTls = mTls;
            s.KVMRfbPassword = KVMRfbPassword;
            s.MyComputer = MyComputer; 
            if (ProxyHost != null)
                s.HttpProxy = new WebProxy(ProxyHost, ProxyPort);
            s.SocksProxy = SocksProxy;
            return s;
        }

        public AmtMpsServer GetMPS()
        {
            if (Hostname.Equals("~~MPS~~") == false)
                return null;
            return AmtMpsServer.UnSerialize(Username);
        }

    }
}
