// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Net;
using System.Net.Sockets;

namespace ManageabilityStack
{
    public class AmtTrapListener
    {
        private UdpClient udpClient;
        private UdpClient udpIpv6Client;
        private int refcount = 0;
        public delegate void AmtNetworkEventHandler(IPEndPoint endpoint, AmtAlert alert);
        public delegate void AmtNetworkEventDebugHandler(AmtAlert alert, string debug1, string debug2);
        public event AmtNetworkEventHandler AmtNetworkEvent;
        public event AmtNetworkEventDebugHandler AmtNetworkDebugEvent;
        public static AmtTrapListener TrapListener = new AmtTrapListener();

        public AmtTrapListener()
        {
        }

        public bool Start()
        {
            if (++refcount == 1)
            {
                Socket ipv4Socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
                ipv4Socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
                ipv4Socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ExclusiveAddressUse, false);

                try
                {
                    ipv4Socket.Bind(new IPEndPoint(IPAddress.Any, 162));
                }
                catch (SocketException)
                {
                    refcount = 0;
                    udpClient = null;
                    return false;
                }

                udpClient = new UdpClient();
                udpClient.Client = ipv4Socket;
                udpClient.BeginReceive(new AsyncCallback(ReceiveSink), udpClient);

                if (Socket.OSSupportsIPv6)
                {
                    Socket ipv6Socket = new Socket(AddressFamily.InterNetworkV6, SocketType.Dgram, ProtocolType.Udp);
                    ipv6Socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);
                    ipv6Socket.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ExclusiveAddressUse, false);

                    try
                    {
                        ipv6Socket.Bind(new IPEndPoint(IPAddress.IPv6Any, 162));
                    }
                    catch (SocketException e)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Warning, "Exception while listening for IPv6 alerts",
                            String.Format("EXCEPTION: {0} (type {1}).\n\nEXCEPTION STACK: {2}", e.Message, e.GetType(), e.StackTrace));
                        udpIpv6Client = null;
                        return true;
                    }

                    udpIpv6Client = new UdpClient(AddressFamily.InterNetworkV6);
                    udpIpv6Client.Client = ipv6Socket;
                    udpIpv6Client.BeginReceive(new AsyncCallback(ReceiveSink), udpIpv6Client);
                }
            }
            return true;
        }

        public void Stop()
        {
            if (--refcount == 0)
            {
                udpClient.Close();
                udpClient = null;

                if (udpIpv6Client != null)
                {
                    udpIpv6Client.Close();
                    udpIpv6Client = null;
                }
            }
        }

        private void ReceiveSink(IAsyncResult result)
        {
            // Retrieve the associated UdpClient for this callback.
            UdpClient assocUdpClient = result.AsyncState as UdpClient;
            if (assocUdpClient == null) return;

            IPEndPoint ep = null;
            byte[] buf;
            try
            {
                buf = assocUdpClient.EndReceive(result, ref ep);
            }
            catch (ObjectDisposedException) { return; }

            AmtAlert alert = null;
            BerSEQUENCE s = null;
            try
            {
                alert = AmtAlert.DecodeAlert(buf, 0, buf.Length);
                s = BerSEQUENCE.DecodeBER(buf, 0, buf.Length);
            }
            catch (Exception ex)
            {
                // Exception occured while parsing this alert. Lets dump it to "exception.txt".
                AmtDebugForm.LogException(this, ex);
                return;
            }

            if (alert != null)
            {
                string d = "";
                d += "-- ALERT --\r\n";
                d += "Received Time: " + alert.ReceivedTime.ToString() + "\r\n";
                if (alert.Community != null) d += "Trap Community: " + alert.Community + "\r\n";
                d += "Platform GUID: " + alert.PlatformGuid.ToString() + "\r\n";
                d += "Sequence Number: " + alert.SequenceNumber.ToString() + "\r\n";
                if (alert.TrapSource != null) d += "Trap Source: " + alert.TrapSource.ToString() + "\r\n";
                d += "Trap Version: " + alert.TrapVersion.ToString() + "\r\n";
                d += "Time Stamp 1: " + alert.TimeStamp.ToString() + "\r\n";
                d += "Time Stamp 2: " + alert.TimeStamp2.ToString() + "\r\n";
                d += "UTC Offset: " + alert.UtcOffset.ToString() + "\r\n";

                if (alert.amtEvent != null)
                {
                    d += "\r\n-- EVENT INFORMATION --\r\n";
                    d += "Device Address: " + alert.amtEvent.DeviceAddress.ToString() + "\r\n";
                    d += "Entity Type: " + alert.amtEvent.EntityTypeStr + "\r\n";
                    d += "Event Offset: " + alert.amtEvent.EventOffsetStr + "\r\n";
                    d += "Event Sensor Type: " + alert.amtEvent.EventSensorTypeStr + "\r\n";
                    d += "Event Severity: " + alert.amtEvent.EventSeverityStr + "\r\n";
                    d += "Event Source: " + alert.amtEvent.EventSourceStr + "\r\n";
                    d += "Event Type: " + alert.amtEvent.EventTypesStr + "\r\n";
                    d += "Event Data: " + alert.amtEvent.EventDataStr.ToString() + "\r\n";
                }

                d += "\r\n-- BER DECODER --\r\n\r\n" + s.GetDebugString();

                if (AmtNetworkDebugEvent != null) AmtNetworkDebugEvent(alert, s.ToString(), d);

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "Received Event Trap from " + ep.ToString() + ". Trap is " + buf.Length.ToString() + " bytes long: " + s.ToString());
                if (alert != null && AmtNetworkEvent != null) AmtNetworkEvent(ep, alert);
            }
            else
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, "Unparsable alert from: " + ep.ToString() + ", HEX: " + Utils.BytesToHex(buf));
            }

            assocUdpClient.BeginReceive(new AsyncCallback(ReceiveSink), assocUdpClient);
        }

        public void DebugInject(AmtAlert alert, string s1, string s2)
        {
            if (AmtNetworkDebugEvent != null) AmtNetworkDebugEvent(alert, s1, s2);
            if (alert != null && AmtNetworkEvent != null) AmtNetworkEvent(new IPEndPoint(alert.TrapSource, 0), alert);
        }

    }
}
