﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;

namespace ManageabilityStack
{
    public class AmtUserConsent
    {
        public enum OptInRequiredValues : uint
        {
            None = 0,
            KVM = 1,
            All = 4294967295
        }
        public enum OptInStates : uint
        {
            NotStarted = 0,
            Requested,
            Displayed,
            Received,
            InSession
        }

        public enum ControlModes
        {
            NOT_PROVISIONED = 0,
            CLIENT_CONTROL_MODE = 1,
            ADMIN_CONTROL_MODE = 2,
            UNKNOWN = 255
        }

        public enum AllowedModes
        {
            ADMIN_ONLY = 1,
            CLIENT_AND_ADMIN = 2,
            UNKNOWN = 255
        }
        
        private AmtSystem _system;
        private IPS_OptInService _service;
        private bool _cached = false;

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="system"></param>
        public AmtUserConsent(AmtSystem system)
        {   
            this._system = system;
        }

        public void ClearCache()
        {
            _cached = false;
            _optInRequired = null;
            _optInState = null;
            _canModifyOptInPolicy = false;
            _CurrentControlMode = null;
            _SupportedControlModes = null;
        }

        private void fetchCache()
        {
            if (!_cached)
            {
                IPS_OptInService service = getOptInService();
                try
                {
                    service.Get();
                    _optInSupported = true; 
                    _optInRequired = (OptInRequiredValues)(service.OptInRequired);
                    _optInState = (OptInStates)(service.OptInState);
                    _canModifyOptInPolicy = (service.CanModifyOptInPolicy == 1);
                   

                    IPS_HostBasedSetupService hostBasedService = new IPS_HostBasedSetupService(_system.WsMan);
                    hostBasedService.Get();
                    _CurrentControlMode = (ControlModes)hostBasedService.CurrentControlMode;

                    byte[] SupportedModes = hostBasedService.AllowedControlModes;

                    if (SupportedModes.Length == 2)
                        _SupportedControlModes = AllowedModes.CLIENT_AND_ADMIN;
                    else if (SupportedModes.Length == 1)
                        _SupportedControlModes = AllowedModes.ADMIN_ONLY;
                    else
                        _SupportedControlModes = AllowedModes.UNKNOWN;
                }
                #region catch block
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.fetchCache WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.fetchCache CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.fetchCache",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                    _optInSupported = false;
                    _CurrentControlMode = ControlModes.UNKNOWN; 
                    return;
                }
                #endregion
                finally
                {
                    _cached = true;
                }
            }

        }

        public bool SendUserCode(uint code)
        {
            try
            {
                IPS_OptInService service = getOptInService();
                uint response = service.SendOptInCode(code);
                ClearCache();
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "Submitted code for user consent.",
                    string.Format("Code = {0}", code));
                return (response == 0);
            }
            #region catch block
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.SendUserCode WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.SendUserCode CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.SendUserCode",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                return false;
            }
            #endregion
        }

        public bool StartOptIn()
        {
            try
            {
                IPS_OptInService service = getOptInService();
                uint retVal = service.StartOptIn();
                ClearCache();
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "Requested user consent.", string.Empty);
                return (retVal == 0);
            }
            #region catch block
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.StartOptIn WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.StartOptIn CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.StartOptIn",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                return false;
            }
            #endregion
        }

        public bool CancelOptIn()
        {
            try
            {
                IPS_OptInService service = getOptInService();
                uint retVal = service.CancelOptIn();
                ClearCache();
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "User consent (or request) cancelled.", string.Empty);
                return retVal == 0;
            }
            #region catch block
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.CancelOptIn WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.CancelOptIn CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.CancelOptIn",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                return false;
            }
            #endregion
            
        }

        public bool SwitchSpriteMonitor()
        {
            try
            {
                IPS_SecIOService service = new IPS_SecIOService(_system.WsMan);
                service.Get();
                byte curDefaultScreen = service.DefaultScreen;
                byte switchTo = (curDefaultScreen == 0) ? (byte)1 : (byte)0;
                service.DefaultScreen = switchTo;
                service.Put();
                return true;
            }
            #region catch block
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.SwitchSpriteMonitor WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.SwitchSpriteMonitor CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.SwitchSpriteMonitor",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                return false;
            }
            #endregion
        }

        private bool? _optInSupported = null;
        public bool? OptInSupported
        {
            get
            {
                fetchCache();
                return _optInSupported;
            }
        }


        private OptInRequiredValues? _optInRequired = null;
        public OptInRequiredValues? OptInRequired
        {
            get
            {
                fetchCache();
                return _optInRequired;
            }
            set
            {
                try
                {
                    IPS_OptInService service = getOptInService();
                    service.Get();
                    service.OptInRequired = (uint)(value);
                    service.Put();
                    ClearCache();
                }
                #region catch block
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.OptInRequired WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.OptInRequired CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.OptInRequired",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                }
                #endregion
            }
        }

        private bool _canModifyOptInPolicy = false;
        public bool CanModifyOptInPolicy
        {
            get
            {
                fetchCache();
                return _canModifyOptInPolicy;
            }
        }

        private OptInStates? _optInState = null;
        public OptInStates? OptInState
        {
            get
            {
                fetchCache();
                return _optInState;
            }
        }

        private ControlModes? _CurrentControlMode = null;
        public ControlModes? CurrentControlMode
        {
            get
            {
                fetchCache();
                return _CurrentControlMode;
            }
        }

        private AllowedModes? _SupportedControlModes = null;
        public AllowedModes? SupportedControlModes
        {
            get
            {
                fetchCache();
                return _SupportedControlModes; 

            }
        }

        public bool DisableClientControlMode()
        {
            try
            {
                IPS_HostBasedSetupService hostBasedService = new IPS_HostBasedSetupService(_system.WsMan);
                hostBasedService.Get();

                bool retVal = hostBasedService.DisableClientControlMode() == 0;

                if (retVal)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "Client control mode disabled.", string.Empty);
                }
                else
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Information, "Client control mode failed to disable.", string.Empty);
                }

                return retVal;
            }

            #region catch block
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.DisableClientControlMode WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.DisableClientControlMode CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtUserConsent.DisableClientControlMode",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                return false; 
            }
            #endregion
        }

        private IPS_OptInService getOptInService()
        {
            if (_service == null)
            {
                _service = new IPS_OptInService(_system.WsMan);
            }
            return _service;
        }


    }
}
