// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Diagnostics;
using System.Threading;
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.Exceptions;
using Intel.Manageability.WSManagement;

namespace ManageabilityStack
{

    public class AmtWatchdogLocal
    {
        public class AmtWatchdog
        {
            public Guid agentID;
            public uint sessionSequenceNumber = 0;
            public ushort agentHeartbeatTime = 10;
            public string name;
            public string processCondition;
            public bool beat;
            public bool Registered;
            public AmtCallStatus lastState;
            public System.DateTime nextBeat;

            public AmtWatchdog(string agentname, Guid agentID)
            {
                this.agentID = agentID;
                this.name = agentname;
                this.beat = true;
                this.Registered = false;
                this.nextBeat = System.DateTime.MinValue;
                this.agentHeartbeatTime = 10;
            }

            public override string ToString()
            {
                return name;
            }
        }

        public delegate void WatchdogHeartbeatHandler(AmtSystem system, AmtWatchdogLocal localAgent, AmtWatchdog watchdog, uint sequenceNumber, AmtCallStatus result);
        public event WatchdogHeartbeatHandler WatchdogHeartbeat;

        public delegate void NotificationHandler(AmtSystem system, int notificationCode);
        public event NotificationHandler Notification;

        private AgentWatchdogLocalService agentWatchdogLocalService;
        private Hashtable watchdogs = new Hashtable();
        private Thread timerThread = null;
        private Guid OutpostAgentGuid = new Guid("C0770F68-9174-479b-87A5-A821FDFEF3C6");
        private Guid WatchdogNotificationGuid = new Guid("C0770F68-9174-479b-87A5-A821FDFEF3C7");
        private Guid CpuUsageNotificationGuid = new Guid("C0770F68-9174-479b-87A5-A821FDFEF3C8");
        private AmtSystem system;
        private const int HeartBeatLeadTime = 5; // Number of seconds before timeout that we shoudl heartbeat.

        public AmtWatchdogLocal(AmtSystem system, AgentWatchdogLocalService service)
        {
            this.system = system;
            agentWatchdogLocalService = service;
        }

        public void Dispose()
        {
            Stop();
            ArrayList l;
            lock (watchdogs)
            {
                l = new ArrayList(watchdogs.Values);
            }
            foreach (AmtWatchdog watchDog in l)
            {
                if (watchDog.Registered == true) AgentWatchdogShutdown(watchDog.agentID, watchDog.sessionSequenceNumber);
            }
            watchdogs.Clear();
            agentWatchdogLocalService.Dispose();
            agentWatchdogLocalService = null;
        }

        public void AddWatchdog(AmtWatchdog watchDog)
        {
            lock (watchdogs)
            {
                watchdogs[watchDog.agentID] = watchDog;
            }
            ushort bt;
            AmtCallStatus r = AgentWatchdogRegister(watchDog.agentID, out watchDog.sessionSequenceNumber, out bt);
            if (r == AmtCallStatus.SUCCESS)
            {
                watchDog.agentHeartbeatTime = bt;
                watchDog.Registered = true;
            }
        }

        public void RemoveWatchdog(AmtWatchdog watchDog)
        {
            lock (watchdogs)
            {
                watchdogs.Remove(watchDog.agentID);
            }
            AgentWatchdogShutdown(watchDog.agentID, watchDog.sessionSequenceNumber);
            watchDog.sessionSequenceNumber = 0;
            watchDog.Registered = false;
        }

        public void RemoveWatchdog(Guid watchdogId)
        {
            AmtWatchdog watchDog = (AmtWatchdog)watchdogs[watchdogId];
            if (watchDog != null) RemoveWatchdog(watchDog);
        }

        public AmtWatchdog GetWatchdog(Guid watchdogId)
        {
            return (AmtWatchdog)watchdogs[watchdogId];
        }

        public AmtWatchdog[] GetWatchdogs()
        {
            // Build a list of all of the watchdogs.
            ArrayList l = new ArrayList(watchdogs.Values);

            // Remove the two "hidden" watchdogs.
            AmtWatchdog w1 = null;
            AmtWatchdog w2 = null;
            AmtWatchdog w3 = null;
            foreach (AmtWatchdog w in l)
            {
                if (w.agentID.Equals(OutpostAgentGuid) == true) w1 = w;
                if (w.agentID.Equals(WatchdogNotificationGuid) == true) w2 = w;
                if (w.agentID.Equals(CpuUsageNotificationGuid) == true) w3 = w;
            }
            if (w1 != null) l.Remove(w1);
            if (w2 != null) l.Remove(w2);
            if (w3 != null) l.Remove(w3);

            // Return the rest.
            return (AmtWatchdog[])(l.ToArray(typeof(AmtWatchdog)));
        }

        public void RemoveAllWatchdogs()
        {
            AmtWatchdog[] wgs = GetWatchdogs();
            foreach (AmtWatchdog wg in wgs)
            {
                RemoveWatchdog(wg);
            }
        }

        public AmtCallStatus AgentWatchdogHeartbeat(Guid agentID, uint sessionSequenceNumber)
        {
            if (system.WsMan != null)
            {
                try
                {
                    // WSMAN
                    AMT_AgentPresenceWatchdog agentPresenceWatchdog = new AMT_AgentPresenceWatchdog(system.WsMan);
                    agentPresenceWatchdog.DeviceID = Convert.ToBase64String(agentID.ToByteArray());
                    return (AmtCallStatus)agentPresenceWatchdog.AssertPresence(sessionSequenceNumber);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWatchdogLocal.AgentWatchdogHeartbeat WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWatchdogLocal.AgentWatchdogHeartbeat CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWatchdogLocal.AgentWatchdogHeartbeat",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                try
                {
                    return (AmtCallStatus)agentWatchdogLocalService.AgentWatchdogHeartbeat(agentID.ToByteArray(), sessionSequenceNumber);
                }
                catch (Exception)
                {
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
        }

        public AmtCallStatus AgentWatchdogRegister(Guid agentID, out uint sessionSequenceNumber, out ushort agentHeartbeatTime)
        {
            if (system.WsMan != null)
            {
                try
                {
                    // WSMAN
                    AMT_AgentPresenceWatchdog agentPresenceWatchdog = new AMT_AgentPresenceWatchdog(system.WsMan);
                    agentPresenceWatchdog.DeviceID = Convert.ToBase64String(agentID.ToByteArray());
                    return (AmtCallStatus)agentPresenceWatchdog.RegisterAgent(out sessionSequenceNumber, out agentHeartbeatTime);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWatchdogLocal.AgentWatchdogRegister WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWatchdogLocal.AgentWatchdogRegister CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWatchdogLocal.AgentWatchdogRegister",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    sessionSequenceNumber = 0;
                    agentHeartbeatTime = 0;
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                try
                {
                    AmtCallStatus r = (AmtCallStatus)agentWatchdogLocalService.AgentWatchdogRegister(agentID.ToByteArray(), out sessionSequenceNumber, out agentHeartbeatTime);
                    return r;
                }
                catch (Exception)
                {
                    sessionSequenceNumber = 0;
                    agentHeartbeatTime = 0;
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
        }

        public AmtCallStatus AgentWatchdogShutdown(Guid agentID, uint sessionSequenceNumber)
        {
            if (system.WsMan != null)
            {
                try
                {
                    // WSMAN
                    AMT_AgentPresenceWatchdog agentPresenceWatchdog = new AMT_AgentPresenceWatchdog(system.WsMan);
                    agentPresenceWatchdog.DeviceID = Convert.ToBase64String(agentID.ToByteArray());
                    return (AmtCallStatus)agentPresenceWatchdog.AssertShutdown(sessionSequenceNumber);
                }
                catch (Exception e)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWatchdogLocal.AgentWatchdogShutdown",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                try
                {
                    return (AmtCallStatus)agentWatchdogLocalService.AgentWatchdogShutdown(agentID.ToByteArray(), sessionSequenceNumber);
                }
                catch (Exception)
                {
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
        }

        public void Start()
        {
            Stop();

            //AddWatchdog(new AmtWatchdog(Properties.Resources.ManageabilityOutpostToolWatchdogName, OutpostAgentGuid));
            //AddWatchdog(new AmtWatchdog(Properties.Resources.NotificationWatchdogName, WatchdogNotificationGuid));
            //AddWatchdog(new AmtWatchdog(Properties.Resources.CpuUsageWatchdogName, CpuUsageNotificationGuid));

            //cpuCounter = new PerformanceCounter();
            //cpuCounter.CategoryName = "Processor";
            //cpuCounter.CounterName = "% Processor Time";
            //cpuCounter.InstanceName = "_Total";

            timerThread = new Thread(new ThreadStart(TimerThread));
            timerThread.Start();
        }

        public void Stop()
        {
            if (timerThread != null)
            {
                timerThread.Interrupt();
                timerThread = null;
            }
        }

        private void TimerThread()
        {
            try
            {
                while (timerThread != null)
                {
                    Process[] procList = Process.GetProcesses();

                    ArrayList l;
                    lock (watchdogs)
                    {
                        l = new ArrayList(watchdogs.Values);
                    }

                    foreach (AmtWatchdog wg in l)
                    {
                        if (wg.beat == true)
                        {
                            if (wg.nextBeat.CompareTo(System.DateTime.Now) <= 0)
                            {
                                bool match = false;
                                if (wg.processCondition != null)
                                {
                                    foreach (Process p in procList)
                                    {
                                        if (p.ProcessName.CompareTo(wg.processCondition) == 0)
                                        {
                                            match = true;
                                            break;
                                        }
                                    }
                                }

                                //if (wg.agentID == this.CpuUsageNotificationGuid)
                                //{
                                //    try
                                //    {
                                //        match = (cpuCounter != null) && (cpuCounter.NextValue() >= 30);
                                //    }
                                //    catch (System.UnauthorizedAccessException)
                                //    {
                                //        // In cases where we are not autorized to read this value.
                                //        cpuCounter = null;
                                //        match = false;
                                //    }
                                //    catch (System.InvalidOperationException)
                                //    {
                                //        // In some cases, this counter does not exist.
                                //        cpuCounter = null;
                                //        match = false;
                                //    }
                                //}

                                if ((wg.processCondition == null && wg.agentID != this.CpuUsageNotificationGuid) || match == true)
                                {
                                    if (wg.Registered == true)
                                    {
                                        AmtCallStatus r = AgentWatchdogHeartbeat(wg.agentID, wg.sessionSequenceNumber);
                                        wg.lastState = r;
                                        if (WatchdogHeartbeat != null) WatchdogHeartbeat(system, this, wg, wg.sessionSequenceNumber, r);
                                        if (r == AmtCallStatus.INVALID_HANDLE)
                                        {
                                            wg.Registered = false;
                                            wg.agentHeartbeatTime = 10;
                                        }
                                        wg.sessionSequenceNumber++;
                                        wg.nextBeat = System.DateTime.Now.AddSeconds(wg.agentHeartbeatTime - HeartBeatLeadTime);
                                    }
                                    else
                                    {
                                        ushort bt;
                                        AmtCallStatus r = AgentWatchdogRegister(wg.agentID, out wg.sessionSequenceNumber, out bt);
                                        if (r == AmtCallStatus.SUCCESS) wg.agentHeartbeatTime = bt;
                                        if (wg.agentID.Equals(WatchdogNotificationGuid) == true)
                                        {
                                            if ((wg.lastState == AmtCallStatus.INVALID_HANDLE || wg.lastState == AmtCallStatus.FAILED_WEB_CALL) && r == AmtCallStatus.SUCCESS)
                                            {
                                                // This is a notification from the console, handle it!
                                                if (Notification != null) Notification(system, wg.agentHeartbeatTime);
                                            }
                                            wg.lastState = r;
                                            if (r == AmtCallStatus.SUCCESS) wg.Registered = true;
                                        }
                                        else
                                        {
                                            wg.lastState = r;
                                            if (WatchdogHeartbeat != null) WatchdogHeartbeat(system, this, wg, wg.sessionSequenceNumber, r);
                                            if (r == AmtCallStatus.SUCCESS) wg.Registered = true;
                                        }
                                        wg.nextBeat = System.DateTime.Now.AddSeconds(wg.agentHeartbeatTime - HeartBeatLeadTime);
                                    }
                                }

                                if (wg.processCondition != null && match == false)
                                {
                                    wg.lastState = AmtCallStatus.NO_PROCESS;
                                    if (WatchdogHeartbeat != null) WatchdogHeartbeat(system, this, wg, wg.sessionSequenceNumber, AmtCallStatus.NO_PROCESS);
                                }
                            }
                        }
                        else
                        {
                            wg.lastState = AmtCallStatus.CALL_SKIPPED;
                            if (WatchdogHeartbeat != null) WatchdogHeartbeat(system, this, wg, wg.sessionSequenceNumber, AmtCallStatus.CALL_SKIPPED);
                        }
                    }
                    Thread.Sleep(2000);
                }
            }
            catch (ThreadInterruptedException) { }
            timerThread = null;
        }
    }
}
