// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Diagnostics;

namespace ManageabilityStack
{
    public class AmtWatchdogRemote
    {
        private AgentWatchdogRemoteService service;

        public AmtWatchdogRemote(AgentWatchdogRemoteService service)
        {
            this.service = service;
        }

        public void Dispose()
        {
            service = null;
        }

        public void ClearCache()
        {
            agentCache = null;
            HwPoliciesCache = null;
            QueryCapabilitiesCache = null;
        }

        public void FetchCache()
        {
            try
            {
                GetAgents();
                GetCbPolicy();
                QueryCapabilities();
            }
            catch (Exception e)
            {
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Failure to retrieve the Remote Watchdog data.",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));            }
        }

        public void Add(AmtWatchdogAgent agent)
        {
            agentCache = null;
            AmtCallStatus r = (AmtCallStatus)service.ConsoleWatchdogCreate(agent.AgentId.ToByteArray(), agent.Description, agent.HeartbeatTime, agent.StartupTime);
        }

        public void Delete(AmtWatchdogAgent agent)
        {
            agentCache = null;
            AmtCallStatus r = (AmtCallStatus)service.ConsoleWatchdogDelete(agent.AgentId.ToByteArray());
        }

        private AmtWatchdogAgent[] agentCache = null;
        public AmtWatchdogAgent[] GetAgents()
        {
            if (agentCache != null) return agentCache;

            ConsoleWatchdogEntryType[] consoleWatchdogEntries = null;
            AmtCallStatus r = (AmtCallStatus)service.ConsoleWatchdogEnumerate(out consoleWatchdogEntries);
            if (consoleWatchdogEntries == null || consoleWatchdogEntries.Length == 0)
            {
                agentCache = new AmtWatchdogAgent[0];
                return null;
            }

            AmtWatchdogAgent[] list = new AmtWatchdogAgent[consoleWatchdogEntries.Length];
            for (int i=0;i<consoleWatchdogEntries.Length;i++)
            {
                list[i] = new AmtWatchdogAgent(consoleWatchdogEntries[i]);
            }
            
            agentCache = list;
            return agentCache;
        }

        public void UpdateAgent(ref AmtWatchdogAgent agent)
        {
            agentCache = null;
            AmtWatchdogAgent[] agents = GetAgents();

            if (agents == null) return;
            foreach (AmtWatchdogAgent a in agents)
            {
                if (a.AgentId == agent.AgentId)
                {
                    agent = a;
                    return;
                }
            }

            return;
        }

        public ConsoleWatchdogActionType[] GetActions(AmtWatchdogAgent agent)
        {
            ConsoleWatchdogActionType[] ConsoleWatchdogActions = null;
            AmtCallStatus r = (AmtCallStatus)service.ConsoleWatchdogGetActions(agent.AgentId.ToByteArray(), out ConsoleWatchdogActions);
            return ConsoleWatchdogActions;
        }

        private CircuitBreakerWGHardwarePolicyType[] HwPoliciesCache = null;
        public CircuitBreakerWGHardwarePolicyType[] GetCbPolicy()
        {
            if (HwPoliciesCache != null) return HwPoliciesCache;
            CircuitBreakerWGHardwarePolicyType[] HwPolicies = null;
            AmtCallStatus r = (AmtCallStatus)service.ConsoleWatchdogGetCbPolicy(out HwPolicies);
            HwPoliciesCache = HwPolicies;
            return HwPoliciesCache;
        }

        public AmtCallStatus SetCbPolicy(CircuitBreakerWGHardwarePolicyType[] HwPolicies)
        {
            HwPoliciesCache = null;
            return (AmtCallStatus)service.ConsoleWatchdogSetCbPolicy(HwPolicies);
        }

        private AgentPresenceCapabilitiesType QueryCapabilitiesCache = null;
        public AgentPresenceCapabilitiesType QueryCapabilities()
        {
            if (QueryCapabilitiesCache != null) return QueryCapabilitiesCache;
            AgentPresenceCapabilitiesType capabilities = null;
            AmtCallStatus r = (AmtCallStatus)service.ConsoleWatchdogQueryCapabilities(out capabilities);
            QueryCapabilitiesCache = capabilities;
            return QueryCapabilitiesCache;
        }

        public void SetActions(AmtWatchdogAgent agent, ConsoleWatchdogActionType[] actions)
        {
            AmtCallStatus r = (AmtCallStatus)service.ConsoleWatchdogSetActions(agent.AgentId.ToByteArray(), actions);
        }

        public static string TransitionStateToString(ushort transitState)
        {
            string o = "";
            if (transitState == 0x00FF)
            {
                o += ManageabilityStack.Properties.Resources.WatchdogTransitState_ANY;
                return o;
            }
            if ((transitState & 0x0001) != 0) o += ManageabilityStack.Properties.Resources.WatchdogTransitState_NOTSTARTED;
            if ((transitState & 0x0002) != 0) o += ManageabilityStack.Properties.Resources.WatchdogTransitState_STOPPED;
            if ((transitState & 0x0004) != 0) o += ManageabilityStack.Properties.Resources.WatchdogTransitState_RUNNING;
            if ((transitState & 0x0008) != 0) o += ManageabilityStack.Properties.Resources.WatchdogTransitState_EXPIRED;
            if ((transitState & 0x0010) != 0) o += ManageabilityStack.Properties.Resources.WatchdogTransitState_SUSPENDED;
            if (o.Length > 2) o = o.Substring(2);
            return o;
        }
    }

    [Serializable]
    public class AmtCompleteWatchdogAgent
    {
        public AmtWatchdogAgent agent;
        public ConsoleWatchdogActionType[] actions;
    }

    [Serializable]
    public class AmtWatchdogAgent
    {
        public Guid AgentId;
        public string Description;
        public ushort HeartbeatTime;
        public ushort StartupTime;
        public uint NumActions;
        public byte State;

        public AmtWatchdogAgent()
        {
            AgentId = Guid.Empty;
            Description = "";
            HeartbeatTime = 0;
            StartupTime = 0;
            NumActions = 0;
            State = 0;
        }

        public AmtWatchdogAgent(ConsoleWatchdogEntryType entry)
        {
            AgentId = new Guid(entry.AgentID);
            Description = entry.AgentDescription;
            HeartbeatTime = entry.AgentHeartbeatTime;
            StartupTime = entry.AgentStartupTime;
            NumActions = entry.NumActions;
            State = entry.State;
        }

        public AmtWatchdogAgent(Guid agentId, string description, ushort heartbeatTime, ushort startupTime)
        {
            AgentId = agentId;
            Description = description;
            HeartbeatTime = heartbeatTime;
            StartupTime = startupTime;
            NumActions = uint.MaxValue;
            State = byte.MaxValue;
        }

        public AmtWatchdogAgent(string description, ushort heartbeatTime, ushort startupTime)
        {
            AgentId = Guid.NewGuid();
            Description = description;
            HeartbeatTime = heartbeatTime;
            StartupTime = startupTime;
            NumActions = uint.MaxValue;
            State = byte.MaxValue;
        }
    }

}
