// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Text;
using System.Timers; 
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.Cim.Untyped;
using Intel.Manageability.WSManagement;
using Intel.Manageability.Exceptions;

namespace ManageabilityStack
{
    public class AmtWireless
    {
        public enum LinkPreference : uint
        {
            ME = 1,
            HOST
        }

        private const int WIRELESS_PREF_TIME = 300; 

        private WirelessConfigurationService service;
        private AmtSystem system;
        private Timer WirelessPreferenceTimer; 

        public AmtWireless(WirelessConfigurationService service, AmtSystem system)
        {
            this.service = service;
            this.system = system;

            WirelessPreferenceTimer = new Timer(WIRELESS_PREF_TIME*1000);
            WirelessPreferenceTimer.Enabled = false;
            WirelessPreferenceTimer.Elapsed += new ElapsedEventHandler(OnTimedEvent);
        }

        public void Dispose()
        {
            service = null;
            system = null;
        }

        public void ClearCache()
        {
            WirelessCapabilitiesCache = null;
            WirelessProfilesCache = null;
            ActiveProfileCache = null;
            IsRadioOnCached = false;
            IsWirelessSupportedCache = null;
        }

        public void FetchCache()
        {
            try
            {
                GetWirelessCapabilities();
                GetWirelessProfiles();
                GetActiveProfile();
            }
            catch (Exception e)
            {
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Failure to retrieve the Wireless data.",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
            }
        }

        private bool? IsWirelessSupportedCache = null;
        public bool IsWirelessSupported()
        {
            if (IsWirelessSupportedCache != null) return (bool)IsWirelessSupportedCache;

            if (system.WsMan != null)
            {
                try
                {
                    CIM_EthernetPort eNetPort = new CIM_EthernetPort(system.WsMan);
                    CIM_EthernetPort.CimKeys keys = new CIM_EthernetPort.CimKeys();
                    // The WiFi Device ID string is documented in the AMT SDK 
                    // document "Intel AMT WS-Management Flows.pdf".
                    keys.DeviceID = "Intel(r) AMT Ethernet Port 1";
                    eNetPort.Get(keys);

                    // Assumed that Wireless is supported as this point.
                    IsWirelessSupportedCache = true;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.IsWirelessSupported WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.IsWirelessSupported CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.IsWirelessSupported",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    IsWirelessSupportedCache = false; // Wireless is assumed unsupported.
                }
            }
            else
            {
                // Unable to determine without WSMAN support.
                IsWirelessSupportedCache = false;
            }

            return (bool)IsWirelessSupportedCache;
        }

        private LinkPreference? wirelessPreferenceCache = null;
        public LinkPreference? GetWirelessPreference()
        {
            if (wirelessPreferenceCache != null) return wirelessPreferenceCache;

            LinkPreference? retVal = null;
            if (system.WsMan != null)
            {
                Collection<AMT_EthernetPortSettings> eps = AMT_EthernetPortSettings.Enumerate(system.WsMan);
                try
                {
                    foreach (AMT_EthernetPortSettings ep in eps)
                    {
                        ep.Get();
                        if (ep.LinkPreferenceExist)
                        {
                            retVal = (LinkPreference)(ep.LinkPreference);
                        }
                    }
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.GetWirelessPreference WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.GetWirelessPreference CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.GetWirelessPreference",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    WirelessProfilesCache = new AmtWirelessProfile[0];
                    return wirelessPreferenceCache;
                }
                wirelessPreferenceCache = retVal;
            }

            return retVal;
        }

        public void SetWirelessPreference(LinkPreference lp)
        {
            uint lpui = (uint)lp;
            Collection<AMT_EthernetPortSettings> eps = AMT_EthernetPortSettings.Enumerate(system.WsMan);
            try
            {
                foreach (AMT_EthernetPortSettings ep in eps)
                {
                    ep.Get();
                    if (ep.LinkPreferenceExist)
                    {
                        ep.SetLinkPreference(lpui, WIRELESS_PREF_TIME);
                    }
                }

                if (lp == LinkPreference.ME)
                {
                    WirelessPreferenceTimer.Enabled = true;
                }
                else
                {
                    WirelessPreferenceTimer.Enabled = false;
                }
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.SetWirelessPreference WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.SetWirelessPreference CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.SetWirelessPreference",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                WirelessProfilesCache = new AmtWirelessProfile[0];
            }

            wirelessPreferenceCache = null;
        }

        private void OnTimedEvent(object source, ElapsedEventArgs e)
        {
            //force cache to update
            wirelessPreferenceCache = null;
            WirelessPreferenceTimer.Enabled = false; 
        }


        private AmtWirelessProfile[] WirelessProfilesCache = null;
        public AmtWirelessProfile[] GetWirelessProfiles()
        {
            if (WirelessProfilesCache != null) return WirelessProfilesCache;

            if (system.WsMan != null)
            {
                ArrayList profiles = new ArrayList();
                try
                {
                    Collection<CIM_WiFiEndpointSettings> wifiEpSettings = CIM_WiFiEndpointSettings.Enumerate(system.WsMan);
                    foreach (CIM_WiFiEndpointSettings wifiEpSetting in wifiEpSettings)
                    {
                        AmtWirelessProfile profile = new AmtWirelessProfile();
                        profile.Name = (wifiEpSetting.ElementNameExist ? wifiEpSetting.ElementName : String.Empty);
                        profile.SSID = (wifiEpSetting.SSIDExist ? wifiEpSetting.SSID : String.Empty);
                        profile.Priority = (wifiEpSetting.PriorityExist ? wifiEpSetting.Priority : new byte());

                        if (wifiEpSetting.AuthenticationMethodExist)
                        {
                            if ((wifiEpSetting.AuthenticationMethod == 4) ||    // WPA PSK
                                (wifiEpSetting.AuthenticationMethod == 5))      // WPA IEEE 802.1x
                                profile.SecurityType = AmtWirelessSecurityType.WPA;
                            else if ((wifiEpSetting.AuthenticationMethod == 6) ||    // WPA2 PSK
                                     (wifiEpSetting.AuthenticationMethod == 7))      // WPA2 IEEE 802.1x
                                profile.SecurityType = AmtWirelessSecurityType.RSN;
                        }

                        if (wifiEpSetting.EncryptionMethodExist)
                        {
                            if (wifiEpSetting.EncryptionMethod == 3)
                                profile.EncryptionType = AmtWirelessEncryptionType.TKIP;
                            else if (wifiEpSetting.EncryptionMethod == 4)
                                profile.EncryptionType = AmtWirelessEncryptionType.CCMP;
                        }

                        if (wifiEpSetting.PSKPassPhraseExist)
                        {
                            // PSK Pass Phrase
                            profile.AuthenticationType = AmtWirelessAuthenticationType.PassPhrase;
                            profile.PassPhrase63 = wifiEpSetting.PSKPassPhrase;
                        }
                        if (wifiEpSetting.PSKValueExist)
                        {
                            // Actual PSK
                            profile.AuthenticationType = AmtWirelessAuthenticationType.RawKey;
                            profile.RawKey256 = wifiEpSetting.PSKValue;
                        }
                        if ((wifiEpSetting.AuthenticationMethodExist) &&
                            ((wifiEpSetting.AuthenticationMethod == 5) ||
                             (wifiEpSetting.AuthenticationMethod == 7)))
                        {
                            // WPA/WPA2 IEEE 802.1x
                            // TODO: Implement support for retrieving the 802.1x information.
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Warning, 
                                "Unable to retrieve 802.1x wireless profile information via WSMAN.",
                                String.Format("Affected profile: {0}", profile.Name));
                            //Collection<CIM_IEEE8021xSettings> ieee8021xSettings = CIM_IEEE8021xSettings.Enumerate(system.WsMan);
                            //foreach (CIM_IEEE8021xSettings ieee8021xSetting in ieee8021xSettings)
                            //{
                            //    if ((ieee8021xSetting.ElementNameExist) &&
                            //        (ieee8021xSetting.ElementName == profile.Name))
                            //    {
                            //        // Found the corresponding IEEE 802.1x Settings
                            //        switch (ieee8021xSetting.AuthenticationProtocol)
                            //        {
                            //            case 0: // EAP-TLS - XProfileTLSType
                            //                XProfileTLSType xProfileTLS = new XProfileTLSType();
                            //                xProfileTLS.Username = (ieee8021xSetting.UsernameExist ? ieee8021xSetting.Username : String.Empty);
                            //                //xProfileTLS.ServerIdentity;
                            //                //xProfileTLS.ClientCertificate;
                            //                //profile.xProfile = xProfileTLS;
                            //                break;
                            //            case 1: // EAP-TTLS/MSCHAPv2 - XProfileTTLS_MSCHAPv2Type
                            //                XProfileTTLS_MSCHAPv2Type xProfileTTLS = new XProfileTTLS_MSCHAPv2Type();
                            //                xProfileTTLS.RoamingIdentity = (ieee8021xSetting.RoamingIdentityExist ? ieee8021xSetting.RoamingIdentity : String.Empty);
                            //                //xProfileTTLS.ServerIdentity;
                            //                xProfileTTLS.UserCredentials.Username = (ieee8021xSetting.UsernameExist ? ieee8021xSetting.Username : String.Empty);
                            //                xProfileTTLS.UserCredentials.Password = (ieee8021xSetting.PasswordExist ? ieee8021xSetting.Password : String.Empty);
                            //                xProfileTTLS.UserCredentials.Domain = (ieee8021xSetting.DomainExist ? ieee8021xSetting.Domain : String.Empty);
                            //                //xProfileTTLS.ClientCertificate;
                            //                break;
                            //            case 2: // PEAPv0/EAP-MSCHAPv2 - XProfilePEAP_MSCHAPv2Type
                            //                break;
                            //            case 3: // PEAPv1/EAP-GTC - XProfileEAP_GTCType
                            //                break;
                            //            case 4: // EAP-FAST/MSCHAPv2 - XProfileEAPFAST_MSCHAPv2Type
                            //                break;
                            //            case 5: // EAP-FAST/GTC - XProfileEAPFAST_GTCType
                            //                break;
                            //            case 10: // EAP-FAST/TLS - XProfileEAPFAST_TLSType
                            //                break;
                            //            default:
                            //                Debug.Print("WARNING: Unexpected 802.1x Authentication protocol.");
                            //                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Warning, "Unexpected 802.1x Authentication protocol");
                            //                break;
                            //        } // END switch (settings802_1x.AuthenticationProtocol)
                            //    }
                            //} // END foreach (CIM_IEEE8021xSettings 8021xSetting in 8021xSettings)
                        }

                        profiles.Add(profile);
                    } // END foreach (CIM_WiFiEndpointSettings wifiEpSetting in wifiEpSettings)

                    WirelessProfilesCache = (AmtWirelessProfile[])profiles.ToArray(typeof(AmtWirelessProfile));
                    return WirelessProfilesCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.GetWirelessProfiles WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.GetWirelessProfiles CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.GetWirelessProfiles",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    WirelessProfilesCache = new AmtWirelessProfile[0];
                    return WirelessProfilesCache;
                }
            }
            else
            {
                string[] profileNames;
                AmtCallStatus r = (AmtCallStatus)service.EnumerateWirelessProfiles(out profileNames);

                if (profileNames == null)
                {
                    WirelessProfilesCache = new AmtWirelessProfile[0];
                    return WirelessProfilesCache;
                }
                ArrayList profiles = new ArrayList();
                foreach (string profileName in profileNames)
                {
                    ProfileType profile;
                    service.GetWirelessProfile(profileName, out profile);
                    profiles.Add(new AmtWirelessProfile(profile));
                }
                WirelessProfilesCache = (AmtWirelessProfile[])profiles.ToArray(typeof(AmtWirelessProfile));
                return WirelessProfilesCache;
            }
        }

        public AmtCallStatus UpdateWirelessProfile(AmtWirelessProfile profile)
        {
            //if (system.WsMan != null)
            //{
            //    try
            //    {
            //        // Retrieve the defined wireless profile.
            //        CIM_WiFiEndpointSettings wifiEpSettings = new CIM_WiFiEndpointSettings(system.WsMan);
            //        CIM_WiFiEndpointSettings.CimKeys keys = new CIM_WiFiEndpointSettings.CimKeys();
            //        keys.InstanceID = profile.Name;
            //        wifiEpSettings.Get(keys);

            //        // Apply the updated settings to the wireless profile.
            //        wifiEpSettings.ElementName = profile.Name;
            //        wifiEpSettings.SSID = profile.SSID;
            //        wifiEpSettings.Priority = profile.Priority;

            //        if ((profile.SecurityType == AmtWirelessSecurityType.WPA) &&
            //            (profile.AuthenticationType != AmtWirelessAuthenticationType.XProfile))
            //            wifiEpSettings.AuthenticationMethod = 4; // WPA PSK
            //        else if ((profile.SecurityType == AmtWirelessSecurityType.WPA) &&
            //                 (profile.AuthenticationType == AmtWirelessAuthenticationType.XProfile))
            //            wifiEpSettings.AuthenticationMethod = 5; // WPA IEEE 802.1x
            //        else if ((profile.SecurityType == AmtWirelessSecurityType.RSN) &&
            //                 (profile.AuthenticationType != AmtWirelessAuthenticationType.XProfile))
            //            wifiEpSettings.AuthenticationMethod = 6; // WPA2 PSK
            //        else if ((profile.SecurityType == AmtWirelessSecurityType.RSN) &&
            //                 (profile.AuthenticationType == AmtWirelessAuthenticationType.XProfile))
            //            wifiEpSettings.AuthenticationMethod = 7; // WPA2 IEEE 802.1x

            //        if (profile.EncryptionType == AmtWirelessEncryptionType.TKIP)
            //            wifiEpSettings.EncryptionMethod = 3;
            //        else if (profile.EncryptionType == AmtWirelessEncryptionType.CCMP)
            //            wifiEpSettings.EncryptionMethod = 4;

            //        if (!String.IsNullOrEmpty(profile.PassPhrase63))
            //            wifiEpSettings.PSKPassPhrase = profile.PassPhrase63;

            //        if (profile.RawKey256 != null)
            //            wifiEpSettings.PSKValue = profile.RawKey256;


            //        if (profile.xProfile != null)
            //        {
            //            // WPA/WPA2 IEEE 802.1x
            //            CIM_IEEE8021xSettings ieee8021xSettings = new CIM_IEEE8021xSettings(system.WsMan);
            //            CIM_IEEE8021xSettings.CimKeys ieeeKeys = new CIM_IEEE8021xSettings.CimKeys();
            //            ieeeKeys.InstanceID = profile.Name;
            //            ieee8021xSettings.Get(ieeeKeys);

            //            ieee8021xSettings.ElementName = profile.Name;
            //            ieee8021xSettings.InstanceID = profile.Name;

            //            if (profile.xProfile.GetType() == typeof(XProfileTLSType)) // EAP-TLS
            //            {
            //            }
            //            else if (profile.xProfile.GetType() == typeof(XProfileTTLS_MSCHAPv2Type)) // EAP-TTLS/MSCHAPv2
            //            {
            //            }
            //            else if (profile.xProfile.GetType() == typeof(XProfilePEAP_MSCHAPv2Type)) // PEAPv0/EAP-MSCHAPv2
            //            {
            //            }
            //            else if (profile.xProfile.GetType() == typeof(XProfileEAP_GTCType)) // PEAPv1/EAP-GTC
            //            {
            //            }
            //            else if (profile.xProfile.GetType() == typeof(XProfileEAPFAST_MSCHAPv2Type)) // EAP-FAST/MSCHAPv2
            //            {
            //            }
            //            else if (profile.xProfile.GetType() == typeof(XProfileEAPFAST_GTCType)) // EAP-FAST/GTC
            //            {
            //            }
            //            else if (profile.xProfile.GetType() == typeof(XProfileEAPFAST_TLSType)) // EAP-FAST/TLS
            //            {
            //            }
            //            else
            //            {
            //                Debug.Print("WARNING: Unexpected 802.1x Authentication protocol.");
            //                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Warning, "Unexpected 802.1x Authentication protocol");
            //                return AmtCallStatus.INVALID_PARAMETER;
            //            }

            //            ieee8021xSettings.Put();
            //        } // END if (profile.xProfile != null)

            //        wifiEpSettings.Put();

            //        WirelessProfilesCache = null;
            //        return AmtCallStatus.SUCCESS;
            //    }
            //    catch (Exception e)
            //    {
            //        WSManException wsManException = e as WSManException;
            //        if (wsManException != null)
            //        {
            //            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.UpdateWirelessProfile WSMAN error",
            //            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
            //            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
            //            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
            //        }
            //        CimException cimException = e as CimException;
            //        if (cimException != null)
            //        {
            //            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.UpdateWirelessProfile CIM error",
            //            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
            //            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
            //            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
            //        }

            //        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.UpdateWirelessProfile",
            //            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
            //        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
            //        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
            //        WirelessProfilesCache = new AmtWirelessProfile[0];
            //        return AmtCallStatus.FAILED_WEB_CALL;
            //    }
            //}
            //else
            //{
                AmtCallStatus r = (AmtCallStatus)service.UpdateWirelessProfile(profile.GetProfile());
                WirelessProfilesCache = null;
                return r;
            //}
        }

        public AmtCallStatus RemoveWirelessProfile(AmtWirelessProfile profile)
        {
            //if (system.WsMan != null)
            //{
            //    try
            //    {
            //        // Retrieve the defined wireless profile.
            //        CIM_WiFiEndpointSettings wifiEpSettings = new CIM_WiFiEndpointSettings(system.WsMan);
            //        CIM_WiFiEndpointSettings.CimKeys keys = new CIM_WiFiEndpointSettings.CimKeys();
            //        keys.InstanceID = profile.Name;
            //        wifiEpSettings.Get(keys);

            //        // Delete it.
            //        wifiEpSettings.Delete();

            //        WirelessProfilesCache = null;
            //        return AmtCallStatus.SUCCESS;
            //    }
            //    catch (Exception e)
            //    {
            //        WSManException wsManException = e as WSManException;
            //        if (wsManException != null)
            //        {
            //            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.RemoveWirelessProfile WSMAN error",
            //            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
            //            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
            //            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
            //        }
            //        CimException cimException = e as CimException;
            //        if (cimException != null)
            //        {
            //            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.RemoveWirelessProfile CIM error",
            //            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
            //            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
            //            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
            //        }

            //        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.RemoveWirelessProfile",
            //            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
            //        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
            //        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
            //        WirelessProfilesCache = new AmtWirelessProfile[0];
            //        return AmtCallStatus.FAILED_WEB_CALL;
            //    }
            //}
            //else
            //{
                AmtCallStatus r = (AmtCallStatus)service.RemoveWirelessProfile(profile.Name);
                WirelessProfilesCache = null;
                return r;
            //}
        }

        public AmtCallStatus AddWirelessProfile(AmtWirelessProfile profile)
        {
            //if (system.WsMan != null)
            //{
            //    try
            //    {
            //        CIM_WiFiEndpointSettings wifiSettings = null;
            //        CIM_IEEE8021xSettings settings802_1x = null;
            //        AMT_PublicKeyCertificate clientCredential = null;
            //        AMT_PublicKeyCertificate caCredential = null;
            //        CimReference newWifiSettings = null;
            //        CimReference newSettings802_1x = null;

            //        wifiSettings = new CIM_WiFiEndpointSettings();
            //        wifiSettings.ElementName = profile.Name;
            //        wifiSettings.Priority = profile.Priority;
            //        wifiSettings.SSID = profile.SSID;

            //        //
            //        // NOTE: The following mappings are from the Intel(R) WS-Management Translator.
            //        //

            //        if ((profile.SecurityType == AmtWirelessSecurityType.WPA) &&
            //            (profile.xProfile == null))
            //            wifiSettings.AuthenticationMethod = 4;
            //        else if ((profile.SecurityType == AmtWirelessSecurityType.WPA) &&
            //                 (profile.xProfile != null))
            //            wifiSettings.AuthenticationMethod = 5;
            //        else if ((profile.SecurityType == AmtWirelessSecurityType.RSN) &&
            //                 (profile.xProfile == null))
            //            wifiSettings.AuthenticationMethod = 6;
            //        else if ((profile.SecurityType == AmtWirelessSecurityType.RSN) &&
            //                 (profile.xProfile != null))
            //            wifiSettings.AuthenticationMethod = 7;
            //        else
            //            throw new Exception("Unexpected wireless security model WS-MAN mapping.");

            //        if (profile.EncryptionType == AmtWirelessEncryptionType.TKIP)
            //            wifiSettings.EncryptionMethod = 3;
            //        else if (profile.EncryptionType == AmtWirelessEncryptionType.CCMP)
            //            wifiSettings.EncryptionMethod = 4;
            //        else
            //            throw new Exception("Unexpected wireless security encryption method.");

            //        if (profile.AuthenticationType == AmtWirelessAuthenticationType.PassPhrase)
            //            wifiSettings.PSKPassPhrase = profile.PassPhrase63;
            //        else if (profile.AuthenticationType == AmtWirelessAuthenticationType.RawKey)
            //            wifiSettings.PSKValue = profile.RawKey256;
            //        else
            //        {
            //            settings802_1x = new CIM_IEEE8021xSettings();

            //            if (profile.xProfile.Item.GetType() == typeof(XProfileTLSType))
            //                settings802_1x.AuthenticationProtocol = 0;
            //            else if (profile.xProfile.Item.GetType() == typeof(XProfileTTLS_MSCHAPv2Type))
            //                settings802_1x.AuthenticationProtocol = 1;
            //            else if (profile.xProfile.Item.GetType() == typeof(XProfilePEAP_MSCHAPv2Type))
            //                settings802_1x.AuthenticationProtocol = 2;
            //            else if (profile.xProfile.Item.GetType() == typeof(XProfileEAP_GTCType))
            //                settings802_1x.AuthenticationProtocol = 3;
            //            else if (profile.xProfile.Item.GetType() == typeof(XProfileEAPFAST_MSCHAPv2Type))
            //                settings802_1x.AuthenticationProtocol = 4;
            //            else if (profile.xProfile.Item.GetType() == typeof(XProfileEAPFAST_GTCType))
            //                settings802_1x.AuthenticationProtocol = 5;
            //            else if (profile.xProfile.Item.GetType() == typeof(XProfileEAPFAST_TLSType))
            //                settings802_1x.AuthenticationProtocol = 10;
            //            else
            //                throw new Exception("Unexpected wireless 802.1x security mapping.");

            //            settings802_1x.ElementName = profile.Name;
            //            settings802_1x.InstanceID = profile.Name;
            //            // TODO: Complete WSMAN support for 802.1x.
            //        }

            //        CIM_WiFiEndpoint wifiEp = new CIM_WiFiEndpoint(system.WsMan);
            //        wifiEp.Get();

            //        AMT_WiFiPortConfigurationService service = new AMT_WiFiPortConfigurationService(system.WsMan);
            //        service.Get();
            //        return (AmtCallStatus)service.AddWiFiSettings(wifiEp.Reference, wifiSettings, null,
            //            null, null, out newWifiSettings, out newSettings802_1x);
            //    }
            //    catch (Exception e)
            //    {
            //        WSManException wsManException = e as WSManException;
            //        if (wsManException != null)
            //        {
            //            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.AddWirelessProfile WSMAN error",
            //            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
            //            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
            //            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
            //        }
            //        CimException cimException = e as CimException;
            //        if (cimException != null)
            //        {
            //            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.AddWirelessProfile CIM error",
            //            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
            //            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
            //            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
            //        }

            //        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.AddWirelessProfile",
            //            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
            //        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
            //        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
            //        WirelessProfilesCache = new AmtWirelessProfile[0];
            //        return AmtCallStatus.FAILED_WEB_CALL;
            //    }
            //}
            //else
            //{
                ProfileType p = profile.GetProfile();
                AmtCallStatus r = (AmtCallStatus)service.AddWirelessProfile(p);
                WirelessProfilesCache = null;
                return r;
            //}
        }

        public enum AmtWirelessSecurityType
        {
            WPA,
            RSN
        }

        public enum AmtWirelessEncryptionType
        {
            TKIP,
            CCMP
        }

        public enum AmtWirelessAuthenticationType
        {
            PassPhrase,
            RawKey,
            XProfile
        }

        [Serializable]
        public class AmtWirelessProfile
        {
            public string Name;
            public string SSID;
            public byte Priority;
            public AmtWirelessSecurityType SecurityType;
            public AmtWirelessEncryptionType EncryptionType;
            public AmtWirelessAuthenticationType AuthenticationType;
            public string PassPhrase63;
            public byte[] RawKey256;
            public XProfileType xProfile;

            public AmtWirelessProfile()
            {
            }

            public ProfileType GetProfile()
            {
                ProfileType p = new ProfileType();
                p.ProfileName = Name;
                p.Priority = Priority;
                p.SSID = UTF8Encoding.UTF8.GetBytes(SSID);

                object DataAuthentication = null;
                switch (AuthenticationType)
                {
                    case AmtWirelessAuthenticationType.PassPhrase:
                        DataAuthentication = PassPhrase63;
                        break;
                    case AmtWirelessAuthenticationType.RawKey:
                        DataAuthentication = RawKey256;
                        break;
                    case AmtWirelessAuthenticationType.XProfile:
                        DataAuthentication = xProfile;
                        break;
                }

                object DataEncryption = null;
                switch (EncryptionType)
                {
                    case AmtWirelessEncryptionType.TKIP:
                        DataEncryption = new DataEncryptionTKIPType();
                        ((DataEncryptionTKIPType)DataEncryption).Item = DataAuthentication;
                        break;
                    case AmtWirelessEncryptionType.CCMP:
                        DataEncryption = new DataEncryptionCCMPType();
                        ((DataEncryptionCCMPType)DataEncryption).Item = DataAuthentication;
                        break;
                }

                object DataSecurity = null;
                switch (SecurityType)
                {
                    case AmtWirelessSecurityType.WPA:
                        DataSecurity = new ProfileSecuritySettingWPAType();
                        ((ProfileSecuritySettingWPAType)DataSecurity).Item = DataEncryption;
                        break;
                    case AmtWirelessSecurityType.RSN:
                        DataSecurity = new ProfileSecuritySettingRSNType();
                        ((ProfileSecuritySettingRSNType)DataSecurity).Item = DataEncryption;
                        break;
                }

                p.Security = new ProfileSecuritySettingsType();
                p.Security.Item = DataSecurity;

                return p;
            }

            public AmtWirelessProfile(ProfileType profile)
            {
                Name = profile.ProfileName;
                Priority = profile.Priority;
                SSID = UTF8Encoding.UTF8.GetString(profile.SSID);

                object DataEncryption = null;
                if (profile.Security.Item.GetType() == typeof(ProfileSecuritySettingWPAType))
                {
                    ProfileSecuritySettingWPAType wpa = (ProfileSecuritySettingWPAType)profile.Security.Item;
                    SecurityType = AmtWirelessSecurityType.WPA;
                    DataEncryption = wpa.Item;
                }
                else if (profile.Security.Item.GetType() == typeof(ProfileSecuritySettingRSNType))
                {
                    ProfileSecuritySettingRSNType rsn = (ProfileSecuritySettingRSNType)profile.Security.Item;
                    SecurityType = AmtWirelessSecurityType.RSN;
                    DataEncryption = rsn.Item;
                }

                object DataAuthentication = null;
                if (DataEncryption.GetType() == typeof(DataEncryptionTKIPType))
                {
                    DataEncryptionTKIPType TKIP = (DataEncryptionTKIPType)DataEncryption;
                    EncryptionType = AmtWirelessEncryptionType.TKIP;
                    DataAuthentication = TKIP.Item;

                }
                else if (DataEncryption.GetType() == typeof(DataEncryptionCCMPType))
                {
                    DataEncryptionCCMPType CCMP = (DataEncryptionCCMPType)DataEncryption;
                    EncryptionType = AmtWirelessEncryptionType.CCMP;
                    DataAuthentication = CCMP.Item;
                }

                if (DataAuthentication.GetType() == typeof(string))
                {
                    // PassPhrase63Type
                    AuthenticationType = AmtWirelessAuthenticationType.PassPhrase;
                    PassPhrase63 = (string)DataAuthentication;
                }
                else if (DataAuthentication.GetType() == typeof(byte[]))
                {
                    // RawKey256Type
                    AuthenticationType = AmtWirelessAuthenticationType.RawKey;
                    RawKey256 = (byte[])DataAuthentication;
                }
                else if (DataAuthentication.GetType() == typeof(XProfileType))
                {
                    // XProfileType
                    AuthenticationType = AmtWirelessAuthenticationType.XProfile;
                    xProfile = (XProfileType)DataAuthentication;
                }

            }
        }

        public class AmtWirelessCapabilities
        {
            public bool a;
            public bool b;
            public bool g;
            public bool n;
        }

        private AmtWirelessCapabilities WirelessCapabilitiesCache = null;
        public AmtWirelessCapabilities GetWirelessCapabilities()
        {
            if (WirelessCapabilitiesCache != null) return WirelessCapabilitiesCache;

            AmtWirelessCapabilities caps = new AmtWirelessCapabilities();
            caps.a = false;
            caps.b = false;
            caps.g = false;
            caps.n = false;

            if (system.WsMan != null)
            {
                try
                {
                    CIM_WiFiPortCapabilities wifiPortCaps = new CIM_WiFiPortCapabilities(system.WsMan);
                    wifiPortCaps.Get();

                    if (!wifiPortCaps.SupportedPortTypesExist)
                        throw new Exception("CIM_WiFiPortCapabilities.SupportedPortTypes does not exist.");

                    foreach (ushort supportedPortType in wifiPortCaps.SupportedPortTypes)
                    {
                        if (supportedPortType == 70) caps.a = true;
                        if (supportedPortType == 71) caps.b = true;
                        if (supportedPortType == 72) caps.g = true;
                        if (supportedPortType == 73) caps.n = true;
                    } // END foreach (ushort supportedPortType in wifiPortCaps.SupportedPortTypes)

                    WirelessCapabilitiesCache = caps;
                    return WirelessCapabilitiesCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.GetWirelessCapabilities WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.GetWirelessCapabilities CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtWireless.GetWirelessCapabilities",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    WirelessCapabilitiesCache = caps;
                    return caps;
                }
            }
            else
            {
                FeatureType[] features;
                AmtCallStatus r;
                try
                {
                    r = (AmtCallStatus)service.GetWirelessCapabilities(out features);
                }
                catch (Exception)
                {
                    WirelessCapabilitiesCache = caps;
                    return caps;
                }

                if (r != AmtCallStatus.SUCCESS) return caps;

                foreach (FeatureType feature in features)
                {
                    if (feature == FeatureType.Item80211a) caps.a = true;
                    if (feature == FeatureType.Item80211b) caps.b = true;
                    if (feature == FeatureType.Item80211g) caps.g = true;
                    if (feature == FeatureType.Item80211n) caps.n = true;
                }

                WirelessCapabilitiesCache = caps;
                return WirelessCapabilitiesCache;
            }
        }

        public string GetWirelessCapabilitiesStr()
        {
            AmtWirelessCapabilities caps = GetWirelessCapabilities();

            // Did not localize here since both Chinese & English should be the same
            StringBuilder s = new StringBuilder();
            if (caps.a) s.Append(", 802.11a");
            if (caps.b) s.Append(", 802.11b");
            if (caps.g) s.Append(", 802.11g");
            if (caps.n) s.Append(", 802.11n");
            if (s.Length >= 2) return s.ToString().Substring(2);
            return s.ToString();
        }

        private AmtWirelessProfile ActiveProfileCache = null;
        public AmtWirelessProfile GetActiveProfile()
        {
            if (ActiveProfileCache != null) return ActiveProfileCache;

            WirelessSettingsType settings;
            AmtCallStatus r = (AmtCallStatus)service.GetWirelessSettings(out settings);

            AmtWirelessProfile[] profiles = GetWirelessProfiles();
            foreach (AmtWirelessProfile profile in profiles)
            {
                if (profile.Name.CompareTo(settings.ActiveProfile) == 0)
                {
                    return profile;
                }
            }
            return null;
        }

        private bool IsRadioOnCache = false;
        private bool IsRadioOnCached = false;
        public AmtCallStatus IsRadioOn(out bool enabled)
        {
            enabled = false;
            if (IsRadioOnCached)
            {
                enabled = IsRadioOnCache;
                return AmtCallStatus.SUCCESS;
            }
            WirelessSettingsType settings;
            AmtCallStatus r;
            try
            {
                r = (AmtCallStatus)service.GetWirelessSettings(out settings);
            }
            catch (Exception)
            {
                return AmtCallStatus.FAILED_WEB_CALL;
            }
            if (r == AmtCallStatus.SUCCESS)
            {
                enabled = settings.RadioOn;
                IsRadioOnCache = settings.RadioOn;
                IsRadioOnCached = true;
            }
            return r;
        }
    }
}
