// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Data;
using System.Text;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using System.Collections.Generic;
using System.Security.Cryptography;
using System.Security.Cryptography.X509Certificates;

namespace ManageabilityStack
{
    public partial class CertificateManagerForm : Form
    {
        private X509Store CertificateStore;
        private X509Store CertificateRootStore = null;

        public CertificateManagerForm()
        {
            InitializeComponent();
        }

        private void CertificateManagerForm_Load(object sender, EventArgs e)
        {
            // Open the certificate stores
            CertificateStore = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadWrite);
            CertificateRootStore = new X509Store(StoreName.Root, StoreLocation.CurrentUser);
            CertificateRootStore.Open(OpenFlags.ReadWrite);

            UpdateCerts();
        }

        private void refreshButton_Click(object sender, EventArgs e)
        {
            UpdateCerts();
        }

        private void closeButton_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void UpdateInfo()
        {
            removeTrustedCertButton.Enabled = viewTrustedCertButton.Enabled = (trustedCertsListView.SelectedItems.Count > 0);
        }

        private void UpdateCerts()
        {
            // Load the list of trusted certificates. Trusted certs must be in both stores.
            trustedCertsListView.BeginUpdate();
            trustedCertsListView.Items.Clear();
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (IsTrusted(cert))
                {
                    int icon = 0;
                    if (cert.HasPrivateKey == true && CertificateOperations.IsKeyExportable(cert) == false) icon = 2;
                    ListViewItem l = new ListViewItem(new string[] { GetCertificateString(cert.Subject) }, icon);
                    l.Tag = cert;
                    l.Group = trustedCertsListView.Groups[0];
                    trustedCertsListView.Items.Add(l);
                }
            }
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (cert.HasPrivateKey)
                {
                    int icon = 0;
                    if (cert.HasPrivateKey == true && CertificateOperations.IsKeyExportable(cert) == false) icon = 2;
                    ListViewItem l = new ListViewItem(new string[] { GetCertificateString(cert.Subject) }, icon);
                    l.Tag = cert;
                    l.Group = trustedCertsListView.Groups[1];
                    trustedCertsListView.Items.Add(l);
                }
            }
            trustedCertsListView.EndUpdate();
            UpdateInfo();
        }

        private bool IsTrusted(X509Certificate2 cert)
        {
            foreach (X509Certificate2 cert2 in CertificateRootStore.Certificates)
            {
                if (cert2.Equals(cert)) return true;
            }
            return false;
        }

        public static string GetCertificateString(string certDataStr)
        {
            string s = "";

            int i = certDataStr.IndexOf("CN=");
            if (i >= 0)
            {
                s = certDataStr.Substring(i + 3);
                i = s.IndexOf(",");
                if (i > 0) s = s.Substring(0, i);
            }

            string s2 = null;
            i = certDataStr.IndexOf("O=");
            if (i >= 0)
            {
                s2 = certDataStr.Substring(i + 2);
                i = s2.IndexOf(",");
                if (i > 0) s2 = s2.Substring(0, i);
                s += ", " + s2;
            }

            return s;
        }

        private void CertificateManagerForm_FormClosed(object sender, FormClosedEventArgs e)
        {
            CertificateStore.Close();
            CertificateRootStore.Close();
        }

        private void viewTrustedCertButton_Click(object sender, EventArgs e)
        {
            if (trustedCertsListView.SelectedItems.Count != 1) return;
            X509Certificate2 cert = (X509Certificate2)trustedCertsListView.SelectedItems[0].Tag;
            X509Certificate2UI.DisplayCertificate(cert, Handle);
        }

        private void trustedCertsListView_SelectedIndexChanged(object sender, EventArgs e)
        {
            UpdateInfo();
        }

        private void importTrustedCertButton_Click(object sender, EventArgs e)
        {
            if (openTrustedCertFileDialog.ShowDialog(this) == DialogResult.OK)
            {
                ImportCert(new FileInfo(openTrustedCertFileDialog.FileName));
            }
        }

        private void ImportCert(FileInfo fi)
        {
            X509Certificate2 cert = null;

            try
            {
                cert = new X509Certificate2(fi.FullName);
            }
            catch (CryptographicException) { }

            if (cert == null)
            {
                AskCertPasswordForm f = new AskCertPasswordForm();
                if (f.ShowDialog(this) == DialogResult.OK)
                {
                    try
                    {
                        cert = new X509Certificate2(fi.FullName, f.Password, X509KeyStorageFlags.Exportable | X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet);
                    }
                    catch (CryptographicException)
                    {
                        MessageBox.Show(this, Properties.Resources.MsgBox_Import_Error, Properties.Resources.MsgBox_Import_Cert, MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                }
                else
                {
                    return;
                }
            }

            CertificateStore.Add(cert);
        }

        private void removeTrustedCertButton_Click(object sender, EventArgs e)
        {
            if (trustedCertsListView.SelectedItems.Count != 1) return;
            ListViewItem selectedItem = trustedCertsListView.SelectedItems[0];

            X509Certificate2 cert = (X509Certificate2)selectedItem.Tag;
            if (selectedItem.Group == trustedCertsListView.Groups[0])
            {
                try
                {
                    CertificateRootStore.Remove(cert);
                }
                catch (Exception) { }

                if (IsTrusted(cert) == false)
                {
                    CertificateStore.Remove(cert);
                    UpdateCerts();
                }
                CertificateStore.Remove(cert);
                UpdateCerts();
            }
            if (selectedItem.Group == trustedCertsListView.Groups[1])
            {
                if (IsTrusted(cert))
                {
                    try
                    {
                        CertificateRootStore.Remove(cert);
                    }
                    catch (Exception) { }

                    if (IsTrusted(cert) == false)
                    {
                        CertificateStore.Remove(cert);
                        UpdateCerts();
                    }
                    CertificateStore.Remove(cert);
                    UpdateCerts();
                }
                else
                {
                    if (MessageBox.Show(this,
                        ManageabilityStack.Properties.Resources.CertMgrForm_DelStr,
                        ManageabilityStack.Properties.Resources.CertMgrForm_Del,
                        MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                    {
                        CertificateStore.Remove(cert);
                        UpdateCerts();
                    }
                }
            }
        }

        public static X509Certificate2 GetCertificateFromThumbprint(string thumbprint)
        {
            if (thumbprint == null) return null;

            // Look in my store
            X509Store CertificateStore = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadOnly);
            X509Certificate2 certificate = null;
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (cert.Thumbprint.CompareTo(thumbprint) == 0)
                {
                    certificate = cert;
                    break;
                }
            }
            CertificateStore.Close();
            if (certificate != null) return certificate;

            // Look in root store
            CertificateStore = new X509Store(StoreName.Root, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadOnly);
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (cert.Thumbprint.CompareTo(thumbprint) == 0)
                {
                    certificate = cert;
                    break;
                }
            }
            CertificateStore.Close();
            return certificate;
        }


    }

}
