// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using System.Windows.Forms;

namespace ManageabilityStack
{
    public class CertificateOperations
    {
        /// <summary>
        /// Return a string containing all of the trusted root certificates in PEM format.
        /// </summary>
        /// <returns></returns>
        public static string AllTrustedRootsToPEM()
        {
            StringBuilder AllTrustedCerts = new StringBuilder();

            foreach (StoreName name in new StoreName[] { StoreName.My, StoreName.Root, StoreName.AuthRoot, StoreName.CertificateAuthority })
            {
                X509Store CertificateStore = new X509Store(name, StoreLocation.CurrentUser);
                CertificateStore.Open(OpenFlags.ReadOnly);

                AllTrustedCerts.Append("\r\n").Append("Certificate Store: " + name.ToString()).Append("\r\n");

                foreach (X509Certificate2 cert in CertificateStore.Certificates)
                {
                    if (ManageabilityStack.CertificateOperations.IsTrustedCert(cert) &&
                        ManageabilityStack.CertificateOperations.IsCaCertificate(cert))
                    {
                        AllTrustedCerts.Append("\r\n").Append(GetCertificateString(cert.Subject)).Append("\r\n");
                        string b1 = Convert.ToBase64String(cert.Export(X509ContentType.Cert));
                        StringBuilder b2 = new StringBuilder();
                        while (b1.Length > 64)
                        {
                            b2.Append(b1.Substring(0, 64)).Append("\r\n");
                            b1 = b1.Substring(64);
                        }
                        b2.Append(b1).Append("\r\n");
                        AllTrustedCerts.Append("-----BEGIN CERTIFICATE-----\r\n").Append(b2.ToString()).Append("-----END CERTIFICATE-----\r\n");
                    }
                } // END foreach (X509Certificate2 cert in CertificateStore.Certificates)

                CertificateStore.Close();
            } // END foreach (StoreName name in new StoreName[] { StoreName.My, StoreName.Root, StoreName.AuthRoot, StoreName.CertificateAuthority })

            return AllTrustedCerts.ToString();
        }

        /// <summary>
        /// Save a list of all trsuted root certificates to a PEM file.
        /// </summary>
        /// <param name="filename"></param>
        /// <returns></returns>
        public static bool AllTrustedRootsToPemFile(string filename)
        {
            // Save trusted certs
            byte[] bytes = UTF8Encoding.UTF8.GetBytes(AllTrustedRootsToPEM());
            FileStream fs = new FileStream(filename, FileMode.Create);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();
            return true;
        }

        public static bool AllRootsToPemFile(string filename, X509Certificate2[] serverCerts)
        {
            byte[] bytes = null;
            FileStream fs = new FileStream(filename, FileMode.Create);

            //
            // Build the cert chains from the server certificates and add that to 
            // the file.
            //
            foreach (X509Certificate2 serverCert in serverCerts)
            {
                if (!IsCaCertificate(serverCert))
                    continue; // Move onto the next certificate.

                bytes = UTF8Encoding.UTF8.GetBytes("\r\n" + GetCertificateString(serverCert.Subject) + "\r\n");
                fs.Write(bytes, 0, bytes.Length);

                bytes = UTF8Encoding.UTF8.GetBytes(CertToPem(serverCert));
                fs.Write(bytes, 0, bytes.Length);
            } // END foreach (X509Certificate2 serverCert in serverCerts)

            fs.Close();
            return true;
        }

        public static X509Certificate2[] GetCertsFromPem(string pem)
        {
            ArrayList l = new ArrayList();
            X509Certificate2 c;
            while ((c = GetFirstCertFromPem(ref pem)) != null) l.Add(c);
            return (X509Certificate2[])l.ToArray(typeof(X509Certificate2));
        }

        private static X509Certificate2 GetFirstCertFromPem(ref string pem)
        {
            int i = pem.IndexOf("-----BEGIN CERTIFICATE-----");
            if (i < 0) return null;
            pem = pem.Substring(i + 27);
            i = pem.IndexOf("-----END CERTIFICATE-----");
            if (i < 0) return null;
            X509Certificate2 cert = new X509Certificate2(Convert.FromBase64String(pem.Substring(0, i)));
            pem = pem.Substring(i + 25);
            return cert;
        }

        private static Process Run(string fileName, string args)
        {
            ProcessStartInfo info = new ProcessStartInfo(fileName, args);
            info.CreateNoWindow = true;
            info.UseShellExecute = false;
            Process processChild = Process.Start(info);
            return processChild;
        }

        private static string GetCertificateString(string certDataStr)
        {
            string s = "";

            int i = certDataStr.IndexOf("CN=");
            if (i >= 0)
            {
                s = certDataStr.Substring(i + 3);
                i = s.IndexOf(",");
                if (i > 0) s = s.Substring(0, i);
                return s;
            }

            i = certDataStr.IndexOf("O=");
            if (i >= 0)
            {
                s = certDataStr.Substring(i + 2);
                if (s[0] == '\"')
                {
                    s = s.Substring(1);
                    int j = s.IndexOf("\"");
                    if (j >= 0)
                    {
                        s = s.Substring(0, j);
                    }
                }
                else
                {
                    i = s.IndexOf(",");
                    if (i > 0) s = s.Substring(0, i);
                }
                return s;
            }

            return "Unknown";
        }

        public static string GetCertificateString(X509Certificate2 cert)
        {
            return CertificateOperations.GetCertificateString(cert.Subject);
        }

        public static string[] GetCertificateCommonNames(X509Certificate2 cert)
        {
            string certDataStr = cert.Subject;
            ArrayList commonNames = new ArrayList();

            int i = certDataStr.IndexOf("CN=");
            while (i >= 0)
            {
                certDataStr = certDataStr.Substring(i + 3);
                i = certDataStr.IndexOf(",");
                if (i > 0) commonNames.Add(certDataStr.Substring(0, i)); else commonNames.Add(certDataStr);
                i = certDataStr.IndexOf("CN=");
            }

            return (string[])commonNames.ToArray(typeof(string));
        }

        public static string GetCertificateLongString(X509Certificate2 cert)
        {
            string certname1 = CertificateOperations.GetCertificateString(cert.Subject);
            X509Certificate2 root = ManageabilityStack.CertificateOperations.GetTrustedRootCert(cert);
            if (root != null)
            {
                return string.Format("{0} / {1}", certname1, CertificateOperations.GetCertificateString(root.Subject));
            }
            else
            {
                return certname1;
            }
        }

        public static X509ChainStatusFlags CheckCertificate(X509Certificate2 cert)
        {
            X509Chain chain = new X509Chain();
            chain.ChainPolicy = new X509ChainPolicy();
            chain.ChainPolicy.RevocationMode = X509RevocationMode.NoCheck;
            if (chain.Build(cert) == false)
            {
                foreach (X509ChainElement e in chain.ChainElements)
                {
                    if (e.ChainElementStatus.Length != 0) return e.ChainElementStatus[0].Status;
                }
            }
            return X509ChainStatusFlags.NoError;
        }

        public static bool IsAmtAgentCert(X509Certificate2 cert)
        {
            return HasKeyUse(cert, "2.16.840.1.113741.1.2.2") == true;
        }

        public static bool IsAmtConsoleCert(X509Certificate2 cert)
        {
            return HasKeyUse(cert, "2.16.840.1.113741.1.2.1") == true;
        }

        public static bool HasKeyUse(X509Certificate2 cert, string keyUseOid)
        {
            foreach (X509Extension ex in cert.Extensions)
            {
                if (ex.Format(false).IndexOf("(" + keyUseOid + ")") >= 0) return true;
            }
            return false;
        }

        public static X509Certificate2[] GetValidAgentCertificates()
        {
            X509Store CertificateStore = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadOnly);

            ArrayList certs = new ArrayList();
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (CertificateOperations.CheckCertificate(cert) == X509ChainStatusFlags.NoError && CertificateOperations.IsAmtAgentCert(cert)) certs.Add(cert);
            }
            CertificateStore.Close();
            return (X509Certificate2[])certs.ToArray(typeof(X509Certificate2));
        }

        public static X509Certificate2[] GetValidConsoleCertificates()
        {
            X509Store CertificateStore = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            CertificateStore.Open(OpenFlags.ReadOnly);

            ArrayList certs = new ArrayList();
            foreach (X509Certificate2 cert in CertificateStore.Certificates)
            {
                if (CertificateOperations.CheckCertificate(cert) == X509ChainStatusFlags.NoError && CertificateOperations.IsAmtConsoleCert(cert)) certs.Add(cert);
            }
            CertificateStore.Close();
            return (X509Certificate2[])certs.ToArray(typeof(X509Certificate2));
        }

        public static string CertToPem(X509Certificate2 certificate)
        {
            string b1 = Convert.ToBase64String(certificate.Export(X509ContentType.Cert));
            StringBuilder b2 = new StringBuilder();
            b2.Append("-----BEGIN CERTIFICATE-----\r\n");
            while (b1.Length > 64)
            {
                b2.Append(b1.Substring(0, 64)).Append("\r\n");
                b1 = b1.Substring(64);
            }
            b2.Append(b1).Append("\r\n-----END CERTIFICATE-----\r\n");
            return b2.ToString();
        }

        public static bool CertToPemFile(X509Certificate2 certificate, string filename)
        {
            // Save trusted certs
            byte[] bytes = UTF8Encoding.UTF8.GetBytes(CertToPem(certificate).ToString());
            FileStream fs = new FileStream(filename, FileMode.Create);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();
            return true;
        }

        public static bool CertToDerFile(X509Certificate2 certificate, string filename)
        {
            // Convert the certificate to a DER file
            Directory.SetCurrentDirectory(Application.StartupPath);
            CertToPemFile(certificate, "openssl\\tempcert.pem");
            System.Diagnostics.Process p;
            p = Run("openssl\\openssl", "x509 -in openssl\\tempcert.pem -inform PEM -out \"" + filename + "\" -outform DER");
            while (p.HasExited == false) System.Threading.Thread.Sleep(100);
            File.Delete("openssl\\tempcert.pem");
            return true;
        }

        public static bool CertAndKeyChainToPemFile(X509Certificate2 certificate, string filename, string password)
        {
            string pem = CertAndKeyChainToPem(certificate, password);
            if (pem == null) return false;

            FileStream fs = new FileStream(filename, FileMode.Create);
            byte[] bytes = UTF8Encoding.UTF8.GetBytes(pem);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            return true;
        }

        public static bool CertAndKeyToPemFile(X509Certificate2 certificate, string filename, string password)
        {
            string pem = CertAndKeyToPem(certificate, password);
            if (pem == null) return false;

            FileStream fs = new FileStream(filename, FileMode.Create);
            byte[] bytes = UTF8Encoding.UTF8.GetBytes(pem);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            return true;
        }

        public static string CertAndKeyChainToPem(X509Certificate2 certificate, string password)
        {
            // Store the private cert
            string certpem = CertAndKeyToPem(certificate, password);

            // Store additional certs
            X509Chain c = new X509Chain();
            c.Build(certificate);
            if (c.ChainElements.Count > 1)
            {
                for (int i = 1; i < c.ChainElements.Count; i++)
                {
                    certpem += CertToPem(c.ChainElements[i].Certificate);
                }
            }

            return certpem;
        }

        public static string CertAndKeyToPem(X509Certificate2 certificate, string password)
        {
            Directory.SetCurrentDirectory(Application.StartupPath);

            string filename = "tempcert.pem"; // TODO: Change this to a random name

            // Save the certificate private key to file
            byte[] bytes = certificate.Export(X509ContentType.Pkcs12, "PASS");
            FileStream fs = new FileStream("openssl\\tempcert.p12", FileMode.Create);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Convert the certificate to a PEM file
            System.Diagnostics.Process p;
            if (password == null)
            {
                p = Run("openssl\\openssl", "pkcs12 -in openssl\\tempcert.p12 -out \"" + filename + "\" -passin pass:PASS -nodes");
            }
            else
            {
                p = Run("openssl\\openssl", "pkcs12 -in openssl\\tempcert.p12 -out \"" + filename + "\" -passin pass:PASS -passout pass:" + password);
            }
            while (p.HasExited == false) System.Threading.Thread.Sleep(100);

            /*
            // Create a BAT file
            fs = new FileStream("tempbat.bat", FileMode.Create);
            if (password == null)
            {
                bytes = UTF8Encoding.UTF8.GetBytes("openssl\\openssl pkcs12 -in openssl\\tempcert.p12 -out \"" + filename + "\" -passin pass:PASS -nodes");
            }
            else
            {
                bytes = UTF8Encoding.UTF8.GetBytes("openssl\\openssl pkcs12 -in openssl\\tempcert.p12 -out \"" + filename + "\" -passin pass:PASS -passout pass:" + password);
            }
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();
            //File.Delete("openssl\\tempbat.bat");
            */

            fs = new FileStream(filename, FileMode.Open);
            bytes = new byte[fs.Length];
            fs.Read(bytes, 0, (int)(fs.Length));
            fs.Close();
            string pem = UTF8Encoding.UTF8.GetString(bytes);

            pem = pem.Replace("\n", "\r\n");

            /*
            pem = pem.Replace("-----BEGIN RSA PRIVATE KEY-----", "\r\n\r\n-----BEGIN RSA PRIVATE KEY-----\r\n\r\n");
            pem = pem.Replace("-----END RSA PRIVATE KEY-----", "\r\n\r\n-----END RSA PRIVATE KEY-----\r\n\r\n");
            pem = pem.Replace("-----BEGIN CERTIFICATE-----", "\r\n\r\n-----BEGIN CERTIFICATE-----\r\n\r\n");
            pem = pem.Replace("-----END CERTIFICATE-----", "\r\n\r\n-----END CERTIFICATE-----\r\n\r\n");
            */

            File.Delete("openssl\\tempcert.p12");
            File.Delete("tempcert.pem");

            return pem;
        }

        public static bool PrivateKeyToPemFile(X509Certificate2 certificate, string filename, string password)
        {
            Directory.SetCurrentDirectory(Application.StartupPath);

            // Save the certificate private key to file
            byte[] bytes = certificate.Export(X509ContentType.Pkcs12, "PASS");
            FileStream fs = new FileStream("openssl\\tempcert.p12", FileMode.Create);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Convert the certificate to a PEM file
            System.Diagnostics.Process p;
            if (password == null)
            {
                p = Run("openssl\\openssl", "pkcs12 -in openssl\\tempcert.p12 -out \"" + filename + "\" -passin pass:PASS -nodes");
            }
            else
            {
                p = Run("openssl\\openssl", "pkcs12 -in openssl\\tempcert.p12 -out \"" + filename + "\" -passin pass:PASS -passout pass:" + password);
            }
            while (p.HasExited == false) System.Threading.Thread.Sleep(100);

            fs = new FileStream(filename, FileMode.Open);
            bytes = new byte[fs.Length];
            fs.Read(bytes, 0, (int)(fs.Length));
            fs.Close();
            string pem = UTF8Encoding.UTF8.GetString(bytes);

            // Keep only the private key.
            int i = pem.IndexOf("-----BEGIN RSA PRIVATE KEY-----");
            if (i >= 0) pem = pem.Substring(i);
            i = pem.IndexOf("-----END RSA PRIVATE KEY-----");
            if (i >= 0) pem = pem.Substring(0,i+29);
            pem = pem.Replace("\n", "\r\n");

            fs = new FileStream(filename, FileMode.Create);
            bytes = UTF8Encoding.UTF8.GetBytes(pem);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            File.Delete("openssl\\tempcert.p12");

            return true;
        }

        public static byte[] CertificateChainToBytes(X509Certificate2[] certificateChain)
        {
            ArrayList l = new ArrayList();
            int TotalLength = 0;
            foreach (X509Certificate2 cert in certificateChain)
            {
                byte[] certBin = cert.GetRawCertData();
                TotalLength += (3 + certBin.Length);
                l.Add(certBin);
            }
            byte[] chainBin = new byte[TotalLength];
            int i = 0;
            foreach (byte[] certBin in l)
            {
                int certSize = certBin.Length;
                chainBin[i + 0] = (byte)(certSize >> 16);
                chainBin[i + 1] = (byte)(certSize >> 8);
                chainBin[i + 2] = (byte)(certSize);
                Array.Copy(certBin, 0, chainBin, i + 3, certBin.Length);
                i += (certBin.Length + 3);
            }
            return chainBin;
        }

        public enum IssueCertType
        {
            SubCA,
            TlsClientLocal,
            TlsClientRemote,
            TlsServer,
            TlsConfig,
            AllPermissions,
            TlsSwitchbox,
        }

        public static X509Certificate2 IssueSignedCert(X509Certificate2 rootCertificate, string commonName, string orgName, string countryCode, int keysize, IssueCertType certType)
        {
            return IssueSignedCert(rootCertificate, new string[] { commonName }, orgName, countryCode, keysize, certType, SignatureAlgorithm.SHA1);
        }

        public static X509Certificate2 IssueSignedCert(X509Certificate2 rootCertificate, string commonName, string orgName, string countryCode, int keysize, IssueCertType certType, SignatureAlgorithm digest)
        {
            return IssueSignedCert(rootCertificate, new string[] { commonName }, orgName, countryCode, keysize, certType, digest);
        }

        public static X509Certificate2 IssueSignedCert(X509Certificate2 rootCertificate, string[] commonNames, string orgName, string countryCode, int keysize, IssueCertType certType)
        {
            return IssueSignedCert(rootCertificate, commonNames, orgName, countryCode, keysize, certType, SignatureAlgorithm.SHA1);
        }

        public static X509Certificate2 IssueSignedCert(X509Certificate2 rootCertificate, string[] commonNames, string orgName, string countryCode, int keysize, IssueCertType certType, SignatureAlgorithm digest)
        {
            Directory.SetCurrentDirectory(Application.StartupPath);

            // Perform a cleanup just in case.
            File.Delete("openssl\\tempconfig.cfg");
            File.Delete("openssl\\temprootcert.p12");
            File.Delete("openssl\\temprootcert.pem");
            File.Delete("openssl\\tempcertreq.pem");
            File.Delete("openssl\\tempserial.txt");
            File.Delete("openssl\\tempindex.txt");
            File.Delete("openssl\\tempindex.txt.attr");
            File.Delete("openssl\\tempserial.txt.old");
            File.Delete("openssl\\tempindex.txt.old");
            File.Delete("openssl\\tempindex.txt.attr.old");
            File.Delete("openssl\\tempcfg.cfg");
            File.Delete("openssl\\tempyy.txt");
            File.Delete("openssl\\tempbat.bat");

            // Build the certificate request
            string extras = "";
            if (certType == IssueCertType.TlsConfig || certType == IssueCertType.AllPermissions || certType == IssueCertType.TlsSwitchbox)
            {
                extras += "organizationalUnitName = OrgUnit\r\norganizationalUnitName_value = Intel(R) Client Setup Certificate\r\n";
            }
            string cfg = "[ req ]\r\ndefault_bits = " + keysize.ToString() + "\r\ndistinguished_name = req_distinguished_name\r\nencrypt_rsa_key = no\r\ndefault_md = sha1\r\n[ req_distinguished_name ]\r\ncountryName = Country Name (2 letter code)\r\ncountryName_default = " + countryCode + "\r\ncountryName_value = " + countryCode + "\r\norganizationName = Organization Name (eg, company)\r\norganizationName_value = " + orgName + "\r\n";
            int j = 0;
            for (int i = commonNames.Length - 1; i >= 0 ; i--)
            {
                cfg += j.ToString() + ".commonName = Common Name (eg, YOUR name)\r\n";
                cfg += j.ToString() + ".commonName_value = \"" + commonNames[i] + "\"\r\n";
                j++;
            }
            cfg += extras + "\r\n";

            FileStream fs = new FileStream("openssl\\tempconfig.cfg", FileMode.Create);
            byte[] bytes = UTF8Encoding.UTF8.GetBytes(cfg);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();
            System.Diagnostics.Process p;
            p = Run("openssl\\openssl", String.Format("req -config openssl\\tempconfig.cfg -new -keyout openssl\\tempsubcakey.pem -out openssl\\tempcertreq.pem -days 7300 -{0}",
                digest.ToString().ToLower()));
            while (p.HasExited == false) System.Threading.Thread.Sleep(100);
            if (p.ExitCode != 0)
            {
                MessageBox.Show("OpenSSL certificate request: " + p.ExitCode, "OpenSSL Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }

            // Save the CA certificate private key to file
            bytes = rootCertificate.Export(X509ContentType.Pkcs12, "PASS");
            fs = new FileStream("openssl\\temprootcert.p12", FileMode.Create);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Convert the CA certificate to a PEM file
            p = Run("openssl\\openssl", "pkcs12 -in openssl\\temprootcert.p12 -out openssl\\temprootcert.pem -passin pass:PASS -passout pass:PASS");
            while (p.HasExited == false) System.Threading.Thread.Sleep(100);
            if (p.ExitCode != 0)
            {
                MessageBox.Show("OpenSSL convert CA cert to PEM file: " + p.ExitCode, "OpenSSL Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }

            // Save a serial number file
            Random r = new Random();
            int serial = r.Next(1000000);
            fs = new FileStream("openssl\\tempserial.txt", FileMode.Create);
            bytes = UTF8Encoding.UTF8.GetBytes(serial.ToString("10000000"));
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Create a fake index file
            fs = new FileStream("openssl\\tempindex.txt", FileMode.Create);
            fs.Close();

            // Create an index attrib file
            fs = new FileStream("openssl\\tempindex.txt.attr", FileMode.Create);
            bytes = UTF8Encoding.UTF8.GetBytes("unique_subject = no");
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            string ext = "";
            if (certType == IssueCertType.SubCA) ext = "v3_ca";
            if (certType == IssueCertType.TlsServer) ext = "usr_server";
            if (certType == IssueCertType.TlsClientRemote) ext = "usr_rclient";
            if (certType == IssueCertType.TlsClientLocal) ext = "usr_lclient";
            if (certType == IssueCertType.TlsConfig || certType == IssueCertType.TlsSwitchbox) ext = "usr_config";
            if (certType == IssueCertType.AllPermissions) ext = "usr_all";

            // --- Extended Key Usage OID's ---
            // 1.3.6.1.5.5.7.3.1            = TLS Server certificate
            // 1.3.6.1.5.5.7.3.2            = TLS Client certificate
            // 2.16.840.1.113741.1.2.1      = Intel AMT Remote Console
            // 2.16.840.1.113741.1.2.2      = Intel AMT Local Console
            // 2.16.840.1.113741.1.2.3      = Intel AMT Client Setup Certificate (Zero-Touch)

            // Create a config file
            fs = new FileStream("openssl\\tempcfg.cfg", FileMode.Create);
            StringBuilder sb = new StringBuilder();
            sb.Append("[ ca ]\r\nx509_extensions = ").Append(ext).Append("\r\ndefault_ca = CA_default\r\n[ CA_default ]\r\ndir = .\r\ndatabase = ./tempindex.txt\r\nnew_certs_dir = .\r\ncertificate = ./temprootcert.pem\r\nserial = ./tempserial.txt\r\nprivate_key = ./temprootcert.pem\r\ndefault_days = 7300\r\ndefault_md = sha1\r\npolicy = policy_any\r\nemail_in_dn = no\r\nname_opt = ca_default\r\ncert_opt = ca_default\r\ncopy_extensions = none\r\n[ policy_any ]\r\ncountryName = optional\r\nstateOrProvinceName = optional\r\nlocalityName = optional\r\norganizationName = optional\r\norganizationalUnitName = optional\r\ncommonName = supplied\r\nemailAddress = optional\r\n");
            sb.Append("[ v3_ca ]\r\nbasicConstraints=CA:TRUE\r\nsubjectKeyIdentifier=hash\r\n#authorityKeyIdentifier=keyid:always,issuer:always\r\nkeyUsage = keyCertSign, cRLSign\r\n");
            sb.Append("[ usr_server ]  \r\nextendedKeyUsage = 1.3.6.1.5.5.7.3.1\r\nkeyUsage = nonRepudiation, digitalSignature, keyEncipherment\r\n");
            sb.Append("[ usr_rclient ] \r\nextendedKeyUsage = 1.3.6.1.5.5.7.3.2,2.16.840.1.113741.1.2.1\r\nkeyUsage = nonRepudiation, digitalSignature, keyEncipherment\r\n");
            sb.Append("[ usr_lclient ] \r\nextendedKeyUsage = 1.3.6.1.5.5.7.3.2,2.16.840.1.113741.1.2.2\r\nkeyUsage = nonRepudiation, digitalSignature, keyEncipherment\r\n");
            sb.Append("[ usr_config ]  \r\nextendedKeyUsage = 1.3.6.1.5.5.7.3.1,2.16.840.1.113741.1.2.3\r\nkeyUsage = nonRepudiation, digitalSignature, keyEncipherment\r\n");
            sb.Append("[ usr_all ]     \r\nextendedKeyUsage = 1.3.6.1.5.5.7.3.1,1.3.6.1.5.5.7.3.2,2.16.840.1.113741.1.2.1,2.16.840.1.113741.1.2.2,2.16.840.1.113741.1.2.3\r\nkeyUsage = nonRepudiation, digitalSignature, keyEncipherment\r\n");
            sb.Append("[ usr_allCA ]   \r\nextendedKeyUsage = 1.3.6.1.5.5.7.3.1,1.3.6.1.5.5.7.3.2,2.16.840.1.113741.1.2.1,2.16.840.1.113741.1.2.2,2.16.840.1.113741.1.2.3\r\nbasicConstraints=CA:TRUE\r\nsubjectKeyIdentifier=hash\r\n#authorityKeyIdentifier=keyid:always,issuer:always\r\nkeyUsage = nonRepudiation, digitalSignature, keyEncipherment, keyCertSign, cRLSign\r\n");
            bytes = UTF8Encoding.UTF8.GetBytes(sb.ToString());
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Create a YY file
            fs = new FileStream("openssl\\tempyy.txt", FileMode.Create);
            bytes = UTF8Encoding.UTF8.GetBytes("y\r\ny\r\n");
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Create certificate start & end time
            string stime = DateTime.Now.AddDays(-10).ToString("yyMMddhhmmss");
            string etime = DateTime.Now.AddYears(20).ToString("yyMMddhhmmss");

            // Create a BAT file
            fs = new FileStream("openssl\\tempbat.bat", FileMode.Create);
            bytes = UTF8Encoding.UTF8.GetBytes("cd openssl\r\nopenssl ca -config tempcfg.cfg -extensions " + ext + " -in tempcertreq.pem -out tempcert.pem -passin pass:PASS -startdate " + stime + "Z -enddate " + etime + "Z -md " + digest.ToString().ToLower() + " < tempyy.txt");
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Have the CA signed the certificate
            p = Run("openssl\\tempbat.bat", "");
            while (p.HasExited == false) System.Threading.Thread.Sleep(100);
            if (p.ExitCode != 0)
            {
                MessageBox.Show("OpenSSL sign certificate exit code: " + p.ExitCode, "OpenSSL Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }

            string sf = serial.ToString("10000000") + ".pem";
            File.Delete(sf);
            File.Delete("openssl\\tempconfig.cfg");
            File.Delete("openssl\\temprootcert.p12");
            File.Delete("openssl\\temprootcert.pem");
            File.Delete("openssl\\tempcertreq.pem");
            File.Delete("openssl\\tempserial.txt");
            File.Delete("openssl\\tempindex.txt");
            File.Delete("openssl\\tempindex.txt.attr");
            File.Delete("openssl\\tempserial.txt.old");
            File.Delete("openssl\\tempindex.txt.old");
            File.Delete("openssl\\tempindex.txt.attr.old");
            File.Delete("openssl\\tempcfg.cfg");
            File.Delete("openssl\\tempyy.txt");
            File.Delete("openssl\\tempbat.bat");

            // Merge certificate & private key 1
            fs = new FileStream("openssl\\tempcert.pem", FileMode.Open);
            byte[] b1 = new byte[fs.Length];
            fs.Read(b1, 0, (int)fs.Length);
            fs.Close();

            // Merge certificate & private key 2
            fs = new FileStream("openssl\\tempsubcakey.pem", FileMode.Open);
            byte[] b2 = new byte[fs.Length];
            fs.Read(b2, 0, (int)fs.Length);
            fs.Close();

            // Write the merged file
            fs = new FileStream("openssl\\tempcert2.pem", FileMode.Create);
            fs.Write(b1, 0, b1.Length);
            fs.Write(b2, 0, b2.Length);
            fs.Close();

            // Convert to PKCS#12
            p = Run("openssl\\openssl", "pkcs12 -export -in openssl\\tempcert2.pem -out openssl\\tempcert.p12 -name \"" + commonNames[0] + "\" -password pass:PASS");
            while (p.HasExited == false) System.Threading.Thread.Sleep(100);
            if (p.ExitCode != 0)
            {
                MessageBox.Show("OpenSSL convert to PKCS12 exit code: " + p.ExitCode, "OpenSSL Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return null;
            }
            X509Certificate2 Certificate = new X509Certificate2("openssl\\tempcert.p12", "PASS", X509KeyStorageFlags.Exportable | X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet);

            File.Delete("openssl\\tempcert.pem");
            File.Delete("openssl\\tempsubcakey.pem");
            File.Delete("openssl\\tempcert2.pem");
            File.Delete("openssl\\tempcert.p12");

            foreach (string pemfile in Directory.GetFiles("openssl", "*.pem"))
            {
                File.Delete(pemfile);
            }

            return Certificate;
        }

        public static bool IsTrustedCert(X509Certificate2 cert)
        {
            X509Chain chain = new X509Chain();
            chain.ChainPolicy.RevocationMode = X509RevocationMode.NoCheck;
            return (chain.Build(cert) == true && chain.ChainStatus.Length == 0);
        }

        public static bool IsCaCertificate(X509Certificate2 cert)
        {
            // TODO - Not sure how to do this, but want to check for CA extensions.
            if (cert.Extensions.Count == 0) return true;
            foreach (X509Extension ex in cert.Extensions)
            {
                if (ex.GetType() == typeof(X509BasicConstraintsExtension))
                {
                    X509BasicConstraintsExtension ex2 = (X509BasicConstraintsExtension)ex;
                    return ex2.CertificateAuthority;
                }
            }
            return false;
        }

        public static X509Certificate2 GetTrustedRootCert(X509Certificate2 cert)
        {
            X509Chain chain = new X509Chain();
            //chain.ChainPolicy = new X509ChainPolicy();
            chain.ChainPolicy.RevocationMode = X509RevocationMode.NoCheck;
            if (chain.Build(cert) == true)
            {
                X509ChainElement e = chain.ChainElements[chain.ChainElements.Count - 1];
                return e.Certificate;
            }
            return null;
        }

        public static byte[] GetPkcs1PrivateKeyFromCert(X509Certificate2 certificate)
        {
            Directory.SetCurrentDirectory(Application.StartupPath);

            if (certificate.HasPrivateKey == false) return null;

            // Save the certificate private key to file
            byte[] bytes = certificate.Export(X509ContentType.Pkcs12, "PASS");
            FileStream fs = new FileStream("openssl\\tempcert.p12", FileMode.Create);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();

            // Convert the certificate to a PEM file
            System.Diagnostics.Process p;
            p = Run("openssl\\openssl", "pkcs12 -in openssl\\tempcert.p12 -out openssl\\tempcert.pem -passin pass:PASS -nodes");

            while (p.HasExited == false) System.Threading.Thread.Sleep(100);

            // Load the PEM file
            fs = new FileStream("openssl\\tempcert.pem", FileMode.Open);
            bytes = new byte[fs.Length];
            fs.Read(bytes, 0, bytes.Length);
            fs.Close();

            // Delete Files
            File.Delete("openssl\\tempcert.p12");
            File.Delete("openssl\\tempcert.pem");

            string pkcs10 = UTF8Encoding.UTF8.GetString(bytes);
            pkcs10 = pkcs10.Substring(pkcs10.IndexOf("-----BEGIN RSA PRIVATE KEY-----\n") + 32);
            pkcs10 = pkcs10.Substring(0, pkcs10.IndexOf("-----END RSA PRIVATE KEY-----\n"));

            // Convert from Base64 and BER decode
            return Convert.FromBase64String(pkcs10);
        }

        public static byte[] GetPkcs10PrivateKeyFromCert(X509Certificate2 certificate)
        {
            byte[] bytes = GetPkcs1PrivateKeyFromCert(certificate);
            BerSEQUENCE2 sequence = BerSEQUENCE2.DecodeBER(bytes, 0, bytes.Length);
            sequence = (BerSEQUENCE2)sequence.sequence[0];

            /*
            RSAPrivateKey ::= SEQUENCE {
                version Version,                                    0
                modulus INTEGER, -- n                               1
                publicExponent INTEGER, -- e                        2
                privateExponent INTEGER, -- d                       3
                prime1 INTEGER, -- p                                4
                prime2 INTEGER, -- q                                5
                exponent1 INTEGER, -- d mod (p-1)                   6
                exponent2 INTEGER, -- d mod (q-1)                   7
                coefficient INTEGER, -- (inverse of q) mod p        8
            }
            */

            // Encode MPINT (partial)
            // P, Q, N and E parameters
            byte[] P = ((byte[])(sequence.sequence[4]));
            byte[] Q = ((byte[])(sequence.sequence[5]));
            byte[] N = ((byte[])(sequence.sequence[1]));
            byte[] E = ((byte[])(sequence.sequence[2]));

            byte[] buf = null;
            buf = ConcatBuf(buf, IntToBuf(P.Length));
            buf = ConcatBuf(buf, P);
            buf = ConcatBuf(buf, IntToBuf(Q.Length));
            buf = ConcatBuf(buf, Q);
            buf = ConcatBuf(buf, IntToBuf(N.Length));
            buf = ConcatBuf(buf, N);
            buf = ConcatBuf(buf, IntToBuf(E.Length));
            buf = ConcatBuf(buf, E);

            // Save Temp File
            //string s = sequence.GetDebugString();
            //bytes = UTF8Encoding.UTF8.GetBytes(s);
            ///fs = new FileStream("openssl\\tempbin.txt", FileMode.Create);
            ///fs.Write(buf, 0, buf.Length);
            ///fs.Close();

            return buf;
        }

        private static byte[] ConcatBuf(byte[] b1, byte[] b2)
        {
            if (b1 == null) return b2;
            if (b2 == null) return b1;
            byte[] cat = new byte[b1.Length + b2.Length];
            Array.Copy(b1, 0, cat, 0, b1.Length);
            Array.Copy(b2, 0, cat, b1.Length, b2.Length);
            return cat;
        }

        private static byte[] IntToBuf(int i)
        {
            byte[] buf = new byte[4];
            buf[0] = (byte)((i & 0xFF000000) >> 24);
            buf[1] = (byte)((i & 0x00FF0000) >> 16);
            buf[2] = (byte)((i & 0x0000FF00) >> 8);
            buf[3] = (byte)((i & 0x000000FF));
            return buf;
        }

        public class BerSEQUENCE2
        {
            public ArrayList sequence = null;

            public BerSEQUENCE2(ArrayList sequence)
            {
                this.sequence = sequence;
            }

            public override string ToString()
            {
                string s = "{";
                foreach (object o in sequence)
                {
                    if (s.Length != 1) s += ", ";
                    s += o.ToString();
                }
                s += "}";
                return s;
            }

            public string GetDebugString()
            {
                return GetDebugString("");
            }

            public string GetDebugString(string ident)
            {
                StringBuilder sb = new StringBuilder();
                sb.Append(ident).Append("SEQUENCE:\r\n");
                sb.Append(ident).Append("{\r\n");
                foreach (object o in sequence)
                {
                    if (o.GetType() == typeof(BerSEQUENCE2))
                    {
                        sb.Append(((BerSEQUENCE2)o).GetDebugString(ident + "   "));
                    }
                    else if (o.GetType() == typeof(BerOCTETS))
                    {
                        BerOCTETS oc = (BerOCTETS)o;
                        sb.Append(ident).Append("   OCTETS:\r\n");
                        sb.Append(ident).Append("   {\r\n");
                        sb.Append(ident).Append("      ").Append(oc.ToString()).Append("\r\n");
                        sb.Append(ident).Append("   }\r\n");
                    }
                    else if (o.GetType() == typeof(BerOID))
                    {
                        BerOID oid = (BerOID)o;
                        sb.Append(ident).Append("   OID: ").Append(oid.ToString()).Append("\r\n");
                    }
                    else if (o.GetType() == typeof(int))
                    {
                        sb.Append(ident).Append("   INTEGER: ").Append(((int)o).ToString()).Append("\r\n");
                    }
                    else if (o.GetType() == typeof(uint))
                    {
                        sb.Append(ident).Append("   UINTEGER: ").Append(((uint)o).ToString()).Append("\r\n");
                    }
                    else if (o.GetType() == typeof(IPAddress))
                    {
                        sb.Append(ident).Append("   IPADDRESS: ").Append(((IPAddress)o).ToString()).Append("\r\n");
                    }
                }
                sb.Append(ident).Append("}\r\n");
                return sb.ToString();
            }

            public static BerSEQUENCE2 DecodeBER(byte[] buffer, int off, int len)
            {
                int ptr = off;
                ArrayList list = new ArrayList();

                while (ptr < off + len)
                {
                    byte tag = buffer[ptr];
                    int size = buffer[ptr + 1];
                    if (size == 0x81)
                    {
                        // Length is over next byte
                        size = (int)buffer[ptr + 2];
                        ptr += 1;
                    }
                    else if (size == 0x82)
                    {
                        // Length is over two next bytes
                        size = (int)((((int)buffer[ptr + 2]) << (int)8) + (int)(buffer[ptr + 3]));
                        ptr += 2;
                    }
                    switch (tag)
                    {
                        case 0x02: // Integer
                        case 0x04: // Octet String
                            byte[] b = new byte[size];
                            Array.Copy(buffer, ptr + 2, b, 0, size);
                            list.Add(b);
                            //list.Add(new BerOCTETS(b));
                            break;
                        case 0x30: // Sequence
                        case 0xA4: // Application Sequence
                            list.Add(DecodeBER(buffer, ptr + 2, size));
                            break;
                    }
                    ptr += (2 + size);
                }

                return new BerSEQUENCE2(list);
            }

        }

        public class BerOID
        {
            public int[] oid = null;

            public BerOID(int[] oid)
            {
                this.oid = oid;
            }

            public override string ToString()
            {
                string s = "\"";
                for (int i = 0; i < oid.Length; i++)
                {
                    if (s.Length != 1) s += ".";
                    s += oid[i].ToString();
                }
                s += "\"";
                return s;
            }
        }

        public class BerOCTETS
        {
            public byte[] octets = null;
            public bool text = false;

            public BerOCTETS(byte[] octets)
            {
                this.octets = octets;
                text = true;
                foreach (byte b in octets)
                {
                    if (b == 00)
                    {
                        text = false;
                        break;
                    }
                }
            }

            public BerOCTETS(byte[] octets, bool text)
            {
                this.octets = octets;
                this.text = text;
            }

            public string GetString()
            {
                return UTF8Encoding.UTF8.GetString(octets, 0, octets.Length);
            }

            public override string ToString()
            {
                if (text)
                {
                    return "\"" + UTF8Encoding.UTF8.GetString(octets, 0, octets.Length) + "\"";
                }
                else
                {
                    string s = "\"";
                    for (int i = 0; i < octets.Length; i++)
                    {
                        if (s.Length != 1) s += ",";
                        s += octets[i].ToString("X2");
                    }
                    s += "\"";
                    return s;
                }
            }
        }

        public enum SignatureAlgorithm
        {
            MD2,
            MD4,
            MD5,
            SHA1,
            SHA256,
            SHA384,
            SHA512,
        }

        public static X509Certificate2 CreateSelfSignedCert(string commonName, string orgName, string countryCode, int keysize)
        {
            return CreateSelfSignedCert(commonName, orgName, countryCode, keysize, SignatureAlgorithm.SHA1);
        }

        public static X509Certificate2 CreateSelfSignedCert(string commonName, string orgName, string countryCode, int keysize, SignatureAlgorithm digest)
        {
            Directory.SetCurrentDirectory(Application.StartupPath);

            string cfg = "[ req ]\r\ndefault_bits = " + keysize.ToString() + "\r\ndistinguished_name = req_distinguished_name\r\nencrypt_rsa_key = no\r\ndefault_md = sha1\r\n[ req_distinguished_name ]\r\ncountryName = Country Name (2 letter code)\r\ncountryName_default = " + countryCode + "\r\ncountryName_value = " + countryCode + "\r\norganizationName = Organization Name (eg, company)\r\norganizationName_value = " + orgName + "\r\ncommonName = Common Name (eg, YOUR name)\r\ncommonName_value = " + commonName + "\r\n";
            FileStream fs = new FileStream("openssl\\tempconfig.cfg", FileMode.Create);
            byte[] bytes = UTF8Encoding.UTF8.GetBytes(cfg);
            fs.Write(bytes, 0, bytes.Length);
            fs.Close();
            System.Diagnostics.Process p;
            p = Run("openssl\\openssl", String.Format("req -config openssl\\tempconfig.cfg -new -x509 -keyout openssl\\tempcert.cer -out openssl\\tempcert.cer -days 7300 -{0}",
                digest.ToString().ToLower()));
            while (p.HasExited == false) System.Threading.Thread.Sleep(100);
            p = Run("openssl\\openssl", "pkcs12 -export -in openssl\\tempcert.cer -out openssl\\tempcert.p12 -name \"" + commonName + "\" -password pass:PASS");
            while (p.HasExited == false) System.Threading.Thread.Sleep(100);
            X509Certificate2 Certificate = new X509Certificate2("openssl\\tempcert.p12", "PASS", X509KeyStorageFlags.Exportable | X509KeyStorageFlags.MachineKeySet | X509KeyStorageFlags.PersistKeySet);
            File.Delete("openssl\\tempconfig.cfg");
            File.Delete("openssl\\tempcert.cer");
            File.Delete("openssl\\tempcert.p12");
            return Certificate;
        }

        public static bool IsKeyExportable(X509Certificate2 cert)
        {
            if (cert.HasPrivateKey == false) return false;

            // TODO: Find a better way to check this.
            try
            {
                cert.Export(X509ContentType.Pkcs12, "PASS");
            }
            catch (Exception)
            {
                return false;
            }
            return true;
        }

    }
}
