// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Runtime.InteropServices;
using System.Text;

namespace ManageabilityStack
{
    public class HeciWrapper
    {
        #region DLL Imports
        [DllImport("HeciWrapper.dll")]
        static extern int HECI_Init(byte[] guid);
        [DllImport("HeciWrapper.dll")]
        static extern void HECI_Deinit();
        [DllImport("HeciWrapper.dll")]
        static extern int HECI_doIoctl(int code, IntPtr inbuf, int inlen, IntPtr outbuf, int outlen);
        [DllImport("HeciWrapper.dll")]
        static extern int HECI_ReceiveMessage(IntPtr buffer, int len, int timeout);
        [DllImport("HeciWrapper.dll")]
        static extern int HECI_SendMessage(IntPtr buffer, int len, int timeout);
        [DllImport("HeciWrapper.dll")]
        static extern ulong HECI_GetBufferSize();
        [DllImport("HeciWrapper.dll")]
        static extern IntPtr HECI_GetHandle();
        #endregion

        #region HECI Support

        public static Guid HECI = new Guid("{E2D1FF34-3458-49A9-88DA-8E6915CE9BE5}"); // Get HECI driver version
        public static Guid WGGD = new Guid("{05B79A6F-4628-4D7F-899D-A91514CB32AB}"); // Connect to HECI Watchdog
        public static Guid PTHI = new Guid("{12F80028-B4B7-4B2D-ACA8-46E0FF65814C}"); // Connect to Intel AMT

        private enum FILE_ACCESS
        {
            ANY_ACCESS = 0,
            SPECIAL_ACCESS = 0,
            FILE_READ_ACCESS = 1,
            FILE_WRITE_ACCESS = 2
        }

        private enum METHOD
        {
            BUFFERED = 0,
            IN_DIRECT = 1,
            OUT_DIRECT = 2,
            NEITHER = 3
        }

        private int CTL_CODE(int DeviceType, int Function, METHOD Method, FILE_ACCESS Access)
        {
            return ((DeviceType) << 16) | (((int)Access) << 14) | ((Function) << 2) | ((int)Method);
        }

        public enum HECISTATUS
        {
            HECI_STATUS_OK = 0x0,
            HECI_STATUS_GENERAL_ERROR = 0x2000,
            HECI_STATUS_LOCATE_DEVICE_ERROR,
            HECI_STATUS_MEMORY_ACCESS_ERROR,
            HECI_STATUS_WRITE_REGISTER_ERROR,
            HECI_STATUS_MEMORY_ALLOCATION_ERROR,
            HECI_STATUS_BUFFER_OVEREFLOW_ERROR,
            HECI_STATUS_NOT_ENOUGH_MEMORY,
            HECI_STATUS_MSG_TRANSMISSION_ERROR,
            HECI_STATUS_VERSION_MISMATCH,
            HECI_STATUS_UNEXPECTED_INTERRUPT_REASON,
            HECI_STATUS_TIMEOUT_ERROR,
            HECI_STATUS_UNEXPECTED_RESPONSE,
            HECI_STATUS_UNKNOWN_MESSAGE,
            HECI_STATUS_CANNOT_FOUND_HOST_CLIENT,
            HECI_STATUS_CANNOT_FOUND_ME_CLIENT,
            HECI_STATUS_CLIENT_ALREADY_CONNECTED,
            HECI_STATUS_NO_FREE_CONNECTION,
            HECI_STATUS_ILLEGAL_PARAMETER,
            HECI_STATUS_FLOW_CONTROL_ERROR,
            HECI_STATUS_NO_MESSAGE,
            HECI_STATUS_BUFFER_TOO_LARGE,
            HECI_STATUS_BUFFER_TOO_SMALL,
            HECI_STATUS_BUFFER_NOT_EMPTY,
        };

        public enum AMTSTATUS
        {
            SUCCESS = 0,
            INTERNAL_ERROR = 1,
            NOT_READY = 2,
            INVALID_AMT_MODE = 3,
            INVALID_MESSAGE_LENGTH = 4,
            TABLE_FINGERPRINT_NOT_AVAILABLE = 5,
            INTEGRITY_CHECK_FAILED = 6,
            UNSUPPORTED_ISVS_VERSION = 7,
            INVALID_REGISTRATION_DATA = 9,
            APPLICATION_DOES_NOT_EXIST = 10,
            NOT_ENOUGH_STORAGE = 11,
            INVALID_NAME = 12,
            BLOCK_DOES_NOT_EXIST = 13,
            INVALID_BYTE_OFFSET = 14,
            INVALID_BYTE_COUNT = 15,
            NOT_PERMITTED = 16,
            NOT_OWNER = 17,
            BLOCK_LOCKED_BY_OTHER = 18,
            BLOCK_NOT_LOCKED = 19,
            INVALID_GROUP_PERMISSIONS = 20,
            GROUP_DOES_NOT_EXIST = 21,
            INVALID_MEMBER_COUNT = 22,
            MAX_LIMIT_REACHED = 23,
            INVALID_AUTH_TYPE = 24,
            AUTHENTICATION_FAILED = 25,
            INVALID_DHCP_MODE = 26,
            INVALID_IP_ADDRESS = 27,
            INVALID_DOMAIN_NAME = 28,
            UNSUPPORTED_VERSION = 29,
            REQUEST_UNEXPECTED = 30,
            INVALID_TABLE_TYPE = 31,
            INVALID_PROVISIONING_MODE = 32,
            UNSUPPORTED_OBJECT = 33,
            INVALID_TIME = 34,
            INVALID_INDEX = 35,
            INVALID_PARAMETER = 36,
            INVALID_NETMASK = 37,
            FLASH_WRITE_LIMIT_EXCEEDED = 38,
            INVALID_IMAGE_LENGTH = 39,
            INVALID_IMAGE_SIGNATURE = 40,
            PROPOSE_ANOTHER_VERSION = 41,
            INVALID_PID_FORMAT = 42,
            INVALID_PPS_FORMAT = 43,
            BIST_COMMAND_BLOCKED = 44,
            CONNECTION_FAILED = 45,
            CONNECTION_TOO_MANY = 46,
            RNG_GENERATION_IN_PROGRESS = 47,
            RNG_NOT_READY = 48,
            CERTIFICATE_NOT_READY = 49,
            AUDIT_RECORD_NOT_PRERSENT = 50,
            REMOTE_ACCESS_NOT_GRANTED = 512,
            REMOTE_ACCESS_HOST_VPN_IS_DISABLED = 513,
            NETWORK_IF_ERROR_BASE = 2048,
            INVALID_HANDLE = 2053,
            DATA_MISSING = 2057,
            UNSUPPORTED = 2066,
            NOT_FOUND = 2068,
            AUDIT_FAIL = 2075,
            BLOCKING_COMPONENT = 2076,
            SDK_DEFINED_ERROR_BASE = 4096
        }

        #endregion

        public object MasterLock = new object();

        public enum IntelPlatform
        {
            Unknown,
            IntelAMT,
            IntelConsumer
        }

        public byte major;
        public byte minor;
        public byte hotfix;
        public ushort build;

        public int maxMessageLength;
        public byte protocolVersion;

        public MeCoreVersions Versions;
        public HeciMeInfo MeInfo = null;
        public HeciClientConnect ClientConnect = null;
        public IntelPlatform Platform = IntelPlatform.Unknown;
        private string BridgeHostname = null;
        private ushort BridgePort = 10000;
        private bool supported = true;

        private decimal VersionCache = 0;
        public decimal GetVersion() { return VersionCache; }
        public static int GetHeciResponseMessageLength(byte[] buf) { if (buf.Length < 12) return 0; else return BitConverter.ToInt32(buf, 8) + 12; }
        public static int GetHeciRequestMessageLength(byte[] buf) { if (buf.Length < 12) return 0; else return BitConverter.ToInt32(buf, 8) + 12; }

        public HeciWrapper(string hostname, ushort port)
        {
            BridgeHostname = hostname;
            BridgePort = port;

            MeInfo = new HeciMeInfo(this);
            if (MeInfo != null)
            {
                Platform = IntelPlatform.IntelAMT;
                Versions = MeInfo.GetVersions();
                MeProvisioningMode pm = MeInfo.GetProvisioningMode();
                if (pm == null)
                {
                    MeInfo = null;
                    Platform = IntelPlatform.IntelConsumer;
                }
                if (Versions != null && Versions.Versions.ContainsKey("AMT"))
                {
                    string v = (string)(Versions.Versions["AMT"]);
                    string[] vv = v.Split('.');
                    VersionCache = decimal.Parse(string.Format("{0}.{1}", vv[0], vv[1]));
                    byte.TryParse(vv[0], out major);
                    byte.TryParse(vv[1], out minor);
                }
            }
        }

        public HeciWrapper()
        {
            if (Connect() == true)
            {
                MeInfo = new HeciMeInfo(this);
                if (major >= 4) ClientConnect = new HeciClientConnect(this);
            }

            if (MeInfo != null)
            {
                Platform = IntelPlatform.IntelAMT;
                Versions = MeInfo.GetVersions();
                MeProvisioningMode pm = MeInfo.GetProvisioningMode();
                if (pm == null)
                {
                    Disconnect();
                    System.Threading.Thread.Sleep(50);
                    Connect();
                    MeInfo = null;
                    Platform = IntelPlatform.IntelConsumer;
                }
                if (Versions != null && Versions.Versions.ContainsKey("AMT"))
                {
                    string v = (string)(Versions.Versions["AMT"]);
                    string[] vv = v.Split('.');
                    VersionCache = decimal.Parse(string.Format("{0}.{1}", vv[0], vv[1]));
                }
                else
                {
                    VersionCache = decimal.Parse(string.Format("{0}.{1}", major, minor));
                }
            }
        }

        public void Dispose()
        {
            Disconnect();
        }

        public static bool CheckHeciWrapper()
        {
            System.IO.Directory.SetCurrentDirectory(System.Windows.Forms.Application.StartupPath);
            return File.Exists("HeciWrapper.dll");
        }

        private bool Connect()
        {
            lock (MasterLock)
            {
                supported = (HECI_Init(HECI.ToByteArray()) == 0);
                if (supported == false) return false;

                // Get the platform HECI version
                IntPtr mem = Marshal.AllocHGlobal(5);
                int r = HECI_doIoctl(CTL_CODE(0x8000, 0x800, METHOD.BUFFERED, FILE_ACCESS.FILE_READ_ACCESS | FILE_ACCESS.FILE_WRITE_ACCESS), IntPtr.Zero, 0, mem, 5);
                if (r != 5)
                {
                    Marshal.FreeHGlobal(mem);
                    return false;
                }
                major = Marshal.ReadByte(mem, 0);
                minor = Marshal.ReadByte(mem, 1);
                hotfix = Marshal.ReadByte(mem, 2);
                build = (ushort)Marshal.ReadInt16(mem, 3);

                // Perform client connection
                byte[] id = PTHI.ToByteArray();
                IntPtr mem2 = Marshal.AllocHGlobal(id.Length);
                Marshal.Copy(id, 0, mem2, id.Length);
                r = HECI_doIoctl(CTL_CODE(0x8000, 0x801, METHOD.BUFFERED, FILE_ACCESS.FILE_READ_ACCESS | FILE_ACCESS.FILE_WRITE_ACCESS), mem2, id.Length, mem, 5);
                if (r != 5)
                {
                    Marshal.FreeHGlobal(mem);
                    Marshal.FreeHGlobal(mem2);
                    return false;
                }
                maxMessageLength = Marshal.ReadInt32(mem, 0);
                protocolVersion = Marshal.ReadByte(mem, 4);
                Marshal.FreeHGlobal(mem);
                Marshal.FreeHGlobal(mem2);

                ManageabilityStack.AmtDebugForm.LogEvent(this, ManageabilityStack.AmtDebugForm.EventType.Information, "Connected to HECI driver.");
                return true;
            }
        }

        private void Disconnect()
        {
            lock (MasterLock)
            {
                if (supported == true)
                {
                    HECI_Deinit();
                    supported = false;
                }
            }
        }
        public int SendHeciMessage(byte[] buffer, int timeout)
        {
            IntPtr mem = Marshal.AllocHGlobal(buffer.Length);
            Marshal.Copy(buffer, 0, mem, buffer.Length);
            int r = HECI_SendMessage(mem, buffer.Length, timeout);
            Marshal.FreeHGlobal(mem);
            return r;
        }

        public byte[] ReceiveMessage(int timeout)
        {
            IntPtr mem = Marshal.AllocHGlobal(maxMessageLength);
            int r = HECI_ReceiveMessage(mem, maxMessageLength, timeout);
            if (r <= 0) return null;
            byte[] buf = new byte[r];
            Marshal.Copy(mem, buf, 0, r);
            Marshal.FreeHGlobal(mem);
            return buf;
        }

        public byte[] ExecuteCommand(byte[] Buf, uint ReceiveBufLng)
        {
            lock (MasterLock)
            {
                if (BridgeHostname != null)
                {
                    return ExecuteCommandTCP(Buf, ReceiveBufLng);
                }
                else
                {
                    int r = SendHeciMessage(Buf, 4000);
                    if (r == 0) return null;
                    return ReceiveMessage(4000);
                }
            }
        }

        private byte[] ExecuteCommandTCP(byte[] Buf, uint ReceiveBufLng)
        {
            byte[] outBuffer = new byte[ReceiveBufLng];
            TcpClient Client = new TcpClient();
            try
            {
                Client.Connect(BridgeHostname, BridgePort);
            }
            catch (SocketException)
            {
                return null;
            }
            Client.Client.ReceiveTimeout = 5000;
            Client.Client.Send(Buf, Buf.Length, SocketFlags.None);
            int receivedBytes = -1;
            int totalReceivedBytes = 0;

            do
            {
                try
                {
                    receivedBytes = -1;
                    receivedBytes = Client.Client.Receive(outBuffer, totalReceivedBytes, (int)(ReceiveBufLng - totalReceivedBytes), SocketFlags.None);
                    totalReceivedBytes += receivedBytes;
                }
                catch (Exception) { }

                if (totalReceivedBytes >= 16)
                {
                    int packetlength = GetHeciResponseMessageLength(outBuffer);
                    if (totalReceivedBytes >= packetlength)
                    {
                        int s = (int)(packetlength);
                        if (outBuffer.Length != s)
                        {
                            byte[] b = new byte[s];
                            Array.Copy(outBuffer, b, s);
                            outBuffer = b;
                        }
                        break;
                    }
                }
            }
            while (receivedBytes > 0 && totalReceivedBytes < ReceiveBufLng);
            Client.Close();

            if (totalReceivedBytes == 0) return null;
            return outBuffer;
        }


    }

    public class HeciMeInfo
    {
        private HeciWrapper parent;

        #region Intel Management Engine Values
        const uint AMT_MAJOR_VERSION = 1;
        const uint AMT_MINOR_VERSION = 1;

        public enum HeciCommand
        {
            STATE_GET_AMT_STATE_REQUEST = 0x01000001,
            STATE_GET_AMT_STATE_RESPONSE = 0x01800001,
            CODE_VERSIONS_REQUEST = 0x0400001A,
            CODE_VERSIONS_RESPONSE = 0x0480001A,
            PROVISIONING_MODE_REQUEST = 0x04000008,
            PROVISIONING_MODE_RESPONSE = 0x04800008,
            PROVISIONING_STATE_REQUEST = 0x04000011,
            PROVISIONING_STATE_RESPONSE = 0x04800011,
            GET_MAC_ADDRESSES_REQUEST = 0x04000025,
            GET_MAC_ADDRESSES_RESPONSE = 0x04800025,
            GENERATE_RNG_SEED_REQUEST = 0x04000028,
            GENERATE_RNG_SEED_RESPONSE = 0x04800028,
            GET_RNG_SEED_STATUS_REQUEST = 0x0400002E,
            GET_RNG_SEED_STATUS_RESPONSE = 0x0480002E,
            GET_ZERO_TOUCH_ENABLED_REQUEST = 0x04000030,
            GET_ZERO_TOUCH_ENABLED_RESPONSE = 0x04800030,
            GET_PROVISIONING_TLS_MODE_REQUEST = 0x0400002B,
            GET_PROVISIONING_TLS_MODE_RESPONSE = 0x0480002B,
            START_CONFIGURATION_REQUEST = 0x04000029,
            START_CONFIGURATION_RESPONSE = 0x04800029,
            SET_PROVISIONING_SERVER_OTP_REQUEST = 0x0400002A,
            SET_PROVISIONING_SERVER_OTP_RESPONSE = 0x0480002A,
            SET_DNS_SUFFIX_REQUEST = 0x0400002F,
            SET_DNS_SUFFIX_RESPONSE = 0x0480002F,
            ENUMERATE_HASH_HANDLES_REQUEST = 0x0400002C,
            ENUMERATE_HASH_HANDLES_RESPONSE = 0x0480002C,
            GET_CERTHASH_ENTRY_REQUEST = 0x0400002D,
            GET_CERTHASH_ENTRY_RESPONSE = 0x0480002D,
            GET_PKI_FQDN_SUFFIX_REQUEST = 0x04000036,
            GET_PKI_FQDN_SUFFIX_RESPONSE = 0x04800036,
            GET_AUDIT_RECORD_REQUEST = 0x04000027,
            GET_AUDIT_RECORD_RESPONSE = 0x04800027,
            GET_PID_REQUEST = 0x0400003B,
            GET_PID_RESPONSE = 0x0480003B,
            SET_CONF_SERVER_FQDN_REQUEST = 0x04000034,
            SET_CONF_SERVER_FQDN_RESPONSE = 0x04800034,
            GET_CONF_SERVER_FQDN_REQUEST = 0x04000033,
            GET_CONF_SERVER_FQDN_RESPONSE = 0x04800033,
            SET_CERTHASH_STATE_REQUEST = 0x04000037,
            SET_CERTHASH_STATE_RESPONSE = 0x04800037,
            ADD_CERTHASH_ENTRY_REQUEST = 0x04000038,
            ADD_CERTHASH_ENTRY_RESPONSE = 0x04800038,
            DELETE_CERTHASH_ENTRY_REQUEST = 0x04000039,
            DELETE_CERTHASH_ENTRY_RESPONSE = 0x04800039,
            SET_ZTC_MODE_REQUEST = 0x0400003A,
            SET_ZTC_MODE_RESPONSE = 0x0480003A,
        }
        #endregion

        public enum HashAlgorithmEnum
        {
            MD5 = 0,  // 16 bytes
            SHA1,     // 20 bytes
            SHA256,   // 32 bytes
            SHA512,   // 64 bytes
        };

        public static int GetHashSize(HashAlgorithmEnum HashAlgorithm)
        {
            if (HashAlgorithm == HashAlgorithmEnum.MD5) return 16;
            if (HashAlgorithm == HashAlgorithmEnum.SHA1) return 20;
            if (HashAlgorithm == HashAlgorithmEnum.SHA256) return 32;
            if (HashAlgorithm == HashAlgorithmEnum.SHA512) return 64;
            return 0;
        }

        public enum ProvisioningStateEnum
        {
            Pre = 0,
            In = 1,
            Post = 2
        };

        public enum ProvisioningTlsModeEnum
        {
            NotReady = 0,
            PSK = 1,
            PKI = 2
        };

        public enum RandomGeneratorStatus
        {
            Exists = 0,
            InProgress = 1,
            NotExist = 2
        };

        public HeciMeInfo(HeciWrapper parent)
        {
            this.parent = parent;
        }

        private object ExecuteCommand(HeciCommand command, byte[] data)
        {
            lock (parent.MasterLock)
            {
                if (data == null) data = new byte[0];
                ManageabilityStack.AmtDebugForm.LogEvent(this, ManageabilityStack.AmtDebugForm.EventType.Information, "Sending HECI request: " + command.ToString(), ManageabilityStack.Utils.BytesToHex(data, 8));
                MemoryStream m = new MemoryStream();
                m.Write(BitConverter.GetBytes((byte)AMT_MAJOR_VERSION), 0, 1);   // Major Version
                m.Write(BitConverter.GetBytes((byte)AMT_MINOR_VERSION), 0, 1);   // Minor Version
                m.Write(BitConverter.GetBytes((short)0), 0, 2);                  // Reserved
                m.Write(BitConverter.GetBytes((uint)command), 0, 4);             // Command
                m.Write(BitConverter.GetBytes(data.Length), 0, 4);               // Data Length
                m.Write(data, 0, data.Length);                                   // Data
                byte[] buf = m.ToArray();

                buf = parent.ExecuteCommand(buf, 2000);                          // Execute
                if (buf == null) return null;

                command = (HeciCommand)BitConverter.ToInt32(buf, 4);
                int length = BitConverter.ToInt32(buf, 8);
                HeciWrapper.AMTSTATUS status = (HeciWrapper.AMTSTATUS)BitConverter.ToInt32(buf, 12);

                ManageabilityStack.AmtDebugForm.LogEvent(this, ManageabilityStack.AmtDebugForm.EventType.Information, "Got HECI message: " + command.ToString(), ManageabilityStack.Utils.BytesToHex(buf, 8));

                switch (command)
                {
                    case HeciCommand.STATE_GET_AMT_STATE_RESPONSE:
                        return true;
                    case HeciCommand.CODE_VERSIONS_RESPONSE:
                        return new MeCoreVersions(buf);
                    case HeciCommand.PROVISIONING_MODE_RESPONSE:
                        return new MeProvisioningMode(buf);
                    case HeciCommand.PROVISIONING_STATE_RESPONSE:
                        return (ProvisioningStateEnum)BitConverter.ToInt32(buf, 16);
                    case HeciCommand.GET_MAC_ADDRESSES_RESPONSE:
                        MeMacAddresses macs = new MeMacAddresses();
                        if (length >= 10)
                        {
                            macs.Dedicated = new byte[6];
                            Array.Copy(buf, 16, macs.Dedicated, 0, 6);
                        }
                        if (length >= 16)
                        {
                            macs.Host = new byte[6];
                            Array.Copy(buf, 22, macs.Host, 0, 6);
                        }
                        return macs;
                    case HeciCommand.GENERATE_RNG_SEED_RESPONSE:
                        return true;
                    case HeciCommand.GET_RNG_SEED_STATUS_RESPONSE:
                        return (RandomGeneratorStatus)BitConverter.ToInt32(buf, 16);
                    case HeciCommand.GET_ZERO_TOUCH_ENABLED_RESPONSE:
                        return (BitConverter.ToInt32(buf, 16) != 0);
                    case HeciCommand.GET_PROVISIONING_TLS_MODE_RESPONSE:
                        return (ProvisioningTlsModeEnum)BitConverter.ToInt32(buf, 16);
                    case HeciCommand.START_CONFIGURATION_RESPONSE:
                        return true;
                    case HeciCommand.SET_PROVISIONING_SERVER_OTP_RESPONSE:
                        return true;
                    case HeciCommand.SET_DNS_SUFFIX_RESPONSE:
                        return true;
                    case HeciCommand.ENUMERATE_HASH_HANDLES_RESPONSE:
                        return buf;
                    case HeciCommand.GET_CERTHASH_ENTRY_RESPONSE:
                        return new MeHashEntry(buf);
                    case HeciCommand.GET_PKI_FQDN_SUFFIX_RESPONSE:
                        if (buf.Length == 16) return null;
                        return UTF8Encoding.UTF8.GetString(buf, 16, buf.Length - 16);
                    case HeciCommand.GET_AUDIT_RECORD_RESPONSE:
                        if (buf.Length == 16) return null;
                        return new MeAuditRecord(buf);
                    case HeciCommand.GET_PID_RESPONSE:
                        if (buf.Length == 16) return null;
                        return UTF8Encoding.UTF8.GetString(buf, 16, buf.Length - 16);
                    case HeciCommand.GET_CONF_SERVER_FQDN_RESPONSE:
                        if (buf.Length == 16) return null;
                        ushort strlen = BitConverter.ToUInt16(buf, 16);
                        if (strlen > 0) return UTF8Encoding.UTF8.GetString(buf, 18, strlen);
                        return null;
                }

                return null;
            }
        }

        private int SendCommand(HeciCommand command, byte[] data)
        {
            if (data == null) data = new byte[0];
            ManageabilityStack.AmtDebugForm.LogEvent(this, ManageabilityStack.AmtDebugForm.EventType.Information, "Sending HECI request: " + command.ToString(), ManageabilityStack.Utils.BytesToHex(data, 8));
            MemoryStream m = new MemoryStream();
            m.Write(BitConverter.GetBytes((byte)AMT_MAJOR_VERSION), 0, 1);   // Major Version
            m.Write(BitConverter.GetBytes((byte)AMT_MINOR_VERSION), 0, 1);   // Minor Version
            m.Write(BitConverter.GetBytes((short)0), 0, 2);                  // Reserved
            m.Write(BitConverter.GetBytes((uint)command), 0, 4);             // Command
            m.Write(BitConverter.GetBytes(data.Length), 0, 4);               // Data Length
            m.Write(data, 0, data.Length);                                   // Data
            byte[] buf = m.ToArray();
            return parent.SendHeciMessage(buf, 2000);                        // Send the data
        }

        public bool GetAmtState()
        {
            lock (parent.MasterLock)
            {
                object o = ExecuteCommand(HeciCommand.STATE_GET_AMT_STATE_REQUEST, new Guid("{89EC5009-1CDE-41F2-880A-274BACBC1159}").ToByteArray());
                if (o == null) return false;
                return (bool)o;
            }
        }

        private MeCoreVersions VersionsCache = null;
        public MeCoreVersions GetVersions()
        {
            if (VersionsCache != null) return VersionsCache;
            VersionsCache = (MeCoreVersions)ExecuteCommand(HeciCommand.CODE_VERSIONS_REQUEST, null);
            return VersionsCache;
        }

        public MeProvisioningMode GetProvisioningMode()
        {
            return (MeProvisioningMode)ExecuteCommand(HeciCommand.PROVISIONING_MODE_REQUEST, null);
        }

        public ProvisioningStateEnum GetProvisioningState()
        {
            return (ProvisioningStateEnum)ExecuteCommand(HeciCommand.PROVISIONING_STATE_REQUEST, null);
        }

        public MeMacAddresses GetMacAddresses()
        {
            return (MeMacAddresses)ExecuteCommand(HeciCommand.GET_MAC_ADDRESSES_REQUEST, null);
        }

        public bool GenerateRandomSeed()
        {
            return (bool)ExecuteCommand(HeciCommand.GENERATE_RNG_SEED_REQUEST, null);
        }

        public RandomGeneratorStatus GetRandomGeneratorStatus()
        {
            return (RandomGeneratorStatus)ExecuteCommand(HeciCommand.GET_RNG_SEED_STATUS_REQUEST, null);
        }

        public bool GetZeroTouchEnabled()
        {
            return (bool)ExecuteCommand(HeciCommand.GET_ZERO_TOUCH_ENABLED_REQUEST, null);
        }

        public RandomGeneratorStatus GetProvisioningTlsModeRequest()
        {
            return (RandomGeneratorStatus)ExecuteCommand(HeciCommand.GET_PROVISIONING_TLS_MODE_REQUEST, null);
        }

        public bool StartConfiguration()
        {
            return (bool)ExecuteCommand(HeciCommand.START_CONFIGURATION_REQUEST, null);
        }

        public bool SetProvisioningServerOTP(string password)
        {
            return (bool)ExecuteCommand(HeciCommand.SET_PROVISIONING_SERVER_OTP_REQUEST, UTF8Encoding.UTF8.GetBytes(password));
        }

        public bool SetDnsSuffix(string suffix)
        {
            return (bool)ExecuteCommand(HeciCommand.SET_DNS_SUFFIX_REQUEST, UTF8Encoding.UTF8.GetBytes(suffix));
        }

        public MeHashEntry[] EnumerateHashHandles()
        {
            byte[] buf = (byte[])ExecuteCommand(HeciCommand.ENUMERATE_HASH_HANDLES_REQUEST, null);

            if (buf == null) return new MeHashEntry[0];
            int handleCount = 0;
            ArrayList l = new ArrayList();
            if (buf.Length > 16)
            {
                handleCount = (int)BitConverter.ToInt32(buf, 16);
                for (int i = 0; i < handleCount; i++)
                {
                    uint handle = BitConverter.ToUInt32(buf, 20 + (i * 4));
                    MeHashEntry he = GetHashEntry(handle);
                    he.Handle = handle;
                    l.Add(he);
                }
            }
            return (MeHashEntry[])l.ToArray(typeof(MeHashEntry));
        }

        private MeHashEntry GetHashEntry(uint handle)
        {
            return (MeHashEntry)ExecuteCommand(HeciCommand.GET_CERTHASH_ENTRY_REQUEST, BitConverter.GetBytes(handle));
        }

        public string GetPkiFqdnSuffix()
        {
            return (string)ExecuteCommand(HeciCommand.GET_PKI_FQDN_SUFFIX_REQUEST, null);
        }

        public MeAuditRecord GetAuditRecord()
        {
            return (MeAuditRecord)ExecuteCommand(HeciCommand.GET_AUDIT_RECORD_REQUEST, null);
        }

        public string GetPID()
        {
            return (string)ExecuteCommand(HeciCommand.GET_PID_REQUEST, null);
        }

        public string GetConfigurationServerFqdn()
        {
            return (string)ExecuteCommand(HeciCommand.GET_CONF_SERVER_FQDN_REQUEST, null);
        }

    }

    public class HeciClientConnect
    {
        #region Client Connection Values
        const byte CP_MAJOR_VERSION = 1;
        const byte CP_MINOR_VERSION = 1;

        public enum HeciCommands
        {
            GET_ENROLLMENT_STATE_REQUEST_COMMAND = 0x07000006,
            GET_ENROLLMENT_STATE_RESPONSE_COMMAND = 0x07800006,
            ENUMERATE_PROFILES_REQUEST_COMMAND = 0x07000000,
            ENUMERATE_PROFILES_RESPONSE_COMMAND = 0x07800000,
            GET_PROFILE_REQUEST_COMMAND = 0x07000001,
            GET_PROFILE_RESPONSE_COMMAND = 0x07800001,
            SET_ACTIVE_PROFILE_REQUEST_COMMAND = 0x07000005,
            SET_ACTIVE_PROFILE_RESPONSE_COMMAND = 0x07800005,
            UNENROLL_REQUEST_COMMAND = 0x07000008,
            UNENROLL_RESPONSE_COMMAND = 0x07800008,
            GET_REMOTE_ACCESS_CONNECTION_STATUS_REQUEST_COMMAND = 0x04000046,
            GET_REMOTE_ACCESS_CONNECTION_STATUS_RESPONSE_COMMAND = 0x04800046,
            OPEN_USER_INITIATED_CONNECTION_REQUEST_COMMAND = 0X04000044,
            OPEN_USER_INITIATED_CONNECTION_RESPONSE_COMMAND = 0X04800044,
            CLOSE_USER_INITIATED_CONNECTION_REQUEST_COMMAND = 0x04000045,
            CLOSE_USER_INITIATED_CONNECTION_RESPONSE_COMMAND = 0x04800045
        }
        #endregion

        private HeciWrapper parent;

        public HeciClientConnect(HeciWrapper parent)
        {
            this.parent = parent;
        }

        private int SendCommand(HeciCommands command, byte[] data)
        {
            if (data == null) data = new byte[0];
            ManageabilityStack.AmtDebugForm.LogEvent(this, ManageabilityStack.AmtDebugForm.EventType.Information, "Sending HECI request: " + command.ToString(), ManageabilityStack.Utils.BytesToHex(data, 8));
            MemoryStream m = new MemoryStream();
            m.Write(BitConverter.GetBytes((byte)CP_MAJOR_VERSION), 0, 1);   // Major Version
            m.Write(BitConverter.GetBytes((byte)CP_MINOR_VERSION), 0, 1);   // Minor Version
            m.Write(BitConverter.GetBytes((short)0), 0, 2);                 // Reserved
            m.Write(BitConverter.GetBytes((uint)command), 0, 4);            // Command
            m.Write(BitConverter.GetBytes(data.Length), 0, 4);              // Data Length
            m.Write(data, 0, data.Length);                                  // Data
            byte[] buf = m.ToArray();
            return parent.SendHeciMessage(buf, 2000);                       // Send the data
        }

        private object ReadCommand()
        {
            byte[] buf = parent.ReceiveMessage(20000);
            if (buf == null) return null;
            HeciCommands command = (HeciCommands)BitConverter.ToInt32(buf, 4);
            int length = BitConverter.ToInt32(buf, 8);

            ManageabilityStack.AmtDebugForm.LogEvent(this, ManageabilityStack.AmtDebugForm.EventType.Information, "Got HECI message: " + command.ToString(), ManageabilityStack.Utils.BytesToHex(buf, 8));

            switch (command)
            {
                case HeciCommands.GET_REMOTE_ACCESS_CONNECTION_STATUS_RESPONSE_COMMAND:
                    return new RemoteAccessStatus(buf);
                case HeciCommands.GET_ENROLLMENT_STATE_RESPONSE_COMMAND:
                    return new EnrollementState(buf);
                case HeciCommands.ENUMERATE_PROFILES_RESPONSE_COMMAND:
                    MspProfile[] Profiles = new MspProfile[buf[16]];
                    for (int i = 0; i < Profiles.Length; i++)
                    {
                        uint handle = (uint)BitConverter.ToInt32(buf, 17 + (i * 4));
                        int r = SendCommand(HeciCommands.GET_PROFILE_REQUEST_COMMAND, BitConverter.GetBytes(handle));
                        Profiles[i] = (MspProfile)ReadCommand();
                        Profiles[i].Handle = handle;
                    }
                    return Profiles;
                case HeciCommands.GET_PROFILE_RESPONSE_COMMAND:
                    return new MspProfile(buf);
                case HeciCommands.SET_ACTIVE_PROFILE_RESPONSE_COMMAND:
                    return true;
                case HeciCommands.UNENROLL_RESPONSE_COMMAND:
                    return true;
                case HeciCommands.OPEN_USER_INITIATED_CONNECTION_RESPONSE_COMMAND:
                    return true;
                case HeciCommands.CLOSE_USER_INITIATED_CONNECTION_RESPONSE_COMMAND:
                    return true;
            }

            return null;
        }

        public RemoteAccessStatus GetConnectionStatus()
        {
            lock (parent.MasterLock)
            {
                int r = SendCommand(HeciCommands.GET_REMOTE_ACCESS_CONNECTION_STATUS_REQUEST_COMMAND, null);
                return (RemoteAccessStatus)ReadCommand();
            }
        }

        public EnrollementState GetEnrollmentState()
        {
            lock (parent.MasterLock)
            {
                int r = SendCommand(HeciCommands.GET_ENROLLMENT_STATE_REQUEST_COMMAND, null);
                return (EnrollementState)ReadCommand();
            }
        }

        public MspProfile[] GetProfiles()
        {
            lock (parent.MasterLock)
            {
                int r = SendCommand(HeciCommands.ENUMERATE_PROFILES_REQUEST_COMMAND, null);
                return (MspProfile[])ReadCommand();
            }
        }

        public bool SetActiveProfile(MspProfile profile, string username, string password)
        {
            lock (parent.MasterLock)
            {
                if (username.Length > 16) username = username.Substring(0, 16);
                if (password.Length > 16) password = password.Substring(0, 16);

                MemoryStream m = new MemoryStream();
                m.Write(BitConverter.GetBytes(profile.Handle), 0, 4);   // Major Version
                byte[] s1 = UTF8Encoding.UTF8.GetBytes(username);
                m.Write(s1, 0, s1.Length);
                int i = s1.Length;
                while (i < 16) { m.WriteByte(0); i++; }
                byte[] s2 = UTF8Encoding.UTF8.GetBytes(password);
                m.Write(s2, 0, s2.Length);
                i = s2.Length;
                while (i < 16) { m.WriteByte(0); i++; }
                byte[] buf = m.ToArray();

                int r = SendCommand(HeciCommands.SET_ACTIVE_PROFILE_REQUEST_COMMAND, buf);
                object o = ReadCommand();
                return true;
            }
        }

        public bool Unenroll()
        {
            lock (parent.MasterLock)
            {
                int r = SendCommand(HeciCommands.UNENROLL_REQUEST_COMMAND, null);
                object o = ReadCommand();
                return true;
            }
        }

        public bool OpenConnection()
        {
            lock (parent.MasterLock)
            {
                int r = SendCommand(HeciCommands.OPEN_USER_INITIATED_CONNECTION_REQUEST_COMMAND, null);
                return (bool)ReadCommand();
            }
        }

        public bool CloseConnection()
        {
            lock (parent.MasterLock)
            {
                int r = SendCommand(HeciCommands.CLOSE_USER_INITIATED_CONNECTION_REQUEST_COMMAND, null);
                return (bool)ReadCommand();
            }
        }

    }

    public class RemoteAccessStatus
    {
        public enum AMT_NETWORK_CONNECTION
        {
            DIRECT = 0,
            VPN = 1,
            OUTSIDE_ENTERPRISE = 2
        };

        public enum REMOTE_ACCESS_CONNECTION_STATUS
        {
            NOT_CONNECTED = 0,
            CONNECTING,
            CONNECTED,
            ERROR = 10000
        };

        public enum REMOTE_ACCESS_CONNECTION_TRIGGER
        {
            USER_INITIATED = 0,
            TRIGGER_ALERT,
            TRIGGER_PERIODIC,
            TRIGGER_PROVISIONING,
        };

        public uint MeStatus;
        public AMT_NETWORK_CONNECTION Connection;
        public REMOTE_ACCESS_CONNECTION_STATUS Status;
        public REMOTE_ACCESS_CONNECTION_TRIGGER Trigger;
        public string MpsHostname;

        public RemoteAccessStatus(byte[] buffer)
        {
            MeStatus = (uint)BitConverter.ToInt32(buffer, 12);
            Connection = (AMT_NETWORK_CONNECTION)BitConverter.ToInt32(buffer, 16);
            Status = (REMOTE_ACCESS_CONNECTION_STATUS)BitConverter.ToInt32(buffer, 20);
            Trigger = (REMOTE_ACCESS_CONNECTION_TRIGGER)BitConverter.ToInt32(buffer, 24);
            ushort namelen = BitConverter.ToUInt16(buffer, 28);
            MpsHostname = UTF8Encoding.UTF8.GetString(buffer, 30, namelen).Trim();
        }

    }

    public class EnrollementState
    {
        public enum ENROLLMENT_STATE
        {
            ENROLLMENT_STATE_UNENROLLED = 0,
            ENROLLMENT_STATE_ENROLLMENT_PENDING,
            ENROLLMENT_STATE_UNENROLLMENT_PENDING,
            ENROLLMENT_STATE_ENROLLED,
            ENROLLMENT_STATE_ERROR = 10000
        } ;

        public uint MeStatus;
        public ENROLLMENT_STATE EnrollementStatus;
        public string MspName;

        public EnrollementState(byte[] buffer)
        {
            MeStatus = (uint)BitConverter.ToInt32(buffer, 12);
            EnrollementStatus = (ENROLLMENT_STATE)BitConverter.ToInt32(buffer, 16);
            MspName = UTF8Encoding.UTF8.GetString(buffer, 20, 33).Trim();
            int i = MspName.IndexOf("\0");
            if (i >= 0) MspName = MspName.Substring(0, i);
        }

    }

    public class MspProfile
    {
        public uint Handle;
        public uint MeStatus;
        public string FriendlyName;
        public ushort Port;
        public uint InactivityTimeout;
        public string GatewayFqdn;

        public MspProfile(byte[] buffer)
        {
            MeStatus = (uint)BitConverter.ToInt32(buffer, 12);
            FriendlyName = UTF8Encoding.UTF8.GetString(buffer, 16, 33).Trim();
            int i = FriendlyName.IndexOf("\0");
            if (i >= 0) FriendlyName = FriendlyName.Substring(0, i);
            Port = (ushort)BitConverter.ToInt16(buffer, 49);
            InactivityTimeout = (uint)BitConverter.ToInt32(buffer, 51);
            int gl = (ushort)BitConverter.ToInt16(buffer, 55);
            GatewayFqdn = UTF8Encoding.UTF8.GetString(buffer, 57, gl).Trim();
            i = GatewayFqdn.IndexOf("\0");
            if (i >= 0) GatewayFqdn = GatewayFqdn.Substring(0, i);
        }
    }

    public class MeMacAddresses
    {
        public byte[] Host;
        public byte[] Dedicated;
    }

    public class MeCoreVersions
    {
        public string BiosVersion;
        public Hashtable Versions;

        public MeCoreVersions(byte[] buffer)
        {
            BiosVersion = UTF8Encoding.UTF8.GetString(buffer, 16, 65);
            int i = BiosVersion.IndexOf("\0");
            if (i >= 0) BiosVersion = BiosVersion.Substring(0, i);
            uint vercount = (uint)BitConverter.ToInt32(buffer, 16 + 65);
            int ptr = 16 + 65 + 4;
            Versions = new Hashtable();
            for (int j = 0; j < vercount; j++)
            {
                ushort k1 = BitConverter.ToUInt16(buffer, ptr);
                ptr += 2;
                string s1 = UTF8Encoding.UTF8.GetString(buffer, ptr, k1);
                ptr += 20;
                ushort k2 = BitConverter.ToUInt16(buffer, ptr);
                ptr += 2;
                string s2 = UTF8Encoding.UTF8.GetString(buffer, ptr, k2);
                ptr += 20;
                Versions[s1] = s2;
            }
        }
    }

    public class MeProvisioningMode
    {
        public enum ProvisioningModeEnum
        {
            None = 0,
            Enterprise = 1,
            Smb = 2
        };

        public ProvisioningModeEnum ProvisioningMode;
        public bool LegacyMode;

        public MeProvisioningMode(byte[] buffer)
        {
            ProvisioningMode = (ProvisioningModeEnum)BitConverter.ToInt32(buffer, 16);
            LegacyMode = (BitConverter.ToInt32(buffer, 20) != 0);
        }
    }

    public class MeHashEntry
    {
        public uint Handle;
        public bool IsDefault;
        public bool IsActive;
        public byte[] Hash;
        public HeciMeInfo.HashAlgorithmEnum HashAlgorithm;
        public string Name;

        public MeHashEntry(byte[] buffer)
        {
            IsDefault = (BitConverter.ToInt32(buffer, 16) != 0);
            IsActive = (BitConverter.ToInt32(buffer, 20) != 0);
            HashAlgorithm = (HeciMeInfo.HashAlgorithmEnum)buffer[88];
            int hashlength = HeciMeInfo.GetHashSize(HashAlgorithm);
            Hash = new byte[hashlength];
            Array.Copy(buffer, 24, Hash, 0, hashlength);
            short namelen = BitConverter.ToInt16(buffer, 89);
            Name = UTF8Encoding.UTF8.GetString(buffer, 91, namelen);
        }
    }

    public class MeAuditRecord
    {
        public HeciMeInfo.ProvisioningTlsModeEnum ProvisioningTLSMode;
        public bool SecureDNS;
        public bool HostInitiated;
        public HeciMeInfo.HashAlgorithmEnum SelectedHashType;
        public byte[] SelectedHashData;
        public byte[] CaCertificateSerial1;
        public byte[] CaCertificateSerial2;
        public byte[] CaCertificateSerial3;
        public bool AdditionalCaSerialNums;
        public bool IsOemDefault;
        public bool IsTimeValid;
        public IPAddress ProvServerIP;
        public DateTime TlsStartTime;
        public string ProvServerFQDN;

        public MeAuditRecord(byte[] buffer)
        {
            ProvisioningTLSMode = (HeciMeInfo.ProvisioningTlsModeEnum)buffer[16];
            SecureDNS = (BitConverter.ToInt32(buffer, 17) != 0);
            HostInitiated = (BitConverter.ToInt32(buffer, 21) != 0);
            SelectedHashType = (HeciMeInfo.HashAlgorithmEnum)BitConverter.ToInt32(buffer, 25);

            int hashlength = HeciMeInfo.GetHashSize(SelectedHashType);
            SelectedHashData = new byte[hashlength];
            Array.Copy(buffer, 29, SelectedHashData, 0, hashlength);

            CaCertificateSerial1 = new byte[16];
            Array.Copy(buffer, 93, CaCertificateSerial1, 0, 16);
            CaCertificateSerial2 = new byte[16];
            Array.Copy(buffer, 109, CaCertificateSerial2, 0, 16);
            CaCertificateSerial3 = new byte[16];
            Array.Copy(buffer, 125, CaCertificateSerial3, 0, 16);

            AdditionalCaSerialNums = (BitConverter.ToInt32(buffer, 141) != 0);
            IsOemDefault = (BitConverter.ToInt32(buffer, 145) != 0);
            IsTimeValid = (BitConverter.ToInt32(buffer, 149) != 0);

            byte[] addr = new byte[4];
            Array.Copy(buffer, 153, addr, 0, 4);
            Array.Reverse(addr);
            ProvServerIP = new IPAddress(addr);

            ushort Year = BitConverter.ToUInt16(buffer, 157);
            ushort Month = BitConverter.ToUInt16(buffer, 159);
            ushort DayOfWeek = BitConverter.ToUInt16(buffer, 161);
            ushort Day = BitConverter.ToUInt16(buffer, 163);
            ushort Hour = BitConverter.ToUInt16(buffer, 165);
            ushort Minute = BitConverter.ToUInt16(buffer, 167);
            ushort Second = BitConverter.ToUInt16(buffer, 169);
            if (Year != 0) TlsStartTime = new DateTime(Year, Month, Day, Hour, Minute, Second);

            ushort strlen = BitConverter.ToUInt16(buffer, 171);
            if (strlen > 0) ProvServerFQDN = UTF8Encoding.UTF8.GetString(buffer, 173, strlen);
        }
    }

}
