﻿// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intel’s prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Diagnostics;
using System.Net;

namespace ManageabilityStack
{
    public enum NetworkNameType
    {
        IPv4_Address,
        IPv6_Address,
        FQDN,
        Hostname
    }

    /// <summary>
    /// Indicates what IP Address option is used when connecting to an Intel(R) AMT System, namely either
    /// forcing IPv4 address, IPv6 address, or default.
    /// </summary>
    public enum ForceIPOption
    {
        UseDefault,
        UseIPv4,
        UseIPv6
    }

    public class NetworkIdentity
    {
        #region Private Fields
        private string _DefaultName;
        private IPAddress _IPAddress;
        private ForceIPOption _ForceIPOption;
        private NetworkNameType _NetworkNameType;
        private bool _IPAddressInvalidated = true;
        #endregion

        #region Constructors
        /// <summary>
        /// Common initialization code utilized by the constructors
        /// </summary>
        /// <param name="networkName">network name to use for this NetworkIdentity object</param>
        /// <param name="forceOption">indication on which Force IP option to use</param>
        private void Initialize(string networkName, ForceIPOption forceOption)
        {
            _DefaultName = networkName;
            _ForceIPOption = forceOption;

            if (IPAddress.TryParse(networkName, out _IPAddress) == true)
            {
                // Check IPv4 or IPv6 family
                Debug.Assert(_IPAddress != null);
                if (_IPAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
                    _NetworkNameType = NetworkNameType.IPv4_Address;
                else if (_IPAddress.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6)
                    _NetworkNameType = NetworkNameType.IPv6_Address;
                else
                    Debug.Assert(false, "IPAddress Family " + _IPAddress.AddressFamily.ToString() + " unexpected!");
            }
            else
            {
                if (networkName.IndexOf('.') == -1)
                    _NetworkNameType = NetworkNameType.Hostname;
                else
                    _NetworkNameType = NetworkNameType.FQDN;
            }
        }

        public NetworkIdentity(string networkName)
        {
            Initialize(networkName, ForceIPOption.UseDefault);
        }

        public NetworkIdentity(string networkName, ForceIPOption forceOption, IPAddress ipAddress)
        {
            Initialize(networkName, forceOption);
            if(ipAddress != null)
                _IPAddress = ipAddress;
        }
        #endregion

        #region Public Properties
        /// <summary>
        /// Default name to use for connections. If a force option is set, an IP Address string will be returned.
        /// Otherwise, the original user-specified name is returned.
        /// </summary>
        public string DefaultName
        {
            get
            {
                if ((ForceIPOption == ForceIPOption.UseIPv4) || (ForceIPOption == ForceIPOption.UseIPv6))
                    return IPAddress.ToString();
                else
                    return _DefaultName;
            }
        }

        /// <summary>
        /// Represents the name of the Intel(R) AMT system exactly how the user typed it in. This is primarily
        /// used in the GUI for display purposes.
        /// </summary>
        public string DisplayName
        {
            get
            {
                return _DefaultName;
            }
        }

        /// <summary>
        /// The ConnectedDisplayName is used for display purposes when the system is in a connected state, used
        /// primarily by the Computer panel.
        /// </summary>
        public string ConnectedDisplayName
        {
            get
            {
                if(_IPAddressInvalidated == true)
                    return _DefaultName;
                Debug.Assert(IPAddress != null);
                return IPAddress.ToString();
            }
        }

        /// <summary>
        /// Represents the last IP Address from a DNS query.
        /// </summary>
        public IPAddress IPAddress
        {
            get
            {
                if (_IPAddress == null)
                    ResolveIPAddress();

                // If no IP address was determinied for the current hostname, 
                // then return an appropriate IP address indicating "none".
                if ((_IPAddress == null) && (ForceIPOption == ForceIPOption.UseIPv6))
                    return IPAddress.IPv6None;
                else if (_IPAddress == null)
                    return IPAddress.None;

                return _IPAddress;
            }
        }

        /// <summary>
        /// The type of the network name that was specified by the user
        /// </summary>
        public NetworkNameType Type
        {
            get
            {
                return _NetworkNameType;
            }
        }

        /// <summary>
        /// The Force IP Address option specified for this machine
        /// </summary>
        public ForceIPOption ForceIPOption
        {
            get
            {
                return _ForceIPOption;
            }
            set
            {
                _ForceIPOption = value;
            }
        }
        #endregion

        #region Public Methods
        /// <summary>
        /// Disconnects by invalidating the IP Address
        /// </summary>
        public void Disconnect()
        {
            _IPAddressInvalidated = true;
        }

        /// <summary>
        /// Queries DNS to find the current IP Address for this Intel(R) AMT system of the desired IP protocol.
        /// </summary>
        public void ResolveIPAddress()
        {
            IPAddress tmpIPAddress;
            if (IPAddress.TryParse(_DefaultName, out tmpIPAddress) == true)
            {
                _IPAddress = tmpIPAddress;
                return;
            }

            Debug.WriteLine("Resolving IPAddress for " + _DefaultName);
            _IPAddressInvalidated = true;
            try
            {
                IPHostEntry dnsEntry = Dns.GetHostEntry(_DefaultName);
                foreach (IPAddress address in dnsEntry.AddressList)
                {
                    if ((address.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork) &&
                        (ForceIPOption == ForceIPOption.UseIPv4))
                    {
                        // TODO: handle case of multiple IP Addresses of same family
                        _IPAddress = address;
                        _IPAddressInvalidated = false;
                        break;
                    }
                    if ((address.AddressFamily == System.Net.Sockets.AddressFamily.InterNetworkV6) &&
                        (ForceIPOption == ForceIPOption.UseIPv6))
                    {
                        // TODO: handle case of multiple IP Addresses of same family
                        _IPAddress = address;
                        _IPAddressInvalidated = false;
                        break;
                    }
                }
            }
            catch (Exception)
            {
                _IPAddress = null;
            }
        }

        /// <summary>
        /// Constructs a Web URL (without HTTP prefix) from the current NetworkIdentity using the specified port
        /// </summary>
        /// <param name="port">Port to use in the connection URL</param>
        /// <returns>string representing the web URL</returns>
        public string ToWebURL(int port)
        {
            string URL = string.Empty;
            if (ForceIPOption == ForceIPOption.UseIPv6)
            {
                URL = "[" + IPAddress.ToString() + "]:" + port.ToString();
            }
            else
            {
                switch (_NetworkNameType)
                {
                    case NetworkNameType.IPv6_Address:
                        URL = "[" + IPAddress.ToString() + "]:" + port.ToString();
                        break;
                    case NetworkNameType.IPv4_Address:
                    default:
                        URL = DefaultName + ":" + port.ToString();
                        break;
                }
            }

            Debug.Print("ToWebHttpURL({0}) = {1}", port, URL);
            return URL;
        }

        /// <summary>
        /// Constructs a Web URL from the current NetworkIdentity using the specified port
        /// </summary>
        /// <param name="port">Port to use in the connection URL</param>
        /// <returns>string representing the web URL</returns>
        public string ToWebHttpURL(int port)
        {
            string URL = string.Empty;
            if (ForceIPOption == ForceIPOption.UseIPv6)
            {
                URL = "http://[" + IPAddress.ToString() + "]:" + port.ToString();
            }
            else
            {
                switch (_NetworkNameType)
                {
                    case NetworkNameType.IPv6_Address:
                        URL = "http://[" + DefaultName + "]:" + port.ToString();
                        break;
                    case NetworkNameType.IPv4_Address:
                    default:
                        URL = "http://" + DefaultName + ":" + port.ToString();
                        break;
                }
            }

            Debug.Print("ToWebHttpURL({0}) = {1}", port, URL);
            return URL;
        }

        /// <summary>
        /// Constructs a secure Web URL from the current NetworkIdentity using the specified port
        /// </summary>
        /// <param name="port">Port to use in the connection URL</param>
        /// <returns>string representing the web URL</returns>
        public string ToWebHttpsURL(int port)
        {
            string URL = string.Empty;
            if (ForceIPOption == ForceIPOption.UseIPv6)
            {
                URL = "https://[" + IPAddress.ToString() + "]:" + port.ToString();
            }
            else
            {
                switch (_NetworkNameType)
                {
                    case NetworkNameType.IPv6_Address:
                        URL = "https://[" + DefaultName + "]:" + port.ToString();
                        break;
                    case NetworkNameType.IPv4_Address:
                    default:
                        URL = "https://" + DefaultName + ":" + port.ToString();
                        break;
                }
            }

            Debug.Print("ToWebHttpsURL({0}) = {1}", port, URL);
            return URL;
        }
        #endregion
    }
}
