// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.IO;
using System.Net.Sockets;
using System.Text;
using System.Threading;

namespace ManageabilityStack
{
    public class SerialHighDriver
    {
        private const int LOOP_TIMEOUT = 100;
        private const int PACKET_SIZE = 50;
        private const int WRITEBUFFER_WATERMARK = 1024;

        private bool exitting = false;
        private SerialLowDriver serial;
        private Thread serialThread;
        private ArrayList PendingBlocks = new ArrayList();
        private TransferBlock CurrentBlock = null;
        private int CurrentBlockPtr = 0;
        private int CurrentBlockSize = 0;
        private string inboundBlockName = null;
        private int inboundBlockSize = 0;
        private MemoryStream inboundBlock = null;

        public enum SpecialCommand
        {
            REPAINT,
            DISCONNECT
        }

        public FileStream RawCaptureStream { get { return serial.RawCaptureStream; } set { serial.RawCaptureStream = value; } }

        public delegate void ProcessCharHandler(SerialHighDriver sender, byte c);
        public event ProcessCharHandler ProcessCharEvent;

        public delegate void ProcessEscapeHandler(SerialHighDriver sender, EscCode code, int[] args);
        public event ProcessEscapeHandler ProcessEscapeEvent;
        
        public delegate void ProcessDataBlockProgressHandler(SerialHighDriver sender, bool inbound, string name, int total, int current);
        public event ProcessDataBlockProgressHandler ProcessDataBlockProgressEvent;
        
        public delegate void ProcessDataBlockHandler(SerialHighDriver sender, string name, byte[] data);
        public event ProcessDataBlockHandler ProcessDataBlockEvent;

        public delegate void ProcessSpecialHandler(SerialHighDriver sender, SpecialCommand command);
        public event ProcessSpecialHandler ProcessSpecialEvent;

        public enum EscCode
        {
            ClearScreen = 'J',
            MoveCursor = 'H',
            ScreenAttribs = 'm',
            KeyUp = 65,
            KeyDown = 66,
            KeyRight = 67,
            KeyLeft = 68,
            EraseLine = 'K',
            ResetDevice = 'c',
            EnableLineWrap = 'h',
            DisableLineWrap = 'l'
        }

        private class TransferBlock
        {
            public string name;
            public byte[] data;
            public int offset;
            public int length;
        }
        
        public SerialHighDriver(AmtRedirectorWrapper port)
        {
            serial = new SerialLowDriver(port);
            serial.ProcessCharEvent += new SerialLowDriver.ProcessCharHandler(ProcessCharEventSink);
            serial.ProcessEscapeEvent += new SerialLowDriver.ProcessEscapeHandler(ProcessEscapeEventSink);
            serial.ProcessPacketEvent += new SerialLowDriver.ProcessPacketHandler(ProcessPacketEventSink);
            serial.ProcessSpecialEvent += new SerialLowDriver.ProcessSpecialHandler(ProcessSpecialEventSink);

            serialThread = new Thread(new ThreadStart(SerialLoop));
            serialThread.Start();
        }

        public SerialHighDriver(NetworkStream NetStream)
        {
            serial = new SerialLowDriver(NetStream);
            serial.ProcessCharEvent += new SerialLowDriver.ProcessCharHandler(ProcessCharEventSink);
            serial.ProcessEscapeEvent += new SerialLowDriver.ProcessEscapeHandler(ProcessEscapeEventSink);
            serial.ProcessPacketEvent += new SerialLowDriver.ProcessPacketHandler(ProcessPacketEventSink);
            serial.ProcessSpecialEvent += new SerialLowDriver.ProcessSpecialHandler(ProcessSpecialEventSink);

            serialThread = new Thread(new ThreadStart(SerialLoop));
            serialThread.Start();
        }

        public void Dispose()
        {
            if (serial != null)
            {
                exitting = true;
                int i = 0;
                while (serialThread.IsAlive && i++ < 20) Thread.Sleep(50);
                serialThread = null;
                serial.ProcessCharEvent -= new SerialLowDriver.ProcessCharHandler(ProcessCharEventSink);
                serial.ProcessEscapeEvent -= new SerialLowDriver.ProcessEscapeHandler(ProcessEscapeEventSink);
                serial.ProcessPacketEvent -= new SerialLowDriver.ProcessPacketHandler(ProcessPacketEventSink);
                serial.ProcessSpecialEvent -= new SerialLowDriver.ProcessSpecialHandler(ProcessSpecialEventSink);
                serial.Dispose();
                serial = null;
            }
        }

        public AmtRedirectorWrapper.IMRResult Start()
        {
            if (serial == null) return AmtRedirectorWrapper.IMRResult.IMR_RES_UNKNOWN;
            return serial.Start();
        }

        private void SerialLoop()
        {
            while (exitting == false)
            {
                if (serial.BytesToWrite < WRITEBUFFER_WATERMARK)
                {
                    if (CurrentBlock == null)
                    {
                        // Block Setup
                        lock (PendingBlocks)
                        {
                            if (PendingBlocks.Count > 0)
                            {
                                CurrentBlock = (TransferBlock)PendingBlocks[0];
                                PendingBlocks.RemoveAt(0);
                                CurrentBlockPtr = CurrentBlock.offset;
                                CurrentBlockSize = CurrentBlock.length;

                                // Send Name
                                ///////////////////////////////////////////////
                                byte[] b = UTF8Encoding.UTF8.GetBytes(CurrentBlock.name);
                                WritePacket(1, b);

                                // Send Size
                                ////////////////////////////////////////////////////////////////
                                b = UTF8Encoding.UTF8.GetBytes(CurrentBlockSize.ToString());
                                WritePacket(2, b);

                                if (CurrentBlockSize == 0)
                                {
                                    // End of writes
                                    WritePacket(4, null);
                                    CurrentBlock = null;
                                    CurrentBlockPtr = 0;
                                }
                            }
                            else
                            {
                                if (exitting == true) return;
                                Thread.Sleep(LOOP_TIMEOUT);
                            }
                        }
                    }
                    if (CurrentBlock != null)
                    {
                        // Block Write
                        int sendsize = Math.Min(PACKET_SIZE, CurrentBlockSize);
                        byte[] bb = new byte[sendsize];
                        Array.Copy(CurrentBlock.data,CurrentBlockPtr,bb,0,sendsize);
                        WritePacket(3, bb);
                        CurrentBlockPtr += sendsize;
                        CurrentBlockSize -= sendsize;

                        if (ProcessDataBlockProgressEvent != null) ProcessDataBlockProgressEvent(this, false, CurrentBlock.name, CurrentBlock.length, CurrentBlock.length - CurrentBlockSize);

                        if (CurrentBlockSize == 0)
                        {
                            // End of writes
                            WritePacket(4, null);
                            CurrentBlock = null;
                            CurrentBlockPtr = 0;
                        }
                    }
                }
                else
                {
                    if (exitting == true) return;
                    Thread.Sleep(LOOP_TIMEOUT);
                }
            }
        }

        private void ProcessCharEventSink(SerialLowDriver sender, byte c)
        {
            if (ProcessCharEvent != null) ProcessCharEvent(this, c);
        }

        private void ProcessEscapeEventSink(SerialLowDriver sender, byte code, int[] args)
        {
            if (code == 'f') code = (byte)'H';
            if (ProcessEscapeEvent != null) ProcessEscapeEvent(this, (EscCode)code, args);
        }

        private void ProcessPacketEventSink(SerialLowDriver sender, int command, byte[] data)
        {
            switch (command)
            {
                case 1: // Get name
                    /////////////////////////////////////////////////////////
                    inboundBlockName = UTF8Encoding.UTF8.GetString(data);
                    break;
                case 2: // Get size
                    ///////////////////////////////////////////////////////////
                    inboundBlockSize = int.Parse(UTF8Encoding.UTF8.GetString(data));
                    inboundBlock = new MemoryStream();
                    if (ProcessDataBlockProgressEvent != null) ProcessDataBlockProgressEvent(this, true, inboundBlockName, inboundBlockSize, (int)inboundBlock.Length);
                    break;
                case 3: // Get data
                    if (inboundBlock == null) inboundBlock = new MemoryStream();
                    inboundBlock.Write(data,0,data.Length);
                    if (ProcessDataBlockProgressEvent != null) ProcessDataBlockProgressEvent(this, true, inboundBlockName, inboundBlockSize, (int)inboundBlock.Length);
                    break;
                case 4: // Finish
                    if (inboundBlockName != null && ProcessDataBlockEvent != null) ProcessDataBlockEvent(this, inboundBlockName, inboundBlock.ToArray());
                    inboundBlock = null;
                    inboundBlockSize = 0;
                    break;
                case 5: // Exit
                    if (ProcessDataBlockEvent != null) ProcessDataBlockEvent(this, "EXIT", null);
                    inboundBlock = null;
                    inboundBlockSize = 0;
                    break;
                case 6: // Ping
                    if (ProcessDataBlockEvent != null) ProcessDataBlockEvent(this, "PING", null);
                    WritePacket(7, null);
                    break;
                case 7: // Pong
                    if (ProcessDataBlockEvent != null) ProcessDataBlockEvent(this, "PONG", null);
                    break;
                case int.MaxValue: // Force QUIT
                    if (ProcessDataBlockEvent != null) ProcessDataBlockEvent(this, "QUIT", null);
                    inboundBlock = null;
                    inboundBlockSize = 0;
                    break;
            }
        }

        private void ProcessSpecialEventSink(SerialLowDriver sender, SerialLowDriver.SpecialCommand command)
        {
            if (ProcessSpecialEvent != null) ProcessSpecialEvent(this, (SpecialCommand)command);
        }

        public void Ping()
        {
            WritePacket(6, null);
        }

        private void WritePacket(int command, byte[] data)
        {
            lock (serial)
            {
                if (data != null && data.Length != 0)
                {
                    WriteEsc((byte)'@', command, data.Length);
                    Write(data);
                }
                else
                {
                    WriteEsc((byte)'@', command, 0);
                }
            }
        }

        public void Write(byte text) { lock (serial) { serial.Write(text); } }
        public void Write(string text) { lock (serial) { serial.Write(text); } }
        public void Write(byte[] buffer) { lock (serial) { serial.Write(buffer); } }
        public void WriteEsc(byte code) { lock (serial) { serial.Write(new byte[3] { 27, 91, code }); } }
        public void WriteEsc(byte code, string args) { lock (serial) { serial.Write("" + (char)27 + (char)91 + args + (char)code); } }
        public void WriteEsc(byte code, int arg) { lock (serial) { serial.Write("" + (char)27 + (char)91 + arg.ToString() + (char)code); } }
        public void WriteEsc(byte code, int arg1, int arg2) { lock (serial) { serial.Write("" + (char)27 + (char)91 + arg1.ToString() + ";" + arg2.ToString() + (char)code); } }
        public void WriteEsc(byte code, int arg1, int arg2, int arg3) { lock (serial) { serial.Write("" + (char)27 + (char)91 + arg1.ToString() + ";" + arg2.ToString() + ";" + arg3.ToString() + (char)code); } }
        public void WriteClearScreen() {WriteEsc((byte)'J', 2);}
        public void WriteKeyUp()        { serial.Write(new byte[] { 27, 91, 65 }); }
        public void WriteKeyDown()      { serial.Write(new byte[] { 27, 91, 66 }); }
        public void WriteKeyRight()     { serial.Write(new byte[] { 27, 91, 67 }); }
        public void WriteKeyLeft()      { serial.Write(new byte[] { 27, 91, 68 }); }
        public void WriteKeyHome()      { serial.Write(new byte[] { 27, 91, (byte)'H' }); }
        public void WriteKeyEnd()       { serial.Write(new byte[] { 27, 91, (byte)'K' }); }
        public void WriteKeyInsert()    { serial.Write(new byte[] { 27, 91, (byte)'@' }); }
        public void WriteKeyDelete()    { serial.Write(new byte[] { 27, 91, (byte)'P' }); }
        public void WriteKeyPageUp()    { serial.Write(new byte[] { 27, 91, (byte)'?' }); }
        public void WriteKeyPageDown()  { serial.Write(new byte[] { 27, 91, (byte)'/' }); }
        public void WriteKeyF1()        { serial.Write(new byte[] { 27, (byte)'[', (byte)'O', (byte)'P' }); }
        public void WriteKeyF2()        { serial.Write(new byte[] { 27, (byte)'[', (byte)'O', (byte)'Q' }); }
        public void WriteKeyF3()        { serial.Write(new byte[] { 27, (byte)'[', (byte)'O', (byte)'w' }); }
        public void WriteKeyF4()        { serial.Write(new byte[] { 27, (byte)'[', (byte)'O', (byte)'x' }); }
        public void WriteKeyF5()        { serial.Write(new byte[] { 27, (byte)'[', (byte)'O', (byte)'t' }); }
        public void WriteKeyF6()        { serial.Write(new byte[] { 27, (byte)'[', (byte)'O', (byte)'u' }); }
        public void WriteKeyF7()        { serial.Write(new byte[] { 27, (byte)'[', (byte)'O', (byte)'q' }); }
        public void WriteKeyF8()        { serial.Write(new byte[] { 27, (byte)'[', (byte)'O', (byte)'r' }); }
        public void WriteKeyF9()        { serial.Write(new byte[] { 27, (byte)'[', (byte)'O', (byte)'p' }); }
        public void WriteKeyF10()       { serial.Write(new byte[] { 27, (byte)'[', (byte)'O', (byte)'M' }); }

        public void WriteSpecialKey(bool press, ushort control, ushort key, ushort scan)
        {
            byte[] data = new byte[7];
            if (press) data[0] = 1;
            data[1] = (byte)(control >> 8);
            data[2] = (byte)control;
            data[3] = (byte)(key >> 8);
            data[4] = (byte)key;
            data[5] = (byte)(scan >> 8);
            data[6] = (byte)scan;
            WriteDataBlock("SKEY", data);
        }

        public void WriteAltDown()  {WriteSpecialKey(true, 0x0000, 0x0000, 0x0012);}
        public void WriteAltUp()    {WriteSpecialKey(false, 0x0002, 0x0000, 0x0012);}
        public void WriteCtrlDown() {WriteSpecialKey(true, 0x0000, 0x0000, 0x0011);}
        public void WriteCtrlUp()   {WriteSpecialKey(false, 0x0008, 0x0000, 0x0011);}

        public void WriteDataBlock(string name, byte[] data)
        {
            TransferBlock t = new TransferBlock();
            t.name = name;
            t.data = data;
            t.offset = 0;
            if (data != null) { t.length = data.Length; } else { t.length = 0; }
            lock (PendingBlocks) { PendingBlocks.Add(t); }
        }

        public void WriteDataBlock(string name, byte[] data, int offset, int length)
        {
            TransferBlock t = new TransferBlock();
            t.name = name;
            t.data = data;
            t.offset = offset;
            t.length = length;
            lock (PendingBlocks) { PendingBlocks.Add(t); }
        }

        public void ReplayStream(FileStream stream)
        {
            if (serial != null) serial.ReplayStream(stream);
        }

    }
}
