// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.IO;
using System.Net.Sockets;
using System.Text;

namespace ManageabilityStack
{
    class SerialLowDriver
    {
        private AmtRedirectorWrapper serial = null;
        private NetworkStream stream = null;
        private byte[] streamBuffer = null;
        private int termstate = 0;
        private int[] escNumber = new int[32];
        private int escNumberPtr = 0;
        private byte[] packetAccumulator = null;
        private int packetAccumulatorPtr = 0;
        private FileStream g_RawCaptureStream = null;

        public enum SpecialCommand
        {
            REPAINT,
            DISCONNECT
        }

        public delegate void ProcessCharHandler(SerialLowDriver sender, byte c);
        public event ProcessCharHandler ProcessCharEvent;
        public delegate void ProcessEscapeHandler(SerialLowDriver sender, byte code, int[] args);
        public event ProcessEscapeHandler ProcessEscapeEvent;
        public delegate void ProcessPacketHandler(SerialLowDriver sender, int command, byte[] data);
        public event ProcessPacketHandler ProcessPacketEvent;
        public delegate void ProcessSpecialHandler(SerialLowDriver sender, SpecialCommand command);
        public event ProcessSpecialHandler ProcessSpecialEvent;
        public FileStream RawCaptureStream { get { return g_RawCaptureStream; } set { g_RawCaptureStream = value; } }
        
        public SerialLowDriver(AmtRedirectorWrapper port)
        {
            serial = port;
        }

        public SerialLowDriver(NetworkStream stream)
        {
            this.stream = stream;
        }

        public AmtRedirectorWrapper.IMRResult Start()
        {
            if (serial != null)
            {
                serial.SerialData += new AmtRedirectorWrapper.SerialDataHandler(DataReceiveSink);
                return serial.StartSerialRedirect();
            }
            if (stream != null)
            {
                streamBuffer = new byte[1024];
                stream.BeginRead(streamBuffer, 0, 1024, ProcessStreamData, null);
                return AmtRedirectorWrapper.IMRResult.IMR_RES_OK;
            }
            return AmtRedirectorWrapper.IMRResult.IMR_RES_UNKNOWN;
        }

        void ProcessStreamData(IAsyncResult result)
        {
            bool exit = false;
            int l = 0;
            try
            {
                l = stream.EndRead(result);
            
                if (l > 0)
                {
                    DataReceiveSink(null, streamBuffer, l);
                    stream.BeginRead(streamBuffer, 0, 1024, ProcessStreamData, null);
                }
            }
            catch (System.IO.IOException) { exit = true; }
            catch (System.ObjectDisposedException) { exit = true; }

            if (l <= 0 || exit == true)
            {
                if (ProcessPacketEvent != null) ProcessPacketEvent(this, int.MaxValue, null); // Send EXIT command
                stream.Close();
                stream = null;
            }
        }

        public void Dispose()
        {
            if (serial != null)
            {
                serial.SerialData -= new AmtRedirectorWrapper.SerialDataHandler(DataReceiveSink);
                serial.StopSerialRedirect();
                serial = null;
            }
        }

        public int BytesToWrite { get { return 0; } } // TODO FLOW CONTROL!

        private void DataReceiveSink(AmtRedirectorWrapper wrapper, byte[] data, int datalength)
        {
            if (data == null)
            {
                // Connection Closed
                if (ProcessSpecialEvent != null) ProcessSpecialEvent(this, SpecialCommand.DISCONNECT);
            }

            if (g_RawCaptureStream != null) g_RawCaptureStream.Write(data, 0, datalength);

            for (int i = 0; i < datalength; i++)
            {
                if (termstate == 3)
                {
                    // FAST PATH: We are accumulating bytes, lets handle a block
                    int s = Math.Min(packetAccumulator.Length - packetAccumulatorPtr, data.Length - i);
                    Array.Copy(data, i, packetAccumulator, packetAccumulatorPtr, s);
                    packetAccumulatorPtr += s;
                    i += (s-1);
                    if (packetAccumulatorPtr == packetAccumulator.Length)
                    {
                        // Packet Completed
                        if (ProcessPacketEvent != null) ProcessPacketEvent(this, escNumber[0], packetAccumulator);
                        packetAccumulator = null;
                        termstate = 0;
                    }
                }
                else
                {
                    // Process this single byte
                    ProcessByte(data[i]);
                }
            }

            if (ProcessEscapeEvent != null) ProcessSpecialEvent(this, SpecialCommand.REPAINT);
        }

        private void ProcessByte(byte b)
        {
            switch (termstate)
            {
                case 0: // Normal Term State
                    switch (b)
                    {
                        case 27: // ESC
                            termstate = 1;
                            break;
                        default:
                            // Process a single char
                            if (ProcessCharEvent != null) ProcessCharEvent(this, b);
                            break;
                    }
                    break;
                case 1:
                    switch (b)
                    {
                        case 91:
                            escNumberPtr = 0;
                            Array.Clear(escNumber,0,32);
                            termstate = 2;
                            break;
                        case 99: // Terminal Reset
                            if (ProcessEscapeEvent != null) ProcessEscapeEvent(this, 99, null);
                            termstate = 0;
                            break;
                        default:
                            termstate = 0;
                            break;
                    }
                    break;
                case 2:
                    if (b >= '0' && b <= '9')
                    {
                        // This is a number
                        escNumber[escNumberPtr] = (escNumber[escNumberPtr] * 10) + (b - '0');
                        break;
                    }
                    else if (b == ';')
                    {
                        // New number
                        escNumberPtr++;
                        break;
                    }
                    else if (b == '<')
                    {
                        // Nop, some BIOS send this, not sure why.
                        break;
                    }
                    else if (b == '@')
                    {
                        // New Packet
                        if (escNumber[1] == 0)
                        {
                            // Empty Command
                            if (ProcessPacketEvent != null) ProcessPacketEvent(this, escNumber[0], null);
                            termstate = 0;
                        }
                        else
                        {
                            packetAccumulatorPtr = 0;
                            packetAccumulator = new byte[escNumber[1]];
                            termstate = 3;
                        }
                        break;
                    }
                    else
                    {
                        // Process Escape Sequence
                        int[] vals = new int[escNumberPtr+1];
                        Array.Copy(escNumber, vals, escNumberPtr + 1);
                        if (ProcessEscapeEvent != null) ProcessEscapeEvent(this, b, vals);
                        termstate = 0;
                    }
                    break;
                case 3: // Packet Accumulator
                    packetAccumulator[packetAccumulatorPtr] = b;
                    packetAccumulatorPtr++;
                    if (packetAccumulatorPtr == packetAccumulator.Length)
                    {
                        // Packet Completed
                        if (ProcessPacketEvent != null) ProcessPacketEvent(this, escNumber[0], packetAccumulator);
                        packetAccumulator = null;
                        termstate = 0;
                    }
                    break;
            }
        }


        public void Write(byte text)
        {
            if (serial != null) lock (serial) { serial.Write(new byte[] { text }); }
            else
            {
                try { stream.WriteByte(text); } catch (IOException) { }
            }
        }
        
        public void Write(string text)
        {
            if (serial != null)
            {
                lock (serial) { lock (serial) { serial.Write(text); } }
            }
            else
            {
                byte[] buf = UTF8Encoding.UTF8.GetBytes(text);
                try { stream.Write(buf, 0, buf.Length); } catch (IOException) { }
            }
        }
        
        public void Write(byte[] buffer)
        {
            if (serial != null)
            {
                lock (serial) { serial.Write(buffer); }
            }
            else
            {
                try { stream.Write(buffer, 0, buffer.Length); } catch (IOException) { }
            }
        }

        public void WriteEsc(byte code) { lock (serial) { serial.Write(new byte[3] { 27, 91, code }); } }
        public void WriteEsc(byte code, string args) { lock (serial) { serial.Write("" + (char)27 + (char)91 + args + (char)code); } }
        public void WriteEsc(byte code, int arg) { lock (serial) { serial.Write("" + (char)27 + (char)91 + arg.ToString() + (char)code); } }
        public void WriteEsc(byte code, int arg1, int arg2) { lock (serial) { serial.Write("" + (char)27 + (char)91 + arg1.ToString() + ";" + arg2.ToString() + (char)code); } }
        public void WriteEsc(byte code, int arg1, int arg2, int arg3) { lock (serial) { serial.Write("" + (char)27 + (char)91 + arg1.ToString() + ";" + arg2.ToString() + ";" + arg3.ToString() + (char)code); } }
        public void WritePacket(int command, byte[] data) {WriteEsc((byte)'@', data.Length);Write(data);}

        public void ReplayStream(FileStream stream)
        {
            byte[] buf = new byte[stream.Length];
            stream.Read(buf, 0, (int)(stream.Length));
            DataReceiveSink(null, buf, buf.Length);
        }

    }
}
