// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

namespace ManageabilityStack
{
    public class TcpRedirector
    {
        private SerialHighDriver serial;
        private int nextId = new Random().Next();
        private Hashtable tcpIdList = new Hashtable();
        public  Hashtable redirectorList = new Hashtable();

        public TcpRedirector(SerialHighDriver serial, string mappings)
        {
            this.serial = serial;
            serial.ProcessDataBlockEvent += new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);

            if (mappings != null && mappings.Length > 0)
            {
                string[] maps = mappings.Split('|');
                foreach (string m in maps)
                {
                    string[] mm = m.Split(':');
                    int local = int.Parse(mm[0]);
                    int remote = int.Parse(mm[1]);
                    AddMapping(local, remote);
                }
            }
        }

        public void Dispose()
        {
            serial.ProcessDataBlockEvent -= new SerialHighDriver.ProcessDataBlockHandler(ProcessDataBlockHandler);
            foreach (int localPort in redirectorList.Keys)
            {
                TcpRedirect tr = (TcpRedirect)redirectorList[localPort];
                tr.Dispose();
            }
            redirectorList.Clear();
        }

        public bool IsMapped(int localPort)
        {
            return redirectorList.ContainsKey(localPort);
        }

        public void AddMapping(int localPort, int remotePort)
        {
            TcpRedirect tr = new TcpRedirect(this, localPort, remotePort);
            lock (redirectorList)
            {
                redirectorList.Add(localPort, tr);
            }
        }

        public void RemoteMapping(int localPort)
        {
            TcpRedirect tr = (TcpRedirect)redirectorList[localPort];
            if (tr == null) return;
            tr.Dispose();
            lock (redirectorList)
            {
                redirectorList.Remove(localPort);
            }
        }

        public void GetMappingData(int localPort, out int connections, out int datain, out int dataout)
        {
            connections = -1;
            datain = 0;
            dataout = 0;
            TcpRedirect tr = (TcpRedirect)redirectorList[localPort];
            if (tr == null) return;
            connections = tr.connections;
            datain = tr.datain;
            dataout = tr.dataout;
        }

        public int[] GetMappings()
        {
            int[] r;
            lock (redirectorList)
            {
                r = new int[redirectorList.Count];
                int i = 0;
                foreach (int localPort in redirectorList.Keys)
                {
                    r[i] = localPort;
                }
            }
            return r;
        }

        public void GetStats(out int Connections, out int InData, out int OutData)
        {
            Connections = 0;
            InData = 0;
            OutData = 0;
            lock (redirectorList)
            {
                foreach (TcpRedirect tr in redirectorList.Values)
                {
                    Connections += tr.connections;
                    InData += tr.datain;
                    OutData += tr.dataout;
                }
            }
        }

        public string GetMappingString()
        {
            string s = "";
            foreach (TcpRedirect r in redirectorList.Values)
            {
                if (s.Length != 0) s += "|"; ;
                s += r.localport + ":" + r.remoteport;
            }
            return s;
        }

        private int GetNextId()
        {
            return nextId++;
        }

        public void ProcessDataBlockHandler(SerialHighDriver sender, string name, byte[] data)
        {
            if (name.Length < 5) return;
            if (name.Substring(0, 3).CompareTo("TCP") != 0) return;

            if (name.Substring(0, 5).CompareTo("TCPL|") == 0)
            {
                // Handle Socket Close
                int id = int.Parse(name.Substring(5));
                TcpRedirect redirect = (TcpRedirect)tcpIdList[id];
                if (redirect != null) redirect.Close(id);
                tcpIdList.Remove(id);
            }

            if (name.Substring(0, 5).CompareTo("TCPD|") == 0)
            {
                // Handle Socket Data
                int id = int.Parse(name.Substring(5));
                TcpRedirect redirect = (TcpRedirect)tcpIdList[id];
                if (redirect == null) return;
                string d = UTF8Encoding.UTF8.GetString(data);

                try
                {
                    TcpClient client = redirect.GetClient(id);
                    if (client != null)
                    {
                        try
                        {
                            client.GetStream().Write(data, 0, data.Length);
                        }
                        catch (Exception) { }
                        redirect.datain += data.Length;
                    }
                }
                catch (IOException)
                {
                    serial.WriteDataBlock("TCPL|" + id.ToString(), null);
                }
            }
        }

        public class TcpRedirect
        {
            public int localport;
            public int remoteport;
            public int dataout;
            public int datain;
            public int connections { get { return tcpList.Count; } }
            public string state = "Unintialized";
            public object Tag = null;

            private TcpListener tcpListener;
            private Hashtable tcpList = new Hashtable();
            private Hashtable tcpBufList = new Hashtable();
            private TcpRedirector parent;

            public TcpRedirect(TcpRedirector parent, int local, int remote)
            {
                this.parent = parent;
                localport = local;
                remoteport = remote;
                dataout = 0;
                datain = 0;

                tcpListener = new TcpListener(IPAddress.Any, localport);
                try
                {
                    tcpListener.Start();
                    tcpListener.BeginAcceptTcpClient(new AsyncCallback(ListenerEvent), this);
                }
                catch (SocketException)
                {
                    tcpListener = null;
                }
                UpdateState();
            }

            public void Dispose()
            {
                if (tcpListener != null)
                {
                    tcpListener.Stop();
                    tcpListener = null;
                }

                foreach (int id in tcpList.Keys)
                {
                    parent.serial.WriteDataBlock("TCPL|" + id.ToString(), null, 0, 0);
                }
                Thread.Sleep(10);
                foreach (int id in tcpList.Keys)
                {
                    TcpClient client = (TcpClient)tcpList[id];
                    client.Close();
                }
                tcpList.Clear();
                tcpBufList.Clear();
                state = "Disposed";
            }

            private void ListenerEvent(IAsyncResult r)
            {
                if (tcpListener == null) return;

                TcpClient client;
                try
                {
                    client = tcpListener.EndAcceptTcpClient(r);
                }
                catch (ObjectDisposedException)
                {
                    return;
                }
                int id = parent.GetNextId();
                tcpList.Add(id, client);
                parent.tcpIdList.Add(id, this);
                tcpListener.BeginAcceptTcpClient(new AsyncCallback(ListenerEvent), this);
                parent.serial.WriteDataBlock("TCPC|127.0.0.1:" + remoteport + "|" + id.ToString(), null);
                UpdateState();

                byte[] buf = new byte[1024];
                tcpBufList[id] = buf;

                client.GetStream().BeginRead(buf, 0, 1024, new AsyncCallback(ReadEvent), id);
            }

            private void ReadEvent(IAsyncResult r)
            {
                int id = (int)r.AsyncState;
                byte[] buf = (byte[])tcpBufList[id];
                TcpClient client = (TcpClient)tcpList[id];
                if (client == null) return;
                int len;
                try
                {
                    len = client.GetStream().EndRead(r);
                }
                catch (InvalidOperationException)
                {
                    parent.serial.WriteDataBlock("TCPL|" + id.ToString(), null, 0, 0);
                    Close(id);
                    return;
                }
                catch (IOException)
                {
                    parent.serial.WriteDataBlock("TCPL|" + id.ToString(), null, 0, 0);
                    Close(id);
                    return;
                }

                if (len == 0)
                {
                    parent.serial.WriteDataBlock("TCPL|" + id.ToString(), null, 0, 0);
                    Close(id);
                    return;
                }

                parent.serial.WriteDataBlock("TCPD|" + id.ToString(), buf, 0, len);
                dataout += len;

                buf = new byte[1024];
                tcpBufList[id] = buf;
                try
                {
                    client.GetStream().BeginRead(buf, 0, 1024, new AsyncCallback(ReadEvent), id);
                }
                catch (ObjectDisposedException) { }
                catch (InvalidOperationException)
                {
                    parent.serial.WriteDataBlock("TCPL|" + id.ToString(), null, 0, 0);
                    Close(id);
                }
                catch (IOException)
                {
                    parent.serial.WriteDataBlock("TCPL|" + id.ToString(), null, 0, 0);
                    Close(id);
                }
            }

            public void Close(int id)
            {
                TcpClient client = (TcpClient)tcpList[id];
                if (client != null)
                {
                    client.Close();
                    tcpList.Remove(id);
                    tcpBufList.Remove(id);
                }
                UpdateState();
            }

            public TcpClient GetClient(int id)
            {
                return (TcpClient)tcpList[id];
            }

            private void UpdateState()
            {
                if (tcpListener == null)
                {
                    state = "Halted";
                    return;
                }

                switch (tcpList.Count)
                {
                    case 0:
                        state = "Accepting";
                        break;
                    case 1:
                        state = "Connected";
                        break;
                    default:
                        state = tcpList.Count.ToString() + " Connections";
                        break;
                }
            }
        }

    }
}
