// =============================================================================
// Copyright 2008-2011. Intel Corporation All Rights Reserved.
//
// The source code contained or described herein and all documents related to 
// the source code ("Material") are owned by Intel Corporation or its suppliers 
// or licensors. Title to the Material remains with Intel Corporation or its 
// suppliers and licensors. The Material may contain trade secrets and 
// proprietary and confidential information of Intel Corporation and its 
// suppliers and licensors, and is protected by worldwide copyright and trade 
// secret laws and treaty provisions. No part of the Material may be used, 
// copied, reproduced, modified, published, uploaded, posted, transmitted, 
// distributed, or disclosed in any way without Intels prior express written 
// permission. 
//
// No license under any patent, copyright, trade secret or other intellectual 
// property right is granted to or conferred upon you by disclosure or delivery 
// of the Materials, either expressly, by implication, inducement, estoppel or 
// otherwise. Any license under such intellectual property rights must be 
// express and approved by Intel in writing.
// =============================================================================
using System;
using System.Collections;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Security.Cryptography.X509Certificates;
using System.Text;
using Intel.Manageability.Cim; 
using Intel.Manageability.Cim.Typed;
using Intel.Manageability.Cim.Untyped;
using Intel.Manageability.Exceptions;
using Intel.Manageability.WSManagement;

namespace ManageabilityStack
{
    public class AmtSecurityAdmin
    {
        private SecurityAdministrationService service;
        private AmtSystem system;
        private bool cached = false;
        public bool IsCached() { return cached || (system.CoreVersion == 0); }
    
        public AmtSecurityAdmin(SecurityAdministrationService service, AmtSystem system)
        {
            this.service = service;
            this.system = system;
        }

        public void ClearCache()
        {
            cached = false;
            CoreVersionCache = null;
            EnabledInterfacesCache = null;
            GetPowerSavingOptionsCache = false;
            GetActivePowerPackageCache = null;
            GetIdleWakeTimeoutCache = uint.MaxValue;
            
            ProvisioningModeCache = ProvisioningMode.Unknown;
            TrustedFqdnCNCache = null;
            TLSCertificateCache = null;
            TrustedRootCertificatesCache = null;
            TlsOptionsCache = false;
            KerberosOptionsCache = null;
            AdminAclEntryCache = null;
            UserAclEntriesCache = null;
            CertificateHashEntriesCache = null;
            ProvisioningServerOTPCache = null;
            GetZeroTouchConfigurationModeSet = false;
            EnvironmentDetectionCache = null;
            EnvironmentDetectionCacheSet = false;

            CertStoreGetCertificatesCache = null;
            CertStoreGetTrustedCertificatesCache = null;
            GetCrlCache = null;
        }

        public void FetchCache()
        {
            try
            {
                GetCoreVersion();
                GetEnabledInterfaces();
                GetPowerPackages();
                GetActivePowerPackage();

                GetProvisioningMode();
                GetTrustedFqdnCN();
                GetPkiCapabilities();
                GetDigestRealm();
                //GetTLSCertificate();
                //GetTrustedRootCertificates();
                TlsAthenticationType local;
                TlsAthenticationType remote;
                GetTlsOptions(out local, out remote);
                KerberosOptionsType k;
                GetKerberosOptions(out k);
                GetAdminAclEntry();
                AmtSecurityAdmin.AmtUserAcl[] UserAclEntries;
                GetUserAclEntries(out UserAclEntries);
                GetZeroTouchConfigurationMode();

                /*
                if (ZeroTouchSupport())
                {
                    GetCertificateHashEntries();
                    GetProvisioningServerOTP();
                }
                */

                int activeStateAC;
                int wakeOnNetAccessThresholdAC;
                int wakeOnNetAccessSleepTimer;
                GetPowerSavingOptions(out activeStateAC, out wakeOnNetAccessThresholdAC, out wakeOnNetAccessSleepTimer);

                EnvironmentDetectionType EnvironmentDetection;
                GetEnvironmentDetection(out EnvironmentDetection);

                AmtStoredCert[] certificates;
                CertStoreGetCertificates(out certificates);
                CertStoreGetTrustedCertificates(out certificates);

                CrlType[] crl;
                GetCRL(out crl);
                cached = true;
            }
            catch (Exception e)
            {
                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Error,
                    "Failure to retrieve the Security Administration data.",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
            }
        }

        public void ClearPowerPolicyCache()
        {
            GetPowerSavingOptionsCache = false;
            GetActivePowerPackageCache = null;
            GetIdleWakeTimeoutCache = uint.MaxValue;
        }

        private string CoreVersionCache = null;
        public string GetCoreVersion()
        {
            if (CoreVersionCache != null) return CoreVersionCache;
            if (system.WsMan != null) CoreVersionCache = GetCoreVersionWSMAN(); else CoreVersionCache = GetCoreVersionSOAP();
            return CoreVersionCache;
        }

        public string GetCoreVersionWSMAN()
        {
            if (system.WsMan == null) return "";
            try
            {
                CIM_SoftwareIdentity.CimKeys keys = new CIM_SoftwareIdentity.CimKeys();
                keys.InstanceID = "AMT FW Core Version";
                CIM_SoftwareIdentity amtWsmanService = new CIM_SoftwareIdentity(system.WsMan);
                amtWsmanService.Get(keys);
                return amtWsmanService.VersionString;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetCoreVersionWSMAN WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetCoreVersionWSMAN CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetCoreVersionWSMAN",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        public string GetCoreVersionSOAP()
        {
            service.GetCoreVersion(out CoreVersionCache);
            return CoreVersionCache;
        }

        private string BuildNumCache = null;
        public string GetBuildNum()
        {
            if (BuildNumCache != null) return BuildNumCache;
            BuildNumCache = GetBuildNumWSMAN();
            return BuildNumCache;
        }

        public string GetBuildNumWSMAN()
        {
            if (system.WsMan == null) return string.Empty;
            try
            {
                CIM_SoftwareIdentity.CimKeys keys = new CIM_SoftwareIdentity.CimKeys();
                keys.InstanceID = "Build Number";
                CIM_SoftwareIdentity amtWsmanService = new CIM_SoftwareIdentity(system.WsMan);
                amtWsmanService.Get(keys);
                return amtWsmanService.VersionString;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetBuildNumWSMAN WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetBuildNumWSMAN CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetBuildNumWSMAN",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        public bool GetWsmanOnly()
        {
            if (system.WsMan == null) return false;
            try
            {
                AMT_GeneralSettings amtWsmanService = new AMT_GeneralSettings(system.WsMan);
                amtWsmanService.Get();
                return amtWsmanService.WsmanOnlyMode;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetWsmanOnly WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetWsmanOnly CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetWsmanOnly",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        public bool SetWsmanOnly(bool wsmanonly)
        {
            if (system.WsMan == null) return false;
            try
            {
                AMT_GeneralSettings amtWsmanService = new AMT_GeneralSettings(system.WsMan);
                amtWsmanService.Get();
                amtWsmanService.WsmanOnlyMode = wsmanonly;
                amtWsmanService.Put();
                return true;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetWsmanOnly WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetWsmanOnly CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetWsmanOnly",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                throw;
            }
        }

        public System.Net.HttpStatusCode GetCoreVersionEx(out string version)
        {
            version = null;
            try
            {
                service.GetCoreVersion(out version);
            }
            catch (System.Net.WebException ex)
            {
                if (ex.Response == null)
                {
                    return System.Net.HttpStatusCode.SeeOther;
                }
                else
                {
                    return ((System.Net.HttpWebResponse)ex.Response).StatusCode;
                }
            }
            catch (Exception)
            {
                return System.Net.HttpStatusCode.SeeOther;
            }
            return System.Net.HttpStatusCode.OK;
        }

        [Serializable]
        public class EnabledInterfaces
        {
            public bool IdeRedirection;
            public bool SerialOverLAN;
            public bool WebUI;
        }

        public bool SerialOverLanEnabled { get { return GetEnabledInterfaces().SerialOverLAN; } }
        public bool IdeRedirectEnabled { get { return GetEnabledInterfaces().IdeRedirection; } }
        public bool WebUiEnabled { get { return GetEnabledInterfaces().WebUI; } }

        private EnabledInterfaces EnabledInterfacesCache = null;
        public EnabledInterfaces GetEnabledInterfaces()
        {
            if (EnabledInterfacesCache != null) return EnabledInterfacesCache;
            EnabledInterfaces eif = new EnabledInterfaces();
            eif.IdeRedirection = false;
            eif.SerialOverLAN = false;
            eif.WebUI = false;

            if (system.AmtRev == 1)
            {
                // Intel AMT 1.0 does not support this method (Assume it's all active)
                eif.IdeRedirection = true;
                eif.SerialOverLAN = true;
                eif.WebUI = true;
                EnabledInterfacesCache = eif;
                return EnabledInterfacesCache;
            }

            if (system.WsMan != null)
            {
                // WSMAN - Get Web UI status
                try
                {
                    AMT_WebUIService amtWsmanService = new AMT_WebUIService(system.WsMan);
                    amtWsmanService.Get();
                    eif.WebUI = (amtWsmanService.EnabledState == 2);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetEnabledInterfaces WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetEnabledInterfaces CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetEnabledInterfaces",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    eif.WebUI = false;
                }

                // Get SOL & IDE-R state
                try
                {
                    AMT_RedirectionService amtWsmanService = new AMT_RedirectionService(system.WsMan);
                    amtWsmanService.Get();
                    if (amtWsmanService.EnabledState == 2 || amtWsmanService.EnabledState == 32771)
                    {
                        eif.SerialOverLAN = true;
                        eif.IdeRedirection = true;
                    }
                    else if (amtWsmanService.EnabledState == 3 || amtWsmanService.EnabledState == 32768)
                    {
                        eif.SerialOverLAN = false;
                        eif.IdeRedirection = false;
                    }
                    else if (amtWsmanService.EnabledState == 32769)
                    {
                        eif.SerialOverLAN = false;
                        eif.IdeRedirection = true;
                    }
                    else if (amtWsmanService.EnabledState == 32770)
                    {
                        eif.SerialOverLAN = true;
                        eif.IdeRedirection = false;
                    }
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetEnabledInterfaces WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetEnabledInterfaces CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetEnabledInterfaces",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return null;
                }

                EnabledInterfacesCache = eif;
                return EnabledInterfacesCache;
            }
            else
            {
                // SOAP
                EnabledInterfacesType[] ei;
                AmtCallStatus r = (AmtCallStatus)service.GetEnabledInterfaces(out ei);
                foreach (EnabledInterfacesType i in ei)
                {
                    if (i == EnabledInterfacesType.IdeRedirection) eif.IdeRedirection = true;
                    if (i == EnabledInterfacesType.SerialOverLAN) eif.SerialOverLAN = true;
                    if (i == EnabledInterfacesType.WebUI) eif.WebUI = true;
                }
                EnabledInterfacesCache = eif;
                return EnabledInterfacesCache;
            }
        }

        public AmtCallStatus SetEnabledInterfaces(EnabledInterfaces enabledInterfaces)
        {
            if (system.AmtRev <= 1 || system.LimitedFeatures == true) return AmtCallStatus.NOT_PERMITTED;

            if (system.WsMan != null)
            {
                // WSMAN - Set WEB UI
                try
                {
                    AMT_WebUIService webUIService = new AMT_WebUIService(system.WsMan);
                    CimReference job;
                    AmtCallStatus r = (AmtCallStatus)webUIService.RequestStateChange((ushort)(enabledInterfaces.WebUI == true ? 2 : 3), null, out job);
                    if (r != AmtCallStatus.SUCCESS)
                        return r;
                    EnabledInterfacesCache = null;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetEnabledInterfaces WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetEnabledInterfaces CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetEnabledInterfaces",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }

                // WSMAN - Set IDE-R / SOL
                try
                {
                    AMT_RedirectionService redirectionService = new AMT_RedirectionService(system.WsMan);
                    redirectionService.Get();
                    ushort newState = 32768;
                    if (enabledInterfaces.IdeRedirection == true && enabledInterfaces.SerialOverLAN == false) newState = 32769;
                    if (enabledInterfaces.IdeRedirection == false && enabledInterfaces.SerialOverLAN == true) newState = 32770;
                    if (enabledInterfaces.IdeRedirection == true && enabledInterfaces.SerialOverLAN == true) newState = 32771;
                    CimReference job;
                    AmtCallStatus r = (AmtCallStatus)redirectionService.RequestStateChange(newState, null, out job);
                    if (r != AmtCallStatus.SUCCESS)
                        return r;

                    // Then set the enable state (Not sure why we need this, but it works...)
                    redirectionService.EnabledState = 2;
                    redirectionService.Put();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetEnabledInterfaces WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetEnabledInterfaces CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetEnabledInterfaces",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
                return AmtCallStatus.SUCCESS;
            }
            else
            {
                // SOAP
                int i = 0;
                if (enabledInterfaces.IdeRedirection == true) i++;
                if (enabledInterfaces.SerialOverLAN == true) i++;
                if (enabledInterfaces.WebUI == true) i++;

                EnabledInterfacesType[] eif = new EnabledInterfacesType[i];
                i = 0;
                if (enabledInterfaces.IdeRedirection == true) { eif[i] = EnabledInterfacesType.IdeRedirection; i++; }
                if (enabledInterfaces.SerialOverLAN == true) { eif[i] = EnabledInterfacesType.SerialOverLAN; i++; }
                if (enabledInterfaces.WebUI == true) { eif[i] = EnabledInterfacesType.WebUI; i++; }

                EnabledInterfacesCache = null;
                AmtCallStatus r = (AmtCallStatus)service.SetEnabledInterfaces(eif);
                return r;
            }
        }

        public AmtCallStatus SetPowerSavingOptions(int activeStateAC, int wakeOnNetAccessThresholdAC, int wakeOnNetAccessSleepTimer)
        {
            if (system.CoreVersion < 2 || system.CoreVersion >= 2.5) return AmtCallStatus.NOT_PERMITTED;
            GetPowerSavingOptionsCache = false;
            PowerStateType p1 = (PowerStateType)activeStateAC;
            PowerStateType p2 = (PowerStateType)wakeOnNetAccessThresholdAC;
            return (AmtCallStatus)service.SetPowerSavingOptions(p1, p2, wakeOnNetAccessThresholdAC >= 0, (ushort)wakeOnNetAccessSleepTimer, wakeOnNetAccessSleepTimer >= 0);
        }

        private bool GetPowerSavingOptionsCache = false;
        private int activeStateAcCache = -1;
        private int wakeOnNetAccessThresholdAcCache = -1;
        private int wakeOnNetAccessSleepTimerCache = -1;
        public void GetPowerSavingOptions(out int activeStateAC, out int wakeOnNetAccessThresholdAC, out int wakeOnNetAccessSleepTimer)
        {
            if (system.CoreVersion < 2 || system.CoreVersion >= 2.5)
            {
                activeStateAC = -1;
                wakeOnNetAccessThresholdAC = -1;
                wakeOnNetAccessSleepTimer = -1;
                return;
            }

            if (GetPowerSavingOptionsCache == true)
            {
                activeStateAC = activeStateAcCache;
                wakeOnNetAccessThresholdAC = wakeOnNetAccessThresholdAcCache;
                wakeOnNetAccessSleepTimer = wakeOnNetAccessSleepTimerCache;
                return;
            }

            PowerStateType xActiveStateAC;
            PowerStateType xWakeOnNetAccessThresholdAC;
            bool wakeOnNetAccessThresholdACspecified;
            ushort xWakeOnNetAccessSleepTimer;
            bool wakeOnNetAccessSleepTimerSpecified;
            service.GetPowerSavingOptions(out xActiveStateAC, out xWakeOnNetAccessThresholdAC, out wakeOnNetAccessThresholdACspecified, out xWakeOnNetAccessSleepTimer, out wakeOnNetAccessSleepTimerSpecified);

            activeStateAC = (int)xActiveStateAC;
            if (wakeOnNetAccessThresholdACspecified == false)
            {
                wakeOnNetAccessThresholdAC = -1;
            }
            else
            {
                wakeOnNetAccessThresholdAC = (int)xWakeOnNetAccessThresholdAC;
                if (wakeOnNetAccessThresholdAC < activeStateAC) wakeOnNetAccessThresholdAC = activeStateAC;
            }
            if (wakeOnNetAccessSleepTimerSpecified == false) wakeOnNetAccessSleepTimer = -1; else wakeOnNetAccessSleepTimer = (int)xWakeOnNetAccessSleepTimer;

            activeStateAcCache = activeStateAC;
            wakeOnNetAccessThresholdAcCache = wakeOnNetAccessThresholdAC;
            wakeOnNetAccessSleepTimerCache = wakeOnNetAccessSleepTimer;
            GetPowerSavingOptionsCache = true;
        }

        public string GetPowerSavingOptionsStr()
        {
            if (system.AmtRev < 2)
            {
                //return "Unsupported Feature";
                return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_0;
            }
            else if (system.AmtRev == 2 && system.AmtRevMin < 5)
            {
                int activeStateAC;
                int wakeOnNetAccessThresholdAC;
                int wakeOnNetAccessSleepTimer;
                GetPowerSavingOptions(out activeStateAC, out wakeOnNetAccessThresholdAC, out wakeOnNetAccessSleepTimer);

                switch (activeStateAC)
                {
                    case -1:
                        // return "Unsupported Feature";
                        return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_1;
                    case 0:
                        if (wakeOnNetAccessThresholdAC == 2) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_2;
                        if (wakeOnNetAccessThresholdAC == 3) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_3;
                        if (wakeOnNetAccessThresholdAC == 4) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_4;
                        if (wakeOnNetAccessThresholdAC == 5) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_5;
                        return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_16;
                    case 1:
                        if (wakeOnNetAccessThresholdAC == 2) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_6;
                        if (wakeOnNetAccessThresholdAC == 3) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_7;
                        if (wakeOnNetAccessThresholdAC == 4) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_8;
                        if (wakeOnNetAccessThresholdAC == 5) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_9;
                        return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_17;
                    case 2:
                        if (wakeOnNetAccessThresholdAC == 3) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_10;
                        if (wakeOnNetAccessThresholdAC == 4) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_11;
                        if (wakeOnNetAccessThresholdAC == 5) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_12;
                        return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_18;
                    case 3:
                        if (wakeOnNetAccessThresholdAC == 4) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_13;
                        if (wakeOnNetAccessThresholdAC == 5) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_14;
                        return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_19;
                    case 4:
                        if (wakeOnNetAccessThresholdAC == 5) return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_15;
                        return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_20;
                    case 5:
                        return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_21;
                }
                return ManageabilityStack.Properties.Resources.SecurityAdmin_PwrSavingOpt_22;
            }
            else
            {
                AmtPowerPackage[] packs = GetPowerPackages();
                if (packs == null || packs.Length == 0) return "";
                AmtPowerPackage ActivePack = GetActivePowerPackage();
                if (ActivePack == null) return String.Empty;
                string s = ActivePack.name;
                if (s.Length > 8 && s.Substring(0, 8).CompareTo("Mobile: ") == 0) s = s.Substring(8);
                else if (s.Length > 9 && s.Substring(0, 9).CompareTo("Desktop: ") == 0) s = s.Substring(9);
                return s;
            }
        }

        public class AmtPowerPackage
        {
            public Guid id;
            public string name;
            public uint handle;
            public AMT_SystemPowerScheme wsman;

            public AmtPowerPackage(Guid id, string name)
            {
                this.id = id;
                this.name = name;
            }
        }

        private class AmtPowerPackageComparerClass : IComparer
        {
            int IComparer.Compare(Object x, Object y)
            {
                AMT_SystemPowerScheme e1 = (AMT_SystemPowerScheme)x;
                AMT_SystemPowerScheme e2 = (AMT_SystemPowerScheme)y;
                return (e1.InstanceID.CompareTo(e2.InstanceID));
            }
        }

        private AmtPowerPackage[] GetPowerPackagesCache = null;
        public AmtPowerPackage[] GetPowerPackages()
        {
            if (GetPowerPackagesCache != null) return GetPowerPackagesCache;
            if (system.CoreVersion == 0) return new AmtPowerPackage[0];

            if (system.WsMan != null)
            {
                Collection<AMT_SystemPowerScheme> r = null;

                // Use WSMAN
                try
                {
                   r = AMT_SystemPowerScheme.Enumerate(system.WsMan);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetPowerPackages WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetPowerPackages CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetPowerPackages",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return null;
                }
                if (r == null) { return null; }
                ArrayList list2 = new ArrayList(r);
                list2.Sort(new AmtPowerPackageComparerClass());
                ArrayList list = new ArrayList();
                foreach (AMT_SystemPowerScheme x in list2)
                {
                    AmtPowerPackage pp = new AmtPowerPackage(new Guid(Convert.FromBase64String(x.SchemeGUID)), x.Description);
                    pp.wsman = x;
                    list.Add(pp);
                }
                GetPowerPackagesCache = (AmtPowerPackage[])list.ToArray(typeof(AmtPowerPackage));
                return GetPowerPackagesCache;
            }
            else
            {
                // Use EOI
                if (system.AmtRev < 2 || (system.AmtRev == 2 && system.AmtRevMin < 5)) return null;

                byte[][] data;
                service.EnumeratePowerPackages(out data);

                AmtPowerPackage[] packages = new AmtPowerPackage[data.Length];
                int i = 0;
                string s;
                foreach (byte[] g in data)
                {
                    service.GetPowerPackage(g, out s);
                    packages[i] = new AmtPowerPackage(new Guid(g), s);
                    i++;
                }

                GetPowerPackagesCache = packages;
                return GetPowerPackagesCache;
            }
        }

        private AmtPowerPackage GetActivePowerPackageCache = null;
        public AmtPowerPackage GetActivePowerPackage()
        {
            if (GetActivePowerPackageCache != null) return GetActivePowerPackageCache;
            if (system.AmtRev < 2 || (system.AmtRev == 2 && system.AmtRevMin < 5)) return null;
            GetPowerPackages();
            if (GetPowerPackagesCache == null) return null;

            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    Collection <CIM_ElementSettingData> settings = CIM_ElementSettingData.Enumerate(system.WsMan);
                    foreach (CIM_ElementSettingData setting in settings)
                    {
                        if ((String.Compare("http://intel.com/wbem/wscim/1/amt-schema/1/AMT_SystemPowerScheme",
                                            setting.SettingData.referenceParameters.ResourceURI, true) == 0) &&
                            setting.IsCurrentExist && (setting.IsCurrent == 1))
                        {
                            AMT_SystemPowerScheme scheme = new AMT_SystemPowerScheme(system.WsMan);
                            scheme.Get(setting.SettingData);
                            AmtPowerPackage o = new AmtPowerPackage(new Guid(Convert.FromBase64String(scheme.SchemeGUID)),
                                (scheme.DescriptionExist ? scheme.Description : String.Empty));
                            o.wsman = scheme;
                            int i = scheme.InstanceID.LastIndexOf(" ");
                            if (i > 0) o.handle = uint.Parse(scheme.InstanceID.Substring(i));
                            GetActivePowerPackageCache = o;
                            return GetActivePowerPackageCache;
                        }
                    } // END foreach (CIM_ElementSettingData setting in settings)
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetActivePowerPackage WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetActivePowerPackage CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetActivePowerPackage",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return null;
                }
                return null;
            }
            else
            {
                // Use SOAP
                byte[] activepackage;
                service.GetActivePowerPackage(out activepackage);
                Guid ap = new Guid(activepackage);
                GetPowerPackages();

                foreach (AmtPowerPackage pp in GetPowerPackagesCache)
                {
                    if (pp.id.CompareTo(ap) == 0)
                    {
                        GetActivePowerPackageCache = pp;
                        return GetActivePowerPackageCache;
                    }
                }

                return null;
            }
        }

        public AmtCallStatus SetActivePowerPackage(AmtPowerPackage powerPackage)
        {
            GetActivePowerPackageCache = null;
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_SystemPowerScheme sysPowerScheme = new AMT_SystemPowerScheme(system.WsMan);
                    sysPowerScheme.InstanceID = powerPackage.wsman.InstanceID;
                    return (AmtCallStatus)sysPowerScheme.SetPowerScheme();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetActivePowerPackage WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetActivePowerPackage CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetActivePowerPackage",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                return (AmtCallStatus)service.SetActivePowerPackage(powerPackage.id.ToByteArray());
            }
        }

        private uint GetIdleWakeTimeoutCache = uint.MaxValue;
        public uint GetIdleWakeTimeout()
        {
            if (GetIdleWakeTimeoutCache != uint.MaxValue) return GetIdleWakeTimeoutCache;

            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AMT_GeneralSettings genSettings = new AMT_GeneralSettings(system.WsMan);
                    genSettings.Get();
                    if (genSettings.IdleWakeTimeoutExist == false) GetIdleWakeTimeoutCache = 0;
                    GetIdleWakeTimeoutCache = genSettings.IdleWakeTimeout;
                    return GetIdleWakeTimeoutCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetIdleWakeTimeout WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetIdleWakeTimeout CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetIdleWakeTimeout",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    GetIdleWakeTimeoutCache = 0;
                    return GetIdleWakeTimeoutCache;
                }
            }
            else
            {
                // USE EOI
                if (system.AmtRev < 2 || (system.AmtRev == 2 && system.AmtRevMin < 5)) return 0;
                GlobalPowerPolicyType p = null;
                AmtCallStatus r = (AmtCallStatus)service.GetGlobalPowerPolicy(out p);
                if (p == null)
                {
                    // Try an alternative
                    int activeStateAC;
                    int wakeOnNetAccessThresholdAC;
                    int wakeOnNetAccessSleepTimer;
                    GetPowerSavingOptions(out activeStateAC, out wakeOnNetAccessThresholdAC, out wakeOnNetAccessSleepTimer);
                    if (wakeOnNetAccessSleepTimer > 1)
                    {
                        GetIdleWakeTimeoutCache = (uint)wakeOnNetAccessSleepTimer;
                    }
                    else
                    {
                        GetIdleWakeTimeoutCache = 0;
                    }
                    return GetIdleWakeTimeoutCache;
                }
                GetIdleWakeTimeoutCache = p.IdleWakeTimeout;
                return GetIdleWakeTimeoutCache;
            }
        }

        public AmtCallStatus SetIdleWakeTimeout(uint IdleWakeTimeout)
        {
            GetIdleWakeTimeoutCache = uint.MaxValue;
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_GeneralSettings gs = new AMT_GeneralSettings(system.WsMan);
                    gs.Get();
                    gs.IdleWakeTimeout = IdleWakeTimeout;
                    gs.Put();
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetIdleWakeTimeout WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetIdleWakeTimeout CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetIdleWakeTimeout",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                GlobalPowerPolicyType p = new GlobalPowerPolicyType();
                p.IdleWakeTimeout = IdleWakeTimeout;
                AmtCallStatus r = (AmtCallStatus)service.SetGlobalPowerPolicy(p);
                return r;
            }
        }

        public enum ProvisioningMode
        {
            Unknown = -1,
            Current = 0,
            Enterprise = 1,
            SmallBusiness = 2
        }

        private ProvisioningMode ProvisioningModeCache = ProvisioningMode.Unknown;
        public ProvisioningMode GetProvisioningMode()
        {
            if (ProvisioningModeCache != ProvisioningMode.Unknown) return ProvisioningModeCache;
            if (system.CoreVersion == 0) return ProvisioningMode.SmallBusiness;

            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_SetupAndConfigurationService sct = new AMT_SetupAndConfigurationService(system.WsMan);
                    sct.Get();
                    if (sct == null) return ProvisioningMode.Unknown;
                    if (sct.ProvisioningMode == 1) ProvisioningModeCache = ProvisioningMode.Enterprise;
                    if (sct.ProvisioningMode == 2) ProvisioningModeCache = ProvisioningMode.SmallBusiness;
                    return ProvisioningModeCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetProvisioningMode WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetProvisioningMode CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetProvisioningMode",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return ProvisioningMode.Unknown;
                }
            }
            else
            {
                // Use SOAP
                uint pm;
                AmtCallStatus r = (AmtCallStatus)service.GetProvisioningMode(out pm);
                ProvisioningModeCache = (ProvisioningMode)pm;
                return ProvisioningModeCache;
            }
        }

        public AmtCallStatus SetProvisioningMode(ProvisioningMode provisioningMode)
        {
            if (system.CoreVersion == 0) return AmtCallStatus.NOT_PERMITTED;
            ProvisioningModeCache = ProvisioningMode.Unknown;
            return (AmtCallStatus)service.SetProvisioningMode((uint)provisioningMode);
        }

        public AmtCallStatus Unprovision(ProvisioningMode provisioningMode)
        {
            if (system.CoreVersion == 0) return AmtCallStatus.NOT_PERMITTED;
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_SetupAndConfigurationService scs = new AMT_SetupAndConfigurationService(system.WsMan);
                    return (AmtCallStatus)scs.Unprovision((uint)provisioningMode);
                }
                catch (Exception e)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.Unprovision",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                return (AmtCallStatus)service.Unprovision((uint)provisioningMode);
            }
        }

        public AmtCallStatus PartialUnprovision()
        {
            if (system.CoreVersion == 0) return AmtCallStatus.NOT_PERMITTED;
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_SetupAndConfigurationService scs = new AMT_SetupAndConfigurationService(system.WsMan);
                    return (AmtCallStatus)scs.PartialUnprovision();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.PartialUnprovision WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.PartialUnprovision CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.PartialUnprovision",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                return (AmtCallStatus)service.PartialUnprovision();
            }
        }

        public AmtCallStatus CommitChanges()
        {
            if (system.CoreVersion == 0) return AmtCallStatus.NOT_PERMITTED;
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_SetupAndConfigurationService scs = new AMT_SetupAndConfigurationService(system.WsMan);
                    return (AmtCallStatus)scs.CommitChanges();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CommitChanges WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CommitChanges CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CommitChanges",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                return (AmtCallStatus)service.CommitChanges();
            }
        }

        public AmtCallStatus ResetFlashWearOutProtection()
        {
            if (system.CoreVersion == 0) return AmtCallStatus.NOT_PERMITTED;
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_SetupAndConfigurationService scs = new AMT_SetupAndConfigurationService(system.WsMan);
                    return (AmtCallStatus)scs.ResetFlashWearOutProtection();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.ResetFlashWearOutProtection WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.ResetFlashWearOutProtection CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.ResetFlashWearOutProtection",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                return (AmtCallStatus)service.ResetFlashWearOutProtection();
            }
        }

        private string[] TrustedFqdnCNCache = null;
        public string[] GetTrustedFqdnCN()
        {
            if (system.AmtRev <= 1 || system.LimitedFeatures == true) return null;
            if (TrustedFqdnCNCache != null) return TrustedFqdnCNCache;

            if (system.WsMan != null)
            {
                Collection<AMT_TLSSettingData> wstls = null;
                // Use WSMAN
                try
                {
                    wstls = AMT_TLSSettingData.Enumerate(system.WsMan);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetTrustedFqdnCN WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetTrustedFqdnCN CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetTrustedFqdnCN",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return null;
                }

                if (wstls == null || wstls.Count == 0) return new string[0];
                if (wstls[0].TrustedCNExist)
                    TrustedFqdnCNCache = wstls[0].TrustedCN;
                else
                    TrustedFqdnCNCache = new string[0];

                return TrustedFqdnCNCache;
            }
            else
            {
                // Use SOAP
                string[] fqdn;
                AmtCallStatus r = (AmtCallStatus)service.GetTrustedFqdnCN(out fqdn);
                TrustedFqdnCNCache = fqdn;
                return TrustedFqdnCNCache;
            }
        }

        public AmtCallStatus EnableTls(TlsAthenticationType local, TlsAthenticationType remote,
            string[] fqdnSuffix, uint handle, bool handleSpecified, bool acceptNonSecureConnections)
        {
            if (system.AmtRev <= 1 || system.LimitedFeatures == true) return AmtCallStatus.CALL_SKIPPED;

            if (local == TlsAthenticationType.NoAuth || remote == TlsAthenticationType.NoAuth) return AmtCallStatus.NOT_PERMITTED;
            if (!handleSpecified) return AmtCallStatus.NOT_PERMITTED;

            AmtCallStatus r = AmtCallStatus.SUCCESS;

            if ((system.WsMan != null) && (system.CoreVersion >= 3))
            {
                #region WSMAN

                // Use WSMAN
                try
                {
                    #region Set TLS Credentials

                    //
                    // Set the TLS Credentials
                    //
                    AMT_PublicKeyCertificate newTlsCert = null;
                    if (handleSpecified)
                    {
                        try
                        {
                            // Get the specified certificate from the Intel(R) AMT ME certificate store.
                            newTlsCert = new AMT_PublicKeyCertificate(system.WsMan);
                            AMT_PublicKeyCertificate.CimKeys keys = new AMT_PublicKeyCertificate.CimKeys();
                            keys.InstanceID = String.Format("Intel(r) AMT Certificate: Handle: {0}", handle);
                            newTlsCert.Get(keys);
                        }
                        catch (Exception e)
                        {
                            WSManException wsManException = e as WSManException;
                            if (wsManException != null)
                            {
                                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsSettings WSMAN error",
                                String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                                wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                            }
                            CimException cimException = e as CimException;
                            if (cimException != null)
                            {
                                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsSettings CIM error",
                                String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                                cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                            }

                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsSettings",
                                String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                            Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                            Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                            newTlsCert = null;
                        }
                    } // END if (handleSpecified)

                    AMT_TLSCredentialContext tlsCredCtx = null;
                    try
                    {
                        tlsCredCtx = new AMT_TLSCredentialContext(system.WsMan);
                        tlsCredCtx.Get();
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        tlsCredCtx = null; // Will need to create new TLS credentials.
                    }

                    // Delete the previous context, if it exists.
                    if (tlsCredCtx != null)
                        tlsCredCtx.Delete();

                    // Create a new TLS credential with the specified certificate.
                    AMT_TLSProtocolEndpointCollection tlsProtocolEPs = new AMT_TLSProtocolEndpointCollection(system.WsMan);
                    tlsProtocolEPs.Get();

                    tlsCredCtx = new AMT_TLSCredentialContext(system.WsMan);
                    tlsCredCtx.ElementInContext = newTlsCert.Reference;
                    tlsCredCtx.ElementProvidingContext = tlsProtocolEPs.Reference;
                    tlsCredCtx.Create();

                    #endregion

                    #region Set TLS Settings

                    //
                    // Remote TLS Options
                    //
                    AMT_TLSSettingData.CimKeys remoteKeys = new AMT_TLSSettingData.CimKeys();
                    remoteKeys.InstanceID = "Intel(r) AMT 802.3 TLS Settings";
                    AMT_TLSSettingData remoteTlsSettings = new AMT_TLSSettingData(system.WsMan);
                    remoteTlsSettings.Get(remoteKeys);

                    if (remote == TlsAthenticationType.NoAuth)
                    {
                        remoteTlsSettings.Enabled = false;
                        remoteTlsSettings.MutualAuthentication = false;
                    }
                    else if (remote == TlsAthenticationType.ServerAuth)
                    {
                        remoteTlsSettings.Enabled = true;
                        remoteTlsSettings.MutualAuthentication = false;
                    }
                    else if (remote == TlsAthenticationType.MutualAuth)
                    {
                        remoteTlsSettings.Enabled = true;
                        remoteTlsSettings.MutualAuthentication = true;
                    }

                    // Trusted FQDN CNs
                    remoteTlsSettings.TrustedCN = fqdnSuffix;

                    // Accept Non-Secure Connection (AMT Release 6 and later)
                    if (system.CoreVersion >= 6.0)
                        remoteTlsSettings.AcceptNonSecureConnections = acceptNonSecureConnections;

                    remoteTlsSettings.Put();

                    //
                    // Local TLS Options
                    //
                    AMT_TLSSettingData.CimKeys localKeys = new AMT_TLSSettingData.CimKeys();
                    localKeys.InstanceID = "Intel(r) AMT LMS TLS Settings";
                    AMT_TLSSettingData localTlsSettings = new AMT_TLSSettingData(system.WsMan);
                    localTlsSettings.Get(localKeys);

                    if (local == TlsAthenticationType.NoAuth)
                    {
                        localTlsSettings.Enabled = false;
                        localTlsSettings.MutualAuthentication = false;
                    }
                    else if (local == TlsAthenticationType.ServerAuth)
                    {
                        localTlsSettings.Enabled = true;
                        localTlsSettings.MutualAuthentication = false;
                    }
                    else if (local == TlsAthenticationType.MutualAuth)
                    {
                        localTlsSettings.Enabled = true;
                        localTlsSettings.MutualAuthentication = true;
                    }

                    // Trusted FQDN CNs
                    localTlsSettings.TrustedCN = fqdnSuffix;

                    // Accept Non-Secure Connection (AMT Release 6 and later)
                    if (system.CoreVersion >= 6.0)
                        localTlsSettings.AcceptNonSecureConnections = acceptNonSecureConnections;

                    localTlsSettings.Put();

                    #endregion

                    // Commit changes
                    AMT_SetupAndConfigurationService scs = new AMT_SetupAndConfigurationService(system.WsMan);
                    return (AmtCallStatus)scs.CommitChanges();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsSettings WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsSettings CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsSettings",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }

                #endregion
            }
            else
            {
                #region EOI/SOAP

                //
                // Set the TLS Options
                //
                TlsOptionsType[] options = new TlsOptionsType[2];

                options[0] = new TlsOptionsType();
                options[0].Interface = InterfaceType.LocalHostInterface;
                options[0].TlsAuthentication = local;

                options[1] = new TlsOptionsType();
                options[1].Interface = InterfaceType.NetworkInterface;
                options[1].TlsAuthentication = remote;
                r = (AmtCallStatus)service.SetTlsOptions(options);
                if (r == AmtCallStatus.SUCCESS)
                    TlsOptionsCache = false;
                else
                    return r;

                //
                // Set the Trusted FQDN CNs
                //
                r = (AmtCallStatus)service.SetTrustedFqdnCN(fqdnSuffix);
                if (r == AmtCallStatus.SUCCESS)
                    TrustedFqdnCNCache = null;
                else
                    return r;

                // SetAcceptNonSecureConnections not supported.

                // Set the TLS Credentials
                r = (AmtCallStatus)service.SetTLSCredentials(handle, handleSpecified);

                // Commit changes.
                r = (AmtCallStatus)service.CommitChanges();
                return r;

                #endregion
            }
        }

        public AmtCallStatus UpdateTlsSettings(TlsAthenticationType local, TlsAthenticationType remote,
            string[] fqdnSuffix, bool acceptNonSecureConnections)
        {
            if (system.AmtRev <= 1 || system.LimitedFeatures == true) return AmtCallStatus.CALL_SKIPPED;

            if (local == TlsAthenticationType.NoAuth && remote != TlsAthenticationType.NoAuth) return AmtCallStatus.NOT_PERMITTED;
            if (remote == TlsAthenticationType.NoAuth && local != TlsAthenticationType.NoAuth) return AmtCallStatus.NOT_PERMITTED;

            AmtCallStatus r = AmtCallStatus.SUCCESS;

            if ((system.WsMan != null) && (system.CoreVersion >= 3))
            {
                #region WSMAN

                // Use WSMAN
                try
                {
                    #region Set TLS Settings

                    //
                    // Remote TLS Options
                    //
                    AMT_TLSSettingData.CimKeys remoteKeys = new AMT_TLSSettingData.CimKeys();
                    remoteKeys.InstanceID = "Intel(r) AMT 802.3 TLS Settings";
                    AMT_TLSSettingData remoteTlsSettings = new AMT_TLSSettingData(system.WsMan);
                    remoteTlsSettings.Get(remoteKeys);

                    if (remote == TlsAthenticationType.NoAuth)
                    {
                        remoteTlsSettings.Enabled = false;
                        remoteTlsSettings.MutualAuthentication = false;
                    }
                    else if (remote == TlsAthenticationType.ServerAuth)
                    {
                        remoteTlsSettings.Enabled = true;
                        remoteTlsSettings.MutualAuthentication = false;
                    }
                    else if (remote == TlsAthenticationType.MutualAuth)
                    {
                        remoteTlsSettings.Enabled = true;
                        remoteTlsSettings.MutualAuthentication = true;
                    }

                    // Trusted FQDN CNs
                    remoteTlsSettings.TrustedCN = fqdnSuffix;

                    // Accept Non-Secure Connection (AMT Release 6 and later)
                    if (system.CoreVersion >= 6.0)
                        remoteTlsSettings.AcceptNonSecureConnections = acceptNonSecureConnections;

                    remoteTlsSettings.Put();

                    //
                    // Local TLS Options
                    //
                    AMT_TLSSettingData.CimKeys localKeys = new AMT_TLSSettingData.CimKeys();
                    localKeys.InstanceID = "Intel(r) AMT LMS TLS Settings";
                    AMT_TLSSettingData localTlsSettings = new AMT_TLSSettingData(system.WsMan);
                    localTlsSettings.Get(localKeys);

                    if (local == TlsAthenticationType.NoAuth)
                    {
                        localTlsSettings.Enabled = false;
                        localTlsSettings.MutualAuthentication = false;
                    }
                    else if (local == TlsAthenticationType.ServerAuth)
                    {
                        localTlsSettings.Enabled = true;
                        localTlsSettings.MutualAuthentication = false;
                    }
                    else if (local == TlsAthenticationType.MutualAuth)
                    {
                        localTlsSettings.Enabled = true;
                        localTlsSettings.MutualAuthentication = true;
                    }

                    // Trusted FQDN CNs
                    localTlsSettings.TrustedCN = fqdnSuffix;

                    // Accept Non-Secure Connection (AMT Release 6 and later)
                    if (system.CoreVersion >= 6.0)
                        localTlsSettings.AcceptNonSecureConnections = acceptNonSecureConnections;

                    localTlsSettings.Put();

                    #endregion

                    // Commit changes
                    AMT_SetupAndConfigurationService scs = new AMT_SetupAndConfigurationService(system.WsMan);
                    return (AmtCallStatus)scs.CommitChanges();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsSettings WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsSettings CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsSettings",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }

                #endregion
            }
            else
            {
                #region EOI/SOAP

                //
                // Set the TLS Options
                //
                TlsOptionsType[] options = new TlsOptionsType[2];

                options[0] = new TlsOptionsType();
                options[0].Interface = InterfaceType.LocalHostInterface;
                options[0].TlsAuthentication = local;

                options[1] = new TlsOptionsType();
                options[1].Interface = InterfaceType.NetworkInterface;
                options[1].TlsAuthentication = remote;
                r = (AmtCallStatus)service.SetTlsOptions(options);
                if (r == AmtCallStatus.SUCCESS)
                    TlsOptionsCache = false;
                else
                    return r;

                //
                // Set the Trusted FQDN CNs
                //
                r = (AmtCallStatus)service.SetTrustedFqdnCN(fqdnSuffix);
                if (r == AmtCallStatus.SUCCESS)
                    TrustedFqdnCNCache = null;
                else
                    return r;

                // SetAcceptNonSecureConnections not supported.

                // Commit changes.
                r = (AmtCallStatus)service.CommitChanges();
                return r;

                #endregion
            }
        }

        public AmtCallStatus DisableTls()
        {
            if (system.AmtRev <= 1 || system.LimitedFeatures == true) return AmtCallStatus.CALL_SKIPPED;

            AmtCallStatus r = AmtCallStatus.SUCCESS;

            if ((system.WsMan != null) && (system.CoreVersion >= 3))
            {
                #region WSMAN

                // Use WSMAN
                try
                {
                    #region Set TLS Settings

                    Collection<AMT_TLSSettingData> tlsSettings = AMT_TLSSettingData.Enumerate(system.WsMan);
                    foreach (AMT_TLSSettingData tlsSetting in tlsSettings)
                    {
                        tlsSetting.Enabled = false;
                        tlsSetting.MutualAuthentication = false;
                        tlsSetting.Put();
                    }

                    #endregion

                    // Commit changes
                    AMT_SetupAndConfigurationService scs = new AMT_SetupAndConfigurationService(system.WsMan);
                    return (AmtCallStatus)scs.CommitChanges();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsSettings WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsSettings CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsSettings",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }

                #endregion
            }
            else
            {
                #region EOI/SOAP

                //
                // Set the TLS Options
                //
                TlsOptionsType[] options = new TlsOptionsType[2];

                options[0] = new TlsOptionsType();
                options[0].Interface = InterfaceType.LocalHostInterface;
                options[0].TlsAuthentication = TlsAthenticationType.NoAuth;

                options[1] = new TlsOptionsType();
                options[1].Interface = InterfaceType.NetworkInterface;
                options[1].TlsAuthentication = TlsAthenticationType.NoAuth;
                r = (AmtCallStatus)service.SetTlsOptions(options);
                if (r == AmtCallStatus.SUCCESS)
                    TlsOptionsCache = false;
                else
                    return r;

                // Set the TLS Credentials
                r = (AmtCallStatus)service.SetTLSCredentials(0, false);

                // Commit changes.
                r = (AmtCallStatus)service.CommitChanges();
                return r;

                #endregion
            }
        }

        public AmtCallStatus SetTrustedFqdnCN(string[] fqdnSuffix)
        {
            if (system.AmtRev <= 1 || system.LimitedFeatures == true) return AmtCallStatus.CALL_SKIPPED;
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    Collection<AMT_TLSSettingData> tlsSettings = AMT_TLSSettingData.Enumerate(system.WsMan);
                    foreach (AMT_TLSSettingData tlsSetting in tlsSettings)
                    {
                        tlsSetting.TrustedCN = fqdnSuffix;
                        tlsSetting.Put();
                    }
                    TrustedFqdnCNCache = null;
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTrustedFqdnCN WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTrustedFqdnCN CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTrustedFqdnCN",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                TrustedFqdnCNCache = null;
                return (AmtCallStatus)service.SetTrustedFqdnCN(fqdnSuffix);
            }
        }

        private PkiCapsType PkiCapabilitiesCache = null;
        public PkiCapsType GetPkiCapabilities()
        {
            if (system.AmtRev <= 1 || system.LimitedFeatures == true) return null;
            if (PkiCapabilitiesCache != null) return PkiCapabilitiesCache;

            if (system.WsMan != null)
            {
                AMT_PublicKeyManagementCapabilities ws_pkicaps;
                // Use WSMAN
                try
                {
                    ws_pkicaps = new AMT_PublicKeyManagementCapabilities(system.WsMan);
                    ws_pkicaps.Get();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetPkiCapabilities WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetPkiCapabilities CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetPkiCapabilities",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                    PkiCapsType c = new PkiCapsType();
                    PkiCapabilitiesCache = c;
                    return PkiCapabilitiesCache;
                }

                if (ws_pkicaps != null)
                {
                    PkiCapsType c = new PkiCapsType();
                    c.CertChainMaxSize = ws_pkicaps.CertChainMaxSize;
                    c.CrlStoreSize = ws_pkicaps.MaximumCRLSize;
                    c.FqdnSuffixMaxEntries = ws_pkicaps.CommonNameMaxCount;
                    c.FqdnSuffixMaxEntryLength = ws_pkicaps.CommonNameMaxSize;
                    c.RootCertMaxInstances = ws_pkicaps.RootCertMaxCount;
                    c.RootCertMaxSize = ws_pkicaps.RootCertMaxSize;
                    c.SupportedKeyLengths = ws_pkicaps.SupportedKeyLengths;
                    PkiCapabilitiesCache = c;
                    return PkiCapabilitiesCache;
                }
                else
                {
                    PkiCapsType c = new PkiCapsType();
                    PkiCapabilitiesCache = c;
                    return PkiCapabilitiesCache;
                }
            }
            else
            {
                // Use SOAP
                AmtCallStatus r = (AmtCallStatus)service.GetPkiCapabilities(out PkiCapabilitiesCache);
                return PkiCapabilitiesCache;
            }
        }

        public byte[] GetServerCertificateReq()
        {
            byte[] request = null;
            AmtCallStatus r = (AmtCallStatus)service.GetServerCertificateReq(out request);
            if (r != AmtCallStatus.SUCCESS) request = null;
            return request;
        }

        private string DigestRealmCache = null;
        public string GetDigestRealm()
        {
            if (system.AmtRev <= 1) return null;
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_GeneralSettings s = new AMT_GeneralSettings(system.WsMan);
                    s.Get();
                    return s.DigestRealm;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetDigestRealm WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetDigestRealm CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetDigestRealm",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return String.Empty;
                }
            }
            else
            {
                // Use SOAP
                if (DigestRealmCache != null) return DigestRealmCache;
                AmtCallStatus r = (AmtCallStatus)service.GetDigestRealm(out DigestRealmCache);
                return DigestRealmCache;
            }
        }

        public AmtCallStatus SetRngKey()
        {
            if (system.WsMan != null)
            {
                // Can't find WSMAN call for this.
                return AmtCallStatus.SUCCESS;
            }
            else
            {
                byte[] b = new byte[20];
                new Random().NextBytes(b);
                RngKeyType k = new RngKeyType();
                k.Type = 21; // Bare
                k.Length = 20;
                k.Data = b;
                return (AmtCallStatus)service.SetRngKey(k);
            }
        }

        private X509Certificate2[] TLSCertificateCache = null;
        public X509Certificate2[] GetTLSCertificate()
        {
            if (TLSCertificateCache != null) return TLSCertificateCache;

            if (system.WsMan != null)
            {
                try
                {
                    AMT_TLSCredentialContext tlsCredCtx = new AMT_TLSCredentialContext(system.WsMan);
                    tlsCredCtx.Get();
                    Debug.Assert(tlsCredCtx.ElementInContext.GetReferenceType() == typeof(AMT_PublicKeyCertificate));
                    AMT_PublicKeyCertificate pubKeyCert = new AMT_PublicKeyCertificate(system.WsMan);
                    pubKeyCert.Get(tlsCredCtx.ElementInContext);
                    TLSCertificateCache = new X509Certificate2[1];
                    TLSCertificateCache[0] = (pubKeyCert.X509CertificateExist ? new X509Certificate2(pubKeyCert.X509Certificate) : null);
                    return TLSCertificateCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetDigestRealm WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetDigestRealm CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetDigestRealm",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                    TLSCertificateCache = new X509Certificate2[1];
                    return TLSCertificateCache;
                }
            }
            else
            {
                if (system.CoreVersion >= 3)
                {
                    uint handle;
                    bool handleSpecified;
                    AmtCallStatus r = (AmtCallStatus)service.GetTLSCredentials(out handle, out handleSpecified);
                    if (r != AmtCallStatus.SUCCESS || handleSpecified == false)
                    {
                        TLSCertificateCache = new X509Certificate2[0];
                    }
                    else
                    {
                        AmtStoredCert[] certificates;
                        r = CertStoreGetCertificates(out certificates);
                        foreach (AmtStoredCert c in certificates)
                        {
                            if (c.Handle == handle)
                            {
                                TLSCertificateCache = new X509Certificate2[1];
                                TLSCertificateCache[0] = c.Certificate;
                                return TLSCertificateCache;
                            }
                        }
                        TLSCertificateCache = new X509Certificate2[0];
                    }
                }
                else
                {
                    CertificateChainType certChain;
                    AmtCallStatus r = (AmtCallStatus)service.GetTLSCertificate(out certChain);

                    byte[] certs = certChain.Value;
                    ArrayList l = new ArrayList();
                    while (certs.Length > 3)
                    {
                        int certlen = (certs[0] << 16) + (certs[1] << 8) + (certs[2]);
                        byte[] certbin = new byte[certlen];
                        Array.Copy(certs, 3, certbin, 0, certlen);
                        try
                        {
                            l.Add(new X509Certificate2(certbin));
                        }
                        catch (Exception)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.Event, Properties.Resources.InvalidCertReturned);
                        }
                        byte[] certs2 = new byte[certs.Length - (3 + certlen)];
                        Array.Copy(certs, 3 + certlen, certs2, 0, certs2.Length);
                        certs = certs2;
                    }

                    TLSCertificateCache = (X509Certificate2[])l.ToArray(typeof(X509Certificate2));
                }

                return TLSCertificateCache;
            }
        }

        public AmtCallStatus SetTLSCertificate(X509Certificate2[] certificateChain)
        {
            if (certificateChain == null)
            {
                AmtCallStatus rr = (AmtCallStatus)service.SetTLSCertificate(null); // TODO: HOW DO WE CLEAR A CERTIFICATE??? ****
                return rr;
            }
            TLSCertificateCache = null;
            CertificateChainType certChain = new CertificateChainType();
            certChain.Encoding = 11;
            certChain.Value = CertificateOperations.CertificateChainToBytes(certificateChain);

            AmtCallStatus r = (AmtCallStatus)service.SetTLSCertificate(certChain);
            return r;
        }

        public AmtCallStatus SetTLSPSK(byte[] pid, byte[] pps)
        {
            if (pid == null || pid.Length != 8 || pps == null || pps.Length != 32) return AmtCallStatus.NOT_PERMITTED;
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_SetupAndConfigurationService scs = new AMT_SetupAndConfigurationService(system.WsMan);
                    return (AmtCallStatus)scs.SetTLSPSK(pid, pps);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTLSPSK WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTLSPSK CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTLSPSK",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                return (AmtCallStatus)service.SetTLSPSK(pid, pps);
            }
        }

        private X509Certificate2[] TrustedRootCertificatesCache = null;
        public X509Certificate2[] GetTrustedRootCertificates()
        {
            if (system.AmtRev <= 1) return null;
            if (TrustedRootCertificatesCache != null) return TrustedRootCertificatesCache;

            if (system.WsMan != null)
            {
                Collection<AMT_PublicKeyCertificate> pubKeyCerts = AMT_PublicKeyCertificate.Enumerate(system.WsMan);
                ArrayList certs = new ArrayList();
                foreach (AMT_PublicKeyCertificate pubKeyCert in pubKeyCerts)
                {
                    if (pubKeyCert.TrustedRootCertficateExist && pubKeyCert.TrustedRootCertficate && pubKeyCert.X509CertificateExist)
                        certs.Add(new X509Certificate2(pubKeyCert.X509Certificate));
                }
                TrustedRootCertificatesCache = (X509Certificate2[])certs.ToArray(typeof(X509Certificate2));
                return TrustedRootCertificatesCache;
            }
            else
            {
                uint[] certHandles;
                AmtCallStatus r = (AmtCallStatus)service.EnumerateTrustedRootCertificates(out certHandles);

                ArrayList certs = new ArrayList();
                foreach (uint h in certHandles)
                {
                    CertificateType certbin;
                    r = (AmtCallStatus)service.GetTrustedRootCertificate(h, out certbin);
                    X509Certificate2 cert = new X509Certificate2(certbin.X509cert);
                    certs.Add(cert);
                }

                TrustedRootCertificatesCache = (X509Certificate2[])certs.ToArray(typeof(X509Certificate2));
                return TrustedRootCertificatesCache;
            }
        }

        public AmtCallStatus SetTrustedRootCertificates(X509Certificate2[] trustedCerts)
        {
            if (system.AmtRev <= 1) return AmtCallStatus.CALL_SKIPPED;

            AmtCallStatus r = AmtCallStatus.SUCCESS;
            foreach (X509Certificate2 trustedCert in trustedCerts)
            {
                uint certHandle;
                CertificateType certbin = new CertificateType();
                certbin.X509cert = trustedCert.GetRawCertData();
                r = (AmtCallStatus)service.AddTrustedRootCertificate(certbin, out certHandle);
                if (r != AmtCallStatus.SUCCESS) break;
            }
            TrustedRootCertificatesCache = null;
            return r;
        }

        public void DeleteAllTrustedRootCertificates()
        {
            if (system.AmtRev <= 1) return;

            uint[] certHandles;
            AmtCallStatus r = (AmtCallStatus)service.EnumerateTrustedRootCertificates(out certHandles);
            foreach (uint h in certHandles) service.DeleteTrustedRootCertificate(h);
            TrustedRootCertificatesCache = null;
        }

        private bool TlsOptionsCache = false;
        private TlsAthenticationType TlsOptionsLocalCache = TlsAthenticationType.NoAuth;
        private TlsAthenticationType TlsOptionsRemoteCache = TlsAthenticationType.NoAuth;
        public AmtCallStatus GetTlsOptions(out TlsAthenticationType local, out TlsAthenticationType remote)
        {
            local = TlsAthenticationType.NoAuth;
            remote = TlsAthenticationType.NoAuth;

            if (system.AmtRev <= 1 || system.LimitedFeatures == true)
            {
                local = TlsAthenticationType.NoAuth;
                remote = TlsAthenticationType.NoAuth;
                return AmtCallStatus.SUCCESS;
            }

            if (TlsOptionsCache == true)
            {
                local = TlsOptionsLocalCache;
                remote = TlsOptionsRemoteCache;
                return AmtCallStatus.SUCCESS;
            }

            if (system.WsMan != null)
            {
                remote = TlsAthenticationType.NoAuth;
                local = TlsAthenticationType.NoAuth;

                try
                {
                    //
                    // Remote TLS Options
                    AMT_TLSSettingData.CimKeys remoteKeys = new AMT_TLSSettingData.CimKeys();
                    remoteKeys.InstanceID = "Intel(r) AMT 802.3 TLS Settings";
                    AMT_TLSSettingData remoteTlsSettings = new AMT_TLSSettingData(system.WsMan);
                    remoteTlsSettings.Get(remoteKeys);

                    if (remoteTlsSettings.Enabled)
                    {
                        if (remoteTlsSettings.MutualAuthenticationExist && remoteTlsSettings.MutualAuthentication)
                            remote = TlsAthenticationType.MutualAuth;
                        else
                            remote = TlsAthenticationType.ServerAuth;
                    }

                    //
                    // Local TLS Options
                    //
                    AMT_TLSSettingData.CimKeys localKeys = new AMT_TLSSettingData.CimKeys();
                    localKeys.InstanceID = "Intel(r) AMT LMS TLS Settings";
                    AMT_TLSSettingData localTlsSettings = new AMT_TLSSettingData(system.WsMan);
                    localTlsSettings.Get(localKeys);

                    if (localTlsSettings.Enabled)
                    {
                        if (localTlsSettings.MutualAuthenticationExist && localTlsSettings.MutualAuthentication)
                            local = TlsAthenticationType.MutualAuth;
                        else
                            local = TlsAthenticationType.ServerAuth;
                    }
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetTlsOptions WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetTlsOptions CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetTlsOptions",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }

                return AmtCallStatus.SUCCESS;
            }
            else
            {
                // Use SOAP
                local = TlsAthenticationType.NoAuth;
                remote = TlsAthenticationType.NoAuth;

                TlsOptionsType[] options;
                AmtCallStatus r = (AmtCallStatus)service.GetTlsOptions(out options);
                if (r != AmtCallStatus.SUCCESS) return r;

                foreach (TlsOptionsType o in options)
                {
                    if (o.Interface == InterfaceType.LocalHostInterface) local = o.TlsAuthentication;
                    if (o.Interface == InterfaceType.NetworkInterface) remote = o.TlsAuthentication;
                }

                TlsOptionsRemoteCache = remote;
                TlsOptionsLocalCache = local;
                TlsOptionsCache = true;
                return r;
            }
        }

        public AmtCallStatus SetTlsOptions(TlsAthenticationType local, TlsAthenticationType remote)
        {
            if (system.AmtRev <= 1 || system.LimitedFeatures == true) return AmtCallStatus.CALL_SKIPPED;

            if (local == TlsAthenticationType.NoAuth && remote != TlsAthenticationType.NoAuth) return AmtCallStatus.NOT_PERMITTED;
            if (remote == TlsAthenticationType.NoAuth && local != TlsAthenticationType.NoAuth) return AmtCallStatus.NOT_PERMITTED;

            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    //
                    // Remote TLS Options
                    AMT_TLSSettingData.CimKeys remoteKeys = new AMT_TLSSettingData.CimKeys();
                    remoteKeys.InstanceID = "Intel(r) AMT 802.3 TLS Settings";
                    AMT_TLSSettingData remoteTlsSettings = new AMT_TLSSettingData(system.WsMan);
                    remoteTlsSettings.Get(remoteKeys);

                    if (remote == TlsAthenticationType.NoAuth)
                    {
                        remoteTlsSettings.Enabled = false;
                        remoteTlsSettings.MutualAuthentication = false;
                    }
                    else if (remote == TlsAthenticationType.ServerAuth)
                    {
                        remoteTlsSettings.Enabled = true;
                        remoteTlsSettings.MutualAuthentication = false;
                    }
                    else if (remote == TlsAthenticationType.MutualAuth)
                    {
                        remoteTlsSettings.Enabled = true;
                        remoteTlsSettings.MutualAuthentication = true;
                    }
                    remoteTlsSettings.Put();

                    //
                    // Local TLS Options
                    //
                    AMT_TLSSettingData.CimKeys localKeys = new AMT_TLSSettingData.CimKeys();
                    localKeys.InstanceID = "Intel(r) AMT LMS TLS Settings";
                    AMT_TLSSettingData localTlsSettings = new AMT_TLSSettingData(system.WsMan);
                    localTlsSettings.Get(localKeys);

                    // Local
                    if (local == TlsAthenticationType.NoAuth)
                    {
                        localTlsSettings.Enabled = false;
                        localTlsSettings.MutualAuthentication = false;
                    }
                    else if (local == TlsAthenticationType.ServerAuth)
                    {
                        localTlsSettings.Enabled = true;
                        localTlsSettings.MutualAuthentication = false;
                    }
                    else if (local == TlsAthenticationType.MutualAuth)
                    {
                        localTlsSettings.Enabled = true;
                        localTlsSettings.MutualAuthentication = true;
                    }
                    localTlsSettings.Put();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsOptions WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsOptions CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsOptions",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }

                return AmtCallStatus.SUCCESS;
            }
            else
            {
                // Use SOAP
                TlsOptionsType[] options = new TlsOptionsType[2];
                options[0] = new TlsOptionsType();
                options[0].Interface = InterfaceType.LocalHostInterface;
                options[0].TlsAuthentication = local;
                options[1] = new TlsOptionsType();
                options[1].Interface = InterfaceType.NetworkInterface;
                options[1].TlsAuthentication = remote;
                TlsOptionsCache = false;
                return (AmtCallStatus)service.SetTlsOptions(options);
            }
        }

        public AmtCallStatus SetTLSKeyAndCertificate(byte[] rsaPrivateKey, X509Certificate2[] certificateChain)
        {
            if (system.AmtRev <= 1) return AmtCallStatus.CALL_SKIPPED;

            RsaKeyType privateKey = new RsaKeyType();
            privateKey.Encoding = 31; // RsaKeyEncodingTypesSshv2Mpint
            privateKey.Value = rsaPrivateKey;
            privateKey.Length = (uint)rsaPrivateKey.Length;

            CertificateChainType certChain = new CertificateChainType();
            certChain.Encoding = 11;
            certChain.Value = CertificateOperations.CertificateChainToBytes(certificateChain);
            certChain.Length = (uint)certChain.Value.Length;

            TLSCertificateCache = null;
            return (AmtCallStatus)service.SetTLSKeyAndCertificate(privateKey, certChain);
        }

        public AmtCallStatus GetAcceptNonSecureConnections(out bool acceptNonSecureConnections)
        {
            acceptNonSecureConnections = false;

            // Only supported in WSMAN and AMT6 or later.
            if (system.WsMan == null)
                return AmtCallStatus.NO_WSMAN;
            if (system.CoreVersion < 6.0)
                return AmtCallStatus.UNSUPPORTED;

            try
            {
                Collection<AMT_TLSSettingData> tlsSettings = AMT_TLSSettingData.Enumerate(system.WsMan);
                foreach (AMT_TLSSettingData tlsSetting in tlsSettings)
                {
                    if (tlsSetting.AcceptNonSecureConnectionsExist)
                        acceptNonSecureConnections = tlsSetting.AcceptNonSecureConnections;

                    return AmtCallStatus.SUCCESS;
                }
                return AmtCallStatus.FAILED_WEB_CALL;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetAcceptNonSecureConnections WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetAcceptNonSecureConnections CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetAcceptNonSecureConnections",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        public AmtCallStatus SetAcceptNonSecureConnections(bool acceptNonSecureConnections)
        {
            // Only supported in WSMAN and AMT6 or later.
            if (system.WsMan == null)
                return AmtCallStatus.NO_WSMAN;
            if (system.CoreVersion < 6.0)
                return AmtCallStatus.UNSUPPORTED;

            try
            {
                Collection<AMT_TLSSettingData> tlsSettings = AMT_TLSSettingData.Enumerate(system.WsMan);
                foreach (AMT_TLSSettingData tlsSetting in tlsSettings)
                {
                    tlsSetting.AcceptNonSecureConnections = acceptNonSecureConnections;
                    tlsSetting.Put();
                }
                return AmtCallStatus.SUCCESS;
            }
            catch (Exception e)
            {
                WSManException wsManException = e as WSManException;
                if (wsManException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetAcceptNonSecureConnections WSMAN error",
                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                }
                CimException cimException = e as CimException;
                if (cimException != null)
                {
                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetAcceptNonSecureConnections CIM error",
                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                }

                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetAcceptNonSecureConnections",
                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                return AmtCallStatus.FAILED_WEB_CALL;
            }
        }

        private bool KerberosOptionsCacheSet = false;
        private KerberosOptionsType KerberosOptionsCache = null;
        public AmtCallStatus GetKerberosOptions(out KerberosOptionsType data)
        {
            if (system.AmtRev <= 1 || system.LimitedFeatures == true)
            {
                data = null;
                return AmtCallStatus.NOT_PERMITTED;
            }
            data = null;
            if (KerberosOptionsCacheSet == true)
            {
                if (KerberosOptionsCache == null)
                {
                    return AmtCallStatus.NOT_PERMITTED;
                }
                else
                {
                    data = KerberosOptionsCache;
                    return AmtCallStatus.SUCCESS;
                }
            }

            if (system.WsMan != null)
            {
                AMT_KerberosSettingData wsker = null;
                // Use WSMAN
                try
                {
                    wsker = new AMT_KerberosSettingData(system.WsMan);
                    wsker.Get();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetKerberosOptions WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetKerberosOptions CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetKerberosOptions",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }

                if (wsker == null || !wsker.RealmNameExist)
                {
                    KerberosOptionsCacheSet = true;
                    KerberosOptionsCache = null;
                    return AmtCallStatus.SUCCESS;
                }
                else
                {
                    data = new KerberosOptionsType();
                    data.KerberosEncryption = KerberosEncryptionType.HMAC_RC4;
                    data.KerberosKeyVersion = wsker.KeyVersion;
                    data.KerberosMasterKey = (wsker.MasterKeyExist ? wsker.MasterKey : new byte[] { });
                    data.KerberosMaximumClockTolerance = (wsker.MaximumClockToleranceExist ? wsker.MaximumClockTolerance : 0);
                    data.KerberosRealmName = wsker.RealmName;
                    ArrayList l = new ArrayList();
                    if (wsker.ServicePrincipalNameExist)
                    {
                        foreach (string s in wsker.ServicePrincipalName)
                        {
                            KerberosSpnType ks = new KerberosSpnType();
                            ks.SpnString = s;
                            ks.SpnProtocolIndex = 0; // TODO????
                        }
                    }
                    data.KerberosSpn = (KerberosSpnType[])l.ToArray(typeof(KerberosSpnType));
                    KerberosOptionsCacheSet = true;
                    KerberosOptionsCache = data;
                    return AmtCallStatus.SUCCESS;
                }
            }
            else
            {
                // Use SOAP
                AmtCallStatus r = AmtCallStatus.NOT_PERMITTED;
                try // This call is not always liked by Intel AMT, so we want to sheild it.
                {
                    r = (AmtCallStatus)service.GetKerberosOptions(out data);
                }
                catch (Exception)
                {
                    KerberosOptionsCacheSet = true;
                    KerberosOptionsCache = null;
                    return r;
                }
                if (r == AmtCallStatus.DATA_MISSING)
                {
                    data = null;
                    r = AmtCallStatus.SUCCESS;
                }
                KerberosOptionsCacheSet = true;
                KerberosOptionsCache = data;
                return r;
            }
        }

        public AmtCallStatus SetKerberosOptions(KerberosOptionsType options)
        {
            KerberosOptionsCacheSet = false;
            KerberosOptionsCache = null;
            return (AmtCallStatus)service.SetKerberosOptions(options);
        }

        private string AdminAclEntryCache = null;
        public string GetAdminAclEntry()
        {
            if (AdminAclEntryCache != null) return AdminAclEntryCache;
            string username = null;
            AmtCallStatus r;
            if (system.CoreVersion == 0) return "admin";
            
            if(system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AMT_AuthorizationService authService = new AMT_AuthorizationService(system.WsMan);
                    r = (AmtCallStatus)authService.GetAdminAclEntry(out username);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetAdminAclEntry WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetAdminAclEntry CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetAdminAclEntry",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    throw;
                }
            }
            else
            {   
                // USE EOI
                r = (AmtCallStatus)service.GetAdminAclEntry(out username);
            }
            AdminAclEntryCache = username;
            return AdminAclEntryCache;
        }

        public AmtCallStatus SetAdminAclEntryEx(string username, string password)
        {
            if (system.AmtRev <= 1) return AmtCallStatus.CALL_SKIPPED;
            AdminAclEntryCache = null;
            
            AmtCallStatus r;
            if(system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AMT_AuthorizationService authService = new AMT_AuthorizationService(system.WsMan);
                    return (AmtCallStatus)authService.SetAdminAclEntryEx(username, ComputerPasswordDigest(username, password));
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetAdminAclEntryEx WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetAdminAclEntryEx CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetAdminAclEntryEx",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    throw;
                }
            }
            else
            {
                // USE EOI
                AdminAclEntryExType entry = new AdminAclEntryExType();
                entry.Username = username;
                entry.DigestPassword = ComputerPasswordDigest(username, password);
                r = (AmtCallStatus)service.SetAdminAclEntryEx(entry); // TODO: Add support for SetAdminAclEntry
            }
            return r;
        }

        public enum AmtUserAclRealm
        {
            InvalidRealm = 0,
            Realm1 = 1,
            RedirectionRealm = 2,
            PTAdministrationRealm = 3,
            HardwareAssetRealm = 4,
            RemoteControlRealm = 5,
            StorageRealm = 6,
            EventManagerRealm = 7,
            StorageAdminRealm = 8,
            AgentPresenceLocalRealm = 9,
            AgentPresenceRemoteRealm = 10,
            CircuitBreakerRealm = 11,
            NetworkTimeRealm = 12,
            GeneralInfoRealm = 13,
            FirmwareUpdateRealm = 14,
            Realm15 = 15,
            LocalUN = 16,
            EndpointAccessControlRealm = 17,
            EndpointAccessControlAdminRealm = 18,
            Realm19,
            Realm20,
            Realm21,
            Realm22,
            Realm23
        };

        public enum AmtUserAclType
        {
            Unknown,
            Digest,
            Kerberos
        }

        [Serializable]
        public class AmtUserAcl
        {
            public AccessPermissionType AccessPermission;
            public AmtUserAclType Type;
            public byte[] KerberosSid;
            public string Username;
            public string Password;
            public byte[] PasswordDigest;
            public AmtUserAclRealm[] Realms;
            public uint Handle;
            public bool HandleSet = false;

            public AmtUserAcl(string username, string password, AccessPermissionType accessPermission, AmtUserAclRealm[] realms)
            {
                Type = AmtUserAclType.Digest;
                Username = username;
                Password = password;
                AccessPermission = accessPermission;
                Realms = realms;
            }

            public AmtUserAcl(byte[] SID, AccessPermissionType accessPermission, AmtUserAclRealm[] realms)
            {
                Type = AmtUserAclType.Kerberos;
                KerberosSid = SID;
                AccessPermission = accessPermission;
                Realms = realms;
            }

            public AmtUserAcl(uint handle, UserAclEntryExType entry)
            {
                Handle = handle;
                HandleSet = true;
                AccessPermission = entry.AccessPermission;
                ArrayList l = new ArrayList();
                foreach (uint r in entry.Realms) l.Add((AmtUserAclRealm)r);
                Realms = (AmtUserAclRealm[])l.ToArray(typeof(AmtUserAclRealm));
                Type = AmtUserAclType.Unknown;
                if (entry.Item.GetType() == typeof(UserEntryDigestType))
                {
                    Type = AmtUserAclType.Digest;
                    UserEntryDigestType d = (UserEntryDigestType)entry.Item;
                    Username = d.Username;
                    PasswordDigest = d.DigestPassword;
                }
                else if (entry.Item.GetType() == typeof(UserEntryKerberosType))
                {
                    Type = AmtUserAclType.Kerberos;
                    UserEntryKerberosType k = (UserEntryKerberosType)entry.Item;
                    KerberosSid = k.Sid;
                }
            }
            
            public AmtUserAcl(uint handle, string username, byte[] passwordDigest, uint accessPermission, uint[] realms)
            {
                Handle = handle;
                HandleSet = true;
                AccessPermission = (AccessPermissionType)accessPermission;
                ArrayList l = new ArrayList();
                if (realms != null) foreach (uint r in realms) l.Add((AmtUserAclRealm)r);
                Realms = (AmtUserAclRealm[])l.ToArray(typeof(AmtUserAclRealm));
                Type = AmtUserAclType.Digest;
                Username = username;
                PasswordDigest = passwordDigest;
            }

            public AmtUserAcl(uint handle, byte[] kerberosSid, uint accessPermission, uint[] realms)
            {
                Handle = handle;
                HandleSet = true;
                AccessPermission = (AccessPermissionType)accessPermission;
                ArrayList l = new ArrayList();
                if (realms != null) foreach (uint r in realms) l.Add((AmtUserAclRealm)r);
                Realms = (AmtUserAclRealm[])l.ToArray(typeof(AmtUserAclRealm));
                Type = AmtUserAclType.Kerberos;
                KerberosSid = kerberosSid;
            }           

            public UserAclEntryExType GetUserAclEntryEx(AmtSecurityAdmin secAdmin)
            {
                UserAclEntryExType entry = new UserAclEntryExType();
                entry.AccessPermission = AccessPermission;

                // Set realms
                ArrayList l = new ArrayList();
                foreach (AmtUserAclRealm r in Realms) l.Add((uint)r);
                entry.Realms = (uint[])l.ToArray(typeof(uint));

                // Set user info
                entry.Item = null;
                if (Type == AmtUserAclType.Digest)
                {
                    UserEntryDigestType d = new UserEntryDigestType();
                    d.Username = Username;
                    if (PasswordDigest == null) PasswordDigest = secAdmin.ComputerPasswordDigest(Username, Password);
                    d.DigestPassword = PasswordDigest;
                    entry.Item = d;
                }
                else if (Type == AmtUserAclType.Kerberos)
                {
                    UserEntryKerberosType d = new UserEntryKerberosType();
                    d.Sid = KerberosSid;
                    entry.Item = d;
                }

                if (entry.Realms == null) entry.Realms = new uint[0];
                return entry;
            }
        }

        private AmtUserAcl[] UserAclEntriesCache = null;
        public AmtCallStatus GetUserAclEntries(out AmtUserAcl[] accounts)
        {
            accounts = null;
            if (UserAclEntriesCache != null)
            {
                accounts = UserAclEntriesCache;
                return AmtCallStatus.SUCCESS;
            }
            if (system.CoreVersion == 0)
            {
                accounts = new AmtUserAcl[0];
                return AmtCallStatus.SUCCESS;
            }

            uint totalCount;
            uint handleCount;
            uint[] handles = null;
            AmtCallStatus r;
            
            if(system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AMT_AuthorizationService authService = new AMT_AuthorizationService(system.WsMan);
                    r = (AmtCallStatus)authService.EnumerateUserAclEntries(1, out totalCount, out handleCount, out handles);
                    if (r != AmtCallStatus.SUCCESS) return r;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetUserAclEntries WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetUserAclEntries CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetUserAclEntries",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    throw;
                }
            }
            else
            {   
                // USE EOI
                r = (AmtCallStatus)service.EnumerateUserAclEntries(1, out totalCount, out handleCount, out handles);
                if (r != AmtCallStatus.SUCCESS) return r;
            }
            
            ArrayList userAcls = new ArrayList();
            foreach (uint h in handles)
            {
                UserAclEntryExType entry;
                try
                {
                    if (system.WsMan != null)
                    {
                        // USE WSMAN
                        try
                        {
                            string digestUsername;
                            byte[] digestPassword;
                            byte[] kerberosUserSid;
                            uint accessPermission;
                            uint[] realms;

                            AMT_AuthorizationService authService = new AMT_AuthorizationService(system.WsMan);
                            r = (AmtCallStatus)authService.GetUserAclEntryEx(h, out digestUsername, out digestPassword,
                                out kerberosUserSid, out accessPermission, out realms);
                            if (r != AmtCallStatus.SUCCESS) return r;

                            if (kerberosUserSid != null)
                            {
                                // Kerberos* type
                                userAcls.Add(new AmtUserAcl(h, kerberosUserSid, accessPermission, realms));
                            }
                            else
                            {
                                // Digest type
                                userAcls.Add(new AmtUserAcl(h, digestUsername, digestPassword, accessPermission, realms));
                            }
                        }
                        catch (Exception e)
                        {
                            WSManException wsManException = e as WSManException;
                            if (wsManException != null)
                            {
                                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetUserAclEntries WSMAN error",
                                String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                                wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                            }
                            CimException cimException = e as CimException;
                            if (cimException != null)
                            {
                                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetUserAclEntries CIM error",
                                String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                                cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                            }

                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetUserAclEntries",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                            throw;
                        }
                    }
                    else
                    {
                        // USE EOI
                        r = (AmtCallStatus)service.GetUserAclEntryEx(h, out entry);
                        if (r != AmtCallStatus.SUCCESS) return r;
                        if (entry.Realms == null) entry.Realms = new uint[0];
                        userAcls.Add(new AmtUserAcl(h, entry));
                    }
                }
                catch (Exception) {break;}
            }

            UserAclEntriesCache = (AmtUserAcl[])userAcls.ToArray(typeof(AmtUserAcl));
            accounts = UserAclEntriesCache;
            return AmtCallStatus.SUCCESS;
        }

        public AmtCallStatus RemoveAllUserAclEntries()
        {
            uint totalCount;
            uint handleCount;
            uint[] handles = null;
            AmtCallStatus r;
            if (system.CoreVersion == 0) return AmtCallStatus.NOT_PERMITTED;

            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AMT_AuthorizationService authService = new AMT_AuthorizationService(system.WsMan);
                    r = (AmtCallStatus)authService.EnumerateUserAclEntries(1, out totalCount, out handleCount, out handles);
                    if (r != AmtCallStatus.SUCCESS) return r;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.RemoveAllUserAclEntries WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.RemoveAllUserAclEntries CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.RemoveAllUserAclEntries",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    throw;
                }
            }
            else
            {
                // USE EOI
                r = (AmtCallStatus)service.EnumerateUserAclEntries(1, out totalCount, out handleCount, out handles);
            }             
            
            if (r != AmtCallStatus.SUCCESS) return r;
            foreach (uint h in handles)
            {
                if(system.WsMan != null)
                {
                    // USE WSMAN
                    try
                    {
                        AMT_AuthorizationService authService = new AMT_AuthorizationService(system.WsMan);
                        r = (AmtCallStatus)authService.RemoveUserAclEntry(h);
                        if (r != AmtCallStatus.SUCCESS) return r;
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        if (wsManException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.RemoveAllUserAclEntries WSMAN error",
                            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                        }
                        CimException cimException = e as CimException;
                        if (cimException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.RemoveAllUserAclEntries CIM error",
                            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                        }

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.RemoveAllUserAclEntries",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                        throw;
                    }
                }
                else
                {
                    // USE EOI
                    r = (AmtCallStatus)service.RemoveUserAclEntry(h);
                }
                if (r != AmtCallStatus.SUCCESS) return r;
            }
            UserAclEntriesCache = null;
            return AmtCallStatus.SUCCESS;
        }

        public AmtCallStatus SetUserAclEntries(AmtUserAcl[] userAcls)
        {
            if (system.CoreVersion == 0) return AmtCallStatus.NOT_PERMITTED;
            foreach (AmtUserAcl userAcl in userAcls)
            {
                if (userAcl.Username != null && userAcl.Username.StartsWith("$$")) continue;
                AmtCallStatus r = AddUserAclEntry(userAcl);
                if (r != AmtCallStatus.SUCCESS) return r;
            }
            UserAclEntriesCache = null;
            return AmtCallStatus.SUCCESS;
        }

        public AmtCallStatus AddUserAclEntry(AmtUserAcl userAcl)
        {
            if (userAcl.Username != null && userAcl.Username.StartsWith("$$")) return AmtCallStatus.CALL_SKIPPED;
            if (system.CoreVersion == 0) return AmtCallStatus.NOT_PERMITTED;

            uint handle;
            AmtCallStatus r;
            try
            {
                if(system.WsMan != null)
                {
                    // USE WSMAN
                    try
                    {
                        ArrayList l = new ArrayList();
                        foreach (AmtUserAclRealm rl in userAcl.Realms)
                        {
                            if ((int)rl <= 15) l.Add((uint)rl);
                            else if ((int)rl <= 17 && system.CoreVersion >= 2.5) l.Add((uint)rl);
                            else if ((int)rl <= 17 && system.CoreVersion >= 3.0) l.Add((uint)rl);
                            else if ((int)rl <= 21 && system.CoreVersion >= 4.0) l.Add((uint)rl);
                            else if ((int)rl <= 23 && system.CoreVersion >= 5.0) l.Add((uint)rl);
                        }

                        AMT_AuthorizationService authService = new AMT_AuthorizationService(system.WsMan);
                        if (userAcl.Type == AmtUserAclType.Digest)
                        {
                            r = (AmtCallStatus)authService.AddUserAclEntryEx(userAcl.Username,
                                ComputerPasswordDigest(userAcl.Username, userAcl.Password), null,
                                (uint)userAcl.AccessPermission, (uint[])l.ToArray(typeof(uint)), out handle);
                        }
                        else if (userAcl.Type == AmtUserAclType.Kerberos)
                        {
                            r = (AmtCallStatus)authService.AddUserAclEntryEx(null, null, userAcl.KerberosSid,
                                (uint)userAcl.AccessPermission, (uint[])l.ToArray(typeof(uint)), out handle);
                        }
                        else
                            throw new Exception("AmtUserAcl Unknown type");
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        if (wsManException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.AddUserAclEntry WSMAN error",
                            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                        }
                        CimException cimException = e as CimException;
                        if (cimException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.AddUserAclEntry CIM error",
                            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                        }

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.AddUserAclEntry",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                        throw;
                    }
                }                
                else
                {
                    // USE EOI
                    AmtUserAcl u2 = new AmtUserAcl("", "", AccessPermissionType.AnyAccessPermission, null);
                    u2.AccessPermission = userAcl.AccessPermission;
                    u2.KerberosSid = userAcl.KerberosSid;
                    u2.Password = userAcl.Password;
                    u2.PasswordDigest = userAcl.PasswordDigest;
                    u2.Type = userAcl.Type;
                    u2.Username = userAcl.Username;
                    ArrayList l = new ArrayList();
                    foreach (AmtUserAclRealm rl in userAcl.Realms)
                    {
                        if ((int)rl <= 15) l.Add(rl);
                        else if ((int)rl <= 17 && system.CoreVersion >= 2.5) l.Add(rl);
                        else if ((int)rl <= 17 && system.CoreVersion >= 3.0) l.Add(rl);
                        else if ((int)rl <= 21 && system.CoreVersion >= 4.0) l.Add(rl);
                        else if ((int)rl <= 23 && system.CoreVersion >= 5.0) l.Add(rl);
                    }
                    u2.Realms = (AmtUserAclRealm[])l.ToArray(typeof(AmtUserAclRealm));
            
                    r = (AmtCallStatus)service.AddUserAclEntryEx(u2.GetUserAclEntryEx(this), out handle);
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                r = AmtCallStatus.CALL_SKIPPED;
            }
            
            if (r != AmtCallStatus.SUCCESS) return r;
            UserAclEntriesCache = null;
            return AmtCallStatus.SUCCESS;
        }

        public AmtCallStatus UpdateUserAclEntry(AmtUserAcl userAcl)
        {
            if (userAcl.Username != null && userAcl.Username.StartsWith("$$")) return AmtCallStatus.CALL_SKIPPED;
            if (system.CoreVersion == 0) return AmtCallStatus.NOT_PERMITTED;

            AmtCallStatus r;
            try
            {
                if(system.WsMan != null)
                {
                    // USE WSMAN
                    try
                    {
                        ArrayList l = new ArrayList();
                        foreach (AmtUserAclRealm rl in userAcl.Realms)
                        {
                            if ((int)rl <= 15) l.Add((uint)rl);
                            else if ((int)rl <= 17 && system.CoreVersion >= 2.5) l.Add((uint)rl);
                            else if ((int)rl <= 17 && system.CoreVersion >= 3.0) l.Add((uint)rl);
                            else if ((int)rl <= 21 && system.CoreVersion >= 4.0) l.Add((uint)rl);
                            else if ((int)rl <= 23 && system.CoreVersion >= 5.0) l.Add((uint)rl);
                        }

                        AMT_AuthorizationService authService = new AMT_AuthorizationService(system.WsMan);
                        if (userAcl.Type == AmtUserAclType.Digest)
                        {
                            r = (AmtCallStatus)authService.UpdateUserAclEntryEx(userAcl.Handle,
                                userAcl.Username, ComputerPasswordDigest(userAcl.Username, userAcl.Password),
                                null, (uint)userAcl.AccessPermission, (uint[])l.ToArray(typeof(uint)));
                        }
                        else if (userAcl.Type == AmtUserAclType.Kerberos)
                        {
                            r = (AmtCallStatus)authService.UpdateUserAclEntryEx(userAcl.Handle, 
                                null, null, userAcl.KerberosSid,
                                (uint)userAcl.AccessPermission, (uint[])l.ToArray(typeof(uint)));
                        }
                        else
                            throw new Exception("AmtUserAcl Unknown type");
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        if (wsManException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.UpdateUserAclEntry WSMAN error",
                            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                        }
                        CimException cimException = e as CimException;
                        if (cimException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.UpdateUserAclEntry CIM error",
                            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                        }

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.UpdateUserAclEntry",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                        throw;
                    }
                }
                else
                {
                    // USE EOI
                    AmtUserAcl u2 = new AmtUserAcl("", "", AccessPermissionType.AnyAccessPermission, null);
                    u2.AccessPermission = userAcl.AccessPermission;
                    u2.KerberosSid = userAcl.KerberosSid;
                    u2.Password = userAcl.Password;
                    u2.PasswordDigest = userAcl.PasswordDigest;
                    u2.Type = userAcl.Type;
                    u2.Username = userAcl.Username;
                    ArrayList l = new ArrayList();
                    foreach (AmtUserAclRealm rl in userAcl.Realms)
                    {
                        if ((int)rl <= 15) l.Add(rl);
                        else if ((int)rl <= 17 && system.CoreVersion >= 2.5) l.Add(rl);
                        else if ((int)rl <= 17 && system.CoreVersion >= 3.0) l.Add(rl);
                        else if ((int)rl <= 21 && system.CoreVersion >= 4.0) l.Add(rl);
                        else if ((int)rl <= 23 && system.CoreVersion >= 5.0) l.Add(rl);
                    }
                    u2.Realms = (AmtUserAclRealm[])l.ToArray(typeof(AmtUserAclRealm));

                    r = (AmtCallStatus)service.UpdateUserAclEntryEx(userAcl.Handle, u2.GetUserAclEntryEx(this));
                }
            }
            catch (System.Web.Services.Protocols.SoapHeaderException)
            {
                r = AmtCallStatus.CALL_SKIPPED;
            }
            if (r != AmtCallStatus.SUCCESS) return r;
            UserAclEntriesCache = null;
            return AmtCallStatus.SUCCESS;
        }

        public AmtCallStatus ResetUserAclEntries(AmtUserAcl[] newUserAcls)
        {
            if (system.AmtRev <= 1) return AmtCallStatus.CALL_SKIPPED;
            AmtUserAcl[] oldUserAcls;
            AmtCallStatus r = GetUserAclEntries(out oldUserAcls);
            if (r != AmtCallStatus.SUCCESS)
            {
                newUserAcls = null;
                return r;
            }

            // Add new users
            foreach (AmtUserAcl newUser in newUserAcls)
            {
                if (newUser.HandleSet == false)
                {
                    r = AddUserAclEntry(newUser);
                    if (r != AmtCallStatus.SUCCESS) return r;
                }
            }

            // Modify users
            foreach (AmtUserAcl newUser in newUserAcls)
            {
                if (newUser.HandleSet == true)
                {
                    if (newUser.Type == AmtUserAclType.Digest && newUser.Password == null) continue;
                    r = UpdateUserAclEntry(newUser);
                    if (r != AmtCallStatus.SUCCESS) return r;
                }
            }

            // Remove all deleted users
            foreach (AmtUserAcl oldUser in oldUserAcls)
            {
                if (oldUser.HandleSet == true)
                {
                    bool found = false;
                    foreach (AmtUserAcl newUser in newUserAcls)
                    {
                        if (newUser.HandleSet == true && oldUser.Handle == newUser.Handle) found = true;
                    }
                    if (found == false)
                    {
                        if (system.WsMan != null)
                        {
                            // USE WSMAN
                            try
                            {
                                AMT_AuthorizationService authService = new AMT_AuthorizationService(system.WsMan);
                                r = (AmtCallStatus)authService.RemoveUserAclEntry(oldUser.Handle);
                            }
                            catch (Exception e)
                            {
                                WSManException wsManException = e as WSManException;
                                if (wsManException != null)
                                {
                                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.ResetUserAclEntries WSMAN error",
                                    String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                    wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                                    wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                                }
                                CimException cimException = e as CimException;
                                if (cimException != null)
                                {
                                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.ResetUserAclEntries CIM error",
                                    String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                                    cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                                    cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                                }

                                AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.ResetUserAclEntries",
                                    String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                                Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                                Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                                throw;
                            }
                        }
                        else
                        {
                            // USE EOI
                            r = (AmtCallStatus)service.RemoveUserAclEntry(oldUser.Handle);
                        }
                        if (r != AmtCallStatus.SUCCESS) return r;
                    }
                }
            }

            UserAclEntriesCache = null;
            return AmtCallStatus.SUCCESS;
        }

        public AmtCallStatus ResetUserAclEntriesEx(AmtUserAcl[] userAcls)
        {
            if (system.AmtRev <= 1) return AmtCallStatus.CALL_SKIPPED;
            AmtCallStatus r = RemoveAllUserAclEntries();
            if (r != AmtCallStatus.SUCCESS) return r;
            if (userAcls == null) return AmtCallStatus.SUCCESS;
            return SetUserAclEntries(userAcls);
        }

        public byte[] ComputerPasswordDigest(string username, string password)
        {
            string hashtext = username + ":" + GetDigestRealm() + ":" + password;
            System.Security.Cryptography.MD5CryptoServiceProvider x = new System.Security.Cryptography.MD5CryptoServiceProvider();
            return x.ComputeHash(System.Text.Encoding.UTF8.GetBytes(hashtext));
        }

        public AmtSecuritySettings GetSecurityProfile()
        {
            return new AmtSecuritySettings(this, system.Redirection);
        }

        private bool GetZeroTouchConfigurationModeCache = false;
        private bool GetZeroTouchConfigurationModeSet = false;
        public bool GetZeroTouchConfigurationMode()
        {
            if (ZeroTouchSupport() == false) return false;
            if (GetZeroTouchConfigurationModeSet == true) return GetZeroTouchConfigurationModeCache;

            if (system.WsMan != null)
            {
                // USE WSMAN
                try
                {
                    AMT_SetupAndConfigurationService scs = new AMT_SetupAndConfigurationService(system.WsMan);
                    scs.Get();
                    if (scs.ZeroTouchConfigurationEnabledExist == true)
                    {
                        GetZeroTouchConfigurationModeCache = scs.ZeroTouchConfigurationEnabled;
                        GetZeroTouchConfigurationModeSet = true;
                    }
                    else
                    {
                        GetZeroTouchConfigurationModeCache = false;
                    }
                    return GetZeroTouchConfigurationModeCache;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetZeroTouchConfigurationMode WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetZeroTouchConfigurationMode CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetZeroTouchConfigurationMode",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    GetZeroTouchConfigurationModeCache = false;
                    GetZeroTouchConfigurationModeSet = false;
                    return GetZeroTouchConfigurationModeCache;
                }
            }
            else
            {
                // Use SOAP
                service.GetZeroTouchConfigurationMode(out GetZeroTouchConfigurationModeCache);
                GetZeroTouchConfigurationModeSet = true;
                return GetZeroTouchConfigurationModeCache;
            }
        }

        public AmtCallStatus SetZeroTouchConfigurationMode(bool enabled)
        {
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_SetupAndConfigurationService scs = new AMT_SetupAndConfigurationService(system.WsMan);
                    scs.Get();
                    scs.ZeroTouchConfigurationEnabled = enabled;
                    scs.Put();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetZeroTouchConfigurationMode WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetZeroTouchConfigurationMode CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetZeroTouchConfigurationMode",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    throw;
                }
                return AmtCallStatus.SUCCESS;
            }
            else
            {
                // Use SOAP
                AmtCallStatus r = (AmtCallStatus)service.SetZeroTouchConfigurationMode(enabled);
                if (r == AmtCallStatus.SUCCESS) GetZeroTouchConfigurationModeSet = false;
                return r;
            }
        }

        /// <summary>
        /// This command extends the time that the network interface is open so that the Intel AMT device is available for setup and configuration.
        /// </summary>
        /// <param name="duration">The extension duration in hours (limited to maximum value of 24 in each call).</param>
        /// <returns>Intel AMT status code</returns>
        public AmtCallStatus ExtendProvisioningPeriod(uint duration)
        {
            AmtCallStatus r = (AmtCallStatus)service.ExtendProvisioningPeriod(duration);
            return r;
        }

        public bool ZeroTouchSupport()
        {
            if (((system.AmtRev == 2 && system.AmtRevMin >= 2 && system.AmtRevMin <= 4) || system.AmtRev >= 3)) return true;
            return false;
        }

        public class CertHashEntry
        {
            public uint Handle;
            public CertHashEntryType Entry;

            public CertHashEntry(uint handle, CertHashEntryType entry)
            {
                Handle = handle;
                Entry = entry;
            }
        }

        private CertHashEntry[] CertificateHashEntriesCache = null;
        public CertHashEntry[] GetCertificateHashEntries()
        {
            if (!ZeroTouchSupport()) return null;
            if (CertificateHashEntriesCache != null) return CertificateHashEntriesCache;

            uint[] handles;
            AmtCallStatus r = (AmtCallStatus)service.EnumerateCertificateHashEntries(out handles);

            ArrayList CertificateHashEntrieList = new ArrayList();
            foreach (uint h in handles)
            {
                CertHashEntryType entry;
                r = (AmtCallStatus)service.GetCertificateHashEntry(h, out entry);
                CertificateHashEntrieList.Add(new CertHashEntry(h,entry));
            }

            CertificateHashEntriesCache = (CertHashEntry[])CertificateHashEntrieList.ToArray(typeof(CertHashEntry));
            return CertificateHashEntriesCache;
        }

        public AmtCallStatus AddCertificateHashEntry(CertHashEntryType entry)
        {
            if (!ZeroTouchSupport()) return AmtCallStatus.CALL_SKIPPED;
            CertificateHashEntriesCache = null;
            uint handle;
            return (AmtCallStatus)service.AddCertificateHashEntry(entry, out handle);
        }

        public AmtCallStatus DeleteCertificateHashEntry(CertHashEntry entry)
        {
            if (!ZeroTouchSupport()) return AmtCallStatus.CALL_SKIPPED;
            CertificateHashEntriesCache = null;
            return (AmtCallStatus)service.DeleteCertificateHashEntry(entry.Handle);
        }

        public AmtCallStatus EnableCertificateHashEntry(CertHashEntry entry, bool enabled)
        {
            if (!ZeroTouchSupport()) return AmtCallStatus.CALL_SKIPPED;
            CertificateHashEntriesCache = null;
            return (AmtCallStatus)service.EnableCertificateHashEntry(entry.Handle, enabled);
        }

        private string ProvisioningServerOTPCache = null;
        public string GetProvisioningServerOTP()
        {
            if (!ZeroTouchSupport()) return null;
            if (ProvisioningServerOTPCache != null) return ProvisioningServerOTPCache;
            byte[] buf;
            AmtCallStatus r = (AmtCallStatus)service.GetProvisioningServerOTP(out buf);
            if (r != AmtCallStatus.SUCCESS) return null;
            ProvisioningServerOTPCache = UTF8Encoding.UTF8.GetString(buf);
            return ProvisioningServerOTPCache;
        }

        public AmtCallStatus SetProvisioningServerOTP(string password)
        {
            if (!ZeroTouchSupport()) return AmtCallStatus.CALL_SKIPPED;
            ProvisioningServerOTPCache = null;
            byte[] buf = UTF8Encoding.UTF8.GetBytes(password);
            AmtCallStatus r = (AmtCallStatus)service.SetProvisioningServerOTP(buf);
            return r;
        }

        /// <summary>
        /// This command is used to enable or disable VPN routing of Intel AMT network messages. This Administration Realm command enables or disables the VpnRouting feature that exposes the remote management interface through the local host to be used in a VPN environment.
        /// </summary>
        /// <param name="enable">Enables or disables the VPN routing feature.</param>
        /// <returns></returns>
        public AmtCallStatus EnableVpnRouting(bool enable)
        {
            if (system.IsMobile == false) return AmtCallStatus.NOT_PERMITTED;
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_EnvironmentDetectionSettingData envDetSettings = new AMT_EnvironmentDetectionSettingData(system.WsMan);
                    return (AmtCallStatus)envDetSettings.EnableVpnRouting(enable);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.EnableVpnRouting WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.EnableVpnRouting CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.EnableVpnRouting",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                AmtCallStatus r = (AmtCallStatus)service.EnableVpnRouting(enable);
                return r;
            }
        }

        private EnvironmentDetectionType EnvironmentDetectionCache = null;
        private bool EnvironmentDetectionCacheSet = false;
        private string[] EnvrionmentDetectionIPv6LocalPrefixesCache = null;

        /// <summary>
        /// Returns the enterprise environment definition. This routine is supported in Intel AMT Release 2.5 and later releases.
        /// </summary>
        /// <param name="EnvironmentDetection">The domains where the manageability interface is permitted to operate. Environment detection is disabled if this parameter returns Null.</param>
        /// <returns></returns>
        public AmtCallStatus GetEnvironmentDetection(out EnvironmentDetectionType EnvironmentDetection)
        {
            string[] EnvrionmentDetectionIPv6LocalPrefixes;
            return GetEnvironmentDetection(out EnvironmentDetection, out EnvrionmentDetectionIPv6LocalPrefixes);
        }

        public AmtCallStatus GetEnvironmentDetection(out EnvironmentDetectionType EnvironmentDetection, out string[] EnvrionmentDetectionIPv6LocalPrefixes)
        {
            EnvironmentDetection = null;
            EnvrionmentDetectionIPv6LocalPrefixes = null;

            if (system.AmtRev < 2 || (system.AmtRev == 2 && system.AmtRevMin < 5) || system.LimitedFeatures == true)
            {
                return AmtCallStatus.NOT_PERMITTED;
            }
            if (EnvironmentDetectionCacheSet == true)
            {
                EnvironmentDetection = EnvironmentDetectionCache;
                EnvrionmentDetectionIPv6LocalPrefixes = EnvrionmentDetectionIPv6LocalPrefixesCache;
                return AmtCallStatus.SUCCESS;
            }

            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_EnvironmentDetectionSettingData envDetSettings = new AMT_EnvironmentDetectionSettingData(system.WsMan);
                    envDetSettings.Get();

                    EnvironmentDetectionCache = new EnvironmentDetectionType();
                    EnvironmentDetectionCache.ExternalCircuitBreakerPolicySpecified = false; // TODO ***************
                    //EnvironmentDetectionCache.ExternalCircuitBreakerPolicy = 0; // TODO **************
                    EnvironmentDetectionCache.LocalDomains = (envDetSettings.DetectionStringsExist ? envDetSettings.DetectionStrings : new string[0]);
                    EnvrionmentDetectionIPv6LocalPrefixesCache = (envDetSettings.DetectionIPv6LocalPrefixesExist ? envDetSettings.DetectionIPv6LocalPrefixes : null);

                    EnvironmentDetection = EnvironmentDetectionCache;
                    EnvrionmentDetectionIPv6LocalPrefixes = EnvrionmentDetectionIPv6LocalPrefixesCache;
                    EnvironmentDetectionCacheSet = true;

                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetEnvironmentDetection WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetEnvironmentDetection CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetEnvironmentDetection",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                    EnvironmentDetectionCache = new EnvironmentDetectionType();
                    EnvironmentDetectionCache.ExternalCircuitBreakerPolicySpecified = false;
                    EnvironmentDetectionCache.ExternalCircuitBreakerPolicy = 0;
                    EnvironmentDetectionCache.LocalDomains = null;

                    EnvironmentDetection = EnvironmentDetectionCache;
                    EnvrionmentDetectionIPv6LocalPrefixesCache = null;
                    EnvironmentDetectionCacheSet = false;

                    return AmtCallStatus.SUCCESS;
                }
            }
            else
            {
                // Use SOAP
                AmtCallStatus r = (AmtCallStatus)service.GetEnvironmentDetection(out EnvironmentDetectionCache);
                if (r == AmtCallStatus.SUCCESS)
                {
                    EnvironmentDetection = EnvironmentDetectionCache;
                    EnvironmentDetectionCacheSet = true;
                }
                return r;
            }
        }

        /// <summary>
        /// This command enables the environment detection mechanism and defines the operating environment where the manageability
        /// interface is permitted to operate. This routine is supported in Intel AMT Release 2.5 and later releases.
        /// </summary>
        /// <param name="EnvironmentDetection">Describes the enterprise operating environment. If the parameter is Null, disables environment detection.</param>
        /// <returns></returns>
        public AmtCallStatus SetEnvironmentDetection(EnvironmentDetectionType EnvironmentDetection)
        {
            return SetEnvironmentDetection(EnvironmentDetection, null);
        }

        public AmtCallStatus SetEnvironmentDetection(EnvironmentDetectionType EnvironmentDetection, string[] EnvrionmentDetectionIPv6LocalPrefixes)
        {
            if (system.AmtRev < 2 || (system.AmtRev == 2 && system.AmtRevMin < 5)) return AmtCallStatus.NOT_PERMITTED;

            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_EnvironmentDetectionSettingData envDetSettings = new AMT_EnvironmentDetectionSettingData(system.WsMan);
                    envDetSettings.Get();

                    envDetSettings.DetectionStrings = EnvironmentDetection.LocalDomains;

                    if (EnvrionmentDetectionIPv6LocalPrefixes != null)
                        envDetSettings.DetectionIPv6LocalPrefixes = EnvrionmentDetectionIPv6LocalPrefixes;

                    envDetSettings.Put();
                    EnvironmentDetectionCache = null;
                    EnvrionmentDetectionIPv6LocalPrefixesCache = null;
                    EnvironmentDetectionCacheSet = false;
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetEnvironmentDetection WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetEnvironmentDetection CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetEnvironmentDetection",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use EOI/SOAP
                AmtCallStatus r = (AmtCallStatus)service.SetEnvironmentDetection(EnvironmentDetection);
                if (r == AmtCallStatus.SUCCESS)
                {
                    EnvironmentDetectionCacheSet = false;
                    EnvironmentDetectionCache = null;
                }
                return r;
            }
        }

        public class AmtStoredCert
        {
            public uint Handle;
            public X509Certificate2 Certificate;
            public CimReference Endpoint;

            public AmtStoredCert(uint handle, X509Certificate2 certificate)
            {
                Handle = handle;
                Certificate = certificate;
            }

            override public string ToString()
            {
                return CertificateOperations.GetCertificateLongString(Certificate);
            }
        }

        private AmtStoredCert[] CertStoreGetCertificatesCache = null;
        public AmtCallStatus CertStoreGetCertificates(out AmtStoredCert[] certificates)
        {
            if (CertStoreGetCertificatesCache != null)
            {
                certificates = CertStoreGetCertificatesCache;
                return AmtCallStatus.SUCCESS;
            }

            certificates = null;
            if (system.AmtRev < 2 || (system.AmtRev == 2 && system.AmtRevMin < 5))
                return AmtCallStatus.NOT_PERMITTED;

            if (system.WsMan != null)
            {
                Collection<AMT_PublicKeyCertificate> certs = null;
                // Use WSMAN
                try
                {
                    certs = AMT_PublicKeyCertificate.Enumerate(system.WsMan);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreGetCertificates WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreGetCertificates CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreGetCertificates",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    

                    //set cert collection to empty collection since none were found
                    //need to do this, otherwise get an error
                    certs = new Collection<AMT_PublicKeyCertificate> { };
                }

                ArrayList l = new ArrayList();
                foreach (AMT_PublicKeyCertificate cert in certs)
                {
                    if (cert.TrustedRootCertficate == false)
                    {
                        AmtStoredCert c = new AmtStoredCert(GetHandleFromString(cert.InstanceID), new X509Certificate2(cert.X509Certificate));
                        c.Endpoint = cert.Reference;
                        l.Add(c);
                    }
                }

                certificates = (AmtStoredCert[])l.ToArray(typeof(AmtStoredCert));
                CertStoreGetCertificatesCache = certificates;
                return AmtCallStatus.SUCCESS;
            }
            else
            {
                // Use SOAP
                uint[] handles;
                AmtCallStatus r = (AmtCallStatus)service.CertStoreEnumerateCertificates(null, null, null, out handles);
                if (r != AmtCallStatus.SUCCESS) return r;

                ArrayList certs = new ArrayList();
                foreach (uint handle in handles)
                {
                    CertificateType c;
                    r = (AmtCallStatus)service.CertStoreGetCertificate(handle, out c);
                    if (r != AmtCallStatus.SUCCESS) return r;
                    certs.Add(new AmtStoredCert(handle, new X509Certificate2(c.X509cert)));
                }

                certificates = (AmtStoredCert[])certs.ToArray(typeof(AmtStoredCert));
                CertStoreGetCertificatesCache = certificates;
                return AmtCallStatus.SUCCESS;
            }
        }

        public AmtCallStatus CertStoreRemoveCertificate(AmtStoredCert cert)
        {
            if (system.WsMan != null)
            {
                // WSMAN
                try
                {
                    AMT_PublicKeyCertificate.Delete(system.WsMan, cert.Endpoint);
                    CertStoreGetCertificatesCache = null;
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreRemoveCertificate WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreRemoveCertificate CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreRemoveCertificate",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // SOAP
                // Remote associated keys
                uint[] khandles = null;
                AmtCallStatus r = (AmtCallStatus)service.CertStoreEnumerateKeys(cert.Handle, true, out khandles);
                if (r == AmtCallStatus.SUCCESS && khandles != null) foreach (uint khandle in khandles) service.CertStoreRemoveKey(khandle);

                // Remove the certificate
                uint[] handles;
                r = (AmtCallStatus)service.CertStoreEnumerateKeys(cert.Handle, true, out handles);
                if (r == AmtCallStatus.SUCCESS && handles.Length > 0) foreach (uint handle in handles) service.CertStoreRemoveKey(handle);
                r = (AmtCallStatus)service.CertStoreRemoveCertificate(cert.Handle);
                if (r == AmtCallStatus.SUCCESS) CertStoreGetCertificatesCache = null;
                return r;
            }
        }

        public AmtCallStatus CertStoreRemoveCertificate(uint handle)
        {
            // Remote associated keys
            uint[] khandles = null;
            AmtCallStatus r = (AmtCallStatus)service.CertStoreEnumerateKeys(handle, true, out khandles);
            if (r == AmtCallStatus.SUCCESS && khandles != null) foreach (uint khandle in khandles) service.CertStoreRemoveKey(khandle);

            // Remove the certificate
            r = (AmtCallStatus)service.CertStoreRemoveCertificate(handle);
            if (r == AmtCallStatus.SUCCESS) CertStoreGetCertificatesCache = null;
            return r;
        }

        public AmtCallStatus GetTLSCredentials(out uint handle, out bool handleSpecified)
        {
            if (system.WsMan != null)
            {
                // Use WSMAN
                handle = 0;
                handleSpecified = false;
                try
                {
                    AMT_TLSCredentialContext tlsCredCtx = new AMT_TLSCredentialContext(system.WsMan);
                    tlsCredCtx.Get();
                    Debug.Assert(tlsCredCtx.ElementInContext.GetReferenceType() == typeof(AMT_PublicKeyCertificate));
                    AMT_PublicKeyCertificate pubKeyCert = new AMT_PublicKeyCertificate(system.WsMan);
                    pubKeyCert.Get(tlsCredCtx.ElementInContext);
                    handle = GetHandleFromString(pubKeyCert.InstanceID);
                    handleSpecified = true;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetTLSCredentials WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetTLSCredentials CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.GetTLSCredentials",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }

                return AmtCallStatus.SUCCESS;
            }
            else
            {
                // SOAP
                AmtCallStatus r = (AmtCallStatus)service.GetTLSCredentials(out handle, out handleSpecified);
                return r;
            }
        }

        public AmtCallStatus CertStoreRemoveTlsCertificate()
        {
            uint handle;
            bool handleSpecified;
            AmtCallStatus r = (AmtCallStatus)service.GetTLSCredentials(out handle, out handleSpecified);
            if (r != AmtCallStatus.SUCCESS) return r;
            if (handleSpecified == true)
            {
                r = (AmtCallStatus)service.SetTLSCredentials(0, false);
                if (r != AmtCallStatus.SUCCESS) return r;
                r = (AmtCallStatus)service.CertStoreRemoveCertificate(handle);
                if (r == AmtCallStatus.SUCCESS) CertStoreGetCertificatesCache = null;
            }
            return r;
        }

        public AmtCallStatus CertStoreRemoveAllKeys()
        {
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    Collection<AMT_PublicKeyManagementService> wskeys = AMT_PublicKeyManagementService.Enumerate(system.WsMan);
                    foreach (AMT_PublicKeyManagementService key in wskeys)
                    {
                        AMT_PublicKeyManagementService.Delete(system.WsMan, key.Reference);
                    }
                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreRemoveAllKeys WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreRemoveAllKeys CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreRemoveAllKeys",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                uint[] handles;
                AmtCallStatus r = (AmtCallStatus)service.CertStoreEnumerateKeys(0, false, out handles);
                if (r == AmtCallStatus.SUCCESS && handles.Length > 0) foreach (uint handle in handles) service.CertStoreRemoveKey(handle);
                return r;
            }
        }

        public AmtCallStatus CertStoreAddCertificate(X509Certificate2 cert, byte[] rsaPrivateKey, out uint handle)
        {
            AmtCallStatus r;
            handle = 0;

            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    // Put the private key
                    AMT_PublicKeyManagementService pubKeyMgmtService = new AMT_PublicKeyManagementService(system.WsMan);
                    CimReference newRef = null;
                    r = (AmtCallStatus)pubKeyMgmtService.AddKey(rsaPrivateKey, out newRef);
                    if (r != AmtCallStatus.SUCCESS) return r;

                    // Put the certificate
                    r = (AmtCallStatus)pubKeyMgmtService.AddCertificate(cert.GetRawCertData(), out newRef);
                    if (r != AmtCallStatus.SUCCESS) return r;
                    CertStoreGetCertificatesCache = null;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreAddCertificate WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreAddCertificate CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreAddCertificate",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
                return AmtCallStatus.SUCCESS;
            }
            else
            {
                // Use SOAP
                uint khandle;
                KeyPairType kp = new KeyPairType();
                kp.Item = new RSAKeyPairType();
                kp.Item.DERKey = rsaPrivateKey;
                r = (AmtCallStatus)service.CertStoreAddKey(kp, out khandle);
                if (r != AmtCallStatus.SUCCESS) return r;

                CertificateType c = new CertificateType();
                c.X509cert = cert.GetRawCertData();
                r = (AmtCallStatus)service.CertStoreAddCertificate(c, out handle);
                if (r != AmtCallStatus.SUCCESS) return r;
                CertStoreGetCertificatesCache = null;

                return r;
            }
        }

        public AmtCallStatus SetTlsCredentials(uint handle, bool handleSpecified)
        {
            if ((system.WsMan != null) && (system.CoreVersion >= 3))
            {
                AMT_PublicKeyCertificate newTlsCert = null;
                if (handleSpecified)
                {
                    try
                    {
                        // Get the specified certificate from the Intel(R) AMT ME certificate store.
                        newTlsCert = new AMT_PublicKeyCertificate(system.WsMan);
                        AMT_PublicKeyCertificate.CimKeys keys = new AMT_PublicKeyCertificate.CimKeys();
                        keys.InstanceID = String.Format("Intel(r) AMT Certificate: Handle: {0}", handle);
                        newTlsCert.Get(keys);
                    }
                    catch (Exception e)
                    {
                        WSManException wsManException = e as WSManException;
                        if (wsManException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsCredentials WSMAN error",
                            String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                            wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                        }
                        CimException cimException = e as CimException;
                        if (cimException != null)
                        {
                            AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsCredentials CIM error",
                            String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                            cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                            cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                        }

                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsCredentials",
                            String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                        Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                        Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                        newTlsCert = null;
                    }
                } // END if (handleSpecified)

                AMT_TLSCredentialContext tlsCredCtx = null;
                try
                {
                    tlsCredCtx = new AMT_TLSCredentialContext(system.WsMan);
                    tlsCredCtx.Get();
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsCredentials WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsCredentials CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsCredentials",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    tlsCredCtx = null; // Will need to create new TLS credentials.
                }

                try
                {
                    if (handleSpecified && (newTlsCert != null) && (tlsCredCtx != null))
                    {
                        // Associate the selected certificate with the TLS credentials.
                        tlsCredCtx.ElementInContext = newTlsCert.Reference;
                        tlsCredCtx.Put();
                    }
                    else if (handleSpecified && (newTlsCert != null) && (tlsCredCtx == null))
                    {
                        // Create a new TLS credential with the specified certificate.
                        AMT_TLSProtocolEndpointCollection tlsProtocolEPs = new AMT_TLSProtocolEndpointCollection(system.WsMan);
                        tlsProtocolEPs.Get();

                        tlsCredCtx = new AMT_TLSCredentialContext(system.WsMan);
                        tlsCredCtx.ElementInContext = newTlsCert.Reference;
                        tlsCredCtx.ElementProvidingContext = tlsProtocolEPs.Reference;
                        tlsCredCtx.Create();
                    }
                    else if (!handleSpecified && (newTlsCert == null) && (tlsCredCtx != null))
                    {
                        // Remove the current TLS credentials.
                        tlsCredCtx.Delete();
                    }
                    else
                        throw new Exception("Unexpected program flow."); // TODO: Make less ambiguous

                    return AmtCallStatus.SUCCESS;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsCredentials WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsCredentials CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.SetTlsCredentials",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                AmtCallStatus r = (AmtCallStatus)service.SetTLSCredentials(handle, handleSpecified);
                return r;
            }
        }

        private AmtStoredCert[] CertStoreGetTrustedCertificatesCache = null;
        public AmtCallStatus CertStoreGetTrustedCertificates(out AmtStoredCert[] certificates)
        {
            if (CertStoreGetTrustedCertificatesCache != null)
            {
                certificates = CertStoreGetTrustedCertificatesCache;
                return AmtCallStatus.SUCCESS;
            }

            certificates = null;
            if (system.CoreVersion < 2.5) return AmtCallStatus.NOT_PERMITTED;

            if (system.WsMan != null)
            {
                Collection<AMT_PublicKeyCertificate> certs = null;
                // Use WSMAN
                try
                {
                    certs = AMT_PublicKeyCertificate.Enumerate(system.WsMan);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreGetTrustedCertificates WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreGetTrustedCertificates CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreGetTrustedCertificates",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);

                    //set cert collection to empty collection since none were found
                    //need to do this, otherwise get an error
                    certs = new Collection<AMT_PublicKeyCertificate> { };
                }

                ArrayList l = new ArrayList();
                foreach (AMT_PublicKeyCertificate cert in certs)
                {
                    if (cert.TrustedRootCertficate == true)
                    {
                        AmtStoredCert c = new AmtStoredCert(GetHandleFromString(cert.InstanceID), new X509Certificate2(cert.X509Certificate));
                        c.Endpoint = cert.Reference;
                        l.Add(c);
                    }
                }

                certificates = (AmtStoredCert[])l.ToArray(typeof(AmtStoredCert));
                CertStoreGetTrustedCertificatesCache = certificates;
                return AmtCallStatus.SUCCESS;
            }
            else
            {
                // Use SOAP
                uint[] handles;
                AmtCallStatus r = (AmtCallStatus)service.EnumerateTrustedRootCertificates(out handles);
                if (r != AmtCallStatus.SUCCESS) return r;

                ArrayList certs = new ArrayList();
                foreach (uint handle in handles)
                {
                    CertificateType c;
                    r = (AmtCallStatus)service.GetTrustedRootCertificate(handle, out c);
                    if (r != AmtCallStatus.SUCCESS) return r;
                    certs.Add(new AmtStoredCert(handle, new X509Certificate2(c.X509cert)));
                }

                certificates = (AmtStoredCert[])certs.ToArray(typeof(AmtStoredCert));
                CertStoreGetTrustedCertificatesCache = certificates;
                return AmtCallStatus.SUCCESS;
            }
        }

        public AmtCallStatus CertStoreRemoveTrustedCertificate(AmtStoredCert cert)
        {
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_PublicKeyCertificate.Delete(system.WsMan, cert.Endpoint);
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreRemoveTrustedCertificate WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreRemoveTrustedCertificate CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreRemoveTrustedCertificate",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
                CertStoreGetTrustedCertificatesCache = null;
                return AmtCallStatus.SUCCESS;
            }
            else
            {
                // Use SOAP
                AmtCallStatus r = (AmtCallStatus)service.DeleteTrustedRootCertificate(cert.Handle);
                if (r == AmtCallStatus.SUCCESS) CertStoreGetTrustedCertificatesCache = null;
                return r;
            }
        }

        public AmtCallStatus CertStoreAddTrustedRootCertificate(X509Certificate2 cert)
        {
            if (system.WsMan != null)
            {
                // Use WSMAN
                try
                {
                    AMT_PublicKeyManagementService pubKeyMgmtService = new AMT_PublicKeyManagementService(system.WsMan);
                    CimReference newCertRef = null;
                    AmtCallStatus r = (AmtCallStatus)pubKeyMgmtService.AddTrustedRootCertificate(cert.GetRawCertData(), out newCertRef);
                    if (r == AmtCallStatus.SUCCESS)
                        CertStoreGetTrustedCertificatesCache = null;
                    return r;
                }
                catch (Exception e)
                {
                    WSManException wsManException = e as WSManException;
                    if (wsManException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreAddTrustedRootCertificate WSMAN error",
                        String.Format("WSMAN Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        wsManException.WSManFault.Code.Value, wsManException.WSManFault.Code.Subcode.Value,
                        wsManException.WSManFault.Detail.Text, wsManException.WSManFault.Reason.Text));
                    }
                    CimException cimException = e as CimException;
                    if (cimException != null)
                    {
                        AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreAddTrustedRootCertificate CIM error",
                        String.Format("CIM Fault\r\nCode: {0} (subcode: {1})\r\nDetail: {2}\r\nReason: {3}",
                        cimException.CimFault.Code.Value, cimException.CimFault.Code.Subcode.Value,
                        cimException.CimFault.Detail.Text, cimException.CimFault.Reason.Text));
                    }

                    AmtDebugForm.LogEvent(this, AmtDebugForm.EventType.CallFailed, "AmtSecurityAdmin.CertStoreAddTrustedRootCertificate",
                        String.Format("{0}\r\n{1}", e.Message, e.StackTrace));
                    Debug.Print("EXCEPTION: {0} (type {1})", e.Message, e.GetType().ToString());
                    Debug.Print("EXCEPTION STACK: {0}", e.StackTrace);
                    return AmtCallStatus.FAILED_WEB_CALL;
                }
            }
            else
            {
                // Use SOAP
                CertificateType c = new CertificateType();
                c.X509cert = cert.GetRawCertData();
                uint handle;
                AmtCallStatus r = (AmtCallStatus)service.AddTrustedRootCertificate(c, out handle);
                if (r == AmtCallStatus.SUCCESS) CertStoreGetTrustedCertificatesCache = null;
                return r;
            }
        }

        private CrlType[] GetCrlCache = null;
        public AmtCallStatus GetCRL(out CrlType[] crl)
        {
            crl = null;
            if (system.AmtRev < 2) return AmtCallStatus.NOT_PERMITTED;
            
            if (system.WsMan != null)
            {
                // Use WSMAN

                // TODO ***************
                crl = new CrlType[0];
                return AmtCallStatus.SUCCESS;
            }
            else
            {
                // Use SOAP
                AmtCallStatus r = (AmtCallStatus)service.GetCRL(out crl);
                if (r == AmtCallStatus.SUCCESS) GetCrlCache = crl;
                return r;
            }
        }

        public AmtCallStatus SetCRL(CrlType[] crl)
        {
            if (system.AmtRev < 2) return AmtCallStatus.NOT_PERMITTED;
            AmtCallStatus r = (AmtCallStatus)service.SetCRL(crl);
            if (r == AmtCallStatus.SUCCESS) GetCrlCache = null;
            return r;
        }

        private static uint GetHandleFromString(string str)
        {
            int i = str.IndexOf(':');
            while (i >= 0)
            {
                str = str.Substring(i + 1);
                i = str.IndexOf(':');
            }
            i = str.IndexOf(' ');
            while (i >= 0)
            {
                str = str.Substring(i + 1);
                i = str.IndexOf(' ');
            }
            return uint.Parse(str);
        }

        public AmtCallStatus GetAuditors(out AmtUserAcl[] accounts)
        {
            accounts = null;
            ArrayList l = new ArrayList();

            // Get user ACL's
            AmtUserAcl[] acls;
            AmtCallStatus r = GetUserAclEntries(out acls);
            if (r != AmtCallStatus.SUCCESS)
            {
                accounts = null;
                return r;
            }
            if (acls != null)
            {
                foreach (AmtUserAcl acl in acls)
                {
                    foreach (AmtUserAclRealm realm in acl.Realms)
                    {
                        if ((int)realm == 20)
                        {
                            l.Add(acl);
                            break;
                        }
                    }
                }
                accounts = (AmtUserAcl[])l.ToArray(typeof(AmtUserAcl));
            }

            return AmtCallStatus.SUCCESS;
        }

    }
}
